package gov.va.med.fpps.extract.csv;

import gov.va.med.hac.edi.utils.jpa.JPAUtil;

import java.io.File;
import java.text.SimpleDateFormat;

import javax.naming.ConfigurationException;
import javax.persistence.EntityManager;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;





public class GenerateCsv {
	
	static final Logger summaryLOGGER = LoggerFactory.getLogger(GenerateCsv.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + GenerateCsv.class);
    @SuppressWarnings({ "unchecked", "unused" })
	public static void main(String[] args) {

        final int quarterYear = 90;
        EntityManager em = null;
        SimpleDateFormat datefmtr = null;
        
		// Parse the command line
		CommandLine line = null;
		Options opts = getCliOptions();
		try {
			CommandLineParser parser = new GnuParser();
			line = parser.parse(opts, args);
			// If the "help" option is specified, just show usage options and
			// exit
			if (line.hasOption("h")) {
				printUsage(opts, args, false);
				System.exit(1);
			}
			// If any required command line parameter is missing, just show
			// usage options and exit
		} catch (MissingOptionException moe) {
			printUsage(opts, args, true);
			System.exit(1);
		} catch (ParseException pe) {
			System.out.println("Problem parsing the command line");
			summaryLOGGER.error("Problem parsing the command line");
			traceLOGGER.error("Problem parsing the command line");
			printUsage(opts, args, false);
			System.exit(1);
		}

        try {

			File configFile = null;
			String configFilePath = line.getOptionValue("f");
			if (configFilePath != null && configFilePath.length() > 0) {
				configFile = JPAUtil.getJpaConfigFile(configFilePath.trim());
			} else {
				configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
			}

			String propertiesFilePath = line.getOptionValue("i");
			
			if ( propertiesFilePath == null) {
				propertiesFilePath = "";
			}
			
			propertiesFilePath = propertiesFilePath.trim();

            if (configFile != null) {
            	em = JPAUtil.getEntityManager(configFile);
                if (em == null) {
                	throw new ConfigurationException(
                    "Unable to configure Entity Manager.");
                }
                GenerateCsvProcess process =  new GenerateCsvProcess();
                process.getDetailRecords(em, propertiesFilePath);
            }

        } catch (Throwable t) {
			t.printStackTrace();
        } finally {
            if ((em != null) && em.isOpen()) {
                em.close();
            }

        }
    }
    
	private static void printUsage(Options opts, String[] inArgs,
			boolean missingArg) {
		final int helpwidth = 80;
		HelpFormatter fmtr = new HelpFormatter();
		System.out.println();
		if (missingArg) {
			System.out
					.println("Some required commandline argument was missing");
		}
		fmtr.printHelp(helpwidth, "GenerateCsv", "", opts, "", true);
		if (missingArg) {
			System.out.println(inArgs.length
					+ " command line arguments were received: ");
			for (int i = 0; i < inArgs.length; i++) {
				System.out.println(inArgs[i]);
			}
		}
	}
	
	/**
	 * Helper method to populate command line options objects.
	 * 
	 * @return the <code>Options</code> object containing the set of command
	 *         line option.
	 */
	@SuppressWarnings("static-access")
	private static Options getCliOptions() {
		Options options = new Options();

		Option configfile = OptionBuilder
				.withArgName("jpaFile")
				.hasArg()
				.withLongOpt("jpaFile")
				.withDescription(
						"The JPA Configuration file, such as hibernate.properties.")
				.create("f");

		Option fileName = OptionBuilder
				.withArgName("fileName")
				.hasArgs(1)
				.isRequired()
				.withLongOpt("fileName")
				.withDescription(
						"The absolute path, including the filename, of the input file that contains data to be loaded into the staging tables.")
				.create("i");

		Option help = OptionBuilder.withLongOpt("help").withDescription(
				"Print this message.").create("h");

		options.addOption(help);
		options.addOption(configfile);
		options.addOption(fileName);

		return options;
	}

}
