package gov.va.med.fpps.extract.csv;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.persistence.EntityManager;
import javax.persistence.Query;

public class GenerateCsvProcess {
	
	
    
	@SuppressWarnings({ "unchecked", "rawtypes", "null" })
	public void getDetailRecords( EntityManager entityManager, String propertiesFilePath) {

		if ( propertiesFilePath.length() == 0 ) {
			propertiesFilePath = getFilePath();
		}
		String fileName = propertiesFilePath+getCsvFileName()+".csv";
		System.out.println("file name :" + fileName);
		File file = new File(propertiesFilePath+getCsvFileName()+".csv");
		Writer fileWriter = null;
		BufferedWriter bufferedWriter = null;

		boolean DEBUG = false;

		Query q;
	    List<?> l;
	    Iterator<?> it;
	    
	    try {
	    	 fileWriter = new FileWriter(file);
	    	 bufferedWriter = new BufferedWriter(fileWriter);
	    	 
	        String sql = "SELECT  c.visn_id_cd, " + "   c.va_facility_cd, " +
	                     " c.claim_index, " + " fcpl.check_date, " +
	        		     " fcpl.check_number, " +  "fcpl.billed_amount " +
	                     " FROM fpps_owner.FEE_CLAIM_PAYMENT_LINES fcpl, fpps_owner.claims c " +
	                     " INNER JOIN fpps_owner.fee_claim_payments fcp " +
	                     " ON c.claim_index = fcp.claim_index " +
	                     " WHERE fcp.fee_claim_payment_id = fcpl.fee_claim_payment_id " +
	                     " ORDER by c.visn_id_cd, c.va_facility_cd, c.claim_index";

        System.out.println("SQL: " + sql);
	        
	        q = entityManager.createNativeQuery(sql);

	        l = q.getResultList();
	        it = l.iterator();

			String csvHeader = "VISN Number,Station Number,FPPS Claim Number,Check Date, Check Amount,Check Number";
			csvHeader += System.getProperty("line.separator");
			bufferedWriter.write(csvHeader);
			
			while (it.hasNext()) {
				Object[] row = (Object[]) it.next();

				if(row != null) {
					        	 
					BigDecimal visn_id = (BigDecimal) row[0];
					String station = (String) row[1];
					BigDecimal claimIndex = (BigDecimal) row[2];
					String checkDate = "";
					if ( row[3] != null) {
						checkDate = new SimpleDateFormat("MM/dd/yyyy").format((java.sql.Timestamp) row[3] );
					}
					String checkNumber = (String) row[4];
					BigDecimal amount = (BigDecimal) row[5];
					if (amount == null){
						amount = new BigDecimal(0);
					}
					String csvLine = visn_id + "," + station + "," + claimIndex + "," + checkDate + "," + checkNumber + "," + amount;
					csvLine += System.getProperty("line.separator");
					bufferedWriter.write(csvLine);
				}
			} // end while
		}catch(Exception ex) {
			
			System.out.println("CSV File Error: " + ex.getMessage());
		}	
	    finally {
	    	if (bufferedWriter != null && fileWriter != null) {
	            try {
	            	bufferedWriter.close();
		            fileWriter.close();
		        } catch (IOException e) {
		        	e.printStackTrace();
		        }
           }
        }
	}
	
    private String getCsvFileName() {
    	
        Calendar c = GregorianCalendar.getInstance();
        c.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);

        DateFormat df = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
        String dateString = "";
        dateString = df.format(c.getTime());

        return dateString;
    }
    
    
    private String getFilePath(){
    
    	String filePath = "";

    	try {    
    		
			BufferedReader br = new BufferedReader(new FileReader("GenerateCsv.properties"));
	
	    	try {    	
	    	    StringBuilder sb = new StringBuilder();
	    	    String line = br.readLine();
	
	    	    while (line != null) {
	    	    	String pathString = line.substring(line.lastIndexOf('=') + 1);
	    	        sb.append(pathString);
	    	        line = br.readLine();
	    	    }
	    	    filePath = sb.toString();
	    	    
			}catch(Exception ex) {
				
				System.out.println("Could not read from GenerateCsv properties file"  + ex.getMessage());
			}
	    	finally {
	    	    try {
					br.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
	    	}
    	}
	    	catch(FileNotFoundException  ex) {
				
				System.out.println("Could not locate GenerateCsv properties file: " + ex.getMessage());
			}
    	
    	
    	return filePath;
    }
    
}
