/** $Id: FeeEditChecks.java,v 1.1 2012/01/05 22:19:17 vhahacbergri Exp $ */
package gov.va.med.hac.edi.edits;

import gov.va.med.hac.edi.utils.jpa.JPAUtil;

import java.io.File;

import javax.naming.ConfigurationException;
import javax.persistence.EntityManager;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



/**
 * Performs the required Fee business edits.
 * 
 */
public class FeeEditChecks {

   static final Logger LOGGER = LoggerFactory.getLogger(FeeEditChecks.class);
   
   
   /** private constructor to prevent unneeded instance creation. */
   private FeeEditChecks() { }

   /**
    * Entry point for the FeeEditChecks process. The JPA connection
    * configuration file name can be passed into this method which
    * will override the normal persistence.xml file found in the
    * META-INF directory of the jar file.
    * <br><br>
    * <code>usage: FEE_EDITS [-f &lt;file&gt;] [-h]</code>
    * 
    * @param args set of options where &lt;file&gt; is the full path name 
    *             of the JPA configuration file.
    */
   public static void main(String[] args) {
      LOGGER.info("\n\n");
      LOGGER.info("<<<<<<<<<<<<< Beginning FEE Edit Checks <<<<<<<<<<<<<<<<<<<<");
      CommandLineParser parser = new GnuParser();
      Options opts = getCliOptions();
     
      EntityManager em = null;
      try {
         String helpOpt    = "h";
         String configOpt  = "f";
         String goLiveDate = "g";
         
         CommandLine line = parser.parse(opts, args);

         if (line.hasOption(helpOpt)) {
            printUsage(opts);
            System.exit(1);
         }

         File configfile = null;
         if (line.hasOption(configOpt)) {
            configfile = JPAUtil.getJpaConfigFile(line.getOptionValue(configOpt));
         }

         String glDate = null;
         if ( line.hasOption( goLiveDate ) ) {
            glDate = line.getOptionValue( goLiveDate );
         } else {
            glDate = "20141001";
         }
        
         if (configfile != null) {
        	 // does this exist anyplace? - not in project?
            em = JPAUtil.getEntityManager(configfile, "feeEditChecks");
         }
         
         if (em == null) {
            throw new ConfigurationException("Unable to configure Entity Manager.");
         }
         
         Process5010FeeEdits pfe = new Process5010FeeEdits();
         pfe.process( em, glDate );
         //pfe.process(em);
      } catch (MissingOptionException moe) {
         printUsage(opts);
      } catch (Throwable t) {
         LOGGER.error("Error in Fee Edit Checks", t);
      } finally {
         if (em != null && em.isOpen()) {
            em.close();
         }
         LOGGER.info("<<<<<<<<<<<<< Ending FEE Edit Checks <<<<<<<<<<<<<<<<<<<<");
      }
   }

   private static void printUsage(Options opts) {
      final int helpwidth = 80;
      HelpFormatter fmtr = new HelpFormatter();
      System.out.println();
      fmtr.printHelp(helpwidth, "FeeEditChecks", "", opts, "", true);
   }

   /**
    * Helper method to populate command line options objects. 
    * 
    * @return the <code>Options</code> object containing the set of
    *         command line option.
    */
   @SuppressWarnings("static-access")
   static Options getCliOptions() {
      Options options = new Options();

      Option configfile = OptionBuilder.withArgName("file")
                                       .isRequired()
                                       .hasArg()
                                       .withLongOpt("configfile")
                                       .withDescription("The EntityManager Configuration file.")
                                       .create("f");

      Option goLiveDate = OptionBuilder.withArgName("ICD-10 Go Live Date YYYYMMDD")
                                       .hasArg()
                                       .withLongOpt("Icd10GoLiveDate")
                                       .withDescription("The Official ICD-10 Go Live Date in YYYYMMDD format")
                                       .create("g");

      Option help = OptionBuilder.withLongOpt("help")
                                 .withDescription("Print this message.")
                                 .create("h");

      options.addOption(help);
      options.addOption(configfile);
      options.addOption(goLiveDate);

      return options;
   }
}



