package gov.va.med.hac.edi.edits;

import gov.va.med.domain.erepos.Claim;
import gov.va.med.domain.erepos.ClaimDt;
import gov.va.med.domain.erepos.DemographicInfo;
import gov.va.med.domain.erepos.DentalSvcLine;
import gov.va.med.domain.erepos.EdiProcessLog;
import gov.va.med.domain.erepos.EntityAddress;
import gov.va.med.domain.erepos.EntityAdtlId;
import gov.va.med.domain.erepos.ForeignCurrency;
import gov.va.med.domain.erepos.HiCodes;
import gov.va.med.domain.erepos.InstClaimCode;
import gov.va.med.domain.erepos.InstSvcLine;
import gov.va.med.domain.erepos.PatientInfo;
import gov.va.med.domain.erepos.PrfsnSvcLine;
import gov.va.med.domain.erepos.RejectClaim;
import gov.va.med.domain.erepos.RejectReasonCodes;
import gov.va.med.domain.erepos.TxnSet;
import gov.va.med.domain.fppsowner.IcdCode;
import gov.va.med.domain.fppsowner.MedicalCode;
import gov.va.med.hac.edi.utils.DateUtil;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

class Process5010FeeEdits {

	EntityManager em;

	Claim currentWorkingClaim = null;

	static final Logger LOGGER = LoggerFactory.getLogger(Process5010FeeEdits.class);

	int iAccCnt = 0;
	int iRejCnt = 0;

	private static final String INST_EDI = "005010X223";
	private static final String PROF_EDI = "005010X222";
	private static final String DENT_EDI = "005010X224";
	private static final String INVALID_ID = "999999999";
	final int DATE_FORMAT_DTM_LENGTH = 12; // CCYYMMDDHHMM
	final int DATE_FORMAT_RD8_LENGTH = 17; // A Date with DTP02 = RD8 contains a range CCYYMMDD-CCYYMMDD
	final int DATE_FORMAT_INDEX_OF_DASH_IN_RD8 = 9;
	final int DATE_FORMAT_YEAR_INDEX = 4;
	final int DATE_FORMAT_MONTH_INDEX = 6;
	final int DATE_FORMAT_DAY_INDEX = 8;
	final int DATE_FORMAT_END_DATE_YEAR_INDEX = 13;
	final int DATE_FORMAT_END_DATE_MONTH_INDEX = 15;

	private static final String ClaimEncounterNotAccepted837All = "A3:484:85";
	private static final String ForeignCurrencyNotAllowed837All = "A3:484:85";
	private static final String MissingBillingTin837DP = "A6:21:85;A6:128:85";
	private static final String MissingBillingTin837I = "A6:21:85;A6:128:85";
	private static final String DefaultSsn837All = "A3:33:IL";
	
	private static final String FutureBeneficiaryDateOfBirth837All = "A3:21:IL;A3:158:IL;A3:510:IL";
	private static final String BeneficiaryDateOfDeathBeforeDateOfService837P = "A3:21:IL;A3:159:IL";
	private static final String UnknownPatientGender837All = "A3:21:IL;A3:157:IL";
	private static final String PlaceOfServiceNotValid837DP = "A7:21:82;A7:249:82";
	private static final String FutureStatementServiceDates837I = "A7:21;A7:188;A7:510";
	private static final String FutureStatementServiceDates837P = "A7:21;A7:188;A7:510";
	private static final String FutureStatementServiceDates837D = "A7:21;A7:188;A7:510";
	private static final String FutureAdmissionDate837I = "A7:21;A7:189;A7:510";
	private static final String InvalidPatientStatus837I = "A7:21;A7:234";

   //START ICD-9 and ICD-10 INVALID CODES -- MEANING THEY DO NOT EXIST IN REJECT_REASON_CODE
   //Principal Diagnosis Code -- BK and ABK
	private static final String InvalidPrincipalDxCode837IP_ICD9 = "A7:254;A7:508";
	private static final String InvalidPrincipalDxCode837IP_ICD10 = "A7:254;A7:700";

   //Admitting Diagnosis Code -- BJ and ABJ
	private static final String InvalidAdmittingDxCode837I_ICD9 = "A7:232;A7:508";
	private static final String InvalidAdmittingDxCode837I_ICD10 = "A7:232;A7:700";

   //Reason For Visit Diagnosis Codes -- PR and APR
	private static final String InvalidPatientReasonForVisit837I_ICD9 = "A7:673;A7:508";
	private static final String InvalidPatientReasonForVisit837I_ICD10 = "A7:673;A7:700";

   //External Cause Of Injury Codes -- BN and ABN
   private static final String InvalidExternalCauseOfInjury837I_ICD9 = "A7:509;A7:508";
   private static final String InvalidExternalCauseOfInjury837I_ICD10 = "A7:509;A7:700";

   //Other Diagnosis Codes -- BF and ABF
	private static final String InvalidDxCode837IP_ICD9 = "A7:488;A7:508";
	private static final String InvalidDxCode837IP_ICD10 = "A7:488;A7:700";

   //Principal Procedure Codes -- BR and BBR
	private static final String InvalidPrinipalProcedureCodeHC837I_ICD9 = "A7:465;A7:508";
	private static final String InvalidPrinipalProcedureCodeHC837I_ICD10 = "A7:465;A7:700";

   //Other Procedure Codes -- BQ and BBQ
	private static final String InvalidOtherProcedureCode837I_ICD9 = "A7:490;A7:508";
	private static final String InvalidOtherProcedureCode837I_ICD10 = "A7:490;A7:700";
   //END ICD-9 and ICD-10 INVALID CODES -- MEANING THEY DO NOT EXIST IN REJECT_REASON_CODE

   //START ICD-9 and ICD-10 TIME LINE SPECIFIC REJECT REASON CODES
	private static final String ICD9_BF_AFTER_ICD10_GO_LIVE = "A7:745;A7:488;A7:508";
	private static final String ICD9_BJ_AFTER_ICD10_GO_LIVE = "A7:745;A7:232;A7:508";
	private static final String ICD9_BK_AFTER_ICD10_GO_LIVE = "A7:745;A7:254;A7:508";
	private static final String ICD9_BN_AFTER_ICD10_GO_LIVE = "A7:745;A7:509;A7:508";
	private static final String ICD9_BQ_AFTER_ICD10_GO_LIVE = "A7:745;A7:490;A7:508";
	private static final String ICD9_BR_AFTER_ICD10_GO_LIVE = "A7:745;A7:465;A7:508";
	private static final String ICD9_PR_AFTER_ICD10_GO_LIVE = "A7:745;A7:673;A7:508";

	private static final String ICD10_ABF_BEFORE_ICD10_GO_LIVE = "A7:745;A7:488;A7:700";
	private static final String ICD10_ABJ_BEFORE_ICD10_GO_LIVE = "A7:745;A7:232;A7:700";
	private static final String ICD10_ABK_BEFORE_ICD10_GO_LIVE = "A7:745;A7:254;A7:700";
	private static final String ICD10_ABN_BEFORE_ICD10_GO_LIVE = "A7:745;A7:509;A7:700";
	private static final String ICD10_APR_BEFORE_ICD10_GO_LIVE = "A7:745;A7:673;A7:700";
	private static final String ICD10_BBQ_BEFORE_ICD10_GO_LIVE = "A7:745;A7:490;A7:700";
	private static final String ICD10_BBR_BEFORE_ICD10_GO_LIVE = "A7:745;A7:465;A7:700";
   //END ICD-9 and ICD-10 TIME LINE SPECIFIC REJECT REASON CODES

	private static final String InvalidPrincipalProcedureCodeComposite837I = "A7:21;A7:465;A7:507";
	private static final String InvalidProcedureCodeComposite837I = "A7:21;A7:738";	
	//TWP - these are broken up because of the code
	private static final String InvalidPrincipalProcedureCodeComposite837P = "A7:21;A7:465;A7:507";
	private static final String InvalidProcedureCodeComposite837P = "A7:21;A7:738";
	private static final String InvalidProcedureCodeComposite837D = "A7:21;A7:454";
	private static final String InvalidRevenueCode837I = "A7:21;A7:455";
	private static final String MissingDentalDateofService837D = "A6:21;A6:187";
	private static final String InvalidBeneficiaryZipCode837ALL = "A3:21:IL;A3:500:IL";
	
   //MJS Added for ICD 10 Start
   private boolean   icd9              = false;
   private boolean   icd10             = false;
   private String    goLiveDate        = null;
   //MJS Added for ICD 10 End

   final void process(EntityManager em, String goLiveDt){
      goLiveDate = goLiveDt;
      LOGGER.debug("MJS -- Go Live Date is <" + goLiveDt + ">");
      process( em );
   }
	final void process(EntityManager entityManager) {
		this.em = entityManager;
		EntityTransaction tx = em.getTransaction();
		try {

			tx.begin();
			// This method performs the edit checks for every claim with
			// status_code = 'NEW'
			processClaims();
			insertProcessLog("ACCEPTED", iAccCnt);
			insertProcessLog("REJECTED", iRejCnt);
			LOGGER.info("Committing the database transaction");
			tx.commit();
			LOGGER.info("Commit successful");
			LOGGER.info("Accepted claims: " + iAccCnt);
			LOGGER.info("Rejected claims: " + iRejCnt);
		} catch (Exception ex) {

			LOGGER.error("An exception occured in FEE EDITS",ex);
			LOGGER.info("Rolling back the database transaction");
			if(tx.isActive())
			{
				tx.rollback();
			}
			if(currentWorkingClaim != null)
			{
				LOGGER.info("Error Processing Claim Key: " + currentWorkingClaim.getClaimKey());
				LOGGER.info("Error Processing File Number: " + currentWorkingClaim.getFileNumber());
				LOGGER.info("Error Processing St02 Control Number: " + currentWorkingClaim.getSt02txnCtlNbr());
			}else
			{
				LOGGER.info("Not currently working with a claim. ");
			}
		} finally {
			if (em != null && em.isOpen()) {
				em.close();
			}
		}
	}

	final void processClaims() throws SQLException {
		final int batchSize = 10;

		// TWP: Does the reciever ID need to be updated from 1211??

		Session session = (Session) em.getDelegate();
		String qstr = "select c from eClaim c "
			+ "left join fetch c.orthodonticInfo "
			+ "left join fetch c.homeHcareInfo "
			+ "left join fetch c.instClaimCode "
			+ "where c.statusCode = 'NEW' "
			+ "  and substr(c.receiverId,1,4) = '1211' "
			+ "  and c.claimType in ('"+INST_EDI+"', '"+PROF_EDI+"', '"+DENT_EDI+"') "

         //MJS Debugging Start
         //+ "  and c.claimKey = 'E3EE2A40-9BE3-BE11-E040-BF0A210C54EC' " //INST
         //+ "  and c.claimKey = 'E9983613-FF05-BB82-E040-BF0A210C5F8E' " //PROF
         //MJS Debugging End

			+ "order by c.dateCreated, c.claimType, c.fileNumber, c.st02txnCtlNbr ";
		ScrollableResults claims = session.createQuery(qstr).scroll(
				ScrollMode.FORWARD_ONLY);

		int count = 0;
		while (claims.next()) {

			Claim claim = (Claim) claims.get(0);
			currentWorkingClaim = claim;
			String vClaimType = claim.getClaimType();

         LOGGER.debug("MJS -- Claim Key: " + claim.getClaimKey() + " Claim Type: " + vClaimType);

         //MJS New Code Start
         setIcdBooleans( claim );
         //MJS New Code End

			boolean passedAllChecks = true;

			if (!checkClaimEncounterNotAccepted837All(claim))
				passedAllChecks = false;
			if (!checkForeignCurrencyNotAllowed837All(claim))
				passedAllChecks = false;
			if ((vClaimType.equalsIgnoreCase(DENT_EDI))
					|| (vClaimType.equalsIgnoreCase(PROF_EDI))) {
				if (!checkMissingBillingTin837DP(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(INST_EDI)) {
				if (!checkMissingBillingTin837I(claim))
					passedAllChecks = false;
			}
			if (!checkDefaultSsn837All(claim))
				passedAllChecks = false;
			if (!checkFutureBeneficiaryDateOfBirth837All(claim))
				passedAllChecks = false;
			if (vClaimType.equalsIgnoreCase(PROF_EDI)) {
				if (!checkBeneficiaryDateOfDeathBeforeDateOfService837P(claim))
					passedAllChecks = false;
			}
			if (!checkUnknownPatientGender837All(claim))
				passedAllChecks = false;
			if ((vClaimType.equalsIgnoreCase(DENT_EDI))
					|| (vClaimType.equalsIgnoreCase(PROF_EDI))) {
				if (!checkPlaceOfServiceNotValid837DP(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(INST_EDI)) {
				if (!checkFutureStatementServiceDates837I(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(PROF_EDI)) {
				if (!checkFutureStatementServiceDates837P(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(DENT_EDI)) {
				if (!checkFutureStatementServiceDates837D(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(INST_EDI)) {
				if (!checkFutureAdmissionDate837I(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(INST_EDI)) {
				if (!checkInvalidPatientStatus837I(claim))
					passedAllChecks = false;
			}

			// Take care of several fee edits related to diagnosis codes at once
			// InvalidPrincipalDxCode837IP_ICD9
			// InvalidPrincipalDxCode837IP_ICD10
			// InvalidAdmittingDxCode837I_ICD9
			// InvalidAdmittingDxCode837I_ICD10
			// InvalidPatientReasonForVisit837I_ICD9
			// InvalidPatientReasonForVisit837I_ICD10
			// InvalidDxCode837IP_ICD9 InvalidDxCode837IP_ICD10
			// InvalidPrinipalProcedureCodeHC837I_ICD9
			// InvalidPrinipalProcedureCodeHC837I_ICD10
			// InvalidOtherProcedureCode837I_ICD9
			// InvalidOtherProcedureCode837I_ICD10

         // MJS checkDxCodes validated
			if (!checkDxCodes(claim))
				passedAllChecks = false;

			if (vClaimType.equalsIgnoreCase(INST_EDI)) {
				if (!checkInvalidPrincipalAndOtherProcedureCodeComposite837I(claim))
					passedAllChecks = false;
			}

			if (vClaimType.equalsIgnoreCase(PROF_EDI)) {
				if (!checkInvalidProcedureCodeComposite837P(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(DENT_EDI)) {
				if (!checkInvalidProcedureCodeComposite837D(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(INST_EDI)) {
				if (!checkInvalidRevenueCode837I(claim))
					passedAllChecks = false;
			}
			if (vClaimType.equalsIgnoreCase(DENT_EDI)) {
				if (!checkMissingDentalDateofService837D(claim))
					passedAllChecks = false;
			}
			if (!checkInvalidBeneficiaryZipCode837ALL(claim))
				passedAllChecks = false;

			if (!passedAllChecks) {
				claim.setStatusCode("REJECTED");
				iRejCnt++;
			} else {
				claim.setStatusCode("ACCEPTED");
				iAccCnt++;
			}

			// keep the count the same as the batch size
			// in the persistence.xml file.
			if (++count % batchSize == 0) {
				session.flush();
				session.clear();
			}
			currentWorkingClaim = null;
		}

		LOGGER.debug("Processed {} claim(s)", count);

	}

	private boolean checkClaimEncounterNotAccepted837All(Claim c)
	throws SQLException {
		boolean passedAllCriteria = true;
		String qstr = "select tx from eTxnSet tx "
			+ "where tx.id.fileNumber = :file "
			+ "  and tx.id.st02txnCtlNbr = :ctlnbr ";
		Query q = em.createQuery(qstr);
		q.setParameter("file", c.getFileNumber());
		q.setParameter("ctlnbr", c.getSt02txnCtlNbr());
		Iterator iter = q.getResultList().iterator();
		if (iter.hasNext()) {
			TxnSet txnSet = (TxnSet) iter.next();
			if (txnSet.getBht06txnTypeCd() != null
					&& (txnSet.getBht06txnTypeCd().equalsIgnoreCase("RP") || txnSet
							.getBht06txnTypeCd().equalsIgnoreCase("31"))) {
				insertRejectClaim(c, ClaimEncounterNotAccepted837All,
						getPriority(ClaimEncounterNotAccepted837All));
				passedAllCriteria = false;
			} else {
				LOGGER.debug("getBht06txnTypeCd is null on claim: "
						+ c.getClaimKey());
			}
		}
		return passedAllCriteria;
	};

	/**
	 * This method checks that the billing provider that sent this claim in a
	 * transaction set is dealing in USD. In the code below Cur01=85 we check
	 * that we have billing provider currency data. We check that that data is
	 * USD which is 840 (840 is the standard code for USD defined in ISO 4217)
	 *
	 * @param c
	 * @return true if the claim contains compliant data false if the claim was
	 *         rejected because the data did not meet our criteria
	 * @throws SQLException
	 */
	private boolean checkForeignCurrencyNotAllowed837All(Claim c)
	throws SQLException {
		boolean passedAllCriteria = true;
		String qstr = "select f from eForeignCurrency f "
			+ "where f.txnSet.id.fileNumber = :file "
			+ "   and f.txnSet.id.st02txnCtlNbr = :ctlnbr " 
			+ "   and f.hrchlLevelKey = :hrchlLevelKey ";
		//TODO:Need to add Billing Hierarchical level key in here
		Query q = em.createQuery(qstr);
		q.setParameter("file", c.getFileNumber());
		q.setParameter("ctlnbr", c.getSt02txnCtlNbr());
		q.setParameter("hrchlLevelKey", c.getBillingHrchlLevelKey());
		Iterator iter = q.getResultList().iterator();

		// TWP: Followed old code where ForeignCurrency does not seem to be
		// required
		if (iter.hasNext()) {
			ForeignCurrency foreignCurrency = (ForeignCurrency) iter.next();
			if (!(foreignCurrency.getCur01crcyIdCd() != null
					&& foreignCurrency.getCur02crcyCd() != null && (foreignCurrency
							.getCur01crcyIdCd().equalsIgnoreCase("85") && foreignCurrency
							.getCur02crcyCd().equalsIgnoreCase("840") ))) {			
				insertRejectClaim(c, ForeignCurrencyNotAllowed837All,
						getPriority(ForeignCurrencyNotAllowed837All));
			}
		}
		return passedAllCriteria;
	};

	/**
	 * Called by both checkMissingBillingTin837DP and checkMissingBillingTin837I
	 * since thier logic is so similar. Checks to see if the transaction set
	 * associated with the claim has a billing provider and the proper
	 * additional information associated with it so that all the claims
	 * submitted by this providers claims may be adjudicated.
	 *
	 * (occurs at transaction set level)
	 *
	 * @param c
	 * @param ref01idQlfr
	 * @return true if the claim contains compliant data false if the claim was
	 *         rejected because the data did not meet our criteria
	 * @throws SQLException
	 */
	private boolean checkMissingBillingTinGeneric(Claim c, List ref01idQlfr, String errorString)
	throws SQLException {
		boolean passedAllCriteria = true;
		Query q2 = getEntitiesAdlIdInLoop2010AA_NM101_837_EISY_Query(c,
				ref01idQlfr);
		List results = q2.getResultList();
		Iterator iter2 = results.iterator();
		// To identify the provider we need either the employer
		// identificationi number E1
		// or the social security number (of the provider?) SY
		// If we have this we are good to go otherwise we need to reject
		// the claim
		if (iter2 == null || !iter2.hasNext()) {
			
			passedAllCriteria = false;
		}
		else {
			EntityAdtlId entityAdtlId = (EntityAdtlId)iter2.next();
			String tin = entityAdtlId.getRef02scndId();
			if(tin == null || tin.trim().isEmpty() || tin.length() != 9)
			{
				passedAllCriteria = false;	
			}
		}
	
		if (passedAllCriteria == false) {
			insertRejectClaim(c, errorString,
					getPriority(errorString));
		}		
		/*Query q1 = getEntitiesInLoop2010AA_NM101_85_Query(c);
		Iterator iter1 = q1.getResultList().iterator();
		if (iter1.hasNext()) {
			// Equals 85
			EntityName entityNameBill = (EntityName) iter1.next();
			// Equals XX
			// TWP Question - Is this nesting correct?
			if (entityNameBill.getNm108idCdQual() == null || entityNameBill.getNm109idCd() == null)
			{
				passedAllCriteria = false;
			}
			else if(entityNameBill.getNm108idCdQual().equalsIgnoreCase("XX")) {
				// Okay we have centers for medicaid services as the billing
				// provider at this point. (XX)
				Query q2 = getEntitiesAdlIdInLoop2010AA_NM101_837_EISY_Query(c,
						ref01idQlfr);
				List results = q2.getResultList();
				Iterator iter2 = results.iterator();
				// To identify the provider we need either the employer
				// identificationi number E1
				// or the social security number (of the provider?) SY
				// If we have this we are good to go otherwise we need to reject
				// the claim
				if (iter2 == null || !iter2.hasNext()) {
					passedAllCriteria = false;
				}
			} else {
				// Does not equal XX - this means that we have a billing
				// provider but it is not a national entity - e.g.
				// the centers for medicaid services - TWP i guess we are only
				// accepting CMS claims
				passedAllCriteria = false;
			}
		} else {
			// We don't have an entry for a billing provider - TWP I guess we
			// need one
			passedAllCriteria = false;
		}

		if (passedAllCriteria == false) {
			insertRejectClaim(c, errorString,
					getPriority(errorString));
		}*/

		return passedAllCriteria;
	}

	/**
	 * This method just return the query to use to see if there is a billing
	 * provider associated with the transaction set that is associated with the
	 * claim.
	 *
	 * @param c
	 * @return
	 */
	/*private Query getEntitiesInLoop2010AA_NM101_85_Query(Claim c) {
		String qstr1 = "select en from eEntityName en "
			+ "where en.txnSet.id.fileNumber = :file "
			+ "  and en.txnSet.id.st02txnCtlNbr = :ctlnbr "
			+ "  and en.loopId =  '2010AA' "
			+ "  and en.nm101entyIdCd = '85' ";
		Query q1 = em.createQuery(qstr1);
		q1.setParameter("file", c.getFileNumber());
		q1.setParameter("ctlnbr", c.getSt02txnCtlNbr());
		return q1;
	}*/

	// TWP: Check to see if the string array will really work in setting
	// ref01idQlfr in the IN clause
	private Query getEntitiesAdlIdInLoop2010AA_NM101_837_EISY_Query(Claim c,
			List ref01idQlfrs) {
		String qstr2 = "select ea from eEntityAdtlId ea "
			+ "where ea.txnSet.id.fileNumber = :file "
			+ "  and ea.txnSet.id.st02txnCtlNbr = :ctlnbr "
			+ "  and ea.loopId = '2010AA' "
			+ "  and ea.txType like '837%' "
			+ "  and ea.ref01idQlfr in (:ref01idQlfr) ";
		Query q2 = em.createQuery(qstr2);
		q2.setParameter("file", c.getFileNumber());
		q2.setParameter("ctlnbr", c.getSt02txnCtlNbr());
		q2.setParameter("ref01idQlfr", ref01idQlfrs);
		return q2;
	};

	/**
	 * Checks to make sure the dental or professional transaction set that is
	 * the parent of this claim has an SSN or EIN associated with the billing
	 * provider. We need that info to adjudicate the claim
	 *
	 * @param c
	 * @return true if the claim contains compliant data false if the claim was
	 *         rejected because the data did not meet our criteria
	 * @throws SQLException
	 */
	private boolean checkMissingBillingTin837DP(Claim c) throws SQLException {
		List ref01idQlfr = new ArrayList();
		ref01idQlfr.add("EI");
		ref01idQlfr.add("SY");
		return checkMissingBillingTinGeneric(c, ref01idQlfr, MissingBillingTin837DP);
	};

	/**
	 * Checks to make sure the institutional claim has an EIN for the billing
	 * provider of this claim We need the billing provider so we can adjudicate
	 * the claim otherwise we reject the claim. (Really you will reject every
	 * claim that is part of this transaction set that was sent from a
	 * particular provider)
	 *
	 * @param c
	 * @return true if the claim contains compliant data false if the claim was
	 *         rejected because the data did not meet our criteria
	 * @throws SQLException
	 */

	private boolean checkMissingBillingTin837I(Claim c) throws SQLException {
		List ref01idQlfr = new ArrayList();
		ref01idQlfr.add("EI");
		return checkMissingBillingTinGeneric(c, ref01idQlfr, MissingBillingTin837I);
	};

	/**
	 * Checks to make sure the SSN is reasonable.
	 *
	 * @param c
	 * @return true if the claim contains compliant data false if the claim was
	 *         rejected because the data did not meet our criteria
	 * @throws SQLException
	 */
	private boolean checkDefaultSsn837All(Claim c) throws SQLException {
		boolean passedAllCriteria = true;
		// The following regex string checks for the reasonableness of the
		// entered ssn.
        // Updated for SSA randomized assignment of SSNs (effective June 25, 2011)
		// String ssnRegexStr = "^(?!000)(?!666)([0-8]\\d{2})(?!00)\\d\\d(?!0000)\\d{4}$";
		
		/* 
		 * As part of the HAPE project.
		 * The FPPS Owner system will no longer apply the front end edits for checking Social Security Number 
		 * criteria on the Member Identification field on an 837 record.
		 */
		
		String ssnRegexStr = "^\\d{9}$";
		
		if (c.getSubscriberId().equalsIgnoreCase(INVALID_ID)
				|| !c.getSubscriberId().matches(ssnRegexStr)) {
			passedAllCriteria = false;
			insertRejectClaim(c, DefaultSsn837All,
					getPriority(DefaultSsn837All));
		}

		return passedAllCriteria;
	};

	/**
	 * This method checks that we have the beneficiaries birth date and it is
	 * not in the future. If we have a birth date in the past it returns true if
	 * we have a birth date in the future it returns false. Uses demographic
	 * data in loop 2010BA DMG02.
	 *
	 * @param c
	 * @return false if we are missing the DOB or it is in the future
	 * @throws SQLException
	 */
	private boolean checkFutureBeneficiaryDateOfBirth837All(Claim c)
	throws SQLException {
		boolean passedAllCriteria = true;
		String vDate = "";

		boolean isFutureDate = false;
		// TWP: Added HrchlLevelKey to get precise date from DMG segment
		String qstr = "select d from eDemographicInfo d "
			+ "where d.txnSet.id.fileNumber = :file "
			+ "  and d.txnSet.id.st02txnCtlNbr = :ctlnbr "
			+ "  and d.loopId = '2010BA' " 
			//TWP Added HrchlLevelKey to select correct date from DMG segment
			+ "  and d.hrchlLevelKey = :hrchlLevelKey";
		Query q = em.createQuery(qstr);
		q.setParameter("file", c.getFileNumber());
		q.setParameter("ctlnbr", c.getSt02txnCtlNbr());
		q.setParameter("hrchlLevelKey", c.getSubscriberHrchlLevelKey());
		Iterator iter = q.getResultList().iterator();
		if (iter.hasNext()) {
			DemographicInfo demographicInfo = (DemographicInfo) iter.next(); 
			vDate = demographicInfo.getDmg02brthDt();
			isFutureDate = editFutureDate(vDate);
			if (isFutureDate) {
				// Here we reject the claim if the beneficiaries date of birth
				// occurs in the future
				passedAllCriteria = false;
			}
		} else {
			// We are also rejecting the claim if there is no birth date
			passedAllCriteria = false;
		}

		if (!passedAllCriteria) {
			insertRejectClaim(c, FutureBeneficiaryDateOfBirth837All,
					getPriority(FutureBeneficiaryDateOfBirth837All));
		}

		return passedAllCriteria;
	};

	/**
	 * This method checks to make sure we have we do not have a date of death
	 * before the date of service. If we do not have a date of death we do not
	 * reject the claim. For 837P Patient Info is situational so we are not
	 * rejecting the claim if we do not have patient info.
	 * Sreedhar OKAY'ed
	 * @param c
	 * @return
	 * @throws SQLException
	 */
	private boolean checkBeneficiaryDateOfDeathBeforeDateOfService837P(Claim c)
	throws SQLException {
		boolean passedAllCriteria = true;

		PatientInfo patientInfo = null;
		
		String qstrp = "select pi from ePatientInfo pi "
			+ "where pi.fileNumber = :fileNumber "
			+ "  and pi.st02txnCtlNbr = :st02txnCtlNbr "
			+ "  and pi.nm109sbsrId = :nm109sbsrId ";

		Query qp = em.createQuery(qstrp);
		qp.setParameter("fileNumber", c.getFileNumber());
		qp.setParameter("st02txnCtlNbr", c.getSt02txnCtlNbr());
		qp.setParameter("nm109sbsrId", c.getSubscriberId());
		Iterator<PatientInfo> iterp = qp.getResultList().iterator();
		if (iterp.hasNext()) {
			patientInfo = (PatientInfo) iterp.next();
			if (patientInfo != null) {
				String deathDateString = patientInfo.getPat06deathDt();
				LOGGER.info("Pat Date Of Death : " + deathDateString + " SUBSCRIBER ID : " + c.getSubscriberId());
				LOGGER.info("FILE NUMBER : " + c.getFileNumber() + " ST02TXN_CTL_NBR : " + c.getSt02txnCtlNbr());
				if (deathDateString != null && deathDateString.trim().length() > 0) {
					Calendar deathDate = this.parseD8FomattedDate(deathDateString);
					String qstr = "select dt from eClaimDt dt "
						+ "where dt.claim.claimKey = :key "
						//+ "  and dt.txType = :type "
						+ "  and dt.dtp01dttmQual = '472' "
						//Sreedhar - Looking for Professional Service Line only
						+ "  and dt.prfsnSvcLine.serviceKey IS NOT NULL";
					Query q = em.createQuery(qstr);
					q.setParameter("key", c.getClaimKey());
					//q.setParameter("type", c.getClaimType());
					Iterator<ClaimDt> iter = q.getResultList().iterator();
					while (iter.hasNext()) {
						ClaimDt dateOfServiceDateObject = (ClaimDt) iter.next();
						Calendar dateOfService = getLastDate(dateOfServiceDateObject);
						if (!(dateOfService != null && dateOfService
								.before((deathDate)))) {
							// Reject the claim either because we are missing
							// service dates
							// or the service date is after the death date
							passedAllCriteria = false;
						}
					}
				}
			}
		}
		if (!passedAllCriteria) {
			insertRejectClaim(c, BeneficiaryDateOfDeathBeforeDateOfService837P,
					getPriority(BeneficiaryDateOfDeathBeforeDateOfService837P));
		}

		return passedAllCriteria;
	};

	/**
	 * Returns false if the patient gender is U - unknown otherwise it returns
	 * true.
	 *
	 * @param c
	 * @return
	 * @throws SQLException
	 */
	private boolean checkUnknownPatientGender837All(Claim c)
	throws SQLException {
		boolean passedAllCriteria = true;

		// TWP: Copied from existing code
		String vGender = "";
		String qstr = "select d from eDemographicInfo d "
			+ "where d.txnSet.id.fileNumber = :file "
			+ "  and d.txnSet.id.st02txnCtlNbr = :ctlnbr "
			+ "  and d.nm109sbsrId = :sbcrid "
			+ "  and d.loopId = '2010BA' "
			//TWP Added HrchlLevelKey to select correct Gender Code from DMG segment
			+ "  and d.hrchlLevelKey = :hrchlLevelKey";
		Query q = em.createQuery(qstr);
		q.setParameter("file", c.getFileNumber());
		q.setParameter("ctlnbr", c.getSt02txnCtlNbr());
		q.setParameter("sbcrid", c.getSubscriberId());
		q.setParameter("hrchlLevelKey", c.getSubscriberHrchlLevelKey());  //TWP: Added hrchlLevelKey 11072011
		Iterator iter = q.getResultList().iterator();
		if (iter.hasNext()) {
			DemographicInfo di = (DemographicInfo) iter.next();
			vGender = di.getDmg03gndr();
			if (vGender.equalsIgnoreCase("U")) {
				insertRejectClaim(c, UnknownPatientGender837All,
						getPriority(UnknownPatientGender837All));
				passedAllCriteria = false;
			}
		}

		return passedAllCriteria;
	};

	/**
	 * This method checks to see that the place of service code submitted on the
	 * incoming claim appears in the place of service table. i.e. it must be a
	 * facility we know.
	 *
	 * @param c
	 * @return
	 * @throws SQLException
	 */
	private boolean checkPlaceOfServiceNotValid837DP(Claim c)
	throws SQLException {
		boolean passedAllCriteria = true;

		// TWP: The requirements indicate we may have to loop over all places of
		// service but,
		// This code only gets a single place of service from the claim is this
		// correct or incorrect?
		String vPlaceOfService = c.getClm05fclyTypeCd();
		Query q = em
		.createQuery("select p from foPlaceOfService p where p.placeOfServiceCd = :code ");
		q.setParameter("code", vPlaceOfService);
		Iterator iter = q.getResultList().iterator();
		if (!iter.hasNext()) {
			insertRejectClaim(c, PlaceOfServiceNotValid837DP,
					getPriority(PlaceOfServiceNotValid837DP));
			passedAllCriteria = false;
		}
		return passedAllCriteria;
	};

	/**
	 * Checks that all the service dates with a given qualifier associated with
	 * a claim are valid, that is they do not occur in the future. If they are
	 * invalid the reject reason is inserted into the database.
	 *
	 * @param c
	 * @param dtp01dttmQual
	 * @param rejectReason
	 * @return
	 * @throws SQLException
	 */
	private boolean checkFutureStatementServiceDates837I(Claim c,
			String dtp01dttmQual, String rejectReason) throws SQLException {
		boolean passedAllCriteria = true;

		String serviceDate = "";
		String qstr = "select dt from eClaimDt dt "
			+ "where dt.claim.claimKey = :key "
			//+ "  and dt.txType = :type "
			+ "  and dt.dtp01dttmQual = :dtp01dttmQual "
			+ "  and dt.dtp02dttmFmtQual in ('RD8', 'D8 ')";
		Query q = em.createQuery(qstr);
		q.setParameter("key", c.getClaimKey());
		//q.setParameter("type", c.getClaimType());
		q.setParameter("dtp01dttmQual", dtp01dttmQual);
		Iterator iter = q.getResultList().iterator();

		while (iter.hasNext()) {
			ClaimDt serviceDateDescriptor = (ClaimDt) iter.next();
			serviceDate = serviceDateDescriptor.getDtp03dtFld();
			if (serviceDate != null && editFutureDate(serviceDate)) {
				passedAllCriteria = false;
			}
		}

		if (!passedAllCriteria) {
			insertRejectClaim(c, rejectReason, getPriority(rejectReason));
		}

		return passedAllCriteria;
	};

	private boolean checkFutureStatementServiceDates837PD(Claim c,
			String dtp01dttmQual, String rejectReason) throws SQLException {
		boolean passedAllCriteria = true;

		String serviceDate = "";
		String qstr = "select dt from eClaimDt dt "
			+ "where dt.claim.claimKey = :key "
			//+ "  and dt.txType = :type "
			+ "  and dt.dtp01dttmQual = :dtp01dttmQual "
			+ "  and dtp02dttmFmtQual in ('RD8', 'D8 ', 'D8')";
		
		//if (c.getClaimType().substring(0, 10).equalsIgnoreCase("005010X222")) {
		if (c.getClaimType().substring(0, 10).equalsIgnoreCase( PROF_EDI )) {
			qstr += "  and dt.prfsnSvcLine.serviceKey IS NOT NULL";
		//} else if (c.getClaimType().substring(0, 10).equalsIgnoreCase("005010X224")) {
		} else if (c.getClaimType().substring(0, 10).equalsIgnoreCase( DENT_EDI )) {
			qstr += "  and dt.dentalSvcLine.serviceKey IS NOT NULL";
		}
			
		Query q = em.createQuery(qstr);
		q.setParameter("key", c.getClaimKey());
		//q.setParameter("type", c.getClaimType());
		q.setParameter("dtp01dttmQual", dtp01dttmQual);
		Iterator iter = q.getResultList().iterator();

		while (iter.hasNext()) {
			ClaimDt serviceDateDescriptor = (ClaimDt) iter.next();
			serviceDate = serviceDateDescriptor.getDtp03dtFld();
			if (serviceDate != null && editFutureDate(serviceDate)) {
				passedAllCriteria = false;
			}
		}

		if (!passedAllCriteria) {
			insertRejectClaim(c, rejectReason, getPriority(rejectReason));
		}

		return passedAllCriteria;
	};
	
	/**
	 * Checks that all the service dates associated with the Institutional Claim
	 * are valid, that is they do not occur in the future. If they are invalid
	 * the rejected
	 *
	 * @param c
	 * @param dtp01dttmQual
	 * @param rejectReason
	 * @return
	 * @throws SQLException
	 */
	private boolean checkFutureStatementServiceDates837I(Claim c)
	throws SQLException {
		return checkFutureStatementServiceDates837I(c, "434",
				FutureStatementServiceDates837I);
	};

	/**
	 * Checks that all the service dates associated with the Professional Claim
	 * are valid, that is they do not occur in the future. If they are invalid
	 * the rejected
	 *
	 * @param c
	 * @param dtp01dttmQual
	 * @param rejectReason
	 * @return
	 * @throws SQLException
	 */
	private boolean checkFutureStatementServiceDates837P(Claim c)
	throws SQLException {
		return checkFutureStatementServiceDates837PD(c, "472",
				FutureStatementServiceDates837P);
	};

	/**
	 * Checks that all the service dates associated with the Professional Claim
	 * are valid, that is they do not occur in the future. If they are invalid
	 * they are rejected
	 *
	 * @param c
	 * @param dtp01dttmQual
	 * @param rejectReason
	 * @return
	 * @throws SQLException
	 */
	private boolean checkFutureStatementServiceDates837D(Claim c)
	throws SQLException {
		return checkFutureStatementServiceDates837PD(c, "472",
				FutureStatementServiceDates837D);
	};

	/**
	 * Checks to make sure the admission date does not occur in the future.
	 *
	 * @param c
	 * @return
	 * @throws SQLException
	 */
	private boolean checkFutureAdmissionDate837I(Claim c) throws SQLException {
		boolean passedAllCriteria = true;
		// Selects the date associated with institutional claim that has the
		// qualifier 435 - Admission Date
		String vDate = getDate(c, "435", "837I");
		// This date is situational so we do not blanked reject it if it is not
		// present
		// we only reject it if it is in the future.
		if (!vDate.equals("") && editFutureDate(vDate)) {
			insertRejectClaim(c, FutureAdmissionDate837I,
					getPriority(FutureAdmissionDate837I));
			passedAllCriteria = false;
		}
		return passedAllCriteria;
	};

	/**
	 * Validates Patient Status Code (CL103).  Patient status code is only validated
	 * for in-patient claims.
	 * @param c
	 * @return true if the patient status code is found in
	 * 		FPPS_OWNER.PATIENT_STATUS.PATIENT_STATUS_CD.
	 * @throws SQLException
	 */
	private boolean checkInvalidPatientStatus837I(Claim c) throws SQLException {
		boolean passedAllCriteria = true;
		String vPatientStatus = "";

		Query q = em
		.createQuery("select i from eInstClaimCode i where i.claim.claimKey = :key ");
		q.setParameter("key", c.getClaimKey());
		List<?> instClaimCodes = q.getResultList();
		if (instClaimCodes.size() > 0 && instClaimCodes.get(0) != null) {
			InstClaimCode instClaimCode = (InstClaimCode) instClaimCodes
			.get(0);
			if (instClaimCode.getCl103ptStatCd() != null) {
				vPatientStatus = instClaimCode.getCl103ptStatCd();
				Query qpts = em
				.createQuery("select ps from foPatientStatus ps where ps.patientStatusCd = :statcd");
				qpts.setParameter("statcd", vPatientStatus);
				List<?> patientStatuses = qpts.getResultList();
				if (patientStatuses.size() == 0
						|| patientStatuses.get(0) == null) {
					passedAllCriteria = false;
				}
			} else {
				passedAllCriteria = false;
			}
		} else {
			passedAllCriteria = false;
		}
		if (!passedAllCriteria) {
			insertRejectClaim(c, InvalidPatientStatus837I,
					getPriority(InvalidPatientStatus837I));
		}
		return passedAllCriteria;
	};

	final boolean checkDxCodes(Claim c) throws SQLException {
		boolean passedAllCriteria = true;

		Query q = em.createQuery("select hi from eHiCodes hi where hi.claim.claimKey = :key order by hi.id.seqNbr");
		q.setParameter("key", c.getClaimKey());

		Iterator iter = q.getResultList().iterator();

		// Check all the diagnosis codes associated with the claim for
		// compliance.
		while (iter.hasNext()) {
			HiCodes rowHiCodes = (HiCodes) iter.next();
			// TWP: Do I still skip rows that start with this qualifier - are
			// there any others to ignore
			if (!(rowHiCodes.getHi01cdListQual().equalsIgnoreCase("DR"))) {
				if (rowHiCodes.getHi01procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi01cdListQual()
							.trim(), rowHiCodes.getHi01procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi02procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi02cdListQual()
							.trim(), rowHiCodes.getHi02procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi03procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi03cdListQual()
							.trim(), rowHiCodes.getHi03procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi04procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi04cdListQual()
							.trim(), rowHiCodes.getHi04procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi05procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi05cdListQual()
							.trim(), rowHiCodes.getHi05procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi06procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi06cdListQual()
							.trim(), rowHiCodes.getHi06procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi07procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi07cdListQual()
							.trim(), rowHiCodes.getHi07procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi08procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi08cdListQual()
							.trim(), rowHiCodes.getHi08procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi09procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi09cdListQual()
							.trim(), rowHiCodes.getHi09procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi10procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi10cdListQual()
							.trim(), rowHiCodes.getHi10procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi11procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi11cdListQual()
							.trim(), rowHiCodes.getHi11procCd()))
						passedAllCriteria = false;
				}

				if (rowHiCodes.getHi12procCd() != null) {
					if (!processIcdCodes(c, rowHiCodes.getHi12cdListQual()
							.trim(), rowHiCodes.getHi12procCd()))
						passedAllCriteria = false;
				}
			}
		}
		return passedAllCriteria;
	}

   //MJS New Code for ICD 10
   final void setIcdBooleans( Claim c ) {
      icd9  = false;
      icd10 = false;

      String latestClmDt = getClaimsLatestServiceDate( c );

      LOGGER.debug( "latestClmDt <" + latestClmDt + ">" );
      if ( latestClmDt == null || latestClmDt.equals("") ) {
         LOGGER.debug( "MJS -- Latest claim date for claim " + c.getClaimKey() + " is null -- defaulting to ICD-10" );
         icd10 = true;
      } else {
         if ( latestClmDt.length() > 8 ) latestClmDt = latestClmDt.substring(0,8);

         if (Integer.parseInt( latestClmDt ) < Integer.parseInt( goLiveDate ) ) {
            icd9 = true;
            LOGGER.debug("MJS -- Only ICD 9 codes are valid");
         } else {
            icd10 = true;
            LOGGER.debug("MJS -- Only ICD 10 codes are valid");
         }
      }
   }

   //MJS New Code for ICD 10
   final boolean validateCodeSet( Claim c, String vHiQual ) throws SQLException {
      boolean codeSetValidated = true;
      String  rejectCode       = null;

      if ( icd9 ) {
         if ( c.getClaimType().equals( PROF_EDI ) ) {
            if ( vHiQual.equalsIgnoreCase( "ABF" ) ) {
               rejectCode = ICD10_ABF_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "ABK" ) ) {
               rejectCode = ICD10_ABK_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            }
         } else if ( c.getClaimType().equals( INST_EDI ) ) {
            if ( vHiQual.equalsIgnoreCase( "ABF" ) ) {
               rejectCode = ICD10_ABF_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "ABJ" ) ) {
               rejectCode = ICD10_ABJ_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "ABK" ) ) {
               rejectCode = ICD10_ABK_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "ABN" ) ) {
               rejectCode = ICD10_ABN_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "APR" ) ) {
               rejectCode = ICD10_APR_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "BBQ" ) ) {
               rejectCode = ICD10_BBQ_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "BBR" ) ) {
               rejectCode = ICD10_BBR_BEFORE_ICD10_GO_LIVE;
               codeSetValidated = false;
            }
         }

         if ( ! codeSetValidated ) insertRejectClaim( c, rejectCode, getPriority( rejectCode ) );
      } else if ( icd10 ) {
         if ( c.getClaimType().equals( PROF_EDI ) ) {
            if ( vHiQual.equalsIgnoreCase( "BF" ) ) {
               rejectCode = ICD9_BF_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "BK" ) ) {
               rejectCode = ICD9_BK_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            }
         } else if ( c.getClaimType().equals( INST_EDI ) ) {
            if ( vHiQual.equalsIgnoreCase( "BF" ) ) {
               rejectCode = ICD9_BF_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "BJ" ) ) {
               rejectCode = ICD9_BJ_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "BK" ) ) {
               rejectCode = ICD9_BK_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "BN" ) ) {
               rejectCode = ICD9_BN_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "BQ" ) ) {
               rejectCode = ICD9_BQ_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "BR" ) ) {
               rejectCode = ICD9_BR_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            } else if ( vHiQual.equalsIgnoreCase( "PR" ) ) {
               rejectCode = ICD9_PR_AFTER_ICD10_GO_LIVE;
               codeSetValidated = false;
            }
         }

         if ( ! codeSetValidated ) insertRejectClaim( c, rejectCode, getPriority( rejectCode ) );
      }

      return codeSetValidated;
   }

	final boolean processIcdCodes(Claim c, String vHiQual, String vHiCode)
	throws SQLException {
		boolean passedAllCriteria = true;
      String  rejectCode = null;

		String vClaimType = c.getClaimType();
		vHiQual = vHiQual.trim();

		String vHiCd = decimalCodes(vHiQual, vHiCode);

      LOGGER.debug("MJS -- Begin processing ICD Codes -- vHiQual " + vHiQual + " vHiCode " + vHiCode + " vHiCd " + vHiCd);

		// We only check hicodes for I an P (really mostly I)
      // MJS -- For ICD 10 we must add an additional check for the code set associated with the claim.  We must still
      // MJS -- check on the validity of the HI Code.  That is, even if we are supposed to be using ICD 10 and we receive
      // MJS -- ICD 9 HI codes, we need to check against the ICD 9 code set to see if the HI code is valid.
		if (vClaimType.equalsIgnoreCase(INST_EDI) || vClaimType.equalsIgnoreCase(PROF_EDI)) {
			if (vHiQual.equalsIgnoreCase("BK")) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidPrincipalDxCode837IP_ICD9) )
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("BJ") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidAdmittingDxCode837I_ICD9) )
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("PR") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if (  !validateIcdCode(c, vHiQual, vHiCd, InvalidPatientReasonForVisit837I_ICD9))
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("BF")) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidDxCode837IP_ICD9))
					passedAllCriteria = false;
         } else if (vHiQual.equalsIgnoreCase("BN") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidExternalCauseOfInjury837I_ICD9))
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("BR") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if (!validateIcdPxCode(c, vHiQual, vHiCd, InvalidPrinipalProcedureCodeHC837I_ICD9))
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("BQ") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if (!validateIcdPxCode(c, vHiQual, vHiCd, InvalidOtherProcedureCode837I_ICD9))
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("ABK")) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidPrincipalDxCode837IP_ICD10) )
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("ABJ") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidAdmittingDxCode837I_ICD10) )
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("APR") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidPatientReasonForVisit837I_ICD10) )
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("ABF")) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidDxCode837IP_ICD10) )
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("ABN") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if ( !validateIcdCode(c, vHiQual, vHiCd, InvalidExternalCauseOfInjury837I_ICD10) )
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("BBR") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if (!validateIcdPxCode(c, vHiQual, vHiCd, InvalidPrinipalProcedureCodeHC837I_ICD10))
					passedAllCriteria = false;
			} else if (vHiQual.equalsIgnoreCase("BBQ") && vClaimType.equalsIgnoreCase(INST_EDI)) {
            passedAllCriteria = validateCodeSet( c, vHiQual );
				if (!validateIcdPxCode(c, vHiQual, vHiCd, InvalidOtherProcedureCode837I_ICD10))
					passedAllCriteria = false;
			}
		}

		return passedAllCriteria;
	}

	@SuppressWarnings("unchecked")
	final boolean validateIcdCode(Claim c, String vHiQual, String vHiCode, String vRejCd) throws SQLException {
		boolean passedAllCriteria = true;
		String icdDxId = null;

		String claimsLatestServiceDate = getClaimsLatestServiceDate(c);

		icdDxId = getIcdDxId(claimsLatestServiceDate, vHiQual, vHiCode);

      //MJS -- Cannot reference null string after call to getIcdDxId finds nothing
		if ( icdDxId == null ) {
         LOGGER.debug( "MJS -- In validateIcdCode after call to geticdDxId() and icdDxId \"null\"\n and vRejCd = <" + vRejCd + ">" );
			insertRejectClaim(c, vRejCd, getPriority(vRejCd));
			passedAllCriteria = false;
		}

		return passedAllCriteria;
	}

	//Sreedhar - Added new method to validate ICD-9 Procedure codes from Medical_Code table
	@SuppressWarnings("unchecked")
	final boolean validateIcdPxCode(Claim c, String vHiQual, String vHiCode, String vRejCd) throws SQLException {
		boolean passedAllCriteria = true;
		String icdPxId = null;

		String claimsLatestServiceDate = getClaimsLatestServiceDate(c);

		icdPxId = getIcdPxId(claimsLatestServiceDate, vHiQual, vHiCode);

		if (icdPxId == null ) {
         LOGGER.debug( "MJS -- In validateIcdPxCode after call to geticdPxId() and icdPxId \"null\"\n and vRejCd = <" + vRejCd + ">" );
			insertRejectClaim(c, vRejCd, getPriority(vRejCd));
			passedAllCriteria = false;
		}
		return passedAllCriteria;
	}

   public String getClaimsLatestServiceDate( Claim c ) {
		Calendar latestServiceDate;
      Calendar serviceDate;
      ClaimDt  serviceDateDescriptor;
      String   cType;
      String   latestServiceDateString;
      String   qstr;
      String   serviceDateString;

      //set the latest date to 1/1/1900, which should be earlier than any claim date we encounter
		latestServiceDate = Calendar.getInstance();
      latestServiceDate.set( 1900, 0, 1, 0, 0, 0 );
      //System.out.println( latestServiceDate );

		latestServiceDateString = "";

		//Sreedhar - Changed the where clause to use appropriate values and conditions
		qstr = "select dt " +
               "from eClaimDt dt " +
              "where dt.claim.claimKey = :key " + 
                "and dtp02dttmFmtQual in ('RD8', 'D8 ', 'D8')";
		
		cType = c.getClaimType().substring(0, 10);
		
		if (cType.equalsIgnoreCase( INST_EDI )) {
			qstr += "  and dt.dtp01dttmQual = '434' ";
		} else if (cType.equalsIgnoreCase( PROF_EDI )) {
			qstr += "  and dt.dtp01dttmQual = '472' and dt.prfsnSvcLine.serviceKey IS NOT NULL";
		} else if (cType.equalsIgnoreCase( DENT_EDI )) {
			qstr += "  and dt.dtp01dttmQual = '472' and dt.dentalSvcLine.serviceKey IS NOT NULL";
		}
		
		Query q = em.createQuery(qstr);
		q.setParameter("key", c.getClaimKey());

		Iterator iter = q.getResultList().iterator();

		while (iter.hasNext()) {
			serviceDateDescriptor = (ClaimDt) iter.next();

			serviceDateString = serviceDateDescriptor.getDtp03dtFld();

         if ( serviceDateDescriptor.getDtp02dttmFmtQual().equalsIgnoreCase( "RD8" ) ) {
			   serviceDate       = parseRD8FomattedDate( serviceDateString )[1];
            serviceDateString = serviceDateString.substring( 9 );
         } else {
			   serviceDate       = parseD8FomattedDate( serviceDateString );
         }

			if(serviceDate.after(latestServiceDate)) {
				latestServiceDate = serviceDate;
				latestServiceDateString = serviceDateString;
			}
		}

		return latestServiceDateString;
	}

	public String getClaimsEarliestServiceDate( Claim c )
	{
		Calendar earliestServiceDate = Calendar.getInstance();
		String earliestServiceDateString = "";

		//Sreedhar - Changed the where clause to use appropriate values and conditions
		String qstr = "select dt from eClaimDt dt "
			+ "where dt.claim.claimKey = :key "
			+ "  and dtp02dttmFmtQual in ('RD8', 'D8 ', 'D8')";
		
		String cType = c.getClaimType().substring(0, 10);
		
		//if (cType.equalsIgnoreCase("005010X223")) {
		if (cType.equalsIgnoreCase( INST_EDI )) {
			qstr += "  and dt.dtp01dttmQual = '434' ";
		//} else if (cType.equalsIgnoreCase("005010X222")) {
		} else if (cType.equalsIgnoreCase( PROF_EDI )) {
			qstr += "  and dt.dtp01dttmQual = '472' and dt.prfsnSvcLine.serviceKey IS NOT NULL";
		//} else if (cType.equalsIgnoreCase("005010X224")) {
		} else if (cType.equalsIgnoreCase( DENT_EDI )) {
			qstr += "  and dt.dtp01dttmQual = '472' and dt.dentalSvcLine.serviceKey IS NOT NULL";
		}
		
		Query q = em.createQuery(qstr);
		q.setParameter("key", c.getClaimKey());

		Iterator iter = q.getResultList().iterator();

		while (iter.hasNext()) {
			ClaimDt serviceDateDescriptor = (ClaimDt) iter.next();
			String serviceDateString = serviceDateDescriptor.getDtp03dtFld();
			Calendar serviceDate = parseD8FomattedDate(serviceDateString);
			if(serviceDate.before(earliestServiceDate))
			{
				earliestServiceDate = serviceDate;
				earliestServiceDateString = serviceDateString;
			}
		}
		return earliestServiceDateString;
	}

	/**
	 * We have accommodated ICD-10 qualifiers as much as possible
	 * but there are no requirements for where to place the decimal
	 * for ICD-10 codes.
	 * @param vHiQual
	 * @param vHiCode
	 * @return
	 */
	final String decimalCodes(String vHiQual, String vHiCode) {
		final int cd2Ln = 2;
		final int cd3Ln = 3;
		final int cd4Ln = 4;
		int vCodeLen = vHiCode.length();

      //ICD 9 Formatting
		if ((vHiQual.trim().equalsIgnoreCase("BF"))
				|| (vHiQual.trim().equalsIgnoreCase("BK"))
				|| (vHiQual.trim().equalsIgnoreCase("BJ"))
				|| (vHiQual.trim().equalsIgnoreCase("PR"))
				|| (vHiQual.trim().equalsIgnoreCase("ZZ"))) {
			if (vHiCode.startsWith("V")) {
				if (vCodeLen > cd3Ln) {
					vHiCode = vHiCode.substring(0, cd3Ln) + "."
					+ vHiCode.substring(cd3Ln, vCodeLen);
				}
			} else if (vHiCode.startsWith("E")) {
				if (vCodeLen > cd4Ln) {
					vHiCode = vHiCode.substring(0, vCodeLen - 1) + "."
					+ vHiCode.substring(vCodeLen - 1, vCodeLen);
				}
			} else {
				if (vCodeLen > cd3Ln) {
					vHiCode = vHiCode.substring(0, cd3Ln) + "."
					+ vHiCode.substring(cd3Ln, vCodeLen);
				}
			}
		}

		if ((vHiQual.trim().equalsIgnoreCase("BR") || vHiQual.trim()
				.equalsIgnoreCase("BQ"))
				&& (vCodeLen > cd2Ln)) {
			vHiCode = vHiCode.substring(0, cd2Ln) + "."
			+ vHiCode.substring(cd2Ln, vCodeLen);
		}

		if (vHiQual.trim().equalsIgnoreCase("BN") && (vCodeLen > cd4Ln)) {
			vHiCode = vHiCode.substring(0, vCodeLen - 1) + "."
			+ vHiCode.substring(vCodeLen - 1, vCodeLen);
		}

      //MJS -- ICD 10 formatting...Use the 3 char ICD qualifier
      //if the length is > 3 then decimal always goes in position 4 else no decimal
      if ( vHiQual.trim().equalsIgnoreCase("ABF") ||
			  vHiQual.trim().equalsIgnoreCase("ABK") ||
           vHiQual.trim().equalsIgnoreCase("ABJ") ||
           vHiQual.trim().equalsIgnoreCase("ABN") ||
           vHiQual.trim().equalsIgnoreCase("APR") ||
           vHiQual.trim().equalsIgnoreCase("BBR") ||
           vHiQual.trim().equalsIgnoreCase("BBQ") ) {
         if ( vHiCode.trim().length() > 3 ) {
            vHiCode = vHiCode.substring(0,3) + "." + vHiCode.substring(3);
         }
      }

		return vHiCode;
	}

   //MJS -- New version including the qualifier so we can determine ICD-9 or ICD-10
	final String getIcdDxId(String earliestServiceDate, String vHiQual, String vIcdDxCode) {
      String codeType = null;
      String esd      = null;
      String sql      = null;
		String vIcdDxId = null;

      esd = earliestServiceDate;
      if ( esd.length() > 8 ) esd = esd.substring(0,8);
		
      //There are 600 or so overlapping ICD-9 and ICD-10 dx codes in ICD_CODE. The CODE_TYPE column is what 
      //differentiates them.  ICD-9 = -9999999, ICD-10 = 1.  Set it based on the length of the qualifier.  If it is 3
      //we are looking for ICD-10, if it is 2 we are looking for ICD-9
      codeType = (vHiQual.length() == 3) ? "1" : "-9999999"; 

      LOGGER.debug("MJS -- getIcdDxId() querying for codeType <" + codeType + ">" );

      sql = "SELECT * "                                     +
            "  FROM icd_code icd "                          +
            " WHERE icd.icd_code = '" + vIcdDxCode + "' "   +
            "   AND ( (icd.expiration_date >= TO_DATE('" + esd + "','YYYYMMDD')) " +
            "    OR icd.expiration_date IS NULL ) " +
            "   AND icd.code_type = '" + codeType + "'"     +
            " ORDER BY icd.expiration_date DESC";

      LOGGER.debug( "MJS -- getIcdDxId() sql is <" + sql + ">" );

		Query  q      = em.createNativeQuery( sql, IcdCode.class );
		Iterator iter = q.getResultList().iterator();

		if (iter.hasNext()) {
			IcdCode icdDxCode = (IcdCode) iter.next();
			vIcdDxId = String.valueOf(icdDxCode.getIcdCdId());
		}

      LOGGER.debug( "MJS -- After query vIcdDxId is <" + vIcdDxId + ">" );

		return vIcdDxId;
	}

   //MJS -- New version including the qualifier so we can determine ICD-9 or ICD-10
   final String getIcdPxId(String earliestServiceDate, String vHiQual, String vIcdPxCode) {
      String esd             = null;
      String medicalCodeType = null;
      String sql             = null;
		String vIcdPxId        = null;
		String cleanIcdPxCode  = null;

      LOGGER.debug( "MJS -- In getIcdPxId() with qualifier " + vHiQual + " and code " + vIcdPxCode + " and svc date " + earliestServiceDate );

      esd = earliestServiceDate;
      if ( esd.length() > 8 ) esd = esd.substring(0,8);

      cleanIcdPxCode = vIcdPxCode;

      LOGGER.debug( "MJS -- In getIcdPxId() and cleanIcdPxCode is <{}>", cleanIcdPxCode );

      if ( vHiQual.length() == 3 ) {
         medicalCodeType = "ICD-10 PCS CODE";
         cleanIcdPxCode = cleanIcdPxCode.replaceAll("\\.","");
      } else {
         medicalCodeType = "ICD-9 PROCEDURE"; 
      }

      LOGGER.debug(
         "MJS -- In getIcdPxId() and medicalCodeType is <{}> and cleanIcdPxCode is <{}>", medicalCodeType, cleanIcdPxCode
      );

      sql = "SELECT * "                                                    +
            "  FROM MEDICAL_CODE icdPx "                                   +
            " WHERE icdPx.MEDICAL_CD = '" + cleanIcdPxCode + "'"               +
            "   AND icdPx.MEDICAL_CODE_TYPE = '" + medicalCodeType + "'"   +
            "   AND ( (icdPx.EXPIRATION_DATE >= TO_DATE('" + esd + "','YYYYMMDD')) " +
            "    OR icdPx.EXPIRATION_DATE IS NULL) "                       +
            " ORDER BY icdPx.EXPIRATION_DATE DESC";

      LOGGER.debug( "MJS -- in getIcdPxId() and sql is {}", sql );

		//System.out.println("Earliest Date : " + earliestServiceDate);
		Query q = em.createNativeQuery( sql, MedicalCode.class );

		Iterator iter = q.getResultList().iterator();

		if (iter.hasNext()) {
			MedicalCode icdPxCode = (MedicalCode) iter.next();
			vIcdPxId = String.valueOf(icdPxCode.getMedicalCd());
		}

      if ( vIcdPxId != null ) LOGGER.debug( "MJS -- After query vIcdPxId is <" + vIcdPxId + ">" );

		return vIcdPxId;
	}

	private boolean checkInvalidPrincipalAndOtherProcedureCodeComposite837I(Claim c)
	throws SQLException {
		boolean passedAllCriteria = true;
		passedAllCriteria = checkProcedureCodesComposite837I(c,
				InvalidPrincipalProcedureCodeComposite837I, InvalidProcedureCodeComposite837I);

		return passedAllCriteria;
	}

	
	//Sreedhar - Created separate method for 837I
	private boolean checkProcedureCodesComposite837I(Claim c,
			String invalidPrincipalProcedureCodeError, String InvalidProcedureCodeError ) throws SQLException {
		String vProcCodeId = "";
		String vProcQual = "";
		boolean passedAllCriteria=true;

		Query q = em
		.createQuery("select l from eInstSvcLine l where l.claim.claimKey = :key order by l.lx01asgnLnNbr");
		q.setParameter("key", c.getClaimKey());
		Iterator it = q.getResultList().iterator();
		while (it.hasNext()) {
			InstSvcLine sl = (InstSvcLine) it.next();
			vProcQual = sl.getSv202svcIdQual();
			vProcCodeId = getMedicalCodeId(sl.getSv202procCd(), vProcQual);
			if (vProcQual != null) {
				vProcQual = vProcQual.trim();
				if ("IV".equalsIgnoreCase(vProcQual)) {
					insertRejectClaim(
							c,
							InvalidProcedureCodeError,
							getPriority(InvalidProcedureCodeError));
					passedAllCriteria=false;
				}
			}
			//Different reject codes
			if ("".equals(vProcCodeId) && vProcQual != null
					&& "HC".equalsIgnoreCase(vProcQual)) {
				insertRejectClaim(c,
						invalidPrincipalProcedureCodeError,
						getPriority(invalidPrincipalProcedureCodeError));
				passedAllCriteria=false;
			}
		}
		return passedAllCriteria;
	};

	//Sreedhar - Created separate methods for 837P
	private boolean checkProcedureCodesComposite837P(Claim c,
			String invalidPrincipalProcedureCodeError, String InvalidProcedureCodeError ) throws SQLException {
		String vProcCodeId = "";
		String vProcQual = "";
		boolean passedAllCriteria=true;

		Query q = em
		.createQuery("select l from ePrfsnSvcLine l where l.claim.claimKey = :key order by l.lx01asgnLnItm");
		q.setParameter("key", c.getClaimKey());
		Iterator it = q.getResultList().iterator();
		while (it.hasNext()) {
			PrfsnSvcLine sl = (PrfsnSvcLine) it.next();
			vProcQual = sl.getSv101procCdQual();
			vProcCodeId = getMedicalCodeId(sl.getSv101procCd(), vProcQual);
			if (vProcQual != null) {
				vProcQual = vProcQual.trim();
				if ("IV".equalsIgnoreCase(vProcQual)) {
					insertRejectClaim(
							c,
							InvalidProcedureCodeError,
							getPriority(InvalidProcedureCodeError));
					passedAllCriteria=false;
				}
			}
			//Different reject codes
			if ("".equals(vProcCodeId) && vProcQual != null
					&& "HC".equalsIgnoreCase(vProcQual)) {
				insertRejectClaim(c,
						invalidPrincipalProcedureCodeError,
						getPriority(invalidPrincipalProcedureCodeError));
				passedAllCriteria=false;
			}
		}
		return passedAllCriteria;
	};
	
	final String getMedicalCodeId(String procCode, String qualifier) {
		String retVal = "";
		StringBuilder qstrb = new StringBuilder(
		"select m from foMedicalCode m ");
		if (qualifier == null) {
			qstrb.append("where m.medicalCd = '").append(procCode).append("' ");
		} else {
			qstrb.append("where m.medicalCd = '").append(procCode).append(
			"' and m.medicalCodeType in ('HCPCS CODE', 'CPT CODE')");
		}
		Query q = em.createQuery(qstrb.toString());
		Iterator iter = q.getResultList().iterator();
		if (iter.hasNext()) {
			MedicalCode mc = (MedicalCode) iter.next();
			retVal = String.valueOf(mc.getMedicalCdId());
		}
		return retVal;
	}


	private boolean checkInvalidProcedureCodeComposite837P(Claim c) throws SQLException {
		boolean passedAllCriteria = true;
		passedAllCriteria = checkProcedureCodesComposite837P(c,InvalidPrincipalProcedureCodeComposite837P, InvalidProcedureCodeComposite837P);
		return passedAllCriteria;
	};

	/**
	 * TWP Looks correct
	 * @param c
	 * @return
	 * @throws SQLException
	 */
	private boolean checkInvalidProcedureCodeComposite837D(Claim c) throws SQLException {
		boolean passedAllCriteria = true;
		String vProcCodeId = "";
		String qstr = "select l from eDentalSvcLine l "
			+ "where l.claim.claimKey = :key "
			+ "and l.sv301adaCdQual = 'AD' "
			+ "order by l.lx01asgnLnItm";
		Query q = em.createQuery(qstr);
		q.setParameter("key", c.getClaimKey());
		Iterator it = q.getResultList().iterator();
		while (it.hasNext()) {
			DentalSvcLine sl = (DentalSvcLine) it.next();
			if (sl.getSv301adaCd() != null) {
				vProcCodeId = getMedicalCodeId(sl.getSv301adaCd(), null);
			}
			if ("".equals(vProcCodeId)) {
				insertRejectClaim(c, InvalidProcedureCodeComposite837D, getPriority(InvalidProcedureCodeComposite837D));
				passedAllCriteria=false;
			}
		}

		return passedAllCriteria;
	};

	/**
	 * TWP Looks Correct.
	 * @param c
	 * @return
	 * @throws SQLException
	 */
	private boolean checkInvalidRevenueCode837I(Claim c) throws SQLException {
		boolean passedAllCriteria = true;
		String vRevCode = "";
		Query q = em.createQuery("select l from eInstSvcLine l where l.claim.claimKey = :key order by l.lx01asgnLnNbr");
		q.setParameter("key", c.getClaimKey());
		Iterator it = q.getResultList().iterator();
		while (it.hasNext()) {
			InstSvcLine l = (InstSvcLine) it.next();
			vRevCode = l.getSv201svcLnRevCd();
			Query q1 = em.createQuery("select r from foRevenueCode r where r.revenueCd = :code ");
			q1.setParameter("code", vRevCode);
			Iterator iter = q1.getResultList().iterator();
			if (!iter.hasNext()) {
				insertRejectClaim(c, InvalidRevenueCode837I, getPriority(InvalidRevenueCode837I));
				passedAllCriteria=false;
			}
		}
		return passedAllCriteria;
	};

	private boolean checkMissingDentalDateofService837D(Claim c) throws SQLException {
		boolean passedAllCriteria = true;


		String strQry = "SELECT COUNT(1) FROM E_REPOS.CLAIM_DT "
			+ "WHERE CLAIM_KEY = '" + c.getClaimKey() + "' "
			+ "AND TX_TYPE = '837D' AND DTP01DTTM_QUAL = '472' "
			+ "AND DENTAL_SERVICE_KEY IS NULL ";
		String vCount = getCount(strQry);
		String vCnt = "";
		if (vCount.equals("0")) {
			strQry = "SELECT COUNT(1) FROM CLAIM_DT "
				+ "WHERE CLAIM_KEY = '" + c.getClaimKey() + "' "
				+ "AND TX_TYPE = '837D' AND DTP01DTTM_QUAL = '472' "
				+ "AND DENTAL_SERVICE_KEY IS NOT NULL ";
			vCount = getCount(strQry);
			strQry = "SELECT COUNT(1) FROM DENTAL_SVC_LINE "
				+ "WHERE CLAIM_KEY = '" + c.getClaimKey() + "' ";
			vCnt = getCount(strQry);
			if (!(vCount.equals(vCnt))) {
				insertRejectClaim(c, MissingDentalDateofService837D, getPriority(MissingDentalDateofService837D));
				passedAllCriteria=false;
			}
		}

		return passedAllCriteria;
	};

	final String getCount(String strQuery) {
		BigDecimal vCount;
		Query q = em.createNativeQuery(strQuery);

		vCount = (BigDecimal) q.getSingleResult();
		return vCount.toString();
	}

	private boolean checkInvalidBeneficiaryZipCode837ALL(Claim c) throws SQLException {
		boolean passedAllCriteria = true;
	      final int zipCodeLength = 5;
	      boolean vRejClm = false;
//	      String qstr = "select ea from eEntityAddress ea "
//	                    + "where ea.txnSet.id.fileNumber = :file "
//	                    + "  and ea.txnSet.id.st02txnCtlNbr = :cntnbr "
//	                    + "  and ea.loopId = '2010BA' ";

	      /**
	       * The above query didn't always return a ZIP that was in our system. In many
	       * cases it returned a ZIP in our system but there was no guarantee that it
	       * was actually the one for the bene being tested.
	       */

	      String qstr = "select ea from eEntityAddress ea "
	                    + "where ea.txnSet.id.fileNumber = :file "
	                    + "  and ea.txnSet.id.st02txnCtlNbr = :cntnbr "
	                    + "  and ea.loopId = '2010BA' "
	                    + "  and ea.nm109sbsrId = :vsbcrid";


	      Query q = null;
	      try {
	        q = em.createQuery(qstr);
	      } catch(Exception ex) {
	          ex.printStackTrace();
	          throw new SQLException(ex);
	      }
	      q.setParameter("file", c.getFileNumber());
	      q.setParameter("cntnbr", c.getSt02txnCtlNbr());
	      q.setParameter("vsbcrid",c.getSubscriberId());

	      Iterator iter = q.getResultList().iterator();
	      if (iter.hasNext()) {
	         EntityAddress entityAddress = (EntityAddress) iter.next();
	         String vZipCode = entityAddress.getN403postalCd();
	         if (vZipCode != null) {
	            if (vZipCode.length() > zipCodeLength) {
	               vZipCode = vZipCode.substring(0, zipCodeLength);
	            }
	            String zipCheck = "select z from foZipCode z where "
	                    + "z.zipCode = :zip and z.activeFlag = 'Y' ";

	            Query q1 = em.createQuery(zipCheck);
	            q1.setParameter("zip", vZipCode);
	            Iterator iter1 = q1.getResultList().iterator();
	            if (!iter1.hasNext()) {
	               vRejClm = true;
	            }
	         } else {
	            vRejClm = true;
	         }
	         if (vRejClm) {
	            insertRejectClaim(c, InvalidBeneficiaryZipCode837ALL, getPriority(InvalidBeneficiaryZipCode837ALL));
	            passedAllCriteria=false;
	         }
	      }

		return passedAllCriteria;
	};

	final RejectReasonCodes findRejectReason(String rejectCode) {
		RejectReasonCodes rc = null;
		Query q = em
		.createQuery("select rc from eRejectReasonCodes rc where rejectCode = :code");
		q.setParameter("code", rejectCode);
		Iterator iter = q.getResultList().iterator();
		if (iter.hasNext()) {
			rc = (RejectReasonCodes) iter.next();
		}
		return rc;
	}

	final void insertRejectClaim(Claim c, String vRejCode, Short vPriority)
	throws SQLException {
      LOGGER.debug( "MJS -- In insertRejectClaim with vRejCode <" + vRejCode + "> and vPriority <" + vPriority + ">" );
      LOGGER.debug( "MJS -- In insertRejectClaim with reject reason <" + findRejectReason(vRejCode).getRejectCodeDesc() + ">");
		RejectClaim rc = new RejectClaim();
		rc.setRejectClaimKey(UUID.randomUUID().toString().toUpperCase());
		rc.setEdiClaimKey(c.getClaimKey());
		rc.setRejectReasonCodes(findRejectReason(vRejCode));
		rc.setClaimType(c.getClaimType());
		rc.setPriority(vPriority);
		rc.setStatus("NEW");
		em.persist(rc);
	}

	final Short getPriority(String vRejCode) {
		Short priority = new Short("999");
		RejectReasonCodes rc = findRejectReason(vRejCode);
		if (rc != null && rc.getPriority() != null) {
			priority = rc.getPriority();
		}
		return priority;
	}

	final void insertProcessLog(String vType, int vCnt) throws SQLException {
		EdiProcessLog plog = new EdiProcessLog();
		plog.setEdiProcessId(UUID.randomUUID().toString().toUpperCase());
		plog.setFileDate(DateUtil.getCurSqlDate());
		plog.setProcessName("X12 EDIT CHECKS");
		plog.setMessage("NUM. OF X12 CLAIMS " + vType + " IN EDIT CHECKS");
		plog.setCounts(new Long(vCnt));
		plog.setClaimType("837");
		plog.setStatus("NEW");
		em.persist(plog);
	}

	/**
	 * This method will tell you if a date, date range, or date time value is in
	 * the future. It accepts D8, RD8 and DT values -(ignores time values). It
	 * does not accept TM Values
	 *
	 * @param vDate
	 *            the date, date range or date time
	 * @return
	 */
	final boolean editFutureDate(String vDate) {

		boolean isDateInTheFuture = false;

		Calendar cal = Calendar.getInstance();
		java.sql.Date today = new java.sql.Date(cal.getTimeInMillis());

		// If the date submitted is a range check the end Date
		if (vDate.length() == DATE_FORMAT_RD8_LENGTH) {
			cal = parseRD8FomattedDate(vDate)[1];
		} else {
			// Otherwise use the date portion we ignore Time
			cal = parseD8FomattedDate(vDate);
		}
		java.sql.Date vSqlDate = new java.sql.Date(cal.getTimeInMillis());
		int iDate = vSqlDate.compareTo(today);
		if (iDate > -1) {
			isDateInTheFuture = true;
		}
		return isDateInTheFuture;
	}

	/**
	 * This method returns a calendar object that represents a D8 formatted
	 * date.
	 *
	 * @param vDate
	 * @return
	 */
	private Calendar parseD8FomattedDate(String vDate) {
		Calendar cal = Calendar.getInstance();
		cal.set(Integer.parseInt(vDate.substring(0, DATE_FORMAT_YEAR_INDEX)),
				Integer.parseInt(vDate.substring(DATE_FORMAT_YEAR_INDEX,
						DATE_FORMAT_MONTH_INDEX)) - 1, Integer.parseInt(vDate
								.substring(DATE_FORMAT_MONTH_INDEX,
										DATE_FORMAT_DAY_INDEX)));
		return cal;
	}

	/**
	 * This method returns a 2 member calendar array that represents a RD8
	 * formatted date range. The first calendar in the array represents the
	 * begin date, the second represents the end date.
	 *
	 * @param vDate
	 *            - a RD8 formatted string that represents a date range
	 * @return an array with a 2 Calendar objects that represent the begin[0]
	 *         and end[1] dates of the range
	 */
	private Calendar[] parseRD8FomattedDate(String vDate) {
		Calendar beginDate = parseD8FomattedDate(vDate);
		Calendar endDate = Calendar.getInstance();
		endDate.set(Integer.parseInt(vDate.substring(
				DATE_FORMAT_INDEX_OF_DASH_IN_RD8,
				DATE_FORMAT_END_DATE_YEAR_INDEX)), Integer.parseInt(vDate
						.substring(DATE_FORMAT_END_DATE_YEAR_INDEX,
								DATE_FORMAT_END_DATE_MONTH_INDEX)) - 1, Integer
								.parseInt(vDate.substring(DATE_FORMAT_END_DATE_MONTH_INDEX,
										DATE_FORMAT_RD8_LENGTH)));

		Calendar[] rangeToReturn = new Calendar[] { beginDate, endDate };
		return rangeToReturn;
	}

	/**
	 * Given a claim date that is a D8 (single date) an RD8 (date range) or a DT
	 * (date time) will return the last chronological date.
	 *
	 * @param vDate
	 * @return
	 */
	private Calendar getLastDate(ClaimDt vDate) {
		String dtTmQualifer = vDate.getDtp01dttmQual();
		if ("RD8".equalsIgnoreCase(dtTmQualifer))
			return parseRD8FomattedDate(vDate.getDtp03dtFld())[1];
		return parseD8FomattedDate(vDate.getDtp03dtFld());

	}

	/**
	 * Gets a claim date from the claim date table using the claim key,
	 * specified qualifier, and transaction type.
	 *
	 * @param c
	 * @param vDateQual
	 * @return
	 */
	final String getDate(Claim c, String vDateQual, String txType) {
		String vDate = "";
		String qstr = "select dt from eClaimDt dt "
			+ "where dt.claim.claimKey = :key "
			//+ "  and dt.txType = :txType  "
			+ "  and dt.dtp01dttmQual = :qual ";
		Query q = em.createQuery(qstr);
		q.setParameter("key", c.getClaimKey());
		q.setParameter("qual", vDateQual);
		//q.setParameter("txType", txType);
		Iterator iter = q.getResultList().iterator();
		if (iter.hasNext()) {
			ClaimDt claimDt = (ClaimDt) iter.next();
			vDate = claimDt.getDtp03dtFld();
		}
		return vDate;
	}
}
