package gov.va.med.hac.edi.era.jpa;

import gov.va.med.hac.edi.reposload.x12.jpa.X12eReposLoad;
import gov.va.med.hac.edi.utils.*;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.jpa.JPAUtil;

import java.io.*;
import javax.naming.NamingException;
import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
 * This class contains the SQL statements that delete data from the 
 * staging tables in the database that is related to a particular 835 file.
 * Data related to an 835 staging table is deleted from the database
 * when one or more data entries in the 835 file is found to be non-compliant.
 * 
 * @author vhahacpennet
 *
 */
public class DeleteStg835ProcessedData {
	
	static final Logger summaryLOGGER = LoggerFactory.getLogger(DeleteStg835ProcessedData.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + DeleteStg835ProcessedData.class);

	// Constructor for this class
	public DeleteStg835ProcessedData() {
		// Store the log file in an instance variable

	} // end of constructor

	/**
	 * This process physically deletes data elements (records) from the 
	 * staging database related to a particular 835 file (indicated by vFileNumber).
	 * 
	 * It logs status information using the PrintWriter identified by vLogFile.
	 * 
	 * @param vFileNumber
	 *            File number to process
	 * @param vLogFile
	 *            PrintWriter
	 * @throws java.lang.Exception
	 */
	protected void processDelete835Data(String vFileNumber, EntityManager dbEM)
			throws DbTransactionEnderException {
		// Reformatted by EAT 07/07
		
		//Select the Transaction Control Number related to the file number
		//that has been processed.
		//Notice that we are selecting records where thier status is deleted -
		//that does not mean they were deleted - it just means they were
		//processed successfully up until this point.
		final String sHdrSel = "(select ST02TXN_CTL_NBR from STG_835_HDR "
				+ "where STSE_STATUS = 'DELETE') " + "and FILE_NUMBER = '"
				+ vFileNumber + "'";
		
		//Get all the primary keys of all the claims related to that transaction
		final String sClpSel = "(select CLP_KEY from STG_835_CLP where "
				+ "ST02TXN_CTL_NBR in " + sHdrSel + " ) ";

		//Delete any data from the staging tables related to those claims 
		//(identified by their primary keys that were selected/subselected above)
		//or that batch of data (idicated by the transaction control number)
		final String[] deletes = {
				"delete from STG_835_SVC_LQ where CLP_KEY in " + sClpSel,
				"delete from STG_835_SVC_CAS where CLP_KEY in " + sClpSel,
				"delete from STG_835_SVC where CLP_KEY in " + sClpSel,
				"delete from STG_835_CLP_NAME where CLP_KEY in " + sClpSel,
				"delete from STG_835_CLP_CAS where CLP_KEY in " + sClpSel,
				"delete from STG_835_CLP where ST02TXN_CTL_NBR in " + sHdrSel,
				"delete from STG_835_HDR_TRN where ST02TXN_CTL_NBR in "
						+ sHdrSel,
				"delete from STG_835_HDR_BPR where ST02TXN_CTL_NBR in "
						+ sHdrSel,
				"delete from STG_835_PLB where ST02TXN_CTL_NBR in " + sHdrSel,
				"delete from STG_835_HDR where STSE_STATUS = 'DELETE' and FILE_NUMBER = '"
						+ vFileNumber + "'" };


		try {	

			dbEM.getTransaction().begin();
			
			//This section of code actually executes the sql above using 
			//The database connection in dbhibernate.properties
			Query deleteStatement;
			for (int i = 0; i < deletes.length; i++) {
				deleteStatement = dbEM.createNativeQuery(deletes[i]);
				deleteStatement.executeUpdate();
			}

			traceLOGGER.debug("Committing the database transaction");
			dbEM.getTransaction().commit();
			traceLOGGER.debug("Commit successful");

			// If any unexpected exception was thrown, rollback and log it, and
			// rethrow
		} catch (Throwable ex) {
			if (dbEM != null) {
				if (dbEM.getTransaction().isActive()) {
					traceLOGGER.debug("Rolling back the database transaction");
					dbEM.getTransaction().rollback();
					traceLOGGER.debug("Rollback successful");
				}
			}

			throw new DbTransactionEnderException(
					"Rolled Back the Transaction", ex);
		}
	}

}
