package gov.va.med.hac.edi.era.jpa;

import gov.va.med.hac.edi.utils.jpa.JPAUtil;
import gov.va.med.hac.edi.utils.jpa.LogUtils;

import java.io.*;
import javax.persistence.EntityManager;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public final class Edi835ErrorQueue {
	static final Logger summaryLOGGER = LoggerFactory.getLogger(Edi835ErrorQueue.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + Edi835ErrorQueue.class);
  /**
   *
   * @param args
   */
  public static void main(String[] args) { // NOPMD
    // Local variables
    String vRunId;
    boolean processSuccess = false;
    EntityManager dbEM = null;

    // Parse the command line
	CommandLine line = null;
	Options opts = getCliOptions();
	try {
		CommandLineParser parser = new GnuParser();
		line = parser.parse(opts, args);
		// If the "help" option is specified, just show usage options and
		// exit
		if (line.hasOption("h")) {
			printUsage(opts, args, false);
			System.exit(1);
		}
		// If any required command line parameter is missing, just show
		// usage options and exit
	} catch (MissingOptionException moe) {
		printUsage(opts, args, true);
		System.exit(1);
	} catch (ParseException pe) {
		System.out.println("Problem parsing the command line");
		summaryLOGGER.error("Problem parsing the command line");
		traceLOGGER.error("Problem parsing the command line");
		printUsage(opts, args, false);
		System.exit(1);
	}

  
      
    // Catch any exception (outside of setting up for logging)
    try {

    
      LogUtils.stampStartingTime("EDI 835 Error Queue Start", summaryLOGGER);

      System.out.println("Code Version: Refactoring Project");
      traceLOGGER.info("Code Version: Refactoring Project");

      vRunId = line.getOptionValue("r");
      if(vRunId != null)
      {
    	  vRunId = vRunId.trim();
      }
      
      System.out.println("Run ID: " + vRunId);
      traceLOGGER.info("Code Version: Refactoring Project");
      
      if(vRunId == null || vRunId.length() < 1) { 
    	  System.out.println("Edi835ErrorQueue: Missing commandline argument: Run ID");
    	  traceLOGGER.info("Edi835ErrorQueue: Missing commandline argument: Run ID");
         
      } else {

				File configFile = null;
				String configFilePath = line.getOptionValue("f");
				if (configFilePath != null && configFilePath.length() > 0) {
					configFile = JPAUtil
							.getJpaConfigFile(configFilePath.trim());
				} else {
					configFile = JPAUtil
							.getJpaConfigFile("dbhibernate.properties");
				}

				dbEM = JPAUtil.getEntityManager(configFile, "eResposExtract");

				// Carry out error queue processing
				final ProcessEdi835ErrorQueue errorQueueProcess = new ProcessEdi835ErrorQueue();
				errorQueueProcess.processErrorQueue(vRunId, dbEM);

				// Put a completion message into the log file
				System.out.println("Process completed successfully");
				traceLOGGER.info("Process completed successfully");
				processSuccess = true;
			}

		} catch (OutOfMemoryError e) {
    	summaryLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);      
    	traceLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
    } catch (Throwable e) {
     	summaryLOGGER.error("Exception in processing", e);
    	traceLOGGER.error("Exception in processing", e);
    	
    } finally {
      if (!processSuccess) {
    	  summaryLOGGER.error("!!!!! Process seems to have failed !!!!!");
    	  traceLOGGER.error("!!!!! Process seems to have failed !!!!!");
      }
        LogUtils.stampEndingTime("EDI 835 Error Queue End", summaryLOGGER);
    }
  }

  private static void printUsage(Options opts, String[] inArgs, boolean missingArg) {
	  final int helpwidth = 80;
	  HelpFormatter fmtr = new HelpFormatter();
	  System.out.println();
	  if (missingArg) {
		  System.out.println("Some required commandline argument was missing");
	  }
	  fmtr.printHelp(helpwidth, "Edi835ErrorQueue", "", opts, "", true);
	  if (missingArg) {
		  System.out.println(inArgs.length + " command line arguments were received: ");
		  for (int i = 0; i < inArgs.length; i++) {
			  System.out.println(inArgs[i]);
		  }
	  }
  }


  /**
   * Helper method to populate command line options objects.
   * 
   * @return the <code>Options</code> object containing the set of command line option.
   */
  @SuppressWarnings("static-access")
  private static Options getCliOptions() {
	  Options options = new Options();

	  Option configfile = OptionBuilder.withArgName("jpaFile").hasArg().withLongOpt("jpaFile")
	  .withDescription("The JPA Configuration file, such as hibernate.properties.").create("f");

	  Option runId = OptionBuilder.withArgName("fileNumber").hasArg().isRequired().
	  withLongOpt("runId").withDescription(
	  "The runID generated by EC Gateway/Map when it ran the compliance check.").create("r");

	  Option help = OptionBuilder.withLongOpt("help").withDescription("Print this message.").create(
	  "h");

	  options.addOption(help);
	  options.addOption(configfile);
	  options.addOption(runId);

	  return options;
  }

}
