package gov.va.med.hac.edi.era.jpa;


import gov.va.med.hac.edi.utils.jpa.JPAUtil;
import gov.va.med.hac.edi.utils.jpa.LogUtils;
import gov.va.med.hac.edi.utils.persistent.DatabaseUtils;
import gov.va.med.hac.edi.utils.persistent.FeeClaimPayment;
import gov.va.med.hac.edi.utils.persistent.FeeClaimPaymentLine;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.cli.Option;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HacCentralFeeLoad {

	static final Logger summaryLOGGER = LoggerFactory.getLogger(HacCentralFeeLoad.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace" + HacCentralFeeLoad.class);

	public static void main(final String[] args) {
		// Database Connection Variables
		String dbUrl = null;
		String dbUser = null;
		String dbPass = null;

		// Directory for Central Fee file, archive, and log files
		String fileDirectory = null;
		String archiveDirectory = null;

		// List of Fee_Claim_Payments and Fee_Claim_Payment_Lines Object from
		// the Central Fee file
		List<FeeClaimPayment> fcpList = new ArrayList<FeeClaimPayment>();
		List<FeeClaimPaymentLine> fcplList = new ArrayList<FeeClaimPaymentLine>();

		// Get configuration properties
		Properties prop = new Properties();
		CommandLineParser parser = new GnuParser();
		CommandLine line = null;
		Options opts = getCliOptions();

		
		try {
			line = parser.parse(opts, args);

	        File configfile = null;
	         if (line.hasOption("f")) {
	            configfile = JPAUtil.getJpaConfigFile(line.getOptionValue("f"));
	         }
	         
	         if (configfile != null) {
	        	 prop.load(new FileInputStream(configfile));
	        	
	        	 dbUrl = prop.getProperty("dbUrl");
	        	 dbUser = prop.getProperty("dbUser");
	        	 dbPass = prop.getProperty("dbPass");
	        	 
	        	 fileDirectory = prop.getProperty("fileDirectory");
	        	 archiveDirectory = prop.getProperty("archivedLocation");
	         }
	         
			//String configFilePath = line.getOptionValue("f");
			//input = ClassLoader.getSystemResourceAsStream(configFilePath);
		} catch (MissingOptionException moe) {
			summaryLOGGER.error("IO error opening the haccentralfeeload.properties file.", moe);
			traceLOGGER.error("IO error opening the haccentralfeeload.properties file.", moe);
		} catch (Throwable t) {
			summaryLOGGER.error("IO error opening the haccentralfeeload.properties file.", t);
			traceLOGGER.error("IO error opening the haccentralfeeload.properties file.", t);
		}

		// Set the directory where Central Fee file(s) can be found
		File dir = new File(fileDirectory);

		// Make a list of Central Fee file(s) that end with either .txt or .TXT from the directory
		File[] fileList = dir.listFiles(new FilenameFilter() {
			public boolean accept(File dir, String name) {
				return name.endsWith(".TXT") || name.endsWith(".txt");
			}
		});

		/*
		 * If the filename list is not empty when task is executed, process the Central Fee file(s) one by one.
		 * If the filename list is empty when the task is executed, log the start and end time.		 * 
		 */
		if (fileList != null) {
			for (File tmpFile : fileList) {
				LogUtils.stampStartingTime("HacCentralFeeLoad", summaryLOGGER);
				summaryLOGGER.info("HacCentralFeeLoad Start");
				summaryLOGGER.info("Start parsing file: " + tmpFile.getName());
				traceLOGGER.info("HacCentralFeeLoad Start");

				// Main Process Logic
				// Parse the Central Fee file
				parseBuffer(fileDirectory, tmpFile.getName(), fcpList, fcplList);

				// Insert parsed Central Fee data into FPPS_OWNER
				insertFeeClaimPayment(dbUrl, dbUser, dbPass, fcpList, fcplList);

				// Move Central Fee file to Archive folder and timestamp
				moveCentralFeeFile(fileDirectory, tmpFile.getName(), archiveDirectory);

				summaryLOGGER.info("End parsing file: " + tmpFile.getName());
				summaryLOGGER.info("HacCentralFeeLoad completed successfully");
				traceLOGGER.info("HacCentralFeeLoad completed successfully");
				LogUtils.stampEndingTime("HacCentralFeeLoad", summaryLOGGER);

				fcpList.clear();
				fcplList.clear();

			}
		} else {
			// Timestamp when the task was executed with no files in the given
			// directory in the hacCentralFeeLoad.properties file
			LogUtils.stampStartingTime("HacCentralFeeLoad", summaryLOGGER);
			summaryLOGGER.info("HacCentralFeeLoad Start");
			summaryLOGGER.info("No Central Fee file(s) were found ");
			traceLOGGER.info("HacCentralFeeLoad Start");

			summaryLOGGER.info("End parsing.");
			summaryLOGGER.info("No Central Fee file(s) were found.");
			traceLOGGER.info("No Central Fee file(s) were found.");
			LogUtils.stampEndingTime("HacCentralFeeLoad", summaryLOGGER);

		}
	}

	// Parse through buffer and place data into appropriate columns variables
	private static void parseBuffer(String fileDirectory, String fileName,
			List<FeeClaimPayment> fcpList, List<FeeClaimPaymentLine> fcplList) {
		File file = new File(fileDirectory + fileName);
	
		BufferedReader reader = null;
		int claimLineCount = 1;
		
		try {
			reader = new BufferedReader(new FileReader(file));
			String text = null;
			

			while ((text = reader.readLine()) != null) {
				// Fields used in both Fee_Claim_Payments and
				// Fee_Claim_Payment_Lines tables
				String authorizationNumber = null;
				String claimType = null;
				double billedAmount = 0.0, paidAmount = 0.0;

				// Set variables for Fee_Claim_Payments
				long claimIndex;
				int coveredDays = 0;
				double diagnosisRelatedGroupWeight = 0.0, disbursedAmount = 0.0, interestAmount = 0.0, claimLevelAllowedAmount = 0.0;
				String stationNumber = null, diagnosisRelatedGroupCode = null, routingNumber = null, accountNumber = null, feeInvoiceNumber = null, bankName = null;
				Date periodStartDate, periodEndDate, feeInvoiceDate = null;

				// Set variables for Fee_Claim_Payment_Lines
				double unitsPaid = 0.0, adjustmentAmount1 = 0.0, adjustmentAmount2 = 0.0, adjustmentAmount3 = 0.0, adjustmentAmount4 = 0.0, adjustmentAmount5 = 0.0;
				String lineItemSequence = null, serviceCodeType = null, servicePaidCode = null, serviceModifierCode1 = null, serviceModifierCode2 = null, serviceModifierCode3 = null, serviceModifierCode4 = null, adjustmentGroupCode1 = null, adjustmentReasonCode1 = null, remittanceRemarkCode1a = null, remittanceRemarkCode1b = null, adjustmentGroupCode2 = null, adjustmentReasonCode2 = null, remittanceRemarkCode2a = null, remittanceRemarkCode2b = null, adjustmentGroupCode3 = null, adjustmentReasonCode3 = null, remittanceRemarkCode3a = null, remittanceRemarkCode3b = null, adjustmentGroupCode4 = null, adjustmentReasonCode4 = null, remittanceRemarkCode4a = null, remittanceRemarkCode4b = null, adjustmentGroupCode5 = null, adjustmentReasonCode5 = null, remittanceRemarkCode5a = null, remittanceRemarkCode5b = null, typeOfPayment, checkNumber = null, paymentMethodology = null;
				Date serviceDate, checkDate = null;

				/*
				 * Claim Type from the Central Fee file determines if this is a
				 * B3=Outpatient, B5=Pharmacy, B9=Inpatient, or T=Travel Data
				 * from Central Fee file will be 3, 5, 9, or T for the different
				 * claim types
				 */
				claimType = text.substring(470, 471);

				/*
				 * Based on ClaimType, the 3 variables below are used in both
				 * FEE_CLAIM_PAYMENTS and FEE_CLAIM_PAYMENT_LINES
				 */
				billedAmount = convertStringToDouble(text.substring(91, 101).trim(), 2);
				paidAmount = convertStringToDouble(text.substring(71, 80).trim(), text.substring(80, 81).trim(), 2);
				authorizationNumber = text.substring(369, 398).trim();

				// FEE_CLAIM_PAYMENTS table data
				claimIndex = convertStringToLong(text.substring(0, 15).trim());
				stationNumber = text.substring(28, 34).trim();
				diagnosisRelatedGroupCode = text.substring(34, 38).trim();
				diagnosisRelatedGroupWeight = convertStringToDouble(text.substring(43, 51).trim(), 4);
				disbursedAmount = convertStringToDouble(text.substring(81, 91).trim(), 2);
				interestAmount = convertStringToDouble(text.substring(101, 111).trim(), 2);
				periodStartDate = convertStringToDate(text.substring(111, 119).trim());
				periodEndDate = convertStringToDate(text.substring(119, 127).trim());
				coveredDays = 0;
				routingNumber = text.substring(311, 336).trim();
				accountNumber = text.substring(336, 353).trim();
				feeInvoiceNumber = text.substring(353, 361).replaceFirst("^0+(?!$)", "");
				feeInvoiceDate = convertStringToDate(text.substring(361, 369).trim());
				claimLevelAllowedAmount = convertStringToDouble(text.substring(398, 408).trim(), 2);
				bankName = text.substring(409, 438).trim();

				// Initialize the Fee Claim Payment Object
				FeeClaimPayment feeClaimPayment = new FeeClaimPayment(
						claimIndex, coveredDays, diagnosisRelatedGroupWeight,
						billedAmount, paidAmount, disbursedAmount,
						interestAmount, claimLevelAllowedAmount, stationNumber,
						diagnosisRelatedGroupCode, routingNumber,
						accountNumber, feeInvoiceNumber, bankName,
						authorizationNumber, claimType, periodStartDate,
						periodEndDate, feeInvoiceDate);

				// Add the Fee Claim Payment object to the Fee Claim Payment list
				fcpList.add(feeClaimPayment);

				// FEE_CLAIM_PAYMENT_LINES table data
				lineItemSequence = text.substring(15, 18).replaceFirst("^0+(?!$)", "").toString();
				serviceDate = convertStringToDate(text.substring(18, 26).trim());
				serviceCodeType = text.substring(26, 28).trim();
				servicePaidCode = text.substring(38, 43).trim();
				serviceModifierCode1 = text.substring(51, 56).trim();
				serviceModifierCode2 = text.substring(56, 61).trim();
				serviceModifierCode3 = text.substring(61, 66).trim();
				serviceModifierCode4 = text.substring(66, 71).trim();
				unitsPaid = convertStringToDouble(text.substring(127, 139).trim(), 5);
				adjustmentGroupCode1 = text.substring(139, 141).trim();
				adjustmentReasonCode1 = text.substring(141, 146).trim();
				remittanceRemarkCode1a = text.substring(146, 152).trim();
				remittanceRemarkCode1b = text.substring(152, 158).trim();
				adjustmentAmount1 = convertStringToDouble(text.substring(158, 167).trim(), text.substring(167, 168), 2);
				adjustmentGroupCode2 = text.substring(168, 170).trim();
				adjustmentReasonCode2 = text.substring(170, 175).trim();
				remittanceRemarkCode2a = text.substring(175, 181).trim();
				remittanceRemarkCode2b = text.substring(181, 187).trim();
				adjustmentAmount2 = convertStringToDouble(text.substring(187, 196).trim(), text.substring(196, 197), 2);
				adjustmentGroupCode3 = text.substring(197, 199).trim();
				adjustmentReasonCode3 = text.substring(199, 204).trim();
				remittanceRemarkCode3a = text.substring(204, 210).trim();
				remittanceRemarkCode3b = text.substring(210, 216).trim();
				adjustmentAmount3 = convertStringToDouble(text.substring(216, 225).trim(), text.substring(225, 226), 2);
				adjustmentGroupCode4 = text.substring(226, 228).trim();
				adjustmentReasonCode4 = text.substring(228, 233).trim();
				remittanceRemarkCode4a = text.substring(233, 239).trim();
				remittanceRemarkCode4b = text.substring(239, 245).trim();
				adjustmentAmount4 = convertStringToDouble(text.substring(245, 254).trim(), text.substring(254, 255), 2);
				adjustmentGroupCode5 = text.substring(255, 257).trim();
				adjustmentReasonCode5 = text.substring(257, 262).trim();
				remittanceRemarkCode5a = text.substring(262, 268).trim();
				remittanceRemarkCode5b = text.substring(268, 274).trim();
				adjustmentAmount5 = convertStringToDouble(text.substring(274, 283).trim(), text.substring(283, 284), 2);
				typeOfPayment = text.substring(284, 287).trim();
				checkNumber = text.substring(287, 303).trim();
				checkDate = convertStringToDate(text.substring(303, 311).trim());
				paymentMethodology = text.substring(408, 409).trim();

				// Initialize the Fee Claim Payment Line Object
				FeeClaimPaymentLine feeClaimPaymentLine = new FeeClaimPaymentLine(
						unitsPaid, billedAmount,
						paidAmount, adjustmentAmount1, adjustmentAmount2,
						adjustmentAmount3, adjustmentAmount4,
						adjustmentAmount5, lineItemSequence, serviceCodeType,
						servicePaidCode, serviceModifierCode1,
						serviceModifierCode2, serviceModifierCode3,
						serviceModifierCode4, adjustmentGroupCode1,
						adjustmentReasonCode1, remittanceRemarkCode1a,
						remittanceRemarkCode1b, adjustmentGroupCode2,
						adjustmentReasonCode2, remittanceRemarkCode2a,
						remittanceRemarkCode2b, adjustmentGroupCode3,
						adjustmentReasonCode3, remittanceRemarkCode3a,
						remittanceRemarkCode3b, adjustmentGroupCode4,
						adjustmentReasonCode4, remittanceRemarkCode4a,
						remittanceRemarkCode4b, adjustmentGroupCode5,
						adjustmentReasonCode5, remittanceRemarkCode5a,
						remittanceRemarkCode5b, typeOfPayment, checkNumber,
						authorizationNumber, claimType, paymentMethodology, serviceDate, checkDate);
				
				// Add the Fee Claim Payment Line object to the Fee Claim Payment Line list
				fcplList.add(feeClaimPaymentLine);
				claimLineCount++;
			}
		} catch (FileNotFoundException e) {
			summaryLOGGER.error("File not found.", e.getStackTrace().toString());
			traceLOGGER.error("File not found.", e.getStackTrace().toString());
			// e.printStackTrace();
		} catch (IOException e) {
			summaryLOGGER.error("IO error with reading  in from the buffer reader for file.", e);
			traceLOGGER.error("IO error with reading  in from the buffer reader for file.", e);
			// e.printStackTrace();
		} catch (StringIndexOutOfBoundsException x) {
			summaryLOGGER
					.error("Parsing error with string index out of bound exception on line: " + claimLineCount, x);
			traceLOGGER
					.error("Parsing error with string index out of bound exception on line: " + claimLineCount, x);
			System.exit(0);
			// e.printStackTrace();
		} catch (Exception ge) {
			summaryLOGGER.error("General Exception: ", ge);
			traceLOGGER.error("General Exception: ", ge);
		} finally {
			try {
				if (reader != null) {
					reader.close();
				}
			} catch (IOException ex) {
				summaryLOGGER.error("IO error for closing buffer reader.", ex);
				traceLOGGER.error("IO error for closing buffer reader.", ex);
			}
		}
	}

	/*
	 * Method converts a string into a date from the Central Fee file Input:
	 * String for a date Return: Date
	 */
	private static Date convertStringToDate(String stringDate) {
		int stringDateLength = stringDate.trim().length();
		if (stringDateLength != 0) {
			DateFormat formatter = new SimpleDateFormat("MMddyyyy");
			Date date = null;
			try {
				date = (Date) formatter.parse(stringDate);
			} catch (ParseException e) {
				summaryLOGGER.warn("Parsing of String to date warning.");
				traceLOGGER.warn("Parsing of String to date warning.");
				// e.printStackTrace();
			}
			// SimpleDateFormat newFormat = new SimpleDateFormat("MM-dd-yyyy");
			return date;
		} else {
			return null;
		}
	}

	/*
	 * Method validates that a string is of numeric value Input: String with
	 * numeric and possibly alpha and spaces Return: Boolean
	 */
	private static boolean isNumeric(String stringValue) {
		return stringValue.matches("-?\\d+(\\.\\d+)?");
	}

	/*
	 * Method validates that a string is off numeric value Input: String with
	 * numeric and possibly alpha and spaces Return: Integer if valid and 0 if
	 * not valid string
	 */
	private static int convertStringToInt(String stringValue) {
		if (isNumeric(stringValue) && stringValue.length() != 0) {
			return Integer.parseInt(stringValue);
		} else {
			return 0;
		}
	}

	/*
	 * Method validates that a string is off numeric value Input: String with
	 * numeric and possibly alpha and spaces Return: Long if valid and 0 if
	 * not valid string
	 */
	private static long convertStringToLong(String stringValue) {
		if (isNumeric(stringValue) && stringValue.length() != 0) { 
			return Long.parseLong(stringValue);
		} else {
			return 0;
		}
	}

	/*
	 * Method validates that a string is off numeric value Input: String with
	 * numeric and possibly alpha and spaces Return: Double if valid and 0.0 if
	 * not valid string
	 */
	private static double convertStringToDouble(String stringValue, String sign, int decimalPlace) {
		if (isNumeric(stringValue) && stringValue.length() != 0) {
			String tmpString = new StringBuilder(stringValue).insert(stringValue.length() - decimalPlace, ".").toString();
			if (sign.equalsIgnoreCase("-")) {
				return Double.parseDouble(tmpString.trim()) * -1;
			} else {
				return Double.parseDouble(tmpString.trim());
			}
		} else {
			return 0.0;
		}
	}

	/*
	 * Method validates that a string is off numeric value Input: String with
	 * numeric and possibly alpha and spaces Return: Double if valid and 0.0 if
	 * not valid string
	 */
	private static double convertStringToDouble(String stringValue,	int decimalPlace) {
		if (isNumeric(stringValue) && stringValue.length() != 0) {
			String tmpString = new StringBuilder(stringValue).insert(stringValue.length() - decimalPlace, ".").toString();
			return Double.parseDouble(tmpString.trim());
		} else {
			return 0.0;
		}
	}

	/*
	 * Method to check if Claim exist from Claim Index number in the Central Fee
	 * file against the CLAIMS table Input: Return: Boolean
	 */
	private static boolean claimIndexExist(long claimIndex, Connection conn) {
		String sql = "SELECT * FROM Claims WHERE claim_index = ?";
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			ps = conn.prepareStatement(sql);
			ps.setObject(1, claimIndex);
			rs = ps.executeQuery();
		} catch (SQLException e) {
			summaryLOGGER.warn("SQL Exception: Function name claimIndexExist().", e);
			traceLOGGER.warn("SQL Exception: Function name claimIndexExist().", e);
		}

		boolean tmpClaimExist = false;
		// Determine if the ResultSet returns record set or not. True if Claim
		// Index exist. False if not.
		try {
			if (rs.next()) {
				tmpClaimExist = true;
			}
		} catch (SQLException e) {
			summaryLOGGER.warn("SQL Exception: Parsing if a Claim Index exist.", e);
			traceLOGGER.warn("SQL Exception: Parsing if a Claim Index exist.", e);			
		}

		DatabaseUtils.close(ps);
		DatabaseUtils.close(rs);

		return tmpClaimExist;
	}

	/*
	 * Method to check if Fee Claim Payment exist in the FEE_CLAIM_PAYMENTS
	 * based of CLAIM_INDEX, CHECK_NUMBER, and FEE_INVOICE_NUMBER Return: String
	 * of the FEE_CLAIM_PAYMENT_ID if exist else NULL
	 */
	private static String feeClaimExist(long claimIndex, Connection conn) {
		String tmpFeeClaimPaymentID = null;
		String sql = "SELECT fee_claim_payment_id FROM FEE_CLAIM_PAYMENTS WHERE claim_index = ?";
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			ps = conn.prepareStatement(sql);
			ps.setObject(1, claimIndex);
			rs = ps.executeQuery();
		} catch (SQLException e) {
			summaryLOGGER.warn("SQL Exception: Function name feeClaimExist().", e);
			traceLOGGER.warn("SQL Exception: Function name feeClaimExist().", e);
		}

		try {
			if (rs.next()) {
				tmpFeeClaimPaymentID = rs.getString("fee_claim_payment_id");
			}

		} catch (SQLException e) {
			summaryLOGGER.warn("SQL Exception: Parsing fee_claim_payment_id.", e);
			traceLOGGER.warn("SQL Exception: Parsing fee_claim_payment_id.", e);
		}

		DatabaseUtils.close(ps);
		DatabaseUtils.close(rs);

		return tmpFeeClaimPaymentID;
	}

	/*
	 * Method to check if Fee Claim Payment Line exist in the
	 * FEE_CLAIM_PAYMENT_LINES based of CLAIM_INDEX, LINE_ITEM_SEQUENCE, and
	 * CHECK_NUMBER Return: String of the FEE_CLAIM_PAYMENT_ID if exist else
	 * NULL
	 */
	private static String feeLineExist(long claimIndex, String lineItemSequence, Connection conn) {
		String tmpFeeClaimPaymentLineID = null;
		String sql = "SELECT FCPL.fee_claim_payment_line_id FROM FEE_CLAIM_PAYMENTS FCP, FEE_CLAIM_PAYMENT_LINES FCPL WHERE "
				+ "FCP.FEE_CLAIM_PAYMENT_ID = FCPL.FEE_CLAIM_PAYMENT_ID AND "
				+ "FCP.claim_index = ? AND FCPL.line_item_sequence = ?";
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			ps = conn.prepareStatement(sql);
			ps.setObject(1, claimIndex);
			ps.setString(2, lineItemSequence);
			rs = ps.executeQuery();
		} catch (SQLException e) {
			summaryLOGGER.warn("SQL Exception: Function name feeLineExist().", e);
			traceLOGGER.warn("SQL Exception: Function name feeLineExist().", e);
			e.printStackTrace();
		}

		try {
			if (rs.next()) {
				tmpFeeClaimPaymentLineID = rs.getString("fee_claim_payment_line_id");
			}

		} catch (SQLException e) {
			
		}

		DatabaseUtils.close(ps);
		DatabaseUtils.close(rs);

		return tmpFeeClaimPaymentLineID;
	}

	/*
	 * Method to compare if Period Start Date is AFTER the Service Date for B3 Claim Type.
	 * If the conditions are meant, UPDATE FEE_CLAIM_PAYMENTS period_start_date with the 
	 * FEE_CLAIM_PAYMENT_LINES service_date. If not, no UPDATE.
	 */
	private static java.sql.Date compareStartAndServiceDateForB3(long claimIndex, java.sql.Date sqlServiceDate, Connection conn) {
		java.sql.Date tmpPeriodStartDate = null;
		String sql = "SELECT period_start_date FROM FEE_CLAIM_PAYMENTS WHERE claim_index = ?";
		String updateSql = "UPDATE FEE_CLAIM_PAYMENTS SET period_start_date = ? WHERE claim_index = ?";  
		PreparedStatement ps = null;
		ResultSet rs = null;
		PreparedStatement psUpdateSql = null;

		try {
			ps = conn.prepareStatement(sql);
			ps.setObject(1, claimIndex);
			rs = ps.executeQuery();
		} catch (SQLException e) {
			summaryLOGGER.warn("SQL Exception: Function name compareStartAndServiceDateForB3().", e);
			traceLOGGER.warn("SQL Exception: Function name compareStartAndServiceDateForB3().", e);
		}

		try {
			if (rs.next()) {
				tmpPeriodStartDate = rs.getDate("period_start_date");
			}
		} catch (SQLException e) {
			summaryLOGGER.warn("SQL Exception: Parsing period_start_date in function compareStartAndServiceDateForB3().", e);
			traceLOGGER.warn("SQL Exception:  Parsing period_start_date in function compareStartAndServiceDateForB3().", e);
		}
		
		if (tmpPeriodStartDate != null) {
			if (tmpPeriodStartDate.after(sqlServiceDate)) {
				try {
					psUpdateSql = conn.prepareStatement(updateSql);
					psUpdateSql.setDate(1, sqlServiceDate);
					psUpdateSql.setObject(2, claimIndex);
					psUpdateSql.executeQuery();
				} catch (SQLException e) {
					summaryLOGGER.warn("SQL Exception: Update period_start_date in function compareStartAndServiceDateForB3().", e);
					traceLOGGER.warn("SQL Exception: Update period_start_date in function compareStartAndServiceDateForB3().", e);
				}				
			}
		} else {
			tmpPeriodStartDate = sqlServiceDate;
		}

		DatabaseUtils.close(ps);
		DatabaseUtils.close(rs);
		DatabaseUtils.close(psUpdateSql);
		
		return tmpPeriodStartDate;
	}


	/*
	 * Method to insert Fee Claim Payment from Central Fee file into
	 * FEE_CLAIM_PAYMENTS table Input: Return:
	 */
	private static void insertFeeClaimPayment(String dbUrl, String dbUser, String dbPass, List<FeeClaimPayment> fcpList, List<FeeClaimPaymentLine> fcplList) {
		/*
		 * Inserting Metric variables for number of Claim Level entries, number
		 * of Line Level entries, number of Line Level duplicate entries, number
		 * of duplicate Claim Level entries, number of Claim that do not exist
		 */
		int totalClaimEntered = 0, totalLineEntered = 0, totalLineDup = 0, totalClaimDup = 0, totalNonClaim = 0;

		/*
		 * Temp variable for flagging if a FEE_CLAIM_PAYMENTS exist then return
		 * FEE_CLAIM_PAYMENT_ID. If not, return null string. Temp variable for
		 * flagging if a FEE_CLAIM_PAYMENT_LINES exist then return
		 * FEE_CLAIM_PAYMENT_LINE_ID. If not, return null string.
		 */
		String tmpFeeClaimPaymentID, tmpFeeClaimPaymentLineID, tmpTransactionTypeCode;

		/*
		 * Temp list of strings to describe which required fields are missing in the Central Fee file
		 */
		List<String> tmpMissingRequiredFieldList = new ArrayList<String>();
		List<String> checkNumberList = new ArrayList<String>();
		checkNumberList.clear();
		List<Long> claimIndexList = new ArrayList<Long>();
		claimIndexList.clear();
		
		/*
		 * Flag for insert Claim level. If true, Claim does not exist in
		 * FEE_CLAIM_PAYMENTS so insert new entry. If false, no entry is
		 * inserted. Flag for insert Line level. If true, Line does not exist in
		 * FEE_CLAIM_PAYMENT_LINES so insert new entry. If false, no entry is
		 * inserted.
		 */
		boolean insertClaimFlag, insertLineFlag, missingRequiredFieldFlag, failedBusinessRuleFlag;
		
		Connection conn = null;
		PreparedStatement psInsertClaimB3B5 = null;
		PreparedStatement psInsertClaimB9 = null;
		PreparedStatement psInsertLineB3B5 = null;
		PreparedStatement psInsertLineB9 = null;
		PreparedStatement psGetClaimID = null;
		PreparedStatement psUpdateResolutionStatus = null;
		ResultSet rs = null;
		java.util.Date today = new java.util.Date();
		int i = 0;

		// INSERT Statement for inserting a new FEE_CLAIM_PAYMENTS record for
		// Claim Type - B3 Outpatient & B5 Pharmacy
		String insertFCPB3B5Sql = "INSERT INTO FEE_CLAIM_PAYMENTS"
				+ "(claim_index, covered_days, diagnosis_related_group_weight, disbursed_amount, "
				+ "interest_amount, station_number, diagnosis_related_group_code, routing_number, "
				+ "account_number, fee_invoice_number, bank_name, period_start_date, "
				+ "period_end_date, fee_invoice_date, created_by, date_created, transaction_type_code) VALUES "
				+ "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

		// INSERT Statement for inserting a new FEE_CLAIM_PAYMENTS record for
		// Claim Type - B9 Inpatient
		String insertFCPB9Sql = "INSERT INTO FEE_CLAIM_PAYMENTS"
				+ "(claim_index, covered_days, diagnosis_related_group_weight, disbursed_amount, "
				+ "interest_amount, station_number, diagnosis_related_group_code, routing_number, "
				+ "account_number, fee_invoice_number, bank_name, period_start_date, "
				+ "period_end_date, fee_invoice_date, created_by, date_created, transaction_type_code, "
				+ "claim_level_allowed_amount, authorization_number, billed_amount, paid_amount) VALUES "
				+ "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

		// SELECT Statement for returning FEE_CLAIM_PAYMENT_ID to be used for
		// inserting a new FEE_CLAIM_PAYMENT_LINES record
		String returnFCPIdSql = "SELECT * FROM (SELECT fee_claim_payment_id FROM FEE_CLAIM_PAYMENTS WHERE claim_index = ? ORDER BY ROWNUM DESC) WHERE ROWNUM <= 1";

		// INSERT Statement for inserting a new FEE_CLAIM_PAYMENT_LINES record
		// for Claim Type - B3 Outpatient & B5 Pharmacy
		String insertFCPLB3B5Sql = "INSERT INTO FEE_CLAIM_PAYMENT_LINES"
				+ "(fee_claim_payment_id, units_paid, adjustment_amount_1, adjustment_amount_2, "
				+ "adjustment_amount_3, adjustment_amount_4, adjustment_amount_5, line_item_sequence, service_code_type, service_paid_code, "
				+ "service_modifier_code_1, service_modifier_code_2, service_modifier_code_3, service_modifier_code_4, adjustment_group_code_1, "
				+ "adjustment_reason_code_1, remittance_remark_code, remittance_remark_code_1b, adjustment_group_code_2, adjustment_reason_code_2, "
				+ "remittance_remark_code_2a, remittance_remark_code_2b, adjustment_group_code_3, adjustment_reason_code_3, remittance_remark_code_3a, "
				+ "remittance_remark_code_3b, adjustment_group_code_4, adjustment_reason_code_4, remittance_remark_code_4a, remittance_remark_code_4b, "
				+ "adjustment_group_code_5, adjustment_reason_code_5, remittance_remark_code_5a, remittance_remark_code_5b, type_of_payment, "
				+ "check_number, service_date, check_date, created_by, date_created, payment_methodology, authorization_number, billed_amount, paid_amount) VALUES "
				+ "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

		// INSERT Statement for inserting a new FEE_CLAIM_PAYMENT_LINES record
		// for Claim Type - B9 Inpatient
		String insertFCPLB9Sql = "INSERT INTO FEE_CLAIM_PAYMENT_LINES"
				+ "(fee_claim_payment_id, units_paid, adjustment_amount_1, adjustment_amount_2, "
				+ "adjustment_amount_3, adjustment_amount_4, adjustment_amount_5, line_item_sequence, service_code_type, service_paid_code, "
				+ "service_modifier_code_1, service_modifier_code_2, service_modifier_code_3, service_modifier_code_4, adjustment_group_code_1, "
				+ "adjustment_reason_code_1, remittance_remark_code, remittance_remark_code_1b, adjustment_group_code_2, adjustment_reason_code_2, "
				+ "remittance_remark_code_2a, remittance_remark_code_2b, adjustment_group_code_3, adjustment_reason_code_3, remittance_remark_code_3a, "
				+ "remittance_remark_code_3b, adjustment_group_code_4, adjustment_reason_code_4, remittance_remark_code_4a, remittance_remark_code_4b, "
				+ "adjustment_group_code_5, adjustment_reason_code_5, remittance_remark_code_5a, remittance_remark_code_5b, type_of_payment, "
				+ "check_number, service_date, check_date, created_by, date_created, payment_methodology) VALUES "
				+ "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";

		/*
		 * Re-worked the logic
		 */
		try {
			conn = DatabaseUtils.createDbConnection(dbUrl, dbUser, dbPass);
			
			conn.setAutoCommit(false);
			// Create PreparedStatement for all SQL Strings
			psInsertClaimB3B5 = conn.prepareStatement(insertFCPB3B5Sql);
			psInsertClaimB9 = conn.prepareStatement(insertFCPB9Sql);
			psInsertLineB3B5 = conn.prepareStatement(insertFCPLB3B5Sql);
			psInsertLineB9 = conn.prepareStatement(insertFCPLB9Sql);
			psGetClaimID = conn.prepareStatement(returnFCPIdSql);

			/*
			 * Loop through the list of FEE_CLAIM_PAYMENT object (fcpList)
			 */
			while (i <fcpList.size()) {
			
				/*
				 * Verify the Claim Index exist in the CLAIMS table before inserting
				 * into FEE_CLAIM_PAYMENTS and FEE_CLAIM_PAYMENT_LINES tables Input:
				 * Claim Index number Return(s): Boolean - True if exist else false
				 * and log transaction
				 */
				tmpTransactionTypeCode = null;
				insertClaimFlag = false;
				insertLineFlag = false;
				missingRequiredFieldFlag = false;
				failedBusinessRuleFlag = false;
				tmpMissingRequiredFieldList.clear();
				
				/*
				 * Required fields from Central Fee File
				 * Fields: Claim Index, Fee Invoice Number, Station Number, Fee Invoice Date,
				 * 			Line Item Sequence, Claim Type (3 = B3, 5 = B5, 9 = B9 and T = Travel) 
				 */
				if (StringUtils.isBlank(String.valueOf(fcpList.get(i).getClaimIndex()))) {
					tmpMissingRequiredFieldList.add("Claim Index");
					missingRequiredFieldFlag = true;
				}
				
				if (StringUtils.isBlank(fcpList.get(i).getFeeInvoiceNumber())) {
					tmpMissingRequiredFieldList.add("Fee Invoice Number");
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					missingRequiredFieldFlag = true;
				}
				
				if (fcpList.get(i).getFeeInvoiceDate() == null) {
					tmpMissingRequiredFieldList.add("Fee Invoice Date");
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					missingRequiredFieldFlag = true;
				}
				
				if (StringUtils.isBlank(String.valueOf(fcpList.get(i).getDisbursedAmount()))) {
					tmpMissingRequiredFieldList.add("Disbursed Amount");
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					missingRequiredFieldFlag = true;
				}
				
				if (StringUtils.isBlank(fcpList.get(i).getStationNumber())) {
					tmpMissingRequiredFieldList.add("Station Number");
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					missingRequiredFieldFlag = true;
				}

				if (StringUtils.isBlank(fcplList.get(i).getLineItemSequence())) {
					tmpMissingRequiredFieldList.add("Line Item Sequence");
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					missingRequiredFieldFlag = true;
				}
				
				if (fcplList.get(i).getServiceDate() == null) {
					tmpMissingRequiredFieldList.add("Service Date");
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					missingRequiredFieldFlag = true;
				}
				
				// Log Missing Required Field(s)
				if (missingRequiredFieldFlag) {
					summaryLOGGER.warn("The following required field(s) are missing in Central Fee file: " + Arrays.toString(tmpMissingRequiredFieldList.toArray()) + " on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");
					traceLOGGER.warn("The following required field(s) are missing in Central Fee file: " + Arrays.toString(tmpMissingRequiredFieldList.toArray()) + " on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");
				}
				
				/*
				 * Business Rule(s) for inserting new FEE_CLAIM_PAYMENTS and FEE_CLAIM_PAYMENT_LINES. If anything 
				 * fails, all lines with the same check number will be set to the RESOLUTION_STATUS_CODE to 
				 * 'REVIEW_LOG' on the FEE_CLAIM_PAYMENT table.
				 * Rule 1. If type of payment is EFT, routing number and account number must exist. 
				 */
				// Rule 1. If type of payment is EFT, routing number and account number must exist. Log EFT Business Rule Error.
				if (fcplList.get(i).getTypeOfPayment().equals("EFT") && (StringUtils.isBlank(fcpList.get(i).getRoutingNumber()) || StringUtils.isBlank(fcpList.get(i).getAccountNumber()))) {
					checkNumberList.add(fcplList.get(i).getCheckNumber());
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					summaryLOGGER.warn("Missing Routing or Account Number for EFT business rule on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");
					traceLOGGER.warn("Missing Routing or Account Number for EFT business rule on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");					
					failedBusinessRuleFlag = true;
				}
				
				// Rule 2. If type of payment is not null, check number and check date are required. Log missing check number and check date for non-null type of payment.
				if (StringUtils.isNotBlank(fcplList.get(i).getTypeOfPayment()) && (StringUtils.isBlank(fcplList.get(i).getCheckNumber()) || fcplList.get(i).getCheckDate() == null)) {
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					summaryLOGGER.warn("Missing Check Number or Check Date for valid Type of Payment business rule on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");
					traceLOGGER.warn("Missing Check Number or Check Date for valid Type of Payment business rule on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");					
					failedBusinessRuleFlag = true;					
				}
				
				// Rule 3. If Claim Type is B9 (9) and Diagnosis Related Group Code is present, then Diagnosis Related Group Weight is required field. Log if is not the case.
				if (fcpList.get(i).getClaimType().equalsIgnoreCase("9") && StringUtils.isNotBlank(fcpList.get(i).getDiagnosisRelatedGroupCode()) && StringUtils.isBlank(String.valueOf(fcpList.get(i).getDiagnosisRelatedGroupWeight()))) {
					claimIndexList.add(fcpList.get(i).getClaimIndex());
					summaryLOGGER.warn("Missing DRG Weight for Claim Type B9 with DRG Code present business rule on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");
					traceLOGGER.warn("Missing DRG Weight for Claim Type B9 with DRG Code present business rule on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");					
					failedBusinessRuleFlag = true;										
				}
				
				
				if (!claimIndexExist(fcpList.get(i).getClaimIndex(), conn) || (fcpList.get(i).getClaimType().equalsIgnoreCase("T"))) {
					totalNonClaim++;
					summaryLOGGER.warn("Claim Index does not exist or Claim Type is not valid on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");
					traceLOGGER.warn("Claim Index does not exist or Claim Type is not valid on line: " + (i + 1) + " (Claim Index: " + fcpList.get(i).getClaimIndex() + ")");					
				}
				
				// Insert Claim if all information are provided and passes all business rules
				if (claimIndexExist(fcpList.get(i).getClaimIndex(), conn) && (!fcpList.get(i).getClaimType().equalsIgnoreCase("T")) && (!missingRequiredFieldFlag) && (!failedBusinessRuleFlag)) {

					tmpFeeClaimPaymentID = null;
					tmpFeeClaimPaymentLineID = null;

					tmpFeeClaimPaymentID = feeClaimExist(fcpList.get(i).getClaimIndex(), conn);
					tmpFeeClaimPaymentLineID = feeLineExist(fcpList.get(i).getClaimIndex(), fcplList.get(i).getLineItemSequence(), conn);
					
					// Set Dates for Claim level
					java.sql.Date sqlFeeInvoiceDate = new java.sql.Date(fcpList.get(i).getFeeInvoiceDate().getTime());
					java.sql.Date sqlPeriodStartDate = null;
					java.sql.Date sqlPeriodEndDate = null;
					if (fcpList.get(i).getPeriodStartDate() != null) {
						sqlPeriodStartDate = new java.sql.Date(fcpList.get(i).getPeriodStartDate().getTime());
					}
					if (fcpList.get(i).getPeriodEndDate() != null) {
						sqlPeriodEndDate = new java.sql.Date(fcpList.get(i).getPeriodEndDate().getTime());
					}
					
					// Set dates for Line level
					java.sql.Date sqlServiceDate = null;
					java.sql.Date sqlCheckDate = null;
					if (fcplList.get(i).getServiceDate() != null) {
						sqlServiceDate = new java.sql.Date(fcplList.get(i).getServiceDate().getTime());
					}
					if (fcplList.get(i).getCheckDate() != null) {
						sqlCheckDate = new java.sql.Date(fcplList.get(i).getCheckDate().getTime());
					}



					if (fcpList.get(i).getClaimType().equalsIgnoreCase("9")) {
						// If the feeClaimExist returns null, Claim Level does
						// not exist so enter new Claim level entry
						if (tmpFeeClaimPaymentID == null) {
							totalClaimEntered++;
							// Set insert flag to true for Claim Level
							insertClaimFlag = true;
							// Set insert flag to true for Line Level
							insertLineFlag = true;	
							// Set transaction_type_code to C
							tmpTransactionTypeCode = "C";
						} else {
							totalClaimDup++;
						}
						
						// Insert B9 Inpatient Claim
						if (insertClaimFlag) {
							psInsertClaimB9.setObject(1, fcpList.get(i).getClaimIndex());
							psInsertClaimB9.setInt(2, fcpList.get(i).getCoveredDays());
							psInsertClaimB9.setDouble(3, fcpList.get(i).getDiagnosisRelatedGroupWeight());
							psInsertClaimB9.setDouble(4, fcpList.get(i).getDisbursedAmount());
							psInsertClaimB9.setDouble(5, fcpList.get(i).getInterestAmount());
							psInsertClaimB9.setString(6, fcpList.get(i).getStationNumber());
							psInsertClaimB9.setString(7, fcpList.get(i).getDiagnosisRelatedGroupCode());
							psInsertClaimB9.setString(8, fcpList.get(i).getRoutingNumber());
							psInsertClaimB9.setString(9, fcpList.get(i).getAccountNumber());
							psInsertClaimB9.setString(10, fcpList.get(i).getFeeInvoiceNumber());
							psInsertClaimB9.setString(11, fcpList.get(i).getBankName());
							psInsertClaimB9.setDate(12, sqlPeriodStartDate);
							psInsertClaimB9.setDate(13, sqlPeriodEndDate);
							psInsertClaimB9.setDate(14, sqlFeeInvoiceDate);
							psInsertClaimB9.setString(15, "CENTRAL_FEE");
							psInsertClaimB9.setTimestamp(16,
									new java.sql.Timestamp(today.getTime()));
							psInsertClaimB9.setString(17, tmpTransactionTypeCode);		
							psInsertClaimB9.setDouble(18, fcpList.get(i).getClaimLevelAllowedAmount());
							psInsertClaimB9.setString(19, fcpList.get(i).getAuthorizationNumber());
							psInsertClaimB9.setDouble(20, fcpList.get(i).getBilledAmount());
							psInsertClaimB9.setDouble(21, fcpList.get(i).getPaidAmount());
							
							psInsertClaimB9.executeUpdate();
							
							psGetClaimID.setObject(1, fcpList.get(i).getClaimIndex());
							rs = psGetClaimID.executeQuery();
							while (rs.next()) {
								tmpFeeClaimPaymentID = rs.getString("fee_claim_payment_id");
							}
							DatabaseUtils.close(rs);
						}

						if (insertLineFlag) {	
							psInsertLineB9.setString(1, tmpFeeClaimPaymentID);
							psInsertLineB9.setDouble(2, fcplList.get(i).getUnitsPaid());
							psInsertLineB9.setDouble(3, fcplList.get(i).getAdjustmentAmount1());
							psInsertLineB9.setDouble(4, fcplList.get(i).getAdjustmentAmount2());
							psInsertLineB9.setDouble(5, fcplList.get(i).getAdjustmentAmount3());
							psInsertLineB9.setDouble(6, fcplList.get(i).getAdjustmentAmount4());
							psInsertLineB9.setDouble(7, fcplList.get(i).getAdjustmentAmount5());
							psInsertLineB9.setString(8, fcplList.get(i).getLineItemSequence());
							psInsertLineB9.setString(9, fcplList.get(i).getServiceCodeType());
							psInsertLineB9.setString(10, fcplList.get(i).getServicePaidCode());
							psInsertLineB9.setString(11, fcplList.get(i).getServiceModifierCode1());
							psInsertLineB9.setString(12, fcplList.get(i).getServiceModifierCode2());
							psInsertLineB9.setString(13, fcplList.get(i).getServiceModifierCode3());
							psInsertLineB9.setString(14, fcplList.get(i).getServiceModifierCode4());
							psInsertLineB9.setString(15, fcplList.get(i).getAdjustmentGroupCode1());
							psInsertLineB9.setString(16, fcplList.get(i).getAdjustmentReasonCode1());
							psInsertLineB9.setString(17, fcplList.get(i).getRemittanceRemarkCode1a());
							psInsertLineB9.setString(18, fcplList.get(i).getRemittanceRemarkCode1b());
							psInsertLineB9.setString(19, fcplList.get(i).getAdjustmentGroupCode2());
							psInsertLineB9.setString(20, fcplList.get(i).getAdjustmentReasonCode2());
							psInsertLineB9.setString(21, fcplList.get(i).getRemittanceRemarkCode2a());
							psInsertLineB9.setString(22, fcplList.get(i).getRemittanceRemarkCode2b());
							psInsertLineB9.setString(23, fcplList.get(i).getAdjustmentGroupCode3());
							psInsertLineB9.setString(24, fcplList.get(i).getAdjustmentReasonCode3());
							psInsertLineB9.setString(25, fcplList.get(i).getRemittanceRemarkCode3a());
							psInsertLineB9.setString(26, fcplList.get(i).getRemittanceRemarkCode3b());
							psInsertLineB9.setString(27, fcplList.get(i).getAdjustmentGroupCode4());
							psInsertLineB9.setString(28, fcplList.get(i).getAdjustmentReasonCode4());
							psInsertLineB9.setString(29, fcplList.get(i).getRemittanceRemarkCode4a());
							psInsertLineB9.setString(30, fcplList.get(i).getRemittanceRemarkCode4b());
							psInsertLineB9.setString(31, fcplList.get(i).getAdjustmentGroupCode5());
							psInsertLineB9.setString(32, fcplList.get(i).getAdjustmentReasonCode5());
							psInsertLineB9.setString(33, fcplList.get(i).getRemittanceRemarkCode5a());
							psInsertLineB9.setString(34, fcplList.get(i).getRemittanceRemarkCode5b());
							psInsertLineB9.setString(35, fcplList.get(i).getTypeOfPayment());
							psInsertLineB9.setString(36, fcplList.get(i).getCheckNumber());
							psInsertLineB9.setDate(37, sqlServiceDate);
							psInsertLineB9.setDate(38, sqlCheckDate);
							psInsertLineB9.setString(39, "CENTRAL_FEE");
							psInsertLineB9.setTimestamp(40,	new java.sql.Timestamp(today.getTime()));
							psInsertLineB9.setString(41, fcplList.get(i).getPaymentMethodology());
							
							psInsertLineB9.executeUpdate();
						}
			
					}
					
					if (fcpList.get(i).getClaimType().equalsIgnoreCase("3") || fcpList.get(i).getClaimType().equalsIgnoreCase("5")) {
						// If the feeLineExist returns null, Line Level does not
						// exist so enter new Claim level entry
						if (tmpFeeClaimPaymentLineID == null && tmpFeeClaimPaymentID == null) {
							totalLineEntered++;
							// Set insert flag to true for Claim Level
							insertClaimFlag = true;
							// Set insert flag to true for Line Level
							insertLineFlag = true;
							// Set transaction_type_code to C
							tmpTransactionTypeCode = "L";
						} 
						if (tmpFeeClaimPaymentLineID == null && tmpFeeClaimPaymentID != null) {
							totalLineEntered++;
							// Set insert flag to true for Line Level
							insertLineFlag = true;
						} 
						if (tmpFeeClaimPaymentLineID != null && tmpFeeClaimPaymentID != null) {
							totalLineDup++;
						}
						
						/*
						 * If Claim Type is B3 and Period Start Date is Null, use the earliest Service Date as the Period Start Date.
						 */
						if (fcpList.get(i).getClaimType().equalsIgnoreCase("3") && sqlPeriodStartDate == null) {
							sqlPeriodStartDate = compareStartAndServiceDateForB3(fcpList.get(i).getClaimIndex(), sqlServiceDate, conn);
						}
							
						
						// Insert B3 Outpatient and B5 Pharmacy Claim 
						if (insertClaimFlag) {
							psInsertClaimB3B5.setObject(1, fcpList.get(i).getClaimIndex());
							psInsertClaimB3B5.setInt(2, fcpList.get(i).getCoveredDays());
							psInsertClaimB3B5.setDouble(3, fcpList.get(i).getDiagnosisRelatedGroupWeight());
							psInsertClaimB3B5.setDouble(4, fcpList.get(i).getDisbursedAmount());
							psInsertClaimB3B5.setDouble(5, fcpList.get(i).getInterestAmount());
							psInsertClaimB3B5.setString(6, fcpList.get(i).getStationNumber());
							psInsertClaimB3B5.setString(7, fcpList.get(i).getDiagnosisRelatedGroupCode());
							psInsertClaimB3B5.setString(8, fcpList.get(i).getRoutingNumber());
							psInsertClaimB3B5.setString(9, fcpList.get(i).getAccountNumber());
							psInsertClaimB3B5.setString(10, fcpList.get(i).getFeeInvoiceNumber());
							psInsertClaimB3B5.setString(11, fcpList.get(i).getBankName());
							psInsertClaimB3B5.setDate(12, sqlPeriodStartDate);
							psInsertClaimB3B5.setDate(13, sqlPeriodEndDate);
							psInsertClaimB3B5.setDate(14, sqlFeeInvoiceDate);
							psInsertClaimB3B5.setString(15, "CENTRAL_FEE");
							psInsertClaimB3B5.setTimestamp(16,
									new java.sql.Timestamp(today.getTime()));
							psInsertClaimB3B5.setString(17, tmpTransactionTypeCode);

							psInsertClaimB3B5.executeUpdate();

							psGetClaimID.setObject(1, fcpList.get(i).getClaimIndex());
							rs = psGetClaimID.executeQuery();
							while (rs.next()) {
								tmpFeeClaimPaymentID = rs.getString("fee_claim_payment_id");
							}
							DatabaseUtils.close(rs);
						}

						if (insertLineFlag) {
							psInsertLineB3B5.setString(1, tmpFeeClaimPaymentID);
							psInsertLineB3B5.setDouble(2, fcplList.get(i).getUnitsPaid());
							psInsertLineB3B5.setDouble(3, fcplList.get(i).getAdjustmentAmount1());
							psInsertLineB3B5.setDouble(4, fcplList.get(i).getAdjustmentAmount2());
							psInsertLineB3B5.setDouble(5, fcplList.get(i).getAdjustmentAmount3());
							psInsertLineB3B5.setDouble(6, fcplList.get(i).getAdjustmentAmount4());
							psInsertLineB3B5.setDouble(7, fcplList.get(i).getAdjustmentAmount5());
							psInsertLineB3B5.setString(8, fcplList.get(i).getLineItemSequence());
							psInsertLineB3B5.setString(9, fcplList.get(i).getServiceCodeType());
							psInsertLineB3B5.setString(10, fcplList.get(i).getServicePaidCode());
							psInsertLineB3B5.setString(11, fcplList.get(i).getServiceModifierCode1());
							psInsertLineB3B5.setString(12, fcplList.get(i).getServiceModifierCode2());
							psInsertLineB3B5.setString(13, fcplList.get(i).getServiceModifierCode3());
							psInsertLineB3B5.setString(14, fcplList.get(i).getServiceModifierCode4());
							psInsertLineB3B5.setString(15, fcplList.get(i).getAdjustmentGroupCode1());
							psInsertLineB3B5.setString(16, fcplList.get(i).getAdjustmentReasonCode1());
							psInsertLineB3B5.setString(17, fcplList.get(i).getRemittanceRemarkCode1a());
							psInsertLineB3B5.setString(18, fcplList.get(i).getRemittanceRemarkCode1b());
							psInsertLineB3B5.setString(19, fcplList.get(i).getAdjustmentGroupCode2());
							psInsertLineB3B5.setString(20, fcplList.get(i).getAdjustmentReasonCode2());
							psInsertLineB3B5.setString(21, fcplList.get(i).getRemittanceRemarkCode2a());
							psInsertLineB3B5.setString(22, fcplList.get(i).getRemittanceRemarkCode2b());
							psInsertLineB3B5.setString(23, fcplList.get(i).getAdjustmentGroupCode3());
							psInsertLineB3B5.setString(24, fcplList.get(i).getAdjustmentReasonCode3());
							psInsertLineB3B5.setString(25, fcplList.get(i).getRemittanceRemarkCode3a());
							psInsertLineB3B5.setString(26, fcplList.get(i).getRemittanceRemarkCode3b());
							psInsertLineB3B5.setString(27, fcplList.get(i).getAdjustmentGroupCode4());
							psInsertLineB3B5.setString(28, fcplList.get(i).getAdjustmentReasonCode4());
							psInsertLineB3B5.setString(29, fcplList.get(i).getRemittanceRemarkCode4a());
							psInsertLineB3B5.setString(30, fcplList.get(i).getRemittanceRemarkCode4b());
							psInsertLineB3B5.setString(31, fcplList.get(i).getAdjustmentGroupCode5());
							psInsertLineB3B5.setString(32, fcplList.get(i).getAdjustmentReasonCode5());
							psInsertLineB3B5.setString(33, fcplList.get(i).getRemittanceRemarkCode5a());
							psInsertLineB3B5.setString(34, fcplList.get(i).getRemittanceRemarkCode5b());
							psInsertLineB3B5.setString(35, fcplList.get(i).getTypeOfPayment());
							psInsertLineB3B5.setString(36, fcplList.get(i).getCheckNumber());
							psInsertLineB3B5.setDate(37, sqlServiceDate);
							psInsertLineB3B5.setDate(38, sqlCheckDate);
							psInsertLineB3B5.setString(39, "CENTRAL_FEE");
							psInsertLineB3B5.setTimestamp(40,	new java.sql.Timestamp(today.getTime()));
							psInsertLineB3B5.setString(41, fcplList.get(i).getPaymentMethodology());
							psInsertLineB3B5.setString(42, fcplList.get(i).getAuthorizationNumber());
							psInsertLineB3B5.setDouble(43, fcplList.get(i).getBilledAmount());
							psInsertLineB3B5.setDouble(44, fcplList.get(i).getPaidAmount());
							
							psInsertLineB3B5.executeUpdate();
						}

					}
					
				} 
				// Increment to the next Claim
				i++;
			}
			
			/*
			 *  If Claim Index List is populated, then Claim Index either failed business rule or missing some required fields.
			 *  Update the FEE_CLAIM_PAYMENTS resolution_status_code to 'REVIEW_LOG' so that HAC OIT staff will review the 
			 *  summary_log on EDI Server
			 */
			if (!claimIndexList.isEmpty()) {
				// Builder dynamic parameter(s) for the Prepared Statement
				StringBuilder builder = new StringBuilder();
				for( int j = 0 ; j < claimIndexList.size(); j++ ) {
				    builder.append("?,");
				}
				
				// Build dynamic UPDATE Statement String to change Resolution Status Code on FEE_CLAIM_PAYMENTS from 'NEW' to 'REVIEW_LOG' 
				// based on Claim Index if a Central Fee file is missing required fields or failed business rules.
				String updateResolutionStatus = "UPDATE FEE_CLAIM_PAYMENTS "
						+ "SET resolution_status_code = 'REVIEW_LOG' "
						+ "WHERE claim_index IN ("
						+ builder.deleteCharAt(builder.length() - 1).toString() + ")";

				psUpdateResolutionStatus = conn.prepareStatement(updateResolutionStatus);

				// Loop through Claim Index List of problem claims and set the Prepared Statement parameter(s).
				int k = 1;
				for(Long o : claimIndexList ) {
					psUpdateResolutionStatus.setLong(k++, o);
				}
				psUpdateResolutionStatus.executeUpdate();
			}
			
			conn.commit();
			conn.setAutoCommit(true);
			DatabaseUtils.close(psInsertClaimB9);	
			DatabaseUtils.close(psInsertLineB9);;	
			DatabaseUtils.close(psInsertClaimB3B5);	
			DatabaseUtils.close(psInsertLineB3B5);
			DatabaseUtils.close(psGetClaimID);
			DatabaseUtils.close(psUpdateResolutionStatus);
			DatabaseUtils.close(conn);
			summaryLOGGER.info("Total Claim(s) Level entered: " + totalClaimEntered);
			summaryLOGGER.info("Total Line(s) Level entered: " + totalLineEntered);
			summaryLOGGER.info("Total Duplicate Claim(s) Level: " + totalClaimDup);
			summaryLOGGER.info("Total Duplicate Line(s) Level: " + totalLineDup);
			summaryLOGGER.info("Total Non-Existent Claim(s): " + totalNonClaim);
			summaryLOGGER.info("Total Claim(s): " + i);
		
		} catch (SQLException e){
			summaryLOGGER.error("SQL Exception: Claim transaction exception. Rollback Transaction(s). ", e);
			traceLOGGER.error("SQL Exception: Claim transaction exception.  Rollback Transaction(s). ", e);
			DatabaseUtils.rollback(conn);
			DatabaseUtils.close(psInsertClaimB9);	
			DatabaseUtils.close(psInsertLineB9);;	
			DatabaseUtils.close(psInsertClaimB3B5);	
			DatabaseUtils.close(psInsertLineB3B5);
			DatabaseUtils.close(psGetClaimID);
			DatabaseUtils.close(psUpdateResolutionStatus);
			DatabaseUtils.close(conn);
		} 
	}

	// Move Central Fee file to Archive or Error folder
	private static void moveCentralFeeFile(String fileDirectory,
			String fileName, String archiveDirectory) {
		DateFormat dateFormat = new SimpleDateFormat("MMddyyyyhhmmss");
		Date date = new Date();

		try {
			File file = new File(fileDirectory + fileName);
			String tmpFileName = null;

			if (fileName.contains(".TXT")) {
				tmpFileName = file.getName().replace(".TXT", "_" + dateFormat.format(date) + ".TXT");
			} else {
				tmpFileName = file.getName().replace(".txt", "_" + dateFormat.format(date) + ".txt");

			}

			if (file.renameTo(new File(fileDirectory + archiveDirectory	+ tmpFileName))) {
				summaryLOGGER.info("Central Fee file archived to: "	+ fileDirectory + archiveDirectory + tmpFileName);
				traceLOGGER.info("Central Fee file archived to: " + fileDirectory + archiveDirectory + tmpFileName);
			} else {
				summaryLOGGER.warn("Destination directory does not exist: " + fileDirectory + archiveDirectory + ". "	+ "File name (" + file.getName() + ") was not archived.");
				traceLOGGER.warn("Destination directory does not exist: " + fileDirectory + archiveDirectory + ". " + "File name (" + file.getName() + ") was not archived.");
			}
		} catch (Exception e) {
			summaryLOGGER.warn("Exception during Central Fee file archiving."
					+ e.getMessage());
			traceLOGGER.warn("Exception during Central Fee file archiving."
					+ e.getMessage());
			// e.printStackTrace();
		}
	}

	/**
	 * Helper method to populate command line options objects.
	 * 
	 * @return the <code>Options</code> object containing the set of command
	 *         line option.
	 */
	@SuppressWarnings("static-access")
	private static Options getCliOptions() {
		Options options = new Options();

		Option configfile = OptionBuilder
				.withArgName("file")
				.hasArg()
				.withLongOpt("configfile")
				.withDescription(
						"The HAC Configuration file, such as hacCentralFeeLoad.properties.")
				.create("f");

		options.addOption(configfile);

		return options;
	}

}
