package gov.va.med.hac.edi.era.jpa;

import gov.va.med.domain.erepos.*;
import gov.va.med.domain.erepos.batchload.*;
import gov.va.med.hac.edi.utils.*;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import javassist.bytecode.Descriptor.Iterator;

import javax.persistence.EntityManager;

public final class InsertionMethods
{
  // Constants
  final private static String TX_TYPE_835 = "835";


  // Insert a single record into table ENTITY_ADTL_ID
  void insertSegmentREF(
        final String pFileNumber
      , final String pSt02txnCtlNbrPadded
      , final String pClpKey
      , final String pSvcKey
      , final String pRef01Qualifier
      , final String pRef02Id
      , final String pLoopId, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row in ENTITY_ADTL_ID
    final EntityAdtlIdDAO trEntityAdtlId = new EntityAdtlIdDAO();

    // Set some fields
    trEntityAdtlId.setRefSegKey(UUID.randomUUID().toString().toUpperCase());
    trEntityAdtlId.setFileNumber(pFileNumber);
    trEntityAdtlId.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trEntityAdtlId.setSeqNbr(new BigDecimal("1"));
    trEntityAdtlId.setTxType(TX_TYPE_835);
    trEntityAdtlId.setLoopId(pLoopId);
    
    // If we have a claim payment key, this is an additional id for
    //   the claim payment
    if (!("".equals(pClpKey))) {
        trEntityAdtlId.setClaimPymtInfoKey(pClpKey);
    }

    // If the loop is 2110, this is an additional id for the service payment
    // WHERE IS SvcPaymentKey DEFINED? DAO or base?
    if ("2110".equals(pLoopId)) {
        trEntityAdtlId.setSvcPymtInfoKey(pSvcKey);
    } 
    
    trEntityAdtlId.setRef01idQlfr(pRef01Qualifier);
    trEntityAdtlId.setRef02scndId(pRef02Id);

    
    // Insert the row
    dbEM.persist(trEntityAdtlId);
  } // end method insertSegmentREF

  // Insert a record into table TRACE_NUMBER
  void insertTraceNumber(
        final Stg835HdrTrn pStg835HdrTrn
      , final String pFileNumber
      , final String pSt02txnCtlNbrPadded
      , final String pPayerId , 
      EntityManager dbEM)
  //throws Exception
  {
	  
	  
	  
    // Create a new trace number record in ERepos
    final TraceNumber trTraceNumber = new TraceNumber();
    TraceNumberId trTraceNumberId = new TraceNumberId(pFileNumber, pSt02txnCtlNbrPadded);
    
    System.out.println("insertTraceNumber() - traceNumberId is:" + trTraceNumberId.toString());
    
    trTraceNumber.setId(trTraceNumberId);		
    trTraceNumber.setTrn01trcTypeCd("1");
    trTraceNumber.setTrn02ckeftTrcNbr(pStg835HdrTrn.getTrn02ckeftTrcNbr());
    trTraceNumber.setTrn03pyrEftNbr(pStg835HdrTrn.getTrn03pyrEftNbr());
    trTraceNumber.setTrn04pyrSuplCd(pPayerId);

    System.out.println("insertTraceNumber() - traceNumber is: " + trTraceNumber.toString());
    
    // Insert the new record
    dbEM.persist(trTraceNumber);
  } // end method insertTraceNumber

  // Insert a record into table CLAIM_DT.  The first parameter indicates
  //   which date, and the second gives the date
  void insertDate(
        final String pFileNumber
      , final String pSt02TxnCtlNbr
      , final String pClpKey
      , final String pSvcKey
      , final String pDtp01
      , final String pDtp03, 
      EntityManager dbEM)
  throws SQLException
  {
    insertDateDTP4010(pFileNumber, pSt02TxnCtlNbr, pClpKey, pSvcKey, pDtp01, pDtp03, dbEM);
    insertDateDTM5010(pFileNumber, pSt02TxnCtlNbr, pClpKey, pSvcKey, pDtp01, pDtp03, dbEM);
} // end method insertDateDTM
  
  // Insert a record into table CLAIM_DTM.  The first parameter indicates
  //   which date, and the second gives the date
  void insertDateDTM5010(
        final String pFileNumber
      , final String pSt02TxnCtlNbr
      , final String pClpKey
      , final String pSvcKey
      , final String pDtm01
      , final String pDtm02, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row
    final ClaimDtmDAO trClaimDtm = new ClaimDtmDAO();
    
    // Set fields
    trClaimDtm.setClaimDtmKey(UUID.randomUUID().toString().toUpperCase());

    // The keys are used under various circumstances.
    // Only set these if not null and not blank
    if (!("".equals(pClpKey))) {
        trClaimDtm.setClaimPymtInfoKey(pClpKey);
    }
    if (!("".equals(pSvcKey))) {
        trClaimDtm.setSvcPymtInfoKey(pSvcKey);
    }
    if (pFileNumber != null && !(pFileNumber.equals(""))) {
        trClaimDtm.setFileNumber(pFileNumber);
    }
    if (pSt02TxnCtlNbr != null && !(pSt02TxnCtlNbr.equals(""))) {
        trClaimDtm.setSt02txnCtlNbr(pSt02TxnCtlNbr);
    }
    trClaimDtm.setSeqNbr(new BigDecimal("1"));
    trClaimDtm.setDtm01Qual(pDtm01);
    trClaimDtm.setDtm02dtFld(pDtm02);

    // Insert the row
    dbEM.persist(trClaimDtm);
  } // end method insertDateDTM

  // Insert a record into table CLAIM_DT.  The first parameter indicates
  //   which date, and the second gives the date
  void insertDateDTP4010(
        final String pFileNumber
      , final String pSt02TxnCtlNbr
      , final String pClpKey
      , final String pSvcKey
      , final String pDtp01
      , final String pDtp03, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row
    final ClaimDtDAO trClaimDt = new ClaimDtDAO();
    
    // Set fields
    trClaimDt.setClaimDtKey(UUID.randomUUID().toString().toUpperCase());

    // The four keys are used under various circumstances.
    // Only set these if not null and not blank
    if (!("".equals(pClpKey))) {
        trClaimDt.setClaimPymtInfoKey(pClpKey);
    }
    if (!("".equals(pSvcKey))) {
        trClaimDt.setSvcPymtInfoKey(pSvcKey);
    }
    if (pFileNumber != null && !(pFileNumber.equals(""))) {
        trClaimDt.setFileNumber(pFileNumber);
    }
    if (pSt02TxnCtlNbr != null && !(pSt02TxnCtlNbr.equals(""))) {
        trClaimDt.setSt02txnCtlNbr(pSt02TxnCtlNbr);
    }
    trClaimDt.setSeqNbr(new BigDecimal("1"));
    trClaimDt.setTxType(TX_TYPE_835);
    trClaimDt.setDtp01dttmQual(pDtp01);
    trClaimDt.setDtp03dtFld(pDtp03);

    // Insert the row
    dbEM.persist(trClaimDt);
  } // end method insertDateDTP

  // Insert a record into ERepos table EDI_835_PROCESS for this file
  void insertEdi835Process(
        final String pFileNumber
      , final String pClearinghouseId
      , final String pPayerId
      , final String pSt02txnCtlNbrPadded, 
      EntityManager dbEM)
  //throws Exception
  {
    // Create a new record for table EDI_835_PROCESS
    final Edi835Process tr835Process = new Edi835Process();

    // Set the file number field
    tr835Process.setFileNumber(pFileNumber);

    //TWP: add a new clearing house 
    // Set the clearinghouse and type
    if (pClearinghouseId.equalsIgnoreCase("133052274")) {
        tr835Process.setChName("EMDEON");
        tr835Process.setType("X12");
    } else if (pClearinghouseId.equalsIgnoreCase("403400000")) {
        tr835Process.setChName("MMI");
        tr835Process.setType("X12");
    } else if (pClearinghouseId.equalsIgnoreCase("403400001")) {
        tr835Process.setChName("MMIACH");
        tr835Process.setType("NCPDP");
    } else if (pClearinghouseId.equalsIgnoreCase("752578509")) {
        tr835Process.setChName("SXC");
        tr835Process.setType("NCPDP");
    }

    //TWP: add else if for 74161 number
    // The source depends on the payer id
    if ((pPayerId.equalsIgnoreCase("84146")) || (pPayerId.equalsIgnoreCase("84147")) || (pPayerId.equalsIgnoreCase("XCNET"))) {
        tr835Process.setSource("HAC");
    } else if ((pPayerId.equalsIgnoreCase("12115")) || (pPayerId.equalsIgnoreCase("12116"))) {
        tr835Process.setSource("FEE");
    }

    // Set the status
    tr835Process.setStatus("NEW");

    // Insert the record
    dbEM.persist(tr835Process);
  } // end method insertEdi835Process

  // Insert a skeleton record into table DETAIL_HEAD_INFO
  String insertProviderSummaryLX(
        final String pFileNumber
      , final String pSt02txnCtlNbrPadded,EntityManager dbEM)
  throws SQLException
  {
    String rDtlHeadInfoKey = null;

    // Create a new row for table DETAIL_HEAD_INFO
    final DetailHeadInfoDAO trDetailHeadInfo = new DetailHeadInfoDAO();
    
    // Set the fields
    rDtlHeadInfoKey = UUID.randomUUID().toString().toUpperCase();
    trDetailHeadInfo.setFileNumber(pFileNumber);
    trDetailHeadInfo.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trDetailHeadInfo.setDetailHeadInfoKey(rDtlHeadInfoKey);
    trDetailHeadInfo.setSeqNbr(new BigDecimal("1"));
    trDetailHeadInfo.setLx01headNbr(new Integer("1"));

    // Insert the row
    dbEM.persist(trDetailHeadInfo);

    return rDtlHeadInfoKey;
  } // end method insertProviderSummaryLX

  // Insert a record into table CLAIM_AMOUNT
  // TODO: This looks like it should be putting in an interest amount, but
  //   variable vIntAmt is never set above zero, so this code does nothing.
  // Return the number of rows inserted, for use in incrementing the segment count
  int insertClaimSuplAmountAMT(
        final String pClpKey
      , final BigDecimal pIntAmt,EntityManager dbEM)
  throws SQLException
  {
    // If there is an interest amount, use it
    if (pIntAmt.intValue() == 0) {
      return 0;
    } else {
      // Create a new row for table CLAIM_AMOUNT
      final ClaimAmountDAO trClaimAmount = new ClaimAmountDAO();

      // Set its fields
      // We only insert at most one Claim Amount for a claim payment, so its
      //   sequence number will be 1.
      trClaimAmount.setSeqNbr(new BigDecimal("1"));
      trClaimAmount.setClaimAmountKey(UUID.randomUUID().toString().toUpperCase());
      trClaimAmount.setClaimPymtInfoKey(pClpKey);
      trClaimAmount.setAmt01amtQualCd("I");
      trClaimAmount.setAmt02amt(pIntAmt);

      // Insert the row
      dbEM.persist(trClaimAmount);

      return 1;
    }
  } // end method insertClaimSuplAmountAMT

  void insertServiceSuplAmountAMT(String pClpKey, String pSvcKey, String pAmt01Qual
	      , final BigDecimal pAllowAmt, String pLoopId, EntityManager dbEM)
	  throws SQLException
  {
      // Create a new row for table CLAIM_AMOUNT
      final ClaimAmountDAO trClaimAmount = new ClaimAmountDAO();

      // Set its fields
      trClaimAmount.setSeqNbr(new BigDecimal("1"));
      trClaimAmount.setClaimAmountKey(UUID.randomUUID().toString().toUpperCase());
      trClaimAmount.setClaimPymtInfoKey(pClpKey);
      trClaimAmount.setSvcPymtInfoKey(pSvcKey);
      trClaimAmount.setLoopId(pLoopId);
      trClaimAmount.setTxType("835");
      trClaimAmount.setAmt01amtQualCd(pAmt01Qual);
      trClaimAmount.setAmt02amt(pAllowAmt);

      // Insert the row
      dbEM.persist(trClaimAmount);
  }// end method insertServiceSuplAmountAMT

  // Insert a record into table HCARE_REMARK_CD
  void insertHcareRemarkCd(
        final Stg835SvcLq pStg835SvcLq
      , final String pSvcKey
      , final int pSeqNbr, 
      EntityManager dbEM)
  //throws Exception
  {
    // Create a new row for this health care remark
    final HcareRemarkCd trHcareRemarkCd = new HcareRemarkCd();
    HcareRemarkCdId trHcareRemarkCdId = new HcareRemarkCdId();
    trHcareRemarkCdId.setSeqNbr(new BigDecimal(Integer.toString(pSeqNbr)));
    trHcareRemarkCdId.setSvcPymtInfoKey(pSvcKey);
    trHcareRemarkCd.setId(trHcareRemarkCdId);
    trHcareRemarkCd.setLq01cdQual(pStg835SvcLq.getLq01cdQual());
    trHcareRemarkCd.setLq02rmkCd(pStg835SvcLq.getLq02rmkCd());

    // Insert the new row
    dbEM.persist(trHcareRemarkCd);
  } // end method insertHcareRemarkCd

  // Insert a single row into table PRV_ADJ, which represents an adjustment
  //   at the provider level
  void insertProviderAdjustment(
        final String pFileNumber
      , final String pSt02txnCtlNbrPadded
      , final Stg835Plb crStg835Plb, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new record for table PRV_ADJ
    final PrvAdjDAO trPrvAdj = new PrvAdjDAO();
    
    // Set the fields
    trPrvAdj.setFileNumber(pFileNumber);
    trPrvAdj.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trPrvAdj.setPrvAdjKey(UUID.randomUUID().toString().toUpperCase());
    trPrvAdj.setSeqNbr(new BigDecimal("1"));
    trPrvAdj.setPlb01prvdId(crStg835Plb.getPlb01prvdId());
    trPrvAdj.setPlb02fsclPerDt(crStg835Plb.getPlb02fsclPerDt());
    trPrvAdj.setPlb03adjRsnCd(crStg835Plb.getPlb03adjRsnCd());
    trPrvAdj.setPlb03prvdAdjId(crStg835Plb.getPlb03prvdAdjId());
    trPrvAdj.setPlb04prvdAdjAmt(crStg835Plb.getPlb04prvdAdjAmt());

    // Insert the row
    dbEM.persist(trPrvAdj);
  } // end method insertProviderAdjustmentPLB

  // Insert a record into table INTCHG_CONTROL
  void insertInterchangeControl(
        final String pFileNumber
      , final String pPayerId
      , final String pClearinghouseId
      , final Stg835Hdr pStg835Hdr
      , final String pTransactionDate
      , final String pTransactionTime, 
      EntityManager dbEM)
  //throws Exception
  {
    // Create a new interchange control record
    final IntchgControl trIntchgControl = new IntchgControl();

    // Set the fields in the interchange control record
    trIntchgControl.setFileNumber(pFileNumber);
    trIntchgControl.setIsa01authQual("00");
    trIntchgControl.setIsa02authInfo("");
    trIntchgControl.setIsa03scrInfoQual("00");
    trIntchgControl.setIsa04scrInfo("");
    trIntchgControl.setIsa05sndIdQual("ZZ");
    trIntchgControl.setIsa06sndId(pPayerId);
    trIntchgControl.setIsa07rcvIdQual("ZZ");
    trIntchgControl.setIsa08rcvId(pClearinghouseId);
    trIntchgControl.setIsa09intchgDt(pTransactionDate.substring(2,pTransactionDate.length()));
    trIntchgControl.setIsa10intchgTm(pTransactionTime);
    trIntchgControl.setIsa11intchgStd('U');
    trIntchgControl.setIsa12intchgVer("00401");
    trIntchgControl.setIsa13intchgNbr(StringUtils.lPadZeros("1", 9));
    trIntchgControl.setIsa14ackRqst('0');
    if(pStg835Hdr.getUsageIndicator() != null)
    {
    	//TWP:Changed this added null check
    	trIntchgControl.setIsa15useInd(pStg835Hdr.getUsageIndicator().charAt(0));
    }
    trIntchgControl.setIsa16cpntEleSep(':');
    trIntchgControl.setTxType(TX_TYPE_835);
    trIntchgControl.setIea01nbrInclGrp("1");
    trIntchgControl.setIea02intchgNbr(StringUtils.lPadZeros("1", 9));

    // Insert the new interchange control record
    dbEM.persist(trIntchgControl);
  } // end method insertInterchangeControl

  // Insert a record into table FUNCTIONAL_GROUP
  FunctionalGroup insertFunctionalGroup(
        final String pFileNumber
      , final String pPayerId
      , final String pClearinghouseId
      , final String pTransactionDate
      , final String pTransactionTime, 
      EntityManager dbEM)
  //throws Exception
  {
    // Create a new functional group record
    final FunctionalGroup vrFunctionalGroup = new FunctionalGroup();

    // Set the fields in the functional group record
    vrFunctionalGroup.setFileNumber(pFileNumber);
    vrFunctionalGroup.setGs01fctlIdCd("HP");
    vrFunctionalGroup.setGs02appSndCd(pPayerId);
    vrFunctionalGroup.setGs03appRcvCd(pClearinghouseId);
    vrFunctionalGroup.setGs04creDt(pTransactionDate);
    vrFunctionalGroup.setGs05creTm(pTransactionTime);
    vrFunctionalGroup.setGs06grpCtlNbr(StringUtils.lPadZeros("1", 9));
    vrFunctionalGroup.setGs07respAgcyCd("X");
    vrFunctionalGroup.setGs08verRlsIndId("004010X091A1");
    vrFunctionalGroup.setTxType(TX_TYPE_835);

    // We have not yet counted the transaction sets, so we have to
    //   update column GE01NBR_TRANS_SETS later
    vrFunctionalGroup.setGe01nbrTranSets("");
    vrFunctionalGroup.setGe02grpCtlNbr(StringUtils.lPadZeros("1", 9));

    // Insert the new functional group record
    dbEM.persist(vrFunctionalGroup);
    return vrFunctionalGroup;
  } // end method insertFunctionalGroup

  // Insert a record into table FINANCIAL_INFO
  void insertFinancialInfo(
        final Stg835HdrBpr pStg835HdrBpr
      , final String pFileNumber
      , final String pSt02txnCtlNbrPadded, 
      EntityManager dbEM)
  //throws Exception
  {
    // Create a new record for table financial_info
    final FinancialInfoId trFinancialInfoId = new FinancialInfoId();
    trFinancialInfoId.setFileNumber(pFileNumber);
    trFinancialInfoId.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    
    FinancialInfo trFinancialInfo = new FinancialInfo();
    // Set some fields on the financial info reocrd
    trFinancialInfo.setId(trFinancialInfoId);

    // Several financial info fields are dependent on the payment method
    final String vBpr04PaymentMethod = pStg835HdrBpr.getBpr04pymtMthdCd();
    
    if (vBpr04PaymentMethod.equalsIgnoreCase("NON")) {
        trFinancialInfo.setBpr01txnHndlCd("H");
    } else {
        trFinancialInfo.setBpr01txnHndlCd("I");
    }
    trFinancialInfo.setBpr02pymtAmt(pStg835HdrBpr.getBpr02pymtAmt());
    trFinancialInfo.setBpr03crdbtFlag("C");
    trFinancialInfo.setBpr04pymtMthdCd(vBpr04PaymentMethod);
    
    
    if (vBpr04PaymentMethod.equalsIgnoreCase("ACH")) {
        trFinancialInfo.setBpr05pymtFmtCd("CCP");
    }
    if ((vBpr04PaymentMethod.equalsIgnoreCase("ACH"))
              || (vBpr04PaymentMethod.equalsIgnoreCase("BOP"))
              || (vBpr04PaymentMethod.equalsIgnoreCase("FWT"))) {
    	
    	
      trFinancialInfo.setBpr06pyrDfiQual("01");
      trFinancialInfo.setBpr07pyrDfiId(pStg835HdrBpr.getBpr07pyrDfiId());
      trFinancialInfo.setBpr08pyrAcctQual("DA");
      trFinancialInfo.setBpr09pyrAcctNbr(pStg835HdrBpr.getBpr09pyrAcctNbr());
      trFinancialInfo.setBpr10pyrId(pStg835HdrBpr.getBpr10pyrId());
      trFinancialInfo.setBpr12rcvDfiQual("01");
      trFinancialInfo.setBpr13rcvDfiId(pStg835HdrBpr.getBpr13rcvDfiId());
      trFinancialInfo.setBpr14rcvAcctQual("DA");
      trFinancialInfo.setBpr15rcvAcctNbr(pStg835HdrBpr.getBpr15rcvAcctNbr());
    }
    trFinancialInfo.setBpr16ckeftDt(pStg835HdrBpr.getBpr16ckeftDt());

    // Insert the new financial info record
    dbEM.persist(trFinancialInfo);
  } // end method insertFinancialInfo

  // Insert a record into table CLAIM_PYMT_INFO
  String insertClaimPymtInfo(
        final Stg835Clp pStg835Clp
      , final Claim rowClaim  
      , final String pFileNumber
      , final String pSt02txnCtlNbrPadded
      , final String pDtlHeadInfoKey
      , final String pClp02ClmStatusCode
      , EntityManager dbEM)
  throws SQLException
  {
	  
	  

	  
    String rClpKey = null;

    // Create a new claim payment record
    final ClaimPymtInfoDAO trClaimPymtInfo = new ClaimPymtInfoDAO();
   
    // Set the fields in the new claim payment record
    rClpKey = UUID.randomUUID().toString().toUpperCase();
    trClaimPymtInfo.setFileNumber(pFileNumber);
    trClaimPymtInfo.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trClaimPymtInfo.setClaimPymtInfoKey(rClpKey);
    trClaimPymtInfo.setDetailHeadInfoKey(pDtlHeadInfoKey);
    trClaimPymtInfo.setClp01ptCtlNbr(pStg835Clp.getClp01ptCtlNbr());
    
    
    //Sreedhar - Changes made for CR 13902
    //trClaimPymtInfo.setClp02clmStatCd(pStg835Clp.getClp02clmStatusCd());
    trClaimPymtInfo.setClp02clmStatCd(pClp02ClmStatusCode);
    trClaimPymtInfo.setClp03clmChrgAmt(pStg835Clp.getClp03clmChrgAmt());
    trClaimPymtInfo.setClp04pymtAmt(pStg835Clp.getClp04pymtAmt());
    if (pStg835Clp.getClp05ptRespAmt() != null &&
    		(!(pStg835Clp.getClp05ptRespAmt().equals(new BigDecimal(0)))))
    {
    	trClaimPymtInfo.setClp05ptRespAmt(pStg835Clp.getClp05ptRespAmt());
    }
    
    trClaimPymtInfo.setClp06clmFileInd("VA");
    //Getting Values from Inbound Claim data
    trClaimPymtInfo.setClp07pyrClmNbr(pStg835Clp.getClp07pyrClmNbr());

    //trClaimPymtInfo.setClp08fclyTypeCd(pStg835Clp.getClp08fclyTypeCd());
    //trClaimPymtInfo.setClp09clmFrqCd(pStg835Clp.getClp09clmFrqCd());
    
    if (rowClaim != null) {
	    trClaimPymtInfo.setClp08fclyTypeCd(rowClaim.getClm05fclyTypeCd());
	    trClaimPymtInfo.setClp09clmFrqCd(rowClaim.getClm05clmFrqCd().toString());
    } else {
	    trClaimPymtInfo.setClp08fclyTypeCd("");
	    trClaimPymtInfo.setClp09clmFrqCd("");   	
    }

    trClaimPymtInfo.setClp11drgCd(pStg835Clp.getClp11drgCd());
    
    // write new EntityAdtlId record if this field has values
    // record loopid is being written as 1000B
    // ClaimPaymentInfoKey is not being written
    EntityAdtlId ead = new EntityAdtlId();
    
    // check base class vs. DAO for definition of this field
    // class and current EndityAdtlId class
    // Stg835Clp is in batchload package
    // compare current vs. base download
    
    if(pStg835Clp.getClp14Ref02() != null) {
    	
    	
    	ead = insertEntityAdtlId("2100", pStg835Clp, rClpKey); 
    	
    	if(ead != null) {
          dbEM.persist(ead);
     	}
    	
    	
	    
    	Set<EntityAdtlId> s = trClaimPymtInfo.getEntityAdtlIds();
    	
//	    Set<EntityAdtlId> s = new HashSet<EntityAdtlId>();
	    s.add(ead);

/**	    
	    if(DEBUG) {
	    	java.util.Iterator<EntityAdtlId> i = s.iterator();
	    	
	    	while(i.hasNext()) {
	    		EntityAdtlId x = i.next();
	    		System.out.println("eaId: " + x.toString());
	    	}
	    }
**/	    
	    
	    
	    trClaimPymtInfo.setEntityAdtlIds(s);
    }
 
    // Insert the new claim payment record
    dbEM.persist(trClaimPymtInfo);
    
    return rClpKey;
  } // end method insertClaimPymtInfo
  
  
  
  public EntityAdtlId insertEntityAdtlId(String loopId, Stg835Clp stg835, String clPymtInfoKey) {
	  
//	boolean DEBUG = true;
	
	final String TX_TYPE = "835";
	final String CREATED_BY = "E_REPOS";
	final java.sql.Date NOW = new java.sql.Date(new java.util.Date().getTime());
	final BigDecimal SEQ_NBR= new BigDecimal("1");

    EntityAdtlIdDAO entityAdtlId = new EntityAdtlIdDAO();
 	
	  entityAdtlId.setRefSegKey(UUID.randomUUID().toString().toUpperCase()); 
      entityAdtlId.setTxType(TX_TYPE);
	  entityAdtlId.setFileNumber(stg835.getFileNumber());
	     
	  entityAdtlId.setCreatedBy(CREATED_BY);
	  entityAdtlId.setDateCreated(NOW);
	  entityAdtlId.setLoopId(loopId);
	  entityAdtlId.setSeqNbr(SEQ_NBR);
	  
 // HERE new fields added 5/5/2016 Bill B.
	  entityAdtlId.setRef01idQlfr(stg835.getClp15Amt01());	// NEW FIELD
	  entityAdtlId.setRef02scndId(stg835.getClp14Ref02());	// NEW FIELD
	  entityAdtlId.setClaimPymtInfoKey(clPymtInfoKey); 
	  entityAdtlId.setSt02txnCtlNbr(stg835.getSt02txnCtlNbr());
	  
	  
	  return entityAdtlId;
	           
     
}	// end insertEeaID()

  
  
  

  // Insert a record into table SVC_PYMT_INFO
  String insertSvcPymtInfo(
        final Stg835Svc pStg835Svc
      , final String pClpKey,EntityManager dbEM)
  throws SQLException
  {
    String rSvcKey = null;

    // Create a new record for table SVC_PYMT_INFO

    final SvcPymtInfoDAO trSvcPymtInfo = new SvcPymtInfoDAO();
    rSvcKey = UUID.randomUUID().toString().toUpperCase();

    // Set fields on the new row
    trSvcPymtInfo.setSvcPymtInfoKey(rSvcKey);
    trSvcPymtInfo.setClaimPymtInfoKey(pClpKey);
    trSvcPymtInfo.setSvc01svcIdQual(pStg835Svc.getSvc01svcIdQual());
    trSvcPymtInfo.setSvc01procCd(pStg835Svc.getSvc01procCd());
    trSvcPymtInfo.setSvc01procMod1(pStg835Svc.getSvc01procMod1());
    trSvcPymtInfo.setSvc01procMod2(pStg835Svc.getSvc01procMod2());
    trSvcPymtInfo.setSvc01procMod3(pStg835Svc.getSvc01procMod3());
    trSvcPymtInfo.setSvc01procMod4(pStg835Svc.getSvc01procMod4());
    trSvcPymtInfo.setSvc02chrgAmt(pStg835Svc.getSvc02chrgAmt());
    trSvcPymtInfo.setSvc03prvdPmt(pStg835Svc.getSvc03prvdPmt());
    trSvcPymtInfo.setSvc04nubcRevCd(pStg835Svc.getSvc04nubcRevCd());
    //Added pStg835Svc.getSvc05unitSvcPaid() != 1 to make sure not to populate when units are 1
    if(pStg835Svc.getSvc05unitSvcPaid() != null &&
       pStg835Svc.getSvc05unitSvcPaid() != 1)
    {
    	trSvcPymtInfo.setSvc05unitSvcPaid(new BigDecimal(pStg835Svc.getSvc05unitSvcPaid()));
    }
    //
    final String svc05 = pStg835Svc.getSvc05unitSvcPaid().toString();
    final String svc07 = pStg835Svc.getSvc07orgnunitSvc().toString();

    if (svc05.equalsIgnoreCase(svc07)) {
        trSvcPymtInfo.setSvc07orgnUnitSvc(null);
    } else {
    	//Added pStg835Svc.getSvc07orgnunitSvc() != 1 condition
    	if(pStg835Svc.getSvc07orgnunitSvc() != null &&
    	   pStg835Svc.getSvc07orgnunitSvc() != 1)
    	{
    		trSvcPymtInfo.setSvc07orgnUnitSvc(new BigDecimal(pStg835Svc.getSvc07orgnunitSvc()));
    	}
    }

    // Insert the new row into SVC_PYMT_INFO
    dbEM.persist(trSvcPymtInfo);

    return rSvcKey;
  } // end method insertSvcPymtInfo

  // Insert a record into table ENTITY_ID
  void insertEntityId(
        final String pFileNumber
      , final String pSt02txnCtlNbrPadded
      , final String pEntityName
      , final String pLoopId
      , final String pN101entyIdCd
      , final String pN103IdCdQual
      , final String pN104IdCd, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row in table ENTITY_ID
    final EntityIdDAO trEntityId = new EntityIdDAO();
    
    trEntityId.setEntityIdKey(UUID.randomUUID().toString().toUpperCase());
    trEntityId.setFileNumber(pFileNumber);
    trEntityId.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trEntityId.setLoopId(pLoopId);
    trEntityId.setN101entyIdCd(pN101entyIdCd);
    trEntityId.setN102name(pEntityName);

    trEntityId.setN103idCdQual(pN103IdCdQual);
    trEntityId.setN104idCd(pN104IdCd);

    dbEM.persist(trEntityId);
  } // end method insertEntityId

  // Insert a record into table ENTITY_ADDRESS for Payer Address
  void insertEntityAddress(
        final Stg835PayerInfo pStf835PayerInfo
      , final String pFileNumber
      , final String pSt02txnCtlNbrPadded, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row in ENTITY_ADDRESS
    final EntityAddressDAO trEntityAddress = new EntityAddressDAO();

    trEntityAddress.setEntityAddressKey(UUID.randomUUID().toString().toUpperCase());
    trEntityAddress.setFileNumber(pFileNumber);
    trEntityAddress.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trEntityAddress.setTxType(TX_TYPE_835);
    trEntityAddress.setLoopId("1000A");
    trEntityAddress.setN301adrLn1(pStf835PayerInfo.getPayerStreet());
    trEntityAddress.setN401city(pStf835PayerInfo.getPayerCity());
    trEntityAddress.setN402st(pStf835PayerInfo.getPayerState());
    trEntityAddress.setN403postalCd(pStf835PayerInfo.getPayerZip());

    dbEM.persist(trEntityAddress);
  } // end method insertEntityAddress 1000A

  // Insert a record into table ENTITY_ADDRESS for Payee Address
  void insertEntityAddress(
        final EntityAddressDAO rowEntityAddress837
      , final String pFileNumber
      , final String pSt02txnCtlNbrPadded, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row in ENTITY_ADDRESS
    final EntityAddressDAO trEntityAddress = new EntityAddressDAO();

    trEntityAddress.setEntityAddressKey(UUID.randomUUID().toString().toUpperCase());
    trEntityAddress.setFileNumber(pFileNumber);
    trEntityAddress.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trEntityAddress.setTxType(TX_TYPE_835);
    trEntityAddress.setLoopId("1000B");
    trEntityAddress.setN301adrLn1(rowEntityAddress837.getN301adrLn1());
    trEntityAddress.setN302adrLn2(rowEntityAddress837.getN302adrLn2());
    trEntityAddress.setN401city(rowEntityAddress837.getN401city());
    trEntityAddress.setN402st(rowEntityAddress837.getN402st());
    trEntityAddress.setN403postalCd(rowEntityAddress837.getN403postalCd());

    dbEM.persist(trEntityAddress);
  } // end method insertEntityAddress 1000B

  
  // Insert a record into table ENTITY_ADDRESS for Payee Address for Pharmacy 835's
  void insertEntityAddress(
        final TradingPartners rowTradingPartners
      , final String pFileNumber
      , final String pSt02txnCtlNbrPadded, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row in ENTITY_ADDRESS
    final EntityAddressDAO trEntityAddress = new EntityAddressDAO();

    trEntityAddress.setEntityAddressKey(UUID.randomUUID().toString().toUpperCase());
    trEntityAddress.setFileNumber(pFileNumber);
    trEntityAddress.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trEntityAddress.setTxType(TX_TYPE_835);
    trEntityAddress.setLoopId("1000B");
    trEntityAddress.setN301adrLn1(rowTradingPartners.getAddressLine1());
    trEntityAddress.setN302adrLn2(rowTradingPartners.getAddressLine2());
    trEntityAddress.setN401city(rowTradingPartners.getCity());
    trEntityAddress.setN402st(rowTradingPartners.getState());
    trEntityAddress.setN403postalCd(rowTradingPartners.getZipCode());

    dbEM.persist(trEntityAddress);
  } // end method insertEntityAddress 1000B
  
  // Insert a record into table CNTC_INFO for Technical Contact Information
  void insertContactInfo(
        final String pFileNumber
      , final String pSt02txnCtlNbrPadded
      , final String pCntcName
      , final String pPhoneNum
      , final String eMail,
      EntityManager dbEM)
  throws SQLException
  {

    final CntcInfoDAO trCntcInfo = new CntcInfoDAO();

    // Set the fields
    trCntcInfo.setContactInfoKey(UUID.randomUUID().toString().toUpperCase());
    trCntcInfo.setFileNumber(pFileNumber);
    trCntcInfo.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trCntcInfo.setTxType(TX_TYPE_835);
    trCntcInfo.setSeqNbr(new BigDecimal("2"));
    trCntcInfo.setLoopId("1000A");
    //trCntcInfo.setPer01cntcFctn("CX");
    trCntcInfo.setPer01cntcFctn("BL");
    trCntcInfo.setPer02cntcNm(pCntcName);
    if (!((eMail == null) || (eMail.equals("")))) {
        trCntcInfo.setPer03commQual1("EM");
        trCntcInfo.setPer04commNbr1(eMail);
    }
    if (!((pPhoneNum == null) || (pPhoneNum.equals("")))) {
        trCntcInfo.setPer05commQual2("TE");
        trCntcInfo.setPer06commNbr2(pPhoneNum);
    }
    
    //Using Payer_Fax column to fill in Business Email
    /*if (!((eMail2 == null) || (eMail2.equals("")))) {
        trCntcInfo.setPer07commQual3("EM");
        trCntcInfo.setPer08commNbr3(eMail2);
    }*/
    
    trCntcInfo.setPer07commQual3("");
    trCntcInfo.setPer08commNbr3("");

    // Insert the new row
    dbEM.persist(trCntcInfo);
  } // end method insertContactInfo
  
  // Insert a record into table CNTC_INFO for Business Contact Information
  void insertContactInfoBus(
        final String pFileNumber
      , final String pSt02txnCtlNbrPadded
      , final String pCntcName
      , final String pPhoneNum
      , final String eMail,
      EntityManager dbEM)
  throws SQLException
  {

    final CntcInfoDAO trCntcInfo = new CntcInfoDAO();

    // Set the fields
    trCntcInfo.setContactInfoKey(UUID.randomUUID().toString().toUpperCase());
    trCntcInfo.setFileNumber(pFileNumber);
    trCntcInfo.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trCntcInfo.setTxType(TX_TYPE_835);
    trCntcInfo.setSeqNbr(new BigDecimal("1"));
    trCntcInfo.setLoopId("1000A");
    trCntcInfo.setPer01cntcFctn("CX");
    trCntcInfo.setPer02cntcNm(pCntcName);
    if (!((eMail == null) || (eMail.equals("")))) {
        trCntcInfo.setPer03commQual1("EM");
        trCntcInfo.setPer04commNbr1(eMail);
    }
    if (!((pPhoneNum == null) || (pPhoneNum.equals("")))) {
        trCntcInfo.setPer05commQual2("TE");
        trCntcInfo.setPer06commNbr2(pPhoneNum);
    }
    
    trCntcInfo.setPer07commQual3("");
    trCntcInfo.setPer08commNbr3("");

    // Insert the new row
    dbEM.persist(trCntcInfo);
  } // end method insertContactInfo

  // Insert a record into table ENTITY_NAME
  void insertEntityNameFromStaging(
        final Stg835ClpName pStg835ClpName
      , final String pFileNumber
      , final String pClpKey
      , final String pSt02txnCtlNbrPadded, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row for target table ENTITY_NAME
    final EntityNameDAO trEntityName = new EntityNameDAO();

    // Set the fields
    trEntityName.setEntityNameKey(UUID.randomUUID().toString().toUpperCase());
    trEntityName.setClaimPymtInfoKey(pClpKey);
    trEntityName.setFileNumber(pFileNumber);
    trEntityName.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trEntityName.setLoopId("2100");
    trEntityName.setTxType(TX_TYPE_835);
    trEntityName.setNm101entyIdCd(pStg835ClpName.getNm101entyIdCd());
    trEntityName.setNm102entyQual(pStg835ClpName.getNm102entyQual());
    trEntityName.setNm103lastNm(pStg835ClpName.getNm103lastNm());
    trEntityName.setNm104fstNm(pStg835ClpName.getNm104fstNm());
    trEntityName.setNm105midNm(pStg835ClpName.getNm105midNm());
    //trEntityName.setNm108idCdQual(pStg835ClpName.getNm108idCdQual());
    trEntityName.setNm108idCdQual("MI");
    trEntityName.setNm109idCd(pStg835ClpName.getNm109entyId());

    // Insert the row
    dbEM.persist(trEntityName);
  } // end method insertEntityName

  // Insert a record into table ENTITY_NAME
  void insertEntityNameFromERepos(
        final EntityName pEntityName
      , final String pFileNumber
      , final String pClpKey
      , final String pSt02txnCtlNbrPadded, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row
    final EntityNameDAO trEntityName = new EntityNameDAO();

    // Set its fields
    trEntityName.setEntityNameKey(UUID.randomUUID().toString().toUpperCase());
    trEntityName.setClaimPymtInfoKey(pClpKey);
    trEntityName.setFileNumber(pFileNumber);
    trEntityName.setSt02txnCtlNbr(pSt02txnCtlNbrPadded);
    trEntityName.setLoopId("2100");
    trEntityName.setTxType(TX_TYPE_835);
    trEntityName.setNm101entyIdCd("82");
    trEntityName.setNm102entyQual(pEntityName.getNm102entyQual());
    trEntityName.setNm103lastNm(pEntityName.getNm103lastNm());
    trEntityName.setNm104fstNm(pEntityName.getNm104fstNm());
    trEntityName.setNm105midNm(pEntityName.getNm105midNm());
    trEntityName.setNm108idCdQual(pEntityName.getNm108idCdQual());
    trEntityName.setNm109idCd(pEntityName.getNm109idCd());

    // Insert the new row
    dbEM.persist(trEntityName);
  } // end method insertEntityName

  // Insert a record into table MEDI_INPT_ADJ
  void insertInpatientAdjId_MIA(
        final String pClpKey
      , EntityManager dbEM)
  throws SQLException
  {
    // Create a new row for target table MEDI_INPT_ADJ
    final MediInptAdjdctDAO trMediInptAdjdct = new MediInptAdjdctDAO();

    // Set the fields
    trMediInptAdjdct.setMiaKey(UUID.randomUUID().toString().toUpperCase());
    trMediInptAdjdct.setClaimPymtInfoKey(pClpKey);
    trMediInptAdjdct.setSeqNbr(new BigDecimal("1"));
    trMediInptAdjdct.setTxType(TX_TYPE_835);
    trMediInptAdjdct.setMia01cvrVisits(new BigDecimal("0"));
    trMediInptAdjdct.setMia05rmkCd1("MA15");

    // Insert the row
    dbEM.persist(trMediInptAdjdct);
  } // end method insertInpatientAdjdct_MIA


  // Insert a record into table MEDI_OUTPT_ADJ
  void insertOutpatientAdjId_MOA(
        final String pClpKey
      , EntityManager dbEM)
  throws SQLException
  {
    // Create a new row for target table MEDI_OUTPT_ADJ
    final MediOutptAdjdctDAO trMediOutptAdjdct = new MediOutptAdjdctDAO();

    // Set the fields
    trMediOutptAdjdct.setMoaKey(UUID.randomUUID().toString().toUpperCase());
    trMediOutptAdjdct.setClaimPymtInfoKey(pClpKey);
    trMediOutptAdjdct.setSeqNbr(new BigDecimal("1"));
    trMediOutptAdjdct.setTxType(TX_TYPE_835);
    trMediOutptAdjdct.setMoa03rmkCd1("MA15");

    // Insert the row
    dbEM.persist(trMediOutptAdjdct);
  } // end method insertOutpatientAdjdct_MIA

    
  // Insert a record into table CASES for an adjustment at the claim level
  void insertClaimAdjustment(
        final Stg835ClpCas pStg835ClpCas
      , final String pClpKey
      , final int pSeqNbr, 
      EntityManager dbEM)
  throws SQLException
  {
    // Create a new row for table CASES
    final CasesDAO trCases = new CasesDAO();

    // Set fields in the new record
    trCases.setCasesKey(UUID.randomUUID().toString().toUpperCase());
    trCases.setClaimPymtInfoKey(pClpKey);
    trCases.setLoopSeqNbr(new BigDecimal("1"));
    trCases.setSeqNbr(new BigDecimal(Integer.toString(pSeqNbr)));
    trCases.setTxType(TX_TYPE_835);
    trCases.setCas01clmAdjGrp(pStg835ClpCas.getCas01clmAdjGrp());
    trCases.setCas02clmAdjRsn1(pStg835ClpCas.getCas02clmAdjRsn1());
    trCases.setCas03adjAmt1(pStg835ClpCas.getCas03adjAmt1());
    trCases.setCas04adjQty1(pStg835ClpCas.getCas04adjQty1());
    
    if (pStg835ClpCas.getCas06adjAmt2() != null){
	    if (!(pStg835ClpCas.getCas06adjAmt2().equals(new BigDecimal(0)))){
	        trCases.setCas05clmAdjRsn2(pStg835ClpCas.getCas05clmAdjRsn2());
	        trCases.setCas06adjAmt2(pStg835ClpCas.getCas06adjAmt2());
	        trCases.setCas07adjQty2(pStg835ClpCas.getCas07adjQty2());    	
	    }
	    else {
	        trCases.setCas05clmAdjRsn2(pStg835ClpCas.getCas05clmAdjRsn2());
	        trCases.setCas06adjAmt2(pStg835ClpCas.getCas06adjAmt2());
	        trCases.setCas07adjQty2(pStg835ClpCas.getCas07adjQty2());     	
	    }
    }

    if (pStg835ClpCas.getCas09adjAmt3()!= null) {
	    if (!(pStg835ClpCas.getCas09adjAmt3().equals(new BigDecimal(0)))){
	        trCases.setCas08clmAdjRsn3(pStg835ClpCas.getCas08clmAdjRsn3());
	        trCases.setCas09adjAmt3(pStg835ClpCas.getCas09adjAmt3());
	        trCases.setCas10adjQty3(pStg835ClpCas.getCas10adjQty3());    	
	    }
	    else {
	        trCases.setCas08clmAdjRsn3(pStg835ClpCas.getCas08clmAdjRsn3());
	        trCases.setCas09adjAmt3(pStg835ClpCas.getCas09adjAmt3());
	        trCases.setCas10adjQty3(pStg835ClpCas.getCas10adjQty3());    	
	    }
    }

    if (pStg835ClpCas.getCas12adjAmt4() != null) {
	    if (!(pStg835ClpCas.getCas12adjAmt4().equals(new BigDecimal(0)))){
	        trCases.setCas11clmAdjRsn4(pStg835ClpCas.getCas11clmAdjRsn4());
	        trCases.setCas12adjAmt4(pStg835ClpCas.getCas12adjAmt4());
	        trCases.setCas13adjQty4(pStg835ClpCas.getCas13adjQty4());    	
	    }
	    else {
	        trCases.setCas11clmAdjRsn4(pStg835ClpCas.getCas11clmAdjRsn4());
	        trCases.setCas12adjAmt4(pStg835ClpCas.getCas12adjAmt4());
	        trCases.setCas13adjQty4(pStg835ClpCas.getCas13adjQty4());    	
	    }
    }

    if (pStg835ClpCas.getCas15adjAmt5() != null) {
	    if (!(pStg835ClpCas.getCas15adjAmt5().equals(new BigDecimal(0)))){
	        trCases.setCas14clmAdjRsn5(pStg835ClpCas.getCas14clmAdjRsn5());
	        trCases.setCas15adjAmt5(pStg835ClpCas.getCas15adjAmt5());
	        trCases.setCas16adjQty5(pStg835ClpCas.getCas16adjQty5());    	
	    }
	    else {
	        trCases.setCas14clmAdjRsn5(pStg835ClpCas.getCas14clmAdjRsn5());
	        trCases.setCas15adjAmt5(pStg835ClpCas.getCas15adjAmt5());
	        trCases.setCas16adjQty5(pStg835ClpCas.getCas16adjQty5());    	
	    }
    }

    if (pStg835ClpCas.getCas18adjAmt6() != null) {
	    if (!(pStg835ClpCas.getCas18adjAmt6().equals(new BigDecimal(0)))){
	        trCases.setCas17clmAdjRsn6(pStg835ClpCas.getCas17clmAdjRsn6());
	        trCases.setCas18adjAmt6(pStg835ClpCas.getCas18adjAmt6());
	        trCases.setCas19adjQty6(pStg835ClpCas.getCas19adjQty6());    	
	    }
	    else {
	        trCases.setCas17clmAdjRsn6(pStg835ClpCas.getCas17clmAdjRsn6());
	        trCases.setCas18adjAmt6(pStg835ClpCas.getCas18adjAmt6());
	        trCases.setCas19adjQty6(pStg835ClpCas.getCas19adjQty6());    	
	    }
    }

    // Insert the new record
    dbEM.persist(trCases);
  } // end method insertClaimAdjustment

  // Insert a record into table CASES for an adjustment at the service line level
  void insertServiceLevelAdjustment(
        final Stg835SvcCas pStg835SvcCas
      , final String pClpKey
      , final String pSvcKey
      , final int pSeqNbr, 
      EntityManager dbEM)
  throws SQLException
  {

    final CasesDAO trCases = new CasesDAO();

    // Set fields on the new row
    trCases.setLoopSeqNbr(new BigDecimal("1"));
    trCases.setSeqNbr(new BigDecimal(Integer.toString(pSeqNbr)));
    trCases.setCasesKey(UUID.randomUUID().toString().toUpperCase());
    trCases.setClaimPymtInfoKey(pClpKey);
    trCases.setSvcPymtInfoKey(pSvcKey);
    trCases.setTxType(TX_TYPE_835);
    trCases.setCas01clmAdjGrp(pStg835SvcCas.getCas01clmAdjGrp());
    trCases.setCas02clmAdjRsn1(pStg835SvcCas.getCas02clmAdjRsn1());
    trCases.setCas03adjAmt1(pStg835SvcCas.getCas03adjAmt1());
    

    if (pStg835SvcCas.getCas06adjAmt2() != null) {
	    if (!(pStg835SvcCas.getCas06adjAmt2().equals(new BigDecimal(0)))){
	        trCases.setCas05clmAdjRsn2(pStg835SvcCas.getCas05clmAdjRsn2());
	        trCases.setCas06adjAmt2(pStg835SvcCas.getCas06adjAmt2());
	        trCases.setCas07adjQty2(pStg835SvcCas.getCas07adjQty2());    	
	    }
	    else {
	        trCases.setCas05clmAdjRsn2(pStg835SvcCas.getCas05clmAdjRsn2());
	        trCases.setCas06adjAmt2(pStg835SvcCas.getCas06adjAmt2());
	        trCases.setCas07adjQty2(pStg835SvcCas.getCas07adjQty2());     	
	    }
    }

    if (pStg835SvcCas.getCas09adjAmt3() != null) {
	    if (!(pStg835SvcCas.getCas09adjAmt3().equals(new BigDecimal(0)))){
	        trCases.setCas08clmAdjRsn3(pStg835SvcCas.getCas08clmAdjRsn3());
	        trCases.setCas09adjAmt3(pStg835SvcCas.getCas09adjAmt3());
	        trCases.setCas10adjQty3(pStg835SvcCas.getCas10adjQty3());    	
	    }
	    else {
	        trCases.setCas08clmAdjRsn3(pStg835SvcCas.getCas08clmAdjRsn3());
	        trCases.setCas09adjAmt3(pStg835SvcCas.getCas09adjAmt3());
	        trCases.setCas10adjQty3(pStg835SvcCas.getCas10adjQty3());    	
	    }
    }

    if (pStg835SvcCas.getCas12adjAmt4() != null) {
	    if (!(pStg835SvcCas.getCas12adjAmt4().equals(new BigDecimal(0)))){
	        trCases.setCas11clmAdjRsn4(pStg835SvcCas.getCas11clmAdjRsn4());
	        trCases.setCas12adjAmt4(pStg835SvcCas.getCas12adjAmt4());
	        trCases.setCas13adjQty4(pStg835SvcCas.getCas13adjQty4());    	
	    }
	    else {
	        trCases.setCas11clmAdjRsn4(pStg835SvcCas.getCas11clmAdjRsn4());
	        trCases.setCas12adjAmt4(pStg835SvcCas.getCas12adjAmt4());
	        trCases.setCas13adjQty4(pStg835SvcCas.getCas13adjQty4());    	
	    }
    }

    if (pStg835SvcCas.getCas15adjAmt5() != null) {
	    if (!(pStg835SvcCas.getCas15adjAmt5().equals(new BigDecimal(0)))){
	        trCases.setCas14clmAdjRsn5(pStg835SvcCas.getCas14clmAdjRsn5());
	        trCases.setCas15adjAmt5(pStg835SvcCas.getCas15adjAmt5());
	        trCases.setCas16adjQty5(pStg835SvcCas.getCas16adjQty5());    	
	    }
	    else {
	        trCases.setCas14clmAdjRsn5(pStg835SvcCas.getCas14clmAdjRsn5());
	        trCases.setCas15adjAmt5(pStg835SvcCas.getCas15adjAmt5());
	        trCases.setCas16adjQty5(pStg835SvcCas.getCas16adjQty5());    	
	    }
    }

    if (pStg835SvcCas.getCas18adjAmt6() != null) {
	    if (!(pStg835SvcCas.getCas18adjAmt6().equals(new BigDecimal(0)))){
	        trCases.setCas17clmAdjRsn6(pStg835SvcCas.getCas17clmAdjRsn6());
	        trCases.setCas18adjAmt6(pStg835SvcCas.getCas18adjAmt6());
	        trCases.setCas19adjQty6(pStg835SvcCas.getCas19adjQty6());    	
	    }
	    else {
	        trCases.setCas17clmAdjRsn6(pStg835SvcCas.getCas17clmAdjRsn6());
	        trCases.setCas18adjAmt6(pStg835SvcCas.getCas18adjAmt6());
	        trCases.setCas19adjQty6(pStg835SvcCas.getCas19adjQty6());    	
	    }
    }

    /*
    trCases.setCas04adjQty1(pStg835SvcCas.getCas04adjQty1());
    trCases.setCas05clmAdjRsn2(pStg835SvcCas.getCas05clmAdjRsn2());
    trCases.setCas06adjAmt2(pStg835SvcCas.getCas06adjAmt2());
    trCases.setCas07adjQty2(pStg835SvcCas.getCas07adjQty2());
    trCases.setCas08clmAdjRsn3(pStg835SvcCas.getCas08clmAdjRsn3());
    trCases.setCas09adjAmt3(pStg835SvcCas.getCas09adjAmt3());
    trCases.setCas10adjQty3(pStg835SvcCas.getCas10adjQty3());
    trCases.setCas11clmAdjRsn4(pStg835SvcCas.getCas11clmAdjRsn4());
    trCases.setCas12adjAmt4(pStg835SvcCas.getCas12adjAmt4());
    trCases.setCas13adjQty4(pStg835SvcCas.getCas13adjQty4());
    trCases.setCas14clmAdjRsn5(pStg835SvcCas.getCas14clmAdjRsn5());
    trCases.setCas15adjAmt5(pStg835SvcCas.getCas15adjAmt5());
    trCases.setCas16adjQty5(pStg835SvcCas.getCas16adjQty5());
    trCases.setCas17clmAdjRsn6(pStg835SvcCas.getCas17clmAdjRsn6());
    trCases.setCas18adjAmt6(pStg835SvcCas.getCas18adjAmt6());
    trCases.setCas19adjQty6(pStg835SvcCas.getCas19adjQty6());
    */

    // Insert the new row
    dbEM.persist(trCases);
  } // end method insertServiceLevelAdjustment

}