package gov.va.med.hac.edi.extract.feeclaimstatus.feeFinal;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * POJO class to hold claim detail payment information
 * @author Bill Blackmon
 * @version R1.0.0
 *
 *HAPE POR VA DO# 118-11-D-1009, TO#118-1009-0001; HARRIS
 */
public class DetailRecord {
	
	private static final String DEFAULT_ZERO = "0.00";
	private BigDecimal billedAmount = null;
	private BigDecimal paidAmount = null;
	private BigDecimal claimProcedureSeq = null;
	private BigDecimal billedUnits = null;
	
	
	private Timestamp serviceFromDate;
	private Timestamp serviceToDate;
		
	private String revenueCode = "";
	private String procCodeDesc = "";
	private String procCode = "";
	private String lineItemControlNumber = "";
	
	
	private String procedureModifier1 = "";						
	private String procedureModifier2 = "";						
	private String procedureModifier3 = "";						
	private String procedureModifier4 = "";	
	
	private String serviceQualifier = "";
	private String emdeonClaimNumber = "";
	
	
	private boolean negativePaidAmount = false;
	
	

	public DetailRecord() {}
	


	public DetailRecord(BigDecimal billedAmount, BigDecimal paidAmount, BigDecimal claimProcedureSequence, String lineItemControlNumber, String procCodeDesc,
			Timestamp serviceFromDate, Timestamp serviceToDate, String revenueCode, BigDecimal billedUnits, String procCode,
			String procedureModifier1, String procedureModifier2, String procedureModifier3, String procedureModifier4, 
			String serviceQualifier, String emdeonClaimNumber, boolean negativePaidAmount) {
		
		clearAll();
		
 		this.billedAmount = billedAmount;
		this.paidAmount = paidAmount;
		
		
// System.out.println("DetailRec constr() - billedAmt and paidAmt are: " + this.billedAmount + " " + paidAmount);		
		
// TEST
		
		this.claimProcedureSeq = claimProcedureSequence;
		this.lineItemControlNumber = lineItemControlNumber;
		this.procCodeDesc = procCodeDesc;
		this.serviceFromDate = serviceFromDate;
		this.serviceToDate = serviceToDate;
		this.revenueCode = revenueCode;
		this.billedUnits = billedUnits;
		this.procCode = procCode;
		this.procedureModifier1 = procedureModifier1;
		this.procedureModifier2 = procedureModifier2;
		this.procedureModifier3 = procedureModifier3;
		this.procedureModifier4 = procedureModifier4;
		this.serviceQualifier = serviceQualifier;
		this.emdeonClaimNumber = emdeonClaimNumber;
		this.negativePaidAmount=negativePaidAmount;
		

 	}	// end constructor()
	
	/**
	 * This method clears all current values
	 * @author Bill Blackmon
	 * 
	 */
	public void clearAll() {
			
		billedAmount = null;
		paidAmount = null;
		
		claimProcedureSeq = null;
		billedUnits = null;
		
		
		serviceFromDate = null;
		serviceToDate = null;
			
		revenueCode = "";
		procCodeDesc = "";
		procCode = "";
		lineItemControlNumber = "";
		
		
		procedureModifier1 = "";						
		procedureModifier2 = "";						
		procedureModifier3 = "";						
		procedureModifier4 = "";	
		
		serviceQualifier = "";
		emdeonClaimNumber = "";
		
			 
	}
	public int getBilledUnits() {
		return this.billedUnits.intValue();
	}
	
	public String getServiceToDateFormatted() {
		String s = "";
		
		if(serviceToDate != null) {
			s = new SimpleDateFormat("yyyyMMdd").format(this.serviceToDate);
		} 
		
		return s;
	}
	
	public String getServiceFromDateFormatted() {
		String s = "";
		
		if(serviceFromDate != null) {
			s = new SimpleDateFormat("yyyyMMdd").format(this.serviceFromDate);
		} 
		
		return s;
	}

	public BigDecimal getBilledAmount() {
		
// System.out.println("DetailRecord.getBilledAmount() returning: " + billedAmount);			
		return billedAmount;
	}

	public void setBilledAmount(BigDecimal billedAmtount) {
		this.billedAmount = billedAmount;
	}

	public BigDecimal getPaidAmount() {
		
// System.out.println("DetailRecord.getPaidAmount() returning: " + paidAmount);		
		return paidAmount;
	}

	public void setPaidAmount(BigDecimal paidAmount) {
		this.paidAmount = paidAmount;
	}

	public BigDecimal getClaimProcedureSeq() {
		return claimProcedureSeq;
	}

	public void setClaimProcedureSeq(BigDecimal claimProcedureSeq) {
		this.claimProcedureSeq = claimProcedureSeq;
	}

	public String getProcCodeDesc() {
		return procCodeDesc;
	}

	public void setProcCodeDesc(String procCodeDesc) {
		this.procCodeDesc = procCodeDesc;
	}


	public void setLineItemControlNumber(String lineItemControlNumber) {
		this.lineItemControlNumber = lineItemControlNumber;
	}


	public Timestamp getServiceFromDate() {
		return serviceFromDate;
	}

	public void setServiceFromDate(Timestamp serviceFromDate) {
		this.serviceFromDate = serviceFromDate;
	}

	public Timestamp getServiceToDate() {
		return serviceToDate;
	}

	public void setServiceToDate(Timestamp serviceToDate) {
		this.serviceToDate = serviceToDate;
	}

	

	public String getRevenueCode() {
		return revenueCode;
	}

	public void setRevenueCode(String revenueCode) {
		this.revenueCode = revenueCode;
	}

	
	
	public String getLineItemControlNumber() {
		return this.lineItemControlNumber;	// .intValue();
	}

	public String getBilledAmountFormatted() {
		
		String out = "";
		
		if(this.billedAmount == null) {
			out = DEFAULT_ZERO;
			return out;
		}
		
	
		
		if( (this.billedAmount.longValue() > 0.00) || (this.billedAmount.longValue() < 0.00) ) {
			DecimalFormat df = new DecimalFormat("#.00");
			out = df.format(this.billedAmount.doubleValue());
		} else {
			out = DEFAULT_ZERO;
		}
		
		return out;
	}
	
	public String getPaidAmountFormatted() {
		
	
		String out = "";
		
		
		if(this.paidAmount == null) {
			out = DEFAULT_ZERO;
			return out;
		}
		
		if( (this.paidAmount.longValue() > 0.00) || (this.paidAmount.longValue() < 0.00))  {
			DecimalFormat df = new DecimalFormat("#.00");
			out = df.format(this.paidAmount.doubleValue());
		} else if (this.paidAmount.longValue() == 0.00) {
			out = DEFAULT_ZERO;
		}
		
		
		return out;
	}	// end getPaidAmountFormatted()



	public String getProcCode() {
		return procCode;
	}


	public void setProcCode(String procCode) {
		this.procCode = procCode;
	}


	public void setBilledUnits(BigDecimal billedUnits) {
		this.billedUnits = billedUnits;
	}


	@Override
	public String toString() {
		return "DetailRecord [billedAmount=" + billedAmount + ", paidAmount="
				+ paidAmount + ", claimProcedureSeq=" + claimProcedureSeq
				+ ", billedUnits=" + billedUnits + ", lineItemControlNumber="
				+ lineItemControlNumber + ", serviceFromDate="
				+ serviceFromDate + ", serviceToDate=" + serviceToDate
				+ ", revenueCode=" + revenueCode + ", procCodeDesc="
				+ procCodeDesc + ", procCode=" + procCode + "]";
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((billedAmount == null) ? 0 : billedAmount.hashCode());
		result = prime * result
				+ ((billedUnits == null) ? 0 : billedUnits.hashCode());
		result = prime
				* result
				+ ((claimProcedureSeq == null) ? 0 : claimProcedureSeq
						.hashCode());
		result = prime
				* result
				+ ((lineItemControlNumber == null) ? 0 : lineItemControlNumber
						.hashCode());
		result = prime * result
				+ ((paidAmount == null) ? 0 : paidAmount.hashCode());
		result = prime * result
				+ ((procCode == null) ? 0 : procCode.hashCode());
		result = prime * result
				+ ((procCodeDesc == null) ? 0 : procCodeDesc.hashCode());
		result = prime * result
				+ ((revenueCode == null) ? 0 : revenueCode.hashCode());
		result = prime * result
				+ ((serviceFromDate == null) ? 0 : serviceFromDate.hashCode());
		result = prime * result
				+ ((serviceToDate == null) ? 0 : serviceToDate.hashCode());
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DetailRecord other = (DetailRecord) obj;
		if (billedAmount == null) {
			if (other.billedAmount != null)
				return false;
		} else if (!billedAmount.equals(other.billedAmount))
			return false;
		if (billedUnits == null) {
			if (other.billedUnits != null)
				return false;
		} else if (!billedUnits.equals(other.billedUnits))
			return false;
		if (claimProcedureSeq == null) {
			if (other.claimProcedureSeq != null)
				return false;
		} else if (!claimProcedureSeq.equals(other.claimProcedureSeq))
			return false;
		if (lineItemControlNumber == null) {
			if (other.lineItemControlNumber != null)
				return false;
		} else if (!lineItemControlNumber.equals(other.lineItemControlNumber))
			return false;
		if (paidAmount == null) {
			if (other.paidAmount != null)
				return false;
		} else if (!paidAmount.equals(other.paidAmount))
			return false;
		if (procCode == null) {
			if (other.procCode != null)
				return false;
		} else if (!procCode.equals(other.procCode))
			return false;
		if (procCodeDesc == null) {
			if (other.procCodeDesc != null)
				return false;
		} else if (!procCodeDesc.equals(other.procCodeDesc))
			return false;
		if (revenueCode == null) {
			if (other.revenueCode != null)
				return false;
		} else if (!revenueCode.equals(other.revenueCode))
			return false;
		if (serviceFromDate == null) {
			if (other.serviceFromDate != null)
				return false;
		} else if (!serviceFromDate.equals(other.serviceFromDate))
			return false;
		if (serviceToDate == null) {
			if (other.serviceToDate != null)
				return false;
		} else if (!serviceToDate.equals(other.serviceToDate))
			return false;
		return true;
	}


	public String getProcedureModifier1() {
		return procedureModifier1;
	}


	public void setProcedureModifier1(String procedureModifier1) {
		this.procedureModifier1 = procedureModifier1;
	}


	public String getProcedureModifier2() {
		return procedureModifier2;
	}


	public void setProcedureModifier2(String procedureModifier2) {
		this.procedureModifier2 = procedureModifier2;
	}


	public String getProcedureModifier3() {
		return procedureModifier3;
	}


	public void setProcedureModifier3(String procedureModifier3) {
		this.procedureModifier3 = procedureModifier3;
	}


	public String getProcedureModifier4() {
		return procedureModifier4;
	}


	public void setProcedureModifier4(String procedureModifier4) {
		this.procedureModifier4 = procedureModifier4;
	}


	public String getServiceQualifier() {
		
		// pad to the right to make it 2 chars long
		if(serviceQualifier.length()==1)
			serviceQualifier+=" ";
		
		
		
		return serviceQualifier;
	}


	public void setServiceQualifier(String serviceQualifier) {
		this.serviceQualifier = serviceQualifier;
	}

	public String getEmdeonClaimNumber() {
		return emdeonClaimNumber;
	}

	public void setEmdeonClaimNumber(String emdeonClaimNumber) {
		this.emdeonClaimNumber = emdeonClaimNumber;
	}

	public boolean isNegativePaidAmount() {
		return negativePaidAmount;
	}

	public void setNegativePaidAmount(boolean negativePaidAmount) {
		this.negativePaidAmount = negativePaidAmount;
	}
	
}	// end DetailRecord class