package gov.va.med.hac.edi.extract.feeclaimstatus.feeFinal;


import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
This class is used to extract various fields not implemented in initial and pending processes
Method names are based on field names FROM the Emdeon Spec., may be abbreviated for readability.
The majority of these fields are set to null or empty String "" in the existing initial and pending
createPrimaryDetailRec() . This is an attempt to codify these fields and extract their values for a
given claim.

HAPE POR VA DO# 118-11-D-1009, TO#118-1009-0001; HARRIS

@author Bill Blackmon
@since 3/14/2014
@version R1.0.0
 *
 */
public class PaymentDataExtraction {



	static final Logger LOGGER = LoggerFactory.getLogger(PaymentDataExtraction.class);

	// not in DB, never read, always ""
	private String billingProviderPayerAssignedNumber="";
	private String serviceProviderPayerAssignedNumber="";

	private String billingProviderFederalTaxID="";
	private String billingProviderNationalProviderID="";
	private String billingProviderLastName="";
	private String billingProviderFirstName="";
	private String billingProviderMiddleName="";
	private String billingProviderNameSuffix="";

	private String serviceProviderFederalTaxID="";
	private String serviceProviderNationalProviderID="";
	private String serviceProviderLastName="";
	private String serviceProviderFirstName="";
	private String serviceProviderMiddleName="";
	private String serviceProviderNameSuffix="";

	private String employerIdentificationNumber="";
	private String employerName="";


	private String subscriberID="";
	private String subscriberLastName="";
	private String subscriberFirstName="";
	private String subscriberMiddleName="";
	private String subscriberNameSuffix="";

	private String patientID="";
	private String patientLastName="";
	private String patientFirstName="";
	private String patientMiddleName="";
	private String patientNameSuffix="";
	private String patientDateOfBirth="";
	private String patientGender="";


	private double claimChargeAmount;
	private double claimPaymentAmount;


	private String claimAdjudicationPaymentDate="";
	private String checkDate="";
	private String checkNumber="";
	private String billType="";

	private String payerClaimIdentificationNumber="";
	private String patientAccountNumber="";
	private String pharmacyPrescriptionNumber="";
	private String voucherIdentifier="";
	private String applicationSystemIdentifier="";
	private String groupNumber="";

	private Date claimServiceDateStart;
	private Date claimServiceDateEnd;


	private String payerID="";
	private String lineItemControlNumber="";
	private String serviceQualifierID="";
	private String serviceIdentificationCode="";
	private String procedureModifier1="";
	private String procedureModifier2="";
	private String procedureModifier3="";
	private String procedureModifier4="";
	private String revenueCode="";
	private int unitsOfService;
	private String emdeonClaimNumber="";
	private Date dateOfServiceStart;
	private Date dateOfServiceEnd;

	private String serviceTypeCode="";
	private String procCode="";


    @SuppressWarnings("rawtypes")
	ArrayList modifiers = new ArrayList();

    private BigDecimal lineItemChargeAmount;
	private BigDecimal lineItemProviderPaymentAmount;


	private EntityManager entityManager;
	private BigDecimal billedUnits;
	private Timestamp serviceFromDate;
	private Timestamp serviceToDate;
	private String procCodeDesc;
	private BigDecimal billedAmt;
	private BigDecimal paidAmt;
	private BigDecimal claimProcedureSeq;

	// flags for negative paid amounts
	private boolean bNegativePaidAmount=false;
	private boolean bNegativeChargeAmount=false;


	// flags for null payment amounts due to BAD data
	private boolean bNullClaimPaymentAmount=false;
	private boolean bNullClaimChargeAmount=false;
	/**
	BigDecimal claimPaymentAmount = null;
    BigDecimal claimChargeAmount = null;
    */


	// default constructor
	public PaymentDataExtraction() {};

	/**
	 * This method calls all of the set..() methods to load the data upfront
	 * NOT USED - 11/25
	 * @author DNS   BLACKW
	 * @param claimIndexId
	 */
	// load all data at once
	// NOTE: all methods here will have to be changed to private once this is implemented and tested.
	public void loadData(String claimIndexId) {

		setBillingProviderInfo(claimIndexId);
		setServiceProviderInfo(claimIndexId);
		setEmdeonClaimNumber(claimIndexId);
		setClaimChargePaymentAmounts(claimIndexId);
		setClaimAdjudicationPaymentDate(claimIndexId);
		setCheckDateAndNumber(claimIndexId);
		setPharmacyPrescriptionNumber(claimIndexId);
		setEmdeonClaimNumber(claimIndexId);





	}	// end loadData()


	/**
	 * This method sets the Emdeon Claim Number, Field # 33  in spec.
	 * @param claimIndexId
	 */
	public void setEmdeonClaimNumber(String claimIndexId) {

		Query q;
	    List<?> l;
	    Iterator<?> it;

			    try {

		    	  String sql =
		    		 " select " +
		    			  " REF02SCND_ID " +
		    		" from " +
		    			"entity_adtl_id e, claims c, claim cc " +
		    		" where " +
		    			"c.claim_index = " + claimIndexId + " and " +
		    			"edi_claim_key = cc.claim_key and " +
		    			" cc.claim_key = e.claim_key and " +
		    			" e.file_number = cc.file_number and " +
		    			" CC.ST02TXN_CTL_NBR = E.ST02TXN_CTL_NBR and E.REF01ID_QLFR = 'D9' ";



						      q = this.entityManager.createNativeQuery(sql);
						      l = q.getResultList();
						      it = l.iterator();

						      String tmp = "";

						      while (it.hasNext()) {
						         Object row = (String) it.next();
						         if(row != null) {
						        	tmp = row.toString();

						        	if(tmp != null)
						        		this.emdeonClaimNumber = tmp;
						         }
						      }

		      }catch(Exception ex) {
//		    	  System.out.println("PaymentDataExtraction.setEmdeonClaimNumber() ERROR: " + ex.getMessage());
		    	  LOGGER.error("Error on PaymentDataExtraction.setEmdeonClaimNumber(), errored on Claim Key: {}",  claimIndexId);
		          LOGGER.error(ex.getMessage(), ex);
		      }


	}	// end setEmdeonCode()

	/**
	 * This method sets the Billing Provider information
	 * @author Bill Blackmon
	 * @param claimIndexId
	 */
	public void setBillingProviderInfo(String claimIndexId) {

		boolean DEBUG = false;

		Query q;
	    List<?> l;
	    Iterator<?> it;
	    int iCount = 0;


	    try {



	    	  String sql = "select provider_npi, first_name, last_name,  middle_name, name_suffix,  provider_tin " +
	    	  " from provider_info " +
	    	  " where " +
	    	  " claim_index = " + claimIndexId + " and provider_type_cd = 'BILLING PROVIDER' ";

					      q = this.entityManager.createNativeQuery(sql);

					      l = q.getResultList();
					      it = l.iterator();

					      // all returned vals are Strings

					      while (it.hasNext()) {
					    	 iCount++;

					         Object[] row = (Object[]) it.next();


					         if(row != null) {
// Field #7
						         if(row[0] != null)
					        		 this.billingProviderNationalProviderID = (String) row[0];
// Field #9
					        	 if(row[1] != null)
					        		 this.billingProviderFirstName = (String) row[1];

// Field #8
					        	 if(row[2] != null)
					        		 this.billingProviderLastName = (String) row[2];
// Field #10
					        	 if(row[3] != null)
					        		 this.billingProviderMiddleName = (String) row[3];

// Field #11
					        	 if(row[4] != null)
					        		 this.billingProviderNameSuffix = (String) row[4];

// Field #5
					        	 if(row[5] != null)
					        		 this.billingProviderFederalTaxID = (String) row[5];


					      }	// end if

					 } // end while

					 if(iCount==0) {
if(DEBUG)
System.out.println("\nsetBillingProviderInfo() - no information found for claim index: " + claimIndexId);
					 }


 if(DEBUG)	{
					    System.out.println("\nBillingProviderInfo for Claim: " + claimIndexId);
					    System.out.println("Federal Tax ID: " + this.billingProviderFederalTaxID);
						System.out.println("National Provider ID: " +  this.billingProviderNationalProviderID);
						System.out.println("First Name: " +  this.billingProviderFirstName);
						System.out.println("Last Name: " + this.billingProviderLastName);
						System.out.println("Middle Name: " + this.billingProviderMiddleName);
						System.out.println("Name suffix: " + this.billingProviderNameSuffix);

						System.out.println("\n");
}



		      }catch(Exception ex) {
		    	  System.out.println("PaymentDataExtraction.setBillingProviderInfo() ERROR: " + ex.getMessage());
		    	  LOGGER.error("Error on PaymentDataExtraction.setBillingProviderInfo, errored on Claim Key: {}",  claimIndexId);
		          LOGGER.error(ex.getMessage(), ex);
		      }	// end try

	}	// end setBillingProviderInfo()


/**
Test scenarios from Anne D, - 7/26
86419549
A.	If both sp.NPI and sp.TaxID are null, display NOTHING for ALL service provider fields (DTL-FL12 thru DTL-FL18)

86320901 - CHECK THIS
B.	If sp.TaxID = bp.TaxId and sp.NPI = bp.NPI, display nothing for all service provider fields (DTL-FL12 thru DTL-FL18)

86932889 - CHECK THIS
C.	If sp.TaxId != bp.TaxId and sp.NPI = bp.NPI, display sp.TaxId value in DTL-FL12, display nothing for DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18

//86932908
D.	IF SP.TAXID = BP.TAXID AND SP.NPI != BP.NPI, DISPLAY NOTHING IN DTL-FL12, DISPLAY SP.NPI VALUE AND the rest of the values

86932909
E.	If sp.TaxId != bp.TaxId and sp.NPI != bp.NPI, display sp.TaxId value in DTL-FL12, display sp.NPI value in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18


86982459
F. If sp.TaxId != pb.TaxId and sp. NPI is null, display sp.TaxId value in DTL-FL12, display nothing in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18





 *
 *
**/
/**
 * Potential method to set BOTH billing and service provider info
 * @author Bill Blackmon
 * @since 11/26/2013
 * @param claimIndexId
 */

public void setProviderInfo(String claimIndexId) {

	boolean DEBUG = false;

	Query q;
    List<?> l;
    Iterator<?> it;

	boolean bSameTaxId = false;
	boolean bSameNPI = false;
	boolean bNullTaxId = false;
	boolean bNullNPI = false;

	boolean bBilling = false;
	boolean bService = false;

	String sProviderType = "";

	// compare sql from both methods -
	// servicePovider sql
	// SELECT provider_tin, provider_npi,  last_name, first_name,  middle_name, name_suffix

	// billing provider sql
	// select provider_npi, first_name, last_name,  middle_name, name_suffix,  provider_tin " +


	 String sql =                 "SELECT provider_tin, provider_npi, last_name, first_name, middle_name, name_suffix," +
	" (CASE " +
   	"	WHEN PROVIDER_TYPE_CD='BILLING PROVIDER' THEN 'B' " +
   	"	WHEN PROVIDER_TYPE_CD='RENDERING PROVIDER' THEN 'R' " +
   	" 	END ) as provider_type " +
   	"	From " +
   	"		provider_info " +
	" Where " +
	"	claim_index = " + claimIndexId + " and " +
  	"		((provider_type_cd = 'BILLING PROVIDER') " +
  	" 	or " +
  	" 	 	(provider_type_cd = 'RENDERING PROVIDER')) ";


if(DEBUG) {
	// System.out.println("setProviderInfo() - sql is: " + sql);
}

try {

    q = this.entityManager.createNativeQuery(sql);
    l = q.getResultList();
    it = l.iterator();

			    while (it.hasNext())
			    {

			    	// reset flags.
			    	bBilling = false;
			    	bService = false;

			        Object[] row = (Object[]) it.next();

/**
 * 	private String billingProviderFederalTaxID="";
	private String billingProviderNationalProviderID="";
	private String billingProviderLastName="";
	private String billingProviderFirstName="";
	private String billingProviderMiddleName="";
	private String billingProviderNameSuffix="";
 */

			       if(row != null)
			       {

			    	   		// GET THE PROVIDER TYPE CODE FIRST...

if(claimIndexId.equalsIgnoreCase("86325504")) {
	System.out.println("claim with both providers found.....be careful");
}
					    	   if(row[6] != null) {
						        	sProviderType = (String) row[6].toString();	// this is a char.....
						        	if(sProviderType.equalsIgnoreCase("B")) {
						        		bBilling = true;
						        	} else 	if(sProviderType.equalsIgnoreCase("R")) {
						        		bService = true;
						        	} else {
// catch the inevitable garbage value from the test database and print an error and exit
						        		if(DEBUG) {
								    	  System.out.println("\n\nPaymentDataExtraction.setProviderInfo() ERROR: INVALID PROVIDER TYPE: " + sProviderType + " for claim: " + claimIndexId);
						        		}
								    	LOGGER.error("PaymentDataExtraction.setProviderInfo() ERROR: INVALID PROVIDER TYPE: " + sProviderType + " for claim: " + claimIndexId);
								    	return;
						        	}

if(DEBUG) {
	System.out.println("setProviderInfo() - providerTypeCode is: " + sProviderType);
}


						        }


					    	   	//FIELD 12		tax id

// insert test for each field to check if it is bProvider or bBilling then assign values from billingProvider or serviceProvider.....
// the insert the nightmarish logic for each at the end of the method......slowly
					        	 if(row[0] != null) {
					        		 if(bService) {
					        			 this.serviceProviderFederalTaxID = (String) row[0];
					        		 } else {
					        			 this.billingProviderFederalTaxID = (String) row[0];
					        		 }
					        	 } else {				// is null and is ServiceProvider
					        		 if(bService) {
					        			 bNullTaxId = true;
					        		 }
					        	 }

			//field 14	 NPI
					        	 if(row[1] != null) {
					        		if(bService) {
					        			this.serviceProviderNationalProviderID = (String) row[1];
					        		} else {
					        			this. billingProviderNationalProviderID = (String) row[1];
					        		}
					        	 } else {
					        		 if(bService) {		// is null and is ServiceProvider
					        			 bNullNPI = true;
					        		 }
						         }

						         if(row[2] != null) {
						        	 if(bService) {
						        		 this.serviceProviderLastName = (String) row[2];
						        	 } else {
						        		 this.billingProviderLastName = (String) row[2];
						        	 }
						         }

								 if(row[3] != null) {
									 if(bService) {
										 this.serviceProviderFirstName = (String) row[3];
									 } else {
										 this.billingProviderLastName = (String) row[3];
									 }
								 }



								 if(row[4] != null) {
									 if(bService) {
										 this.serviceProviderMiddleName = (String) row[4];
									 } else {
										 this.billingProviderMiddleName = (String) row[4];
									 }
								 }



								 if(row[5] != null) {
									 if(bService) {
										 this.serviceProviderNameSuffix = (String) row[5];
									 } else {
										 this.billingProviderNameSuffix = (String) row[5];
									 }
								 }

			       }	// end null check





			}	// END RECORD LOOP.....

			    if(DEBUG) {
				    System.out.println("\n");
				    System.out.println("setProviderInfo().......");
					System.out.println("ServiceProvider Info form claim: " + claimIndexId);
					System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
					System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
					System.out.println("First Name: " + this.serviceProviderFirstName);
					System.out.println("Last Name: " + this.serviceProviderLastName);
					System.out.println("Middle Name: " + this.serviceProviderMiddleName);
					System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
					System.out.println("\n\n");
				   }	// END DEBUG

			    if(DEBUG)	{
				    System.out.println("BillingProviderInfo for Claim: " + claimIndexId);
				    System.out.println("Federal Tax ID: " + this.billingProviderFederalTaxID);
					System.out.println("National Provider ID: " +  this.billingProviderNationalProviderID);
					System.out.println("First Name: " +  this.billingProviderFirstName);
					System.out.println("Last Name: " + this.billingProviderLastName);
					System.out.println("Middle Name: " + this.billingProviderMiddleName);
					System.out.println("Name suffix: " + this.billingProviderNameSuffix);

					System.out.println("\n");
			    }

// logic......
			    if(DEBUG) {

		    	 	if(((! (this.serviceProviderFederalTaxID.equalsIgnoreCase(""))) && (! (this.serviceProviderNationalProviderID.equalsIgnoreCase("")) ) )) {
							System.out.println("setProviderInfo() - NEITHER TAX ID or PROVIDER ID are empty for id: " + claimIndexId);
					}

		// tax id is empty and provider id is NOT empty
		    	 	if(((this.serviceProviderFederalTaxID.equalsIgnoreCase("")) && (! (this.serviceProviderNationalProviderID.equalsIgnoreCase("")) ) )) {
		    	 			System.out.println("setProviderInfo() - TAX ID IS EMPTY and PROVIDER ID isn't for id: " + claimIndexId);
		    	 	}

		// tax id is NOT empty, provider id IS empty
		    	 	if(((!this.serviceProviderFederalTaxID.equalsIgnoreCase("")) && (this.serviceProviderNationalProviderID.equalsIgnoreCase("")) )) {
							System.out.println("setProviderInfo() - TAX ID IS NOT EMPTY and PROVIDER ID IS EMPTY for id: " + claimIndexId);
		    	 	}
			    }

			    if((this.serviceProviderFederalTaxID.equalsIgnoreCase("")) && (this.serviceProviderNationalProviderID.equalsIgnoreCase(""))) {
			    	if(DEBUG) {
			    	System.out.println("setProviderInfo() BOTH serviceProviderFederalTaxID and serviceProviderNationalProviderID are EMPTY for claim: " + claimIndexId);
			    	System.out.println("setProviderInfo() - SETTING servicedProvider information to null - WILL NOT BE DISPLAYED");
			    	}

			    	 this.serviceProviderLastName = "";
			    	 this.serviceProviderFirstName = "";
			    	 this.serviceProviderMiddleName = "";
			    	 this.serviceProviderNameSuffix = "";

			     return;

			    }



			    //
			    // ALSO CATCHES D...less specific - leave out for now.....7/28
			    /**
			     *
			     *
			      *
			    if( !this.serviceProviderNationalProviderID.equalsIgnoreCase(this.billingProviderNationalProviderID)) {

			    	if(DEBUG) {
			    		System.out.println("setProviderInfo() - Top Level NPI comparison - service NPI is DIFFERENT than billing NPI for claim: " + claimIndexId + " checking for null sp.TaxId");

			    	}


			    	if( !bNullTaxId ) {

			    		this.serviceProviderFederalTaxID=this.serviceProviderNationalProviderID;

			    		 if(DEBUG) {
			    			    System.out.println("setProviderInfo() - Top Level NPI comparison - sp.TaxId is NOT NULL - setting sp.TaxId to sp.NPI");
			    			  	System.out.println("\nServiceProvider Info for claim: " + claimIndexId);
			    				System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
			    				System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
			    				System.out.println("First Name: " + this.serviceProviderFirstName);
			    				System.out.println("Last Name: " + this.serviceProviderLastName);
			    				System.out.println("Middle Name: " + this.serviceProviderMiddleName);
			    				System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
			    				System.out.println("EXITING EVALUATION\n");

			    		  }	// end debug

			    		 return;
			    	}	// end null check

			    } else {

			    	if(DEBUG) {
			    		System.out.println("setProviderInfo() - Top Level NPI comparison - service NPI is THE SAME than billing NPI - continuing");
			    	}


			    }	// END top-level bp.NPI and sp.NPI comparison
			    **/



			    /**
			     * Please see 86932908, in this ex, Renderings tax id = Billings tax id, so Service Provider Federal Tax ID is null (captured correctly),
			     */


			    // 86419549
			    // If both sp.NPI and sp.TaxID are null, display NOTHING for ALL service provider fields (DTL-FL12 thru DTL-FL18)


			    		      if((bNullNPI) && (bNullTaxId) ) {


			    		    	  if(DEBUG) {
			    	    			  System.out.println("setProviderInfo() - Top Null check - If sp.TaxID = bp.TaxId and sp.NPI = bp.NPI, display nothing for all service provider fields (DTL-FL12 thru DTL-FL18) MET for id: " + claimIndexId);

			    	    			  	System.out.println("\nServiceProvider Info for claim: " + claimIndexId);
			    						System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
			    						System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
			    						System.out.println("First Name: " + this.serviceProviderFirstName);
			    						System.out.println("Last Name: " + this.serviceProviderLastName);
			    						System.out.println("Middle Name: " + this.serviceProviderMiddleName);
			    						System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
			    						System.out.println("EXITING EVALUATION\n");

			    	    		  }

			    				 this.serviceProviderLastName = "";
			    				 this.serviceProviderFirstName = "";
			    				 this.serviceProviderMiddleName = "";
			    				 this.serviceProviderNameSuffix = "";

			    				 return;

			    		      }


			    /**
			     * 86320901 -
			     *
			    B.	If sp.TaxID = bp.TaxId and sp.NPI = bp.NPI, display nothing for all service provider fields (DTL-FL12 thru DTL-FL18)

			     */

			    		      if(this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID)) {
			    		    	  if(this.serviceProviderNationalProviderID.equalsIgnoreCase(this.billingProviderNationalProviderID)) {

			    		    		  // clear ALL values
			    		    		  this.serviceProviderFederalTaxID="";
			    		    		  this.serviceProviderNationalProviderID="";
			    		    		  this.serviceProviderLastName = "";
			    					  this.serviceProviderFirstName = "";
			    					  this.serviceProviderMiddleName = "";
			    					  this.serviceProviderNameSuffix = "";

			    		    		  if(DEBUG) {
			    		    			  System.out.println("Check B. - If sp.TaxID = bp.TaxId and sp.NPI = bp.NPI, display nothing for all service provider fields (DTL-FL12 thru DTL-FL18) MET for id: " + claimIndexId);

			    		    			  	System.out.println("\nServiceProvider Info for claim: " + claimIndexId);
			    							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
			    							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
			    							System.out.println("First Name: " + this.serviceProviderFirstName);
			    							System.out.println("Last Name: " + this.serviceProviderLastName);
			    							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
			    							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
			    							System.out.println("EXITING EVALUATION\n");

			    		    		  }


			    		    		  // do not continue
			    					  return;

			    		    	  }

			    		      }	// end if


			    // end check B.

			    // check C.
			    /**
			     * 86932889
			     C.	If sp.TaxId != bp.TaxId and sp.NPI = bp.NPI, display sp.TaxId value in DTL-FL12, display nothing for DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18

			    **/

			    		      if(!this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID)) {
			    		    	  if(this.serviceProviderNationalProviderID.equalsIgnoreCase(billingProviderNationalProviderID)) {

			    		    		  // field NOTHING in field 14 NPI
			    		    		  this.serviceProviderNationalProviderID = "";

			    		    		  if(DEBUG) {
			    		    			  System.out.println("Check C. - If sp.TaxId != bp.TaxId and sp.NPI = bp.NPI, display sp.TaxId value in DTL-FL12, display nothing for DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18 for claim: " + claimIndexId);


			    		    			    System.out.println("\nServiceProvider Info for claim: " + claimIndexId);
			    							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
			    							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
			    							System.out.println("First Name: " + this.serviceProviderFirstName);
			    							System.out.println("Last Name: " + this.serviceProviderLastName);
			    							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
			    							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
			    							System.out.println("EXITING EVALUATION\n");
			    			    		  }

			    		    		  return;


			    		    	  }	// end if

			    		      }	// end if


			    // end check C.



			    		               if(this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID) ) {
			    		            		   if (!this.serviceProviderNationalProviderID.equalsIgnoreCase(this.billingProviderNationalProviderID)) {

			    		            			//  DISPLAY NOTHING IN sp.TaxId
			    		            	   		this.serviceProviderFederalTaxID = "";

			    		            	   			if(DEBUG) {
			    		    				    	    System.out.println("\n");
			    System.out.println("Check D. - If sp.TaxId != bp.TaxId and sp.NPI = bp.NPI, display sp.TaxId value in DTL-FL12, display nothing for DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18 for claim: " + claimIndexId);

			    		    							System.out.println("\nServiceProvider Info form claim: " + claimIndexId);
			    		    							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
			    		    							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
			    		    							System.out.println("First Name: " + this.serviceProviderFirstName);
			    		    							System.out.println("Last Name: " + this.serviceProviderLastName);
			    		    							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
			    		    							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
			    		    							System.out.println("EXITING EVALUATION\n");
			    		    						}

			    		            	   		// EXIT ALL PROCESSING......
			    		            	   		return;
			    		            		   }   // end IF

			    		               }	 // end IF



			    // end conditionn D


			    /**
			     * 86932909
			    E.	If sp.TaxId != bp.TaxId and sp.NPI != bp.NPI (BUT sp.NPI IS NOT NULL), display sp.TaxId value in DTL-FL12, display sp.NPI value in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18

			     */
			    		               if(!bNullNPI) { // null is different from "" - see F.
			    			               if(!this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID)) {
			    			            	   if(!this.serviceProviderNationalProviderID.equalsIgnoreCase(billingProviderNationalProviderID)) {

			    			            		   // display everything as-is

			    			         	   			if(DEBUG) {
			    		    				    	    System.out.println("\n");
			    	System.out.println("Check E. - If sp.TaxId != bp.TaxId and sp.NPI != bp.NPI, display sp.TaxId value in DTL-FL12, display sp.NPI value in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18 for claim: " + claimIndexId);

			    		    							System.out.println("\nServiceProvider Info form claim: " + claimIndexId);
			    		    							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
			    		    							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
			    		    							System.out.println("First Name: " + this.serviceProviderFirstName);
			    		    							System.out.println("Last Name: " + this.serviceProviderLastName);
			    		    							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
			    		    							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
			    		    							System.out.println("EXITING EVALUATION\n");
			    		    						}

			    			         	   			return;
			    			            	   }	// end if
			    			               }	// end if
			    		               }

			    // end condition E.


			    /**
			     * F. 86982459
			    If sp.TaxId != pb.TaxId and sp. NPI is null, display sp.TaxId value in DTL-FL12, display nothing in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18
			     */

			    		               	if(!this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID)) {

			    		               		if(bNullNPI) {
			    		               			// display nothing in DTL-FL14
			    		               			this.serviceProviderNationalProviderID = "";

			    		               			if(DEBUG) {
			    	    				    	    System.out.println("\n");
			    	System.out.println("Check F. - If sp.TaxId != pb.TaxId and sp. NPI is null, display sp.TaxId value in DTL-FL12, display nothing in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18 for claim: " + claimIndexId);

			    	    							System.out.println("\nServiceProvider Info form claim: " + claimIndexId);
			    	    							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
			    	    							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
			    	    							System.out.println("First Name: " + this.serviceProviderFirstName);
			    	    							System.out.println("Last Name: " + this.serviceProviderLastName);
			    	    							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
			    	    							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
			    	    							System.out.println("EXITING EVALUATION\n");
			    	    						}   // end debug

			    			               		return;

			    		               		}	// end if
			    		               	}	// end if
			    // end Condition F.


			    // condition 2




			    // 14 Service Provider National Provider ID test
			    					   // test for different ServiceProviderId and Billing Provider Id
			    // condititon 3


	  }catch(Exception ex) {
		  System.out.println("PaymentDataExtraction.setProviderInfo() ERROR: " + ex.getMessage());
		  ex.printStackTrace();
		  LOGGER.error("Error on PaymentDataExtraction.setProviderInfo, errored on Claim Key: {}",  claimIndexId);
	      LOGGER.error(ex.getMessage(), ex);
	  }	// END TRY/CATCH




	} // end setProviderInfo()

/**
	* NOTE: field #13 Service Provider Payer Assigned Number NOT included here! Under investigation 3/20
	 * This method sets the Service Provider Information
	 * @author Bill Blackmon
	 * @param claimIndexId
	 */
	public void setServiceProviderInfo(String claimIndexId) {


		boolean DEBUG=false;

		boolean bSameTaxId = false;
		boolean bSameNPI = false;
		boolean bNullTaxId = false;
		boolean bNullNPI = false;

		Query q;
	    List<?> l;
	    Iterator<?> it;
	    // int iCount = 0;

	    try {

	    	  String sql = "SELECT provider_tin, provider_npi,  last_name, first_name,  middle_name, name_suffix  " +
	    			  		" FROM provider_info p WHERE " +
	    			  		" claim_index = " + claimIndexId + " and " +
	    			  		" provider_type_cd = 'RENDERING PROVIDER' ";


	    	  				// " provider_type_cd = 'SERVICE_FACILITY ";

					if(DEBUG)	{
						// System.out.println("setServiceProviderInfo() - SQL IS: " + sql);
						System.out.println("setServiceProviderInfo() - CLAIM IS: " + claimIndexId);
					}

					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();

if(DEBUG) {
if(l.size()==0) {
	System.out.println("\nsetServiceProviderInfo() - NO RECORDS FOR CLAIM: " + claimIndexId + "\n");
	return;
}

	if(l.size() > 1) {
		System.out.println("\n\nsetServiceProviderInfo() -WHHOOAAAHHH! - multiple records for claim: " + claimIndexId + "\n\n");
	}
}

					      // all returned vals are Strings
					      String temp = "";

					      while (it.hasNext())
					      {
					          Object[] row = (Object[]) it.next();



					         if(row != null)
					         {



// FIELD 12		tax id
							        	 if(row[0] != null) {
							        		 this.serviceProviderFederalTaxID = (String) row[0];
							        	 } else {
							        		 bNullTaxId = true;
							        	 }


// field 14	 NPI
							        	 if(row[1] != null) {
								        	this.serviceProviderNationalProviderID = (String) row[1];
							        	 } else {
							        		 bNullNPI = true;
								        	 }

								         if(row[2] != null) {
											 this.serviceProviderLastName = (String) row[2];
								         }

										 if(row[3] != null) {
											 this.serviceProviderFirstName = (String) row[3];
										 }

										 if(row[4] != null) {
											 this.serviceProviderMiddleName = (String) row[4];
										 }

										 if(row[5] != null) {
											 this.serviceProviderNameSuffix = (String) row[5];
										 }

					         }


					 }	// END RECORD LOOP.....




if(DEBUG) {

    	 	if(((! (this.serviceProviderFederalTaxID.equalsIgnoreCase(""))) && (! (this.serviceProviderNationalProviderID.equalsIgnoreCase("")) ) )) {
    	 		/**
    	 		if(DEBUG) {
					System.out.println("setServiceProviderInfo() - NEITHER TAX ID or PROVIDER ID are empty for id: " + claimIndexId);
    	 		}
    	 		**/
			}

// tax id is empty and provider id is NOT empty
    	 	if(((this.serviceProviderFederalTaxID.equalsIgnoreCase("")) && (! (this.serviceProviderNationalProviderID.equalsIgnoreCase("")) ) )) {
    	 		/**
    	 		if(DEBUG) {
    	 			System.out.println("setServiceProviderInfo() - TAX ID IS EMPTY and PROVIDER ID isn't for id: " + claimIndexId);
    	 		}
    	 		**/
    	 	}

// tax id is NOT empty, provider id IS empty
    	 	if(((!this.serviceProviderFederalTaxID.equalsIgnoreCase("")) && (this.serviceProviderNationalProviderID.equalsIgnoreCase("")) )) {
    	 		/**
    	 		if(DEBUG) {
					System.out.println("setServiceProviderInfo() - TAX ID IS NOT EMPTY and PROVIDER ID IS EMPTY for id: " + claimIndexId);
    	 		}
    	 		**/
    	 	}
}


		if((this.serviceProviderFederalTaxID.equalsIgnoreCase("")) && (this.serviceProviderNationalProviderID.equalsIgnoreCase(""))) {

			if(DEBUG) {
			System.out.println("setServiceProviderInfo() BOTH serviceProviderFederalTaxID and serviceProviderNationalProviderID are EMPTY for claim: " + claimIndexId);
			System.out.println("setServiceProviderInfo() - SETTING servicedProvider information to null - WILL NOT BE DISPLAYED");
			}


			 this.serviceProviderLastName = "";
			 this.serviceProviderFirstName = "";
			 this.serviceProviderMiddleName = "";
			 this.serviceProviderNameSuffix = "";

		 return;

		}



//
// ALSO CATCHES D...less specific - leave out for now.....7/28
/**
 *
 *
  *
if( !this.serviceProviderNationalProviderID.equalsIgnoreCase(this.billingProviderNationalProviderID)) {

	if(DEBUG) {
		System.out.println("setServiceProviderInfo() - Top Level NPI comparison - service NPI is DIFFERENT than billing NPI for claim: " + claimIndexId + " checking for null sp.TaxId");

	}


	if( !bNullTaxId ) {

		this.serviceProviderFederalTaxID=this.serviceProviderNationalProviderID;

		 if(DEBUG) {
			    System.out.println("setServiceProviderInfo() - Top Level NPI comparison - sp.TaxId is NOT NULL - setting sp.TaxId to sp.NPI");
			  	System.out.println("\nServiceProvider Info for claim: " + claimIndexId);
				System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
				System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
				System.out.println("First Name: " + this.serviceProviderFirstName);
				System.out.println("Last Name: " + this.serviceProviderLastName);
				System.out.println("Middle Name: " + this.serviceProviderMiddleName);
				System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
				System.out.println("EXITING EVALUATION\n");

		  }	// end debug

		 return;
	}	// end null check

} else {

	if(DEBUG) {
		System.out.println("setServiceProviderInfo() - Top Level NPI comparison - service NPI is THE SAME than billing NPI - continuing");
	}


}	// END top-level bp.NPI and sp.NPI comparison
**/



/**
 * Please see 86932908, in this ex, Renderings tax id = Billings tax id, so Service Provider Federal Tax ID is null (captured correctly),
 */


// 86419549
// If both sp.NPI and sp.TaxID are null, display NOTHING for ALL service provider fields (DTL-FL12 thru DTL-FL18)


		      if((bNullNPI) && (bNullTaxId) ) {


		    	  if(DEBUG) {
	    			  System.out.println("setServiceProviderInfo() - Top Null check - If sp.TaxID = bp.TaxId and sp.NPI = bp.NPI, display nothing for all service provider fields (DTL-FL12 thru DTL-FL18) MET for id: " + claimIndexId);

	    			  	System.out.println("\nServiceProvider Info for claim: " + claimIndexId);
						System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
						System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
						System.out.println("First Name: " + this.serviceProviderFirstName);
						System.out.println("Last Name: " + this.serviceProviderLastName);
						System.out.println("Middle Name: " + this.serviceProviderMiddleName);
						System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
						System.out.println("EXITING EVALUATION\n");

	    		  }


				 this.serviceProviderLastName = "";
				 this.serviceProviderFirstName = "";
				 this.serviceProviderMiddleName = "";
				 this.serviceProviderNameSuffix = "";

				 return;

		      }


/**
 * 86320901 -
 *
B.	If sp.TaxID = bp.TaxId and sp.NPI = bp.NPI, display nothing for all service provider fields (DTL-FL12 thru DTL-FL18)

 */

		      if(this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID)) {
		    	  if(this.serviceProviderNationalProviderID.equalsIgnoreCase(this.billingProviderNationalProviderID)) {

		    		  // clear ALL values
		    		  this.serviceProviderFederalTaxID="";
		    		  this.serviceProviderNationalProviderID="";
		    		  this.serviceProviderLastName = "";
					  this.serviceProviderFirstName = "";
					  this.serviceProviderMiddleName = "";
					  this.serviceProviderNameSuffix = "";


		    		  if(DEBUG) {
		    			  System.out.println("setServiceProviderInfo()  Check B. - If sp.TaxID = bp.TaxId and sp.NPI = bp.NPI, display nothing for all service provider fields (DTL-FL12 thru DTL-FL18) MET for id: " + claimIndexId);

		    			  	System.out.println("\nServiceProvider Info for claim: " + claimIndexId);
							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
							System.out.println("First Name: " + this.serviceProviderFirstName);
							System.out.println("Last Name: " + this.serviceProviderLastName);
							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
							System.out.println("EXITING EVALUATION\n");

		    		  }



		    		  // do not continue
					  return;

		    	  }

		      }	// end if


// end check B.

// check C.
/**
 * 86932889
 C.	If sp.TaxId != bp.TaxId and sp.NPI = bp.NPI, display sp.TaxId value in DTL-FL12, display nothing for DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18

**/

		      if(!this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID)) {
		    	  if(this.serviceProviderNationalProviderID.equalsIgnoreCase(billingProviderNationalProviderID)) {

		    		  // field NOTHING in field 14 NPI
		    		  this.serviceProviderNationalProviderID = "";


		    		  if(DEBUG) {
		    			  System.out.println("setServiceProviderInfo() Check C. - If sp.TaxId != bp.TaxId and sp.NPI = bp.NPI, display sp.TaxId value in DTL-FL12, display nothing for DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18 for claim: " + claimIndexId);


		    			    System.out.println("\nServiceProvider Info for claim: " + claimIndexId);
							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
							System.out.println("First Name: " + this.serviceProviderFirstName);
							System.out.println("Last Name: " + this.serviceProviderLastName);
							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
							System.out.println("EXITING EVALUATION\n");
			    	 	}

		    		  return;


		    	  }	// end if

		      }	// end if


// end check C.



		               if(this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID) ) {
		            		   if (!this.serviceProviderNationalProviderID.equalsIgnoreCase(this.billingProviderNationalProviderID)) {

		            			//  DISPLAY NOTHING IN sp.TaxId
		            	   		this.serviceProviderFederalTaxID = "";


		            	   			if(DEBUG) {
		    				    	    System.out.println("\n");
System.out.println("setServiceProviderInfo() Check D. - If sp.TaxId != bp.TaxId and sp.NPI = bp.NPI, display sp.TaxId value in DTL-FL12, display nothing for DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18 for claim: " + claimIndexId);

		    							System.out.println("\nServiceProvider Info form claim: " + claimIndexId);
		    							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
		    							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
		    							System.out.println("First Name: " + this.serviceProviderFirstName);
		    							System.out.println("Last Name: " + this.serviceProviderLastName);
		    							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
		    							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
		    							System.out.println("EXITING EVALUATION\n");
		    						}


		            	   		// EXIT ALL PROCESSING......
		            	   		return;
		            		   }   // end IF

		               }	 // end IF



// end conditionn D


/**
 * 86932909
E.	If sp.TaxId != bp.TaxId and sp.NPI != bp.NPI (BUT sp.NPI IS NOT NULL), display sp.TaxId value in DTL-FL12, display sp.NPI value in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18

 */
		               if(!bNullNPI) { // null is different from "" - see F.
			               if(!this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID)) {
			            	   if(!this.serviceProviderNationalProviderID.equalsIgnoreCase(billingProviderNationalProviderID)) {

			            		   // display everything as-is

			         	   			if(DEBUG) {
		    				    	    System.out.println("\n");
	System.out.println("setServiceProviderInfo() - Check E. - If sp.TaxId != bp.TaxId and sp.NPI != bp.NPI, display sp.TaxId value in DTL-FL12, display sp.NPI value in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18 for claim: " + claimIndexId);

		    							System.out.println("\nServiceProvider Info form claim: " + claimIndexId);
		    							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
		    							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
		    							System.out.println("First Name: " + this.serviceProviderFirstName);
		    							System.out.println("Last Name: " + this.serviceProviderLastName);
		    							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
		    							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
		    							System.out.println("EXITING EVALUATION\n");
		    						}

			         	   			return;
			            	   }	// end if
			               }	// end if
		               }

// end condition E.


/**
 * F. 86982459
If sp.TaxId != pb.TaxId and sp. NPI is null, display sp.TaxId value in DTL-FL12, display nothing in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18
 */

		               	if(!this.serviceProviderFederalTaxID.equalsIgnoreCase(this.billingProviderFederalTaxID)) {

		               		if(bNullNPI) {
		               			// display nothing in DTL-FL14
		               			this.serviceProviderNationalProviderID = "";


		               			if(DEBUG) {
	    				    	    System.out.println("\n");
	System.out.println("setServiceProviderInfo() - Check F. - If sp.TaxId != pb.TaxId and sp. NPI is null, display sp.TaxId value in DTL-FL12, display nothing in DTL-FL14, and pull corresponding values for DTL-FL15 thru DTL-FL18 for claim: " + claimIndexId);

	    							System.out.println("\nServiceProvider Info form claim: " + claimIndexId);
	    							System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
	    							System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
	    							System.out.println("First Name: " + this.serviceProviderFirstName);
	    							System.out.println("Last Name: " + this.serviceProviderLastName);
	    							System.out.println("Middle Name: " + this.serviceProviderMiddleName);
	    							System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
	    							System.out.println("EXITING EVALUATION\n");
	    						}   // end debug

			               		return;

		               		}	// end if
		               	}	// end if
// end Condition F.

// debug output
if(DEBUG) {
	System.out.println("\nsetServiceProviderInfo() ServiceProvider Info form claim: " + claimIndexId);
	System.out.println("Federal Tax ID: " + this.serviceProviderFederalTaxID);
	System.out.println("National Provider ID: " + this.serviceProviderNationalProviderID);
	System.out.println("First Name: " + this.serviceProviderFirstName);
	System.out.println("Last Name: " + this.serviceProviderLastName);
	System.out.println("Middle Name: " + this.serviceProviderMiddleName);
	System.out.println("Name Suffix: " + this.serviceProviderNameSuffix);
	System.out.println("EXITING EVALUATION\n");
}

		      }catch(Exception ex) {
		    	  System.out.println("PaymentDataExtraction.setServiceProviderInfo() ERROR: " + ex.getMessage());
		    	  ex.printStackTrace();
		    	  LOGGER.error("Error on PaymentDataExtraction.setServiceProviderInfo, errored on Claim Key: {}",  claimIndexId);
		          LOGGER.error(ex.getMessage(), ex);
		      }	// END TRY/CATCH


} // end setServiceProviderInfo()







	/**
	 * This method sets the modifiers
	 * @author Bill Blackmon
	 * @param String claimIndexId
	 * @param String sequence number
	 *
	 */
	public void setModifiers(String claimIndexId, String procSeqNbr) {

		 String sql = "select modifier1, modifier2, modifier3, modifier4 from claim_procedure where CLAIM_INDEX = " + claimIndexId + " and claim_procedure_seq = " + procSeqNbr;

		Query q;
	    List<?> l;
	    Iterator<?> it;

// clear previous values.
	    this.procedureModifier1="";
	    this.procedureModifier2="";
	    this.procedureModifier3="";
	    this.procedureModifier4="";

		// check for nulls first - can't trust data
		if( (claimIndexId != null) && (procSeqNbr != null) ) {

			try {
				      q = this.entityManager.createNativeQuery(sql);
				      l = q.getResultList();
				      it = l.iterator();

				      // loop through and assign vals
				      String tmp = "";
				      while (it.hasNext()) {
					         Object[] row = (Object[]) it.next();


					         if(row != null) {
					        	 // check for null values before assignment...
					        	 tmp = (String) row[0];
					        	 if(tmp != null) {
					        		 this.procedureModifier1 = (String) row[0];
					        	 }

					        	 tmp = (String) row[1];
					        	 if(tmp != null) {
					        		 this.procedureModifier2 = (String) row[1];
					        	 }

					        	 tmp = (String) row[2];
					        	 if(tmp != null) {
					        		 this.procedureModifier3 = (String) row[2];
					        	 }

					        	 tmp = (String) row[3];
					        	 if(tmp != null) {
					        		 this.procedureModifier4 = (String) row[3];
					        	 }

						        } // end if
			         }   	// end while
			} catch(Exception ex) {
		    	  System.out.println("PaymentDataExtraction.setModifiers() ERROR: " + ex.getMessage());
		    	  LOGGER.error("Error on PaymentDataExtraction.setModifiers(), errored on Claim Key: {}",  claimIndexId);
		          LOGGER.error(ex.getMessage(), ex);
			}	// end try/catch

		}	// end if


	}	// end setModifiers()





	/**
	 * Get Entity-level Line Item Control Numbers
	 * @author Bill Blackmon
	 * @param String claim index
	 * @return ArrayList
	**/
	@SuppressWarnings("unchecked")
	public ArrayList getEntityLineItemControlNumbers(String claimIndexId) {

		boolean DEBUG = false;

		ArrayList list = new ArrayList();

		Query q;
	    List<?> l;
	    Iterator<?> it;
	    String tmp = "";

	    try {

	    	  String sql = " SELECT ref02scnd_id AS LINE_ITEM_CONTROL_NUMBER " +
	    			  	   " FROM Claims, entity_adtl_id " +
	    	               " WHERE claim_index = '" + claimIndexId + "' and Edi_claim_key = claim_key and loop_id = '2400' and ref01id_qlfr = '6R' order by LINE_ITEM_CONTROL_NUMBER  ";

					      q = this.entityManager.createNativeQuery(sql);
// fail....invalid number
// SELECT ref02scnd_id AS LINE_ITEM_CONTROL_NUMBER  FROM entity_adtl_id  WHERE claim_key = 86435644 and loop_id = '2400' and ref01id_qlfr = '6R' order by LINE_ITEM_CONTROL_NUMBER
					      l = q.getResultList();
					      it = l.iterator();


					      while (it.hasNext()) {
// System.out.println("Top Level lineItemControlNumbers found for id: " + claimIndexId);
// System.out.println("SQL IS: " + sql);
// CHECK FOR VALUES HERE THAT ARE PUT INTO ARRAY.....
						     Object row = (String) it.next();

					         if(row != null) {

tmp = row.toString();if(DEBUG) {
	System.out.println("pde.getEntityLineItemControlNumbers() - array item is: " + tmp);
}





					        	list.add(tmp);
					         }
					      }

	      }catch(Exception ex) {
	    	  System.out.println("PaymentDataExtraction.getEntityLineItemControlNumbers() ERROR: " + ex.getMessage());
	    	  LOGGER.error("Error on PaymentDataExtraction.getEntityLineItemControlNumbers(), errored on Claim Key: {}",  claimIndexId);
	          LOGGER.error(ex.getMessage(), ex);
	      }


		return list;
	} // end getEntityLineItemControlNumbers()




/**
 * ROUGH DRAFT - get detail records SQL
 *
 * find service code type first......


These line item control numbers have PRECEDENCE over the LINE_ITEM_CONTROL_NUMBERS in the separate
service-code-type based LINE_ITEM_CONTROL_NUMBERS.

This SQL may return multiple records or 0 records.
If 0 records, use the service-code-type LINE_ITEM_CONTROL_NUMBERs,
else use the TOP LEVEL items.....

Test this with new data to ensure that the count of TOP LEVEL records matches the count of detail records
for a given Claim.......

Move to separate method - getEntityLineItemControlNumber(String claimIndexId)
select ref02scnd_id AS LINE_ITEM_CONTROL_NUMBER	// this is a String
from
entity_adtl_id
where claim_key = xxx and
loop_id = '2400' and ref01id_qlfr = '6R'
order by ref02scnd_id
// multiple records returned.
//

use line_item_control_number here if found

in any case, run the individual sqls based on service code and disregard l.line_item_cotrol

@author Bill Blackmon
@param String claim index id
@return List of detail records for given claim
 *
 */
@SuppressWarnings({ "unchecked", "rawtypes", "null" })
public List getDetailRecords(String claimIndexId) {

	boolean DEBUG = false;

	List lDetailRecords = new ArrayList();
	String curSQL = "";
	boolean bTopFound = false;
	int pos = 0;
	int posTest = 0;

	Query q;
    List<?> l;
    Iterator<?> it;


 // new - 7/17
    String sqlProfessional = " SELECT " +
	    " claim_procedure_seq, proc_code_desc, cp.billed_amount, " +
	    " cpr.paid_amount, PSL.LX01ASGN_LN_ITM AS LINE_ITEM_CONTROL_NUMBER, " +
	    " cp.service_from_date,  cp.service_to_date, cp.revenue_cd, cp.billed_units, cp.proc_code, psl.SV101PROC_CD_QUAL " +
	 " FROM " +
	     " fpps_owner.claims c " +
	     " join fpps_owner.claim_procedure cp on c.claim_index = cp.claim_index " +
	     " join e_repos.claim cc on c.edi_claim_key = cc.claim_key " +
	     " join prfsn_svc_line psl  on cc.claim_key = psl.claim_key  and PSL.LX01ASGN_LN_ITM = CP.CLAIM_PROCEDURE_SEQ " +
	     " left join claim_proc_recon cpr on CP.CLAIM_PROC_ID = CPR.CLAIM_PROC_ID " +
	     " where " +
	     " c.claim_index =   " + claimIndexId + " ORDER BY claim_procedure_seq ";

		// new 7/17

		String sqlInstitutional = " SELECT " +
		   " claim_procedure_seq, proc_code_desc, cp.billed_amount, cpr.paid_amount, isl.LX01ASGN_LN_nbr AS LINE_ITEM_CONTROL_NUMBER, " +
		   " cp.service_from_date, cp.service_to_date, cp.revenue_cd, cp.billed_units, cp.proc_code, isl.SV202SVC_ID_QUAL " +
		" FROM "  +
		   " fpps_owner.claims c " +
		   " join fpps_owner.claim_procedure cp on c.claim_index = cp.claim_index " +
		   " join e_repos.claim cc on c.edi_claim_key = cc.claim_key " +
		   " join inst_svc_line isl  on cc.claim_key = isl.claim_key  and iSL.LX01ASGN_LN_nbr = CP.CLAIM_PROCEDURE_SEQ " +
		   " left join claim_proc_recon cpr on cpr.claim_proc_id = CP.CLAIM_PROC_ID " +
		" where " +
		  " c.claim_index =" + claimIndexId + " ORDER BY claim_procedure_seq ";


// DENTAL claims only  New 7/17

		String sqlDental = " SELECT " +
			" claim_procedure_seq, proc_code_desc, cp.billed_amount, cpr.paid_amount, dSL.LX01ASGN_LN_itm AS LINE_ITEM_CONTROL_NUMBER, " +
			" cp.service_from_date, cp.service_to_date, cp.revenue_cd, cp.billed_units,  cp.proc_code, " +
			"dsl.SV301ADA_CD_QUAL " +
			 " FROM " +
		         " fpps_owner.claims c " +
		         " join fpps_owner.claim_procedure cp on c.claim_index = cp.claim_index " +
		         " join e_repos.claim cc on c.edi_claim_key = cc.claim_key " +
		         " join dental_svc_line dsl  on cc.claim_key = dsl.claim_key  and dSL.LX01ASGN_LN_itm = CP.CLAIM_PROCEDURE_SEQ " +
		         " left join claim_proc_recon cpr on cp.claim_proc_id= cpr.claim_proc_id " +
			" where " +
		    	" c.claim_index = " + claimIndexId + " ORDER BY claim_procedure_seq ";



	// find out type of record - DENTAL, PROFESSIONAL or INSTITUTIONAL
	// based on serviceTypeCode, call one of the following SQL statements below.
	// all should return the same fields that are necessary to get ALL fields necessary for a detail record

	// set the serviceTypeCode for this particular claim
	try {
					// set the serviceTypeCode for this particular claim


					String stc = getServiceTypeCode();

					// THEN SET THIS VALUE FOR EACH CLAIM
					// setServiceQualifierID(claimIndexId);


// System.out.println("getDetailRecords() - service type code is: " + stc + " for claim: " + claimIndexId);

// test trap, 6/19 B.B.
					if( (stc.equalsIgnoreCase("")) || (stc.length()==0) || (stc == null) ) {
// EXIT METHOD......or exception will be thrown below
						return lDetailRecords;// new - 7/17
					}



					// get list of Entity-level line item control numbers
if(DEBUG) {
	System.out.println("\npde.getDetailRecords() - calling getLineItemControlNumbers");
	getNow();
}

// NOTE: POSSIBLY MOVE THIS CALL TO TOP OF RECORDS LOOP?
					ArrayList lcns = getEntityLineItemControlNumbers(claimIndexId);

if(DEBUG) {
	System.out.println("\npde.getDetailRecords() - DONE calling getLineItemControlNumbers");
	getNow();
}



// System.out.println("getDetailRecords() id = " +  claimIndexId + " - top-level lineItemControlNumbers array size is: " + lcns.size());

					if(lcns.size() > 0) {
//						 System.out.println("getDetailRecords() - TOP LEVEL DETAIL RECORDS FOUND for ID: " + claimIndexId);
						bTopFound = true;
					} else {
//						 System.out.println("getDetailRecords() - NO TOP LEVEL DETAIL RECORDS FOUND for ID: " + claimIndexId);
					}

					// run SQL based on stc
				 if ("DENT".equalsIgnoreCase(stc)) {
					 curSQL = sqlDental;
// System.out.println("PaymentDataExtraction.getDetailRecords() - DENT serviceTypeCode");
		    	   } else if("PROF".equalsIgnoreCase(serviceTypeCode)) {
		    		 curSQL = sqlProfessional;
// System.out.println("PaymentDataExtraction.getDetailRecords() - PROF serviceTypeCode");
		    	   } else if("INST".equalsIgnoreCase(serviceTypeCode)) {
		    		   curSQL = sqlInstitutional;
// System.out.println("PaymentDataExtraction.getDetailRecords() - INST serviceTypeCode");
		    	   }
// System.out.println("getDetailRecords() - executing SQL: " + curSQL);

if(DEBUG) {
	System.out.println("pde.getDetailRecords() - RUNNING QUERY..");
	getNow();
}
		 q = this.entityManager.createNativeQuery(curSQL);
	     l = q.getResultList();

if(DEBUG) {
		System.out.println("pde.getDetailRecords() - END RUNNING QUERY..");
		getNow();
}



			     it = l.iterator();

// System.out.println("getDetailRecords() record count is: " + l.size());

				 // extract values from resultset
				 while (it.hasNext()) {

if(DEBUG) {
//		System.out.println("\npde.getDetailRecords() - in record loop...");
//		getNow();
}


			         Object[] row = (Object[]) it.next();

			         if(row != null) {
			        	 if(row[0] != null) {
			        		 this.claimProcedureSeq = (BigDecimal) row[0];
			        	 } else {
// System.out.println("getDetailRecords() - claimProcedureSeq is null for id: " + claimIndexId);
			        	 }


			        	 if(row[1] != null)
			        		 this.procCodeDesc = (String) row[1];


			        	 if(row[2] != null) {
			        		 this.billedAmt = (BigDecimal) row[2];

// System.out.println("getDetailRecords() - billed amt is: " + this.billedAmt);
			        	 } else {
// System.out.println("getDetailRecords() - NULL BILLED AMOUNT");
			        	 }



			        	 if(row[3] != null) {
			        		 this.paidAmt = (BigDecimal) row[3];

			        		 if(this.paidAmt.signum() == -1) {
// 			        			 System.out.println("NEGATIVE AMT AND ID ARE: " + this.paidAmt + ", " + claimIndexId);
			        			 this.bNegativePaidAmount=true;
			        		 }
			        	 } else {
// System.out.println("getDetailRecords() - NULL PAID AMOUNT for: " + claimIndexId);

			        	 }


			        	 if(row[4] != null) {
			        		 // line item control numbers found....dSL.LX01ASGN_LN_itm AS LINE_ITEM_CONTROL_NUMBER
			        		 if(bTopFound) {
								// System.out.println("getDetailRecords() - topFound - swapping lcn - item in foundArray is: " + lcns.get(pos));

								if(DEBUG) {
										System.out.println("getDetailRecords() - lcns size is: " + lcns.size());
										System.out.println("posTest is: " + posTest);
										System.out.println("pos is: " + pos);
										System.out.println("claimIndexId is: " + claimIndexId);
									}

/**
 * java.lang.IndexOutOfBoundsException: Index: 2, Size: 2	- ERROR - pos is being incremented when it shouldn't be.....
 * WHAT IS THE CONDITION FOR IT TO BE INCREMENTED?
                at java.util.ArrayList.RangeCheck(Unknown Source) [na:1.6.0_29]
                at java.util.ArrayList.get(Unknown Source) [na:1.6.0_29]
 */
									try {
											// prevent index overrun
										    if(pos < lcns.size()) {
												this.lineItemControlNumber = (String) lcns.get(pos).toString();
												if(DEBUG) {
													System.out.println("pde.getDetailRecords() - lineItemControlNumber set to: " + this.lineItemControlNumber);
												}
										    }
									}catch(Exception x) {
										System.out.println("ERROR W ARRAY ID IS: " + claimIndexId);
										System.out.println("Error with array: " + x.getMessage());
									}
									pos++;

			        		 } else {
			        			 this.lineItemControlNumber = (String) row[4].toString();
// System.out.println("getDetailRecords() - topFound = FALSE - bottom lcn is: " + lineItemControlNumber);
			        		 }	// end top line item control number check
// System.out.println("getDetailRecords() - FINAL lcn is: " + lineItemControlNumber);
			        	 }	// end line item control number processing....


			        	 if(row[5] != null)
			        		 this.serviceFromDate = (Timestamp) row[5];

			        	 if(row[6] != null)
			        		 this.serviceToDate = (Timestamp) row[6];


			        	 if(row[7] != null) {
			        		 this.revenueCode = (String) row[7];
// System.out.println("getDetailRecords() - revenueCode is: " + this.revenueCode);
			        	 }
			        	 if(row[8] != null)
			        		 this.billedUnits = (BigDecimal) row[8];


// Detail item #7 Service Identification Code
			        	 if(row[9] != null) {
			        		 this.procCode = (String) row[9];
			        	 } else {
//			        		 System.out.println("getDetailRecords() - proc code is null - setting to empty string");
			        		 this.procCode = "";
			        	 }



// System.out.println("getDetailRecords() - Initial Claim ID, procCode and procCodeDescription values are: " + claimIndexId +": " + this.procCode + ": " +  this.procCodeDesc);

// HERE....
// from Mary - 8/26
/**
 * Inpatient institutional claims will often have no procedure codes
 * so you have to fall back to the Revenue code and then the qualifier
 * will be NU.  It was an institutional claim that failed.
 */
/** new logic - 8/22 **/
/**
 * 		So, if  Service Code is empty or null
		Set it to Revenue Code (if it exists)
		Set 6 Service Qualifier ID to NU     (regardless of Revenue Code)
 */




			        	 // set serviceQualifierID value
 // field 6 Service Qualifier ID

/**
if(DEBUG) {
		System.out.println("\npde.getDetailRecords() looping - CALLING setServiceQualifierID()...");
		getNow();
}
**/

// NOTE: possibly move this out to top record loop?
// setServiceQualifierID(claimIndexId);

/**
if(DEBUG) {
		System.out.println("\npde.getDetailRecords() looping - DONE CALLING setServiceQualifierID()...");
		getNow();
}
**/

// System.out.println("getDetailRecords() -  ServiceQualifierID returned is: " + this.getServiceQualifierID());


		        		 if(this.getServiceQualifierID().equalsIgnoreCase("NU")) {
// System.out.println("getDetailRecords() - serviceQualifierId is 'NU' - setting procCode to revenueCode: " + this.revenueCode);
							this.procCode = this.revenueCode;
		        		 }


		        		 if(this.getServiceQualifierID().equalsIgnoreCase("HC")) {
		        			 if(this.procCode.equalsIgnoreCase("")) {
		        				 this.serviceQualifierID="NU";
		        				 this.procCode=this.revenueCode;
		        			 }
		        		 }

// this test will pass if the above test is met.....
		        		 if(this.procCode.equalsIgnoreCase("")) {
							if(this.revenueCode.length() > 0)  {
								this.procCode=this.revenueCode;
							}
		        			this.serviceQualifierID="NU";
		        		 }		// end empty procCode trap.


						 // get modifiers list here....
		        		 if(this.claimProcedureSeq != null) {
		        			 setModifiers(claimIndexId, this.claimProcedureSeq.toString());
		        		 }

						// create new DetailRecord and add to list here.....
						 DetailRecord dRec = new DetailRecord(this.billedAmt, this.paidAmt, this.claimProcedureSeq, this.lineItemControlNumber, this.procCodeDesc, this.serviceFromDate,
								 this.serviceToDate, this.revenueCode, this.billedUnits, this.procCode,
								 	this.procedureModifier1, this.procedureModifier2, this.procedureModifier3,
								 	this.procedureModifier4, this.serviceQualifierID, this.emdeonClaimNumber, this.bNegativePaidAmount);

						 lDetailRecords.add(dRec);
// System.out.println("getDetailRecords() - new DetailRecord added to detailRecords arrayList");

				     }	// end if


			      }	// end while




	} catch(Exception ex) {
		System.out.println("PaymentDataExtraction.getDetailRecords() ERROR: " + ex.getMessage());
		ex.printStackTrace();
  	  	LOGGER.error("Error on PaymentDataExtraction.getDetailRecords(), errored on Claim Key: {}",  claimIndexId);
        LOGGER.error(ex.getMessage(), ex);
	}

//	    System.out.println("getDetailRecords() - size of detail records array returned is: " + lDetailRecords.size());

if(DEBUG) {
		System.out.println("\npde.getDetailRecords() END");
		getNow();
}

	return lDetailRecords;

}


	/**
	 * This method deterrmines the Service Qualifier ID
	 * #6 Service Qualifier ID in Line Level records via spec.
	 * @author Bill Blackmon
	 * @param String claim index id
	 *
	 */
	public void setServiceQualifierID(String claimIndexId) {

		boolean DEBUG = false;


if(DEBUG) {
	System.out.println("setServiceQualifierID() claim is: " + claimIndexId);
}

		@SuppressWarnings("unchecked")
		ArrayList<String> list = new ArrayList();
		list.add("AC");
		list.add("ER");
		list.add("HC");
		list.add("HP");
		list.add("IV");
		list.add("N4");
		list.add("NU");;
		list.add("WK");


		Query q;
	    List<?> l;
	    Iterator<?> it;
	    String tmp = "";
	    String defCode = "";
	    String retVal = "";




				String curSQL = "";

				// If its institutional use this chunk:
				String sqlINST = "select  " +
					    "max(sv202svc_id_qual) as CODE " +
					" from inst_svc_line i, claims c " +
					" where " +
					  " c.claim_index = " + claimIndexId + " and " +
					  " C.EDI_CLAIM_KEY = i.CLAIM_KEY ";


				// If its dental use this chunk:
				String sqlDENT = "select " +
					    " max(sv301ada_cd_qual) as CODE " +
					" from " +
					   " dental_svc_line i, " +
					   " claims c " +
					" where " +
					  " c.claim_index = " + claimIndexId + " and " +
					  "  C.EDI_CLAIM_KEY = i.CLAIM_KEY ";

				// If its professional use this chunk:
				String sqlPROF = "	select " +
					    " max(sv101proc_cd_qual) as CODE " +
					" from " +
					   " prfsn_svc_line i, claims c " +
					" where " +
					   " c.claim_index = " + claimIndexId + " and " +
					   " C.EDI_CLAIM_KEY = i.CLAIM_KEY ";



					if(this.serviceTypeCode.equalsIgnoreCase("PROF")) {
// System.out.println("setServiceQualifierID() - code is PROF ");
							curSQL = sqlPROF;
							defCode = "HC";
					} else if (this.serviceTypeCode.equalsIgnoreCase("DENT")) {
// System.out.println("setServiceQualifierID() - code is DENT ");
							curSQL = sqlDENT;
							defCode = "AD";
					} else if(this.serviceTypeCode.equalsIgnoreCase("INST")) {
// System.out.println("setServiceQualifierID() - code is INST - default Code is 'HC' ");
							curSQL = sqlINST;
							defCode = "HC";
					} else {
						// 	// error
System.out.println("setServiceQualifierID() - BAD code: " + this.serviceTypeCode);
					}

					// get one field and assign value to serviceQualifierId - this needs to be passed into DetailRecord for display

					 try {
					      q = this.entityManager.createNativeQuery(curSQL);
					      l = q.getResultList();
					      it = l.iterator();


					      while (it.hasNext()) {

					         Object row = (Object) it.next();

					         if(row != null) {
							      	tmp = row.toString().trim();

							       	if(tmp != null) {
								       	boolean found = list.contains(tmp);

							       		if(found) {
							       			this.serviceQualifierID = tmp;
// System.out.println("\nsetServiceTypeCode() - VALID CODE: serviceQualifierId is: " + tmp);

							       		} else {

							       			this.serviceQualifierID = defCode;

// System.out.println("\nsetServiceTypeCode() - INVALID CODE: " + tmp + ", setting to default value of: " + defCode);
// System.out.println("\n");


							       		}	// end if
								     }	// end if
							  }	// end if
					      }	// end while


if(DEBUG) {
	System.out.println("pde.setServiceQualifierId() - id is: " + this.serviceQualifierID);
	// getNow();

}


				      }catch(Exception ex) {
				    	  System.out.println("PaymentDataExtraction.setServiceQualifierID() ERROR: " + ex.getMessage() + "on ID : " + claimIndexId);
				    	  ex.printStackTrace();
				    	  LOGGER.error("Error on PaymentDataExtraction.setServiceQualifierID(), errored on Claim Key: {}",  claimIndexId);
				          LOGGER.error(ex.getMessage(), ex);
				      }

	}	// end setServiceQualifierID


	/**
	 * Step 1 of determining Service Type.....
	 * This method determines whether or not a claim is of type 'PROF', 'DENT' or 'INST'
	 * @author Bill Blackmon
	 * @param String claimIndexId
	 */
	public void setServiceTypeCode(String claimIndexId) {


		Query q;
	    List<?> l;
	    Iterator<?> it;
	    String tmp = "";

	    try {

	    	  			  String sql = "Select service_type_cd FROM fpps_owner.claims WHERE claim_index = " + claimIndexId;

					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();


					      while (it.hasNext()) {
					         Object row = (String) it.next();
					         if(row != null) {
					        	tmp = row.toString();

					        	// double check for bad data
					        	if( (tmp != null) && (tmp.length() > 0) ) {
					        		this.serviceTypeCode = tmp;
					        	}


					         } else {
					        		// bad record - no serviceTypeCode
// System.out.println("PaymentDataExtraction.setServiceTypeCode() no valid service type code for Claim Key: " + claimIndexId);

					        }	// end null check

					      }	// end while


// System.out.println("setServiceTypeCode() for claim Id: " + claimIndexId + "; returning code: " + tmp);



	      }catch(Exception ex) {
	    	  System.out.println("PaymentDataExtraction.setServiceTypeCode() ERROR: " + ex.getMessage());
	    	  ex.printStackTrace();
	    	  LOGGER.error("Error on PaymentDataExtraction.setServiceTypeCode(), errored on Claim Key: {}",  claimIndexId);
	          LOGGER.error(ex.getMessage(), ex);
	      }

// System.out.println("serviceTypeCode is: " + this.serviceTypeCode);

		// this.serviceTypeCode = serviceTypeCode;
	}	// end setServiceTypeCode()


///////////////////////////////////

	public String getServiceTypeCode() {
		return serviceTypeCode;
	}



/**
 * 6/14/2013
 * Problem - What do we want to display in the output file in the case of null values for either of the returned fields?
 * This method determines the payment and charge amounts
 * @author Bill Blackmon
 * @param claimIndexId
 */

	public void setClaimChargePaymentAmounts(String claimIndexId) {

		boolean DEBUG = false;

		Query q;
	    List<?> l;
	    Iterator<?> it;
	    BigDecimal claimPaymentAmount = null;
	    BigDecimal claimChargeAmount = null;


	    try {

// New 7/10
/**
 String sql =
  " SELECT sum(cpay.paid_amount), sum(cp.billed_amount) " +
  " FROM claim_procedure cp, claim_proc_recon cpr, claim_payment cpay " +
  " WHERE " +
  " CP.CLAIM_PROC_ID = CPR.CLAIM_PROC_ID " +
  " and CPR.CLAIM_PAYMENT_ID = CPAY.CLAIM_PAYMENT_ID and Cp.claim_index =  " + claimIndexId;
**/

/** Take 2 - 7/11 **/
/**
String sql = "select sum(cpay.paid_amount), sum(cp.billed_amount) " +
			 " From " +
             " claim_procedure cp, " +
             " claim_payment cpay " +
			 " where cp.claim_index = cpay.claim_index and cpay.claim_index = " + claimIndexId;
**/

// take 3  - 7/12
/**
String sql = "select sum(cpay.paid_amount), sum(cp.billed_amount)  From  claim_procedure cp,  claim_payment cpay  , claim_proc_recon cpr " +
	" where " +
	" cpr.claim_index = " + claimIndexId + " and " +
	" cpr.claim_proc_id = cp.claim_proc_id and " +
	" cpr.claim_payment_id = cpay.claim_payment_id ";
**/

// 7/17
	    	String sql = "select sum(cpay.paid_amount), sum(cp.billed_amount)  From  claim_procedure cp,  claim_payment cpay  , claim_proc_recon cpr " +
	    			" where " +
	    			" cp.claim_index = " + claimIndexId + " and " +
	    			" cp.claim_proc_id = cpr.claim_proc_id(+) and " +
	    			" cpr.claim_payment_id = cpay.claim_payment_id(+) ";




if(DEBUG)
System.out.println("setClaimChargePaymentAmounts() - sql is: " + sql);

					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();

					      // all returned vals are BigDecimals - deal with formatting after correct vals are assured.

					      while (it.hasNext()) {
					         Object[] row = (Object[]) it.next();
					         if(row != null) {
					         	 claimPaymentAmount = (BigDecimal) row[0];
					        	 claimChargeAmount = (BigDecimal) row[1];

					        	 if(claimPaymentAmount != null) {
						           	 if(claimPaymentAmount.signum() == -1) {
						        		 this.bNegativePaidAmount=true;
						        	 }
						           	this.setClaimPaymentAmount(claimPaymentAmount.doubleValue());

if(DEBUG)
System.out.println("setClaimChargePaymentAmounts() - ClaimPaymentAmt is: " + this.claimPaymentAmount);

						           	this.bNullClaimPaymentAmount=false;

					        	 }	else {
					        		 this.bNullClaimPaymentAmount=true;

if(DEBUG)
System.out.println("NULL CLAIM PAYMENT AMOUNT IS TRUE");

					        	 }



					        	 if(claimChargeAmount != null) {
					        		 /**
					        		 if(claimPaymentAmount.signum() == -1) {
						        		 this.bNegativeChargeAmount=true;
						        	 }
						        	 **/


					        	 	this.setClaimChargeAmount(claimChargeAmount.doubleValue());
					        	 	this.bNullClaimChargeAmount=false;

if(DEBUG)
System.out.println("setClaimChargePaymentAmounts() - ClaimChargeAmt is: " + this.claimChargeAmount);

					        	 } else {
					        		 this.bNullClaimChargeAmount=true;
					        	 }

						     }


if(DEBUG) {
	if( (this.bNullClaimChargeAmount==true) || (this.bNullClaimPaymentAmount==true) ) {
		 System.out.println("\nsetClaimChargePaymentAmounts() - NULL PAYMENT AMOUNT(S) for claim #: " + claimIndexId);
		 System.out.println("setClaimChargePaymentAmounts() - claim id, CHARGED and PAID amts are: " + claimIndexId +", " + claimChargeAmount + ", " + claimPaymentAmount);
	     System.out.println("setClaimChargePaymentAmounts() - bNullClaimChargeAmount and bNullClaimPaymentAmount are: " + bNullClaimChargeAmount + ", " + bNullClaimPaymentAmount);
	}
}


					      }

		      }catch(Exception ex) {
		    	  System.out.println("PaymentDataExtraction.setClaimChargePaymentAmounts() ERROR: " + ex.getMessage());
		    	  ex.printStackTrace();
		    	  LOGGER.error("Error on PaymentDataExtraction.setClaimChargePaymentAmounts(), errored on Claim Key: {}",  claimIndexId);
		          LOGGER.error(ex.getMessage(), ex);
	      }

	}	// end setClaimChargePaymentAmounts()


	private void setClaimPaymentAmount(double claimPaymentAmount2) {
		this.claimPaymentAmount = claimPaymentAmount2;

	}

	public double getClaimPaymentAmount() {

			return this.claimPaymentAmount;
	}

	/**
	 * This method returns a formatted String of the payment amount
	 * @author Bill Blackmon
	 * @return String
	 */
	public String getClaimPaymentAmountFormatted() {
		DecimalFormat df = new DecimalFormat("#.00");
		String s = "";

		// check for null amount from BAD data
		if(this.bNullClaimPaymentAmount==true) {
				s="";
				return s;
		}

		// use getter method here....
		/**
		if(   (claimPaymentAmount > 0) || (claimPaymentAmount < 0)     ) {
			s = df.format(claimPaymentAmount);
		} else {
			s="0.00";
			// s="";
		}
		**/
		if(  (this.getClaimPaymentAmount() > 0.00 ) || (this.getClaimPaymentAmount() < 0.00 )   ) {
			s = df.format(claimPaymentAmount);
		} else {
			s = "0.00";
			// s="";
		}


		return s;
	}




	private boolean getbNullClaimPaymentAmount() {
		return false;
	}

	public String getCheckNumber() {
		return checkNumber;
	}


// 37 and 38 - sets check date and check numnber
	/**
	 * This method sets the check date and check number for a given claim payment
	 * @author Bill Blackmon
	 * @param String claimIndexId
	 */
	public void setCheckDateAndNumber(String claimIndexId) {


		Query q;
	    List<?> l;
	    Iterator<?> it;
	    String stringDate = "";

	    boolean DEBUG=false;


	       try {

	    	   /** ORIGINAL
				String sql = "SELECT check_date, max(check_number) FROM fee_claim_payments f, fee_claim_payment_lines l " +
				" WHERE " +
					" f.claim_index = " + claimIndexId + " and 	F.FEE_CLAIM_PAYMENT_ID = L.FEE_CLAIM_PAYMENT_ID and " +
					" l.type_of_payment in ('CHK','EFT') and " +
					" check_date = ( " +
							" select max(check_date) from " +
							"fee_claim_payments ff, fee_claim_payment_lines ll " +
								"where " +
								" ff.claim_index = f.claim_index and " +
								" FF.FEE_CLAIM_PAYMENT_ID = LL.FEE_CLAIM_PAYMENT_ID and " +
								" LL.TYPE_OF_PAYMENT in ('CHK','EFT') ) group by check_date ";
				**/



/**
 *    CLM-FL37(check/EFT date): didnt take the LATER date (max) of treasury_payment_date. claim id 86982489
 */

				// New 7/11
				  String sql = " SELECT  max(treasury_payment_date), payment_tracking_number " +
				    " FROM " +
					  "	claim_payment cpay " +
					  " where " +
					  "	CLAIM_PAYMENT_TYPE_CODE in ('CHECK','EFT') and " +
					  "	claim_index = " + claimIndexId + " group by payment_tracking_number ";


					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();

					      while (it.hasNext()) {
					         Object[] row = (Object[]) it.next();
					         if(row != null) {
					        	 // get date then number
					        	 	Object checkDate = (java.sql.Timestamp) row[0];
					        	 	if(checkDate != null) {
							        	 SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
							        	 stringDate = formatter.format(checkDate);
							        	 this.checkDate=stringDate;
					        	 	} else {
					        	 		 this.checkDate= "";
					        	 	}

					        	 	String temp = (String) row[1];
					        	 	if(temp != null) {
					        	 		this.checkNumber=temp;
					        	 	} else {
					        	 		this.checkNumber = "";
					        	 	}
						     }
					      }


if(DEBUG) {
  System.out.println("\n");
  System.out.println("setCheckDateAndNumber() - claimIndex is: " + claimIndexId);
  System.out.println("setCheckDateAndNumber() - sql is: " + sql);
  System.out.println("setCheckDateAndNumber() checkDate is: " + checkDate);
  System.out.println("setCheckDateAndNumber() checkNumber is: " + checkNumber);
  System.out.println("\n");
}



		      }catch(Exception ex) {
		    	  System.out.println("PaymentDataExtraction.setCheckDateAndNumber() ERROR: " + ex.getMessage());
		    	  ex.printStackTrace();
		    	  LOGGER.error("Error on PaymentDataExtraction.setCheckDateAndNumber(), errored on Claim Key: {}",  claimIndexId);
		          LOGGER.error(ex.getMessage(), ex);
	      }
	}		// end setCheckDateAndNumber()

	// 37 Check/EFT Date
	/**
	 * @return String
	 */
		public String getCheckDate() {
			return checkDate;
		}



// #36 Claim Adjudication/Payment Date - 	MUST BE FORMATTED  DT Date format of CCYYMMDD
	public void setClaimAdjudicationPaymentDate(String claimIndexId) {

		/**
		 * SELECT max(treasury_payment_date) FROM claim_payment WHERE claim_index = xxx

		 */
		Query q;
	    List<?> l;
	    Iterator<?> it;
	    String stringDate = "";


/**
 * //   CLM-FL37(check/EFT date): didnt take the LATER date (max) of treasury_payment_date. claim id 86982489 ?? DIDN'T TAKE? SQL error?
 */
	      try {

	    	  String sql = "SELECT max(PAYMENT_BATCH_DATE) FROM claim_payment WHERE claim_index = " + claimIndexId;


					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();


					      while (it.hasNext()) {
					         Object row = (java.sql.Timestamp) it.next();
					         if(row != null) {
					        	 SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
					        	 stringDate = formatter.format(row);

/**
System.out.println("\n");
System.out.println("setClaimAdjudicationPaymentDate() - sql is: " + sql);
System.out.println("setClaimAdjudicationPaymentDate() PAYMENT DATE FOR ID: " + claimIndexId + " is: " + stringDate);
System.out.println("\n");
**/



					         }
					      }
	      }catch(Exception ex) {
	    	  System.out.println("PaymentDataExtraction.setClaimAdjudicationPaymentDate() ERROR: " + ex.getMessage());
	    	  LOGGER.error("Error on PaymentDataExtraction.setClaimAdjudicationPaymentDate, errored on Claim Key: {}",  claimIndexId);
	          LOGGER.error(ex.getMessage(), ex);
	      }

		this.claimAdjudicationPaymentDate = stringDate;

	}	// end setClaimAdjudicationPaymentDate()


	public String getClaimAdjudicationPaymentDate() {
		return claimAdjudicationPaymentDate;
	}






	// 42 Pharmacy Prescription Number
	public String getPharmacyPrescriptionNumber() {
		return pharmacyPrescriptionNumber;
	}


	public void setPharmacyPrescriptionNumber(String claimIndexId) {

		  Query q;
	      List<?> l;
	      Iterator<?> it;


	      try {

	    	  String sql = "SELECT prescription_number FROM claim_procedure WHERE claim_index = " + claimIndexId + " and prescription_number is not null";


					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();


					      while (it.hasNext()) {
					         Object row = it.next();
					         if(row != null) {
					        	 this.pharmacyPrescriptionNumber = row.toString();
					         }
					      }

	      }catch(Exception ex) {
	    	  System.out.println("PaymentDataExtraction.setPharmacyPrescriptionNumber() ERROR: " + ex.getMessage());
	    	  LOGGER.error("Error on PaymentDataExtraction.setPharmacyPrescriptionNumber, errored on Claim Key: {}",  claimIndexId);
	          LOGGER.error(ex.getMessage(), ex);
	      }

	}	// end setPharmacyPrescriptionNumber()






	public String getProcedureModifier1() {
		return procedureModifier1;
	}


	public void setProcedureModifier1(String procedureModifier1) {
		this.procedureModifier1 = procedureModifier1;
	}




	// #6 Billing Provider Payer Assigned Number
	public String getServiceProviderPayerAssignedNumber() {
		return serviceProviderPayerAssignedNumber;
	}


	public void setServiceProviderPayerAssignedNumber(String claimIndexId) {

		  Query q;
	      List<?> l;
	      Iterator<?> it;


	      try {

	    	  String sql = "SELECT provider_npi FROM provider_info WHERE claim_index = " + claimIndexId + " and (provider_type_cd = 'SERVICE FACILITY' or provider_type_cd = 'RENDERING PROVIDER')";


					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();


					      while (it.hasNext()) {
					         Object row = it.next();
					         if(row != null) {
					        	 this.serviceProviderPayerAssignedNumber = row.toString();
					         } else {
					        	 this.serviceProviderPayerAssignedNumber = "";
					         }
					      }

	      }catch(Exception ex) {
	    	  System.out.println("PaymentDataExtraction.setServiceProviderPayerAssignedNumber() ERROR: " + ex.getMessage());
	    	  LOGGER.error("Error on PaymentDataExtraction.setServiceProviderPayerAssignedNumber, errored on Claim Key: {}",  claimIndexId);
	          LOGGER.error(ex.getMessage(), ex);
	      }


	}	// end setServiceProviderPayerAssignedNumber()


	// #7 Billing Provider National Provider ID
	public String getServiceProviderNationalProviderID() {
		return serviceProviderNationalProviderID;
	}


	public void setServiceProviderNationalProviderID(String claimIndexId) {

		  Query q;
	      List<?> l;
	      Iterator<?> it;


	      try {

	    	  String sql = "SELECT provider_npi FROM provider_info WHERE claim_index = " + claimIndexId + " and lower(provider_type_cd) = 'billing provider'";


					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();


					      while (it.hasNext()) {
					         Object row = it.next();
					         if(row != null) {
					        	 this.billingProviderNationalProviderID = row.toString();
					         } else {
					        	 this.billingProviderNationalProviderID = null;
					         }
					      }

	      }catch(Exception ex) {
	    	  System.out.println("PaymentDataExtraction.setBillingProviderNationalProviderID() ERROR: " + ex.getMessage());
	    	  LOGGER.error("Error on PaymentDataExtraction.setBillingProviderNationalProviderID, errored on Claim Key: {}",  claimIndexId);
	          LOGGER.error(ex.getMessage(), ex);
	      }

	}	// end setServiceProviderNationalProviderID()


	public String getBillingProviderNationalProviderID() {
		return billingProviderNationalProviderID;
	}

	/**
	 * This method sets the National Billing Provider Id
	 * @author Bill Blackmon
	 * @param claimIndexId
	 */
	public void setBillingProviderNationalProviderID(String claimIndexId) {
		  Query q;
	      List<?> l;
	      Iterator<?> it;


	      try {

	    	  String sql = "SELECT provider_npi FROM provider_info WHERE claim_index = " + claimIndexId + " and lower(provider_type_cd) = 'billing provider'";


					      q = this.entityManager.createNativeQuery(sql);
					      l = q.getResultList();
					      it = l.iterator();


					      while (it.hasNext()) {
					         Object row = it.next();
					         if(row != null) {
					              this.billingProviderNationalProviderID = row.toString();
					         } else {
					        	 this.billingProviderNationalProviderID = "";
					         }
					      }

	      }catch(Exception ex) {
	    	  System.out.println("PaymentDataExtraction.setBillingProviderNationalProviderID() ERROR: " + ex.getMessage() + "on claimId: " + claimIndexId);
	    	  ex.printStackTrace();
	    	  LOGGER.error("Error on PaymentDataExtraction.setBillingProviderNationalProviderID, errored on Claim Key: {}",  claimIndexId);
	          LOGGER.error(ex.getMessage(), ex);
	      }

	}	// end setBillingProviderNationalProviderID()



	public EntityManager getEntityManager() {
		return entityManager;
	}


	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}


	public String getBillingProviderPayerAssignedNumber() {
		return this.billingProviderPayerAssignedNumber;
	}


	public void setBillingProviderFederalTaxID(String billingProviderFederalTaxID) {
		this.billingProviderFederalTaxID = billingProviderFederalTaxID;
	}

	public String getBillingProviderFederalTaxID() {
		return billingProviderFederalTaxID;
	}




	public String getBillingProviderLastName() {
		return billingProviderLastName;
	}


	public void setBillingProviderLastName(String billingProviderLastName) {
		this.billingProviderLastName = billingProviderLastName;
	}


	public String getBillingProviderFirstName() {
		return billingProviderFirstName;
	}


	public void setBillingProviderFirstName(String billingProviderFirstName) {
		this.billingProviderFirstName = billingProviderFirstName;
	}


	public String getBillingProviderMiddleName() {
		return billingProviderMiddleName;
	}


	public void setBillingProviderMiddleName(String billingProviderMiddleName) {
		this.billingProviderMiddleName = billingProviderMiddleName;
	}


	public String getBillingProviderNameSuffix() {
		return billingProviderNameSuffix;
	}


	public void setBillingProviderNameSuffix(String billingProviderNameSuffix) {
		this.billingProviderNameSuffix = billingProviderNameSuffix;
	}


	public String getServiceProviderFederalTaxID() {
		return serviceProviderFederalTaxID;
	}


	public void setServiceProviderFederalTaxID(String serviceProviderFederalTaxID) {
		this.serviceProviderFederalTaxID = serviceProviderFederalTaxID;
	}



	public String getServiceProviderLastName() {
		return serviceProviderLastName;
	}


	public void setServiceProviderLastName(String serviceProviderLastName) {
		this.serviceProviderLastName = serviceProviderLastName;
	}


	public String getServiceProviderFirstName() {
		return serviceProviderFirstName;
	}


	public void setServiceProviderFirstName(String serviceProviderFirstName) {
		this.serviceProviderFirstName = serviceProviderFirstName;
	}




	public void setServiceProviderMiddleName(String serviceProviderMiddleName) {
		this.serviceProviderMiddleName = serviceProviderMiddleName;
	}


	public String getServiceProviderMiddleName() {
		return serviceProviderMiddleName;
	}

	public String getServiceProviderNameSuffix() {
		return serviceProviderNameSuffix;
	}


	public void setServiceProviderNameSuffix(String serviceProviderNameSuffix) {
		this.serviceProviderNameSuffix = serviceProviderNameSuffix;
	}


	public String getEmployerIdentificationNumber() {
		return employerIdentificationNumber;
	}


	public void setEmployerIdentificationNumber(String employerIdentificationNumber) {
		this.employerIdentificationNumber = employerIdentificationNumber;
	}


	public String getEmployerName() {
		return employerName;
	}


	public void setEmployerName(String employerName) {
		this.employerName = employerName;
	}


	public String getSubscriberID() {
		return subscriberID;
	}


	public void setSubscriberID(String subscriberID) {
		this.subscriberID = subscriberID;
	}


	public String getSubscriberLastName() {
		return subscriberLastName;
	}


	public void setSubscriberLastName(String subscriberLastName) {
		this.subscriberLastName = subscriberLastName;
	}


	public String getSubscriberFirstName() {
		return subscriberFirstName;
	}


	public void setSubscriberFirstName(String subscriberFirstName) {
		this.subscriberFirstName = subscriberFirstName;
	}


	public String getSubscriberMiddleName() {
		return subscriberMiddleName;
	}


	public void setSubscriberMiddleName(String subscriberMiddleName) {
		this.subscriberMiddleName = subscriberMiddleName;
	}


	public String getSubscriberNameSuffix(String claimIndex) {
		return subscriberNameSuffix;
	}


	public void setSubscriberNameSuffix(String subscriberNameSuffix) {
		this.subscriberNameSuffix = subscriberNameSuffix;
	}


	public String getPatientID() {
		return patientID;
	}


	public void setPatientID(String patientID) {
		this.patientID = patientID;
	}


	public String getPatientLastName() {
		return patientLastName;
	}


	public void setPatientLastName(String patientLastName) {
		this.patientLastName = patientLastName;
	}


	public String getPatientFirstName() {
		return patientFirstName;
	}


	public void setPatientFirstName(String patientFirstName) {
		this.patientFirstName = patientFirstName;
	}


	public String getPatientMiddleName() {
		return patientMiddleName;
	}


	public void setPatientMiddleName(String patientMiddleName) {
		this.patientMiddleName = patientMiddleName;
	}


	public String getPatientNameSuffix() {
		return patientNameSuffix;
	}


	public void setPatientNameSuffix(String patientNameSuffix) {
		this.patientNameSuffix = patientNameSuffix;
	}


	public String getPatientDateOfBirth() {
		return patientDateOfBirth;
	}


	public void setPatientDateOfBirth(String patientDateOfBirth) {
		this.patientDateOfBirth = patientDateOfBirth;
	}


	public String getPatientGender() {
		return patientGender;
	}


	public void setPatientGender(String patientGender) {
		this.patientGender = patientGender;
	}


	public double getClaimChargeAmount() {
		return claimChargeAmount;
	}

	public String getClaimChargeAmountFormatted() {
		DecimalFormat df = new DecimalFormat("#.00");
		String s = "";

		// check for null amount from BAD data
		if(this.bNullClaimChargeAmount==true) {
				s="";
				return s;
		}

		if(  (this.getClaimChargeAmount() > 0.00 ) || (this.getClaimChargeAmount() < 0.00 )   ) {
			s = df.format(claimChargeAmount);
		} else {
			s = "0.00";
			// s="";
		}



		return s;
	}



	public void setClaimChargeAmount(double claimChargeAmount) {
		this.claimChargeAmount = claimChargeAmount;
	}




	public String getBillType() {
		return billType;
	}


	public void setBillType(String billType) {
		this.billType = billType;
	}


	public String getPayerClaimIdentificationNumber() {
		return payerClaimIdentificationNumber;
	}


	public void setPayerClaimIdentificationNumber(
			String payerClaimIdentificationNumber) {
		this.payerClaimIdentificationNumber = payerClaimIdentificationNumber;
	}


	public String getPatientAccountNumber() {
		return patientAccountNumber;
	}


	public void setPatientAccountNumber(String patientAccountNumber) {
		this.patientAccountNumber = patientAccountNumber;
	}





	public String getVoucherIdentifier() {
		return voucherIdentifier;
	}


	public void setVoucherIdentifier(String voucherIdentifier) {
		this.voucherIdentifier = voucherIdentifier;
	}


	public String getApplicationSystemIdentifier() {
		return applicationSystemIdentifier;
	}


	public void setApplicationSystemIdentifier(String applicationSystemIdentifier) {
		this.applicationSystemIdentifier = applicationSystemIdentifier;
	}


	public String getGroupNumber() {
		return groupNumber;
	}


	public void setGroupNumber(String groupNumber) {
		this.groupNumber = groupNumber;
	}


	public Date getClaimServiceDateStart() {
		return claimServiceDateStart;
	}


	public void setClaimServiceDateStart(Date claimServiceDateStart) {
		this.claimServiceDateStart = claimServiceDateStart;
	}


	public Date getClaimServiceDateEnd() {
		return claimServiceDateEnd;
	}


	public void setClaimServiceDateEnd(Date claimServiceDateEnd) {
		this.claimServiceDateEnd = claimServiceDateEnd;
	}


	public String getPayerID() {
		return payerID;
	}


	public void setPayerID(String payerID) {
		this.payerID = payerID;
	}


	public String getLineItemControlNumber() {
		return lineItemControlNumber;
	}


	public void setLineItemControlNumber(String lineItemControlNumber) {
		this.lineItemControlNumber = lineItemControlNumber;
	}


	public String getServiceQualifierID() {
		return serviceQualifierID;
	}




	public String getServiceIdentificationCode() {
		return serviceIdentificationCode;
	}


	public void setServiceIdentificationCode(String serviceIdentificationCode) {
		this.serviceIdentificationCode = serviceIdentificationCode;
	}

	public String getProcedureModifier2() {
		return procedureModifier2;
	}


	public void setProcedureModifier2(String procedureModifier2) {
		this.procedureModifier2 = procedureModifier2;
	}


	public String getProcedureModifier3() {
		return procedureModifier3;
	}


	public void setProcedureModifier3(String procedureModifier3) {
		this.procedureModifier3 = procedureModifier3;
	}


	public String getProcedureModifier4() {
		return procedureModifier4;
	}


	public void setProcedureModifier4(String procedureModifier4) {
		this.procedureModifier4 = procedureModifier4;
	}


	public BigDecimal getLineItemChargeAmount() {
		return lineItemChargeAmount;
	}


	public void setLineItemChargeAmount(BigDecimal lineItemChargeAmount) {
		this.lineItemChargeAmount = lineItemChargeAmount;
	}


	public BigDecimal getLineItemProviderPaymentAmount() {
		return lineItemProviderPaymentAmount;
	}


	public void setLineItemProviderPaymentAmount(
			BigDecimal lineItemProviderPaymentAmount) {
		this.lineItemProviderPaymentAmount = lineItemProviderPaymentAmount;
	}


	public String getRevenueCode() {
		return revenueCode;
	}


	public void setRevenueCode(String revenueCode) {
		this.revenueCode = revenueCode;
	}


	public int getUnitsOfService() {
		return unitsOfService;
	}


	public void setUnitsOfService(int unitsOfService) {
		this.unitsOfService = unitsOfService;
	}


	public Date getDateOfServiceStart() {
		return dateOfServiceStart;
	}


	public void setDateOfServiceStart(Date dateOfServiceStart) {
		this.dateOfServiceStart = dateOfServiceStart;
	}


	public Date getDateOfServiceEnd() {
		return dateOfServiceEnd;
	}


	public void setDateOfServiceEnd(Date dateOfServiceEnd) {
		this.dateOfServiceEnd = dateOfServiceEnd;
	}


	public void setBillingProviderPayerAssignedNumber(String billingProviderPayerAssignedNumber) {
		this.billingProviderPayerAssignedNumber = billingProviderPayerAssignedNumber;
	}


	public String getProcCode() {
		return procCode;
	}


	public void setProcCode(String procCode) {
		this.procCode = procCode;
	}




	public String getSubscriberNameSuffix() {
		return subscriberNameSuffix;
	}


	public String getEmdeonClaimNumber() {
		return emdeonClaimNumber;
	}


	@SuppressWarnings("rawtypes")
	public ArrayList getModifiers() {
		return modifiers;
	}

	public void setModifiers(@SuppressWarnings("rawtypes") ArrayList modifiers) {
		this.modifiers = modifiers;
	}

	public BigDecimal getBilledUnits() {
		return billedUnits;
	}

	public void setBilledUnits(BigDecimal billedUnits) {
		this.billedUnits = billedUnits;
	}

	public Timestamp getServiceFromDate() {
		return serviceFromDate;
	}

	public void setServiceFromDate(Timestamp serviceFromDate) {
		this.serviceFromDate = serviceFromDate;
	}

	public Timestamp getServiceToDate() {
		return serviceToDate;
	}

	public void setServiceToDate(Timestamp serviceToDate) {
		this.serviceToDate = serviceToDate;
	}

	public String getProcCodeDesc() {
		return procCodeDesc;
	}

	public void setProcCodeDesc(String procCodeDesc) {
		this.procCodeDesc = procCodeDesc;
	}

	public BigDecimal getBilledAmt() {
		return billedAmt;
	}

	public void setBilledAmt(BigDecimal billedAmt) {
		this.billedAmt = billedAmt;
	}

	public BigDecimal getPaidAmt() {
		return paidAmt;
	}

	public void setPaidAmt(BigDecimal paidAmt) {
		this.paidAmt = paidAmt;
	}

	public BigDecimal getClaimProcedureSeq() {
		return claimProcedureSeq;
	}

	public void setClaimProcedureSeq(BigDecimal claimProcedureSeq) {
		this.claimProcedureSeq = claimProcedureSeq;
	}

	public boolean isbNegativePaidAmount() {
		return bNegativePaidAmount;
	}

	public void setbNegativePaidAmount(boolean bNegativePaidAmount) {
		this.bNegativePaidAmount = bNegativePaidAmount;
	}


	/**
	 * @author Bill Blackmon
	 * Set all class-level variables to initial state
	 */
	@SuppressWarnings("rawtypes")
	public void clearAll() {

		this.billingProviderNationalProviderID="";
		this.billingProviderLastName="";
		this.billingProviderFirstName="";
		this.billingProviderMiddleName="";
		this.billingProviderNameSuffix="";

		this.billType="";

		this.serviceProviderFederalTaxID="";
		this.serviceProviderPayerAssignedNumber="";
		this.serviceProviderNationalProviderID="";
		this.serviceProviderLastName="";
		this.serviceProviderFirstName="";
		this.serviceProviderMiddleName="";
		this.serviceProviderNameSuffix="";

		this.serviceTypeCode="";

		this.claimChargeAmount = 0.00;;
		this.claimPaymentAmount = 0.00;

		this.paidAmt=null;
		this.billedAmt=null;

// System.out.println("clearAll() - claimCharge amt is now: " + this.getClaimChargeAmount());
// System.out.println("clearAll() - claimPayment amt is now: " + this.getClaimPaymentAmount())	;


		this.claimAdjudicationPaymentDate="";

		this.checkDate="";
		this.checkNumber="";


		this.payerClaimIdentificationNumber="";
		this.patientAccountNumber="";
		this.pharmacyPrescriptionNumber="";
		this.voucherIdentifier="";
		this.applicationSystemIdentifier="";
		this.groupNumber="";

		this.claimServiceDateStart = null;
		this.claimServiceDateEnd = null;


		this.payerID="";
		this.lineItemControlNumber="";
		this.serviceQualifierID="";
		this.serviceIdentificationCode="";

		this.procedureModifier1="";
		this.procedureModifier2="";
		this.procedureModifier3="";
		this.procedureModifier4="";

		this.lineItemChargeAmount=null;
		this.lineItemProviderPaymentAmount=null;
		this.revenueCode="";
		this.unitsOfService=0;
		this.emdeonClaimNumber="";
		this.dateOfServiceStart=null;
		this.dateOfServiceEnd=null;


		this.procCode="";

		this.modifiers=new ArrayList();

		this.bNegativePaidAmount=false;
		this.bNegativeChargeAmount=false;
		this.bNullClaimPaymentAmount=false;
		this.bNullClaimChargeAmount=false;
	}


	public void getNow() {
		 Date dNow = new Date( );
	      SimpleDateFormat ft =
	      new SimpleDateFormat ("hh:mm:ss:ms");

	      System.out.println(ft.format(dNow));
	}



}	// end of class
