package gov.va.med.hac.edi.extract.feeclaimstatus.pending;

import gov.va.med.domain.erepos.Claim;
import gov.va.med.domain.erepos.DemographicInfo;
import gov.va.med.domain.erepos.EdiProcessLog;
import gov.va.med.domain.erepos.EntityAdtlId;
import gov.va.med.domain.erepos.EntityName;
import gov.va.med.domain.fppsowner.PersonInfo;
import gov.va.med.hac.edi.utils.DateUtil;
import gov.va.med.hac.edi.utils.StringUtils;

import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;

import java.math.BigDecimal;

import java.text.SimpleDateFormat;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.PersistenceException;
import javax.persistence.Query;


class ProcessEmdeonPendingClaimStatus {
    static final Logger LOGGER = LoggerFactory.getLogger(ProcessEmdeonPendingClaimStatus.class);
    private static final String PAYER_ID = "VAFEE";
    private static final String TRADING_PARTNER_ID = "VAFNH";

    //TWP I changed these types below based on the guides I was using is there anything more up to date
    //The guides were published in 2006
    private static final String PROFESSIONAL_CLAIM_TYPE = "005010X222";
    private static final String INSTITUTIONAL_CLAIM_TYPE = "005010X223";
    private static final String DENTAL_CLAIM_TYPE = "005010X224";
    private static final String INSTITUTIONAL_CLAIM_TYPE_4010 = "004010X096A1";
    private static final String PROFESSIONAL_CLAIM_TYPE_4010 = "004010X098A1";
    private static final String DENTAL_CLAIM_TYPE_4010 = "004010X097A1";
    static final String CRLF = "\r\n";
    static final String DELIMITER = "|";
    static final boolean REQUIRED = true;
    static final boolean OPTIONAL = false;
    EntityManager eReposEM;
    boolean dailyCstat;
    String claimKey = "";
    String fileNbr = "";
    String st02CtlNbr = "";
    String billPrvdId = "";
    String sbcrId = "";
    String claimType = "";
    String statusCd = "";
    String currentDate = "";
    String currentTime = "";
    Date beginDate;
    Date endDate;
    int cnt = 1;
    int rejCnt = 0;
    int accCnt = 0;

    /*
    * Called during daily CSTAT file creation
    */
    final void process(String clearinghouseId, String filePath,
        EntityManager entityManager, EntityManager fppsEM,
        EntityManager programTrackingEM) {
        LOGGER.debug("processing with clearinghouseId: {}  filePath: {} ",
            clearinghouseId, filePath);
        this.dailyCstat = true;
        this.eReposEM = entityManager;
        processClaimStatus(clearinghouseId, filePath, fppsEM, programTrackingEM);
    }

    /*
     * Called during monthly CSTAT file creation
     * NOTE: beginDate and end endDate
     */
    final void process(String clearinghouseId, String filePath, Date beginDate,
        Date endDate, EntityManager entityManager, EntityManager fppsEM,
        EntityManager programTrackingEM) {
        LOGGER.debug("processing with clearinghouseId: {}  filePath: {} ",
            clearinghouseId, filePath);
        this.beginDate = beginDate;
        this.endDate = endDate;
        this.dailyCstat = false;
        this.eReposEM = entityManager;
        processClaimStatus(clearinghouseId, filePath, fppsEM, programTrackingEM);
    }

    /**
     * Gets the claim keys for claims that have recently been picked up by field stations
     * and marks them so that another process will not pick them up.
     *
     * Background.
     * The program tracking database is used to track claims as they are processed in the VA system.
     * It tracks when claims move from one system to another.
     * As a claim progresses through its life cycle the record status field tracks where the claim is in the lifecycle.
     * When a claim moves from eTrans to FPPS record status is updated to 8 - awaiting move to sql
     * When the claim is moved into PROGRAM_TRACKING from FPPS the status is updated to 1 - awaiting send to field
     * When a station picks up a claim the status is updated to 2 - sent to field
     *
     * Additionally, as claims move through the system different keys are used to track the claims in different systems.
     * When claims are initially loaded into the system via the eTrans repository, they are assigned a unique GUID known
     * as the claim key.
     * When claims are moved from eTrans to the FPPS schema, the claim are assigned another GUID known as the claim index.
     * Program tracker uses the FPPS claim index to track where the claim is in the system
     * (i.e. claim tracker begins tracking claims from the point they enter FPPS)
     * Therefore, although we get the claim index from tracker, we need the claim key in order to generate the claim status file
     * (claim status data come from eTrans repository not fpps). So, When you look at the query below you
     * will notice we join across two different schema to preform this lookup.
     * (In the essence of time we chose to do a cross schema join but in the future it may be desirable to
     * modify the program tracker load process to include more data so that a cross schema join is not required)
     *
     * @param - the name of the claim status file being generated
     * @param senderId - the sender that the claim status file is being generated for
     * @param programTrackingEM - entity manager assoicated with the program tracking database
     * @return
     */
    final List<String> getClaimKeysAndLock(String filename, String senderId,
        EntityManager programTrackingEM) {
        programTrackingEM.getTransaction().begin();

        List<Object[]> claimKeyClaimIndexList = getClaimKeyToClaimIndexList(senderId,
                programTrackingEM);

        if ((claimKeyClaimIndexList == null) ||
                claimKeyClaimIndexList.isEmpty()) {
            return null;
        }

        LOGGER.info("Found " + claimKeyClaimIndexList.size() +
            " claims to include in the pending claim status file. ");

        List claimKeys = new ArrayList();
        List claimIndexes = new ArrayList();

        for (Object[] keyIndexPair : claimKeyClaimIndexList) {
            claimKeys.add(keyIndexPair[0]);
            claimIndexes.add(keyIndexPair[1]);
        }

        String pendingStatusKey = createPendingStatusFileGenerationStartedRecord(filename,
                programTrackingEM);
        associateClaimsWithThisPendingStatusFile(programTrackingEM,
            claimIndexes, pendingStatusKey);
        programTrackingEM.getTransaction().commit();
        LOGGER.debug(
            "Set PENDING_STATUS_TRACKING GENERATION_STATUS to 0 to indicate that a pending status file is being generated");
        LOGGER.debug(
            "and made associations in the db between this file and the claims that will be included in it.");

        return claimKeys;
    }

    /**
     * This method is used when generating the quarterly status file.
     * When it is passed a date range it will generate a list of claim keys
     * that represents all the claims put in a status file over that date range.
     *
     * @param senderId - the sender to generate the pending claim status file for
     * @param beginDate - the beginning of the range
     * @param endDate - the end of the range
     * @param programTrackingEM
     * @return
     */
    final List<String> getClaimIndexesForTimePeriod(String senderId,
        Date beginDate, Date endDate, EntityManager programTrackingEM) {
        String getClaimKeysStr = "select edi_claim_key from " +
            " FPPS_OWNER.claims where TRADING_PARTNER_ID_CD=:senderId AND claim_index in " +
            "(select CLAIM_INDEX from PROGRAM_TRACKING.VA_IMPLEMENTATION_TRACKING " +
            "where PENDING_STATUS_TRACKING_KEY IN " +
            "(select PENDING_STATUS_TRACKING_KEY " +
            "from PROGRAM_TRACKING.pending_status_tracking where generation_status= 2" +
            "and DATE_STATUS_UPDATED between :startDate AND :endDate ))";

        /*String getClaimKeysStr = "SELECT * FROM CLAIM WHERE CLAIM_KEY IN(select edi_claim_key from " +
           " FPPS_OWNER.claims where TRADING_PARTNER_ID_CD=:senderId AND claim_index in " +
        "(select CLAIM_INDEX from PROGRAM_TRACKING.VA_IMPLEMENTATION_TRACKING "+
                                                 "where PENDING_STATUS_TRACKING_KEY IN " +
                                                 "(select PENDING_STATUS_TRACKING_KEY " +
                                                 "from PROGRAM_TRACKING.pending_status_tracking where generation_status= 2" +
                                                 "and DATE_STATUS_UPDATED between :startDate AND :endDate )))";*/
        Query getClaimKeyClaimIndexQuery = programTrackingEM.createNativeQuery(getClaimKeysStr);
        getClaimKeyClaimIndexQuery.setParameter("senderId", senderId);
        getClaimKeyClaimIndexQuery.setParameter("startDate", beginDate);
        getClaimKeyClaimIndexQuery.setParameter("endDate", endDate);

        return getClaimKeyClaimIndexQuery.getResultList();
    }

    /**
     * Since a dailyClaim status file is generated using multiple schemas - we mark the file that
     * the claim status for a particular claim is supposed to appear in by setting the claim's
     * PENDING_STATUS_TRACKING_KEY which basically associates the two.  By looking in the PENDING_STATUS_TRACKING
     * table you can find out if the file is in the process of generating (from another schema) or
     * if it has been generated (see GENERATION_STATUS column)
     * @param programTrackingEM
     * @param claimIndexes
     * @param pendingStatusKey
     */
    private void associateClaimsWithThisPendingStatusFile(
        EntityManager programTrackingEM, List claimIndexes,
        String pendingStatusKey) {
        //This next section sets the PENDING_STATUS_TRACKING key of the VA_IMPLEMENTATION_TRACKING records 
        //to indicate that a particular claim is about to be included in the claim status file we are currently generating	   
        //In order to do this we need to deal with the "in clause" limit imposed by Oracle - it only allows 1000 elements in an in clause 	   
        int IN_CLAUSE_BLOCK_SIZE = 500;

        for (int startIndex = 0; startIndex < claimIndexes.size();
                startIndex = startIndex + IN_CLAUSE_BLOCK_SIZE) {
            int endIndex = startIndex + IN_CLAUSE_BLOCK_SIZE;

            if (endIndex > claimIndexes.size()) {
                endIndex = claimIndexes.size();
            }

            //Note fromIndex is inclusive, to index is exclusive
            List listThatWontBreakInClause = claimIndexes.subList(startIndex,
                    endIndex);

            //Note although PROGRAM_TRACKING_VA does not have a primary key or constraints
            //The cliam index is belived to be a PK - if this assumption is wrong it could create issues
            String updateVaImplTracking = "update VA_IMPLEMENTATION_TRACKING set PENDING_STATUS_TRACKING_KEY=:pendingStatusTrackingKey " +
                "where CLAIM_INDEX in (:claimIndexes) ";
            Query queryVaImplTracking = programTrackingEM.createNativeQuery(updateVaImplTracking);
            queryVaImplTracking.setParameter("pendingStatusTrackingKey",
                pendingStatusKey);
            queryVaImplTracking.setParameter("claimIndexes",
                listThatWontBreakInClause);

            int rowsVaImplTrackingUpdated = queryVaImplTracking.executeUpdate();
        }
    }

    /**
     * This method adds a record into PENDING_STATUS_TRACKING to indicate that a
     * new claim status file is being generated. Since the file is generated
     * from a different schema (using 2 different db connections) we track the
     * progress of the claim status file generation here. The status will be
     * updated to finished when the file generation completes.
     *
     * @param filename
     * @param programTrackingEM
     * @return
     */
    private String createPendingStatusFileGenerationStartedRecord(
        String filename, EntityManager programTrackingEM) {
        //The PENDING_STATUS_TRACKING is a table that contains information about each pending status file that is generated
        //At this point, a record is inserted into PENDING_STATUS_TRACKING to indicates 
        //that a claim status file is about to be generated
        //The status at this point is set to 1 - to indicate file generation has started
        //when the file generation is complete the status will be updated to 2
        //Below this code we will add a key to all the claims in the VA_IMPLEMENTATION_TRACKING table that will be included in
        //the claim status file we are about to generate so it is possible to identify the claims associated with the file
        String pendingStatusKey = UUID.randomUUID().toString().toUpperCase();
        String updatePendingStatusStr = "insert into PENDING_STATUS_TRACKING (PENDING_STATUS_TRACKING_KEY, FILE_NAME, GENERATION_STATUS, DATE_STATUS_UPDATED) VALUES (:pendingStatusKey,:filename,:startedStatus,:dateProcessed)";
        Query updatePendingStatusQuery = programTrackingEM.createNativeQuery(updatePendingStatusStr);
        updatePendingStatusQuery.setParameter("pendingStatusKey",
            pendingStatusKey);
        updatePendingStatusQuery.setParameter("filename", filename);
        updatePendingStatusQuery.setParameter("startedStatus", 1);

        String ORACLE_DATE_FORMAT = "dd-MMM-yyyy";
        SimpleDateFormat oracleDateFormatter = new SimpleDateFormat(ORACLE_DATE_FORMAT);
        updatePendingStatusQuery.setParameter("dateProcessed",
            oracleDateFormatter.format(new GregorianCalendar().getTime()));

        int rowsPendingStatusUpdated = updatePendingStatusQuery.executeUpdate();

        return pendingStatusKey;
    }

    /**
     * updates the records in PENDING_STATUS_TRACKING that indicates that a
     * particular claim status file has finished generating successfully.
     *
     * @param filename
     * @param programTrackingEM
     * @return
     */
    private int updatePendingStatusFileGenerationFinishedRecord(
        String filename, EntityManager programTrackingEM) {
        programTrackingEM.getTransaction().begin();

        String updatePendingStatusStr = "update PENDING_STATUS_TRACKING set GENERATION_STATUS = :finishedStatus where FILE_NAME = :filename";
        Query updatePendingStatusQuery = programTrackingEM.createNativeQuery(updatePendingStatusStr);
        updatePendingStatusQuery.setParameter("finishedStatus", 2);
        updatePendingStatusQuery.setParameter("filename", filename);

        int rowsPendingStatusUpdated = updatePendingStatusQuery.executeUpdate();
        programTrackingEM.getTransaction().commit();

        return rowsPendingStatusUpdated;
    }

    /**
     * Returns a list of claim key / claim index pairs of all the claims that have been recently picked
     * up by FBCS stations but the status of these claims has not been including in a claim status file (pending).
     * @param senderId
     * @param programTrackingEM
     * @return
     */
    private List<Object[]> getClaimKeyToClaimIndexList(String senderId,
        EntityManager programTrackingEM) {
        //This query should get the indexes of all claims that have been picked up by the field but have
        //not been put in a claim status file
        String getClaimKeysStr = " select fpps_claims.edi_claim_key, fpps_claims.claim_index from " +
            " FPPS_OWNER.claims fpps_claims, PROGRAM_TRACKING.va_implementation_tracking program_tracking_va " +
            " where " + " ( " +
            " (PROGRAM_TRACKING_VA.PENDING_STATUS_TRACKING_KEY IS NULL) " +
            "	and ( PROGRAM_TRACKING_VA.CLAIM_INDEX = FPPS_CLAIMS.CLAIM_INDEX ) " +
            " 	and ( FPPS_CLAIMS.TRADING_PARTNER_ID_CD = :senderId) " +
            " 	and ( PROGRAM_TRACKING_VA.RECORD_STATUS >= 2) " +
            "	and ( PROGRAM_TRACKING_VA.RECORD_STATUS < 8) " + ") ";

        Query getClaimKeyClaimIndexQuery = programTrackingEM.createNativeQuery(getClaimKeysStr);
        getClaimKeyClaimIndexQuery.setParameter("senderId", senderId);

        List<Object[]> claimKeyClaimIndexList = getClaimKeyClaimIndexQuery.getResultList();

        if (claimKeyClaimIndexList.isEmpty()) {
            LOGGER.info(
                "There are no new Claims that are pending (Pending = station picked up claim from FBCS).");

            return null;
        }

        return claimKeyClaimIndexList;
    }

    //TWP: Future Enhancement Write recovery code.
    public static void resetFailedLoads() {
        //You may recover by deleting the forgien keys in VA_IMPLEMENTATION_TRACKING
        //that point to a file where file generation was started more than 2 days ago 
        //but is still stuck in GENERATION_STATUS =1
        //After the references to the file are deleted, 
        //delete the record related to that file in the PENDING_STATUS_TRACKING 
        //table - that way the claim status will be generated during the next run	   
    }

    final void processClaimStatus(String chId, String filePath,
        EntityManager fppsEM, EntityManager programTrackingEM) {
        EntityTransaction tx = null;

        try {
            // DEF005231 - The counter was being reset when transitioning
            // from 12115 to 12116. This caused the record counts in one or
            // the other to be incorrect.
            this.cnt = 1;

            Calendar cal = new GregorianCalendar();
            java.util.Date date = cal.getTime();
            SimpleDateFormat transDate = new SimpleDateFormat("yyyyMMdd");
            SimpleDateFormat transTime = new SimpleDateFormat("HHmmss");
            this.currentDate = transDate.format(date);
            this.currentTime = transTime.format(date);

            /* write out the CSTAT FILE */
            // TWP Changed Naming Convention for 5010 may require updates to
            // scheduler etc.
            String fileName = this.currentDate + this.currentTime + "_" +
                PAYER_ID + ".cstat";

            List<String> claimKeys = null;

            if (this.dailyCstat) {
                // This code was updated from the ack status file to accommodate
                // the pending status file
                // To reuse existing code an oracle temp table is used to enable
                // the use of
                // the scrollable result set employed in subsequent sections of
                // code
                claimKeys = getClaimKeysAndLock(fileName, chId,
                        programTrackingEM);
            } else {
                claimKeys = getClaimIndexesForTimePeriod(chId, beginDate,
                        endDate, programTrackingEM);
            }

            // createOracleTempTableWClaimKeys(eReposEM, claimKeys);
            if ((claimKeys != null) && (claimKeys.size() > 0)) {
                insertClaimsIntoTempTable(eReposEM, claimKeys);
                tx = eReposEM.getTransaction();
                tx.begin();

                Session session = (Session) eReposEM.getDelegate();
                String qstr = "select * from CLAIM c, TEMP_CLAIM_KEYS_CLAIMSTAT tc where c.CLAIM_KEY = tc.CLAIM_KEY ";
                org.hibernate.Query claimQuery = session.createSQLQuery(qstr)
                                                        .addEntity(Claim.class);
                ScrollableResults claims = claimQuery.scroll(ScrollMode.FORWARD_ONLY);

                // removeOracleTempTableWClaimKeys(eReposEM);
                if (claims != null) {
                    File csFile = new File(filePath + fileName);
                    PrintWriter vCsFile = new PrintWriter(new FileOutputStream(
                                csFile));
                    createHdrRec(vCsFile);
                    createPrimaryDetailRec(claims, session, vCsFile);
                    createTrlRec(vCsFile);
                    vCsFile.flush();
                    vCsFile.close();

                    if (this.dailyCstat) {
                        // Need to mark the file created in the PROGRAM_TRACKING DB
                        // HERE
                        // Note there are no changes to the status in eRepos for the
                        // pending file
                        updatePendingStatusFileGenerationFinishedRecord(fileName,
                            programTrackingEM);
                    }

                    tx.commit();
                    LOGGER.info("Claim Status File is Successfully Created!!!");
                } else {
                    LOGGER.info("There were no claims to process!!!");
                }
            } else {
                LOGGER.info(
                    "There were no new claims to put in a claim status file.");
            }
        } catch (Exception ex) {
            LOGGER.error("Unable to create CSTAT, errored on Claim Key: {}",
                this.claimKey);
            LOGGER.error(ex.getMessage(), ex);

            if ((tx != null) && tx.isActive()) {
                tx.rollback();
            }
        }
    }

    //   private void createOracleTempTableWClaimKeys(EntityManager eReposEM,
    //		   List<String> claimKeys) {
    //	   String createTempTableSQL = "CREATE GLOBAL TEMPORARY TABLE TEMP_CLAIM_KEYS_CLAIMSTAT ( CLAIM_KEY VARCHAR2(36) ) ON COMMIT DELETE ROWS";
    //	   eReposEM.createNativeQuery(createTempTableSQL).executeUpdate();
    //	   insertClaimsIntoTempTable(eReposEM, claimKeys);
    //   }
    private void insertClaimsIntoTempTable(EntityManager eReposEM,
        List<String> claimKeys) {
        int COMMIT_SIZE = 200;

        if (claimKeys != null) {
            eReposEM.getTransaction().begin();
            eReposEM.createNativeQuery(
                "TRUNCATE TABLE TEMP_CLAIM_KEYS_CLAIMSTAT").executeUpdate();
            eReposEM.getTransaction().commit();

            int startIndex = 0;
            int endIndex = claimKeys.size();

            while (startIndex < claimKeys.size()) {
                if (claimKeys.size() > (startIndex + COMMIT_SIZE)) {
                    endIndex = startIndex + COMMIT_SIZE;
                } else {
                    endIndex = claimKeys.size();
                }

                eReposEM.getTransaction().begin();

                List<String> claimKeyBatch = claimKeys.subList(startIndex,
                        endIndex);

                //Preform intermediate commits
                String insertKeysSQL = "INSERT ALL ";

                for (String key : claimKeyBatch) {
                    insertKeysSQL += (" INTO TEMP_CLAIM_KEYS_CLAIMSTAT (CLAIM_KEY) VALUES ('" +
                    key + "') ");
                }

                insertKeysSQL += "SELECT * FROM dual";
                eReposEM.createNativeQuery(insertKeysSQL).executeUpdate();
                eReposEM.getTransaction().commit();
                startIndex = endIndex;
            }

            //TWP: Newly added check
            java.math.BigDecimal countOfRecords = (java.math.BigDecimal) eReposEM.createNativeQuery(
                    "select count(1) from TEMP_CLAIM_KEYS_CLAIMSTAT")
                                                                                 .getSingleResult();
            LOGGER.debug("Temp table has " + countOfRecords +
                " entries should equal # of claim Keys " + claimKeys.size());
        }
    }

    //   
    //   private void removeOracleTempTableWClaimKeys(EntityManager eReposEM) {
    //	   String createTempTableSQL = "drop TABLE TEMP_CLAIM_KEYS_CLAIMSTAT";
    //	   eReposEM.createNativeQuery(createTempTableSQL).executeUpdate();
    //   }
    private StringBuffer append(StringBuffer sb, String value) {
        sb.append(value);
        sb.append(DELIMITER);

        return sb;
    }

    final void createHdrRec(PrintWriter vCsFile) {
        StringBuffer sbHdrRec = new StringBuffer();
        sbHdrRec.append("HDR"); //Record ID  - TWP: Changed to 3 char for 5010
        delimit(sbHdrRec);
        sbHdrRec.append(this.currentDate + this.currentTime); //File Group ID TWP: No one really knows what this is
        delimit(sbHdrRec);
        sbHdrRec.append("01"); //File Group Sequence Number - I guess there is only 1 file per group?
        delimit(sbHdrRec);
        sbHdrRec.append("01"); //File Group Count - looks like we are not using the whole file group concept
        delimit(sbHdrRec);
        sbHdrRec.append(this.currentDate); //Creation Date 8 chars yyyyMMdd
        delimit(sbHdrRec);
        sbHdrRec.append(this.currentTime); //Creation Time 6 chars HHmmss
        delimit(sbHdrRec);
        sbHdrRec.append(TRADING_PARTNER_ID); //Trading Partner Id  TWP: Changed to pad to 10 from 5
        delimit(sbHdrRec);
        sbHdrRec.append("VA Fee Basis Program"); //Submitter Name  - TWP: is this correct?
        delimit(sbHdrRec);
        sbHdrRec.append("Contact Local VAMC"); //Payer Contact Name
        delimit(sbHdrRec);
        sbHdrRec.append(""); //Payer Support Telephone Number
        delimit(sbHdrRec);
        sbHdrRec.append(""); //PayerSupport Email
        delimit(sbHdrRec);

        if (this.dailyCstat) { //Load type - full or incremental
            sbHdrRec.append("I");
        } else {
            sbHdrRec.append("F");
        }

        delimit(sbHdrRec);
        sbHdrRec.append(StringUtils.rPadSpaces(this.currentDate +
                this.currentTime + "_VAFEE", 20)); //Payer Uniqie File Identifier
        delimit(sbHdrRec);
        sbHdrRec.append("Cstat"); //File Type - TWP updated for 5010 but spec does not make sense max len = 4 but value must = Cstat which is 5 long?
        delimit(sbHdrRec);
        sbHdrRec.append("03"); //Version Code TWP updated from 02 to 03
        delimit(sbHdrRec);
        sbHdrRec.append("00"); //Release Code
                               //      delimit(sbHdrRec);

        sbHdrRec.append(CRLF); //Record Teminator new for 5010
        vCsFile.print(sbHdrRec); //TWP Changed to print rather than println since I am explicitly adding CRLF  
        vCsFile.flush();
        sbHdrRec = null;
    }

    final void createPrimaryDetailRec(ScrollableResults claims,
        Session session, PrintWriter vCsFile) {
        int count = 0;
        int batchSize = 10;

        while (claims.next()) {
            Claim claim = (Claim) claims.get(0);
            this.claimKey = claim.getClaimKey();
            this.fileNbr = claim.getFileNumber();
            this.st02CtlNbr = claim.getSt02txnCtlNbr();
            this.billPrvdId = claim.getNm109billPrvdId();
            this.sbcrId = claim.getSubscriberId();
            this.claimType = claim.getClaimType();
            this.statusCd = claim.getStatusCode();
            this.cnt++;

            StringBuffer sbDtlRec = new StringBuffer();
            sbDtlRec.append("CLM"); //Record ID - 3 long in 5010
            delimit(sbDtlRec);
            sbDtlRec.append(formatOutputString(Integer.toString(this.cnt),
                    REQUIRED, 1, 10, StringFormat.LPADZERO)); //Record Number         
            sbDtlRec.append(PAYER_ID); //Payer Id
            delimit(sbDtlRec);

            sbDtlRec.append("021"); //Maintenance Type Code 
                                    //TWP As Per converstation with dave we are always adding records
                                    //in all our incremental loads
                                    //We do not send a update or delete record - is this correct Sreedhar?

            delimit(sbDtlRec);

            //The billing provider ois identified by loop 2010AA and the Entity Identifier Code = 85
            EntityName entityNameBillingProv = getEntityName("2010AA",
                    "AND NM101ENTY_ID_CD = '85' ");
            String nm108CdQual = entityNameBillingProv.getNm108idCdQual();

            //TWP: Changed where we get the EIN/TIN from. it comes from entity additional information now
            //loop 2010AA with a qualifier of EI for Employers Identification Number or SY for
            //Social Security Number
            String qstr2 = "select ea from eEntityAdtlId ea " +
                "where ea.txnSet.id.fileNumber = :file " +
                "  and ea.txnSet.id.st02txnCtlNbr = :ctlnbr " +
                "  and ea.loopId = '2010AA' " + "  and ea.txType like '837%' " +
                "  and ea.ref01idQlfr in ('EI', 'SY') ";
            Query q2 = eReposEM.createQuery(qstr2);
            q2.setParameter("file", claim.getFileNumber());
            q2.setParameter("ctlnbr", claim.getSt02txnCtlNbr());

            List<EntityAdtlId> entityAdtls = q2.getResultList();
            Iterator<EntityAdtlId> entityAdtlsIterator = entityAdtls.iterator();

            String entityDataEINorSSNQualifier = ""; //This is the qualifier that tells us what is in ref02 
            String entityEINorSSN = "";

            if (entityAdtlsIterator.hasNext()) {
                EntityAdtlId eAID = entityAdtlsIterator.next();
                entityDataEINorSSNQualifier = eAID.getRef01idQlfr();
                entityEINorSSN = eAID.getRef02scndId();
            }

            //If we have an EIN or SSN we will populate it here
            if ((entityNameBillingProv != null) &&
                    ("24".equals(nm108CdQual) || "34".equals(nm108CdQual))) {
                sbDtlRec.append(formatOutputString(
                        entityNameBillingProv.getNm109idCd(), REQUIRED, 9, 9,
                        StringFormat.RPADSPACES)); //Billing Provider Tax Id 
            } else if ("EI".equals(entityDataEINorSSNQualifier) ||
                    "SY".equals(entityDataEINorSSNQualifier)) {
                sbDtlRec.append(formatOutputString(entityEINorSSN, REQUIRED, 9,
                        9, StringFormat.RPADSPACES)); //Billing Provider Tax Id
            } else {
                sbDtlRec.append(formatOutputString("", OPTIONAL, 9, 9,
                        StringFormat.RPADSPACES)); //Billing Provider Tax Id             
            }

            // We have either the Tax ID or the NPI so the
            // 'Billing Provider Federal Tax ID' isn't needed
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Billing Provider Assigned Payer Number  TWP Changed length from 20 to 50

            if ("XX".equals(nm108CdQual)) {
                //XX indicates we have a Centers for Medicare and Medicaid Services National Provider Identifier
                //Therefore we send the Identification Code which is an NPI
                sbDtlRec.append(formatOutputString(this.billPrvdId, OPTIONAL,
                        10, 10, StringFormat.RPADSPACES)); //Billing Provider National Provider ID
            } else {
                sbDtlRec.append(formatOutputString("", OPTIONAL, 10, 10,
                        StringFormat.RPADSPACES)); //Billing Provider National Provider ID            
            }

            sbDtlRec.append(formatOutputString(
                    entityNameBillingProv.getNm103lastNm(), OPTIONAL, 1, 60,
                    null)); //Billing Provider Last Name TWP: changed to length 60 from 35
            sbDtlRec.append(formatOutputString(
                    entityNameBillingProv.getNm104fstNm(), OPTIONAL, 1, 35, null)); //Billing Provider First Name TWP: changed to length 35 from 25                          
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                    StringFormat.RPADSPACES)); //Billing Provider Middle Name TWP: added this element
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                    StringFormat.RPADSPACES)); //Billing Provider Name Suffix TWP: added this element for 5010       

            EntityName serviceProvider = getEntityName(
                    "AND NM101ENTY_ID_CD = '82' AND CLAIM_KEY='" + claimKey +
                    "'");
            String serviceProviderTaxId = null;
            String serviceProviderPayerAssignedNumber = null;
            String serviceProviderNPI = null;
            String serviceProviderLastName = null;
            String serviceProviderFirstName = null;
            String serviceProviderQualifier = "";

            if (serviceProvider != null) {
                serviceProviderQualifier = serviceProvider.getNm108idCdQual();
            }

            if ((serviceProvider != null) &&
                    (serviceProviderQualifier != null) &&
                    !serviceProviderQualifier.trim().isEmpty()) {
                if (claimType.startsWith("004010")) {
                    if (("24".equalsIgnoreCase(serviceProviderQualifier) ||
                            "34".equalsIgnoreCase(serviceProviderQualifier)) &&
                            (serviceProvider.getNm109idCd() != null) &&
                            !serviceProvider.getNm109idCd().trim().isEmpty()) {
                        serviceProviderTaxId = serviceProvider.getNm109idCd();
                    }

                    if ("XX".equalsIgnoreCase(serviceProviderQualifier) &&
                            (serviceProvider.getNm109idCd() != null) &&
                            !serviceProvider.getNm109idCd().trim().isEmpty()) {
                        serviceProviderNPI = serviceProvider.getNm109idCd();
                    }
                }

                if (claimType.startsWith("005010") &&
                        "XX".equalsIgnoreCase(serviceProviderQualifier) &&
                        (serviceProvider.getNm109idCd() != null) &&
                        !serviceProvider.getNm109idCd().trim().isEmpty()) {
                    serviceProviderNPI = serviceProvider.getNm109idCd();
                }

                if ((serviceProviderTaxId != null) ||
                        (serviceProviderNPI != null)) {
                    if ((serviceProvider.getNm103lastNm() != null) &&
                            !serviceProvider.getNm103lastNm().trim().isEmpty()) {
                        serviceProviderLastName = serviceProvider.getNm103lastNm();
                    }

                    if ((serviceProvider.getNm104fstNm() != null) &&
                            !serviceProvider.getNm104fstNm().trim().isEmpty()) {
                        serviceProviderFirstName = serviceProvider.getNm104fstNm();
                    }
                }
            }

            sbDtlRec.append(formatOutputString(serviceProviderTaxId, false, 9,
                    9, StringFormat.RPADSPACES));
            sbDtlRec.append(formatOutputString(
                    serviceProviderPayerAssignedNumber, false, 1, 50,
                    StringFormat.RPADSPACES));
            sbDtlRec.append(formatOutputString(serviceProviderNPI, false, 10,
                    10, StringFormat.RPADSPACES));
            sbDtlRec.append(formatOutputString(serviceProviderLastName, false,
                    1, 60, StringFormat.RPADSPACES));
            sbDtlRec.append(formatOutputString(serviceProviderFirstName, false,
                    1, 35, StringFormat.RPADSPACES));

            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                    StringFormat.RPADSPACES)); //Service Provider Middle Name
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                    StringFormat.RPADSPACES)); //Service Provider Name suffix 
            sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 80,
                    StringFormat.RPADSPACES)); //Employer Identification Number 
            sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 80,
                    StringFormat.RPADSPACES)); //Employer Name

            PersonInfo person = getPersonName(this.sbcrId);

            //TWP Seems Like the old code made the assumption that the subscriber and patient were the same
            //am I interpreting this correctly?
            if (person != null) {
                sbDtlRec.append(formatOutputString(person.getPersonId(),
                        OPTIONAL, 2, 80, StringFormat.RPADSPACES)); //Subscriber Id
                sbDtlRec.append(formatOutputString(person.getLastName(),
                        OPTIONAL, 1, 60, StringFormat.RPADSPACES)); //Subscriber Last Name            
                sbDtlRec.append(formatOutputString(person.getFirstName(),
                        OPTIONAL, 1, 35, StringFormat.RPADSPACES)); //Subscriber First Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                        StringFormat.RPADSPACES)); //Subscriber Middle Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                        StringFormat.RPADSPACES)); //Subscriber Name Suffix            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 80,
                        StringFormat.RPADSPACES)); //Patient ID - TWP looks like we did not send this in the past
                sbDtlRec.append(formatOutputString(person.getLastName(),
                        OPTIONAL, 1, 60, StringFormat.RPADSPACES)); //Patient Last Name            
                sbDtlRec.append(formatOutputString(person.getFirstName(),
                        OPTIONAL, 1, 35, StringFormat.RPADSPACES)); //Patient First Name
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                        StringFormat.RPADSPACES)); //Patient Middle Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                        StringFormat.RPADSPACES)); //Patient Name Suffix
            } else {
                //IL Selects the insured or subscriber - MI selects the Member identification number
                String vQl = "AND NM101ENTY_ID_CD = 'IL' AND NM108ID_CD_QUAL = 'MI' ";
                EntityName entityNameS = getEntityName("2010BA", vQl);
                sbDtlRec.append(formatOutputString(entityNameS.getNm109idCd(),
                        OPTIONAL, 2, 80, StringFormat.RPADSPACES)); //Subscriber Id
                sbDtlRec.append(formatOutputString(
                        entityNameS.getNm103lastNm(), OPTIONAL, 1, 60,
                        StringFormat.RPADSPACES)); //Subscriber Last Name            
                sbDtlRec.append(formatOutputString(
                        entityNameS.getNm104fstNm(), OPTIONAL, 1, 35,
                        StringFormat.RPADSPACES)); //Subscriber First Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                        StringFormat.RPADSPACES)); //Subscriber Middle Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                        StringFormat.RPADSPACES)); //Subscriber Name Suffix            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 80,
                        StringFormat.RPADSPACES)); //Patient ID - TWP looks like we did not send this in the past
                sbDtlRec.append(formatOutputString(
                        entityNameS.getNm103lastNm(), OPTIONAL, 1, 60,
                        StringFormat.RPADSPACES)); //Patient Last Name            
                sbDtlRec.append(formatOutputString(
                        entityNameS.getNm104fstNm(), OPTIONAL, 1, 35,
                        StringFormat.RPADSPACES)); //Patient First Name
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                        StringFormat.RPADSPACES)); //Patient Middle Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                        StringFormat.RPADSPACES)); //Patient Name Suffix                      
            }

            //TWP: Patient Date of Birth is now required - should I try to get it from PersonInfo as well???
            //This does not appear to meet the new spec
            DemographicInfo demographicInfo = getDemographicInfo();

            if (demographicInfo != null) {
                sbDtlRec.append(formatOutputString(
                        demographicInfo.getDmg02brthDt(), REQUIRED, 8, 8,
                        StringFormat.RPADSPACES)); //Patient Date of Birth

                if ((demographicInfo.getDmg03gndr() != null) &&
                        ("M".equalsIgnoreCase(demographicInfo.getDmg03gndr()) ||
                        "F".equalsIgnoreCase(demographicInfo.getDmg03gndr()))) {
                    sbDtlRec.append(formatOutputString(
                            demographicInfo.getDmg03gndr(), OPTIONAL, 1, 1,
                            StringFormat.RPADSPACES)); //Patient Gender
                } else {
                    sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 1,
                            StringFormat.RPADSPACES)); //Patient Gender
                }
            } else {
                sbDtlRec.append(formatOutputString("", REQUIRED, 8, 8,
                        StringFormat.RPADSPACES)); //Patient Date of Birth
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 1,
                        StringFormat.RPADSPACES)); //Patient Gender
            }

            //sbDtlRec.append(StringUtils.rPadSpaces(getRefValue("D9"), 15));  TWP removed webMDClaim Number
            //TWP What do we use instead
            sbDtlRec.append(formatOutputString(getRefValue("D9"), OPTIONAL, 1,
                    50, StringFormat.RPADSPACES)); //Emdeon Claim Number - TWP Do we have this somewhere?

            sbDtlRec.append(formatOutputString(claim.getClm02clmChrgAmt(),
                    REQUIRED, 1, 18, StringFormat.RPADSPACES)); //Claim Charge Amount          
            sbDtlRec.append(formatOutputString("0.00", OPTIONAL, 1, 18,
                    StringFormat.LPADZERO)); //Claim Payment Amount TWP:Changed to 18

            sbDtlRec.append(formatOutputString("", OPTIONAL, 8, 8,
                    StringFormat.RPADSPACES)); //Claim Adjudication/Payment Date
            sbDtlRec.append(formatOutputString("", OPTIONAL, 8, 8,
                    StringFormat.RPADSPACES)); //Check EFT/Date
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 16,
                    StringFormat.RPADSPACES)); //Check EFT Number TWP:to 16
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 3,
                    StringFormat.RPADSPACES)); //Bill Type

            sbDtlRec.append(formatOutputString(getClaimIndex(claimKey),
                    REQUIRED, 1, 50, StringFormat.RPADSPACES)); //Payer Claim Identification Number TWP:to 50

            sbDtlRec.append(formatOutputString(claim.getClm01ptAcctNbr(),
                    REQUIRED, 1, 50, StringFormat.RPADSPACES)); //Patient Account Number TWP:Changed to 50                
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Pharmacy Prescription Number
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Voucher Identifier
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Application of Location System Identifier
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Group Number

            sbDtlRec = fillClaimServiceDates(sbDtlRec); //Claim Service Date Start & Claim Service Date End

            vCsFile.print(sbDtlRec);
            vCsFile.print(CRLF);
            vCsFile.flush();

            //Create claim status lines for this cliam
            String payerClaimId = getClaimIndex(claimKey);
            writeSTLRec(claim, payerClaimId, "P1", "20", "", vCsFile);

            //We flush and clear aren't we losing our status updates? TWP
            if ((++count % batchSize) == 0) {
                session.flush();
                session.clear();
            }

            LOGGER.debug("{} records written", count);
        }
    }

    final void createTrlRec(PrintWriter vCsFile) {
        StringBuffer sbTrlRec = new StringBuffer();
        sbTrlRec.append("TRLR"); //Record ID
        delimit(sbTrlRec);
        //      sbTrlRec.append(PAYER_ID);  //Trading Partner ID
        //      delimit(sbTrlRec);
        sbTrlRec.append(StringUtils.lPadZeros(Integer.toString(this.cnt - 1), 10)); //Record Count  (Note format output will delimit the string)
                                                                                    //TWP spec says record count length is 1 has to be a typo
                                                                                    //      delimit(sbTrlRec);

        vCsFile.print(sbTrlRec);
        vCsFile.print(CRLF); //Record Terminator
        sbTrlRec = null;
        ;
    }

    @SuppressWarnings("unchecked")
    final EntityName getEntityName(String vLoopId, String vQual) {
        String qstr = "select e from eEntityName e " +
            "where e.txnSet.id.fileNumber = :file " +
            "  and e.txnSet.id.st02txnCtlNbr = :ctlnbr " +
            "  and e.loopId =  :loopid " + vQual;
        Query q = eReposEM.createQuery(qstr);
        q.setParameter("file", this.fileNbr);
        q.setParameter("ctlnbr", this.st02CtlNbr);
        q.setParameter("loopid", vLoopId);

        EntityName en = null;
        List<EntityName> entityNames = q.getResultList();

        if ((entityNames.size() > 0) && (entityNames.get(0) != null)) {
            en = entityNames.get(0);
        } else {
            LOGGER.warn("Error in getting EntityName for fileNumber: {}, st02txnCtlNbr: {}, loopId: {}, qal: {}",
                new Object[] { this.fileNbr, this.st02CtlNbr, vLoopId, vQual });
        }

        return en;
    }

    @SuppressWarnings("unchecked")
    final EntityName getEntityName(String vQual) {
        String qstr = "select e from eEntityName e " +
            "where e.txnSet.id.fileNumber = :file " +
            "  and e.txnSet.id.st02txnCtlNbr = :ctlnbr " + vQual;
        Query q = eReposEM.createQuery(qstr);
        q.setParameter("file", this.fileNbr);
        q.setParameter("ctlnbr", this.st02CtlNbr);

        EntityName en = null;
        List<EntityName> entityNames = q.getResultList();

        if ((entityNames.size() > 0) && (entityNames.get(0) != null)) {
            en = entityNames.get(0);
        } else {
            LOGGER.debug("Did not get EntityName for fileNumber: {}, st02txnCtlNbr: {}, loopId: {}, qal: {} note:it may not be required",
                new Object[] { this.fileNbr, this.st02CtlNbr, vQual });
        }

        return en;
    }

    @SuppressWarnings("unchecked")
    final PersonInfo getPersonName(String vSsn) {
        Query q = eReposEM.createQuery(
                "select p from foPersonInfo p where p.personId = :id ");
        q.setParameter("id", vSsn);

        PersonInfo pi = null;
        List<PersonInfo> pis = q.getResultList();

        if ((pis.size() > 0) && (pis.get(0) != null)) {
            pi = (PersonInfo) pis.get(0);
        } else {
            LOGGER.debug("Unable to get PersonInfo for person Id: {}", vSsn);
        }

        return pi;
    }

    @SuppressWarnings("unchecked")
    final DemographicInfo getDemographicInfo() {
        String qstr = "select d from eDemographicInfo d " +
            "where d.txnSet.id.fileNumber = :file " +
            "  and d.txnSet.id.st02txnCtlNbr = :ctlnbr " +
            "  and d.nm109sbsrId = :sbcrid " +
            "  and d.loopId in ('2010BA', '2010CA')";
        Query q = eReposEM.createQuery(qstr);
        q.setParameter("file", this.fileNbr);
        q.setParameter("ctlnbr", this.st02CtlNbr);
        q.setParameter("sbcrid", this.sbcrId);

        DemographicInfo di = null;
        List dis = q.getResultList();

        if ((dis.size() > 0) && (dis.get(0) != null)) {
            di = (DemographicInfo) dis.get(0);
        } else {
            LOGGER.debug("Unable to get Demographic Info for file: {}, st02txnCtlNbr: {}, nm109sbsrId: {} ",
                new Object[] { this.fileNbr, this.st02CtlNbr, this.sbcrId });
        }

        return di;
    }

    @SuppressWarnings("unchecked")
    final String getRefValue(String vRefQual) {
        String vRefVal = "";
        StringBuilder qstrb = new StringBuilder(
                "select e from eEntityAdtlId e ");

        if (vRefQual.equalsIgnoreCase("EI")) {
            qstrb.append("where e.txnSet.id.fileNumber = '").append(this.fileNbr)
                 .append("' ").append("and e.txnSet.id.st02txnCtlNbr = '")
                 .append(this.st02CtlNbr).append("' ")
                 .append("and e.loopId = '2010AA' ")
                 .append("and e.ref01idQlfr = '").append(vRefQual).append("' ");
        } else {
            qstrb.append("where e.claim.claimKey = '").append(this.claimKey)
                 .append("' ").append("and e.ref01idQlfr = '").append(vRefQual)
                 .append("' ");
        }

        Query q = eReposEM.createQuery(qstrb.toString());
        List entityAdtlIds = q.getResultList();

        if ((entityAdtlIds.size() > 0) && (entityAdtlIds.get(0) != null)) {
            EntityAdtlId entityAdtlId = (EntityAdtlId) entityAdtlIds.get(0);
            vRefVal = entityAdtlId.getRef02scndId();
        } else {
            LOGGER.debug("Unable to get EntityAtlId info for filenumber: {}, st02txnCtlNbr: {}, loopId: '2010AA', ref01IdQlfr: {}",
                new Object[] { this.fileNbr, this.st02CtlNbr, vRefQual });
        }

        return vRefVal;
    }

    final StringBuffer fillClaimServiceDates(StringBuffer sbDtlRec) {
        String vClmStartDt = "";
        String vClmEndDt = "";
        String strDtQry = "";

        if (this.claimType.equalsIgnoreCase(INSTITUTIONAL_CLAIM_TYPE) ||
                this.claimType.equalsIgnoreCase(INSTITUTIONAL_CLAIM_TYPE_4010)) {
            /*
             * DEF005887 - EAT - 20081006 Check for 'type' of date sent. It may be
             * either YYYYMMDD or YYYYMMDD-YYYYMMDD (a range) If it is a single
             * date than that is both the start date and end date.
             */
            strDtQry = "SELECT DTP03DT_FLD FROM CLAIM_DT " +
                "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                "AND TX_TYPE = '837I' AND DTP01DTTM_QUAL = '434' " +
                "ORDER BY DTP03DT_FLD DESC ";
            vClmStartDt = getDate(strDtQry);

            if (vClmStartDt.indexOf('-') == -1) {
                vClmEndDt = vClmStartDt;
            } else {
                vClmEndDt = vClmStartDt.substring(9, 17);
                vClmStartDt = vClmStartDt.substring(0, 8);
            }

            writeServiceDates(sbDtlRec, vClmStartDt, vClmEndDt);
        } else if (this.claimType.equalsIgnoreCase(DENTAL_CLAIM_TYPE) ||
                this.claimType.equalsIgnoreCase(DENTAL_CLAIM_TYPE_4010)) {
            //TWP these qualifiers may not be correct? Check them
            strDtQry = "SELECT DTP03DT_FLD FROM CLAIM_DT " +
                "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                "AND TX_TYPE = '837D' AND DTP01DTTM_QUAL = '472' " +
                "AND DENTAL_SERVICE_KEY IS NULL ";
            vClmStartDt = getDate(strDtQry);

            if (vClmStartDt.equals("")) {
                strDtQry = "SELECT MIN(TO_NUMBER(DTP03DT_FLD)) FROM CLAIM_DT " +
                    "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                    "AND TX_TYPE = '837D' AND DTP01DTTM_QUAL = '472' ";
                vClmStartDt = getDate(strDtQry);
                strDtQry = "SELECT MAX(TO_NUMBER(DTP03DT_FLD)) FROM CLAIM_DT " +
                    "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                    "AND TX_TYPE = '837D' AND DTP01DTTM_QUAL = '472' ";
                vClmEndDt = getDate(strDtQry);
            } else {
                if (vClmStartDt.length() > 8) {
                    vClmEndDt = vClmStartDt.substring(9, 17);
                    vClmStartDt = vClmStartDt.substring(0, 8);
                } else {
                    vClmEndDt = vClmStartDt;
                }
            }

            writeServiceDates(sbDtlRec, vClmStartDt, vClmEndDt);
        } else if (this.claimType.equalsIgnoreCase(PROFESSIONAL_CLAIM_TYPE) ||
                this.claimType.equalsIgnoreCase(PROFESSIONAL_CLAIM_TYPE_4010)) {
            strDtQry = "SELECT DTP03DT_FLD FROM CLAIM_DT " +
                "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                "AND TX_TYPE = '837P' AND DTP01DTTM_QUAL = '472' " +
                "AND DTP02DTTM_FMT_QUAL = 'RD8' " +
                "ORDER BY DTP03DT_FLD DESC ";
            vClmStartDt = getDate(strDtQry);

            if (vClmStartDt.equals("")) {
                strDtQry = "SELECT MIN(TO_NUMBER(DTP03DT_FLD)) FROM CLAIM_DT " +
                    "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                    "AND TX_TYPE = '837P' AND DTP01DTTM_QUAL = '472' ";
                vClmStartDt = getDate(strDtQry);
                strDtQry = "SELECT MAX(TO_NUMBER(DTP03DT_FLD)) FROM CLAIM_DT " +
                    "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                    "AND TX_TYPE = '837P' AND DTP01DTTM_QUAL = '472' ";
                vClmEndDt = getDate(strDtQry);
            } else {
                vClmEndDt = vClmStartDt.substring(9, 17);
                vClmStartDt = vClmStartDt.substring(0, 8);
            }

            writeServiceDates(sbDtlRec, vClmStartDt, vClmEndDt);
        } else {
            sbDtlRec.append(formatOutputString("17760704", REQUIRED, 8, 8,
                    StringFormat.RPADSPACES)); //Claim Service Date Start
            sbDtlRec.append(formatOutputString("17760704", REQUIRED, 8, 8,
                    StringFormat.RPADSPACES, false)); //Claim Service Date End
        }

        return sbDtlRec;
    }

    private void writeServiceDates(StringBuffer sbDtlRec, String vClmStartDt,
        String vClmEndDt) {
        if (vClmStartDt.equals("")) {
            sbDtlRec.append(formatOutputString("17760704", REQUIRED, 8, 8,
                    StringFormat.RPADSPACES)); //Claim Service Date Start
        } else {
            sbDtlRec.append(formatOutputString(vClmStartDt, REQUIRED, 8, 8,
                    StringFormat.RPADSPACES)); //Claim Service Date Start
        }

        if (vClmEndDt.equals("")) {
            sbDtlRec.append(formatOutputString("17760704", REQUIRED, 8, 8,
                    StringFormat.RPADSPACES, false)); //Claim Service Date End
        } else {
            sbDtlRec.append(formatOutputString(vClmEndDt, REQUIRED, 8, 8,
                    StringFormat.RPADSPACES, false)); //Claim Service Date End
        }
    }

    @SuppressWarnings("unchecked")
    final String getDate(String strDateQuery) {
        String vDt = "";

        Query q = eReposEM.createNativeQuery(strDateQuery);
        List results = q.getResultList();

        if ((results.size() > 0) && (results.get(0) != null)) {
            vDt = results.get(0).toString();
        }

        return vDt;
    }

    final String getClaimIndex(String ediClaimKey) {
        String retVal = "";
        Query q = eReposEM.createQuery(
                "select c.claimIndex from foClaim c where c.ediClaimKey = :key ");
        q.setParameter("key", ediClaimKey);

        try {
            Long index = (Long) q.getSingleResult();
            retVal = index.toString();
        } catch (PersistenceException pe) {
            LOGGER.warn("Unable to get FEE Claim Index for EDI claim : {} ",
                ediClaimKey);
        }

        return retVal;
    }

    final String getSeqNbr() {
        String strSeqNbr = "SELECT E_REPOS.WEBMD_REJ_CLAIM_SEQ.NEXTVAL FROM DUAL";
        Query q = eReposEM.createNativeQuery(strSeqNbr);

        return q.getSingleResult().toString();
    }

    final void insertProcessLog(String vType, int count) {
        EdiProcessLog ediProcessLog = new EdiProcessLog();
        ediProcessLog.setEdiProcessId(UUID.randomUUID().toString().toUpperCase());
        ediProcessLog.setFileDate(DateUtil.getCurSqlDate());
        ediProcessLog.setProcessName("X12 PENDING CLAIM STATUS");
        ediProcessLog.setMessage(
            "NUM. OF X12 PENDING CLAIMS SENT WITH CLAIM STATUS ");
        ediProcessLog.setCounts(Long.valueOf(count));
        ediProcessLog.setClaimType("837");
        ediProcessLog.setStatus("NEW");
        eReposEM.persist(ediProcessLog);
    }

    private void delimit(StringBuffer sb) {
        sb.append(DELIMITER);
    }

    private void writeSTLRec(Claim claim, String payerClaimId,
        String statusCategoryCode, String statusCode, String entityCode,
        PrintWriter vCsFile) {
        StringBuffer sbHdrRec = new StringBuffer();
        sbHdrRec.append("STC"); //Record ID
        delimit(sbHdrRec);
        this.cnt++;
        sbHdrRec.append(formatOutputString(Integer.toString(this.cnt),
                REQUIRED, 1, 10, StringFormat.LPADZERO)); //Record Number - must be in sync with other records
        sbHdrRec.append(PAYER_ID); //Payer ID
        delimit(sbHdrRec);
        sbHdrRec.append(formatOutputString(payerClaimId, REQUIRED, 1, 50,
                StringFormat.RPADSPACES));

        //TWP What do we use for an identification number - we have not assigned it yet
        //for a claim that has only been accepted ???? we have a GUID
        //Should we really be running this file after the FPPS PL/SQL loading runs so we can assign it a claim index?
        sbHdrRec.append(""); //Line Item Control Number
        delimit(sbHdrRec);

        Date today = Calendar.getInstance().getTime();
        String dateFormat = "yyyyMMddHHmmss";
        sbHdrRec.append(new SimpleDateFormat(dateFormat).format(today)); //Status Information Effective Date
        delimit(sbHdrRec);

        sbHdrRec.append(statusCategoryCode); //Claim Status Category Code  
        delimit(sbHdrRec);

        sbHdrRec.append(statusCode); //Claim Status Code 
        delimit(sbHdrRec);

        sbHdrRec.append(entityCode); //Entity Code 
        delimit(sbHdrRec);

        sbHdrRec.append(""); //Data In Error - we do not have this from what I can tell so leave it blank
        delimit(sbHdrRec);
        sbHdrRec.append(""); //Emdeon Status Code - I don;t see this either
                             //	   delimit(sbHdrRec);	      

        sbHdrRec.append(CRLF); //Record Teminator new for 5010
        vCsFile.print(sbHdrRec); //TWP Changed to print rather than println since I am explicitly adding CRLF  
        vCsFile.flush();
        sbHdrRec = null;
    }

    private String formatOutputString(Object dataToFormat, boolean required,
        int minLength, int maxLength, StringFormat format) {
        return formatOutputString(dataToFormat, required, minLength, maxLength,
            format, true);
    }

    /**
     * This method should be called for each field which
     * will make the code above basically match the spec.
     * It will format the field to the specs given and delimit
     * each value returned.
     *
     * @param dataToFormat
     * @param minLenght
     * @param maxLength
     * @param format
     * @param delimit
     * @return
     */
    private String formatOutputString(Object dataToFormat, boolean required,
        int minLength, int maxLength, StringFormat format, boolean delimit) {
        //If we don't have data for a required field we just balk right now - what do you really want to happen?
        if ((required && (dataToFormat == null)) ||
                (required &&
                (dataToFormat instanceof String &&
                (((String) dataToFormat).trim().length() < 1)))) {
            throw new RuntimeException(
                "We don't have this required data see stack trace.");
        }

        //We are assuming if we send a value it must meet the spec minLength and maxLength
        //But optional fields may be left blank - i.e. a blank field will appear in the output
        //For example see ;; in the following sample line -> 
        //													;nonblankfield;;nonblankfield;
        String stringToReturn = "";

        if ((dataToFormat != null) &&
                !(dataToFormat instanceof String &&
                (((String) dataToFormat).trim().length() < 1))) {
            //If the data needs to be formatted a specific way format it
            if (format != null) {
                stringToReturn = this.formatString(dataToFormat, minLength,
                        maxLength, format);
            } else {
                stringToReturn = dataToFormat.toString();
            }

            //Check to see if the value complies with EMDEONS minimum length requirement 		  	 
            int stringLength = stringToReturn.length();

            if (stringLength < minLength) {
                throw new RuntimeException(
                    "Please specify a formatter for this data so it can be formatted to meet the minimum lenght requirement." +
                    dataToFormat);
            }

            //Check to see if the value complies with EMDEONS max length requirement - 
            //If it does not we make truncate it - this seems dangerous - TWP.	   
            if (stringLength > maxLength) {
                stringToReturn = truncate(stringToReturn, dataToFormat,
                        maxLength);
            }
        }

        if (delimit) {
            stringToReturn += DELIMITER;
        }

        return stringToReturn;
    }

    private String truncate(String formattedString, Object dataToFormat,
        int maxLength) {
        int beginIndex = formattedString.length() - maxLength;
        String stringToReturn = formattedString.substring(beginIndex);
        LOGGER.warn("Truncating data from " + formattedString + " to " +
            stringToReturn);

        return stringToReturn;
    }

    /**
     * Throws a runtime exception if it can not format the data to the specification given.
     * Will also throw an exception if null is passed in - it expects to format a string.
     *
     * @param stringToFormat
     * @param minLength
     * @param maxLength
     * @param format
     * @return
     */
    private String formatString(Object stringToFormat, int minLength,
        int maxLength, StringFormat format) {
        String stringToReturn = null;

        if ((stringToFormat != null) && stringToFormat instanceof BigDecimal) {
            stringToFormat = stringToFormat.toString();
        }

        if ((stringToFormat != null) &&
                (((String) stringToFormat).length() < minLength)) {
            if (StringFormat.RPADSPACES == format) {
                stringToReturn = StringUtils.padString((String) stringToFormat,
                        minLength, ' ', false);
            }

            if (StringFormat.LPADZERO == format) {
                stringToReturn = StringUtils.padString((String) stringToFormat,
                        minLength, '0', true);
            }
        } else {
            if ((stringToFormat != null) &&
                    (((String) stringToFormat).length() >= minLength)) {
                stringToReturn = (String) stringToFormat;
            }
        }

        if (StringFormat.LPADDECZERO2PLACES == format) {
            stringToReturn = StringUtils.lPadDecimalZeros2Places((String) stringToFormat,
                    maxLength);
        }

        if (stringToReturn == null) {
            throw new RuntimeException("Could not format the object - " +
                stringToFormat +
                " - conversion of this type of data is not yet supported.");
        }

        return stringToReturn;
    }
    private enum StringFormat {RPADSPACES,
        LPADZERO,
        LPADDECZERO2PLACES;
    }
}
