package gov.va.med.hac.edi.extract.x12.jpa;

import gov.va.med.hac.edi.utils.StringUtils;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

import java.math.BigDecimal;

import org.hibernate.Session;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import javax.persistence.EntityManager;
import javax.persistence.Query;

import gov.va.med.domain.erepos.*;
// import gov.va.med.domain.erepos

import gov.va.med.domain.erepos.EntityAdtlId;
import gov.va.med.domain.erepos.PrfsnSvcLine;


/**
 * This class contains methods that are used by more than one of its subclasses.
 * The methods on this class fall into four categories
 * 
 * process methods The process methods drive the querying and writing of a
 * particular record type.
 * 
 * get methods The get methods query the database and return records
 * 
 * write methods The write methods write data into an extract file based on the
 * eRepos to Flat File.xls spec
 * 
 * utility methods The utility methods create strings that are padded to a
 * certain length so fields may be inserted into the extract file properly
 * 
 * @author vhahacpennet
 * 
 */
public class ProcessCacheRecords {
	static final Logger summaryLOGGER = LoggerFactory
			.getLogger(ProcessCacheRecords.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace."
			+ ProcessCacheRecords.class);
	private List listOfClaimsToPersist = new ArrayList();

	// protected PrintWriter vLogFile;

	//protected int vQueryCnt = 0;

	/**
	 * Writes VA routing information into the extract file indicated by vFile to
	 * help coordinate a transaction. This "envelope" information includes items
	 * like the ISA # and GS Control # that help correlate data interchanges.
	 * 
	 * @param rowIntchgControl
	 *            - an instance of an interchange control structure that
	 *            contains data that helps with the coordination of the data
	 *            interchange (ISA/IEA envelope information)
	 * 
	 * @param rowFunctionalGroup
	 *            - Contains high level identifiers that identify a group of
	 *            related transaction sets
	 * 
	 * @param vFile
	 *            - the file to write the extract data to
	 */
	protected void writeA005Rec(IntchgControl rowIntchgControl,
			FunctionalGroup rowFunctionalGroup, PrintWriter vFile) {
		final StringBuffer a005Rec = new StringBuffer("A005");  //A005 REC-ID
		a005Rec.append(StringUtils.rPadSpaces("", 18));			//A005 3051 FILLER 1
		a005Rec.append(StringUtils.lPadZeros(rowIntchgControl	//A005 ISA CONTROL NO
				.getIsa13intchgNbr(), 10));
		a005Rec.append(StringUtils.rPadSpaces("20"				//A005 FILE-CREATE-DATE
				+ rowIntchgControl.getIsa09intchgDt(), 8));		//A005 GRP-CONTROL-NO
		a005Rec.append(StringUtils.lPadZeros(rowFunctionalGroup //A005 GRP-CONTROL-NO
				.getGs06grpCtlNbr(), 10));
		a005Rec.append(StringUtils.rPadSpaces(					//A005 FILE-CREATE-DATE
				rowFunctionalGroup.getGs04creDt(), 8)); 
		a005Rec.append(StringUtils.rPadSpaces(rowIntchgControl	//A005 - VERSION
				.getIsa12intchgVer(), 5));
		traceLOGGER.debug("A005Rec: " + a005Rec.toString());
		vFile.println(a005Rec.toString());
		a005Rec.delete(0, a005Rec.length());
	}

	/**
	 * Writes transaction set data (ID From Table 0/Table 1) to the extract
	 * file. This includes the transcation set type, id and other information
	 * such as TS creation date.
	 * 
	 * @param vTransmissionType
	 *            - from entity adl id
	 * @param rowTxnSet
	 *            - the transaction set
	 * @param vFile
	 *            the extract file to write the data to
	 */
	protected void writeA010Rec(String vTransmissionType, TxnSet rowTxnSet,
			PrintWriter vFile) {
		final StringBuffer a010Rec = new StringBuffer("A010"); //A010 REC-ID
		a010Rec.append(StringUtils.rPadSpaces(vTransmissionType, 15));	//A010-TRANSMISSION-TYPE-ID 
		a010Rec.append(StringUtils.rPadSpaces(rowTxnSet.getSt01txnId(), 3));	//A010 TRANSACTION-SET- ID CODE
		a010Rec.append(StringUtils.rPadSpaces(rowTxnSet.getId()	//A010-Transaction Set Control Number
				.getSt02txnCtlNbr(), 10));
		a010Rec
		.append(StringUtils
				.rPadSpaces(rowTxnSet.getBht01hrchlStrc(), 4)); //HL Structure Code
		a010Rec
		.append(StringUtils
				.rPadSpaces(rowTxnSet.getBht02txnPrpsCd(), 2)); //A010 Purpose Code
		a010Rec.append(StringUtils
				.rPadSpaces(rowTxnSet.getBht03orgnTxnId(), 50)); //A010 Submission Number 5010:Changed from 30 to 50 len
		a010Rec.append(StringUtils.rPadSpaces(rowTxnSet.getBht04txnCreDt(), 8)); //TS Creation Date
		a010Rec.append(StringUtils.lPadZeros(rowTxnSet.getBht05txnCreTm(), 8)); //TS Creation Time
		a010Rec
		.append(StringUtils
				.rPadSpaces(rowTxnSet.getBht06txnTypeCd(), 2));  //Transaction Type Code
		a010Rec.append(StringUtils.lPadZeros("", 1)); //5010:Removed this filler
		a010Rec.append(StringUtils.rPadSpaces(
				rowTxnSet.getId().getFileNumber(), 20)); //File Authorization number from header
		traceLOGGER.debug("A010Rec: " + a010Rec.toString());
		vFile.println(a010Rec.toString());
		a010Rec.delete(0, a010Rec.length());
	}

	/**
	 * Writes information about the submitter to the extract file vFile.
	 * 
	 * @param rowEntityName
	 *            - information about the submitter
	 * @param vFile
	 *            the extract file to write the data to
	 */
	protected void writeA015Rec(EntityName rowEntityName, PrintWriter vFile) {
		final StringBuffer a015Rec = new StringBuffer("A015");

//		 The width of 35 in old code was a typo that probably had no adverse
//		 effects.
		a015Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm103lastNm(), 60)); //Submitter Name 5010: C 30 to 60
		a015Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm109idCd(), 30)); //Submitter Id Num 5010: C 20 to 30
		traceLOGGER.debug("A015Rec: " + a015Rec.toString());
		vFile.println(a015Rec.toString());
		a015Rec.delete(0, a015Rec.length());
	}

	/**
	 * Writes information about the receiver to the extract file vFile.
	 * 
	 * @param rowEntityName
	 *            - information about the receiver (is this the VA?)
	 * @param vFile
	 *            the extract file to write the data to
	 */
	protected void writeA020Rec(EntityName rowEntityName, PrintWriter vFile) {
		final StringBuffer a020Rec = new StringBuffer("A020");
		a020Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm103lastNm(),
				60)); //5010: C 30 to 60
		a020Rec
		.append(StringUtils
				.rPadSpaces(rowEntityName.getNm109idCd(), 30));  //5010: C 15 to 30
		traceLOGGER.debug("A020Rec: " + a020Rec.toString());
		vFile.println(a020Rec.toString());
		a020Rec.delete(0, a020Rec.length());
	}

	/**
	 * Writes trailer information to the extract file - i.e. end of envelope
	 * marker.
	 * 
	 * @param vFile
	 *            the extrat file
	 */
	protected void writeA999Rec(PrintWriter vFile) {
		final StringBuffer a999Rec = new StringBuffer("A999");
		a999Rec.append(StringUtils.lPadZeros("1", 5)); 
		traceLOGGER.debug("A999Rec: " + a999Rec.toString());
		vFile.println(a999Rec.toString());
		a999Rec.delete(0, a999Rec.length());
	}

	/**
	 * Writes information about the billing provider to the extract file.
	 * 
	 * @param vPrvTaxonamyCd
	 *            - provider taxonomy code
	 * @param vCurrencyCd
	 *            - provider currency code
	 * @param vPrvIdSec
	 *            - Provider Secondary Identification
	 * @param vName
	 *            - Provider name
	 * @param vNm109idCd
	 *            - the providers tax identification code
	 * @param rowEntityAddress
	 *            - the providers address
	 * @param vFile
	 *            - the extract file
	 */ 
	protected void writeB000Rec(String vPrvTaxonamyCd, String vCurrencyCd,
			String vPrvIdSec, String vName, String vNm109idCd,
			EntityAddress rowEntityAddress, String vNameSuffix, PrintWriter vFile) {
		final StringBuffer b000Rec = new StringBuffer("B000");  //Record Id
		b000Rec.append(StringUtils.rPadSpaces(vPrvTaxonamyCd, 50)); //Provider Taxonomy Code 5010: C 15 to 50
		b000Rec.append(StringUtils.rPadSpaces(vCurrencyCd, 3)); //Provider Currency Code
		b000Rec.append(StringUtils.rPadSpaces(vName, 120));  //Provider Name 
							//5010: C 35 to This is the concatenation of first middle last name and suffix (60+35+25+10)
		
		// 5010: If there is no NM109, which can only happen for a 5010 claim,
		//         use the REF value for the required TIN in position 178 or so
		if (vNm109idCd == null || vNm109idCd.length() == 0) {
      b000Rec.append(StringUtils.rPadSpaces(vPrvIdSec, 30));  //Provider Tax Identification Number  5010:C 15 to 30
      b000Rec.append(StringUtils.rPadSpaces(vNm109idCd, 50)); //Provider Tax Secondary Identification 5010:C 30 to 50
		}
		// The rest of this logic is the same as it was in 4010.
		// Check the length of the NM109 value.  If it is 10, it is an NPI,
		//   and the REF value is used for the required TIN.
		// If the NM109 value is of length 9, it is used for the TIN
		// The qualifiers are not consulted for this decision
		// If this is a 5010 claim with an NM109, that value must be an NPI, meaning length 10,
		//   and the REF value will be used for the TIN
		else if (vNm109idCd.length() > 9) {
			b000Rec.append(StringUtils.rPadSpaces(vPrvIdSec, 30));	//Provider Tax Identification Number  5010:C 15 to 30
			b000Rec.append(StringUtils.rPadSpaces(vNm109idCd, 50)); //Provider Tax Secondary Identification 5010:C 30 to 50
		} else {
			b000Rec.append(StringUtils.rPadSpaces(vNm109idCd, 30));  //Provider Tax Identification Number  5010:C 15 to 30
			b000Rec.append(StringUtils.rPadSpaces(vPrvIdSec, 50));  //Provider Tax Secondary Identification 5010:C 30 to 50
		}

		b000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN301adrLn1(),     //Provider Address Line 1 (Street)
				30));
		b000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN302adrLn2(),		//Provider Address Line 2 (Street 2)
				30));
		b000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN401city(),		//Provider City
				20));
		b000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN402st(), 2));	//Provider State
		b000Rec.append(StringUtils.rPadSpaces(rowEntityAddress
				.getN403postalCd(), 15));											//Provider Zip Code
		b000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN404ctryCd(),		//Provider Country
				3));
		b000Rec.append(StringUtils.rPadSpaces(vNameSuffix, 10));  //Name Suffix
		traceLOGGER.debug("B000Rec: " + b000Rec.toString());
		vFile.println(b000Rec);
		b000Rec.delete(0, b000Rec.length());
	}

	/**
	 * Writes contact information about the billing provider to the extract
	 * file.
	 * 
	 * @param vCntcName
	 *            Name of the contact person at the provider
	 * @param vTelNbr
	 *            - providers phone #
	 * @param vFax
	 *            - providers fax #
	 * @param vEmail
	 *            - providers email #
	 * @param vFile
	 */
	protected void writeB001Rec(String vCntcName, String vTelNbr, String vFax,
			String vEmail, PrintWriter vFile) {
		final StringBuffer b001Rec = new StringBuffer("B001");
		b001Rec.append(StringUtils.rPadSpaces(vCntcName, 30));
		b001Rec.append(StringUtils.rPadSpaces(vTelNbr, 30));
		b001Rec.append(StringUtils.rPadSpaces(vFax, 30));
		b001Rec.append(StringUtils.rPadSpaces(vEmail, 35));
		traceLOGGER.debug("B001Rec: " + b001Rec.toString());
		vFile.println(b001Rec);
		b001Rec.delete(0, b001Rec.length());
	}

	/**
	 * Writes information about the provider to receive payment to the extract
	 * file.
	 * 
	 * @param vName
	 *            - name of the provider
	 * @param vNm109idCd
	 *            - Pay to provider's id (i think tin)
	 * @param vPrvIdSec
	 *            - Pay-to provider's Secondary ID No.
	 * @param rowEntityAddress
	 *            the providers address
	 * @param vFile
	 *            - extract file to write data to
	 */
	protected void writeB005Rec(String vName, String vNm109idCd,
			String vPrvIdSec, EntityAddress rowEntityAddress, EntityName rowEntityName, 
			EntityName payToPlanName, EntityAddress payToPlanEntityAddress, 
			String payToPlanRefId, String payToPlanTaxRefId, PrintWriter vFile) {
		final StringBuffer b005Rec = new StringBuffer("B005");

		b005Rec.append(StringUtils.rPadSpaces(vName, 120));  //5010: Changed from 35 to 60
		
		// 5010: NM109 is no longer required in this area, in fact it never shows up.
		// The null check should take care of this.
		// If there is no NM109, then the else will run, and two sets of blanks will print out
		if (vNm109idCd != null && vNm109idCd.length() > 9) {
			b005Rec.append(StringUtils.rPadSpaces(vPrvIdSec, 30));  //5010: Pay-To TIN C: 15-30
			b005Rec.append(StringUtils.rPadSpaces(vNm109idCd, 30)); //5010: Pay-To Secondary ID C:15-50
		} else {
			b005Rec.append(StringUtils.rPadSpaces(vNm109idCd, 30)); //5010: Pay-To TIN changed C:15-30
			b005Rec.append(StringUtils.rPadSpaces(vPrvIdSec, 30));  //5010: Pay-To Secondary ID C:15-50
		}

		b005Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN301adrLn1(),
				30));
		b005Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN302adrLn2(),
				30));
		b005Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN401city(),
				30));
		b005Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN402st(), 2));

		// The old code had this mistakenly using 15 chars
		// b005Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN403postalCd(),
		// 15));
		b005Rec.append(StringUtils.rPadSpaces(rowEntityAddress
				.getN403postalCd(), 11));

		b005Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN404ctryCd(),
				3));
		
		//All the fields below this line have been added for 5010
		if(payToPlanName != null)
		{
			b005Rec.append(StringUtils.rPadSpaces(payToPlanName.getNm103lastNm(),60)); //5010: Pay To Plan Name
			b005Rec.append(StringUtils.rPadSpaces(payToPlanName.getNm109idCd(),80)); //5010: Pay To Plan ID
		}else
		{
			b005Rec.append(StringUtils.rPadSpaces("",60)); //5010: Pay To Plan Name
			b005Rec.append(StringUtils.rPadSpaces("",80)); //5010: Pay To Plan ID
		}
		if(payToPlanEntityAddress != null)
		{
			b005Rec.append(StringUtils.rPadSpaces(payToPlanEntityAddress.getN301adrLn1(),30)); //5010: Pay To Plan Address1
			b005Rec.append(StringUtils.rPadSpaces(payToPlanEntityAddress.getN302adrLn2(),30)); //5010: Pay To Plan Address2
			b005Rec.append(StringUtils.rPadSpaces(payToPlanEntityAddress.getN401city(),20)); //5010: Pay To Plan City
			b005Rec.append(StringUtils.rPadSpaces(payToPlanEntityAddress.getN402st(),2)); //5010: Pay To Plan State
			b005Rec.append(StringUtils.rPadSpaces(payToPlanEntityAddress.getN403postalCd(),11)); //5010: Pay To Plan Zip
			b005Rec.append(StringUtils.rPadSpaces(payToPlanEntityAddress.getN404ctryCd(),3)); //5010: Pay To Plan Country

		}else
		{
			b005Rec.append(StringUtils.rPadSpaces("",30)); //5010: Pay To Plan Address1
			b005Rec.append(StringUtils.rPadSpaces("",30)); //5010: Pay To Plan Address2
			b005Rec.append(StringUtils.rPadSpaces("",20)); //5010: Pay To Plan City
			b005Rec.append(StringUtils.rPadSpaces("",2)); //5010: Pay To Plan State
			b005Rec.append(StringUtils.rPadSpaces("",11)); //5010: Pay To Plan Zip
			b005Rec.append(StringUtils.rPadSpaces("",3)); //5010: Pay To Plan Country
		}
		b005Rec.append(StringUtils.rPadSpaces(payToPlanRefId,50)); //5010: Pay To Plan Reference Identification
		b005Rec.append(StringUtils.rPadSpaces(payToPlanTaxRefId,50)); //5010: Pay To Plan Tax ID Reference Identification
		traceLOGGER.debug("B005Rec: " + b005Rec.toString());
		vFile.println(b005Rec);
		b005Rec.delete(0, b005Rec.length());
	}

	/**
	 * Writes information about the subscriber/patient to the extract file.
	 * 
	 * @param vSbrSecId
	 *            - Patient Secondary ID
	 * @param vDtOfDeath
	 *            - patient's date of death if dead
	 * @param rowSubscriberInfo
	 *            - information like who is responsible, relationship to
	 *            subscriber, etc.
	 * @param rowEntityName
	 *            - subscribers name
	 * @param rowEntityAddress
	 *            - subscribers address
	 * @param rowDemographicInfo
	 *            - subscribers gender/dob etc.
	 * @param vFile
	 */
	protected void writeC000Rec(String vSbrSecId, String vDtOfDeath,
			SubscriberInfo rowSubscriberInfo, EntityName rowEntityName,
			EntityAddress rowEntityAddress, DemographicInfo rowDemographicInfo,
			PrintWriter vFile) {
		final StringBuffer c000Rec = new StringBuffer("C000");	//Record Id
		c000Rec.append(rPadSpaces(rowSubscriberInfo.getSbr01pyrRespSeq(), 1)); //Payment Resp Code
		c000Rec.append(StringUtils.rPadSpaces(rowSubscriberInfo //Relationship to Sponsor
				.getSbr02indvRltCd(), 2));
		c000Rec.append(StringUtils.rPadSpaces(rowSubscriberInfo	//Patient Claim Filling Ind Code
				.getSbr09clmFileInd(), 2));

		c000Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm103lastNm(),
				60));	//Patient Last Name
		c000Rec.append(StringUtils
				.rPadSpaces(rowEntityName.getNm104fstNm(), 35)); //Patient First Name
		c000Rec.append(StringUtils
				.rPadSpaces(rowEntityName.getNm105midNm(), 25)); //Patient Middle Name
		c000Rec.append(StringUtils
				.rPadSpaces(rowEntityName.getNm107nmSfx(), 10)); ////Patient Name Suffix
		c000Rec
		.append(StringUtils
				.rPadSpaces(rowEntityName.getNm109idCd(), 30));	//Patient SSN or Auth Number

		if(rowEntityAddress != null)  //5010 makes this element situational and emdon data was breaking the process
		{
			c000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN301adrLn1(),	30));
			c000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN302adrLn2(), 30));
			c000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN401city(), 20));
			c000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN402st(), 2));
			c000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN403postalCd(), 11));
			c000Rec.append(StringUtils.rPadSpaces(rowEntityAddress.getN404ctryCd(),
				3));
		}else
		{
			c000Rec.append(StringUtils.rPadSpaces("", 30));
			c000Rec.append(StringUtils.rPadSpaces("", 30));
			c000Rec.append(StringUtils.rPadSpaces("", 20));
			c000Rec.append(StringUtils.rPadSpaces("", 2));
			c000Rec.append(StringUtils.rPadSpaces("", 11));
			c000Rec.append(StringUtils.rPadSpaces("", 3));			
		}
		
		if (rowDemographicInfo == null) {
			c000Rec.append(StringUtils.rPadSpaces("", 8));
			c000Rec.append(StringUtils.rPadSpaces("", 1));
		} else {
			c000Rec.append(StringUtils.rPadSpaces(rowDemographicInfo.getDmg02brthDt(), 8)); //Patient Birth Date
			c000Rec.append(StringUtils.rPadSpaces(rowDemographicInfo.getDmg03gndr(), 1));  //Patient Gender Code
		}
		c000Rec.append(StringUtils.rPadSpaces(vSbrSecId, 50));   	//Patient Secondary Id	c:30-50	
		c000Rec.append(StringUtils.rPadSpaces(vDtOfDeath, 8));  	//Patient Date of Death
		c000Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm102entyQual(), 1));  //Entity Type Qualifier																								
		traceLOGGER.debug("C000Rec: " + c000Rec.toString());
		vFile.println(c000Rec.toString());
		c000Rec.delete(0, c000Rec.length());
	}

	/**
	 * Writes information about the destination payer (the payer who is
	 * specified in the Subscriber/Payer loop) to the extract file.
	 * 
	 * @param rowEntityName
	 *            - destination payor name, etc.
	 * @param vFile
	 *            - extract file to write data to
	 */
	protected void writeC005Rec(EntityName rowEntityName, PrintWriter vFile) {
		final StringBuffer c005Rec = new StringBuffer("C005");

		c005Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm103lastNm(),
				60));  												//Destination Payer Name C:30-60
		final String vDestId = rowEntityName.getNm109idCd();
		if ("84146".equals(vDestId) || "84147".equals(vDestId)) {
			c005Rec.append(StringUtils.rPadSpaces("841469824", 30)); //Destination Payer ID C:15-60
		} else {
			c005Rec.append(StringUtils.rPadSpaces(vDestId, 30));
		}
		traceLOGGER.debug("C005Rec: " + c005Rec.toString());
		vFile.println(c005Rec);
		c005Rec.delete(0, c005Rec.length());
	}

	
	/**
	 * Writes basic claim data to the extract file. Place of Service/TOB Place
	 * of Service Qualifier Frequency of Service Provider Assignmnt of Benefits
	 * Related Causes #1 Related Causes #2 Related Causes #3 Related Cause
	 * Accident State Related Cause Accident Country Claim Submission Reason
	 * Code Delay Reason Code HDR claim record ID number
	 * 
	 * @param rowClaim
	 *            - the claim
	 * @param vFclyQual
	 *            - Frequency of Service
	 * @param vFile
	 *            - the extract file to write the data to
	 */
	protected void writeE000Rec(Claim rowClaim, String vFclyQual, String vTPClaimRecId,
			PrintWriter vFile) {
		final StringBuffer e000Rec = new StringBuffer("E000");
		e000Rec
		.append(StringUtils
				.rPadSpaces(rowClaim.getClm01ptAcctNbr(), 38));
		e000Rec
		.append(StringUtils
				.rPadSpaces(rowClaim.getClm05fclyTypeCd(), 2));
		e000Rec.append(StringUtils.rPadSpaces(rowClaim.getClm05fclyCdQual(), 1));
		e000Rec.append(rPadSpaces(rowClaim.getClm05clmFrqCd(), 1));
		e000Rec.append(rPadSpaces(rowClaim.getClm08asgnOfBnft(), 1));
		e000Rec
		.append(StringUtils
				.rPadSpaces(rowClaim.getClm11rltCauses1(), 2));
		e000Rec
		.append(StringUtils
				.rPadSpaces(rowClaim.getClm11rltCauses2(), 2));
		e000Rec
		.append(StringUtils
				.rPadSpaces(rowClaim.getClm11rltCauses3(), 2));
		e000Rec.append(StringUtils.rPadSpaces(rowClaim.getClm11st(), 2));
		e000Rec.append(StringUtils.rPadSpaces(rowClaim.getClm11ctryCd(), 3));
		e000Rec.append(StringUtils
				.rPadSpaces(rowClaim.getClm19predterBnft(), 2));
		e000Rec
		.append(StringUtils
				.rPadSpaces(rowClaim.getClm20delayRsnCd(), 2));
		e000Rec.append(StringUtils.rPadSpaces(rowClaim.getClaimKey(), 36));  //5010: changed 30 to 50
		e000Rec.append(StringUtils.rPadSpaces(vTPClaimRecId, 50));  
		traceLOGGER.debug("E000Rec: " + e000Rec.toString());
		vFile.println(e000Rec);
		e000Rec.delete(0, e000Rec.length());
	}

	/**
	 * Writes claim dates to the extract file.
	 * 
	 * @param vStmtDt
	 *            - Statement From Date/Initial Treatmt
	 * @param vAdmissionDt
	 *            - Admission Date
	 * @param vDischargeDt
	 *            - Discharge Date
	 * @param vInjuryDt
	 *            - Injury Date
	 * @param vAccidentDt
	 *            - Accident Date
	 * @param vFile
	 *            - extract file to write claim data to
	 */
	protected void writeE005Rec(String vStmtDt, String vAdmissionDt,
			String vDischargeDt, String vInjuryDt, String vAccidentDt,
			String vLastSeenDt, String vPrescriptionDt, PrintWriter vFile) {
		
		 traceLOGGER.debug("In writeE005 with vStmtDt: " + vStmtDt + ", vAdmissionDt: "
			+ vAdmissionDt + ", vDischargeDt: " + vDischargeDt);	  
	  

		//HRL - TWP Removed check to avoid crashing cache when statement dates are not
		//sent in for Professional this may write a 
		// black record but this is preferred since it wont crash cache 
		/*
		if ((vStmtDt != null) || (vAdmissionDt != null)
				|| (vDischargeDt != null) || (vInjuryDt != null)
				|| (vAccidentDt != null)) {
		*/
			final StringBuffer e005Rec = new StringBuffer("E005");
			if (vStmtDt == null) {
				e005Rec.append(StringUtils.rPadSpaces("", 8)); //statement from date
				e005Rec.append(StringUtils.rPadSpaces("", 8));  //statement to date
			} else {
				if (vStmtDt.length() > 7) {
					e005Rec.append(StringUtils.rPadSpaces(vStmtDt, 8)); //statement from date
				} else {
					e005Rec.append(StringUtils.rPadSpaces("", 8)); //statement from date
				}

				if (vStmtDt.length() > 15) {
					e005Rec.append(StringUtils.rPadSpaces(vStmtDt.substring(9,
							17), 8)); //statement to date
				} else {
					e005Rec.append(StringUtils.rPadSpaces("", 8)); //statement to date
				}
			}

			if (vAdmissionDt == null) {
				e005Rec.append(StringUtils.rPadSpaces("", 8));  //admission date
				e005Rec.append(StringUtils.lPadZeros("", 2));  //admission hour
			} else {
				if (vAdmissionDt.length() > 7) {
					e005Rec.append(StringUtils.rPadSpaces(vAdmissionDt, 8)); //admission date
				} else {
					e005Rec.append(StringUtils.rPadSpaces("", 8)); //admission date
				}

				if (vAdmissionDt.length() > 10) {
					e005Rec.append(StringUtils.lPadZeros(vAdmissionDt
							.substring(8, 10), 2));   //admission hour
				} else if (vAdmissionDt.length() < 11) {
					e005Rec.append(StringUtils.lPadZeros(vAdmissionDt
							.substring(8, vAdmissionDt.length()), 2)); //admission hour
				} else {
					e005Rec.append(StringUtils.lPadZeros("", 2)); //admission hour
				}
			}

			if (vDischargeDt == null) {
				e005Rec.append(StringUtils.rPadSpaces("", 8));  //discharge date
				e005Rec.append(StringUtils.lPadZeros("", 2)); //discharge hour
			} else {
				if (vDischargeDt.length() > 6) {
					e005Rec.append(StringUtils.rPadSpaces(vDischargeDt, 8)); //discharge date
				} else {
				 	e005Rec.append(StringUtils.rPadSpaces("", 8)); //discharge date
				}

				if (vDischargeDt.length() > 2) {
					e005Rec.append(StringUtils.lPadZeros(vDischargeDt, 2)); //discharge hour
				} else if (vDischargeDt.length() < 3) {
					e005Rec.append(StringUtils.lPadZeros(vDischargeDt, 2)); //discharge hour
				} else {
					e005Rec.append(StringUtils.lPadZeros("", 2)); //discharge hour
				}
			}

			e005Rec.append(StringUtils.rPadSpaces(vInjuryDt, 8));
			e005Rec.append(StringUtils.rPadSpaces(vAccidentDt, 8));
			e005Rec.append(StringUtils.rPadSpaces(vLastSeenDt, 8));  // date last seen
			e005Rec.append(StringUtils.rPadSpaces(vPrescriptionDt, 8));  //hearing vision prescription date			
			traceLOGGER.debug("E005Rec: " + e005Rec.toString());
			vFile.println(e005Rec);
			e005Rec.delete(0, e005Rec.length());
			
		//} //HRL - TWP Also removed closing bracket - 
	}

	/**
	 *  Method to write orthodontic information   
	 */
	protected void writeE006Rec( OrthodonticInfo orthodonticInfo, PrintWriter vFile )
	{
		final StringBuffer e006Rec = new StringBuffer("E006");		
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e006Rec.append(padDecimal(orthodonticInfo.getDn101orthTrmtMonthsCnt(), 18)); //Orthodontic Total Months Count

		// 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e006Rec.append(padDecimal(orthodonticInfo.getDn102orthTrmtMonthsRemCnt(), 18)); //Orthodontic Treatment Months Remaining Count
		
		e006Rec.append(StringUtils.rPadSpaces(orthodonticInfo.getDn104orthDesc(), 80)); //Description
		vFile.println(e006Rec);
		e006Rec.delete(0, e006Rec.length());
	}

	
	/**
	 *  Tooth status information.   
	 */
	protected void writeE007Rec(ToothStatus toothStatus, PrintWriter vFile )
	{
		final StringBuffer e007Rec = new StringBuffer("E007");		
		e007Rec.append(StringUtils.rPadSpaces(toothStatus.getDn201toothNbr(), 30)); //Tooth Number
		e007Rec.append(StringUtils.rPadSpaces(toothStatus.getDn202toothStat(), 2)); //Tooth Status Code
		vFile.println(e007Rec);
		e007Rec.delete(0, e007Rec.length());
	}
	
	
	/**
	 * Write Pat. D/C Status Code to the extract file.
	 * 
	 * @param vCl103StatCd
	 *            Patient D/C Status Code
	 * @param vFile
	 *            - the extract file to write the data to
	 */
	protected void writeE008Rec(InstClaimCode rowInstClaimCode, PrintWriter vFile) {
		final StringBuffer e008Rec = new StringBuffer("E008");		
		e008Rec.append(StringUtils.rPadSpaces(rowInstClaimCode.getCl103ptStatCd(), 2));
		// TWP: Why no nursing home data?
		e008Rec.append(StringUtils.rPadSpaces("", 1));
		e008Rec.append(StringUtils.rPadSpaces(rowInstClaimCode.getCl101admTypeCd(), 1));
		e008Rec.append(StringUtils.rPadSpaces(rowInstClaimCode.getCl102admSrcCd(), 1));
		traceLOGGER.debug("E008Rec: " + e008Rec.toString());
		vFile.println(e008Rec);
		e008Rec.delete(0, e008Rec.length());
	}

	/**
	 * Writes PWK Attach data (data about an attachment) to the extract file.
	 * e.g. Rec Type Code for Attachment Record Transmission Format Attachment
	 * Control Number
	 * 
	 * @param rowClaimSuplInfo
	 *            - contains information about the attachment
	 * @param vFile
	 *            the extract file to write the data to
	 */
	protected void writeE009Rec(ClaimSuplInfo rowClaimSuplInfo,
			PrintWriter vFile) {
		final StringBuffer e009Rec = new StringBuffer("E009");
		e009Rec.append(StringUtils.rPadSpaces(rowClaimSuplInfo
				.getPwk01rptTypeCd(), 2));
		e009Rec.append(StringUtils.rPadSpaces(rowClaimSuplInfo
				.getPwk02rptTrnsmCd(), 2));
		e009Rec.append(StringUtils.rPadSpaces(rowClaimSuplInfo
				.getPwk06atmntCtlNbr(), 30));
		traceLOGGER.debug("E009Rec: " + e009Rec.toString());
		vFile.println(e009Rec);
		e009Rec.delete(0, e009Rec.length());
	}

	/**
	 * Writes data about claim amounts to the extract file.
	 * 
	 * @param vTotBilled
	 *            - total charges billed
	 * @param vPatAmtPaid
	 *            - Patients amount paid
	 * @param vBalDue
	 *            - patients balance
	 * @param vFile
	 *            - the extract file to write the data to
	 */
	protected void writeE010Rec(BigDecimal vTotBilled, BigDecimal vPatAmtPaid,
			BigDecimal vBalDue, PrintWriter vFile) {
		final StringBuffer e010Rec = new StringBuffer("E010");
		e010Rec.append(padDecimal(vTotBilled, 18));
		e010Rec.append(padDecimal(vPatAmtPaid, 18));
		e010Rec.append(padDecimal(vBalDue, 18));
		traceLOGGER.debug("E010Rec: " + e010Rec.toString());
		vFile.println(e010Rec);
		e010Rec.delete(0, e010Rec.length());
	}

	/**
	 * Writes misc. claim data to the extract file
	 * 
	 * @param vIcnDcn
	 *            Original Ref No. (FL-37)
	 * @param vPreAuth1
	 *            Pre-Authorization Number
	 * @param vSubCtlNum
	 *            Submitter (ACH) Control No.
	 * @param vMedRecNum
	 *            Medical Record No. (276/277)
	 * @param vMedVerNum
	 *            Medicare Crossover Version No.
	 * @param vPreAuth2
	 *            Pre-Authorization Number 2
	 * @param vFile
	 *            - the extract file to write the data to
	 */
	protected void writeE015Rec(String vIcnDcn, String vPreAuth1,
			String vSubCtlNum, String vMedRecNum, String vMedVerNum,
			String vPreAuth2, String referralNumber1,String referralNumber2, 
			String autoAccidentState, PrintWriter vFile) {
		final StringBuffer e015Rec = new StringBuffer("E015");
		e015Rec.append(StringUtils.rPadSpaces(vIcnDcn, 50)); 
		e015Rec.append(StringUtils.rPadSpaces(vPreAuth1, 50));
		e015Rec.append(StringUtils.rPadSpaces(vSubCtlNum, 50));
		e015Rec.append(StringUtils.rPadSpaces(vMedRecNum, 50));
		e015Rec.append(StringUtils.rPadSpaces(vMedVerNum, 50));
		e015Rec.append(StringUtils.rPadSpaces(vPreAuth2, 50));
		e015Rec.append(StringUtils.rPadSpaces(referralNumber1, 50));
		e015Rec.append(StringUtils.rPadSpaces(referralNumber2, 50));
		e015Rec.append(StringUtils.rPadSpaces(autoAccidentState, 50));
		traceLOGGER.debug("E015Rec: " + e015Rec.toString());
		vFile.println(e015Rec);
		e015Rec.delete(0, e015Rec.length());
	}
	

	protected void writeE035Rec(DblMdclEqpCert dblMdclEqpCert, PrintWriter vFile) {
		final StringBuffer e035Rec = new StringBuffer("E035");
		e035Rec.append(StringUtils.rPadSpaces(dblMdclEqpCert.getCr301certTypeCd() + "", 1));  //Certification Type Code
		e035Rec.append(StringUtils.rPadSpaces(dblMdclEqpCert.getCr302uomCd(), 2));	//Certification Unit of Time
		e035Rec.append(StringUtils.rPadSpaces(dblMdclEqpCert.getCr303dmeDrtn(), 9)); //Time Period of Cert (months)
		e035Rec.append(StringUtils.rPadSpaces("", 1)); //5010: Insulin Dep Code - not part of 5010 database 
		e035Rec.append(StringUtils.rPadSpaces("", 1));//5010: PAT-PROGNOSIS - not part of 5010 
		traceLOGGER.debug("E035Rec: " + e035Rec.toString());
		vFile.println(e035Rec);
		e035Rec.delete(0, e035Rec.length());
	}
	
  
  

	/**
	 * Returns a string that represents health care codes.
	 * 
	 * @param vHiQual
	 *            - qualifier
	 * @param vHiCd
	 *            code
	 * @param inHiDt
	 *            code date
	 * @param vHiAmt
	 *            health care code amount
	 * @return
	 */

	
	protected String writeHiRec(String vHiQual, String vHiCd, String vHiCndtnResponseCd, String inHiDt,	BigDecimal vHiAmt, BigDecimal vHiQnty) {
		final StringBuffer vHiRec = new StringBuffer("");

		vHiRec.append(StringUtils.rPadSpaces(vHiQual, 3));
		vHiRec.append(StringUtils.rPadSpaces(vHiCd, 30)); //5010 c: 8-30
		//summaryLOGGER.info ("Adding in the condition response code, length is: " + vHiRec.length());
		vHiRec.append(StringUtils.rPadSpaces(vHiCndtnResponseCd,1));
		//summaryLOGGER.info ("Record is now length: " + vHiRec.length());
		String vHiDt = inHiDt;
		if (vHiDt.length() > 8) {
			vHiDt = vHiDt.substring(0, 8) + vHiDt.substring(9, 17);  //HC Code Date 1 - Date2
		}
		vHiRec.append(StringUtils.rPadSpaces(vHiDt, 16)); 		

		if (vHiAmt.signum() == 1) {  //TWP are we still only outputting positive amounts?
			vHiRec.append(padDecimal(vHiAmt, 22));   //5010 C:18-22
		} else {
			vHiRec.append(StringUtils.rPadSpaces("", 22));   //5010 C:18-22
		}
		
		if (vHiQnty != null && vHiQnty.signum() == 1) {
			vHiRec.append(padDecimal(vHiQnty, 22));   
		} else {
			vHiRec.append(StringUtils.rPadSpaces("", 22));  
		}	

		return vHiRec.toString();
	}

	protected void writeHiRecs(String e100, String e105, String e110,
			PrintWriter vFile) {
		vFile.println(e100);
		if (e105.length() > 4) {
			vFile.println(e105);
		}
		if (e110.length() > 4) {

			vFile.println(e110);
		}
	}

	/**
	 * Writes Health Care Pricing information into the extract file.
	 * 
	 * @param rowClaimPrcReprc
	 *            - pricing information
	 * @param vFile
	 *            - the extract file to write the data to
	 */
	protected void writeE125Rec(ClaimPrcReprc rowClaimPrcReprc,
			PrintWriter vFile) {
		final StringBuffer e125Rec = new StringBuffer("E125");
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc
				.getHcp01prcMthdlg(), 2));
		e125Rec.append(padDecimal(rowClaimPrcReprc.getHcp02alowAmt(), 18));
		e125Rec.append(padDecimal(rowClaimPrcReprc.getHcp03svgAmt(), 18));
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc
				.getHcp04rprcOrgId(), 50));  //5010: C:30-50
		e125Rec.append(padDecimal(rowClaimPrcReprc.getHcp05prcRt(), 9));
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc
				.getHcp06aprvDrgCd(), 50));	//5010: C 10-50
		e125Rec.append(padDecimal(rowClaimPrcReprc.getHcp07aprvDrgAmt(), 18));
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc
				.getHcp08aprvRevCd(), 10));
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc
				.getHcp09svcIdQual(), 2));
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc
				.getHcp10aprvProcCd(), 15));
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp11uomCd(),
				2));
		e125Rec.append(StringUtils.lPadZeros(rowClaimPrcReprc
				.getHcp12aprvSvcUnit(), 10));
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc
				.getHcp13rejRsnCd(), 2));
		e125Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc
				.getHcp14plcyCmplCd(), 2));
		e125Rec.append(StringUtils.rPadSpaces(
				rowClaimPrcReprc.getHcp15excpCd(), 2));
		traceLOGGER.debug("E125Rec: " + e125Rec.toString());
		vFile.println(e125Rec.toString());
		e125Rec.delete(0, e125Rec.length());
	}

	/**
	 * Writes information about the location of services to the extract file.
	 * 
	 * @param rowEntityName
	 *            - name of the location
	 * @param rowEntityAddress
	 *            - address of the services location
	 * @param vType
	 *            - type of record to write to the file - controls the first
	 *            data element written to the extract file
	 * @param vFile
	 *            - the extract file to write the data to
	 */
	protected void writeSvcLocRec(EntityName rowEntityName,
			EntityAddress rowEntityAddress, String vType, PrintWriter vFile) {
		final StringBuffer vSvcLocRec = new StringBuffer();
		vSvcLocRec.append(StringUtils.rPadSpaces(
				rowEntityName.getNm103lastNm(), 60)); //service location name 5010: C:30-50
		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityName.getNm104fstNm(),	35)); //service location name 5010: C:20-35

		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityName.getNm105midNm(),	25)); //service location name 5010: C:20-25
		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityName.getNm109idCd(),	30));//service location TIN 5010: C:15-30

		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityAddress.getN301adrLn1(), 55)); //service location addr1 5010: C:30-55
		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityAddress.getN302adrLn2(), 55)); //service location addr2 5010: C:30-55
		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityAddress.getN401city(), 30)); //service location city 5010: C:20-30
		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityAddress.getN402st(), 2));
		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityAddress.getN403postalCd(), 15)); //service location zip 5010: C:11-15
		vSvcLocRec.append(StringUtils.rPadSpaces(rowEntityAddress.getN404ctryCd(), 3));
		
		if (vType.equalsIgnoreCase("CLAIM")) {
			
			vSvcLocRec.append(StringUtils.rPadSpaces("", 15));  //Service location taxonomy code					
			vFile.println("E205" + vSvcLocRec.toString());
		} else {
			traceLOGGER.debug("SvcLocRec: " + vSvcLocRec.toString());
			vFile.println("F032" + vSvcLocRec.toString());
		}

		vSvcLocRec.delete(0, vSvcLocRec.length());
	}
	
	

	/**
	 * Writes information about a physician to the extract file.
	 * 
	 * @param rowEntityName
	 *            Physician's name
	 * @param vTaxonomyCd
	 *            - Physician's specialty code
	 * @param vType
	 *            - type of record to write to the file - controls the first
	 *            data element written to the extract file
	 * @param vFile
	 *            the extract file to write the data to
	 */
	protected void writePhyRec(EntityName rowEntityName, String vTaxonomyCd,
			String vType, PrintWriter vFile) {
		final StringBuffer vPhyRec = new StringBuffer();
		vPhyRec.append(StringUtils.rPadSpaces(rowEntityName.getNm103lastNm(),
				60)); //5010: c:30-60
		vPhyRec.append(StringUtils
				.rPadSpaces(rowEntityName.getNm104fstNm(), 35)); //5010: c:20-35
		vPhyRec.append(StringUtils
				.rPadSpaces(rowEntityName.getNm105midNm(), 25)); //5010: c:20-25
		
		vPhyRec.append(StringUtils.rPadSpaces(rowEntityName.getNm109idCd(), 30));

		TreeSet<String> excludeTaxonomyCodeList = new TreeSet<String>();
		excludeTaxonomyCodeList.add("F035");
		excludeTaxonomyCodeList.add("F036");
		excludeTaxonomyCodeList.add("E207");
		excludeTaxonomyCodeList.add("E211");		
		if(!excludeTaxonomyCodeList.contains(vType))
		{		
			vPhyRec.append(StringUtils.rPadSpaces(vTaxonomyCd, 50)); //5010: c:15-50
		}
		traceLOGGER.debug("PhyRec: " + vPhyRec.toString());
		vFile.println(vType + vPhyRec.toString());
		vPhyRec.delete(0, vPhyRec.length());
	}

	/**
	 * Writes Other Insurance Info.-Claim Related information to the extract
	 * file.
	 * 
	 * @param rowSubscriberInfo
	 * @param elementsE210
	 * @param vFile
	 *            the extract file to write the data to
	 */
	protected void writeE210Rec(SubscriberInfo rowSubscriberInfo,
			ArrayList elementsE210, PrintWriter vFile) {
		final StringBuffer e210Rec = new StringBuffer("E210");
		e210Rec.append(rPadSpaces(rowSubscriberInfo.getSbr01pyrRespSeq(), 1));
		e210Rec.append(StringUtils.rPadSpaces(rowSubscriberInfo
				.getSbr05insTypeCd(), 3));
		e210Rec.append(StringUtils.rPadSpaces(rowSubscriberInfo
				.getSbr09clmFileInd(), 2));
		

		
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(0), 18)); //ohi paid
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(1), 18)); //amount due
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(2), 18)); //patient paid amount
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(3), 18)); //ohi total submitted
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(4), 18)); //ohi allowed amount
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(5), 18)); //total denied
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(6), 18)); //non covered
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(7), 18)); //ohi approved amount
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(8), 18)); //ohi discount amount
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(9), 18)); //ohi covered amount
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(10), 18)); //patient responsibility amount
		e210Rec
				.append(StringUtils
						.rPadSpaces((String) elementsE210.get(11), 1));  //OHI claim assignment indicator

		/*** TWP Changes Start For Crossover Part A/B***/
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(12), 18)); //paid
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(13), 18)); //100%
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(14), 18)); //80%
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(15), 18)); //A
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(16), 18)); //B
		/*** TWP Changes For Crossover Part A/B End ***/
		
		e210Rec.append(padDecimal((BigDecimal) elementsE210.get(17), 18)); //remaining patient liability 

		traceLOGGER.debug("E210Rec: " + e210Rec.toString());
		vFile.println(e210Rec.toString());
		e210Rec.delete(0, e210Rec.length());
	}

	/**
	 * Writes Medicare Inpatient Adjudication Info to the extract file.
	 * 
	 * @param rowMediInptAdjdct
	 *            - medicare adjudication information
	 * @param vFile
	 *            - the extract file to write the data to
	 */
	protected void writeE214Rec(MediInptAdjdct rowMediInptAdjdct,
			PrintWriter vFile) {
		final StringBuffer e214Rec = new StringBuffer("E214");

    // 5010 Bug 51: Changed lPadBigDecimalZeros to rPadSpaces
		e214Rec.append(StringUtils.rPadSpaces(rowMediInptAdjdct.getMia01cvrVisits(), 8));//5010:added

    // 5010 Bug 51: Changed lPadBigDecimalZeros to rPadSpaces
		e214Rec.append(StringUtils.rPadSpaces(rowMediInptAdjdct.getMia02ltRsvDays(), 8));//5010:added

    // 5010 Bug 51: Changed lPadBigDecimalZeros to rPadSpaces
		e214Rec.append(StringUtils.rPadSpaces(rowMediInptAdjdct.getMia03ltPsychDays(), 8));//5010:added

		// 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia04drgAmt(), 12));//5010:added
		
		e214Rec.append(StringUtils.rPadSpaces(rowMediInptAdjdct.getMia05rmkCd1(), 50)); //5010" C: 8-50
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia06dsprpntAmt(), 12)); //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia07mspPthruAmt(), 12));//5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia08ppsCptlAmt(), 12)); //5010:added

    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia09cptlFspDrg(), 12)); //5010:added

    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia10cptlHspDrg(), 12)); //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia11cptlDhsDrg(), 12)); //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia12oldCptlAmt(), 12)); //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia13cptlIme(), 12));    //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia14operHsp(), 12));    //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia15cstRptDays(), 8));  //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia16operFsp(), 12));    //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia17cptlOtlr(), 12));   //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia18idirTchAmt(), 12)); //5010:added
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e214Rec.append(padDecimal(rowMediInptAdjdct.getMia19prfsnCpmt(), 12));  //5010:added
		
		e214Rec.append(StringUtils.rPadSpaces(rowMediInptAdjdct
				.getMia20rmkCd2(), 50)); //5010" C: 8-50
		e214Rec.append(StringUtils.rPadSpaces(rowMediInptAdjdct
				.getMia21rmkCd3(), 50)); //5010" C: 8-50
		e214Rec.append(StringUtils.rPadSpaces(rowMediInptAdjdct
				.getMia22rmkCd4(), 50)); //5010" C: 8-50
		e214Rec.append(StringUtils.rPadSpaces(rowMediInptAdjdct
				.getMia23rmkCd5(), 50)); //5010" C: 8-50
		e214Rec.append(StringUtils.rPadSpaces("", 12));
		traceLOGGER.debug("E214Rec: " + e214Rec.toString());
		vFile.println(e214Rec.toString());
		e214Rec.delete(0, e214Rec.length());
	}

	/**
	 * Writes Description: Medicare Outpatient Adjudication Info to the extract
	 * file.
	 * 
	 * @param rowMediOutptAdjdct
	 * @param vFile
	 */
	protected void writeE216Rec(MediOutptAdjdct rowMediOutptAdjdct,
			PrintWriter vFile) {
		final StringBuffer e216Rec = new StringBuffer("E216");
		
		// 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e216Rec.append(padDecimal(rowMediOutptAdjdct.getMoa01rmbrsRt(), 10)); // C: 8-10 added for 5010

    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
    e216Rec.append(padDecimal(rowMediOutptAdjdct.getMoa02hcpcsPybl(), 18)); //5010: added
		
    e216Rec.append(StringUtils.rPadSpaces(rowMediOutptAdjdct.getMoa03rmkCd1(), 50));//5010" C: 8-50
		e216Rec.append(StringUtils.rPadSpaces(rowMediOutptAdjdct.getMoa04rmkCd2(), 50));//5010" C: 8-50
		e216Rec.append(StringUtils.rPadSpaces(rowMediOutptAdjdct.getMoa05rmkCd3(), 50));//5010" C: 8-50
		e216Rec.append(StringUtils.rPadSpaces(rowMediOutptAdjdct.getMoa06rmkCd4(), 50));//5010" C: 8-50
		e216Rec.append(StringUtils.rPadSpaces(rowMediOutptAdjdct.getMoa07rmkCd5(), 50));//5010" C: 8-50
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e216Rec.append(padDecimal(rowMediOutptAdjdct.getMoa08esrdPymtAmt(), 18));
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		e216Rec.append(padDecimal(rowMediOutptAdjdct.getMoa09prfsnCpmt(), 18));
		
		traceLOGGER.debug("E216Rec: " + e216Rec.toString());
		vFile.println(e216Rec.toString());
		e216Rec.delete(0, e216Rec.length());
	}

	/**
	 * Writes case information to the extract file.
	 * 
	 * @param rowCases
	 * @param vType
	 * @param vFile
	 */
	protected void writeCasesRec(Cases rowCases, String vType, PrintWriter vFile) {
		final StringBuffer vCasesRec = new StringBuffer(vType);
		vCasesRec.append(StringUtils
				.rPadSpaces(rowCases.getCas01clmAdjGrp(), 2)); //Group Code
		vCasesRec.append(StringUtils.rPadSpaces(rowCases.getCas02clmAdjRsn1(),
				5)); //Reason Code 1
		vCasesRec.append(padDecimal(rowCases.getCas03adjAmt1(), 18)); //Reason Amount
		vCasesRec.append(padDecimal(rowCases.getCas04adjQty1(), 15)); //Reason Quantity 5010: C: 10-15
		vCasesRec.append(StringUtils.rPadSpaces(rowCases.getCas05clmAdjRsn2(),
				5));
		vCasesRec.append(padDecimal(rowCases.getCas06adjAmt2(), 18));
		vCasesRec.append(padDecimal(rowCases.getCas07adjQty2(), 15));//Reason Quantity 5010: C: 10-15
		vCasesRec.append(StringUtils.rPadSpaces(rowCases.getCas08clmAdjRsn3(),
				5));
		vCasesRec.append(padDecimal(rowCases.getCas09adjAmt3(), 18));
		vCasesRec.append(padDecimal(rowCases.getCas10adjQty3(), 15));//Reason Quantity 5010: C: 10-15
		vCasesRec.append(StringUtils.rPadSpaces(rowCases.getCas11clmAdjRsn4(),
				5));
		vCasesRec.append(padDecimal(rowCases.getCas12adjAmt4(), 18));
		vCasesRec.append(padDecimal(rowCases.getCas13adjQty4(), 15));//Reason Quantity 5010: C: 10-15
		vCasesRec.append(StringUtils.rPadSpaces(rowCases.getCas14clmAdjRsn5(),
				5));
		vCasesRec.append(padDecimal(rowCases.getCas15adjAmt5(), 18));
		vCasesRec.append(padDecimal(rowCases.getCas16adjQty5(), 15));//Reason Quantity 5010: C: 10-15
		vCasesRec.append(StringUtils.rPadSpaces(rowCases.getCas17clmAdjRsn6(),
				5));
		vCasesRec.append(padDecimal(rowCases.getCas18adjAmt6(), 18));
		vCasesRec.append(padDecimal(rowCases.getCas19adjQty6(), 15));//Reason Quantity 5010: C: 10-15
		traceLOGGER.debug("CasesRec: " + vCasesRec.toString());
		vFile.println(vCasesRec.toString());
		vCasesRec.delete(0, vCasesRec.length());
	}

	/**
	 * Writes Description: Other Insurance-General Info to the extract file.
	 * 
	 * @param rowEntityName
	 * @param vOhiDate
	 * @param vFile
	 */
	protected void writeE225Rec(EntityName rowEntityName, String vOhiDate,
			PrintWriter vFile) {
		final StringBuffer e225Rec = new StringBuffer("E225");
		e225Rec.append(StringUtils.rPadSpaces(rowEntityName.getNm103lastNm(),
				60)); //5010: C: 30-60
		e225Rec
				.append(StringUtils
						.rPadSpaces(rowEntityName.getNm109idCd(), 80)); //5010: C: 15-80
		e225Rec.append(StringUtils.rPadSpaces(vOhiDate, 8));
		e225Rec.append(StringUtils.rPadSpaces("", 80));		
		traceLOGGER.debug("E225Rec: " + e225Rec.toString());
		vFile.println(e225Rec.toString());
		e225Rec.delete(0, e225Rec.length());
	}

	/**
	 * Writes a Professional Service Line to the extract file.
	 * 
	 * @param rowPrfsnSvcLine
	 * @param elementsF000
	 * @param vFile
	 */
	protected void writeF000Rec(PrfsnSvcLine rowPrfsnSvcLine,
			ArrayList elementsF000, PrintWriter vFile) {
		final StringBuffer f000Rec = new StringBuffer("F000");
		f000Rec.append(StringUtils.lPadZeros(
				rowPrfsnSvcLine.getLx01asgnLnItm(), 6));
		f000Rec.append(StringUtils.rPadSpaces(rowPrfsnSvcLine
				.getSv101procCdQual(), 2));
		f000Rec.append(StringUtils.rPadSpaces(rowPrfsnSvcLine.getSv101procCd(),
				48)); //5010: C: 15-48
		f000Rec.append(StringUtils.rPadSpaces(rowPrfsnSvcLine
				.getSv101procMod1(), 2));
		f000Rec.append(StringUtils.rPadSpaces(rowPrfsnSvcLine
				.getSv101procMod2(), 2));
		f000Rec.append(StringUtils.rPadSpaces(rowPrfsnSvcLine
				.getSv101procMod3(), 2));
		f000Rec.append(StringUtils.rPadSpaces(rowPrfsnSvcLine
				.getSv101procMod4(), 2));
		f000Rec.append(padDecimal(rowPrfsnSvcLine.getSv102lineItmChrg(), 18));
		f000Rec.append(StringUtils.rPadSpaces(rowPrfsnSvcLine.getSv103uomCd(),
				2));
		f000Rec.append(padDecimal(rowPrfsnSvcLine.getSv104svcUnitCnt(), 15)); //5010: C: 10-15
		f000Rec.append(StringUtils.rPadSpaces(rowPrfsnSvcLine
				.getSv105placeSvcCd(), 2));
		f000Rec.append(StringUtils.rPadSpaces("", 2));
		
		
		f000Rec.append(StringUtils.lPadZerosRPadSpaces(rowPrfsnSvcLine
				.getSv107diagCdPtr1(), 2, 2));//5010: C: 10-2
		f000Rec.append(StringUtils.lPadZerosRPadSpaces(rowPrfsnSvcLine
				.getSv107diagCdPtr2(), 2, 2));//5010: C: 10-2
		f000Rec.append(StringUtils.lPadZerosRPadSpaces(rowPrfsnSvcLine
				.getSv107diagCdPtr3(), 2, 2));//5010: C: 10-2
		f000Rec.append(StringUtils.lPadZerosRPadSpaces(rowPrfsnSvcLine
				.getSv107diagCdPtr4(), 2, 2));//5010: C: 10-2
		
		
		f000Rec.append(StringUtils.rPadSpaces("", 10));
		f000Rec.append(StringUtils.rPadSpaces("", 10));
		f000Rec.append(StringUtils.rPadSpaces((String) elementsF000.get(0), 8));  //service date from
		f000Rec.append(StringUtils.rPadSpaces((String) elementsF000.get(1), 8)); //to
		f000Rec
				.append(StringUtils
						.rPadSpaces((String) elementsF000.get(2), 50)); //line item control number //5010: C: 30-50 
		f000Rec
				.append(StringUtils
						.rPadSpaces((String) elementsF000.get(3), 50)); //preauth number //5010: C: 30-50
		f000Rec.append(padDecimal((BigDecimal) elementsF000.get(4), 18)); //svc line sales tax
		f000Rec.append(padDecimal((BigDecimal) elementsF000.get(5), 18)); //svc line postage amnt
		f000Rec.append(padDecimal((BigDecimal) elementsF000.get(6), 18)); //ohi approved amount
		f000Rec.append(StringUtils.rPadSpaces((String) elementsF000.get(7), 8)); //begin therapy date 
		f000Rec.append(StringUtils.rPadSpaces((String) elementsF000.get(8), 8)); //last cert date
		f000Rec.append(StringUtils.rPadSpaces((String) elementsF000.get(9), 8)); //initial treatment date
		traceLOGGER.debug("F000Rec: " + f000Rec.toString());
		vFile.println(f000Rec.toString());
		f000Rec.delete(0, f000Rec.length());
	}

	/**
	 * Writes a Institutional Service Line to the extract file.
	 * 
	 * @param rowInstSvcLine
	 * @param elementsF005
	 * @param vFile
	 */
	protected void writeF005Rec(InstSvcLine rowInstSvcLine,
			ArrayList elementsF005, PrintWriter vFile) {
		final StringBuffer f005Rec = new StringBuffer("F005");
		f005Rec.append(StringUtils.lPadZeros(rowInstSvcLine.getLx01asgnLnNbr(),
				6));  //Service Line Number
		f005Rec.append(StringUtils.rPadSpaces(rowInstSvcLine
				.getSv201svcLnRevCd(), 48)); //5010: C: 10-48 Revenue Code
		f005Rec.append(StringUtils.rPadSpaces(rowInstSvcLine
				.getSv202svcIdQual(), 2)); //Procedure Qualifer
		f005Rec.append(StringUtils.rPadSpaces(rowInstSvcLine.getSv202procCd(),
				48)); //5010: C 15-48 Procedure code
		f005Rec.append(StringUtils.rPadSpaces(
				rowInstSvcLine.getSv202procMod1(), 2));
		f005Rec.append(StringUtils.rPadSpaces(
				rowInstSvcLine.getSv202procMod2(), 2));
		f005Rec.append(padDecimal(rowInstSvcLine.getSv203lnItmChrg(), 18));  //billed charge for service line
		f005Rec.append(StringUtils
				.rPadSpaces(rowInstSvcLine.getSv204uomCd(), 2));  //unit qualifier
		f005Rec.append(padDecimal(rowInstSvcLine.getSv205svcUnitCnt(), 15)); //5010: C 10-15
		f005Rec.append(padDecimal(rowInstSvcLine.getSv206svcLnRt(), 18));  //Accommodation rate
		f005Rec.append(padDecimal(rowInstSvcLine.getSv207denyChrgAmt(), 18)); //non-covered
		f005Rec.append(StringUtils.rPadSpaces("", 10));  //hdr line id
		f005Rec.append(StringUtils.rPadSpaces("", 10)); //hdr line date
		f005Rec.append(StringUtils.rPadSpaces((String) elementsF005.get(0), 8)); //service from
		f005Rec.append(StringUtils.rPadSpaces((String) elementsF005.get(1), 8)); //service to
		f005Rec.append(StringUtils.rPadSpaces("", 25));  //service description
		f005Rec.append(StringUtils.rPadSpaces("", 50)); //line item control number
		f005Rec.append(padDecimal((BigDecimal) elementsF005.get(2), 18)); //service line sales tax
		f005Rec.append(padDecimal((BigDecimal) elementsF005.get(3), 18)); //facility tax amount
		f005Rec.append(StringUtils.rPadSpaces(
				rowInstSvcLine.getSv202procMod3(), 2));  //hc code mod 3
		f005Rec.append(StringUtils.rPadSpaces(
				rowInstSvcLine.getSv202procMod4(), 2));  //hc code mod 3		
		traceLOGGER.debug("F005Rec: " + f005Rec.toString());
		vFile.println(f005Rec.toString());
		f005Rec.delete(0, f005Rec.length());
	}

	/**
	 * Writes Dental Service Line Info to the extract file
	 * 
	 * @param rowDentalSvcLine
	 * @param elementsF006
	 * @param vFile
	 */
	protected void writeF006Rec(DentalSvcLine rowDentalSvcLine,
			ArrayList elementsF006, PrintWriter vFile) {
		final StringBuffer f006Rec = new StringBuffer("F006");
		f006Rec.append(StringUtils.lPadZeros(rowDentalSvcLine
				.getLx01asgnLnItm(), 6));
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine
				.getSv301adaCdQual(), 2));
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine.getSv301adaCd(),
				48));  //5010: C: 15-48
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine
				.getSv301procMod1(), 2));
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine
				.getSv301procMod2(), 2));
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine
				.getSv301procMod3(), 2));
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine
				.getSv301procMod4(), 2));
		f006Rec.append(padDecimal(rowDentalSvcLine.getSv302lnItmChrg(), 18));
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine.getSv303fclyTypeCd(), 2));
		f006Rec.append(padDecimal(rowDentalSvcLine.getSv306procCnt(), 15)); //5010: C:10-15
		f006Rec.append(StringUtils.rPadSpaces((String) elementsF006.get(0), 8)); //service from date
		f006Rec.append(StringUtils.rPadSpaces((String) elementsF006.get(1), 8)); //service to date
		f006Rec.append(StringUtils.rPadSpaces((String) elementsF006.get(2), 50)); //5010" c:30-50  line item ctrl number
		f006Rec.append(padDecimal((BigDecimal) elementsF006.get(3), 18)); //ohi approved
		f006Rec.append(padDecimal((BigDecimal) elementsF006.get(4), 18)); //svc line sales tax
		f006Rec.append(StringUtils.rPadSpaces("", 10)); //hdr line identifier
		f006Rec.append(StringUtils.rPadSpaces("", 10));//hdr line date identifier
		f006Rec.append(StringUtils.rPadSpaces((String) elementsF006.get(5), 50)); //5010 c:20-50  Preauth #
		
		//New fields for 5010		
		Byte diagPtr1 = rowDentalSvcLine.getSv311diagPntr1();
		Byte diagPtr2 = rowDentalSvcLine.getSv311diagPntr2();
		Byte diagPtr3 = rowDentalSvcLine.getSv311diagPntr3();
		Byte diagPtr4 = rowDentalSvcLine.getSv311diagPntr4();
		
		f006Rec.append(StringUtils.rPadSpaces((diagPtr1 != null) ? diagPtr1.toString() : "", 2));  //diag code pointer 1
		f006Rec.append(StringUtils.rPadSpaces((diagPtr2 != null) ? diagPtr2.toString() : "", 2));  //diag code pointer 2
		f006Rec.append(StringUtils.rPadSpaces((diagPtr3 != null) ? diagPtr3.toString() : "", 2));  //diag code pointer 3
		f006Rec.append(StringUtils.rPadSpaces((diagPtr4 != null) ? diagPtr4.toString() : "", 2));  //diag code pointer 4
		
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine.getSv304oralDsgt1(), 3));  //oral dsg 1
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine.getSv304oralDsgt2(), 3));  //oral dsg 2
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine.getSv304oralDsgt3(), 3));  //oral dsg 3
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine.getSv304oralDsgt4(), 3));  //oral dsg 4
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine.getSv304oralDsgt5(), 3));  //oral dsg 5
		f006Rec.append(StringUtils.rPadSpaces(rowDentalSvcLine.getSv305plcmtCd(),1));  //prosthesis crown or inlay
		
		traceLOGGER.debug("F006Rec: " + f006Rec.toString());
		vFile.println(f006Rec.toString());
		f006Rec.delete(0, f006Rec.length());
	}

	/**
	 * Writes tooth information to the extract file.
	 * 
	 * @param rowToothInfo
	 * @param vFile
	 */
	protected void writeF007Rec(ToothInformation rowToothInfo, PrintWriter vFile) {
		final StringBuffer f007Rec = new StringBuffer("F007");
		f007Rec.append(StringUtils
				.rPadSpaces(rowToothInfo.getToo02tthNbr(), 30)); //Dental tooth number 5010: C:15-30
		f007Rec.append(StringUtils.rPadSpaces(
				rowToothInfo.getToo03tthSrfcCd1(), 2));
		f007Rec.append(StringUtils.rPadSpaces(
				rowToothInfo.getToo03tthSrfcCd2(), 2));
		f007Rec.append(StringUtils.rPadSpaces(
				rowToothInfo.getToo03tthSrfcCd3(), 2));
		f007Rec.append(StringUtils.rPadSpaces(
				rowToothInfo.getToo03tthSrfcCd4(), 2));
		f007Rec.append(StringUtils.rPadSpaces(
				rowToothInfo.getToo03tthSrfcCd5(), 2));
		traceLOGGER.debug("F007Rec: " + f007Rec.toString());
		vFile.println(f007Rec);
		f007Rec.delete(0, f007Rec.length());
	}

	/**
	 * Writes DME Service (Medical Equipment) information to the extract file.
	 * 
	 * @param rowDblMedEqpSvc
	 * @param vFile
	 */
	protected void writeF008Rec(DblMedEqpSvc rowDblMedEqpSvc, PrintWriter vFile) {
		final StringBuffer f008Rec = new StringBuffer("F008");
		f008Rec.append(StringUtils.rPadSpaces(rowDblMedEqpSvc.getSv501procId(),
				2));
		f008Rec.append(StringUtils.rPadSpaces(rowDblMedEqpSvc
				.getSv501procCode(), 48));  //5010: C: 15-48
		f008Rec.append(StringUtils.rPadSpaces(rowDblMedEqpSvc
				.getSv502measUnit(), 2));
		f008Rec.append(StringUtils.lPadZeros(rowDblMedEqpSvc
				.getSv503lnthMedNec(), 15));
		f008Rec.append(padDecimal(rowDblMedEqpSvc.getSv504rentalPrice(), 18));
		f008Rec.append(padDecimal(rowDblMedEqpSvc.getSv505purchPrice(), 18));
		f008Rec.append(rPadSpaces(rowDblMedEqpSvc.getSv506freqCode(), 1));
		traceLOGGER.debug("F008Rec: " + f008Rec.toString());
		vFile.println(f008Rec.toString());
		f008Rec.delete(0, f008Rec.length());
	}

	
	/**
	 * Writes Lvl Ambulance Cert.
	 * 
	 * @param rowDblMedEqpSvc
	 * @param vFile
	 */
	protected void writeF010Rec(AmblnTrnptInfo amblnTrnptInfo, ClaimQuantity claimQuantity, PrintWriter vFile) {
		final StringBuffer f010Rec = new StringBuffer("F010");
		
		f010Rec.append(StringUtils.rPadSpaces( amblnTrnptInfo.getCr103amblnTrnCd() ,1));//amb transport code
		f010Rec.append(StringUtils.rPadSpaces( amblnTrnptInfo.getCr104amblnTrnRsn(),1));//amb transport reason code
		f010Rec.append(StringUtils.rPadSpaces( amblnTrnptInfo.getCr105uomDist(),2));//transport distance code
		f010Rec.append(StringUtils.rPadSpaces( amblnTrnptInfo.getCr106trnspDist(),8));//transport distance
		f010Rec.append(StringUtils.rPadSpaces( amblnTrnptInfo.getCr102ptWt(),10));//patient weight at transport
		f010Rec.append(StringUtils.rPadSpaces( amblnTrnptInfo.getCr109rtrpPrps(),80));//round trip purpose
		f010Rec.append(StringUtils.rPadSpaces( amblnTrnptInfo.getCr110strchPrps(),80));//stretcher purpose
		if(claimQuantity != null)
		{
			f010Rec.append(StringUtils.rPadSpaces( claimQuantity.getQty01qtyQual(),2));//amb patient count qty qual
			f010Rec.append(StringUtils.rPadSpaces( claimQuantity.getQty02clmDaysCnt(),15));//amb patient count
		}else
		{
			f010Rec.append(StringUtils.rPadSpaces( "",2));//amb patient count qty qual
			f010Rec.append(StringUtils.rPadSpaces( "",15));//amb patient count			
		}
		traceLOGGER.debug("F010Rec: " + f010Rec.toString());
		vFile.println(f010Rec.toString());
		f010Rec.delete(0, f010Rec.length());
	}

	
	protected void writeF014Rec(DblMdclEqpCert dme, PrintWriter vFile) {
		final StringBuffer f014Rec = new StringBuffer("F014");
		f014Rec.append(StringUtils.rPadSpaces(dme.getCr301certTypeCd(), 1));
		f014Rec.append(StringUtils.rPadSpaces(dme.getCr302uomCd(), 2));
		f014Rec.append(padDecimal(dme.getCr303dmeDrtn(), 15));		
		traceLOGGER.debug("F014Rec: " + f014Rec.toString());
		vFile.println(f014Rec.toString());
		f014Rec.delete(0, f014Rec.length());
	}

	
	/**
	 * Writes Health Care Pricing information to the extract file. (this is at
	 * the line level)
	 * 
	 * @param rowClaimPrcReprc
	 * @param vFile
	 */
	protected void writeF020Rec(ClaimPrcReprc rowClaimPrcReprc,
			PrintWriter vFile) {
		final StringBuffer f020Rec = new StringBuffer("F020");
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp01prcMthdlg(), 2));
		f020Rec.append(padDecimal(rowClaimPrcReprc.getHcp02alowAmt(), 18));
		f020Rec.append(padDecimal(rowClaimPrcReprc.getHcp03svgAmt(), 18));
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp04rprcOrgId(), 50)); //5010: C:30-50
		f020Rec.append(padDecimal(rowClaimPrcReprc.getHcp05prcRt(), 9));
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp06aprvDrgCd(), 50)); //5010: C:10-50
		f020Rec.append(padDecimal(rowClaimPrcReprc.getHcp07aprvDrgAmt(), 18));
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp08aprvRevCd(), 48));//5010: C:30-48
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp09svcIdQual(), 2));
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp10aprvProcCd(), 48));//5010: C:15-50
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp11uomCd(),2));
		f020Rec.append(StringUtils.lPadZeros(rowClaimPrcReprc.getHcp12aprvSvcUnit(), 15));//5010: C:10-15
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp13rejRsnCd(), 2));
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp14plcyCmplCd(), 2));
		f020Rec.append(StringUtils.rPadSpaces(rowClaimPrcReprc.getHcp15excpCd(), 2));
		traceLOGGER.debug("F020Rec: " + f020Rec.toString());
		vFile.println(f020Rec.toString());
		f020Rec.delete(0, f020Rec.length());
	}

	/**
	 * Writes information about a Dispensed Drug to the extract file.
	 * 
	 * @param rowDrugPricing
	 * @param vNdcCd
	 * @param vRxNumber
	 * @param vFile
	 */
	protected void writeF021Rec(DrugPricing rowDrugPricing, String vNdcCd,
			String vRxNumber, PrintWriter vFile) {
		final StringBuffer f021Rec = new StringBuffer("F021");
		f021Rec.append(StringUtils.rPadSpaces(vNdcCd, 48)); //5010:C 13-48
		f021Rec.append(padDecimal(rowDrugPricing.getCtp03drugUnitPrc(), 17));
		f021Rec.append(padDecimal(rowDrugPricing.getCtp04drugUnitCnt(), 15));
		f021Rec.append(StringUtils.rPadSpaces(
				rowDrugPricing.getCtp05measUnit(), 2));
		f021Rec.append(StringUtils.rPadSpaces(vRxNumber, 50)); //5010: C 12-50
		traceLOGGER.debug("F021Rec: " + f021Rec.toString());
		vFile.println(f021Rec.toString());
		f021Rec.delete(0, f021Rec.length());
	}

	/**
	 * Writes OHI Service Line Information to the extract file
	 * 
	 * @param rowSvcLineAdjdct
	 * @param vSvcAdjDt
	 * @param vFile
	 */
	protected void writeF040Rec(SvcLineAdjdct rowSvcLineAdjdct,
			String vSvcAdjDt, BigDecimal vRemainingPatientAmount,
			PrintWriter vFile) {
		final StringBuffer f040Rec = new StringBuffer("F040");
		f040Rec.append(StringUtils.rPadSpaces(rowSvcLineAdjdct.getSvd01pyrId(),	80));//5010: C: 20-80
		f040Rec.append(padDecimal(rowSvcLineAdjdct.getSvd02svcLnPaid(), 18));
		f040Rec.append(StringUtils.rPadSpaces(rowSvcLineAdjdct.getSvd03svcIdQual(), 2));
		f040Rec.append(StringUtils.rPadSpaces(rowSvcLineAdjdct.getSvd03procCd(), 48));//5010: C: 15-48
		f040Rec.append(StringUtils.rPadSpaces(rowSvcLineAdjdct.getSvd03procMod1(), 2));
		f040Rec.append(StringUtils.rPadSpaces(rowSvcLineAdjdct.getSvd03procMod2(), 2));
		f040Rec.append(StringUtils.rPadSpaces(rowSvcLineAdjdct.getSvd03procMod3(), 2));
		f040Rec.append(StringUtils.rPadSpaces(rowSvcLineAdjdct.getSvd03procMod4(), 2));
		f040Rec.append(StringUtils.rPadSpaces("", 48));//5010: C: 30-48
		f040Rec.append(StringUtils.rPadSpaces(rowSvcLineAdjdct.getSvd04svcLnRevCd(), 10));
		f040Rec.append(padDecimal(rowSvcLineAdjdct.getSvd05adjQty(), 15)); //5010: C: 10-15
		f040Rec.append(StringUtils.lPadZeros(rowSvcLineAdjdct.getSvd06lineNbr(), 6)); //5010:C 10-6
		f040Rec.append(StringUtils.rPadSpaces(vSvcAdjDt, 8));
		
    // 5010 Bug 51: Changed lPadBigDecimalZeros to padDecimal
		f040Rec.append(padDecimal(vRemainingPatientAmount, 18));  //remaining patient amount
		
		traceLOGGER.debug("F040Rec: " + f040Rec.toString());
		vFile.println(f040Rec.toString());
		f040Rec.delete(0, f040Rec.length());
	}

	/**
	 * Writes information about Supporting Documentation (DMERC, Other) to the
	 * extract file.
	 * 
	 * @param rowFormLqId
	 * @param vFile
	 */
	protected void writeF060Rec(FormLqId rowFormLqId, PrintWriter vFile) {
		final StringBuffer f060Rec = new StringBuffer("F060");
		f060Rec.append(StringUtils.rPadSpaces(rowFormLqId.getLq02formId(), 3));
		f060Rec.append(StringUtils
				.rPadSpaces(rowFormLqId.getLq01formIdCd(), 30)); //5010: C 20-30
		traceLOGGER.debug("F060Rec: " + f060Rec.toString());
		vFile.println(f060Rec.toString());
		f060Rec.delete(0, f060Rec.length());
	}

	/**
	 * Writes information about Supporting Documentation (DMERC, Other) to the
	 * extract file.
	 * 
	 * @param rowSuptngDoc
	 * @param vFile
	 */
	protected void writeF065Rec(SuptngDoc rowSuptngDoc, PrintWriter vFile) {
		final StringBuffer f065Rec = new StringBuffer("F065");
		f065Rec.append(StringUtils.rPadSpaces(rowSuptngDoc.getFrm01questionId(), 20)); //5010: C 5-20
		f065Rec.append(rPadSpaces(rowSuptngDoc.getFrm02respYesno(), 1));
		f065Rec.append(StringUtils.rPadSpaces(rowSuptngDoc.getFrm03respTxt(),50)); //5010: C: 30-50
		f065Rec.append(StringUtils.rPadSpaces(rowSuptngDoc.getFrm04respDt(), 8));
		f065Rec.append(StringUtils.lPadZeros(rowSuptngDoc.getFrm05respPct(), 6));
		traceLOGGER.debug("F065Rec: " + f065Rec.toString());
		vFile.println(f065Rec);
	}

	/**
	 * Marks a claim built - this is called once the claim has been added to the
	 * extract file
	 * 
	 * @param rowClaim
	 * @param dbEM
	 */
	protected void updateClaim(Claim rowClaim, EntityManager dbEM) {
		rowClaim.setStatusCode("BUILT");
		listOfClaimsToPersist.add(rowClaim);
	}

	/**
	 * Returns a list of claims that have been added to the extract file.
	 * 
	 * @return
	 */
	public List getListOfClaimsToPersist() {
		return listOfClaimsToPersist;
	}

	/**
	 * Returns a string representing the "last first middle" name of the entity.
	 * 
	 * @param rowEntityName
	 * @return
	 */
	protected String getCnctName(EntityName rowEntityName) {
		final StringBuffer vReturnName = new StringBuffer();

		// Start with the last name
		String lastName = rowEntityName.getNm103lastNm();
		//if the pay to provider last name is null it is okay
		if (lastName == null && (!"2010AB".equalsIgnoreCase(rowEntityName.getLoopId()) || "2010AC".equalsIgnoreCase(rowEntityName.getLoopId())) ) {
			throw new RuntimeException("Contact Last Name was null");
		}
		
		if(lastName == null &&
				( 
				  "2010AB".equalsIgnoreCase(rowEntityName.getLoopId()) 
				  || "2010AC".equalsIgnoreCase(rowEntityName.getLoopId()))
				)
		{
			lastName = "";
		}
		
		vReturnName.append(StringUtils.rPadSpaces(lastName,60));

		// If there is a first name, use it, after a blank space
		final String firstName = rowEntityName.getNm104fstNm();
		if (firstName != null) {			
			vReturnName.append(StringUtils.rPadSpaces(firstName,35));
		}

		// If there is a middle name, use it, after a blank space
		final String middleName = rowEntityName.getNm105midNm();
		if (middleName != null) {			
			vReturnName.append(StringUtils.rPadSpaces(middleName,25));
		}				

		// Return only the first 35 chars 5010:changed 35 to 120
		return vReturnName.substring(0, Math.min(120, vReturnName.length()));

	} // end getCnctName

	/**
	 * Returns a string of the character passed in via the parameter c that is
	 * reqLen characters long.
	 * 
	 * @param c
	 * @param reqLen
	 * @return
	 */
	protected String rPadSpaces(Character c, int reqLen) {
		StringBuffer stringBuffer = new StringBuffer();
		if (c != null)
			stringBuffer.append(c);
		final StringBuffer spaces = new StringBuffer("");
		final int loopTimes = reqLen - stringBuffer.length();
		for (int i = 0; i < loopTimes; i++) {
			spaces.append(' ');
		}
		return spaces.toString() + stringBuffer.toString();
	}

	// Pad the given string with spaces, adding a decimal point and two decimal
	// places
	// if it does not have one. Zeros are turned into blanks. The string is not,
	// however, checked for numerically.
	// TODO: Make sure the input can be parsed as a number?
	protected String padDecimal(String padDec, int reqLen) {
		StringBuffer stringBuffer; // = new StringBuffer(stringToPad);

		// If the the input string is null or the single digit '0', make it a
		// blank string
		if (padDec == null || "0".equals(padDec)) {
			stringBuffer = new StringBuffer("");
		} else {
			stringBuffer = new StringBuffer(padDec);
			// If there is no decimal point in the input string, put one in, and
			// show
			// two places after it
			final int idxDecimal = stringBuffer.indexOf(".");
			if (idxDecimal < 0) {
				stringBuffer.append(".00");
			}
		}

		// Left-pad the result with spaces out to the given length
		final StringBuffer spaces = new StringBuffer("");
		final int loopTimes = reqLen - stringBuffer.length();
		for (int i = 0; i < loopTimes; i++) {
			spaces.append(' ');
		}
		return spaces.toString() + stringBuffer.toString();
	}

	// Pad the given decimal number with spaces.
	// This is done by turning it into a string, then calling the other method
	protected String padDecimal(BigDecimal padDec, int reqLen) {
		String inputAsString = null;
		if (padDec != null) {
			inputAsString = padDec.toString();
		}
		return padDecimal(inputAsString, reqLen);
	}

	protected String getContactInfoDatum(CntcInfo inCntcInfo, String inType) {
		String returnValue = "";

		// Pull any data out of field combination 3/4
		if (inType.equalsIgnoreCase(inCntcInfo.getPer03commQual1())) {
			returnValue = inCntcInfo.getPer04commNbr1();
		}

		// Pull any data out of field combination 5/6
		if (inType.equalsIgnoreCase(inCntcInfo.getPer05commQual2())) {
			returnValue = inCntcInfo.getPer06commNbr2();
		}

		// Pull any data out of field combination 7/8
		if (inType.equalsIgnoreCase(inCntcInfo.getPer07commQual3())) {
			returnValue = inCntcInfo.getPer08commNbr3();
		}

		return returnValue;
	}

	/**
	 * Gets a date associated with a claim at the claim level using the qulifier
	 * code.
	 * 
	 * @param vClaimKey
	 *            - claim key of the claim the date is associated with
	 * @param vDateQual
	 *            - X12 qualifier
	 * @param dbEM
	 * @return
	 */
	protected String getDate(String vClaimKey, String vDateQual,
			EntityManager dbEM) {
		// This logic returns the first instance of date associated with the
		// claim regardless of the number associated with it
		// using the qualifier.
		Query getBatchQuery = dbEM
				.createNativeQuery(
						"SELECT * FROM CLAIM_DT WHERE CLAIM_KEY=:claimKey and DTP01DTTM_QUAL=:dateQual",
						ClaimDt.class);
		getBatchQuery.setParameter("claimKey", vClaimKey);
		getBatchQuery.setParameter("dateQual", vDateQual);
		List batches = getBatchQuery.getResultList();
		Iterator batchIterator = batches.iterator();
		String vDate = null;
		if (batchIterator.hasNext()) {
			final ClaimDt rowClaimDt = (ClaimDt) batchIterator.next();
			vDate = rowClaimDt.getDtp03dtFld();
		}
		detach(batches, dbEM);
		return vDate;
	}

	/**
	 * Gets a data associated with a claim at the service line level
	 * 
	 * @param vClaimKey
	 *            - claim key
	 * @param vSvcKey
	 *            - service key
	 * @param vDateQual
	 *            - the X12 qualifier
	 * @param dbEM
	 * @return
	 */
	protected String getDate(String vClaimKey, String vSvcKey,
			String vDateQual, EntityManager dbEM) {

		// This logic returns the first instance of date associated with the
		// claim regardless of the number associated with it
		// using the qualifier.
		Query getClaimDts = dbEM.createNativeQuery(
				"SELECT * FROM CLAIM_DT WHERE CLAIM_KEY=:claimKey " + vSvcKey
						+ " and DTP01DTTM_QUAL=:dateQual", ClaimDt.class);
		getClaimDts.setParameter("claimKey", vClaimKey);
		getClaimDts.setParameter("dateQual", vDateQual);
		List claimDts = getClaimDts.getResultList();
		Iterator claimDtIterator = claimDts.iterator();
		String vDate = null;
		if (claimDtIterator.hasNext()) {
			final ClaimDt rowClaimDt = (ClaimDt) claimDtIterator.next();
			vDate = rowClaimDt.getDtp03dtFld();
		}
		detach(claimDts, dbEM);
		return vDate;
	}


	/**
	 * Gets an amount associated with the claim at the claim level using the
	 * qualifier passed in and the loopId.
	 * 
	 * @param vClaimKey
	 * @param vAmtQual
	 * @param dbEM
	 * @return
	 */
	protected BigDecimal getClaimAmtByLoopId(String vClaimKey, String vAmtQual, String loopId, EntityManager dbEM) {

		BigDecimal vAmt = null;
		Query getClaimAmountQuery = dbEM
				.createQuery("SELECT ca FROM gov.va.med.domain.erepos.ClaimAmount ca WHERE (ca.claim.claimKey=:claimKey AND ca.amt01amtQualCd=:amt01amtQualCd AND ca.loopId=:loopId)");
		getClaimAmountQuery.setParameter("claimKey", vClaimKey);
		getClaimAmountQuery.setParameter("amt01amtQualCd", StringUtils
				.rPadSpaces(vAmtQual, 3));
		getClaimAmountQuery.setParameter("loopId", loopId);
		List claimAmounts = getClaimAmountQuery.getResultList();
		Iterator claimAmountIterator = claimAmounts.iterator();
		if (claimAmountIterator.hasNext()) {
			final ClaimAmount rowClaimAmount = (ClaimAmount) claimAmountIterator.next();
			vAmt = rowClaimAmount.getAmt02amt();
		}
		
		detach(claimAmounts, dbEM);
		return vAmt;
	}

	
	/**
	 * Gets an amount associated with the claim at the claim level using the
	 * qualifier passed in.
	 * 
	 * @param vClaimKey
	 * @param vAmtQual
	 * @param dbEM
	 * @return
	 */
	protected BigDecimal getClaimAmt(String vClaimKey, String vAmtQual,
			EntityManager dbEM) {

		BigDecimal vAmt = null;

		// This logic returns the first instance of an amount associated with
		// the claim regardless of the number associated with it
		// using the qualifier passed in.
		Query getClaimAmountQuery = dbEM
				.createQuery("SELECT ca FROM gov.va.med.domain.erepos.ClaimAmount ca WHERE (ca.claim.claimKey=:claimKey AND ca.amt01amtQualCd=:amt01amtQualCd)");
		getClaimAmountQuery.setParameter("claimKey", vClaimKey);
		getClaimAmountQuery.setParameter("amt01amtQualCd", StringUtils
				.rPadSpaces(vAmtQual, 3));

		// Query getClaimAmountQuery = dbEM
		// .createNativeQuery("SELECT * FROM CLAIM_AMOUNT  WHERE (CLAIM_KEY='"+vClaimKey+"' and AMT01AMT_QUAL_CD='"+vAmtQual+"')",
		// ClaimAmount.class);

		List claimAmounts = getClaimAmountQuery.getResultList();
		Iterator claimAmountIterator = claimAmounts.iterator();
		if (claimAmountIterator.hasNext()) {
			final ClaimAmount rowClaimAmount = (ClaimAmount) claimAmountIterator
					.next();
			vAmt = rowClaimAmount.getAmt02amt();
		}
		detach(claimAmounts, dbEM);
		return vAmt;
	}

	/**
	 * Gets an amount associated with the claim at the service line level using
	 * the qualifier passed in.
	 * 
	 * @param vClaimKey
	 *            - claim key
	 * @param vSvcKey
	 *            - service key
	 * @param vDateQual
	 *            - the X12 qualifier
	 * @param dbEM
	 * @return
	 */
	protected BigDecimal getClaimAmt(String vClaimKey, String vSvcKey,
			String vAmtQual, EntityManager dbEM) {

		// This logic returns the first instance of an amount associated with
		// the service line regardless of the number associated with it
		// using the qualifier passed in.
		BigDecimal vAmt = null;
		Query getClaimAmountQuery = dbEM.createNativeQuery(
				"SELECT * FROM CLAIM_AMOUNT WHERE CLAIM_KEY=:claimKey "
						+ vSvcKey + " AND AMT01AMT_QUAL_CD=:amtQualCd",
				ClaimAmount.class);
		getClaimAmountQuery.setParameter("claimKey", vClaimKey);
		getClaimAmountQuery.setParameter("amtQualCd", StringUtils.rPadSpaces(
				vAmtQual, 3));
		List claimAmounts = getClaimAmountQuery.getResultList();
		Iterator claimAmountIterator = claimAmounts.iterator();
		if (claimAmountIterator.hasNext()) {
			final ClaimAmount rowClaimAmount = (ClaimAmount) claimAmountIterator
					.next();
			vAmt = rowClaimAmount.getAmt02amt();
		}
		detach(claimAmounts, dbEM);
		return vAmt;
	}

	/**
	 * Gets the entity address using the where clause passed in.
	 * 
	 * @param vWhereCls
	 *            dynamic where clause used to select the entity address
	 * @param dbEM
	 * @return an EntityAddress
	 */
	protected EntityAddress getEntityAddress(String vWhereCls,
			EntityManager dbEM) {

		// This logic returns the first entity address found.
		EntityAddress addressToReturn = null;
		Query getEntityAddressQuery = dbEM.createNativeQuery(
				"SELECT * FROM ENTITY_ADDRESS WHERE " + vWhereCls,
				EntityAddress.class);
		List entityAddresses = getEntityAddressQuery.getResultList();
		Iterator entityAddressesIterator = entityAddresses.iterator();
		String vDate = null;
		if (entityAddressesIterator.hasNext()) {
			addressToReturn = (EntityAddress) entityAddressesIterator.next();
		}
		return addressToReturn;
	}

	/**
	 * Gets the entity name using the where clause passed in.
	 * 
	 * @param vWhereCls
	 *            dynamic where clause used to select the entity name
	 * @param dbEM
	 * @return
	 */
	protected EntityName getEntityName(String vWhereCls, EntityManager dbEM) {

		// This logic returns the first entity name found.
		EntityName entityNameToReturn = null;
		Query getEntityNameQuery = dbEM.createNativeQuery(
				"SELECT * FROM ENTITY_NAME WHERE " + vWhereCls,
				EntityName.class);
		List entityNames = getEntityNameQuery.getResultList();
		Iterator entityNameIterator = entityNames.iterator();
		String vDate = null;
		if (entityNameIterator.hasNext()) {
			entityNameToReturn = (EntityName) entityNameIterator.next();
		}
		return entityNameToReturn;
	}

	/**
	 * Gets the taxonomy code (speciality info e.g. orthopedics) using the where
	 * clause passed in.
	 * 
	 * @param vTxWhereCls
	 *            - the criteria used to select the taxonomy code for a provider
	 * @param dbEM
	 * @return
	 */
	protected String getTaxonomyCd(String vTxWhereCls, EntityManager dbEM) {

		// This logic returns the first taxonomy code found.
		String vTaxonomyCd = null;
		Query getPrvSpclInfoQuery = dbEM.createNativeQuery(
				"SELECT * FROM PRV_SPCL_INFO WHERE " + vTxWhereCls,
				PrvSpclInfo.class);
		List prvSpclInfos = getPrvSpclInfoQuery.getResultList();
		Iterator prvSpclInfoIterator = prvSpclInfos.iterator();
		String vDate = null;
		if (prvSpclInfoIterator.hasNext()) {
			vTaxonomyCd = ((PrvSpclInfo) prvSpclInfoIterator.next())
					.getPrv03prvdSpcCd();
		}
		detach(prvSpclInfos, dbEM);
		return vTaxonomyCd;
	}

	/**
	 * Gets an additional reference value - like a tin,ssn,ein. Using the where
	 * clause passed in so the information can be be selected at the proper
	 * level (provider, claim, line level).
	 * 
	 * @param vWhereCls
	 *            - the selection criteria used to select the reference value.
	 * @param dbEM
	 *            - the EntityManager tied to the db
	 * @return the reference value selected
	 */
	protected String getRefValue(String vWhereCls, EntityManager dbEM) {

		// This logic returns the first reference value that meets the criteria.
		String vRefVal = null;
		Query getEntityAdtlIdQuery = dbEM.createNativeQuery(
				"SELECT * FROM ENTITY_ADTL_ID WHERE " + vWhereCls
						+ " ORDER BY SEQ_NBR", EntityAdtlId.class);
		List entityAdtlIds = getEntityAdtlIdQuery.getResultList();
		Iterator entityAdtlIdIterator = entityAdtlIds.iterator();
		String vDate = null;
		if (entityAdtlIdIterator.hasNext()) {
			final EntityAdtlId rowEntityAdtlId = (EntityAdtlId) entityAdtlIdIterator
					.next();
			vRefVal = rowEntityAdtlId.getRef02scndId();
			detach(rowEntityAdtlId, dbEM);
		}
		return vRefVal;
	}

	protected void writeARec(String vFileNumber, String vSt02txnCtlNbr,
			PrintWriter vOutputFile, EntityManager dbEM) {

		Query getIntchgControlQuery = dbEM
				.createQuery("SELECT i FROM gov.va.med.domain.erepos.IntchgControl i WHERE i.fileNumber=:fileNumber");
		getIntchgControlQuery.setParameter("fileNumber", vFileNumber);
		final IntchgControl rowIntchgControl = (IntchgControl) getIntchgControlQuery
				.getSingleResult();

		Query getFunctionalGroupQuery = dbEM
				.createQuery("SELECT f FROM gov.va.med.domain.erepos.FunctionalGroup f WHERE f.fileNumber=:fileNumber");
		getFunctionalGroupQuery.setParameter("fileNumber", vFileNumber);
		final FunctionalGroup rowFunctionalGroup = (FunctionalGroup) getFunctionalGroupQuery
				.getSingleResult();

		// Write the A005 Record
		writeA005Rec(rowIntchgControl, rowFunctionalGroup, vOutputFile);
		
		Query getTxnSetQuery = dbEM
		.createQuery("SELECT t FROM gov.va.med.domain.erepos.TxnSet t WHERE t.id.fileNumber=:fileNumber AND t.id.st02txnCtlNbr=:st02txnCtlNbr");
		getTxnSetQuery.setParameter("fileNumber", vFileNumber);
		getTxnSetQuery.setParameter("st02txnCtlNbr", vSt02txnCtlNbr);
		final TxnSet rowTxnSet = (TxnSet) getTxnSetQuery.getSingleResult();

		// Write the A010 Record		
		final String vTransmissionType = rowFunctionalGroup.getGs08verRlsIndId();
		writeA010Rec(vTransmissionType, rowTxnSet, vOutputFile);

		// Write the A015 Record
		final String vEntWhereClause = "FILE_NUMBER = '" + vFileNumber + "' "
				+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' ";
//		EntityName rowEntityName = getEntityName(vEntWhereClause
//				+ " AND LOOP_ID = '1000A' ", dbEM);   // 5010: Changes this line to include no101enty_id_cd='41'
		EntityName rowEntityName = getEntityName(vEntWhereClause
				+ " AND LOOP_ID = '1000A' AND NM101ENTY_ID_CD='41' ", dbEM);
		writeA015Rec(rowEntityName, vOutputFile);

		// Write the A020 Record
//		rowEntityName = getEntityName(vEntWhereClause
//				+ "AND LOOP_ID = '1000B' ", dbEM);
		rowEntityName = getEntityName(vEntWhereClause
				+ "AND LOOP_ID = '1000B' AND NM101ENTY_ID_CD='40' ", dbEM);
		writeA020Rec(rowEntityName, vOutputFile);

//		vOutputFile.flush();
	} // end writeARec

	protected void writeBRec(Claim rowClaim, PrintWriter vOutputFile, EntityManager dbEM) {
		EntityName rowEntityName = null;
		EntityAddress rowEntityAddress = null;
		PrvSpclInfo rowPrvSpclInfo = null;
		ForeignCurrency rowForeignCurrency = null;
		CntcInfo rowCntcInfo = null;
		String vPrvTaxonamyCd = "";
		String vCurrencyCd = "";
		String vPrvIdSec = "";
		String vFileNumber = rowClaim.getFileNumber(); 
		String vSt02txnCtlNbr = rowClaim.getSt02txnCtlNbr();
		String vBillinghrchlLevelKey = rowClaim.getBillingHrchlLevelKey();

		Query getPrvSpclInfosQuery = dbEM
				.createQuery("SELECT p FROM gov.va.med.domain.erepos.PrvSpclInfo p WHERE p.txnSet.id.fileNumber=:fileNumber AND p.txnSet.id.st02txnCtlNbr=:st02txnCtlNbr AND p.loopId=:loopId AND p.hrchlLevelKey=:hrchlLevelKey");
		getPrvSpclInfosQuery.setParameter("fileNumber", vFileNumber);
		getPrvSpclInfosQuery.setParameter("st02txnCtlNbr", vSt02txnCtlNbr);
		getPrvSpclInfosQuery.setParameter("loopId", "2000A");
		getPrvSpclInfosQuery.setParameter("hrchlLevelKey", vBillinghrchlLevelKey);		
		List prvSpclInfos = getPrvSpclInfosQuery.getResultList();
		Iterator prvSpclInfoIterator = prvSpclInfos.iterator();
		if (prvSpclInfoIterator.hasNext()) {
			rowPrvSpclInfo = (PrvSpclInfo) prvSpclInfoIterator.next();
			vPrvTaxonamyCd = rowPrvSpclInfo.getPrv03prvdSpcCd();
		}


		//5010 C: added  f.cur01crcyIdCd = '85' to the ForeignCurrency query
		Query getForeignCurrencyQuery = dbEM 
		.createQuery("SELECT f FROM gov.va.med.domain.erepos.ForeignCurrency f WHERE f.txnSet.id.fileNumber=:fileNumber AND f.txnSet.id.st02txnCtlNbr=:st02txnCtlNbr AND f.cur01crcyIdCd = '85' AND f.hrchlLevelKey=:hrchlLevelKey");
		getForeignCurrencyQuery.setParameter("fileNumber", vFileNumber);
		getForeignCurrencyQuery.setParameter("st02txnCtlNbr", vSt02txnCtlNbr);
		getForeignCurrencyQuery.setParameter("hrchlLevelKey", vBillinghrchlLevelKey);
		List foreignCurrencies = getForeignCurrencyQuery.getResultList();
		Iterator foreignCurrencyIterator = foreignCurrencies.iterator();
		if (foreignCurrencyIterator.hasNext()) {
			rowForeignCurrency = (ForeignCurrency) foreignCurrencyIterator
					.next();
			vCurrencyCd = rowForeignCurrency.getCur02crcyCd();			
		}

		//5010 C: added NM101ENTY_ID_CD = '85' to the ForeignCurrency query
		String vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' " + "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
		+ "AND LOOP_ID = '2010AA' AND  NM101ENTY_ID_CD='85' "  
		+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";
		 

		rowEntityName = getEntityName(vWhereCls, dbEM);
		String vName = getCnctName(rowEntityName);	
		
		
		String vNm109idCd_Tin = rowEntityName.getNm109idCd();

		/**
		 * DEV009309 - For professional claims billing provider TIN may be
		 * incorrectly selected There was a bit of a problem adding a qulaifier
		 * to the original query. Then same WHERE clause was being used against
		 * two different tables. The following should produce the desired
		 * results. Without concrete examples of the claims the problem was
		 * exposed in, its a little difficult ascertaining the correctness of
		 * the code change
		 */
		String vWhereClsRefValue = "FILE_NUMBER = '" + vFileNumber + "' "
				+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
				+ "AND LOOP_ID = '2010AA' "
				+ "AND REF01ID_QLFR IN ('EI', 'SY') " 
				+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";

		vPrvIdSec = getRefValue(vWhereClsRefValue, dbEM);
		
		//5010 C: could not match spec with NM101ENTY_ID_CD = '85' - replaced with  HRCHL_LEVEL_KEY key join
		//think the error is in the spec
		vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
			+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
			+ "AND LOOP_ID = '2010AA' " 
			+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";

		rowEntityAddress = getEntityAddress(vWhereCls, dbEM);
		
		// 5010" Added the suffix to the billing provider name
		// If there is a suffix name, use it, after a blank space
		String vNameSuffix = rowEntityName.getNm107nmSfx();	
		
		writeB000Rec(vPrvTaxonamyCd, vCurrencyCd, vPrvIdSec, vName, vNm109idCd_Tin,
				rowEntityAddress, vNameSuffix, vOutputFile );
		
		Query getCntcInfoQuery = dbEM
				.createQuery("SELECT c FROM gov.va.med.domain.erepos.CntcInfo c WHERE c.txnSet.id.fileNumber=:fileNumber AND c.txnSet.id.st02txnCtlNbr=:st02txnCtlNbr AND c.loopId=:loopId AND c.hrchlLevel.hrchlLevelKey=:hrchlLevelKey");
		getCntcInfoQuery.setParameter("fileNumber", vFileNumber);
		getCntcInfoQuery.setParameter("st02txnCtlNbr", vSt02txnCtlNbr);
		getCntcInfoQuery.setParameter("loopId", "2010AA");
		getCntcInfoQuery.setParameter("hrchlLevelKey", vBillinghrchlLevelKey);
		List cntcInfos = getCntcInfoQuery.getResultList();
		Iterator cntcInfosIterator = cntcInfos.iterator();
		if (cntcInfosIterator.hasNext()) {
			// Call a common method to get the values
			rowCntcInfo = (CntcInfo) cntcInfosIterator.next();
			final String vTelNbr = getContactInfoDatum(rowCntcInfo, "TE");
			final String vFax = getContactInfoDatum(rowCntcInfo, "FX");
			final String vEmail = getContactInfoDatum(rowCntcInfo, "EM");

			// If any of telephone number, fax, or email is not blank, write a
			// B001 record
			if (!(("".equals(vTelNbr)) && ("".equals(vFax)) && (""
					.equals(vEmail)))) {
				writeB001Rec(rowCntcInfo.getPer02cntcNm(), vTelNbr, vFax, vEmail, vOutputFile);
			}

		}

		/**
		 * DEV009309 - For professional claims billing provider TIN may be
		 * incorrectly selected
		 */
		vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
				+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
				+ "AND LOOP_ID = '2010AB' "
				+ "AND REF01ID_QLFR IN ('EI', 'SY') "
				+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";

		vPrvIdSec = getRefValue(vWhereCls, dbEM);

		//This query is for a pay to address if it is 2010AB
		vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
				+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
				+ "AND LOOP_ID = '2010AB' AND NM101ENTY_ID_CD='87'"
				+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";
		rowEntityName = getEntityName(vWhereCls, dbEM);
		
			
		if (rowEntityName != null) {
			vName = getCnctName(rowEntityName);			
			vNm109idCd_Tin = rowEntityName.getNm109idCd();				
		}
		
		//TWP: For 5010 the 2010AB loop is not sending a pay to provider tin/npi vNm109idCd_Tin/vPrvIdSec
		//I'm not sure if I should just put spaces in the B005 write record it probably
		//would be fine instead I am explicitly making a fix here that says if 5010 does not
		//send this information set these field to "" to avoid crashing the system		
		if(vNm109idCd_Tin == null)
		{
			vNm109idCd_Tin = "";
		}
		if(vPrvIdSec == null)
		{
			vPrvIdSec = "";
		}

		//5010: Added hierarchical level key to the query to link the two
		vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
		+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
		+ "AND LOOP_ID = '2010AB' "			
		+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";		
		
		rowEntityAddress = getEntityAddress(vWhereCls, dbEM);  //This is the payto address
		
		//5010: adding fields for pay to plan
		//We are ignoring all kinds of qualifiers
		//each individual field seems to have some check the spec
		//TXN_SET.BHT06TXN_TYPE_CD='31' AND ENITIY_NAME.NM101ENTY_ID_CD='PE'
		vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
		+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
		+ "AND LOOP_ID = '2010AC' "
		+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";	;
		
		EntityName payToPlanName = getEntityName(vWhereCls, dbEM); //This is the pay to plan name
		
		vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
		+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
		+ "AND LOOP_ID = '2010AC' "
		+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";	
		
		EntityAddress payToPlanEntityAddress = 
			getEntityAddress(vWhereCls, dbEM);  //This is the pay to plan address						
		
		//5010: adding fields for pay to plan
		vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
		+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
		+ "AND LOOP_ID = '2010AC' AND REF01ID_QLFR in ('2U','FY','NF')"
		+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";	
		String payToPlanRefId = this.getRefValue(vWhereCls, dbEM);
		
		//5010: adding fields for pay to plan
		vWhereCls = "FILE_NUMBER = '" + vFileNumber + "' "
		+ "AND ST02TXN_CTL_NBR = '" + vSt02txnCtlNbr + "' "
		+ "AND LOOP_ID = '2010AC' AND REF01ID_QLFR in ('EI') "
		+ "AND HRCHL_LEVEL_KEY ='" + vBillinghrchlLevelKey +"'";		
		String payToPlanTaxRefId = this.getRefValue(vWhereCls, dbEM);
		
		
		//TWP: Should this really be an && check ?
		if ((rowEntityName != null) && (rowEntityAddress != null)) {

			writeB005Rec(vName, vNm109idCd_Tin, vPrvIdSec, rowEntityAddress,
					rowEntityName, payToPlanName, payToPlanEntityAddress, 
					payToPlanRefId,	payToPlanTaxRefId, vOutputFile);
		}
//		vOutputFile.flush();
	} // end writeBRec


	protected String getRefValue(String vClaimKey, String vLoopId,
			String vRefQual, EntityManager dbEM) {

		String vRefVal = "";
		Query getEntityAdtlIdQuery = dbEM
				.createNativeQuery(
						"SELECT * FROM ENTITY_ADTL_ID WHERE CLAIM_KEY =:claimKey AND LOOP_ID=:loopId AND REF01ID_QLFR=:refQual",
						EntityAdtlId.class);
		getEntityAdtlIdQuery.setParameter("claimKey", vClaimKey);
		getEntityAdtlIdQuery.setParameter("loopId", vLoopId);
		getEntityAdtlIdQuery.setParameter("refQual", vRefQual);
		List entityAdtlIds = getEntityAdtlIdQuery.getResultList();
		Iterator entityAdtlIdIterator = entityAdtlIds.iterator();
		String vDate = null;
		if (entityAdtlIdIterator.hasNext()) {
			final EntityAdtlId rowEntityAdtlId = (EntityAdtlId) entityAdtlIdIterator
					.next();
			vRefVal = rowEntityAdtlId.getRef02scndId();
		}
		return vRefVal;
	}

	protected String getRefValue(String vClaimKey, String vLoopId,
			String vSvcKey, String vRefQual, EntityManager dbEM) {

		String vRefVal = "";
		Query getEntityAdtlIdQuery = dbEM.createNativeQuery(
				"SELECT * FROM ENTITY_ADTL_ID WHERE CLAIM_KEY =:claimKey AND LOOP_ID=:loopId "
						+ vSvcKey + " AND REF01ID_QLFR=:refQual",
				EntityAdtlId.class);
		getEntityAdtlIdQuery.setParameter("claimKey", vClaimKey);
		getEntityAdtlIdQuery.setParameter("loopId", vLoopId);
		getEntityAdtlIdQuery.setParameter("refQual", vRefQual);
		List entityAdtlIds = getEntityAdtlIdQuery.getResultList();
		Iterator entityAdtlIdIterator = entityAdtlIds.iterator();
		String vDate = null;
		if (entityAdtlIdIterator.hasNext()) {
			final EntityAdtlId rowEntityAdtlId = (EntityAdtlId) entityAdtlIdIterator
					.next();
			vRefVal = rowEntityAdtlId.getRef02scndId();
		}
		return vRefVal;

	}

	/**
	 * Dives the acquisition of data about an attachment and writing of that
	 * data to the extract file.
	 * 
	 * @param vClaimKey
	 * @param vOutputFile
	 * @param dbEM
	 */
	protected void processE009Rec(String vClaimKey, PrintWriter vOutputFile,
			EntityManager dbEM) {

		Query getIntchgControlQuery = dbEM
				.createNativeQuery(
						"SELECT * FROM CLAIM_SUPL_INFO WHERE CLAIM_KEY =:claimKey AND INST_SERVICE_KEY IS NULL AND PRFSN_SERVICE_KEY IS NULL ",
						ClaimSuplInfo.class);
		getIntchgControlQuery.setParameter("claimKey", vClaimKey);
		List claimSuplInfos = getIntchgControlQuery.getResultList();
		Iterator claimSuplInfoIterator = claimSuplInfos.iterator();
		if (claimSuplInfoIterator.hasNext()) {
			final ClaimSuplInfo rowClaimSuplInfo = (ClaimSuplInfo) claimSuplInfoIterator
					.next();
			writeE009Rec(rowClaimSuplInfo, vOutputFile);
			this.detach(rowClaimSuplInfo, dbEM);
		}
	}

	/**
	 * Dives the acquisition Miscellaneous Claim Data and writes that data to
	 * the extract file.
	 * 
	 * @param vClaimKey
	 * @param vOutputFile
	 * @param dbEM
	 */
	protected void processE015Rec(String vClaimKey, PrintWriter vOutputFile,
			EntityManager dbEM) {
		final String vIcnDcn = getRefValue(vClaimKey, "2300", "F8", dbEM);
		String vPreAuth1 = "";
		final String vSubCtlNum = getRefValue(vClaimKey, "2300", "D9", dbEM);
		final String vMedRecNum = getRefValue(vClaimKey, "2300", "EA", dbEM);
		final String vMedVerNum = getRefValue(vClaimKey, "2300", "F5", dbEM);
		String vPreAuth2 = "";
		final String vRefWhereCls = "CLAIM_KEY = '" + vClaimKey + "' "
				+ "AND LOOP_ID = '2300' " + "AND REF01ID_QLFR IN ('G1') ";

		Query getEntityAdtlIdQuery = dbEM.createNativeQuery(
				"SELECT * FROM ENTITY_ADTL_ID WHERE " + vRefWhereCls
						+ " ORDER BY SEQ_NBR", EntityAdtlId.class);
		List entityAdtlIds = getEntityAdtlIdQuery.getResultList();
		Iterator entityAdtlIdIterator = entityAdtlIds.iterator();
		String vDate = null;
		if (entityAdtlIds.size() > 1) {
			for (int i = 0; i < 2; i++) {
				final EntityAdtlId rowEntityAdtlId = (EntityAdtlId) entityAdtlIdIterator
						.next();
				if (i == 0) {
					vPreAuth1 = rowEntityAdtlId.getRef02scndId();
				}
				if (i == 1) {
					vPreAuth2 = rowEntityAdtlId.getRef02scndId();
				}
			}
		} else if (entityAdtlIds.size() == 1) {
			final EntityAdtlId rowEntityAdtlId = (EntityAdtlId) entityAdtlIdIterator
					.next();
			vPreAuth1 = rowEntityAdtlId.getRef02scndId();
		}

		//Maybe we could remove the sequence number check in this query?
		String vWhereClause = "CLAIM_KEY = '"+vClaimKey+ "' AND LOOP_ID='2300' AND REF01ID_QLFR='9F'"; 
		String refferalNumber1 = getRefValue(vWhereClause,dbEM);
		
//		vWhereClause = "CLAIM_KEY = '"+vClaimKey+ "' AND LOOP_ID='2300' AND REF01ID_QLFR='9F' AND SEQ_NBR=2";
//		String referralNumber2 = getRefValue(vWhereClause,dbEM);
		String referralNumber2 = "";
		
		vWhereClause = "CLAIM_KEY = '"+vClaimKey+ "' AND LOOP_ID='2300' AND REF01ID_QLFR='LU'";
		String autoAccidentState = getRefValue(vWhereClause,dbEM);
		
		// Write the E015 record if we have any data for it at all
		if ((vIcnDcn != null) || (StringUtils.hasChars(vPreAuth1))
				|| (vSubCtlNum != null) || (vMedRecNum != null)
				|| (vMedVerNum != null) || (StringUtils.hasChars(vPreAuth2))) {
			writeE015Rec(vIcnDcn, vPreAuth1, vSubCtlNum, vMedRecNum,
					vMedVerNum, vPreAuth2, refferalNumber1, referralNumber2, autoAccidentState, vOutputFile);
		}

		this.detach(entityAdtlIds, dbEM);

	}

	//5010: TWP I think this should probably be at the line level
	protected void processE035Rec(String vClaimKey, PrintWriter vOutputFile, EntityManager dbEM) {
		Query getDblMdclEqpCertQuery = dbEM.createNativeQuery(
			"SELECT * FROM DBL_MDCL_EQP_CERT WHERE CLAIM_KEY =:claimKey AND PRFSN_SERVICE_KEY IS NULL ",
			DblMdclEqpCert.class);
		getDblMdclEqpCertQuery.setParameter("claimKey", vClaimKey);
		List dblMdclEqpCerts = getDblMdclEqpCertQuery.getResultList();
		Iterator dblMdclEqpCertsIterator = dblMdclEqpCerts.iterator();
		if (dblMdclEqpCertsIterator.hasNext()) {
			final DblMdclEqpCert rowDblMdclEqpCert = (DblMdclEqpCert) dblMdclEqpCertsIterator.next();
			writeE035Rec(rowDblMdclEqpCert, vOutputFile);
			this.detach(rowDblMdclEqpCert, dbEM);
		}
	}

	/**
	 * Drives the acquisition Other Insurance Info.-Claim Related and writes
	 * that data to the extract file.
	 * 
	 * @param vClaimKey
	 * @param vPatAmtPaid
	 * @param vPatBalDue
	 * @param vOutputFile
	 * @param dbEM
	 */
	protected void processE210Rec(String vClaimKey, BigDecimal vPatAmtPaid,
			BigDecimal vPatBalDue, PrintWriter vOutputFile, EntityManager dbEM) {
		final BigDecimal vOhiPaid = getClaimAmt(vClaimKey, "D", dbEM);  //5010: changed qualifier from C4 to D
		final BigDecimal vTotSubCharge = getClaimAmt(vClaimKey, "T3", dbEM);
		final BigDecimal vOhiAllAmt = getClaimAmt(vClaimKey, "B6", dbEM);
		final BigDecimal vTotDeniedAmt = getClaimAmt(vClaimKey, "YT", dbEM);
		final BigDecimal vNonCovAmt = getClaimAmt(vClaimKey, "A8", dbEM);
		final BigDecimal vOhiAppAmt = getClaimAmt(vClaimKey, "AAE", dbEM);
		final BigDecimal vOhiDiscAmt = getClaimAmt(vClaimKey, "D8", dbEM);
		final BigDecimal vOhiCovAmt = getClaimAmt(vClaimKey, "AU", dbEM);
		final BigDecimal vOhiptRespAmt = getClaimAmt(vClaimKey, "F2", dbEM);
		
/*** TWP For Crossover Part A/B End Start***/
		final BigDecimal vTotMedicarePaidAmt = getClaimAmtByLoopId(vClaimKey, "N1","2320", dbEM);
		final BigDecimal vMedicarePaidAt100Percent= getClaimAmtByLoopId(vClaimKey, "KF","2320", dbEM);
		final BigDecimal vMedicarePaidAt80Percent = getClaimAmtByLoopId(vClaimKey, "PG", "2320",dbEM);
		final BigDecimal vPaidFromPartAMedicareTrustFundAmt = getClaimAmtByLoopId(vClaimKey, "AA","2320", dbEM);
		final BigDecimal vPaidFromPartBMedicareTrustFundAmt = getClaimAmtByLoopId(vClaimKey, "B1", "2320",dbEM);
/*** TWP For Crossover Part A/B End  ***/
		
		final BigDecimal vReminingPatientRespAmt = getClaimAmt(vClaimKey, "EAF", dbEM);
		
		final ArrayList elementsE210 = new ArrayList(12);
		String vAssignInd = "";
		OtInsuranceCov rowOtInsuranceCov = null;


		Query getOtInsuranceCov = dbEM.createNativeQuery(
				"SELECT * FROM OT_INSURANCE_COV WHERE CLAIM_KEY =:claimKey",
				OtInsuranceCov.class);
		getOtInsuranceCov.setParameter("claimKey", vClaimKey);
		List otInsuranceCovs = getOtInsuranceCov.getResultList();
		Iterator otInsuranceCovIterator = otInsuranceCovs.iterator();
		if (otInsuranceCovIterator.hasNext()) {
			rowOtInsuranceCov = (OtInsuranceCov) otInsuranceCovIterator.next();
			Character c = rowOtInsuranceCov.getOi03asgnBnftInd();
			vAssignInd = (c == null) ? null : c.toString();
		}

		elementsE210.add(vOhiPaid);  //ohi paid
		elementsE210.add(vPatBalDue); //amount due
										//approved amount
		elementsE210.add(vPatAmtPaid);  //patient paid amount
		elementsE210.add(vTotSubCharge); //ohi total submitted
		elementsE210.add(vOhiAllAmt);  //ohi allowed amount
		elementsE210.add(vTotDeniedAmt); //total denied
		elementsE210.add(vNonCovAmt); //non covered
		elementsE210.add(vOhiAppAmt); //ohi approved amount
		elementsE210.add(vOhiDiscAmt); //ohi discount amount
		elementsE210.add(vOhiCovAmt); //ohi covered amount
		elementsE210.add(vOhiptRespAmt); //patient responsibility amount
		if (vAssignInd == null) {
			elementsE210.add(" "); //OHI claim assignment indicator
		} else {
			elementsE210.add(vAssignInd);//OHI claim assignment indicator
		}
		
							
		SubscriberInfo rowSubscriberInfo = null;
		/*** TWP Changes Start***/
		elementsE210.add(vTotMedicarePaidAmt);
		elementsE210.add(vMedicarePaidAt100Percent);
		elementsE210.add(vMedicarePaidAt80Percent);
		elementsE210.add(vPaidFromPartAMedicareTrustFundAmt);
		elementsE210.add(vPaidFromPartBMedicareTrustFundAmt);
		/*** TWP Changes End ***/
		
		elementsE210.add(vReminingPatientRespAmt);//remaining patient liability

		/*HRL - TWP Workaround to synchronize OHI Payer Info with EWV*/ 
		Query getSubscriberInfos = dbEM
				.createQuery("SELECT s FROM gov.va.med.domain.erepos.SubscriberInfo s WHERE s.claimKey=:claimKey and s.loopId = :loopId ORDER BY s.seqNbr");
		getSubscriberInfos.setParameter("claimKey", vClaimKey);
		getSubscriberInfos.setParameter("loopId", "2320");
		List subscriberInfos = getSubscriberInfos.getResultList();
		Iterator subscriberInfoIterator = subscriberInfos.iterator();
		if (subscriberInfoIterator.hasNext()) {  
			//TWP: if we are not going to write all these claim payments unless we have a subscriber
			//why perform all the queries above outside the loop?
			rowSubscriberInfo = (SubscriberInfo) subscriberInfoIterator.next();
			writeE210Rec(rowSubscriberInfo, elementsE210, vOutputFile);
		}
		detach(subscriberInfos, dbEM);
		detach(otInsuranceCovs, dbEM);
	}

	/**
	 * Drives the acquisition OHI Claim Adjustment-Claim Related data and writes
	 * that data to the extract file.
	 * 
	 * @param vClaimKey
	 * @param vOutputFile
	 * @param dbEM
	 */
	protected void processE220Rec(String vClaimKey, PrintWriter vOutputFile,
			EntityManager dbEM) {

		Cases rowCases = null;
		Query getCasesQuery = dbEM
				.createNativeQuery(
						"SELECT * FROM CASES WHERE CLAIM_KEY =:claimKey AND INST_SERVICE_KEY IS NULL AND PRFSN_SERVICE_KEY IS NULL AND DENTAL_SERVICE_KEY IS NULL ",
						Cases.class);
		getCasesQuery.setParameter("claimKey", vClaimKey);
		List cases = getCasesQuery.getResultList();
		Iterator casesIterator = cases.iterator();
		while (casesIterator.hasNext()) {
			rowCases = (Cases) casesIterator.next();
			writeCasesRec(rowCases, "E220", vOutputFile);
			detach(rowCases, dbEM);
		}
	}

	/**
	 * Drives the acquisition O Other Insurance-General Info data and writes
	 * that data to the extract file.
	 * 
	 * @param vClaimKey
	 * @param vOutputFile
	 * @param dbEM
	 */
	protected void processE225Rec(String vClaimKey, PrintWriter vOutputFile,
			EntityManager dbEM) {
		final String vOhiDate = getDate(vClaimKey, "573", dbEM);
		final String vEntWhereClause = "CLAIM_KEY = '" + vClaimKey + "' "
				+ "AND NM101ENTY_ID_CD = 'PR' ";
		/*HRL - TWP Workaround to synchronize OHI Payer Info with EWV*/ 
		final EntityName rowEntityName = getEntityName(vEntWhereClause
				+ "AND LOOP_ID = '2330B' ORDER BY ENTITY_NAME_KEY", dbEM);

		if (rowEntityName != null) {
			writeE225Rec(rowEntityName, vOhiDate, vOutputFile);
			detach(rowEntityName, dbEM);
		}

	}

	/**
	 * Returns a string with the format vChIdtypeTextyyyyMMddHHmmssSSS.txt";
	 * 
	 * @param vChId
	 * @param typeText
	 * @return
	 */
	protected String createFileName(String vChId, String typeText) {
		final java.util.Date date = new GregorianCalendar().getTime();
		final SimpleDateFormat transDate = new SimpleDateFormat("yyyyMMdd",
				Locale.US);
		final SimpleDateFormat transTime = new SimpleDateFormat("HHmmssSSS",
				Locale.US);
		final String vTransDate = transDate.format(date);
		final String vTransTime = transTime.format(date);
		final String fileName = vChId + typeText + vTransDate + "_"
				+ vTransTime + ".txt";

		return fileName;
	}


	public void detach(Object entity, EntityManager dbEM) {
		if (entity == null)
			return;
		org.hibernate.Session session = (Session) dbEM.getDelegate();
		session.evict(entity);
	}

	public void detach(List aList, EntityManager dbEM) {
		org.hibernate.Session session = (Session) dbEM.getDelegate();
		if (aList == null)
			return;
		Iterator listIterator = aList.iterator();
		while (listIterator.hasNext()) {
			session.evict(listIterator.next());
		}
	}


	public static List<Claim> cloneAndClearClaims(List<Claim> claims)
	{
		Iterator<Claim> i = claims.iterator();
		List<Claim> claimList = new ArrayList<Claim>();
		while(i.hasNext())
		{
			Claim current = (Claim) i.next();
			Claim copy = new Claim();
			claimList.add(copy);			
			copy.setClaimKey(current.getClaimKey());
			copy.setFileNumber(current.getFileNumber());
			copy.setSt02txnCtlNbr(current.getSt02txnCtlNbr());
			copy.setNm109billPrvdId(current.getNm109billPrvdId());
			copy.setNm109payPrvdId(current.getNm109payPrvdId());
			copy.setSenderId(current.getSenderId());
			copy.setReceiverId(current.getReceiverId());
			copy.setSubscriberId(current.getSubscriberId());
			copy.setPatientId(current.getPatientId());
			copy.setClm01ptAcctNbr(current.getClm01ptAcctNbr());
			copy.setClm02clmChrgAmt(current.getClm02clmChrgAmt());
			copy.setClm05fclyTypeCd(current.getClm05fclyTypeCd());
			copy.setClm05fclyCdQual(current.getClm05fclyCdQual());
			copy.setClm05clmFrqCd(current.getClm05clmFrqCd());
			copy.setClm06prvdSgnInd(current.getClm06prvdSgnInd());
			copy.setClm07mdcrAsgnCd(current.getClm07mdcrAsgnCd());
			copy.setClm08asgnOfBnft(current.getClm08asgnOfBnft());
			copy.setClm09relInfoCd(current.getClm09relInfoCd());
			copy.setClm10patSignCd(current.getClm10patSignCd());
			copy.setClm11rltCauses1(current.getClm11rltCauses1());
			copy.setClm11rltCauses2(current.getClm11rltCauses2());
			copy.setClm11rltCauses3(current.getClm11rltCauses3());
			copy.setClm11st(current.getClm11st());
			copy.setClm11ctryCd(current.getClm11ctryCd());
			copy.setClm12spclPgmCd(current.getClm12spclPgmCd());
			copy.setClm16prvAgmentCd(current.getClm16prvAgmentCd());
			copy.setClm18explOfBnft(current.getClm18explOfBnft());
			copy.setClm19predterBnft(current.getClm19predterBnft());
			copy.setClm20delayRsnCd(current.getClm20delayRsnCd());
			copy.setSubmissionIndex(current.getSubmissionIndex());
			copy.setClaimType(current.getClaimType());
			copy.setStatusCode(current.getStatusCode());	
			copy.setBillingHrchlLevelKey(current.getBillingHrchlLevelKey());
			copy.setSubscriberHrchlLevelKey(current.getSubscriberHrchlLevelKey());
			copy.setPatientHrchlLevelKey(current.getPatientHrchlLevelKey());
		}
		claims.clear();
		return claimList;
	}
	
	/** Start 3 TWP Added to accommodate new file name w receiverId and writing to tmp file first ***/
	protected File createTemporaryFileAndDir(String vFilePath, String fileName)
	{
		String vTmpFilePath = vFilePath + "tmp" + System.getProperty("file.separator");
		File fileToReturn = null;

		File newDirectory = new File(vTmpFilePath);
		boolean success = newDirectory.mkdir() || newDirectory.exists();	 
		if(success)
		{
			fileToReturn = new File(vTmpFilePath + fileName);
		}

		return fileToReturn; 
	}
	/** End 3 TWP Added to accommodate new file name w receiverId and writing to tmp file first ***/
	
	 /**
	   * Returns the professional and dental statement date(s) associated with the claim using an additional qualifier.
	   * 
	   * @param vClaimKey
	   * @param dbEM
	   * @return
	   */

	  protected String getStmtDate(String vClaimKey, String vAdtlQual,
	      EntityManager dbEM) {
	    String vStatementDate = null;
	    StringBuffer vStmtDate = new StringBuffer("");
	    String strQry = "SELECT MIN(TO_NUMBER(SUBSTR(DTP03DT_FLD,1,8))) FROM CLAIM_DT "
	        + "WHERE CLAIM_KEY = '"
	        + vClaimKey
	        + "' "
	        + "AND DTP01DTTM_QUAL = '472' " + vAdtlQual;

	    Query claimDtMinQuery = dbEM.createNativeQuery(strQry);
	    List mins = claimDtMinQuery.getResultList();
	    Iterator minIterator = mins.iterator();
   
	    if (minIterator.hasNext()) {
	    	Object fieldsReturned =  minIterator.next();
	      if (fieldsReturned != null) {
	    	  vStmtDate.append(fieldsReturned.toString());
	      }
	    }

	    strQry = "SELECT MAX(TO_NUMBER(SUBSTR(DTP03DT_FLD,1,8))) FROM CLAIM_DT "
			   + "WHERE CLAIM_KEY = '"
			   + vClaimKey
			   + "' "
			   + "AND DTP01DTTM_QUAL = '472' " + vAdtlQual;
	    
	    claimDtMinQuery = dbEM.createNativeQuery(strQry);
	    mins = claimDtMinQuery.getResultList();
	    minIterator = mins.iterator();
   
	    if (minIterator.hasNext()) {
	      Object fieldsReturned =  minIterator.next();
	      if (fieldsReturned != null) {
	    	  vStmtDate.append("-");
	    	  vStmtDate.append(fieldsReturned.toString());
	      }
	    }	    
	    
	    vStatementDate = vStmtDate.toString();
	    
	    return vStatementDate;
	  }
	  
	  
}