package gov.va.med.hac.edi.extract.x214;

import gov.va.med.domain.erepos.*;
import gov.va.med.domain.fppsowner.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author            
 *
 */
public class Build277CASegments {
	static final Logger LOGGER = LoggerFactory
			.getLogger(Build277CASegments.class);
	private final static String COMPONENT_ELEMENT_SEPARATOR = ":";
	private final static String DATA_ELEMENT_SEPARATOR = "*";
	private final static String SEGMENT_TERMINATOR = "~";
	private final static String REPETITION_SEPARATOR = "^";
	private static final String ACCEPTED = "277U_ACCPT_SENT";
	private static final String REJECTED = "277U_REJ_SENT";
	private static final String ACCEPTED_WITH_ERRORS = "FPPS LOAD ERROR";
	gov.va.med.domain.erepos.Claim reposClaim = null;
	gov.va.med.domain.fppsowner.Claim fppsClaim = null;
	EntityManager em;
	File csFile = null;
	final String CRLF = "\r\n";
	PrintWriter vCsFile = null;
	String filePath = "";
	String ediClaimKey = "";
	String sClaimKey = "";
	EntityName senderInfo = null;
	EntityName receiverInfo = null;
	EntityName billingProviderInfo = null;
	PersonInfo oPatient = null;
	PatientInfo oPatientInfo = null;
	FunctionalGroup oFunctionalGroup = null;
	TxnSet oTxnSet = null;
	EntityAdtlId oEntityAdtlId = null;
	ClaimDt oClaimDt = null;
	Session oSession = null;
	ScrollableResults claims = null;
	long claimKey;
	boolean bInClaim = true;
	Connection conn;
	PreparedStatement pStatement;

	/**
	 * 
	 * @param claim
	 * @param ediClaimKey
	 */
	protected void createISAHeader() {
		LOGGER.debug("Begin Creating ISAHeader");
		StringBuffer sbHdrRec = new StringBuffer();
		sbHdrRec.append("ISA");
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append("00");
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append("          ");
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append("00");
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append("          ");
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append("ZZ");
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append(getReceiverId());
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append("ZZ");
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append(getSenderId());
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append(getCreatedDateyyMMdd());
		dataElementSeparator(sbHdrRec);
		sbHdrRec.append(getCreatedTimeHHmm());
		dataElementSeparator(sbHdrRec);
		repetationSeperator(sbHdrRec);
		dataElementSeparator(sbHdrRec);
		// Interchange Control Version Number
		sbHdrRec.append("00501");
		dataElementSeparator(sbHdrRec);
		// Interchange Control Number
		sbHdrRec.append(getInterChangeCtrlNumber());
		dataElementSeparator(sbHdrRec);
		// Acknowledgment Requested
		sbHdrRec.append("0");
		dataElementSeparator(sbHdrRec);
		// Interchange Usage Indicator
		sbHdrRec.append("P");
		dataElementSeparator(sbHdrRec);
		componentElementSeparator(sbHdrRec);
		segmentTerminator(sbHdrRec);
		sbHdrRec.append(CRLF);
		vCsFile.print(sbHdrRec);
		vCsFile.flush();
		sbHdrRec = null;
		LOGGER.debug("End Creating ISAHeader");
	}

	/**
	 * 
	 */
	protected void createGS() {
		LOGGER.debug("Begin Creating GS Segment");
		StringBuffer strGS = new StringBuffer();
		setFunctionalGroup();
		strGS.append("GS");
		dataElementSeparator(strGS);
		strGS.append("HN");
		dataElementSeparator(strGS);
		strGS.append(getCheckNull(oFunctionalGroup.getGs03appRcvCd()));
		dataElementSeparator(strGS);
		strGS.append(getCheckNull(oFunctionalGroup.getGs02appSndCd()));
		dataElementSeparator(strGS);
		strGS.append(getCreatedDateyyyyMMdd());
		dataElementSeparator(strGS);
		strGS.append(getCreatedTimeHHmmssSS());
		dataElementSeparator(strGS);
		// Interchange Control Number
		strGS.append(getCheckNull(oFunctionalGroup.getGs06grpCtlNbr()));
		dataElementSeparator(strGS);
		strGS.append(getCheckNull(oFunctionalGroup.getGs07respAgcyCd()));
		dataElementSeparator(strGS);
		strGS.append("005010X214");
		segmentTerminator(strGS);
		strGS.append(CRLF);
		vCsFile.print(strGS);
		vCsFile.flush();
		strGS = null;
		LOGGER.debug("End Creating GS Segment");
	}

	/**
	 * 
	 */
	protected void createST() {
		LOGGER.debug("Begin Creating ST Segment");
		StringBuffer strST = new StringBuffer();
		strST.append("ST");
		dataElementSeparator(strST);
		strST.append("277");
		dataElementSeparator(strST);
		// Transaction Control Number
		strST.append(reposClaim.getSt02txnCtlNbr());
		dataElementSeparator(strST);
		strST.append("005010X214");
		segmentTerminator(strST);
		strST.append(CRLF);
		vCsFile.print(strST);
		vCsFile.flush();
		strST = null;
		LOGGER.debug("End Creating ST Segment");
	}

	/**
	 * 
	 */
	protected void createBHT() {
		LOGGER.debug("Begin Creating BHT Segment");
		setBHT();
		StringBuffer strBHT = new StringBuffer();
		strBHT.append("BHT");
		dataElementSeparator(strBHT);
		strBHT.append("0085");
		dataElementSeparator(strBHT);
		strBHT.append("08");
		dataElementSeparator(strBHT);
		// Reference Identification...
		strBHT.append(oTxnSet.getBht03orgnTxnId());
		dataElementSeparator(strBHT);
		strBHT.append(getCreatedDateyyyyMMdd());
		dataElementSeparator(strBHT);
		strBHT.append(getCreatedTimeHHmmssSS());
		dataElementSeparator(strBHT);
		strBHT.append("TH");
		segmentTerminator(strBHT);
		strBHT.append(CRLF);
		vCsFile.print(strBHT);
		vCsFile.flush();
		strBHT = null;
		LOGGER.debug("End Creating BHT Segment");
	}

	/**
	 * 
	 */
	protected void createHL1() {
		LOGGER.debug("Begin Creating HL1 Segment");
		StringBuffer strHL1 = new StringBuffer();
		strHL1.append("HL");
		dataElementSeparator(strHL1);
		strHL1.append("1");
		dataElementSeparator(strHL1);
		dataElementSeparator(strHL1);
		strHL1.append("20");
		dataElementSeparator(strHL1);
		strHL1.append("1");
		segmentTerminator(strHL1);
		strHL1.append(CRLF);
		vCsFile.print(strHL1);
		vCsFile.flush();
		strHL1 = null;
		LOGGER.debug("End Creating HL1 Segment");
	}

	/**
	 * Create Organization Information...
	 */
	protected void createNM1_1() {
		LOGGER.debug("Begin Creating NM1 Segment");
		StringBuffer strNM1 = new StringBuffer();
		strNM1.append("NM1");
		dataElementSeparator(strNM1);
		strNM1.append("PR");
		if (null != this.receiverInfo) {
			String temp = "";
			dataElementSeparator(strNM1);
			strNM1.append(receiverInfo.getNm102entyQual());
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(receiverInfo.getNm103lastNm()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(receiverInfo.getNm104fstNm()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(receiverInfo.getNm105midNm()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(receiverInfo.getNm106nmPfx()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(receiverInfo.getNm107nmSfx()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(receiverInfo.getNm108idCdQual()));
			dataElementSeparator(strNM1);
			// Identification Code X AN
			strNM1.append(getCheckNull(receiverInfo.getNm109billPrvdId()));
			temp = getCheckNull(receiverInfo.getNm110entyRelCd());
			if (temp != null && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}

			temp = getCheckNull(receiverInfo.getNm111entyIdCd());
			if (temp != null && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}

			temp = getCheckNull(receiverInfo.getNm112lastNmOrganizNm());
			if (temp != null && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
		}
		segmentTerminator(strNM1);
		strNM1.append(CRLF);
		vCsFile.print(strNM1);
		vCsFile.flush();
		strNM1 = null;
		LOGGER.debug("End Creating NM1 Segment");
	}

	/**
	 * 
	 */
	protected void createTRN1() {
		LOGGER.debug("Begin Creating TRN1 Segment");
		StringBuffer strTRN1 = new StringBuffer();
		strTRN1.append("TRN");
		dataElementSeparator(strTRN1);
		strTRN1.append("1");
		dataElementSeparator(strTRN1);
		strTRN1.append(getInterChangeCtrlNumber());
		segmentTerminator(strTRN1);
		strTRN1.append(CRLF);
		vCsFile.print(strTRN1);
		vCsFile.flush();
		strTRN1 = null;
		LOGGER.debug("End Creating TRN1 Segment");
	}

	/**
	 * 
	 */
	protected void createDTP1() {
		LOGGER.debug("Begin Creating DTP1 Segment");
		StringBuffer strDTP1 = new StringBuffer();
		strDTP1.append("DTP");
		dataElementSeparator(strDTP1);
		strDTP1.append("050");
		dataElementSeparator(strDTP1);
		strDTP1.append("D8");
		dataElementSeparator(strDTP1);
		strDTP1.append(getCreatedDateyyyyMMdd());
		segmentTerminator(strDTP1);
		strDTP1.append(CRLF);
		vCsFile.print(strDTP1);
		vCsFile.flush();
		strDTP1 = null;
		LOGGER.debug("End Creating DTP1 Segment");
	}

	/**
	 * 
	 */
	protected void createDTP2() {
		LOGGER.debug("Begin Creating DTP2 Segment");
		StringBuffer strDTP2 = new StringBuffer();
		strDTP2.append("DTP");
		dataElementSeparator(strDTP2);
		strDTP2.append("009");
		dataElementSeparator(strDTP2);
		strDTP2.append("D8");
		dataElementSeparator(strDTP2);
		strDTP2.append(getCreatedDateyyyyMMdd());
		dataElementSeparator(strDTP2);
		segmentTerminator(strDTP2);
		strDTP2.append(CRLF);
		vCsFile.print(strDTP2);
		vCsFile.flush();
		strDTP2 = null;
		LOGGER.debug("End Creating DTP2 Segment");
	}

	/**
	 * 
	 */
	protected void createHL2() {
		LOGGER.debug("Begin Creating HL2 Segment");
		StringBuffer strHL2 = new StringBuffer();
		strHL2.append("HL");
		dataElementSeparator(strHL2);
		strHL2.append("2");
		dataElementSeparator(strHL2);
		strHL2.append("1");
		dataElementSeparator(strHL2);
		strHL2.append("21");
		dataElementSeparator(strHL2);
		strHL2.append("1");
		segmentTerminator(strHL2);
		strHL2.append(CRLF);
		vCsFile.print(strHL2);
		vCsFile.flush();
		strHL2 = null;
		LOGGER.debug("End Creating HL2 Segment");
	}

	/**
	 * 
	 */
	protected void createNM1_2() {
		LOGGER.debug("Begin Creating NM1 receiver Segment");
		StringBuffer strNM1 = new StringBuffer();
		strNM1.append("NM1");
		dataElementSeparator(strNM1);
		strNM1.append("41");
		if (null != this.senderInfo) {
			String temp = "";
			dataElementSeparator(strNM1);
			strNM1.append(senderInfo.getNm102entyQual());
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(senderInfo.getNm103lastNm()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(senderInfo.getNm104fstNm()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(senderInfo.getNm105midNm()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(senderInfo.getNm106nmPfx()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(senderInfo.getNm107nmSfx()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(senderInfo.getNm108idCdQual()));
			dataElementSeparator(strNM1);
			strNM1.append(getCheckNull(senderInfo.getNm109billPrvdId()));
			temp = getCheckNull(senderInfo.getNm110entyRelCd());
			if (temp != null && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
			temp = getCheckNull(senderInfo.getNm111entyIdCd());
			if (temp != null && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
			temp = getCheckNull(senderInfo.getNm112lastNmOrganizNm());
			if (temp != null && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
		}
		segmentTerminator(strNM1);
		strNM1.append(CRLF);
		vCsFile.print(strNM1);
		vCsFile.flush();
		strNM1 = null;
		LOGGER.debug("End Creating NM1 receiver Segment");
	}

	/**
	 * 
	 */
	protected void createTRN2() {
		LOGGER.debug("Begin Creating TRN receiver Segment");
		StringBuffer strTRN2 = new StringBuffer();
		strTRN2.append("TRN");
		dataElementSeparator(strTRN2);
		strTRN2.append("2");
		dataElementSeparator(strTRN2);
		strTRN2.append(this.getInterChangeCtrlNumber());
		segmentTerminator(strTRN2);
		strTRN2.append(CRLF);
		vCsFile.print(strTRN2);
		vCsFile.flush();
		strTRN2 = null;
		LOGGER.debug("End Creating TRN receiver Segment");
	}

	/**
	 * 
	 */
	protected void createSTC1() {
		LOGGER.debug("Begin Creating STC1 Segment");
		StringBuffer strSTC = new StringBuffer();
		strSTC.append("STC");
		dataElementSeparator(strSTC);

		if (null != reposClaim.getStatusCode() && reposClaim.getStatusCode().equals(REJECTED)) {
			strSTC.append("A3:21");
			dataElementSeparator(strSTC);
			strSTC.append(getCreatedDateyyyyMMdd());
			dataElementSeparator(strSTC);
			strSTC.append("U");
		} else {
			strSTC.append("A1:20");
			dataElementSeparator(strSTC);
			strSTC.append(getCreatedDateyyyyMMdd());
			dataElementSeparator(strSTC);
			strSTC.append("WQ");
		}
		dataElementSeparator(strSTC);
		if (null != reposClaim.getClm02clmChrgAmt()) {
			strSTC.append(reposClaim.getClm02clmChrgAmt());
		}
		segmentTerminator(strSTC);
		strSTC.append(CRLF);
		vCsFile.print(strSTC);
		vCsFile.flush();
		strSTC = null;
		LOGGER.debug("End Creating STC1 Segment");
	}

	/**
	 * 
	 */
	protected void createQTY() {
		LOGGER.debug("Begin Creating QTY Segment");
		StringBuffer strQTY = new StringBuffer();
		strQTY.append("QTY");
		dataElementSeparator(strQTY);
		strQTY.append("90");
		dataElementSeparator(strQTY);
		strQTY.append("1");
		segmentTerminator(strQTY);
		strQTY.append(CRLF);
		vCsFile.print(strQTY);
		vCsFile.flush();
		strQTY = null;
		LOGGER.debug("End Creating QTY Segment");
	}

	/**
	 * 
	 */
	protected void createAMT() {
		LOGGER.debug("Begin Creating AMT Segment");
		StringBuffer strAMT = new StringBuffer();
		strAMT.append("AMT");
		dataElementSeparator(strAMT);
		strAMT.append("YU");
		dataElementSeparator(strAMT);
		strAMT.append(this.reposClaim.getClm02clmChrgAmt());
		segmentTerminator(strAMT);
		strAMT.append(CRLF);
		vCsFile.print(strAMT);
		vCsFile.flush();
		strAMT = null;
		LOGGER.debug("End Creating AMT Segment");
	}

	/**
	 * 
	 */
	protected void createHL3() {
		LOGGER.debug("Begin Creating HL Segment Billing Provider");
		StringBuffer strHL = new StringBuffer();
		strHL.append("HL");
		dataElementSeparator(strHL);
		strHL.append("3");
		dataElementSeparator(strHL);
		strHL.append("2");
		dataElementSeparator(strHL);
		strHL.append("19");
		dataElementSeparator(strHL);
		strHL.append("1");
		segmentTerminator(strHL);
		strHL.append(CRLF);
		vCsFile.print(strHL);
		vCsFile.flush();
		strHL = null;
		LOGGER.debug("End Creating HL Segment Billing Provider");
	}

	/**
	 * 
	 */
	protected void createNM1_3() {
		LOGGER.debug("Begin Creating NM1 Billing Provider");
		StringBuffer strNM1 = new StringBuffer();
		strNM1.append("NM1");
		dataElementSeparator(strNM1);
		strNM1.append("85");
		if (null != billingProviderInfo) {
			String temp = "";
			dataElementSeparator(strNM1);
			strNM1.append(billingProviderInfo.getNm102entyQual());
			temp = getCheckNull(billingProviderInfo.getNm103lastNm());
			if (null != temp && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
			temp = getCheckNull(billingProviderInfo.getNm104fstNm());
			if (null != temp && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
			temp = getCheckNull(billingProviderInfo.getNm105midNm());
			if (null != temp && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
			temp = getCheckNull(billingProviderInfo.getNm106nmPfx());
			if (null != temp && temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}

			temp = getCheckNull(billingProviderInfo.getNm107nmSfx());
			if (null != null & temp.length() > 0) {
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
			temp = getCheckNull(billingProviderInfo.getNm109ptntId());
			if (temp != null && temp.length() > 0) {
				String temp1 = getCheckNull(billingProviderInfo
						.getNm108idCdQual());
				if (temp1 != null && temp1.length() > 0) {
					dataElementSeparator(strNM1);
					strNM1.append(temp1);
				}
				dataElementSeparator(strNM1);
				strNM1.append(temp);
			}
		}
		segmentTerminator(strNM1);
		strNM1.append(CRLF);
		vCsFile.print(strNM1);
		vCsFile.flush();
		strNM1 = null;
		LOGGER.debug("End Creating NM1 Billing Provider");
	}

	/**
	 * 
	 */
	protected void createTRN3() {
		LOGGER.debug("Begin Creating TRN Billing Provider");
		StringBuffer strTRN3 = new StringBuffer();
		strTRN3.append("TRN");
		dataElementSeparator(strTRN3);
		strTRN3.append("1");
		dataElementSeparator(strTRN3);
		strTRN3.append(this.getInterChangeCtrlNumber());
		segmentTerminator(strTRN3);
		strTRN3.append(CRLF);
		vCsFile.print(strTRN3);
		vCsFile.flush();
		strTRN3 = null;
		LOGGER.debug("End Creating TRN Billing Provider");
	}

	/**
	 * 
	 */
	protected void createHL4() {
		LOGGER.debug("Begin Creating HL Patient Info");
		StringBuffer strHL = new StringBuffer();
		strHL.append("HL");
		dataElementSeparator(strHL);
		strHL.append("4");
		dataElementSeparator(strHL);
		strHL.append("3");
		dataElementSeparator(strHL);
		strHL.append("PT");
		segmentTerminator(strHL);
		strHL.append(CRLF);
		vCsFile.print(strHL);
		vCsFile.flush();
		strHL = null;
		LOGGER.debug("End Creating HL Patient Info");
	}

	/**
	 * 
	 */
	protected void createNM1_4() {
		LOGGER.debug("Begin Creating Nm1 Patient Info");
		StringBuffer strNM1 = new StringBuffer();
		oPatient = getPersonInfo();
		strNM1.append("NM1");
		dataElementSeparator(strNM1);
		strNM1.append("QC");
		dataElementSeparator(strNM1);
		strNM1.append("1");
		dataElementSeparator(strNM1);
		strNM1.append(getCheckNull(oPatient.getLastName()));
		dataElementSeparator(strNM1);
		strNM1.append(getCheckNull(oPatient.getFirstName()));
		dataElementSeparator(strNM1);
		strNM1.append(getCheckNull(oPatient.getMiddleName()));
		dataElementSeparator(strNM1);
		dataElementSeparator(strNM1);
		strNM1.append(getCheckNull(oPatient.getNameSuffix()));
		dataElementSeparator(strNM1);
		strNM1.append("MI");
		dataElementSeparator(strNM1);
		strNM1.append(getCheckNull(oPatient.getPersonId()));
		segmentTerminator(strNM1);
		strNM1.append(CRLF);
		vCsFile.print(strNM1);
		vCsFile.flush();
		strNM1 = null;
		LOGGER.debug("End Creating Nm1 Patient Info");
	}

	/**
	 * 
	 */
	protected void createTRN4() {
		LOGGER.debug("Begin Creating TRN Patient Info");
		StringBuffer strTRN4 = new StringBuffer();
		oPatientInfo = getPatientInfo();
		strTRN4.append("TRN");
		dataElementSeparator(strTRN4);
		strTRN4.append("2");
		dataElementSeparator(strTRN4);
		strTRN4.append(this.reposClaim.getSt02txnCtlNbr());
		segmentTerminator(strTRN4);
		strTRN4.append(CRLF);
		vCsFile.print(strTRN4);
		vCsFile.flush();
		strTRN4 = null;
		LOGGER.debug("End Creating TRN Patient Info");
	}

	/**
	 * 
	 */
	protected void createSTC2() {
		LOGGER.debug("Begin Creating STC2");
		StringBuffer strSTC = new StringBuffer();
		strSTC.append("STC");
		dataElementSeparator(strSTC);
		strSTC.append("A2:20");
		dataElementSeparator(strSTC);
		strSTC.append(getCreatedDateyyyyMMdd());
		dataElementSeparator(strSTC);
		if (null != fppsClaim.getClaimStatus()
				&& null != fppsClaim.getClaimStatus().getClaimStatusCd()
				&& fppsClaim.getClaimStatus().getClaimStatusCd()
						.equalsIgnoreCase("REJECTED")) {
			strSTC.append("U");
		} else {
			strSTC.append("WQ");
		}
		dataElementSeparator(strSTC);
		if (null != reposClaim.getClm02clmChrgAmt()) {
			strSTC.append(reposClaim.getClm02clmChrgAmt());
		}
		segmentTerminator(strSTC);
		strSTC.append(CRLF);
		vCsFile.print(strSTC);
		vCsFile.flush();
		strSTC = null;
		LOGGER.debug("End Creating STC2");
	}

	/**
	 * 
	 */
	protected void createREF() {
		LOGGER.debug("Begin Creating REF");
		setEntityAdtlId();
		StringBuffer strREF = new StringBuffer();
		strREF.append("REF");
		dataElementSeparator(strREF);
		if (null != oEntityAdtlId) {
			strREF.append(oEntityAdtlId.getRef01idQlfr());
			dataElementSeparator(strREF);
			strREF.append(oEntityAdtlId.getRef02scndId());
		}
		segmentTerminator(strREF);
		strREF.append(CRLF);
		vCsFile.print(strREF);
		vCsFile.flush();
		strREF = null;
		LOGGER.debug("End Creating REF");
	}

	/**
	 * 
	 */
	protected void createDTP() {
		LOGGER.debug("Begin Creating DTP");
		setClaimDate();
		StringBuffer strDTP = new StringBuffer();
		strDTP.append("DTP");
		dataElementSeparator(strDTP);
		strDTP.append("472");
		if (null != oClaimDt) {
			dataElementSeparator(strDTP);
			strDTP.append(oClaimDt.getDtp02dttmFmtQual());
			dataElementSeparator(strDTP);
			strDTP.append(oClaimDt.getDtp03dtFld());
		}
		segmentTerminator(strDTP);
		strDTP.append(CRLF);
		vCsFile.print(strDTP);
		vCsFile.flush();
		strDTP = null;
		LOGGER.debug("End Creating DTP");
	}

	/**
	 * 
	 */
	protected void createSE() {
		LOGGER.debug("Begin Creating SE");
		StringBuffer strSE = new StringBuffer();
		strSE.append("SE");
		dataElementSeparator(strSE);
		strSE.append("23");
		dataElementSeparator(strSE);
		strSE.append(this.reposClaim.getSt02txnCtlNbr());
		segmentTerminator(strSE);
		strSE.append(CRLF);
		vCsFile.print(strSE);
		vCsFile.flush();
		strSE = null;
		LOGGER.debug("End Creating SE");
	}

	/**
	 * 
	 */
	protected void createGE() {
		LOGGER.debug("Begin Creating GE");
		StringBuffer strGE = new StringBuffer();
		strGE.append("GE");
		dataElementSeparator(strGE);
		strGE.append("1");
		dataElementSeparator(strGE);
		strGE.append(oFunctionalGroup.getGs06grpCtlNbr());
		segmentTerminator(strGE);
		strGE.append(CRLF);
		vCsFile.print(strGE);
		vCsFile.flush();
		strGE = null;
		LOGGER.debug("End Creating GE");
	}

	/**
	 * 
	 */
	protected void createIEA() {
		LOGGER.debug("Begin Creating IEA");
		StringBuffer strIEA = new StringBuffer();
		strIEA.append("IEA");
		dataElementSeparator(strIEA);
		strIEA.append("1");
		dataElementSeparator(strIEA);
		strIEA.append(getInterChangeCtrlNumber());
		segmentTerminator(strIEA);
		strIEA.append(CRLF);
		vCsFile.print(strIEA);
		vCsFile.flush();
		strIEA = null;
		LOGGER.debug("End Creating IEA");
	}

	/**
	 * 
	 * @return PersonInfo
	 */
	@SuppressWarnings("unchecked")
	private PersonInfo getPersonInfo() {
		LOGGER.debug("Begin extract Person Information");
		Query q = em.createNativeQuery(
				"select * from PERSON_INFO where CLAIM_INDEX = :claimIndex",
				PersonInfo.class);
		q.setParameter("claimIndex", this.fppsClaim.getClaimIndex());
		PersonInfo oPatient = null;
		List<PersonInfo> patientList = q.getResultList();

		if (null != patientList && patientList.size() > 0) {
			oPatient = patientList.get(0);
		}
		LOGGER.debug("End extract Person Information");
		return oPatient;
	}

	@SuppressWarnings("unchecked")
	private void setBHT() {
		LOGGER.debug("Begin extract BHT Information");
		Query q = em
				.createQuery("select ts from gov.va.med.domain.erepos.TxnSet ts where ts.id.fileNumber = :fileNumber and ts.se02txnCtlNbr = :txnCtlNbr");
		q.setParameter("fileNumber", reposClaim.getFileNumber());
		q.setParameter("txnCtlNbr", reposClaim.getSt02txnCtlNbr());
		List<TxnSet> txnSetList = q.getResultList();
		if (null != txnSetList && txnSetList.size() > 0) {
			oTxnSet = txnSetList.get(0);
		}
		LOGGER.debug("End extract BHT Information");
	}

	/**
	 * 
	 * @return PatientInfo
	 */
	@SuppressWarnings("unchecked")
	private PatientInfo getPatientInfo() {
		LOGGER.debug("Begin extract Patient Information");
		String sql = "select * from PATIENT_INFO where PATIENT_INFO_KEY=:patientInfoKey";
		Query q = em.createNativeQuery(sql);
		q.setParameter("patientInfoKey", reposClaim.getPatientInfo());
		List<PatientInfo> patientInfoList = q.getResultList();

		if (null != patientInfoList && patientInfoList.size() > 0) {
			oPatientInfo = patientInfoList.get(0);
		}
		LOGGER.debug("End extract Patient Information");
		return oPatientInfo;
	}

	/**
	 * 
	 * @return String
	 */
	private String getSenderId() {
		LOGGER.debug("Begin extract Sender ID Information");
		String strSenderId = "";
		if (reposClaim != null) {
			strSenderId = reposClaim.getSenderId();
			if (strSenderId != null && strSenderId.length() < 15) {
				for (int i = strSenderId.length(); i < 15; i++) {
					strSenderId = strSenderId.concat(" ");
				}
			}
		}
		LOGGER.debug("End extract sender ID Information");
		return strSenderId;
	}

	/**
	 * 
	 * @return String
	 */
	private String getReceiverId() {
		LOGGER.debug("Begin extract Receiver ID Information");
		String strReceiverId = "";
		if (null != reposClaim) {
			strReceiverId = reposClaim.getReceiverId();
			if (null != strReceiverId && strReceiverId.length() < 15) {
				for (int i = strReceiverId.length(); i < 15; i++) {
					strReceiverId = strReceiverId.concat(" ");
				}
			}
		}
		LOGGER.debug("End extract Receiver ID Information");
		return strReceiverId;
	}

	/**
	 * 
	 */
	final void setClaimDate() {
		LOGGER.debug("Begin extract Claim Date Information");
		try {
			String sql = "SELECT * FROM CLAIM_DT WHERE CLAIM_KEY= ? AND DTP01DTTM_QUAL= ?";
			pStatement = conn.prepareStatement(sql);
			pStatement.setString(1, ediClaimKey);
			pStatement.setString(2, "472");
			ResultSet rs = pStatement.executeQuery();
			if (null != rs) {
				oClaimDt = new ClaimDt();
				while (rs.next()) {
					oClaimDt.setDtp02dttmFmtQual(rs.getString(
							"DTP02DTTM_FMT_QUAL").trim());
					oClaimDt.setDtp03dtFld(rs.getString("DTP03DT_FLD"));
				}
				rs.close();
			}
			if (null != pStatement) {
				pStatement.close();
			}
		} catch (Exception exp) {
			exp.printStackTrace();
		}
		LOGGER.debug("End extract Claim Date Information");
	}

	/**
	 * 
	 */
	@SuppressWarnings("unchecked")
	final void setEntityAdtlId() {
		LOGGER.debug("Begin extract Entity Additional ID Information");
		String qstr = "select * from ENTITY_ADTL_ID e where e.CLAIM_KEY =:claimKey";
		Query q = em.createNativeQuery(qstr, EntityAdtlId.class);
		q.setParameter("claimKey", ediClaimKey);
		List<EntityAdtlId> entityAdtlIdList = q.getResultList();
		if (null != entityAdtlIdList && entityAdtlIdList.size() > 0) {
			oEntityAdtlId = entityAdtlIdList.get(0);
		}
		LOGGER.debug("End extract Entity Aditional ID Information");
	}

	/**
	 * 
	 * @return EntityName
	 */
	@SuppressWarnings("unchecked")
	final void setEntityInfo() {
		LOGGER.debug("Begin extract Entity Name Information");
		String qstr = "select e from eEntityName e where e.claim.claimKey = :ediClaimKey";
		Query q = em.createQuery(qstr);
		q.setParameter("ediClaimKey", ediClaimKey);
		List<EntityName> entityNames = q.getResultList();
		if (null != entityNames && (entityNames.size() > 0)) {
			for (EntityName anEntity : entityNames) {
				if (anEntity.getLoopId().equalsIgnoreCase(("1000A"))) {
					senderInfo = anEntity;
				} else if (anEntity.getLoopId().equalsIgnoreCase(("1000B"))) {
					receiverInfo = anEntity;
				} else {
					this.billingProviderInfo = anEntity;
				}
			}
		} else {
			LOGGER.debug(
					"Did not get ediClaimKey: {}, SenderId: {} note:it may not be required",
					new Object[] { ediClaimKey, reposClaim.getSenderId() });
		}
		LOGGER.debug("End extract Entity Name Information");
	}

	/**
	 * 
	 */
	final void setFippsEntity() {
		LOGGER.debug("Begin set Fipss_Owner row to Entity");
		String sql = "select c.* from fpps_owner.claims c where c.edi_claim_key  = ?";
		try {
			pStatement = conn.prepareStatement(sql);
			pStatement.setString(1, ediClaimKey);
			ResultSet rs = pStatement.executeQuery();
			if (null != rs) {
				fppsClaim = new gov.va.med.domain.fppsowner.Claim();
				while (rs.next()) {
					fppsClaim.setAccidentDate(rs.getDate("ACCIDENT_DATE"));
					fppsClaim.setAccidentState(rs.getString("ACCIDENT_STATE"));
					fppsClaim.setAdmissionDate(rs.getDate("ADMISSION_DATE"));
					fppsClaim.setAdmissionSourceCd(rs
							.getString("ADMISSION_SOURCE_CD"));
					fppsClaim.setAdmissionTypeCd(rs
							.getString("ADMISSION_TYPE_CD"));
					fppsClaim.setAmbulanceTransportText(rs
							.getString("AMBULANCE_TRANSPORT_TEXT"));
					fppsClaim.setAmountRejected(rs
							.getBigDecimal("AMOUNT_REJECTED"));
					fppsClaim.setAppliancePlacementDate(rs
							.getDate("APPLIANCE_PLACEMENT_DATE"));
					fppsClaim.setBeginServiceDate(rs
							.getDate("BEGIN_SERVICE_DATE"));
					fppsClaim.setBeneAsgnCertInd(rs
							.getString("BENE_ASGN_CERT_IND"));
					fppsClaim.setBillTypeCd(rs.getString("BILL_TYPE_CD"));
					fppsClaim.setClaimIndex(rs.getLong("CLAIM_INDEX"));
					fppsClaim.setBillTypeCd(rs.getString("CLAIM_STATUS_CD"));
					fppsClaim.setCreatedBy(rs.getString("CREATED_BY"));
					fppsClaim.setDateCreated(rs.getDate("DATE_CREATED"));
					fppsClaim.setDateModified(rs.getDate("DATE_MODIFIED"));
					fppsClaim.setEdiClaimKey(rs.getString("EDI_CLAIM_KEY"));
					ServiceType s = new ServiceType();
					s.setServiceTypeCd(rs.getString("SERVICE_TYPE_CD"));
					fppsClaim.setServiceType(s);
				}
				rs.close();
			}
			pStatement.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		LOGGER.debug("End set Fipss_Owner row to Entity");
	}

	/**
	 * 
	 */
	final void setEreposClaim() {
		LOGGER.debug("Begin extract FPPS_OWNER and E_REPOS CLAIM Information");
		String sqlSelectAll = "SELECT e.*   FROM fpps_owner.claims c , e_repos.claim e WHERE e.claim_key = c.edi_claim_key and e.CLAIM_KEY not in (select a.CLAIM_KEY from e_repos.claim_277ca a) and rownum < 10";
		org.hibernate.Query claimQuery = oSession.createSQLQuery(sqlSelectAll)
				.addEntity(gov.va.med.domain.erepos.Claim.class);
		claims = claimQuery.scroll(ScrollMode.FORWARD_ONLY);
		LOGGER.debug("End extract FPPS_OWNER and E_REPOS CLAIM Information");
	}

	/**
	 * 
	 */
	private void setFunctionalGroup() {
		LOGGER.debug("Begin extract Functional Group Information");
		try {
			String sql = "select * from FUNCTIONAL_GROUP where FILE_NUMBER= ?";
			pStatement = conn.prepareStatement(sql);
			pStatement.setString(1, reposClaim.getFileNumber());
			ResultSet rs = pStatement.executeQuery();
			if (null != rs) {
				oFunctionalGroup = new FunctionalGroup();
				while (rs.next()) {
					oFunctionalGroup.setFileNumber(rs.getString("FILE_NUMBER"));
					oFunctionalGroup.setTxType(rs.getString("TX_TYPE"));
					oFunctionalGroup.setGe01nbrTranSets(rs
							.getString("GS01FCTL_ID_CD"));
					oFunctionalGroup.setGe02grpCtlNbr(rs
							.getString("GS02APP_SND_CD"));
					oFunctionalGroup.setGs03appRcvCd(rs
							.getString("GS03APP_RCV_CD"));
					oFunctionalGroup.setGs04creDt(rs.getString("GS04CRE_DT"));
					oFunctionalGroup.setGs05creTm(rs.getString("GS05CRE_TM"));
					oFunctionalGroup.setGs06grpCtlNbr(rs
							.getString("GS06GRP_CTL_NBR"));
					oFunctionalGroup.setGs07respAgcyCd(rs
							.getString("GS07RESP_AGCY_CD"));
					oFunctionalGroup.setGs08verRlsIndId(rs
							.getString("GS08VER_RLS_IND_ID"));
					oFunctionalGroup.setGe01nbrTranSets(rs
							.getString("GE01NBR_TRAN_SETS"));
					oFunctionalGroup.setGe02grpCtlNbr(rs
							.getString("GE02GRP_CTL_NBR"));
					oFunctionalGroup.setCreatedBy(rs.getString("CREATED_BY"));
					oFunctionalGroup.setModifiedBy(rs.getString("MODIFIED_BY"));
				}
				rs.close();
			}
			if (null != pStatement) {
				pStatement.close();
			}
		} catch (Exception exp) {
			exp.printStackTrace();
		}
		LOGGER.debug("End extract Functional Group Information");
	}

	/**
	 * 
	 */
	final void getFppsClaimStatusToProcess() {
		LOGGER.debug("Begin extract FPPS OWner Claims Information");
		try {
			String sql = "select CLAIM_STATUS_CD from FPPS_OWNER.CLAIMS where EDI_CLAIM_KEY = ?";
			pStatement = conn.prepareStatement(sql);
			pStatement.setString(1, ediClaimKey);
			ResultSet rs = pStatement.executeQuery();
			String statusCd = "";
			if (null != rs) {
				while (rs.next()) {
					statusCd = (String) rs.getString("CLAIM_STATUS_CD");
				}
				rs.close();
			}
			if (null != statusCd && statusCd.length() > 0) {
				sql = "select * from CLAIM_STATUS where CLAIM_STATUS_CD = ?";
				pStatement = conn.prepareStatement(sql);
				pStatement.setString(1, statusCd);
				rs = pStatement.executeQuery();
				if (null != rs) {
					ClaimStatus oClaimStatus = null;
					while (rs.next()) {
						oClaimStatus = new ClaimStatus();
						oClaimStatus.setClaimStatusCd(rs.getString(1));
						oClaimStatus.setDescription(rs.getString(2));
						oClaimStatus.setCreatedBy(rs.getString(3));
					}
					rs.close();
					fppsClaim.setClaimStatus(oClaimStatus);
				}
			}
			if (null != pStatement) {
				pStatement.close();
			}
		} catch (Exception exp) {
			exp.printStackTrace();
		}
		LOGGER.debug("End extract FPPS OWner Claims Information");
	}

	/**
	 * 
	 */
	final void persist277CA() {
		String sqlInsert = "";
		Blob blob = null;
		try {
			String checkRecSql = "select * from E_REPOS.CLAIM_277CA where CLAIM_KEY = ?";
			pStatement = conn.prepareStatement(checkRecSql);
			pStatement.setString(1, this.reposClaim.getClaimKey());
			ResultSet rs = pStatement.executeQuery();
			if (null == rs || !rs.next()) {
				sqlInsert = "insert into E_REPOS.CLAIM_277CA (\"CLAIM_KEY\", \"NOTIFY_SOURCE\", \"MESSAGE\", \"FILE_NAME\", \"STATUS_CODE\") values (?,?,?,?,?)";
				pStatement = conn.prepareStatement(sqlInsert);
				pStatement.setString(1, reposClaim.getClaimKey());
				pStatement.setString(2, "EREPOS");
				String tmp = readFileToString();
				blob = conn.createBlob();
				blob.setBytes(1, tmp.getBytes());
				pStatement.setBlob(3, blob);
				pStatement.setString(4, csFile.getName());
				pStatement.setString(5, reposClaim.getStatusCode());
			} else {
				sqlInsert = "update E_REPOS.CLAIM_277CA set NOTIFY_SOURCE = ?,  MESSAGE = ?, FILE_NAME= ?, STATUS_CODE = ? where CLAIM_KEY = ?";
				pStatement = conn.prepareStatement(sqlInsert);
				pStatement.setString(4, reposClaim.getClaimKey());
				pStatement.setString(1, "EREPOS");
				String tmp = readFileToString();
				blob = conn.createBlob();
				blob.setBytes(1, tmp.getBytes());
				pStatement.setBlob(2, blob);
				pStatement.setString(3, csFile.getName());
				pStatement.setString(4, reposClaim.getStatusCode());
			}
			pStatement.executeUpdate();
			pStatement.close();
			rs.close();
		} catch (Exception exp) {
			exp.printStackTrace();
		}
	}

	/**
	 * 
	 * @return String
	 */
	private String readFileToString() {
		FileInputStream fis = null;
		String str = "";
		try {
			fis = new FileInputStream(this.csFile);
			int content;
			while ((content = fis.read()) != -1) {
				str += (char) content;
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (fis != null)
					fis.close();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		return str;
	}

	/**
	 * 
	 * @return String
	 */
	private static String getCreatedDateyyyyMMdd() {
		return new SimpleDateFormat("yyyyMMdd").format(new Date());
	}

	/**
	 * 
	 * @return String
	 */
	private static String getCreatedDateyyMMdd() {
		return new SimpleDateFormat("yyMMdd").format(new Date());
	}

	/**
	 * 
	 * @return String
	 */
	private static String getCreatedTimeHHmm() {
		return new SimpleDateFormat("HHmm").format(new Date());
	}

	/**
	 * 
	 * @return String
	 */
	private static String getCreatedTimeHHmmssSS() {
		return new SimpleDateFormat("HHmmssSS").format(new Date());
	}

	/**
	 * 
	 * @return String
	 */
	private String getInterChangeCtrlNumber() {
		return reposClaim.getSt02txnCtlNbr();
	}

	/**
	 * 
	 * @return String
	 */
	private String getGroupCtrlNumber() {
		String strGrpCtlNum = reposClaim.getSt02txnCtlNbr();
		strGrpCtlNum = Integer.valueOf(strGrpCtlNum).toString();
		return strGrpCtlNum;
	}

	/**
	 * Component Element Separator :
	 * 
	 * @param sb
	 */
	private static void componentElementSeparator(StringBuffer sb) {
		sb.append(COMPONENT_ELEMENT_SEPARATOR);
	}

	/**
	 * Data Element Separator *
	 * 
	 * @param sb
	 */
	private static void dataElementSeparator(StringBuffer sb) {
		sb.append(DATA_ELEMENT_SEPARATOR);
	}

	/**
	 * Segment Terminator ~
	 * 
	 * @param sb
	 */
	private static void segmentTerminator(StringBuffer sb) {
		sb.append(SEGMENT_TERMINATOR);
	}

	/**
	 * 
	 * @param sb
	 */
	private static void repetationSeperator(StringBuffer sb) {
		sb.append(REPETITION_SEPARATOR);
	}

	/**
	 * 
	 * @return String
	 */
	protected static String getNow() {
		Date dNow = new Date();
		SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss:ms");
		String out = ft.format(dNow);
		return out;
	}

	/**
	 * 
	 * @param strInString
	 * @return String
	 */
	private static String getCheckNull(String strInString) {
		String strOutString = "";
		if (null != strInString && strInString.length() > 0) {
			strOutString = strInString.trim();
		}
		return strOutString;

	}
}
