package gov.va.med.hac.edi.extract.x214;

import gov.va.med.hac.edi.utils.jpa.JPAUtil;

import java.io.File;
import java.text.SimpleDateFormat;

import javax.naming.ConfigurationException;
import javax.persistence.EntityManager;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Options;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author            
 *
 */
public class HCCA277Generation {

	/** the class logger. */
	static final Logger LOGGER = LoggerFactory.getLogger(HCCA277Generation.class);
	private static final String HELP_OPT = "h";
	private static final String CONFIG_OPT = "f";
	private static final String OUTPUT_OPT = "o";

	/** private constructor to prevent unneeded instance creation. */
	private HCCA277Generation() {
	}
	
	/**
	 * 
	 * @return Options
	 */
	static Options getCliOptions() {
		Options options = new Options();
		options.addOption(HELP_OPT, "help", false, "show help.");
		options.addOption(CONFIG_OPT, "configfile", true, "The EntityManager Configuration file..");
		options.addOption(OUTPUT_OPT, "outputDir", true, "The 277CA EDI file Output directory.");
		return options;
	}

	/**
	 * Entry point for the Fee 277 Flat File Generation process. The JPA
	 * connection configuration file name can be passed into this method which
	 * will override the normal persistence.xml file found in the META-INF
	 * directory of the jar file. <br>
	 * <br>
	 * <code>usage: FeeFinal -f &lt;file&gt; [-h] -o
	 * &lt;outputdir&gt; [-q] -s &lt;senderid&gt;</code>
	 *
	 * @param args
	 *            set of options where &lt;file&gt; is the full path name of the
	 *            JPA configuration file.
	 * @param iLookBackEndDays
	 */
	public static void main(String[] args) {
		LOGGER.info("\n\n");
		LOGGER.info("<<<<<<<<<<<<< Beginning HealthCare Claims Acknowledgement 277 Creation <<<<<<<<<<<<<<");
		EntityManager em = null;
		Options opts = getCliOptions();
		SimpleDateFormat datefmtr = null;
		try {
			CommandLineParser parser = new GnuParser();
			CommandLine line = parser.parse(opts, args);
			
			if (line.hasOption(HELP_OPT)) {
				printUsage(opts);
				System.exit(1);
			}

			File configfile = null;
			// -f option in command line args
			if (line.hasOption(CONFIG_OPT)) {
				configfile = JPAUtil.getJpaConfigFile(line
						.getOptionValue(CONFIG_OPT));
			}

			if (configfile != null) {
				em = JPAUtil.getEntityManager(configfile);
				if (em == null) {
					throw new ConfigurationException(
							"Unable to configure Entity Manager.");
				}
			}

			String outputDir = null;
			if (line.hasOption(OUTPUT_OPT)) {
				outputDir = line.getOptionValue(OUTPUT_OPT);
			}
			//		path to output file, entity Manager..
			HCCA277GenerationHelper oHCCA277GenerationHelper = new HCCA277GenerationHelper();
			oHCCA277GenerationHelper.process(outputDir, em);
		} catch (MissingOptionException moe) {
			printUsage(opts);
		} catch (IllegalArgumentException iae) {
			printUsage(opts);
		} catch (Throwable t) {
			// System.out.println("main() - error: " + t.getMessage());
			t.printStackTrace();
			LOGGER.error("main() - Unable to Create Fee Claim Status file.", t);
		} finally {
			if ((em != null) && em.isOpen()) {
				em.close();
			}
			LOGGER.info("<<<<<<<<<<<<< Ending Claim Status Creation <<<<<<<<<<<<<<");
		}
	}

	/**
	 * 
	 * @param opts
	 */
	static final void printUsage(Options opts) {
		final int helpwidth = 80;
		HelpFormatter fmtr = new HelpFormatter();
		fmtr.printHelp(helpwidth, "Fee 277 Flat File Generation", "", opts, "",
				true);
	}
}
