package gov.va.med.hac.edi.reposload.fee;

import gov.va.med.hac.edi.utils.jpa.JPAUtil;

import java.io.File;
import java.text.SimpleDateFormat;

import javax.persistence.EntityManager;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class is responsible for moving Claims from the E_REPOS to the FPPS_OWNER
 * schema.
 * 
 */
public class FeeClaimLoad {

	static final Logger LOGGER = LoggerFactory.getLogger(FeeClaimLoad.class);
	static final SimpleDateFormat SDF = new SimpleDateFormat("yyyyMMdd");
	static final String DATESTR = SDF.format(new java.util.Date());
	static final Integer UNLOCK = new Integer(DATESTR);

	private FeeClaimLoad() {
	}

	/**
	 * This is the entry point for the FEE claim load process. This calls the DB
	 * stored procedure
	 * <code>fpps_owner.fee_claim_load.start_load_process</code>. The
	 * transaction state is maintained in the stored procedure.
	 * 
	 * @param args
	 *            no arguments needed
	 */
	public static void main(String[] args) {
		
		boolean DEBUG = true;
		
		
		EntityManager em = null;
		try {
			LOGGER.info("\n\n");
			LOGGER
					.info("<<<<<<<<<<<<< Beginning FEE Claim Load <<<<<<<<<<<<<<<<<<<<");
			CommandLine line = null;
			Options opts = getCliOptions();
			CommandLineParser parser = new GnuParser();
			line = parser.parse(opts, args);
			File configFile = null;
			String configFileName = line.getOptionValue("f");
			if (configFileName != null && configFileName.length() > 0) {
				configFile = JPAUtil.getJpaConfigFile(configFileName);
			} else {
				
// config file is wrong..........bad connection properties.............try FPPS_OWNER connection props				
				configFile = JPAUtil.getJpaConfigFile("ediHibernate.properties");
				// configFile = JPAUtil.getJpaConfigFile("fppsHibernate.properties");
			}
			
			em = JPAUtil.getEntityManager(configFile);
			
			
			String call = "{call fpps_owner.fpps_owner_load.start_load_process(?1)}";
			
if(DEBUG) {
	
	System.out.println("FeeClaimLoad.main() - calling stored proc");
}
			JPAUtil.runDbProcedure(em, call, UNLOCK);
			
			
if(DEBUG) {
	
	System.out.println("FeeClaimLoad.main() - DONE calling stored proc");
}			
			
			LOGGER.info("SUCCESSFULLY COMPLETED FEE CLAIM LOAD");
			
			System.out.println("SUCCESSFULLY COMPLETED FEE CLAIM LOAD");
			
			
		} catch (Throwable t) {
			
			if(DEBUG) {
				System.out.println("FeeClaimLoad.main() - error running SP against Oracle: " + t.getMessage());
				t.printStackTrace();
			}
			
			/**
			 * 
java.sql.SQLException: ORA-06550: line 1, column 7:
PLS-00201: identifier 'FPPS_OWNER.FPPS_OWNER_LOAD' must be declared
ORA-06550: line 1, column 7:
PL/SQL: Statement ignored

			 */
			
			LOGGER.error(t.getMessage(), t);
		} finally {
			if (em != null && em.isOpen()) {
				em.close();
			}
			LOGGER
					.info("<<<<<<<<<<<<< Ending FEE Claim Load <<<<<<<<<<<<<<<<<<<<");
		}
	}

	/**
	 * Helper method to populate command line options objects.
	 * 
	 * @return the <code>Options</code> object containing the set of command
	 *         line option.
	 */
	@SuppressWarnings("static-access")
	private static Options getCliOptions() {
		Options options = new Options();

		Option configfile = OptionBuilder
				.withArgName("jpaFile")
				.hasArg()
				.withLongOpt("jpaFile")
				.withDescription(
						"The JPA Configuration file, such as hibernate.properties.")
				.create("f");

		Option help = OptionBuilder.withLongOpt("help").withDescription(
				"Print this message.").create("h");

		options.addOption(help);
		options.addOption(configfile);
		return options;
	}
}
