package gov.va.med.hac.edi.reposload.x12.jpa;

import gov.va.med.hac.edi.utils.jpa.ConversionUtil;
import gov.va.med.hac.edi.utils.jpa.FileUtils;

import java.io.IOException;
import java.io.BufferedReader;
import java.math.BigDecimal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import javax.persistence.EntityManager;
import gov.va.med.domain.erepos.*;
import gov.va.med.domain.erepos.batchload.*;
import gov.va.med.domain.erepos.EntityAdtlId;

public class eReposLoad {
  static final Logger summaryLOGGER = LoggerFactory.getLogger(X12eReposLoad.class);
  static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + X12eReposLoad.class);

  private String loadRec = "";
  protected String vFileNumber = "";

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an IntchgControl object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadIntchgControl(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("IntchgControl as read:" + loadRec);
        
      final String[] tokens = loadRec.split("\\^", -1);
      final IntchgControl crIntchgControl = new IntchgControl();
      
      
      vFileNumber = tokens[0];

      crIntchgControl.setFileNumber(tokens[0]);
      
      
      
      crIntchgControl.setTxType(tokens[1]);
      crIntchgControl.setIsa01authQual(tokens[2]);
      crIntchgControl.setIsa02authInfo(tokens[3]);
      crIntchgControl.setIsa03scrInfoQual(tokens[4]);
      crIntchgControl.setIsa04scrInfo(tokens[5]);
      crIntchgControl.setIsa05sndIdQual(tokens[6]);
      crIntchgControl.setIsa06sndId(tokens[7]);
      crIntchgControl.setIsa07rcvIdQual(tokens[8]);
      crIntchgControl.setIsa08rcvId(tokens[9]);
      crIntchgControl.setIsa09intchgDt(tokens[10]);
      crIntchgControl.setIsa10intchgTm(tokens[11]);
      crIntchgControl.setIsa11intchgStd(ConversionUtil.stringToSingleChar(tokens[12]));
      crIntchgControl.setIsa12intchgVer(tokens[13]);
      crIntchgControl.setIsa13intchgNbr(tokens[14]);
      crIntchgControl.setIsa14ackRqst(ConversionUtil.stringToSingleChar(tokens[15]));
      crIntchgControl.setIsa15useInd(ConversionUtil.stringToSingleChar(tokens[16]));
      crIntchgControl.setIsa16cpntEleSep(ConversionUtil.stringToSingleChar(tokens[17]));
      crIntchgControl.setIea01nbrInclGrp(tokens[18]);
      crIntchgControl.setIea02intchgNbr(tokens[19]);
      
      traceLOGGER.trace("IntchgControl fieldvalues:" + crIntchgControl.toString());
      
     
     
     
         em.persist(crIntchgControl);
    }
    FileUtils.closeFile(in);
  
  }



/*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an FunctionalGroup object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadFunctionalGroup(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("FunctionalGroup:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final FunctionalGroup crFunctionalGroup = new FunctionalGroup();
      crFunctionalGroup.setFileNumber(tokens[0]);
      crFunctionalGroup.setTxType(tokens[1]);
      crFunctionalGroup.setGs01fctlIdCd(tokens[2]);
      crFunctionalGroup.setGs02appSndCd(tokens[3]);
      crFunctionalGroup.setGs03appRcvCd(tokens[4]);
      crFunctionalGroup.setGs04creDt(tokens[5]);
      crFunctionalGroup.setGs05creTm(tokens[6]);
      crFunctionalGroup.setGs06grpCtlNbr(tokens[7]);
      crFunctionalGroup.setGs07respAgcyCd(tokens[8]);
      crFunctionalGroup.setGs08verRlsIndId(tokens[9]);
      crFunctionalGroup.setGe01nbrTranSets(tokens[10]);
      crFunctionalGroup.setGe02grpCtlNbr(tokens[11]);
      traceLOGGER.trace("FunctionalGroup :" + crFunctionalGroup.getFileNumber() + ","
          + crFunctionalGroup.getTxType() + ","
          + crFunctionalGroup.getGs01fctlIdCd() + "," + crFunctionalGroup.getGs02appSndCd() + ","
          + crFunctionalGroup.getGs03appRcvCd() + "," + crFunctionalGroup.getGs04creDt() + ","
          + crFunctionalGroup.getGs05creTm() + "," + crFunctionalGroup.getGs06grpCtlNbr() + ","
          + crFunctionalGroup.getGs07respAgcyCd() + "," + crFunctionalGroup.getGs08verRlsIndId()
          + ","
          + crFunctionalGroup.getGe01nbrTranSets() + "," + crFunctionalGroup.getGe02grpCtlNbr());
      // traceLOGGER.trace("FunctionalGroup fieldvalues {}", crFunctionalGroup);
      em.persist(crFunctionalGroup);
    }
    FileUtils.closeFile(in);
    System.out.println("Created rows for FunctionalGroup table");
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an TxnSet/TxnSetID object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadTxnSet(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("TxnSet:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final TxnSet crTxnSet = new TxnSet();
      final TxnSetId crTxnSetId = new TxnSetId();

      crTxnSetId.setFileNumber(tokens[0]);
      crTxnSetId.setSt02txnCtlNbr(tokens[3]);
      crTxnSet.setId(crTxnSetId);

      crTxnSet.setTxType(tokens[1]);
      crTxnSet.setSt01txnId(tokens[2]);

      crTxnSet.setBht01hrchlStrc(tokens[4]);
      crTxnSet.setBht02txnPrpsCd(tokens[5]);
      crTxnSet.setBht03orgnTxnId(tokens[6]);
      crTxnSet.setBht04txnCreDt(tokens[7]);
      crTxnSet.setBht05txnCreTm(tokens[8]);
      crTxnSet.setBht06txnTypeCd(tokens[9]);
      crTxnSet.setSe01txnSgmtCnt(new BigDecimal(tokens[10]));
      crTxnSet.setSe02txnCtlNbr(tokens[11]);
      crTxnSet.setSt03implmtCnvntnReg(tokens[16]);
      traceLOGGER.trace("TxnSet values:" + crTxnSet.getId().getFileNumber() + ","
          + crTxnSet.getTxType() + ","
          + crTxnSet.getSt01txnId() + "," + crTxnSet.getSe02txnCtlNbr() + ","
          + crTxnSet.getBht01hrchlStrc() + ","
          + crTxnSet.getBht02txnPrpsCd() + "," + crTxnSet.getBht03orgnTxnId() + ","
          + crTxnSet.getBht04txnCreDt() + "," + crTxnSet.getBht05txnCreTm() + ","
          + crTxnSet.getBht06txnTypeCd() + ","
          + crTxnSet.getSe01txnSgmtCnt() + "," + crTxnSet.getSe02txnCtlNbr() + ","
          + crTxnSet.getSt03implmtCnvntnReg());
      // traceLOGGER.trace("TxnSet fieldvalues {}", crTxnSet);
      em.persist(crTxnSet);
    }
    FileUtils.closeFile(in);
    System.out.println("Created rows for TxnSet table");
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ForeignCurrencyDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadForeignCurrency(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ForeignCurrency:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final ForeignCurrencyDAO crForeignCurrency = new ForeignCurrencyDAO();
      crForeignCurrency.setForeignCurKey(tokens[0]);
      crForeignCurrency.setFileNumber(tokens[1]);
      crForeignCurrency.setSt02txnCtlNbr(tokens[2]);
      crForeignCurrency.setNm109billPrvdId(tokens[3]);
      crForeignCurrency.setTxType(tokens[4]);
      crForeignCurrency.setCur01crcyIdCd(tokens[5]);
      crForeignCurrency.setCur02crcyCd(tokens[6]); 	
      crForeignCurrency.setHrchlLevelKey(tokens[11]);
      traceLOGGER.trace("ForeignCurrency fieldvalues {}", crForeignCurrency);
      em.persist(crForeignCurrency);
    }
    FileUtils.closeFile(in);
    System.out.println("Created rows for ForeignCurrency table");
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an HrchlLevel object. Once the data is mapped it to the object it is saved
   * to the database using the EntityManager passed in (em);
   */
  protected void loadHrchlLevel(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("HrchlLevel:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final HrchlLevel crHrchlLevel = new HrchlLevel();
      crHrchlLevel.setHrchlLevelKey(tokens[0]);
      crHrchlLevel.setFileNumber(tokens[1]);
      crHrchlLevel.setSt02txnCtlNbr(tokens[2]);
      crHrchlLevel.setNm109billPrvdId(tokens[3]);
      crHrchlLevel.setNm109sbsrId(tokens[4]);
      crHrchlLevel.setLoopId(tokens[5]);
      crHrchlLevel.setHl01hrchlId(tokens[6]);
      crHrchlLevel.setHl02hrchlPrntId(tokens[7]);
      crHrchlLevel.setHl03hrchlLvlCd(tokens[8]);
      crHrchlLevel.setHl04hrchlChildCd(ConversionUtil.stringToSingleCharacter(tokens[9]));
      traceLOGGER.trace("HrchlLevel fieldvalues:" + crHrchlLevel.getHrchlLevelKey() + ","
          + crHrchlLevel.getFileNumber() + ","
          + crHrchlLevel.getSt02txnCtlNbr() + "," + crHrchlLevel.getNm109billPrvdId() + ","
          + crHrchlLevel.getNm109sbsrId() + ","
          + crHrchlLevel.getLoopId() + "," + crHrchlLevel.getHl01hrchlId() + ","
          + crHrchlLevel.getHl02hrchlPrntId() + ","
          + crHrchlLevel.getHl03hrchlLvlCd() + "," + crHrchlLevel.getHl04hrchlChildCd());
      // traceLOGGER.trace("HrchlLevel fieldvalues {}", crHrchlLevel);
      em.persist(crHrchlLevel);
    }
    FileUtils.closeFile(in);
    System.out.println("Created rows for HrchlLevel table");
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an SubscriberInfo object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadSubscriberInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("SubscriberInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      
      if(tokens.length < 19) {
    	  return;
      }
      final SubscriberInfoDAO crSubscriberInfo = new SubscriberInfoDAO();
      crSubscriberInfo.setSubscriberInfoKey(tokens[0]);
      crSubscriberInfo.setFileNumber(tokens[1]);
      crSubscriberInfo.setSt02txnCtlNbr(tokens[2]);
      crSubscriberInfo.setNm109billPrvdId(tokens[3]);
      crSubscriberInfo.setNm109sbsrId(tokens[4]);
      crSubscriberInfo.setClaimKey(tokens[5]);
      crSubscriberInfo.setLoopId(tokens[6]);
      crSubscriberInfo.setSeqNbr(new BigDecimal(tokens[7]));
      crSubscriberInfo.setSbr01pyrRespSeq(ConversionUtil.stringToSingleCharacter(tokens[8]));
      crSubscriberInfo.setSbr02indvRltCd(tokens[9]);
      crSubscriberInfo.setSbr03plcyNbr(tokens[10]);
      crSubscriberInfo.setSbr04plcyNm(tokens[11]);
      crSubscriberInfo.setSbr05insTypeCd(tokens[12]);
      crSubscriberInfo.setSbr06cobCd(ConversionUtil.stringToSingleCharacter(tokens[13]));
      crSubscriberInfo.setSbr09clmFileInd(tokens[14]);
   	  crSubscriberInfo.setHrchlLevelKey(tokens[19]);
   	  
   	  
      traceLOGGER.trace("SubscriberInfo fieldvalues {}", crSubscriberInfo);
      em.persist(crSubscriberInfo);
    }
    FileUtils.closeFile(in);
    System.out.println("Created rows for SubscriberInfo table");
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an PatientInfoDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadPatientInfo(BufferedReader in, EntityManager em)
      throws IOException {
	  
	boolean DEBUG = true;  
	  
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("PatientInfo:" + loadRec);
        
      final String[] tokens = loadRec.split("\\^", -1);
      final PatientInfoDAO crPatientInfo = new PatientInfoDAO();

      crPatientInfo.setPatientInfoKey(tokens[0]);
      crPatientInfo.setSubscriberInfoKey(tokens[1]);
      crPatientInfo.setFileNumber(tokens[2]);
      crPatientInfo.setSt02txnCtlNbr(tokens[3]);
      crPatientInfo.setNm109billPrvdId(tokens[4]);
      crPatientInfo.setNm109sbsrId(tokens[5]);
      crPatientInfo.setSeqNbr(new BigDecimal(tokens[6]));
      crPatientInfo.setLoopId(tokens[7]);
      crPatientInfo.setPat01indvRltCd(tokens[8]);
      crPatientInfo.setPat04stdntStatCd(ConversionUtil.stringToSingleCharacter(tokens[9]));
      crPatientInfo.setPat05dttmFmtQual(tokens[10]);
      crPatientInfo.setPat06deathDt(tokens[11]);
      crPatientInfo.setPat07uomCd(tokens[12]);
      if (!((tokens[13].equals("")) || (tokens[13] == null))) {
        crPatientInfo.setPat08ptWt(new BigDecimal(tokens[13]));
      }
      crPatientInfo.setPat09pregInd(ConversionUtil.stringToSingleCharacter(tokens[14]));
      
      
     
    	  crPatientInfo.setHrchlLevelKey(tokens[19]);
     
      traceLOGGER.trace("PatientInfo fieldvalues {}", crPatientInfo);
      
      em.persist(crPatientInfo);
    }
    FileUtils.closeFile(in);
    System.out.println("Created rows for PatientInfo table");
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ClaimDAO object. Once the data is mapped it to the object it is saved
   * to the database using the EntityManager passed in (em). Notice that this method uses a DAO
   * object to store data. It uses this DAO (which really isn't a DAO by the classical definition)
   * so that it may directly store some foreign keys found in the .DAT file in the database.
   * Normally, the keys would be implicitly stored as a result of an object to object relationship.
   * Since we don't have enough data at this point to build the proper object to object
   * relationship, we explicitly store the key via an key attribute on a DAO. There are a few fields
   * that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadClaim(BufferedReader in, EntityManager em)
      throws IOException {
	  
	boolean DEBUG=true; 
	  
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("Claim:" + loadRec);
      
        
      final String[] tokens = loadRec.split("\\^", -1);
      final ClaimDAO crClaim = new ClaimDAO();
      
    
      crClaim.setClaimKey(tokens[0]);
      crClaim.setFileNumber(tokens[1]);
      crClaim.setSt02txnCtlNbr(tokens[2]);	
 
      crClaim.setPatientInfoKey(tokens[3]);
      
      crClaim.setSubscriberInfoKey(tokens[4]);
      crClaim.setNm109billPrvdId(tokens[5]);
      crClaim.setNm109payPrvdId(tokens[6]);
      crClaim.setSenderId(tokens[7]);
      crClaim.setReceiverId(tokens[8]);
      crClaim.setSubscriberId(tokens[9]);
      crClaim.setPatientId(tokens[10]);
      crClaim.setClm01ptAcctNbr(tokens[11]);
      
      if (!((tokens[13].equals("")) || (tokens[12] == null))) {
        crClaim.setClm02clmChrgAmt(new BigDecimal(tokens[12]));
      }
      crClaim.setClm05fclyTypeCd(tokens[13]);
      if (!(tokens[14].equals(""))) {
        crClaim.setClm05fclyCdQual(tokens[14]);
      }
      
      crClaim.setClm05clmFrqCd(ConversionUtil.stringToSingleCharacter(tokens[15]));
      crClaim.setClm06prvdSgnInd(ConversionUtil.stringToSingleCharacter(tokens[16]));
      crClaim.setClm07mdcrAsgnCd(ConversionUtil.stringToSingleCharacter(tokens[17]));
      crClaim.setClm08asgnOfBnft(ConversionUtil.stringToSingleCharacter(tokens[18]));
      crClaim.setClm09relInfoCd(ConversionUtil.stringToSingleCharacter(tokens[19]));
      crClaim.setClm10patSignCd(tokens[20]);
      crClaim.setClm11rltCauses1(tokens[21]);
      crClaim.setClm11rltCauses2(tokens[22]);
      crClaim.setClm11rltCauses3(tokens[23]);
      crClaim.setClm11st(tokens[24]);
      crClaim.setClm11ctryCd(tokens[25]);
      crClaim.setClm12spclPgmCd(tokens[26]);
      crClaim.setClm16prvAgmentCd(tokens[27]);
      crClaim.setClm18explOfBnft(ConversionUtil.stringToSingleCharacter(tokens[28]));
      crClaim.setClm19predterBnft(tokens[29]);
      crClaim.setClm20delayRsnCd(tokens[30]);
      crClaim.setSubmissionIndex(tokens[31]);
      crClaim.setClaimType(tokens[32]);
      crClaim.setStatusCode("NEW");
  	  crClaim.setBillingHrchlLevelKey(tokens[38]);
   	  crClaim.setSubscriberHrchlLevelKey(tokens[39]);
   	  crClaim.setPatientHrchlLevelKey(tokens[40]);
   	  
      traceLOGGER.trace("Claim fieldvalues {}", crClaim.toString());
      
    
	       
      em.persist(crClaim);
    }
    
    FileUtils.closeFile(in);
  
    
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an HiCodes object. Once the data is mapped it to the object it is saved to
   * the database using the EntityManager passed in (em);
   */
	protected void loadHiCodes(BufferedReader in, EntityManager em)
			throws IOException {
		
	//	boolean DEBUG = true;
		int tokenLength = 113;
		
		while ((loadRec = in.readLine()) != null) {
			final String[] tokens = loadRec.split("\\^", -1);
		
	
			
			traceLOGGER.trace("HiCodes:" + loadRec);
			final HiCodes crHiCodes = new HiCodes();
			
			
			HiCodesId id = new HiCodesId(tokens[0], new BigDecimal(tokens[1]));
			
			crHiCodes.setId(id);
			crHiCodes.setHi01cdListQual(tokens[2]);
			crHiCodes.setHi01procCd(tokens[3]);
			crHiCodes.setHi01dttmQual(tokens[4]);
			
			int x = tokens.length;
			crHiCodes.setHi01procDt(tokens[5]);
			
			if (!((tokens[6].equals("")) || (tokens[6] == null))) {
				crHiCodes.setHi01amount(new BigDecimal(tokens[6]));
			}
			
			
			crHiCodes.setHi02cdListQual(tokens[7]);
			crHiCodes.setHi02procCd(tokens[8]);
			crHiCodes.setHi02dttmQual(tokens[9]);
			crHiCodes.setHi02procDt(tokens[10]);
			
			
			if (!((tokens[11].equals("")) || (tokens[11] == null))) {
				crHiCodes.setHi02amount(new BigDecimal(tokens[11]));
			}
			
			
			crHiCodes.setHi03cdListQual(tokens[12]);
			crHiCodes.setHi03procCd(tokens[13]);
			crHiCodes.setHi03dttmQual(tokens[14]);
			crHiCodes.setHi03procDt(tokens[15]);
			
			
			if (!((tokens[16].equals("")) || (tokens[16] == null))) {
				crHiCodes.setHi03amount(new BigDecimal(tokens[16]));
			}
			
			crHiCodes.setHi04cdListQual(tokens[17]);
			crHiCodes.setHi04procCd(tokens[18]);
			crHiCodes.setHi04dttmQual(tokens[19]);
			crHiCodes.setHi04procDt(tokens[20]);
			
			if (!((tokens[21].equals("")) || (tokens[21] == null))) {
				crHiCodes.setHi04amount(new BigDecimal(tokens[21]));
			}
			
			crHiCodes.setHi05cdListQual(tokens[22]);
			crHiCodes.setHi05procCd(tokens[23]);
			crHiCodes.setHi05dttmQual(tokens[24]);
			crHiCodes.setHi05procDt(tokens[25]);
			
			
			if (!((tokens[26].equals("")) || (tokens[26] == null))) {
				crHiCodes.setHi05amount(new BigDecimal(tokens[26]));
			}
			
			
			crHiCodes.setHi06cdListQual(tokens[27]);
			crHiCodes.setHi06procCd(tokens[28]);
			crHiCodes.setHi06dttmQual(tokens[29]);
			crHiCodes.setHi06procDt(tokens[30]);
			
			
			if (!((tokens[31].equals("")) || (tokens[31] == null))) {
				crHiCodes.setHi06amount(new BigDecimal(tokens[31]));
			}
			
			
			crHiCodes.setHi07cdListQual(tokens[32]);
			crHiCodes.setHi07procCd(tokens[33]);
			crHiCodes.setHi07dttmQual(tokens[34]);
			crHiCodes.setHi07procDt(tokens[35]);
			
			
			if (!((tokens[36].equals("")) || (tokens[36] == null))) {
				crHiCodes.setHi07amount(new BigDecimal(tokens[36]));
			}
			
			
			crHiCodes.setHi08cdListQual(tokens[37]);
			crHiCodes.setHi08procCd(tokens[38]);
			crHiCodes.setHi08dttmQual(tokens[39]);
			crHiCodes.setHi08procDt(tokens[40]);
			
			
			if (!((tokens[41].equals("")) || (tokens[41] == null))) {
				crHiCodes.setHi08amount(new BigDecimal(tokens[41]));
			}
			
			
			crHiCodes.setHi09cdListQual(tokens[42]);
			crHiCodes.setHi09procCd(tokens[43]);
			crHiCodes.setHi09dttmQual(tokens[44]);
			crHiCodes.setHi09procDt(tokens[45]);
			
			
			if (!((tokens[46].equals("")) || (tokens[46] == null))) {
				crHiCodes.setHi09amount(new BigDecimal(tokens[46]));
			}
			crHiCodes.setHi10cdListQual(tokens[47]);
			crHiCodes.setHi10procCd(tokens[48]);
			crHiCodes.setHi10dttmQual(tokens[49]);
			crHiCodes.setHi10procDt(tokens[50]);
			
			
			if (!((tokens[51].equals("")) || (tokens[51] == null))) {
				crHiCodes.setHi10amount(new BigDecimal(tokens[51]));
			}
			
			
			
			crHiCodes.setHi11cdListQual(tokens[52]);
			crHiCodes.setHi11procCd(tokens[53]);
			crHiCodes.setHi11dttmQual(tokens[54]);
			crHiCodes.setHi11procDt(tokens[55]);
			
			
			if (!((tokens[56].equals("")) || (tokens[56] == null))) {
				crHiCodes.setHi11amount(new BigDecimal(tokens[56]));
			}
			
			
			crHiCodes.setHi12cdListQual(tokens[57]);
			crHiCodes.setHi12procCd(tokens[58]);
			crHiCodes.setHi12dttmQual(tokens[59]);
			crHiCodes.setHi12procDt(tokens[60]);
			
			
			if (!((tokens[61].equals("")) || (tokens[61] == null))) {
				crHiCodes.setHi12amount(new BigDecimal(tokens[61]));
			}

			if (!((tokens[66].equals("")) || (tokens[66] == null))) {
				crHiCodes.setHi01qnty(new BigDecimal(tokens[66]));
			}
			
			
			crHiCodes.setHi01versionId(tokens[67]);
			crHiCodes.setHi01indstryCd(tokens[68]);
			crHiCodes.setHi01cndtnResponseCd(
					(tokens[69]));
			if (!((tokens[70].equals("")) || (tokens[70] == null))) {
				crHiCodes.setHi02qnty(new BigDecimal(tokens[70]));
			}
			crHiCodes.setHi02versionId(tokens[71]);
			crHiCodes.setHi02indstryCd(tokens[72]);
			crHiCodes.setHi02cndtnResponseCd((tokens[73]));
			
			
			if (!((tokens[74].equals("")) || (tokens[74] == null))) {
				crHiCodes.setHi03qnty(new BigDecimal(tokens[74]));
			}
			
			
			crHiCodes.setHi03versionId(tokens[75]);
			crHiCodes.setHi03indstryCd(tokens[76]);
			crHiCodes.setHi03cndtnResponseCd((tokens[77]));
			
			
			if (!((tokens[78].equals("")) || (tokens[78] == null))) {
				crHiCodes.setHi04qnty(new BigDecimal(tokens[78]));
			}
			
			
			crHiCodes.setHi04versionId(tokens[79]);
			crHiCodes.setHi04indstryCd(tokens[80]);
			crHiCodes.setHi04cndtnResponseCd(
					(tokens[81]));
			if (!((tokens[82].equals("")) || (tokens[82] == null))) {
				crHiCodes.setHi05qnty(new BigDecimal(tokens[82]));
			}
			
			crHiCodes.setHi05versionId(tokens[83]);
			crHiCodes.setHi05indstryCd(tokens[84]);
			crHiCodes.setHi05cndtnResponseCd((tokens[85]));
			
			
			if (!((tokens[86].equals("")) || (tokens[86] == null))) {
				crHiCodes.setHi06qnty(new BigDecimal(tokens[86]));
			}
			
			crHiCodes.setHi06versionId(tokens[87]);
			crHiCodes.setHi06indstryCd(tokens[88]);
			crHiCodes.setHi06cndtnResponseCd(
					(tokens[89]));
			if (!((tokens[90].equals("")) || (tokens[90] == null))) {
				crHiCodes.setHi07qnty(new BigDecimal(tokens[90]));
			}
			
			crHiCodes.setHi07versionId(tokens[91]);
			crHiCodes.setHi07indstryCd(tokens[92]);
			crHiCodes.setHi07cndtnResponseCd((tokens[93]));
			
			
			if (!((tokens[94].equals("")) || (tokens[94] == null))) {
				crHiCodes.setHi08qnty(new BigDecimal(tokens[94]));
			}
			
			
			crHiCodes.setHi08versionId(tokens[95]);
			crHiCodes.setHi08indstryCd(tokens[96]);
			crHiCodes.setHi08cndtnResponseCd((tokens[97]));
			
			
			if (!((tokens[98].equals("")) || (tokens[98] == null))) {
				crHiCodes.setHi09qnty(new BigDecimal(tokens[98]));
			}
			
			
			crHiCodes.setHi09versionId(tokens[99]);
			crHiCodes.setHi09indstryCd(tokens[100]);
			crHiCodes.setHi09cndtnResponseCd((tokens[101]));
			
			
			if (!((tokens[102].equals("")) || (tokens[102] == null))) {
				crHiCodes.setHi10qnty(new BigDecimal(tokens[102]));
			}
			
			
			crHiCodes.setHi10versionId(tokens[103]);
			crHiCodes.setHi10indstryCd(tokens[104]);
			crHiCodes.setHi10cndtnResponseCd(
					(tokens[105]));
			
			
			if (!((tokens[106].equals("")) || (tokens[106] == null))) {
				crHiCodes.setHi11qnty(new BigDecimal(tokens[106]));
			}
			crHiCodes.setHi11versionId(tokens[107]);
			crHiCodes.setHi11indstryCd(tokens[108]);
			crHiCodes.setHi11cndtnResponseCd((tokens[109]));
			
			
			if (!((tokens[110].equals("")) || (tokens[110] == null))) {
				crHiCodes.setHi12qnty(new BigDecimal(tokens[110]));
			}
			
			
			crHiCodes.setHi12versionId(tokens[111]);
			crHiCodes.setHi12indstryCd(tokens[112]);
			crHiCodes.setHi12cndtnResponseCd(
					(tokens[113]));

			
			
			 traceLOGGER.trace("HiCodes fieldvalues {}", crHiCodes);
			
			em.persist(crHiCodes);
		}
		FileUtils.closeFile(in);
		
	}

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an MediInptAdjdctDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadMediInptAdjdct(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("MediInptAdjdct:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final MediInptAdjdctDAO crMediInptAdjdct = new MediInptAdjdctDAO();
      crMediInptAdjdct.setMiaKey(tokens[0]);
      crMediInptAdjdct.setClaimKey(tokens[1]);
      crMediInptAdjdct.setClaimPymtInfoKey(tokens[2]);
      crMediInptAdjdct.setSeqNbr(new BigDecimal(tokens[3]));
      crMediInptAdjdct.setTxType(tokens[4]);
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crMediInptAdjdct.setMia01cvrVisits(new BigDecimal(tokens[5]));
      }
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crMediInptAdjdct.setMia02ltRsvDays(new BigDecimal(tokens[6]));
      }
      if (!((tokens[7].equals("")) || (tokens[7] == null))) {
        crMediInptAdjdct.setMia03ltPsychDays(new BigDecimal(tokens[7]));
      }
      if (!((tokens[8].equals("")) || (tokens[8] == null))) {
        crMediInptAdjdct.setMia04drgAmt(new BigDecimal(tokens[8]));
      }
      if (!((tokens[9].equals("")) || (tokens[9] == null))) {
        crMediInptAdjdct.setMia05rmkCd1(tokens[9]);
      }
      if (!((tokens[10].equals("")) || (tokens[10] == null))) {
        crMediInptAdjdct.setMia06dsprpntAmt(new BigDecimal(tokens[10]));
      }
      if (!((tokens[11].equals("")) || (tokens[11] == null))) {
        crMediInptAdjdct
            .setMia07mspPthruAmt(new BigDecimal(tokens[11]));
      }
      if (!((tokens[12].equals("")) || (tokens[12] == null))) {
        crMediInptAdjdct.setMia08ppsCptlAmt(new BigDecimal(tokens[12]));
      }
      if (!((tokens[13].equals("")) || (tokens[13] == null))) {
        crMediInptAdjdct.setMia09cptlFspDrg(new BigDecimal(tokens[13]));
      }
      if (!((tokens[14].equals("")) || (tokens[14] == null))) {
        crMediInptAdjdct.setMia10cptlHspDrg(new BigDecimal(tokens[14]));
      }
      if (!((tokens[15].equals("")) || (tokens[15] == null))) {
        crMediInptAdjdct.setMia11cptlDhsDrg(new BigDecimal(tokens[15]));
      }
      if (!((tokens[16].equals("")) || (tokens[16] == null))) {
        crMediInptAdjdct.setMia12oldCptlAmt(new BigDecimal(tokens[16]));
      }
      if (!((tokens[17].equals("")) || (tokens[17] == null))) {
        crMediInptAdjdct.setMia13cptlIme(new BigDecimal(tokens[17]));
      }
      if (!((tokens[18].equals("")) || (tokens[18] == null))) {
        crMediInptAdjdct.setMia14operHsp(new BigDecimal(tokens[18]));
      }
      if (!((tokens[19].equals("")) || (tokens[19] == null))) {
        crMediInptAdjdct.setMia15cstRptDays(new BigDecimal(tokens[19]));
      }
      if (!((tokens[20].equals("")) || (tokens[20] == null))) {
        crMediInptAdjdct.setMia16operFsp(new BigDecimal(tokens[20]));
      }
      if (!((tokens[21].equals("")) || (tokens[21] == null))) {
        crMediInptAdjdct.setMia17cptlOtlr(new BigDecimal(tokens[21]));
      }
      if (!((tokens[22].equals("")) || (tokens[22] == null))) {
        crMediInptAdjdct.setMia18idirTchAmt(new BigDecimal(tokens[22]));
      }
      if (!((tokens[23].equals("")) || (tokens[23] == null))) {
        crMediInptAdjdct.setMia19prfsnCpmt(new BigDecimal(tokens[23]));
      }
      crMediInptAdjdct.setMia20rmkCd2(tokens[24]);
      crMediInptAdjdct.setMia21rmkCd3(tokens[25]);
      crMediInptAdjdct.setMia22rmkCd4(tokens[26]);
      crMediInptAdjdct.setMia23rmkCd5(tokens[27]);
      if (!((tokens[28].equals("")) || (tokens[28] == null))) {
        crMediInptAdjdct.setMia24cptlExcp(new BigDecimal(tokens[28]));
      }
      crMediInptAdjdct.setSubscriberInfoKey(tokens[33]);
      
      traceLOGGER.trace("MediInptAdjdct fieldvalues {}", crMediInptAdjdct);
      em.persist(crMediInptAdjdct);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an MediOutptAdjdctDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadMediOutptAdjdct(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("MediOutptAdjdct:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final MediOutptAdjdctDAO crMediOutptAdjdct = new MediOutptAdjdctDAO();
      crMediOutptAdjdct.setMoaKey(tokens[0]);
      crMediOutptAdjdct.setClaimKey(tokens[1]);
      crMediOutptAdjdct.setClaimPymtInfoKey(tokens[2]);
      crMediOutptAdjdct.setSeqNbr(new BigDecimal(tokens[3]));
      crMediOutptAdjdct.setTxType(tokens[4]);
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crMediOutptAdjdct.setMoa01rmbrsRt(new BigDecimal(tokens[5]));
      }
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crMediOutptAdjdct.setMoa02hcpcsPybl(new BigDecimal(tokens[6]));
      }
      crMediOutptAdjdct.setMoa03rmkCd1(tokens[7]);
      crMediOutptAdjdct.setMoa04rmkCd2(tokens[8]);
      crMediOutptAdjdct.setMoa05rmkCd3(tokens[9]);
      crMediOutptAdjdct.setMoa06rmkCd4(tokens[10]);
      crMediOutptAdjdct.setMoa07rmkCd5(tokens[11]);
      if (!((tokens[12].equals("")) || (tokens[12] == null))) {
        crMediOutptAdjdct
            .setMoa08esrdPymtAmt(new BigDecimal(tokens[12]));
      }
      if (!((tokens[13].equals("")) || (tokens[13] == null))) {
        crMediOutptAdjdct.setMoa09prfsnCpmt(new BigDecimal(tokens[13]));
      }
      crMediOutptAdjdct.setSubscriberInfoKey(tokens[18]);
      traceLOGGER.trace("MediOutptAdjdct fieldvalues {}", crMediOutptAdjdct);
      em.persist(crMediOutptAdjdct);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an OtInsuranceCov object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadOtInsuranceCov(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("OtInsuranceCov:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final OtInsuranceCovDAO crOtInsuranceCov = new OtInsuranceCovDAO();
      OtInsuranceCovId id = new OtInsuranceCovId(tokens[0], new BigDecimal(tokens[1]));
      crOtInsuranceCov.setId(id);
      crOtInsuranceCov.setOi03asgnBnftInd(ConversionUtil.stringToSingleCharacter(tokens[2]));
      crOtInsuranceCov.setOi04ptSgnSrcCd(ConversionUtil.stringToSingleCharacter(tokens[3]));
      crOtInsuranceCov.setOi06relOfInfoCd(ConversionUtil.stringToSingleCharacter(tokens[4]));
      crOtInsuranceCov.setSubscriberInfoKey(tokens[9]);
      traceLOGGER.trace("OtInsuranceCov fieldvalues:" + crOtInsuranceCov.getId().getClaimKey()
          + "," + crOtInsuranceCov.getOi03asgnBnftInd() + ","
          + crOtInsuranceCov.getOi04ptSgnSrcCd() + "," + crOtInsuranceCov.getOi06relOfInfoCd());
      // traceLOGGER.trace("OtInsuranceCov fieldvalues {}", crOtInsuranceCov);
      em.persist(crOtInsuranceCov);
    }
    FileUtils.closeFile(in);
 
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an HomeHcarePlanIn/HomeHcarePlanInId object. Once the data is mapped it to
   * the object it is saved to the database using the EntityManager passed in (em);
   */
  protected void loadHomeHcarePlanIn(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("HomeHcarePlanIn:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final HomeHcarePlanIn crHomeHcarePlanIn = new HomeHcarePlanIn();
      HomeHcarePlanInId id = new HomeHcarePlanInId(tokens[0], new BigDecimal(tokens[1]));
      crHomeHcarePlanIn.setId(id);
      crHomeHcarePlanIn.setCr701dscplnType(tokens[2]);
      if (!((tokens[3].equals("")) || (tokens[3] == null))) {
        crHomeHcarePlanIn
            .setCr702visitsRcert(new BigDecimal(tokens[3]));
      }
      if (!((tokens[4].equals("")) || (tokens[4] == null))) {
        crHomeHcarePlanIn.setCr703visitsPrj(new BigDecimal(tokens[4]));
      }
      em.persist(crHomeHcarePlanIn);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ContractInfoDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
	protected void loadContractInfo(BufferedReader in, EntityManager em)
			throws IOException {
		while ((loadRec = in.readLine()) != null) {
			traceLOGGER.trace("HomeHcarePlanIn:" + loadRec);
			final String[] tokens = loadRec.split("\\^", -1);
			final ContractInfoDAO crContractInfo = new ContractInfoDAO();
			// TWP: 5010 This method was significantly re-arranged
			crContractInfo.setClaimKey(tokens[0]);
			crContractInfo.setCn101ctrType(tokens[1]);
			if (!((tokens[2].equals("")) || (tokens[2] == null))) {
				crContractInfo.setCn102ctrAmt(new BigDecimal(tokens[2]));
			}
			if (!((tokens[3].equals("")) || (tokens[3] == null))) {
				crContractInfo.setCn103ctrPct(new BigDecimal(tokens[3]));
			}
			crContractInfo.setCn104ctrCd(tokens[4]);
			if (!((tokens[5].equals("")) || (tokens[5] == null))) {
				crContractInfo.setCn105termDiscPct(new BigDecimal(tokens[5]));
			}
			crContractInfo.setCn106verId(tokens[6]);

			crContractInfo.setPrfsnServiceKey(tokens[11]);
			crContractInfo.setContractInfoKey(tokens[12]);
			crContractInfo.setDentalServiceKey(tokens[13]);

			traceLOGGER.trace("ContractInfo fieldvalues {}", crContractInfo);
			em.persist(crContractInfo);
		}
		FileUtils.closeFile(in);
		
	}

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an OrthodonticInfo object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadOrthodonticInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("OrthodonticInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final OrthodonticInfo crOrthodonticInfo = new OrthodonticInfo();
      crOrthodonticInfo.setClaimKey(tokens[0]);
      //Sreedhar - Checking for null values for tokens 1 and 2
      if (!((tokens[1].equals("")) || (tokens[1] == null))) {
          crOrthodonticInfo.setDn101orthTrmtMonthsCnt(new BigDecimal(
                  tokens[1]));    	  
      }

      if (!((tokens[2].equals("")) || (tokens[2] == null))) {
          crOrthodonticInfo.setDn102orthTrmtMonthsRemCnt(new BigDecimal(
                  tokens[2]));    	  
      }

      crOrthodonticInfo.setDn103orthRespCd(ConversionUtil.stringToSingleCharacter(tokens[3]));
      crOrthodonticInfo.setDn104orthDesc(tokens[8]);
      em.persist(crOrthodonticInfo);
    }
    FileUtils.closeFile(in);
  
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ToothStatus object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadToothStatus(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ToothStatus:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final ToothStatus crToothStatus = new ToothStatus();
      ToothStatusId id = new ToothStatusId(tokens[0], new BigDecimal(tokens[1]));
      crToothStatus.setId(id);
      crToothStatus.setDn201toothNbr(tokens[2]);
      crToothStatus.setDn202toothStat(tokens[3]);
      traceLOGGER.trace("ToothStatus fieldvalues:" + crToothStatus.getId().getClaimKey() + ","
          + crToothStatus.getDn201toothNbr() + "," + crToothStatus.getDn202toothStat());
      // traceLOGGER.trace("ToothStatus fieldvalues {}", crToothStatus);
      em.persist(crToothStatus);
    }
    FileUtils.closeFile(in);
  
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an InstClaimCode object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadInstClaimCode(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("InstClaimCode:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final InstClaimCode crInstClaimCode = new InstClaimCode();
      ;
      crInstClaimCode.setClaimKey(tokens[0]);
      crInstClaimCode.setCl101admTypeCd(ConversionUtil.stringToSingleCharacter(tokens[1]));
      crInstClaimCode.setCl102admSrcCd(ConversionUtil.stringToSingleCharacter(tokens[2]));
      crInstClaimCode.setCl103ptStatCd(tokens[3]);
      traceLOGGER.trace("InstClaimCode fieldvalues:" + crInstClaimCode.getClaimKey() + ","
          + crInstClaimCode.getCl101admTypeCd() + ","
          + crInstClaimCode.getCl102admSrcCd() + "," + crInstClaimCode.getCl103ptStatCd());
      // traceLOGGER.trace("InstClaimCode fieldvalues {}", crInstClaimCode);
      em.persist(crInstClaimCode);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an InstSvcLineDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadInstSvcLine(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("InstSvcLine:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final InstSvcLineDAO crInstSvcLine = new InstSvcLineDAO();
      ;
      crInstSvcLine.setClaimKey(tokens[0]);
      crInstSvcLine.setServiceKey(tokens[1]);
      crInstSvcLine.setLx01asgnLnNbr(new BigDecimal(tokens[2]));
      crInstSvcLine.setSv201svcLnRevCd(tokens[3]);
      crInstSvcLine.setSv202svcIdQual(tokens[4]);
      crInstSvcLine.setSv202procCd(tokens[5]);
      crInstSvcLine.setSv202procMod1(tokens[6]);
      crInstSvcLine.setSv202procMod2(tokens[7]);
      crInstSvcLine.setSv202procMod3(tokens[8]);
      crInstSvcLine.setSv202procMod4(tokens[9]);
      if (!((tokens[10].equals("")) || (tokens[10] == null))) {
        crInstSvcLine.setSv203lnItmChrg(new BigDecimal(tokens[10]));
      }
      crInstSvcLine.setSv204uomCd(tokens[11]);
      if (!((tokens[12].equals("")) || (tokens[12] == null))) {
        crInstSvcLine.setSv205svcUnitCnt(new BigDecimal(tokens[12]));
      }
      if (!((tokens[13].equals("")) || (tokens[13] == null))) {
        crInstSvcLine.setSv206svcLnRt(new BigDecimal(tokens[13]));
      }
      if (!((tokens[14].equals("")) || (tokens[14] == null))) {
        crInstSvcLine.setSv207denyChrgAmt(new BigDecimal(tokens[14]));
      }
      crInstSvcLine.setSv2028prdctSvcId(tokens[19]);
      crInstSvcLine.setSv202Desc(tokens[20]);
      
      traceLOGGER.trace("InstSvcLine fieldvalues {}", crInstSvcLine);
      em.persist(crInstSvcLine);
    }
    FileUtils.closeFile(in);
 
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an PrfsnSvcLineDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadPrfsnSvcLine(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("PrfsnSvcLine:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final PrfsnSvcLineDAO crPrfsnSvcLine = new PrfsnSvcLineDAO();
      ;
      crPrfsnSvcLine.setClaimKey(tokens[0]);
      crPrfsnSvcLine.setServiceKey(tokens[1]);
      crPrfsnSvcLine.setLx01asgnLnItm(new BigDecimal(tokens[2]));
      crPrfsnSvcLine.setSv101procCdQual(tokens[3]);
      crPrfsnSvcLine.setSv101procCd(tokens[4]);
      crPrfsnSvcLine.setSv101procMod1(tokens[5]);
      crPrfsnSvcLine.setSv101procMod2(tokens[6]);
      crPrfsnSvcLine.setSv101procMod3(tokens[7]);
      crPrfsnSvcLine.setSv101procMod4(tokens[8]);
      if (!((tokens[9].equals("")) || (tokens[9] == null))) {
        crPrfsnSvcLine.setSv102lineItmChrg(new BigDecimal(tokens[9]));
      }
      crPrfsnSvcLine.setSv103uomCd(tokens[10]);
      if (!((tokens[11].equals("")) || (tokens[11] == null))) {
        crPrfsnSvcLine.setSv104svcUnitCnt(new BigDecimal(tokens[11]));
      }
      crPrfsnSvcLine.setSv105placeSvcCd(tokens[12]);
      crPrfsnSvcLine.setSv107diagCdPtr1(tokens[13]);
      crPrfsnSvcLine.setSv107diagCdPtr2(tokens[14]);
      crPrfsnSvcLine.setSv107diagCdPtr3(tokens[15]);
      crPrfsnSvcLine.setSv107diagCdPtr4(tokens[16]);
      crPrfsnSvcLine.setSv109emergencyInd(ConversionUtil.stringToSingleCharacter(tokens[17]));
      crPrfsnSvcLine.setSv111epsdtInd(ConversionUtil.stringToSingleCharacter(tokens[18]));
      crPrfsnSvcLine.setSv112fmlyPlnInd(ConversionUtil.stringToSingleCharacter(tokens[19]));
      crPrfsnSvcLine.setSv115copayStatCd(ConversionUtil.stringToSingleCharacter(tokens[20]));
      crPrfsnSvcLine.setSv1018prdctSvcId(tokens[25]);
      crPrfsnSvcLine.setSv101Desc(tokens[26]);
      traceLOGGER.trace("PrfsnSvcLine fieldvalues {}", crPrfsnSvcLine);
      em.persist(crPrfsnSvcLine);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an DentalSvcLineDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadDentalSvcLine(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("DentalSvcLine:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final DentalSvcLineDAO crDentalSvcLine = new DentalSvcLineDAO();
      ;
      crDentalSvcLine.setClaimKey(tokens[0]);
      crDentalSvcLine.setServiceKey(tokens[1]);
      crDentalSvcLine.setLx01asgnLnItm(new BigDecimal(tokens[2]));
      crDentalSvcLine.setSv301adaCdQual(tokens[3]);
      crDentalSvcLine.setSv301adaCd(tokens[4]);
      crDentalSvcLine.setSv301procMod1(tokens[5]);
      crDentalSvcLine.setSv301procMod2(tokens[6]);
      crDentalSvcLine.setSv301procMod3(tokens[7]);
      crDentalSvcLine.setSv301procMod4(tokens[8]);
      if (!((tokens[9].equals("")) || (tokens[9] == null))) {
        crDentalSvcLine.setSv302lnItmChrg(new BigDecimal(tokens[9]));
      }
      crDentalSvcLine.setSv303fclyTypeCd(tokens[10]);
      crDentalSvcLine.setSv304oralDsgt1(tokens[11]);
      crDentalSvcLine.setSv304oralDsgt2(tokens[12]);
      crDentalSvcLine.setSv304oralDsgt3(tokens[13]);
      crDentalSvcLine.setSv304oralDsgt4(tokens[14]);
      crDentalSvcLine.setSv304oralDsgt5(tokens[15]);
      crDentalSvcLine.setSv305plcmtCd(ConversionUtil.stringToSingleCharacter(tokens[16]));
      if (!((tokens[17].equals("")) || (tokens[17] == null))) {
        crDentalSvcLine.setSv306procCnt(new BigDecimal(tokens[17]));
      }
      
      if(!("".equals(tokens[22])) && ConversionUtil.checkForNumericType(tokens[22]))
      {
    	  crDentalSvcLine.setSv311diagPntr1(ConversionUtil.convertTypeByte(tokens[22]));
      }
      if(!("".equals(tokens[23])) && ConversionUtil.checkForNumericType(tokens[23]))
      {
    	  crDentalSvcLine.setSv311diagPntr2(ConversionUtil.convertTypeByte(tokens[23]));
      }
      if(!("".equals(tokens[24])) && ConversionUtil.checkForNumericType(tokens[24]))
      {
    	  crDentalSvcLine.setSv311diagPntr3(ConversionUtil.convertTypeByte(tokens[24]));
      }
      if(!("".equals(tokens[25])) && ConversionUtil.checkForNumericType(tokens[25]))
      {
    	  crDentalSvcLine.setSv311diagPntr4(ConversionUtil.convertTypeByte(tokens[25]));
      }
      crDentalSvcLine.setSv301Desc(tokens[26]);
    		  
      traceLOGGER.trace("DentalSvcLine fieldvalues {}", crDentalSvcLine);
      em.persist(crDentalSvcLine);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an PrvSpclInfoDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadPrvSpclInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("PrvSpclInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final PrvSpclInfoDAO crPrvSpclInfo = new PrvSpclInfoDAO();
    
      crPrvSpclInfo.setPrvSpclInfoKey(tokens[0]);
      crPrvSpclInfo.setFileNumber(tokens[1]);
      crPrvSpclInfo.setSt02txnCtlNbr(tokens[2]);
      crPrvSpclInfo.setNm109billPrvdId(tokens[3]);
      crPrvSpclInfo.setNm109sbsrId(tokens[4]);
      crPrvSpclInfo.setClaimKey(tokens[5]);
      crPrvSpclInfo.setDentalServiceKey(tokens[6]);
      crPrvSpclInfo.setPrfsnServiceKey(tokens[7]);
      crPrvSpclInfo.setLoopId(tokens[8]);
      crPrvSpclInfo.setSeqNbr(new BigDecimal(tokens[9]));
      crPrvSpclInfo.setPrv01prvdTypeCd(tokens[10]);
      crPrvSpclInfo.setPrv02prvdSpcQual(tokens[11]);
      crPrvSpclInfo.setPrv03prvdSpcCd(tokens[12]);   	
      crPrvSpclInfo.setHrchlLevelKey(tokens[17]);
      traceLOGGER.trace("PrvSpclInfo fieldvalues {}", crPrvSpclInfo);
      em.persist(crPrvSpclInfo);
    }
    FileUtils.closeFile(in);
 
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an EntityNameDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadEntityName(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("EntityName:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final EntityNameDAO crEntityName = new EntityNameDAO();

      crEntityName.setEntityNameKey(tokens[0]);
      crEntityName.setFileNumber(tokens[1]);
      crEntityName.setSt02txnCtlNbr(tokens[2]);
      crEntityName.setNm109billPrvdId(tokens[3]);
      crEntityName.setNm109sbsrId(tokens[4]);
      crEntityName.setNm109ptntId(tokens[5]);
      crEntityName.setClaimKey(tokens[6]);
      crEntityName.setInstServiceKey(tokens[7]);
      crEntityName.setPrfsnServiceKey(tokens[8]);
      crEntityName.setDentalServiceKey(tokens[9]);
      crEntityName.setClaimPymtInfoKey(tokens[10]);
      crEntityName.setLoopId(tokens[11]);
      crEntityName.setTxType(tokens[12]);
      crEntityName.setNm101entyIdCd(tokens[13]);
      
      crEntityName.setNm102entyQual(ConversionUtil.stringToSingleCharacter(tokens[14]));
      
      crEntityName.setNm103lastNm(tokens[15]);
      crEntityName.setNm104fstNm(tokens[16]);
      crEntityName.setNm105midNm(tokens[17]);
      crEntityName.setNm107nmSfx(tokens[18]);
      crEntityName.setNm108idCdQual(tokens[19]);
      crEntityName.setNm109idCd(tokens[20]);
      crEntityName.setNm112lastNmOrganizNm(tokens[25]);
      crEntityName.setNm106nmPfx(tokens[26]);
      crEntityName.setNm110entyRelCd(tokens[27]);
      crEntityName.setNm111entyIdCd(tokens[28]);
      crEntityName.setHrchlLevelKey(tokens[29]);
      crEntityName.setSubscriberInfoKey(tokens[30]);
      crEntityName.setOhiSubscriberKey(tokens[30]);
      traceLOGGER.trace("EntityName fieldvalues {}", crEntityName);
      em.persist(crEntityName);
    }
    FileUtils.closeFile(in);
 
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an EntityAdtlIdDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadEntityAdtlId(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("EntityAdtlId:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      
       
      final EntityAdtlIdDAO crEntityAdtlId = new EntityAdtlIdDAO();
      
      crEntityAdtlId.setRefSegKey(tokens[0]);
      crEntityAdtlId.setFileNumber(tokens[1]);
      crEntityAdtlId.setSt02txnCtlNbr(tokens[2]);
      crEntityAdtlId.setNm109billPrvdId(tokens[3]);
      crEntityAdtlId.setNm109sbsrId(tokens[4]);
      crEntityAdtlId.setClaimKey(tokens[5]);
      crEntityAdtlId.setInstServiceKey(tokens[6]);
      crEntityAdtlId.setPrfsnServiceKey(tokens[7]);
      crEntityAdtlId.setDentalServiceKey(tokens[8]);
      crEntityAdtlId.setClaimPymtInfoKey(tokens[9]);
      crEntityAdtlId.setSvcPymtInfoKey(tokens[10]);
      crEntityAdtlId.setSeqNbr(new BigDecimal(tokens[11]));
      crEntityAdtlId.setLoopId(tokens[12]);
      crEntityAdtlId.setTxType(tokens[13]);
      crEntityAdtlId.setRef01idQlfr(tokens[14]);
      crEntityAdtlId.setRef02scndId(tokens[15]);
      crEntityAdtlId.setRef041refIdQlfr(tokens[20]);
      crEntityAdtlId.setRef042othrPyrPrmryId(tokens[21]);
      crEntityAdtlId.setRef043refIdQlfr(tokens[22]);
      crEntityAdtlId.setRef044refId(tokens[23]);
      crEntityAdtlId.setRef045refIdQlfr(tokens[24]);
      crEntityAdtlId.setRef046refId(tokens[25]);
      crEntityAdtlId.setRef03desc(tokens[26]);
      crEntityAdtlId.setHrchlLevelKey(tokens[27]);
      crEntityAdtlId.setSubscriberInfoKey(tokens[28]);
      
      traceLOGGER.trace("EntityAdtlId fieldvalues {}", crEntityAdtlId);
      em.persist(crEntityAdtlId);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an EntityAddressDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadEntityAddress(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("EntityAddress:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final EntityAddressDAO crEntityAddress = new EntityAddressDAO();
        
      crEntityAddress.setEntityAddressKey(tokens[0]);
      crEntityAddress.setFileNumber(tokens[1]);
      crEntityAddress.setSt02txnCtlNbr(tokens[2]);
      
      crEntityAddress.setNm109billPrvdId(tokens[3]);
      
      crEntityAddress.setNm109sbsrId(tokens[4]);
      
      crEntityAddress.setClaimKey(tokens[5]);
      crEntityAddress.setInstServiceKey(tokens[6]);
      crEntityAddress.setPrfsnServiceKey(tokens[7]);
      crEntityAddress.setDentalServiceKey(tokens[8]);
      crEntityAddress.setLoopId(tokens[9]);
      crEntityAddress.setTxType(tokens[10]);
      crEntityAddress.setN301adrLn1(tokens[11]);
      crEntityAddress.setN302adrLn2(tokens[12]);
      crEntityAddress.setN401city(tokens[13]);
      crEntityAddress.setN402st(tokens[14]);
      crEntityAddress.setN403postalCd(tokens[15]);
      crEntityAddress.setN404ctryCd(tokens[16]);
      crEntityAddress.setN407ctrySubdvsnCd(tokens[21]);
      crEntityAddress.setN405locationQual(tokens[22]);
      crEntityAddress.setN406locationId(tokens[23]);
      crEntityAddress.setHrchlLevelKey(tokens[24]);
      crEntityAddress.setSubscriberInfoKey(tokens[25]);
      traceLOGGER.trace("EntityAddress fieldvalues {}", crEntityAddress);
      em.persist(crEntityAddress);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an CntcInfoDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadCntcInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("CntcInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final CntcInfoDAO crCntcInfo = new CntcInfoDAO();
      crCntcInfo.setContactInfoKey(tokens[0]);
      crCntcInfo.setFileNumber(tokens[1]);
      crCntcInfo.setSt02txnCtlNbr(tokens[2]);
      crCntcInfo.setNm109billPrvdId(tokens[3]);
      crCntcInfo.setNm109sbsrId(tokens[4]);
      crCntcInfo.setClaimKey(tokens[5]);
      crCntcInfo.setPrfsnServiceKey(tokens[6]);
      crCntcInfo.setClaimPymtInfoKey(tokens[7]);
      crCntcInfo.setTxType(tokens[8]);
      crCntcInfo.setLoopId(tokens[9]);
      crCntcInfo.setSeqNbr(new BigDecimal(tokens[10]));
      crCntcInfo.setPer01cntcFctn(tokens[11]);
      crCntcInfo.setPer02cntcNm(tokens[12]);
      crCntcInfo.setPer03commQual1(tokens[13]);
      crCntcInfo.setPer04commNbr1(tokens[14]);
      crCntcInfo.setPer05commQual2(tokens[15]);
      crCntcInfo.setPer06commNbr2(tokens[16]);
      crCntcInfo.setPer07commQual3(tokens[17]);
      crCntcInfo.setPer08commNbr3(tokens[18]);
      crCntcInfo.setPer09cntcInqRef(tokens[23]);
      crCntcInfo.setHrchlLevelKey(tokens[24]);
      traceLOGGER.trace("CntcInfo fieldvalues {}", crCntcInfo);
      em.persist(crCntcInfo);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an DemographicInfoDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadDemographicInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("DemographicInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
       
      final DemographicInfoDAO crDemographicInfo = new DemographicInfoDAO();
      
      crDemographicInfo.setDmgInfoKey(tokens[0]);
      crDemographicInfo.setFileNumber(tokens[1]);
      crDemographicInfo.setSt02txnCtlNbr(tokens[2]);
      crDemographicInfo.setNm109billPrvdId(tokens[3]);
      crDemographicInfo.setNm109sbsrId(tokens[4]);
      crDemographicInfo.setClaimKey(tokens[5]);
      crDemographicInfo.setLoopId(tokens[6]);
      crDemographicInfo.setSeqNbr(new BigDecimal(tokens[7]));
      crDemographicInfo.setDmg01dttmFmtQual(tokens[8]);
      crDemographicInfo.setDmg02brthDt(tokens[9]);
      crDemographicInfo.setDmg03gndr(tokens[10]);
   	  crDemographicInfo.setDmg10CdLstQualCd(tokens[15]);
   	  crDemographicInfo.setDmg11IndustryCd(tokens[16]);
   	  crDemographicInfo.setHrchlLevelKey(tokens[17]);
            
      traceLOGGER.trace("DemographicInfo fieldvalues {}", crDemographicInfo);
      em.persist(crDemographicInfo);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an SpnlMpltSvcInfoDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadSpnlMpltSvcInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("SpnlMpltSvcInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final SpnlMpltSvcInfoDAO crSpnlMpltSvcInfo = new SpnlMpltSvcInfoDAO();
      crSpnlMpltSvcInfo.setSpnlMpltSvcInfoKey(tokens[0]);
      crSpnlMpltSvcInfo.setClaimKey(tokens[1]);
      crSpnlMpltSvcInfo.setPrfsnServiceKey(tokens[2]);
      crSpnlMpltSvcInfo.setSeqNbr(new BigDecimal(tokens[3]));
      if (!((tokens[4].equals("")) || (tokens[4] == null))) {
        crSpnlMpltSvcInfo.setCr201trtmSerNbr(new BigDecimal(tokens[4]));
      }
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crSpnlMpltSvcInfo.setCr202trtmCnt(new BigDecimal(tokens[5]));
      }
      crSpnlMpltSvcInfo.setCr203slxtnLvlCd1(tokens[6]);
      crSpnlMpltSvcInfo.setCr204slxtnLvlCd2(tokens[7]);
      crSpnlMpltSvcInfo.setCr205uomCd(tokens[8]);
      if (!((tokens[9].equals("")) || (tokens[9] == null))) {
        crSpnlMpltSvcInfo.setCr206trtmPerCnt(new BigDecimal(tokens[9]));
      }
      if (!((tokens[10].equals("")) || (tokens[10] == null))) {
        crSpnlMpltSvcInfo.setCr207moTrtmCnt(new BigDecimal(tokens[10]));
      }
      crSpnlMpltSvcInfo.setCr208ptCondCd(ConversionUtil.stringToSingleCharacter(tokens[11]));
      crSpnlMpltSvcInfo.setCr209cmplcnInd(ConversionUtil.stringToSingleCharacter(tokens[12]));
      crSpnlMpltSvcInfo.setCr210ptCondDesc1(tokens[13]);
      crSpnlMpltSvcInfo.setCr211ptCondDesc2(tokens[14]);
      crSpnlMpltSvcInfo.setCr212xrayAvlInd(ConversionUtil.stringToSingleCharacter(tokens[15]));
      traceLOGGER.trace("SpnlMpltSvcInfo fieldvalues {}", crSpnlMpltSvcInfo);
      em.persist(crSpnlMpltSvcInfo);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ClaimDtDAO object. Once the data is mapped it to the object it is saved
   * to the database using the EntityManager passed in (em). Notice that this method uses a DAO
   * object to store data. It uses this DAO (which really isn't a DAO by the classical definition)
   * so that it may directly store some foreign keys found in the .DAT file in the database.
   * Normally, the keys would be implicitly stored as a result of an object to object relationship.
   * Since we don't have enough data at this point to build the proper object to object
   * relationship, we explicitly store the key via an key attribute on a DAO. There are a few fields
   * that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadClaimDt(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ClaimDt:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final ClaimDtDAO crClaimDt = new ClaimDtDAO();
      crClaimDt.setClaimDtKey(tokens[0]);
      crClaimDt.setClaimKey(tokens[1]);
      crClaimDt.setInstServiceKey(tokens[2]);
      crClaimDt.setPrfsnServiceKey(tokens[3]);
      crClaimDt.setDentalServiceKey(tokens[4]);
      crClaimDt.setClaimPymtInfoKey(tokens[5]);
      crClaimDt.setSvcPymtInfoKey(tokens[6]);
      crClaimDt.setTxType(tokens[7]);
      crClaimDt.setSeqNbr(new BigDecimal(tokens[8]));
      crClaimDt.setDtp01dttmQual(tokens[9]);
      crClaimDt.setDtp02dttmFmtQual(tokens[10]);
      crClaimDt.setDtp03dtFld(tokens[11]);
      crClaimDt.setFileNumber(tokens[16]);
      crClaimDt.setSt02txnCtlNbr(tokens[17]);
      crClaimDt.setSubscriberInfoKey(tokens[18]);
      traceLOGGER.trace("ClaimDt fieldvalues {}", crClaimDt);
      em.persist(crClaimDt);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ClaimAmountDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadClaimAmount(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ClaimAmount:" + loadRec);

      final String[] tokens = loadRec.split("\\^", -1);
      final ClaimAmountDAO crClaimAmount = new ClaimAmountDAO();
      crClaimAmount.setClaimAmountKey(tokens[0]);
      crClaimAmount.setClaimKey(tokens[1]);
      crClaimAmount.setInstServiceKey(tokens[2]);
      crClaimAmount.setPrfsnServiceKey(tokens[3]);
      crClaimAmount.setDentalServiceKey(tokens[4]);
      crClaimAmount.setClaimPymtInfoKey(tokens[5]);
      crClaimAmount.setSvcPymtInfoKey(tokens[6]);
      crClaimAmount.setTxType(tokens[7]);      
      crClaimAmount.setSeqNbr(new BigDecimal(tokens[8]));
      crClaimAmount.setAmt01amtQualCd(tokens[9]);
      if (!((tokens[10].equals("")) || (tokens[10] == null))) {
        crClaimAmount.setAmt02amt(new BigDecimal(tokens[10]));
      }
      crClaimAmount.setLoopId(tokens[15]);
      //TWP Check to make sure of the value in here
      crClaimAmount.setAmt03creditDebitFlgCd(ConversionUtil.checkForBooleanType(tokens[16]));
      crClaimAmount.setSubscriberInfoKey(tokens[17]);
      
      traceLOGGER.trace("ClaimAmount fieldvalues {}", crClaimAmount);
      em.persist(crClaimAmount);
    }
    
    
    
    FileUtils.closeFile(in);
  ;
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ClaimQuantityDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadClaimQuantity(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ClaimQuantity:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final ClaimQuantityDAO crClaimQuantity = new ClaimQuantityDAO();
      crClaimQuantity.setQuantityKey(tokens[0]);
      crClaimQuantity.setClaimKey(tokens[1]);
      crClaimQuantity.setDentalServiceKey(tokens[2]);
      crClaimQuantity.setClaimPymtInfoKey(tokens[3]);
      //??????????????????????????????????????????????????????????????????
      //TWP TODO Why is this commented out? I am leaving it out for now????
      //??????????????????????????????????????????????????????????????????
      // crClaimQuantity.setSvcPymtInfoKey(tokens[4]);
      crClaimQuantity.setSeqNbr(new BigDecimal(tokens[5]));
      crClaimQuantity.setQty01qtyQual(tokens[6]);
      if (!((tokens[7].equals("")) || (tokens[7] == null))) {
        crClaimQuantity.setQty02clmDaysCnt(new BigDecimal(tokens[7]));
      }
      crClaimQuantity.setQty03uomCd(tokens[8]);
      crClaimQuantity.setQty04feeFormMsg(tokens[13]);
      traceLOGGER.trace("ClaimQuantity fieldvalues {}", crClaimQuantity);
      em.persist(crClaimQuantity);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ClaimNoteDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadClaimNote(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ClaimNote:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final ClaimNoteDAO crClaimNote = new ClaimNoteDAO();
      crClaimNote.setClaimNoteKey(tokens[0]);
      crClaimNote.setClaimKey(tokens[1]);
      crClaimNote.setPrfsnServiceKey(tokens[2]);
      crClaimNote.setDentalServiceKey(tokens[3]);
      crClaimNote.setSeqNbr(new BigDecimal(tokens[4]));
      crClaimNote.setNte01clmNoteQual(tokens[5]);
      crClaimNote.setNte02clmNoteTxt(tokens[6]);
      traceLOGGER.trace("ClaimNote fieldvalues {}", crClaimNote);
      em.persist(crClaimNote);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an CasesDAO object. Once the data is mapped it to the object it is saved
   * to the database using the EntityManager passed in (em). Notice that this method uses a DAO
   * object to store data. It uses this DAO (which really isn't a DAO by the classical definition)
   * so that it may directly store some foreign keys found in the .DAT file in the database.
   * Normally, the keys would be implicitly stored as a result of an object to object relationship.
   * Since we don't have enough data at this point to build the proper object to object
   * relationship, we explicitly store the key via an key attribute on a DAO. There are a few fields
   * that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadCases(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("Cases:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final CasesDAO crCases = new CasesDAO();
      crCases.setCasesKey(tokens[0]);
      crCases.setClaimKey(tokens[1]);
      crCases.setInstServiceKey(tokens[2]);
      crCases.setPrfsnServiceKey(tokens[3]);
      crCases.setDentalServiceKey(tokens[4]);
      crCases.setClaimPymtInfoKey(tokens[5]);
      crCases.setSvcPymtInfoKey(tokens[6]);
      crCases.setTxType(tokens[7]);
      crCases.setLoopSeqNbr(new BigDecimal(tokens[8]));
      crCases.setSeqNbr(new BigDecimal(tokens[9]));
      crCases.setCas01clmAdjGrp(tokens[10]);
      crCases.setCas02clmAdjRsn1(tokens[11]);
      if (!((tokens[12].equals("")) || (tokens[12] == null))) {
        crCases.setCas03adjAmt1(new BigDecimal(tokens[12]));
      }
      if (!((tokens[13].equals("")) || (tokens[13] == null))) {
        crCases.setCas04adjQty1(new BigDecimal(tokens[13]));
      }
      crCases.setCas05clmAdjRsn2(tokens[14]);
      if (!((tokens[15].equals("")) || (tokens[15] == null))) {
        crCases.setCas06adjAmt2(new BigDecimal(tokens[15]));
      }
      if (!((tokens[16].equals("")) || (tokens[16] == null))) {
        crCases.setCas07adjQty2(new BigDecimal(tokens[16]));
      }
      crCases.setCas08clmAdjRsn3(tokens[17]);
      if (!((tokens[18].equals("")) || (tokens[18] == null))) {
        crCases.setCas09adjAmt3(new BigDecimal(tokens[18]));
      }
      if (!((tokens[19].equals("")) || (tokens[19] == null))) {
        crCases.setCas10adjQty3(new BigDecimal(tokens[19]));
      }
      crCases.setCas11clmAdjRsn4(tokens[20]);
      if (!((tokens[21].equals("")) || (tokens[21] == null))) {
        crCases.setCas12adjAmt4(new BigDecimal(tokens[21]));
      }
      if (!((tokens[22].equals("")) || (tokens[22] == null))) {
        crCases.setCas13adjQty4(new BigDecimal(tokens[22]));
      }
      crCases.setCas14clmAdjRsn5(tokens[23]);
      if (!((tokens[24].equals("")) || (tokens[24] == null))) {
        crCases.setCas15adjAmt5(new BigDecimal(tokens[24]));
      }
      if (!((tokens[25].equals("")) || (tokens[25] == null))) {
        crCases.setCas16adjQty5(new BigDecimal(tokens[25]));
      }
      crCases.setCas17clmAdjRsn6(tokens[26]);
      if (!((tokens[27].equals("")) || (tokens[27] == null))) {
        crCases.setCas18adjAmt6(new BigDecimal(tokens[27]));
      }
      if (!((tokens[28].equals("")) || (tokens[28] == null))) {
        crCases.setCas19adjQty6(new BigDecimal(tokens[28]));
      }
      crCases.setSubscriberInfoKey(tokens[33]);
      traceLOGGER.trace("Cases fieldvalues {}", crCases);
      em.persist(crCases);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ClaimSuplInfoDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadClaimSuplInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ClaimSuplInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final ClaimSuplInfoDAO crClaimSuplInfo = new ClaimSuplInfoDAO();
      crClaimSuplInfo.setPwkKey(tokens[0]);
      crClaimSuplInfo.setClaimKey(tokens[1]);
      crClaimSuplInfo.setInstServiceKey(tokens[2]);
      crClaimSuplInfo.setPrfsnServiceKey(tokens[3]);
      crClaimSuplInfo.setSeqNbr(new BigDecimal(tokens[4]));
      crClaimSuplInfo.setPwk01rptTypeCd(tokens[5]);
      crClaimSuplInfo.setPwk02rptTrnsmCd(tokens[6]);
      crClaimSuplInfo.setPwk05idCdQual(tokens[7]);
      crClaimSuplInfo.setPwk06atmntCtlNbr(tokens[8]);
      crClaimSuplInfo.setPwk07dsc(tokens[9]);
      crClaimSuplInfo.setPwk03rptCopyNeeded(tokens[14]);
      crClaimSuplInfo.setPwd04entyIdCd(tokens[15]);
      crClaimSuplInfo.setPwk09rqstCatCd(tokens[16]);
      traceLOGGER.trace("ClaimSuplInfo fieldvalues {}", crClaimSuplInfo);
      em.persist(crClaimSuplInfo);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an FileInformationDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadFileInformation(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("FileInformation:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final FileInformationDAO crFileInformation = new FileInformationDAO();
      crFileInformation.setFileInfoKey(tokens[0]);
      crFileInformation.setClaimKey(tokens[1]);
      crFileInformation.setInstServiceKey(tokens[2]);
      crFileInformation.setPrfsnServiceKey(tokens[3]);
      crFileInformation.setSeqNbr(new BigDecimal(tokens[4]));
      crFileInformation.setK301fixedFmtInfo(tokens[5]);
      traceLOGGER.trace("FileInformation fieldvalues {}", crFileInformation);
      em.persist(crFileInformation);
    }
    FileUtils.closeFile(in);
  
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an HomeHcareInfo object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadHomeHcareInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("HomeHcareInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final HomeHcareInfo crHomeHcareInfo = new HomeHcareInfo();
      crHomeHcareInfo.setClaimKey(tokens[0]);
      crHomeHcareInfo.setCr601prognosisCd(ConversionUtil.stringToSingleChar(tokens[1]));
      crHomeHcareInfo.setCr602svcFromDt(tokens[2]);
      crHomeHcareInfo.setCr603dttmFmtQual(tokens[3]);
      crHomeHcareInfo.setCr604certPer(tokens[4]);
      crHomeHcareInfo.setCr605diagDt(tokens[5]);
      crHomeHcareInfo.setCr606sklNursFcly(ConversionUtil.stringToSingleChar(tokens[6]));
      crHomeHcareInfo.setCr607mdcrCvgInd(ConversionUtil.stringToSingleChar(tokens[7]));
      crHomeHcareInfo.setCr608certTypeCd(ConversionUtil.stringToSingleChar(tokens[8]));
      crHomeHcareInfo.setCr609sgryDt(tokens[9]);
      crHomeHcareInfo.setCr610svcIdQual(tokens[10]);
      crHomeHcareInfo.setCr611srgclProcCd(tokens[11]);
      crHomeHcareInfo.setCr612physOrdDt(tokens[12]);
      crHomeHcareInfo.setCr613lastVisitDt(tokens[13]);
      crHomeHcareInfo.setCr614physCntcDt(tokens[14]);
      crHomeHcareInfo.setCr615lastAdmQual(tokens[15]);
      crHomeHcareInfo.setCr616lastAdmPer(tokens[16]);
      crHomeHcareInfo.setCr617fclyTypeCd(ConversionUtil.stringToSingleChar(tokens[17]));
      crHomeHcareInfo.setCr618scndDiagDt1(tokens[18]);
      crHomeHcareInfo.setCr619scndDiagDt2(tokens[19]);
      crHomeHcareInfo.setCr620scndDiagDt3(tokens[20]);
      crHomeHcareInfo.setCr621scndDiagDt4(tokens[21]);
      traceLOGGER.trace("HomeHcareInfo fieldvalues {}", crHomeHcareInfo);
      em.persist(crHomeHcareInfo);
    }
    FileUtils.closeFile(in);
 
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an CrcSegmentDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadCrcSegment(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("rcSegment:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final CrcSegmentDAO crCrcSegment = new CrcSegmentDAO();
      crCrcSegment.setCrcSegKey(tokens[0]);
      crCrcSegment.setClaimKey(tokens[1]);
      crCrcSegment.setPrfsnServiceKey(tokens[2]);
      crCrcSegment.setSgmtId(tokens[3]);
      crCrcSegment.setSeqNbr(new BigDecimal(tokens[4]));
      crCrcSegment.setCrc01cdCat(tokens[5]);
      crCrcSegment.setCrc02certCondInd(ConversionUtil.stringToSingleCharacter(tokens[6]));
      crCrcSegment.setCrc03hmHlthCd(tokens[7]);
      crCrcSegment.setCrc04hmHlthCd(tokens[8]);
      crCrcSegment.setCrc05hmHlthCd(tokens[9]);
      crCrcSegment.setCrc06hmHlthCd(tokens[10]);
      crCrcSegment.setCrc07hmHlthCd(tokens[11]);
      traceLOGGER.trace("CrcSegment fieldvalues {}", crCrcSegment);
      em.persist(crCrcSegment);
    }
    FileUtils.closeFile(in);
 
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an HcareServiceDlvrDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadHcareServiceDlvr(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("HcareServiceDlvr:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      
      final HcareServiceDlvrDAO crHcareServiceDlvr = new HcareServiceDlvrDAO();
      
      crHcareServiceDlvr.setHcareSvcDlvrKey(tokens[0]);
      crHcareServiceDlvr.setClaimKey(tokens[1]);
      crHcareServiceDlvr.setInstServiceKey(tokens[2]);
      crHcareServiceDlvr.setPrfsnServiceKey(tokens[3]);      
      // End DEV05506
      crHcareServiceDlvr.setLoopSeqNbr(new BigDecimal(tokens[4]));
      crHcareServiceDlvr.setSeqNbr(new BigDecimal(tokens[5]));
      crHcareServiceDlvr.setHsd01qtyQual(tokens[6]);
      if (tokens[7] != null && !("".equals(tokens[7]))) {
        crHcareServiceDlvr
            .setHsd02nbrOfVisits(new BigDecimal(tokens[7]));
      }
      crHcareServiceDlvr.setHsd03frqPer(tokens[8]);
      if (tokens[9] != null && !("".equals(tokens[9]))) {
        crHcareServiceDlvr.setHsd04frqCnt(new BigDecimal(tokens[9]));
      }
      crHcareServiceDlvr.setHsd05frqTmPer(tokens[10]);
      if (tokens[11] != null && !("".equals(tokens[11]))) {
        crHcareServiceDlvr.setHsd06nbrOfPer(new BigDecimal(tokens[11]));
      }
      crHcareServiceDlvr.setHsd07calPtrnCd(tokens[12]);
      crHcareServiceDlvr.setHsd08tmPtrnCd(tokens[13]);
      traceLOGGER.trace("HcareServiceDlvr fieldvalues {}", crHcareServiceDlvr);
      em.persist(crHcareServiceDlvr);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an AmblnTrnptInfoDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em);
   */
  protected void loadAmblnTrnptInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("AmblnTrnptInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final AmblnTrnptInfoDAO crAmblnTrnptInfo = new AmblnTrnptInfoDAO();

      crAmblnTrnptInfo.setAmblnTptInfoKey(tokens[0]);
      crAmblnTrnptInfo.setClaimKey(tokens[1]);
      crAmblnTrnptInfo.setPrfsnServiceKey(tokens[2]);
      crAmblnTrnptInfo.setCr101uomCd(tokens[3]);
      if (!((tokens[4].equals("")) || (tokens[4] == null))) {
        crAmblnTrnptInfo.setCr102ptWt(new BigDecimal(tokens[4]));
      }
      crAmblnTrnptInfo.setCr103amblnTrnCd(ConversionUtil.stringToSingleCharacter(tokens[5]));
      crAmblnTrnptInfo.setCr104amblnTrnRsn(ConversionUtil.stringToSingleCharacter(tokens[6]));
      crAmblnTrnptInfo.setCr105uomDist(tokens[7]);
      if (!((tokens[8].equals("")) || (tokens[8] == null))) {
        crAmblnTrnptInfo.setCr106trnspDist(new BigDecimal(tokens[8]));
      }
      crAmblnTrnptInfo.setCr109rtrpPrps(tokens[9]);
      crAmblnTrnptInfo.setCr110strchPrps(tokens[10]);
      traceLOGGER.trace("AmblnTrnptInfo fieldvalues {}", crAmblnTrnptInfo);
      em.persist(crAmblnTrnptInfo);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ClaimPrcReprcDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadClaimPrcReprc(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ClaimPrcReprc:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final ClaimPrcReprcDAO crClaimPrcReprc = new ClaimPrcReprcDAO();
      crClaimPrcReprc.setClaimPrcReprcKey(tokens[0]);
      crClaimPrcReprc.setClaimKey(tokens[1]);
      crClaimPrcReprc.setInstServiceKey(tokens[2]);
      crClaimPrcReprc.setPrfsnServiceKey(tokens[3]);
      crClaimPrcReprc.setHcp01prcMthdlg(tokens[4]);
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crClaimPrcReprc.setHcp02alowAmt(new BigDecimal(tokens[5]));
      }
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crClaimPrcReprc.setHcp03svgAmt(new BigDecimal(tokens[6]));
      }
      crClaimPrcReprc.setHcp04rprcOrgId(tokens[7]);
      if (!((tokens[8].equals("")) || (tokens[8] == null))) {
        crClaimPrcReprc.setHcp05prcRt(new BigDecimal(tokens[8]));
      }
      crClaimPrcReprc.setHcp06aprvDrgCd(tokens[9]);
      if (!((tokens[10].equals("")) || (tokens[10] == null))) {
        crClaimPrcReprc.setHcp07aprvDrgAmt(new BigDecimal(tokens[10]));
      }
      crClaimPrcReprc.setHcp08aprvRevCd(tokens[11]);
      crClaimPrcReprc.setHcp09svcIdQual(tokens[12]);
      crClaimPrcReprc.setHcp10aprvProcCd(tokens[13]);
      crClaimPrcReprc.setHcp11uomCd(tokens[14]);
      if (!((tokens[15].equals("")) || (tokens[15] == null))) {
        crClaimPrcReprc.setHcp12aprvSvcUnit(new BigDecimal(tokens[15]));
      }
      crClaimPrcReprc.setHcp13rejRsnCd(tokens[16]);
      crClaimPrcReprc.setHcp14plcyCmplCd(tokens[17]);
      crClaimPrcReprc.setHcp15excpCd(tokens[18]);
      traceLOGGER.trace("ClaimPrcReprc fieldvalues {}", crClaimPrcReprc);
      em.persist(crClaimPrcReprc);
    }
    FileUtils.closeFile(in);
  
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an DblMedEqpSvc object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadDblMedEqpSvc(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("DblMedEqpSvc:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final DblMedEqpSvc crDblMedEqpSvc = new DblMedEqpSvc();
      crDblMedEqpSvc.setClaimKey(tokens[0]);
      crDblMedEqpSvc.setPrfsnServiceKey(tokens[1]);
      crDblMedEqpSvc.setSv501procId(tokens[2]);
      crDblMedEqpSvc.setSv501procCode(tokens[3]);
      crDblMedEqpSvc.setSv502measUnit(tokens[4]);
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crDblMedEqpSvc.setSv503lnthMedNec(new BigDecimal(tokens[5]));
      }
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crDblMedEqpSvc.setSv504rentalPrice(new BigDecimal(tokens[6]));
      }
      if (!((tokens[7].equals("")) || (tokens[7] == null))) {
        crDblMedEqpSvc.setSv505purchPrice(new BigDecimal(tokens[7]));
      }
      crDblMedEqpSvc.setSv506freqCode(ConversionUtil.stringToSingleCharacter(tokens[8]));
      crDblMedEqpSvc.setSv5018prdctSvcId(tokens[13]);
      traceLOGGER.trace("DblMedEqpSvc fieldvalues {}", crDblMedEqpSvc);
      em.persist(crDblMedEqpSvc);
    }
    FileUtils.closeFile(in);
   
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an HomeOxnThpyCertDAO object. Once the data is mapped it to the object it
   * is saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadHomeOxnThpyCert(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("HomeOxnThpyCert:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
       final HomeOxnThpyCertDAO crHomeOxnThpyCert = new HomeOxnThpyCertDAO();
      crHomeOxnThpyCert.setHOxnThpyCertKey(tokens[0]);
      crHomeOxnThpyCert.setClaimKey(tokens[1]);
      crHomeOxnThpyCert.setPrfsnServiceKey(tokens[2]);
      crHomeOxnThpyCert.setCr501certTypeCd(ConversionUtil.stringToSingleCharacter(tokens[3]));
      if (!((tokens[4].equals("")) || (tokens[4] == null))) {
        crHomeOxnThpyCert.setCr502trtmPerCnt(new BigDecimal(tokens[4]));
      }
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crHomeOxnThpyCert
            .setCr510artrlBldGas(new BigDecimal(tokens[5]));
      }
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crHomeOxnThpyCert.setCr511oxgnStrtn(new BigDecimal(tokens[6]));
      }
      crHomeOxnThpyCert.setCr512oxgnCondCd(ConversionUtil.stringToSingleCharacter(tokens[7]));
      crHomeOxnThpyCert.setCr513oxgnFindCd1(ConversionUtil.stringToSingleCharacter(tokens[8]));
      crHomeOxnThpyCert.setCr514oxgnFindCd2(ConversionUtil.stringToSingleCharacter(tokens[9]));
      crHomeOxnThpyCert.setCr515oxgnFindCd3(ConversionUtil.stringToSingleCharacter(tokens[10]));
      traceLOGGER.trace("HomeOxnThpyCert {}", crHomeOxnThpyCert);
      em.persist(crHomeOxnThpyCert);
    }
    FileUtils.closeFile(in);
  
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an DrugIdDAO object. Once the data is mapped it to the object it is saved
   * to the database using the EntityManager passed in (em). Notice that this method uses a DAO
   * object to store data. It uses this DAO (which really isn't a DAO by the classical definition)
   * so that it may directly store some foreign keys found in the .DAT file in the database.
   * Normally, the keys would be implicitly stored as a result of an object to object relationship.
   * Since we don't have enough data at this point to build the proper object to object
   * relationship, we explicitly store the key via an key attribute on a DAO. There are a few fields
   * that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadDrugId(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("DrugId:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final DrugIdDAO crDrugId = new DrugIdDAO();
      crDrugId.setDrugIdKey(tokens[0]);
      crDrugId.setClaimKey(tokens[1]);
      crDrugId.setInstServiceKey(tokens[2]);
      crDrugId.setPrfsnServiceKey(tokens[3]);
      crDrugId.setSeqNbr(new BigDecimal(tokens[4]));
      crDrugId.setLin02svcIdQual(tokens[5]);
      crDrugId.setLin03natlDrugCd(tokens[6]);
      traceLOGGER.trace("DrugId {}", crDrugId);
      em.persist(crDrugId);
    }
    FileUtils.closeFile(in);
 
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an DrugPricingDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadDrugPricing(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("DrugPricing:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final DrugPricingDAO crDrugPricing = new DrugPricingDAO();
      crDrugPricing.setDrugPricingKey(tokens[0]);
      crDrugPricing.setClaimKey(tokens[1]);
      crDrugPricing.setInstServiceKey(tokens[2]);
      crDrugPricing.setPrfsnServiceKey(tokens[3]);
      crDrugPricing.setSeqNbr(new BigDecimal(tokens[4]));
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crDrugPricing.setCtp03drugUnitPrc(new BigDecimal(tokens[5]));
      }
      if (!((tokens[6].equals("")) || (tokens[6] == null))) {
        crDrugPricing.setCtp04drugUnitCnt(new BigDecimal(tokens[6]));
      }
      crDrugPricing.setCtp05measUnit(tokens[7]);
      traceLOGGER.trace("DrugPricing {}", crDrugPricing);
      em.persist(crDrugPricing);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an TestResultMea/TestResultMeaId object. Once the data is mapped it to the
   * object it is saved to the database using the EntityManager passed in (em);
   */
  protected void loadTestResultMea(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("TestResultMea:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final TestResultMea crTestResultMea = new TestResultMea();
      TestResultMeaId id = new TestResultMeaId(tokens[1], new BigDecimal(tokens[2]));
      crTestResultMea.setId(id);
      crTestResultMea.setClaimKey(tokens[0]);
      crTestResultMea.setMea01measIdCd(tokens[3]);
      crTestResultMea.setMea02measQual(tokens[4]);
      if (!((tokens[5].equals("")) || (tokens[5] == null))) {
        crTestResultMea.setMea03testRslt(new BigDecimal(tokens[5]));
      }
      crTestResultMea.setMea11cdLstQualCd(tokens[10]);
      crTestResultMea.setMea12indstryCd(tokens[11]);
      traceLOGGER.trace("TestResultMea fieldvalues:" + crTestResultMea.getClaimKey() + ","
          + crTestResultMea.getId().getPrfsnServiceKey() + ","
          + crTestResultMea.getMea01measIdCd() + "," + crTestResultMea.getMea02measQual() + ","
          + crTestResultMea.getMea03testRslt());
      // traceLOGGER.trace("TestResultMea {}",crTestResultMea);
      em.persist(crTestResultMea);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an PurchServiceInfo object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadPurchServiceInfo(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("PurchServiceInfo:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final PurchServiceInfo crPurchServiceInfo = new PurchServiceInfo();
      crPurchServiceInfo.setClaimKey(tokens[0]);
      crPurchServiceInfo.setPrfsnServiceKey(tokens[1]);
      crPurchServiceInfo.setPs101purchSvcId(tokens[2]);
      if (!((tokens[3].equals("")) || (tokens[3] == null))) {
        crPurchServiceInfo
            .setPs102purchSvcChg(new BigDecimal(tokens[3]));
      }
      traceLOGGER.trace("PurchServiceInfo {}", crPurchServiceInfo);
      em.persist(crPurchServiceInfo);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an DblMdclEqpCert object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em);
   */
  protected void loadDblMdclEqpCert(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("DblMdclEqpCert:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final DblMdclEqpCert crDblMdclEqpCert = new DblMdclEqpCert();
      crDblMdclEqpCert.setClaimKey(tokens[0]);
      crDblMdclEqpCert.setPrfsnServiceKey(tokens[1]);
      crDblMdclEqpCert.setCr301certTypeCd(ConversionUtil.stringToSingleCharacter(tokens[2]));
      crDblMdclEqpCert.setCr302uomCd(tokens[3]);
      if (!((tokens[4].equals("")) || (tokens[4] == null))) {
        crDblMdclEqpCert.setCr303dmeDrtn(new BigDecimal(tokens[4]));
      }
      traceLOGGER.trace("DblMdclEqpCert {}", crDblMdclEqpCert);
      em.persist(crDblMdclEqpCert);
    }
    FileUtils.closeFile(in);

  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an SuptngDocDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadSuptngDoc(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("SuptngDoc:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final SuptngDocDAO crSuptngDoc = new SuptngDocDAO();
      crSuptngDoc.setSuptngDocKey(tokens[0]);
      crSuptngDoc.setClaimKey(tokens[1]);
      crSuptngDoc.setPrfsnServiceKey(tokens[2]);
      crSuptngDoc.setLoopSeqNbr(new BigDecimal(tokens[3]));
      crSuptngDoc.setSeqNbr(new BigDecimal(tokens[4]));
      crSuptngDoc.setFrm01questionId(tokens[5]);
      crSuptngDoc.setFrm02respYesno(ConversionUtil.stringToSingleCharacter(tokens[6]));
      crSuptngDoc.setFrm03respTxt(tokens[7]);
      crSuptngDoc.setFrm04respDt(tokens[8]);
      if (!((tokens[9].equals("")) || (tokens[9] == null))) {
        crSuptngDoc.setFrm05respPct(new BigDecimal(tokens[9]));
      }
      traceLOGGER.trace("SuptngDoc {}", crSuptngDoc);
      em.persist(crSuptngDoc);
    }
    FileUtils.closeFile(in);
  
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an SvcLineAdjdctDAO object. Once the data is mapped it to the object it is
   * saved to the database using the EntityManager passed in (em). Notice that this method uses a
   * DAO object to store data. It uses this DAO (which really isn't a DAO by the classical
   * definition) so that it may directly store some foreign keys found in the .DAT file in the
   * database. Normally, the keys would be implicitly stored as a result of an object to object
   * relationship. Since we don't have enough data at this point to build the proper object to
   * object relationship, we explicitly store the key via an key attribute on a DAO. There are a few
   * fields that common force us to use DAO's they are: St02txnCtlNbr, SeqNbr, PrfsnServiceKey,
   * DentalServiceKey, InstServiceKey, ClaimKey
   */
  protected void loadSvcLineAdjdct(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("SvcLineAdjdct:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final SvcLineAdjdctDAO crSvcLineAdjdct = new SvcLineAdjdctDAO();
      crSvcLineAdjdct.setSvcLineAdjdctKey(tokens[0]);
      crSvcLineAdjdct.setClaimKey(tokens[1]);
      crSvcLineAdjdct.setDentalServiceKey(tokens[2]);
      crSvcLineAdjdct.setInstServiceKey(tokens[3]);
      crSvcLineAdjdct.setPrfsnServiceKey(tokens[4]);    
      crSvcLineAdjdct.setSeqNbr(new BigDecimal(tokens[5]));
      crSvcLineAdjdct.setSvd01pyrId(tokens[6]);
      if (!((tokens[7].equals("")) || (tokens[7] == null))) {
        crSvcLineAdjdct.setSvd02svcLnPaid(new BigDecimal(tokens[7]));
      }
      crSvcLineAdjdct.setSvd03svcIdQual(tokens[8]);
      crSvcLineAdjdct.setSvd03procCd(tokens[9]);
      crSvcLineAdjdct.setSvd03procMod1(tokens[10]);
      crSvcLineAdjdct.setSvd03procMod2(tokens[11]);
      crSvcLineAdjdct.setSvd03procMod3(tokens[12]);
      crSvcLineAdjdct.setSvd03procMod4(tokens[13]);
      crSvcLineAdjdct.setSvd03procDesc(tokens[14]);
      crSvcLineAdjdct.setSvd04svcLnRevCd(tokens[15]);
      if (!((tokens[16].equals("")) || (tokens[16] == null))) {
        crSvcLineAdjdct.setSvd05adjQty(new BigDecimal(tokens[16]));
      }
      if (!((tokens[17].equals("")) || (tokens[17] == null))) {
        crSvcLineAdjdct.setSvd06lineNbr(new BigDecimal(tokens[17]));
      }
      crSvcLineAdjdct.setSvd038prdctSvcId(tokens[22]);
      traceLOGGER.trace("SvcLineAdjdct {}", crSvcLineAdjdct);
      em.persist(crSvcLineAdjdct);
    }
    FileUtils.closeFile(in);
 
  }

  /*
   * This method reads line by line from the data file associated with the buffered reader "in",
   * splits each line based on the delimiter "^" and maps each of the resulting tokens to the
   * appropriate field on an ToothInformation/ToothInformationId object. Once the data is mapped it
   * to the object it is saved to the database using the EntityManager passed in (em);
   */
  protected void loadToothInformation(BufferedReader in, EntityManager em)
      throws IOException {
    while ((loadRec = in.readLine()) != null) {
      traceLOGGER.trace("ToothInformation:" + loadRec);
      final String[] tokens = loadRec.split("\\^", -1);
      final ToothInformation crToothInformation = new ToothInformation();
      ToothInformationId id = new ToothInformationId(tokens[1], new BigDecimal(tokens[2]));
      crToothInformation.setId(id);
      crToothInformation.setClaimKey(tokens[0]);
      crToothInformation.setToo01cdQual(tokens[3]);
      crToothInformation.setToo02tthNbr(tokens[4]);
      crToothInformation.setToo03tthSrfcCd1(tokens[5]);
      crToothInformation.setToo03tthSrfcCd2(tokens[6]);
      crToothInformation.setToo03tthSrfcCd3(tokens[7]);
      crToothInformation.setToo03tthSrfcCd4(tokens[8]);
      crToothInformation.setToo03tthSrfcCd5(tokens[9]);
      traceLOGGER.trace("ToothInformation {}", crToothInformation);
      em.persist(crToothInformation);
    }
    FileUtils.closeFile(in);
  }
  
  protected void loadRmttncDlvryMthd(BufferedReader in, EntityManager em)
    throws IOException {
    	 while ((loadRec = in.readLine()) != null) {
    	      traceLOGGER.trace("RmttncDlvryMthd:" + loadRec);
    	      final String[] tokens = loadRec.split("\\^", -1);
    	      final RmttncDlvryMthdDAO crRmttncDlvryMthd = new RmttncDlvryMthdDAO();
    	      crRmttncDlvryMthd.setRmttncDlvryMthdKey(tokens[0]);
    	      crRmttncDlvryMthd.setFileNumber(tokens[1]);
    	      crRmttncDlvryMthd.setSt02txnCtlNbr(tokens[2]);
    	      crRmttncDlvryMthd.setRdm01RptTrmsnCd(tokens[3]);
    	      crRmttncDlvryMthd.setRdm02Name(tokens[4]);
    	      crRmttncDlvryMthd.setRdm03ComNbr(tokens[5]);
    	      traceLOGGER.trace("RmttncDlvryMthd {}", crRmttncDlvryMthd);
    	      em.persist(crRmttncDlvryMthd);
    }
    FileUtils.closeFile(in);
  }
}
