package gov.va.med.hac.edi.utils;

import java.io.*;
import java.util.logging.*;
import javax.naming.NamingException;

// This process takes information on a directory and file names, and runs through the
//   files found there, calling the EdiUtil process for each one
public class X12ClaimStatFormatter  {

  public static void main(final String[] args) {
    // Local variables
    PrintWriter vLogFile;
    boolean processSuccess = false;
        
    // Set up a log file
    // If there is any problem with the log file, go no further
    try {
      vLogFile = MainUtils.establishLogFile("Log", "X12ClaimStatFormatterLog.txt");
    
    // If there was a problem setting up the log file, we cannot log to it
    //   Log through the Java Logging API, which we could configure to do
    //   something useful in the future
    } catch(Exception ex) {
      Logger.getLogger(gov.va.med.hac.edi.utils.X12ClaimStatFormatter.class.getName()).log(Level.SEVERE, null, ex);
      return;
    }
      
    // Catch any exception (outside of setting up for logging)
    try {
      MainUtils.stampStartingTime("X12ClaimStatFormatter Start", vLogFile);
      vLogFile.println("Code Version: New");
      vLogFile.flush();

      String vDirectoryPath = "";
      String vFilePattern = "";
      String vFileSuffixParameter = "";
      String vFileSuffix = "1";
      if (args != null) {
        if (args.length != 0) {
          vDirectoryPath = args[0];
        }
        if (args.length > 1) {
          vFilePattern = args[1];
        }

        if (args.length > 2) {
          vFileSuffixParameter = args[2];
          if ("NO_SUFFIX".equals(vFileSuffixParameter.toUpperCase()))
          {
            vFileSuffix = "";
            System.out.println("Not using a file suffix");
            vLogFile.println("Not useing a file suffix");
          } else {
            System.out.println("Using the default file suffix of: " + vFileSuffix);
            vLogFile.println("Using the default file suffix of: " + vFileSuffix);
          }
        }
      }
      
      if (vDirectoryPath != null && vDirectoryPath.length() > 0
                        && vFilePattern != null && vFilePattern.length() > 0) {
          vLogFile.println("Directory Path: " + vDirectoryPath);
          vLogFile.println("File Pattern: " + vFilePattern);
          vLogFile.flush();
          X12ClaimStatFormatter x12ClaimStatFormatter = new X12ClaimStatFormatter();
          x12ClaimStatFormatter.processX12ClaimStatFormatter(
            vDirectoryPath,
            vFilePattern,
            vFileSuffix,
            vLogFile);
      } else {
          vLogFile.println("X12ClaimStatFormatter: Please Pass both Path and Pattern");
          System.out.println("X12ClaimStatFormatter: Please Pass both Path and Pattern");
      }
      // Put a completion message into the log file
      vLogFile.println ("Process completed successfully");
      vLogFile.flush();
      processSuccess = true;
      
    } catch (OutOfMemoryError e) {
        Logger.getLogger(gov.va.med.hac.edi.utils.X12ClaimStatFormatter.class.getName()).log(Level.SEVERE,
              "!!!!! Out Of Memory Error !!!!!", e);
        vLogFile.println("!!!!! Out Of Memory Error !!!!!");
    } catch (Exception e) {
        Logger.getLogger(gov.va.med.hac.edi.utils.X12ClaimStatFormatter.class.getName()).log(Level.SEVERE,
            "Exception in processing", e);
        e.printStackTrace(vLogFile);
    } finally {
      if (!processSuccess) {
        vLogFile.println("!!!!! Process seems to have failed !!!!!");
      }
      if (vLogFile != null) {
        MainUtils.stampEndingTime("X12ClaimStatFormatter End", vLogFile);
        vLogFile.flush();
      }
    }
  }

  private void processX12ClaimStatFormatter(String vDirectoryPath
                                  , String vFilePattern
                                  , String vFileSuffix
                                  , PrintWriter vLogFile)
  throws IOException
  {
    File[] filesToProcess = null;
    boolean fileSuccess;
    
    // Open the directory
    File givenDirectory = new File(vDirectoryPath);
    
    // If the directory does not exist, log the fact
    if (!givenDirectory.exists()) {
      vLogFile.println ("Given directory not found");
      System.out.println("Given directory not found");
    } else {
      // Get all the files with the given prefix
      X12ClaimStatFormatter.SubstringFileFilter substringFileFilter
          = new X12ClaimStatFormatter.SubstringFileFilter(vFilePattern);
      filesToProcess = givenDirectory.listFiles(substringFileFilter);
    }
    
    // If no files were found, log the fact
    if (filesToProcess == null || filesToProcess.length == 0) {
      vLogFile.println("No Files Found!");
      System.out.println("No files found!");
    } else {
      vLogFile.println("Files Found: " + filesToProcess.length);
      // Loop through the files found
      for (int i = 0; i < filesToProcess.length; i++) {
        File nextFile = filesToProcess[i];
        String nextFilePath = nextFile.getAbsolutePath();
         // Call reformat for each file found
         String altFilePath = createReformattedFile(nextFilePath);
         File reformattedFile = new File(altFilePath);
         
         // Delete the original file
         fileSuccess = nextFile.delete();
         if (!fileSuccess) {
           throw new IOException ("Problem with deleting original file");
         }
         
         // Rename the formatted file
         File fileWithSuffix = new File (nextFilePath + vFileSuffix);
         reformattedFile.renameTo(fileWithSuffix);
//         // Move the reformatted file to overwrite the old file
//         fileSuccess = moveFileOver(reformattedFile, nextFile);
         
      }
    }
  }
  
  // This file filter accepts files that have the given string inside their name, 
  class SubstringFileFilter implements FileFilter {
    
    String substring;
    
    SubstringFileFilter(String inString) {
      substring = inString.toLowerCase();
    }
    
    public boolean accept(File f) {
        if (f.isDirectory()) return false;
        String name = f.getName().toLowerCase();
        if (name.indexOf(substring) >= 0) {
          return true;
        } else {
          return false;
        }
        //return name.endsWith(substring);
    } // end method accept
  } // end inner class SubstringFileFilter

  // This method creates a new version of the file with the given path, and
  //   returns the full path of the new file
  // The new file will generally combine lines that were split by Cache to overcome
  //   some line length limits in Cache.
  // A tilde denotes an extension line.
  // Note that this is not general line-combining code.  In particular, the third
  //   line is skipped altogether.
  public static String createReformattedFile(String inFileName)
  {
		String nextLine = null ;
    String reformattedFileName = null;
		final StringBuffer lineStorage = new StringBuffer();
		int lineCounter = 0 ;

		try
		{
			// Set up the input and output files
      final BufferedReader in = new BufferedReader( new FileReader(inFileName) ) ;
			reformattedFileName = inFileName + "_NO_TILDES";
      final File resFile = new File(reformattedFileName) ;
			PrintWriter out = new PrintWriter(new FileOutputStream(resFile) ) ;

			// Loop through the lines in the input file
      while ( (nextLine = in.readLine()) != null) {
				
        // Ignore the third line, a special case in our processing
        if (lineCounter != 2) {
				  // If the current line begins with a tilde ...
          if (nextLine.substring(0,1).equalsIgnoreCase("~")) {
            
            // ... print out both together ...
            out.println(lineStorage + nextLine.substring(1,nextLine.length()));
            
            // ... and clear out the line storage.
            lineStorage.delete(0, lineStorage.length()) ;
				  }
				  
          // If the current line does not begin with a tilde, store it.
          else {
					  lineStorage.append(nextLine) ;
				  }
				}
				lineCounter++ ;
			}
      
      // Close up
      in.close();
		  out.flush() ;
      out.close();
		}
		catch (Exception ex)
		{
			ex.printStackTrace() ;
			System.err.println(ex) ;
		}
    
    return reformattedFileName;
	}

     // Move the first file over the second one
  public static boolean moveFileOver(File inFileThatStays, File inFileThatGoes)
  throws IOException
  {
     boolean fileSuccess;
     
     File tempFile = new File (inFileThatGoes.getAbsolutePath() + ".delete");
    
     // Move the file that goes to a temporary one
     fileSuccess = inFileThatGoes.renameTo(tempFile);
     if (!fileSuccess) {
       throw new IOException ("Problem with renaming second file to temporary file");
     }
     
     // Move the first file over the second one
     fileSuccess = inFileThatStays.renameTo(inFileThatGoes);
     if (!fileSuccess) {
       throw new IOException ("Problem with renaming first file to second file");
     }
     
     // Delete the temporary file
     fileSuccess = tempFile.delete();
     if (!fileSuccess) {
       throw new IOException ("Problem with deleting temporary file");
     }
     
     return true;
  }
}