package gov.va.med.hac.edi.utils.persistent;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DatabaseUtils {
	static final Logger summaryLOGGER = LoggerFactory.getLogger(DatabaseUtils.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace" + DatabaseUtils.class);
	
	
    public static Connection createDbConnection(String url, String username, String password)
    {
    	try {
    		// Load Oracle JDBC Driver
    		DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
    		// Connect to Oracle Database
    		return DriverManager.getConnection(url, username, password);
    	} catch (Exception e) {
    		summaryLOGGER.error("Database connection error.");
    		traceLOGGER.error("Database connection error.");
    		// e.printStackTrace();
    		return null;
    	}
    }

    public static void close(Connection connection)
    {
        try
        {
            if (connection != null)
            {
                connection.close();
            }
        }
        catch (SQLException e)
        {
    		summaryLOGGER.error("Connection closing warning.");
    		traceLOGGER.error("Connection closing warning.");
        	// e.printStackTrace();
        }
    }


    public static void close(PreparedStatement prepStmt)
    {
        try
        {
            if (prepStmt != null)
            {
            	prepStmt.close();
            }
        }
        catch (SQLException e)
        {
    		summaryLOGGER.error("SQL Prepared Statement closing warning.");
    		traceLOGGER.error("SQL Prepared Statement closing warning.");
            // e.printStackTrace();
        }
    }

    public static void close(Statement st)
    {
        try
        {
            if (st != null)
            {
                st.close();
            }
        }
        catch (SQLException e)
        {
    		summaryLOGGER.error("SQL Statement closing warning.");
    		traceLOGGER.error("SQL Statement closing warning.");
            // e.printStackTrace();
        }
    }

    public static void close(ResultSet rs)
    {
        try
        {
            if (rs != null)
            {
                rs.close();
            }
        }
        catch (SQLException e)
        {
    		summaryLOGGER.error("SQL Resultset closing warning.");
    		traceLOGGER.error("SQL Resultset closing warning.");
        	// e.printStackTrace();
        }
    }

    public static void rollback(Connection connection)
    {
        try
        {
            if (connection != null)
            {
                connection.rollback();
            }
        }
        catch (SQLException e)
        {
    		summaryLOGGER.error("SQL Rollback exception warning.");
    		traceLOGGER.error("SQL Rollback exception warning.");
        	// e.printStackTrace();
        }
    }

    public static List<Map<String, Object>> map(ResultSet rs) throws SQLException
    {
        List<Map<String, Object>> results = new ArrayList<Map<String, Object>>();

        try
        {
            if (rs != null)
            {
                ResultSetMetaData meta = rs.getMetaData();
                int numColumns = meta.getColumnCount();
                while (rs.next())
                {
                    Map<String, Object> row = new HashMap<String, Object>();
                    for (int i = 1; i <= numColumns; ++i)
                    {
                        String name = meta.getColumnName(i);
                        Object value = rs.getObject(i);
                        row.put(name, value);
                    }
                    results.add(row);
                }
            }
        }
        finally
        {
            close(rs);
        }

        return results;
    }	

}

