package gov.va.med.hac.edi.utils.persistent;

import gov.va.med.hac.edi.utils.StringUtils;
import gov.va.med.hac.edi.utils.jpa.JPAUtil;
import gov.va.med.hac.edi.utils.jpa.LogUtils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/*
    selectStatement = "select CPE_SYS_UTILS.cpe_guid() from dual"
    cursor = connection.cursor()
    cursor.execute(selectStatement)
    row = cursor.fetchone()
    guid = row[0]        
    insertStatement = "INSERT into REPORTS_835 columns (BATCH_TRACKING_KEY,BATCH_NUMBER,CONTAINING_FILENAME)" +\
    "VALUES('" +guid+ "','" +str(prewrapFile)+ "','"+str(runId)+"')"            
    cursor.execute(insertStatement)
*/  

public class EDIWrapUtil {
	
	
	
	static final Logger LOGGER = LoggerFactory.getLogger(EDIWrapUtil.class);
    public static void main(String[] args) {
    	LOGGER.info("Starting EDIWrapUtil");
        File configFile = null;
        EntityManager em = null;
        try {
			configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
			em = JPAUtil.getEntityManager(configFile, "noClasses");
		} catch (FileNotFoundException e) {
	    	LOGGER.error("Error getting entity manager", e);
			return;
		} catch (IOException e) {
			LOGGER.error("Error getting entity manager", e);
			return;
		}

        // Other local variables
        final StringBuffer rLine = new StringBuffer();
        String line = "";
        String wLine = "";
        PrintWriter out = null;

        if (args.length == 3 || args.length == 4) {
          // Get the program parameters
          final String wrapFileName = args[0];
          String outFileName = args[1];
          final String fileType = args[2];
          String fileSuffix = null;
          
          if(args.length == 4)
          {
        	  fileSuffix = args[3];
          }
          
          LOGGER.info("wrapFileName: " + wrapFileName);
          LOGGER.info("outFileName: " + outFileName);
          LOGGER.info("fileType: " + fileType);
        
            try {
                final BufferedReader in = new BufferedReader(new FileReader(wrapFileName));
                while ((line = in.readLine()) != null) {
                    rLine.append(line);
                }

                line = rLine.toString();

                // RUssell 05/16 - this is about the place to rename the file so it can work for moveIT
                if ("835".equals(fileType)) {
                	String payerId = line.substring(35, 40);
                    if (line.substring(35, 40).equals("84146")) {                        							
                        outFileName += "HAC84146"+fileSuffix+".DAT";
                    } else if (line.substring(35, 40).equals("84147")) {                        
                        outFileName += "HAC84147"+fileSuffix+".DAT";
                    } else if (line.substring(35, 40).equals("12115")) {                        
                        outFileName += "FEE12115"+fileSuffix+".DAT";
                    } else if (line.substring(35, 40).equals("12116")) {                        
                        outFileName += "FEE12116"+fileSuffix+".DAT";
                    }
                    int startIndexOut = outFileName.lastIndexOf("\\");
                    int startIndexWrap = wrapFileName.lastIndexOf("\\");
                    String insertStatement = "INSERT into REPORTS_835 columns (BATCH_TRACKING_KEY,BATCH_NUMBER,CONTAINING_FILENAME) " +
                    "VALUES(cpe_sys_utils.cpe_guid(),'" +wrapFileName.substring(startIndexWrap+1)+ "','"+outFileName.substring(startIndexOut+1)+"')";                    
                    Query insertQuery = em.createNativeQuery(insertStatement);
                    
                    String updateTransactionCount = "update REPORTS_835 set NUM_835_IN_BATCH = (select count(*) from CLAIM_PYMT_INFO where file_number='"
                    + wrapFileName.substring(startIndexWrap+1)+"') where batch_number='"+wrapFileName.substring(startIndexWrap+1)+"'";
                    Query updateTransactionCountQuery = em.createNativeQuery(updateTransactionCount);
                    
                    String update835ErrorCount = " update REPORTS_835 set ERRORS_IN_BATCH = (select count(*) from TRLOG where run_id in ( " +
                    " select distinct RUN_ID from TRLOG where filename like '%" + wrapFileName.substring(startIndexWrap+1) +
                    "' and DIR='CMP' AND TRANS_NAME='835' AND RUN_DATE > TRUNC(SYSDATE - 4)) AND STAT = 'E' AND RUN_DATE > TRUNC(SYSDATE - 4)), PAYER_ID='" 
                    + payerId + 
                    "', HCCH_NAME= (select CH_NAME from EDI_835_PROCESS where FILE_NUMBER='" + wrapFileName.substring(startIndexWrap+1) + 
                    "')  where batch_number='"+wrapFileName.substring(startIndexWrap+1)+"'";
                    Query update835ErrorCountQuery = em.createNativeQuery(update835ErrorCount);
                    
                    
                    String updateErrorDetail = "insert into REPORTS_835_DETAIL (REPORT_835_DETAIL_KEY, BATCH_NUMBER,TRANS_NO, MSG_TEXT,FIELDVAL) " +                               "(select  cpe_sys_utils.cpe_guid(),'"+wrapFileName.substring(startIndexWrap+1)+"',TRANS_NO, MSG_TEXT,FIELDVAL from TRLOG where " + 
                    "RUN_ID IN  (select distinct RUN_ID from TRLOG where filename like '%"+wrapFileName.substring(startIndexWrap+1)+
                    "' and DIR='CMP' AND TRANS_NAME='835' AND RUN_DATE > TRUNC(SYSDATE - 4))  AND STAT='E' AND RUN_DATE > TRUNC(SYSDATE - 4))";
                    Query updateErrorDetailQuery = em.createNativeQuery(updateErrorDetail);
                    
                    em.getTransaction().begin();
                    insertQuery.executeUpdate();
                    updateTransactionCountQuery.executeUpdate(); 
                    update835ErrorCountQuery.executeUpdate();
                    updateErrorDetailQuery.executeUpdate();
                    em.getTransaction().commit();
                }

                final File resFile = new File(outFileName);
                out = new PrintWriter(new FileOutputStream(resFile, true));

                if (line.length() > 80) {
                    for (int k = 0; k < rLine.length() / 80; k++) {

                        wLine = line.substring(0, 80);

                        line = line.substring(80, line.length());
                        out.println(wLine);
                    }
                }

                if (line.length() > 0) {
                    wLine = StringUtils.rPadSpaces(line, 80);
                    out.println(wLine);
                }
            } catch (Exception ioe) {
                ioe.printStackTrace();
                LOGGER.error("An exception occured in 835_Wrap: ", ioe);
            }
            out.flush();
        } else {
        	LOGGER.error("Please execute 835 Wrap by passing the input file path, output file path, and either '997' or '835'");
        }
        LOGGER.info("Finishing EDIWrapUtil");
    }         
}
