package gov.va.med.hac.edi.regression;

import gov.va.med.hac.edi.extract.x12.jpa.CacheX12Extract;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;
import java.util.Properties;

import javax.naming.NamingException;

public class CacheX12ExtractRegression extends ExtractRegression {

	public CacheX12ExtractRegression(Connection inConnection,
			Properties inProperties, boolean inVerbose) {
		super(inConnection, inProperties, inVerbose);
	}

	public int compareFilesLegacyToNew() throws SQLException {		
		// TWP:Need to pull in a diff utility - I added code below but am diffing manually for now
		return 0;
	}

	/**
	 * This method executes the refactored code in this JVM. If the refactored
	 * process does not run correctly you may want to convert this method to use
	 * a ProcessBuilder that sets up an isolated environment.
	 * 
	 * @throws Exception
	 */
	public void runNewCode() throws IOException, NamingException,
			DbTransactionEnderException {
		String destinationDirectory = (String) properties.get("new.extract.dir");
		String clearinghouseId = (String) properties.get("clearinghouse.id");
		String[] args = new String[] { "-s " + clearinghouseId,
		    "-p " + destinationDirectory,
		    "-c"
		    //,"-noPoa"
        };

		System.out.println("Started running new code - " + Calendar.getInstance().getTime());
		CacheX12Extract.main(args);
		System.out.println("Finished running new code - "	+ Calendar.getInstance().getTime());
	}

	public void resetStatusesOnSourceTables() throws SQLException {
		Statement stmt = connection.createStatement();
		ResultSet rs;
		String resetStatusesString = "UPDATE CLAIM " + "SET STATUS_CODE = 'NEW'";
		rs = stmt.executeQuery(resetStatusesString);
		connection.commit();
	}

	/**
	 * 
	 * @throws IOException
	 */
	public void copyExtractFilesToExternalTablesDir() throws IOException {
		System.out
				.println("Not copying Files To an external dir for this extract. - Please just diff the files");
	}

	public int getClaimCount(String inStatus) throws SQLException {
		// Make sure data got loaded
		Statement stmt = connection.createStatement();
		String claimCountQueryString = "SELECT COUNT (*) AS NUM_CLAIMS FROM CLAIM";
		if (inStatus != null) {
			claimCountQueryString += " WHERE STATUS_CODE = '" + inStatus + "'";
		}
		ResultSet rs;
		rs = stmt.executeQuery(claimCountQueryString);

		// Get the result
		int numClaims = -1;
		if (rs.next()) {
			numClaims = rs.getInt("NUM_CLAIMS");
		}

		return numClaims;
	}

	void diff(String[] x, String[] y) {
		// number of lines of each file
		int M = x.length;
		int N = y.length;

		// opt[i][j] = length of LCS of x[i..M] and y[j..N]
		int[][] opt = new int[M + 1][N + 1];

		// compute length of LCS and all subproblems via dynamic programming
		for (int i = M - 1; i >= 0; i--) {
			for (int j = N - 1; j >= 0; j--) {
				if (x[i].equals(y[j]))
					opt[i][j] = opt[i + 1][j + 1] + 1;
				else
					opt[i][j] = Math.max(opt[i + 1][j], opt[i][j + 1]);
			}
		}

		// recover LCS itself and print out non-matching lines to standard
		// output
		int i = 0, j = 0;
		while (i < M && j < N) {
			if (x[i].equals(y[j])) {
				i++;
				j++;
			} else if (opt[i + 1][j] >= opt[i][j + 1])
				System.out.println("< " + x[i++]);
			else
				System.out.println("> " + y[j++]);
		}

		// dump out one remainder of one string if the other is exhausted
		while (i < M || j < N) {
			if (i == M)
				System.out.println("> " + y[j++]);
			else if (j == N)
				System.out.println("< " + x[i++]);
		}
	}

}
