package gov.va.med.hac.edi.regression;

import gov.va.med.hac.edi.era.jpa.ERepos835Load;
import gov.va.med.hac.edi.era.jpa.Edi835ErrorQueue;
import gov.va.med.hac.edi.era.jpa.HacStg835Load;
import gov.va.med.hac.edi.reposload.ncpdp.jpa.NCPDPLoad;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import javax.naming.NamingException;

public class Edi835ErrorQueueRegression extends LoadRegression {

  public Edi835ErrorQueueRegression(Connection inConnection, Properties inProperties, Boolean inVerbose) {
    super(inConnection, inProperties, inVerbose);
  }

  /**
   */
  public void setupBeforeLegacyRunSingleSchema() throws SQLException, IOException {

	  //Make a back up copy of the erepos/hac stg tables so the new code can run with it
	  //after the legacy code runs
	  dropTables(newSchema, "_HOLD");
	  cloneTables(newSchema, newSuffix, newSchema, "_HOLD");
	        
  }
  
  /**
   */
  public void setupBeforeLegacyRunTwoSchemas() throws SQLException,
      IOException {	  
	  
  }


  
  /**
   */
  public void setupBetweenRunsSingleSchema() throws SQLException, IOException {
    // Clear out the "legacy" tables
    dropTables(legacySchema, legacySuffix);
    
    // Copy the "legacy" data into the tables that have the legacy suffix
    cloneTables(newSchema, newSuffix, legacySchema, legacySuffix);
        
    // Get rid of them we are about to copy the data back into them
    dropTables(newSchema, newSuffix);

    //Start with the data that the legacy code started with
    cloneTables(newSchema, "_HOLD", newSchema, newSuffix);
        
  }

  /**
   */
  public void setupBetweenRunsTwoSchemas() throws SQLException, IOException {
  }


  public void runLegacyCode() throws IOException, InterruptedException {
	    String javaExecPropertyName = "java.exec";
	    String legacyClasspathPropertyName = "legacy.classpath3";
	    String legacyProcessPropertyName = "legacy.process3";
	    String legacyJavaArgsPropertyName = "legacy.javaargs3";
	    String legacyAppArgsPropertyName = "legacy.appargs3";
	    runLegacyCode(javaExecPropertyName, legacyClasspathPropertyName, legacyProcessPropertyName, legacyJavaArgsPropertyName, legacyAppArgsPropertyName);
  }
  
  /**
   * This method applies the "template method" pattern.
   * It drives the overall algorithm for loading data into database tables.
   * Subclasses may change the implementations of the pieces, but not the algorithm.
   * 
   */
  public int dataLoadRegression() throws IOException, SQLException,
       InterruptedException, DbTransactionEnderException, NamingException {
	 
	seedTRLog();
	  
	if(isSingleSchema())
	{
		dropTables(legacySchema, legacySuffix);
	}
    // The legacy code deletes the source files, so copy them in again
    setupBeforeLegacyRun();

    // Run the legacy version of the code, which will fill the target tables, not the legacy tables.
    runLegacyCode();
    
    // The legacy code deletes the source files, so copy them in again
    setupBetweenRuns();

    // Fill the new tables using the new version of the code
    runNewCode();
    
    // Call a PL/SQL procedure to compare the legacy contents against the "new" contents.
    int diffs = compareAllTablesLegacyToNew();
    
    cleanUpTRLog();
    
	if(isSingleSchema())
	{    
		dropTables(newSchema, "_HOLD");
	}

    return diffs;
  }

  
  /**
   * This method executes the refactored code in this JVM. If the refactored
   * process does not run correctly you may want to convert this method to use
   * a ProcessBuilder that sets up an isolated environment.
   * 
   * @throws Exception
   */
  public void runNewCode() throws IOException, NamingException, DbTransactionEnderException {    
    String runId = (String) properties.get("new.run.id");
    String[] errorQueueArgs = new String[]{"-r " + runId};
    Edi835ErrorQueue.main(errorQueueArgs);    
  }


  public void seedTRLog() throws SQLException
  {
	  if(isSingleSchema())
	  {
		  runSQLUpdatesForErrorQueue(null);
	  }
	  else
	  {
		  runSQLUpdatesForErrorQueue(legacySchema);
		  runSQLUpdatesForErrorQueue(newSchema);
	  }
  }

private void runSQLUpdatesForErrorQueue(String schemaName) throws SQLException {
	
	  Statement stmt = connection.createStatement();
	  String sql1 = (String) properties.get("compcheck.sql1"); 
	  stmt.executeUpdate( replaceSchema(sql1, schemaName) );
	 
	  String sql2 = (String) properties.get("compcheck.sql2");
	  if( sql2 != null && sql2.toString().trim().length() > 0)
	  {
		  stmt.executeUpdate( replaceSchema(sql2, schemaName) );
	  }
	  
	  String sql3 = (String) properties.get("compcheck.sql3");
	  if( sql3 != null && sql3.toString().trim().length() > 0)
	  {
		  stmt.executeUpdate( replaceSchema(sql3, schemaName) );
	  }
	  connection.commit();
	  stmt.close();
}

private String replaceSchema(String line, String schemaName)
{
	String lineToReturn = null;
	if(schemaName == null)
	{
		lineToReturn = line.replaceAll("<SCHEMA>.", "");
	}else
	{
		lineToReturn = line.replaceAll("<SCHEMA>", schemaName);
	}
	return lineToReturn;
}
  
  public void cleanUpTRLog() throws SQLException
  {
		 if(isSingleSchema())
		  {
			 runSQLCleanUpForErrorQueue(null);
		  }
		  else
		  {
			  runSQLCleanUpForErrorQueue(legacySchema);
			  runSQLCleanUpForErrorQueue(newSchema);
		  } 
  }

private void runSQLCleanUpForErrorQueue(String schemaName) throws SQLException {
	  Statement stmt = connection.createStatement();
	  String sql = (String) properties.get("compcheck.cleanUp");
	  stmt.executeUpdate( replaceSchema(sql, schemaName) );
	  connection.commit();
	  stmt.close();
}

}
