package gov.va.med.hac.edi.regression;

import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.jpa.FileUtils;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Properties;

import javax.naming.NamingException;

public abstract class ExtractRegression extends RegressionBase {
  public ExtractRegression(Connection inConnection, Properties inProperties, Boolean inVerbose) {
    super(inConnection, inProperties, inVerbose);
  }

  public abstract void resetStatusesOnSourceTables() throws SQLException;

  public abstract void copyExtractFilesToExternalTablesDir() throws IOException;

  public int compareFilesLegacyToNew() throws SQLException {

    System.out.println("In compareFilesLegacyToNew");

    return 1;
  }

  /**
   * This method applies the "template method" pattern.
   * It drives the overall algorithm for extracting data from database tables.
   * Subclasses may change the implementations of the pieces, but not the algorithm.
   * 
   */
  public final int dataExtractRegression() throws IOException, SQLException,
       InterruptedException, DbTransactionEnderException, NamingException {

    // Legacy code will look at the status in table CLAIM.  In case it was left at something other
    //   than "NEW", we update to NEW now.
    resetStatusesOnSourceTables();
    
    System.out.println("Going to legacyExtractRegression");
    legacyExtractRegression();
    
    // We can reuse the records that are in the tables, and just used to run the legacy code.
    // We don't need to copy that code in again, since we assume the loading matched
    // But, we might need to update the statuses of those records so they get picked up again
    resetStatusesOnSourceTables();
    
    // Run the new version of the code
    System.out.println("Going to newExtractRegression");
    newExtractRegression();

    // These extracted files can be compared with a file-comparison utiltiy, because
    //     they involve ORDER BY clauses.
    
//    // Clear out the external tables directory
//    String externalTablesDir = (String) properties.get("external.tables.dir");
//    System.out.println ("Clearing external tables directory: " + externalTablesDir);
//    clearDirectory(externalTablesDir); 
//    
//    copyExtractFilesToExternalTablesDir();
//    
//    // Call a PL/SQL procedure to compare the legacy file against the new file.
//    int diffs = compareFilesLegacyToNew();
//
//    return diffs;
    return 1;
  }

  public final void legacyExtractRegression()
    throws IOException, InterruptedException
    {
    // Clear out the directory that serves as the target of extraction
    String legacyExtractDirString = (String) properties.get("legacy.extract.dir");
    clearDirectory(legacyExtractDirString); 
    System.out.println("Cleared Legacy Extract Directory");
    
    // Run the legacy version of the code
    System.out.println("Going to runLegacyCode");
    System.err.println("Starting legacy " + Calendar.getInstance().getTime());
    runLegacyCode();
    System.err.println("Ending legacy " + Calendar.getInstance().getTime());
  }
  
  public final void newExtractRegression()
  throws IOException, InterruptedException, DbTransactionEnderException, NamingException
  {
    // Clear out the directory that serves as the target of extraction
    String newExtractDirString = (String) properties.get("new.extract.dir");
    clearDirectory(newExtractDirString); 
    System.out.println("Cleared New Extract Directory");

    System.err.println("Starting New " + Calendar.getInstance().getTime());
    runNewCode();
    System.err.println("Ending New " + Calendar.getInstance().getTime());
    
  
  }
}
