package gov.va.med.hac.edi.regression;

import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Properties;

import javax.naming.NamingException;

public abstract class LoadRegression extends RegressionBase {

  public LoadRegression(Connection inConnection, Properties inProperties, Boolean inVerbose) {
    super(inConnection, inProperties, inVerbose);
  }

  /**
   * This method applies the "template method" pattern.
   * It drives the overall algorithm for loading data into database tables.
   * Subclasses may change the implementations of the pieces.
   * 
   */
  public int dataLoadRegression() throws IOException, SQLException,
       InterruptedException, DbTransactionEnderException, NamingException {
    // The legacy code deletes the source files, so copy them in again
    setupBeforeLegacyRun();

    // Run the legacy version of the code, which will fill the target tables, not the legacy tables.
    System.out.println("Starting legacy " + Calendar.getInstance().getTime());
    runLegacyCode();
    System.out.println("Ending legacy " + Calendar.getInstance().getTime());
    
    // The legacy code deletes the source files, so copy them in again
    setupBetweenRuns();

    // Fill the new tables using the new version of the code
    System.out.println("Starting new " + Calendar.getInstance().getTime());
    runNewCode();
    System.out.println("Ending new " + Calendar.getInstance().getTime());

    // Call a PL/SQL procedure to compare the legacy contents against the "new" contents.
    int diffs = compareAllTablesLegacyToNew();

    return diffs;
  }
  
  /**
   */
  public void setupBeforeLegacyRunSingleSchema() throws SQLException, IOException {
    // Clear out the tables where the legacy code will write
    // With a single schema, these are the "new" tables
    clearTables(newSchema, newSuffix);
    
    // Clear any files or folders out of the directory that is the source of legacy loading
    String legacyLoadingDir = (String) properties.get("legacy.loading.dir");
    clearDirectory(legacyLoadingDir); 
    
    // Copy input data from staging location to the directory that is the source of legacy loading
    String legacyStagingDir = (String) properties.get("legacy.staging.dir");
    copyFiles(legacyStagingDir, legacyLoadingDir);
    
  }

  /**
   */
  public void setupBeforeLegacyRunTwoSchemas() throws SQLException, IOException {
    // Clear out the tables where the legacy code will write\
    // With two schemas, these are the "legacy" tables
    clearTables(legacySchema, legacySuffix);
    
    // Clear any files or folders out of the directory that is the source of legacy loading
    String legacyLoadingDir = (String) properties.get("legacy.loading.dir");
    clearDirectory(legacyLoadingDir); 
    
    // Copy input data from staging location to the directory that is the source of legacy loading
    String legacyStagingDir = (String) properties.get("legacy.staging.dir");
    copyFiles(legacyStagingDir, legacyLoadingDir);
    
  }

  /**
   */
  public void setupBetweenRunsSingleSchema() throws SQLException, IOException {
    // Clear out the "legacy" tables
    dropTables(legacySchema, legacySuffix);
    
    // Copy the "legacy" data into the tables that have the legacy suffix
    cloneTables(newSchema, newSuffix, legacySchema, legacySuffix);
    
    // Clear the legacy data out of the "new" tables
    clearTables(newSchema, newSuffix);
    
    // Clear any files or folders out of the directory that is the source of new loading
    String newLoadingDir = (String) properties.get("new.loading.dir");
    clearDirectory(newLoadingDir); 
    
    // Copy input data from staging location to the directory that is the source of new loading
    String newStagingDir = (String) properties.get("new.staging.dir");
    copyFiles(newStagingDir, newLoadingDir);
    
  }

  /**
   */
  public void setupBetweenRunsTwoSchemas() throws SQLException, IOException {
    // Clear any leftover data out of the "new" tables
    clearTables(newSchema, newSuffix);
    
    // Clear any files or folders out of the directory that is the source of new loading
    String newLoadingDir = (String) properties.get("new.loading.dir");
    clearDirectory(newLoadingDir); 
    
    // Copy input data from staging location to the directory that is the source of new loading
    String newStagingDir = (String) properties.get("new.staging.dir");
    copyFiles(newStagingDir, newLoadingDir);
    
  }

}
