package gov.va.med.hac.edi.regression;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import javax.naming.NamingException;

import gov.va.med.hac.edi.extract.b1.jpa.RxExtractB1;
import gov.va.med.hac.edi.regression.*;
import gov.va.med.hac.edi.reposload.ncpdp.jpa.NCPDPLoad;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.jpa.FileUtils;

public class RxExtractB1Regression extends ExtractRegression {

  public RxExtractB1Regression(Connection inConnection, Properties inProperties, Boolean inVerbose) {
    super(inConnection, inProperties, inVerbose);
  }

  public int compareFilesLegacyToNew() throws SQLException {

    // Call a PL/SQL procedure to compare the legacy contents against the "new" contents.
    int diffs = compareAllTablesLegacyToNew();

    return diffs;
  }

  /**
   * This method executes the refactored code in this JVM. If the refactored
   * process does not run correctly you may want to convert this method to use
   * a ProcessBuilder that sets up an isolated environment.
   * 
   * @throws Exception
   */
  public void runNewCode() throws IOException, NamingException, DbTransactionEnderException {
    String destinationDirectoryOption = "-p";
    String destinationDirectory = (String) properties.get("new.extract.dir");
    String senderIdOption = "-s";
    String senderId = (String) properties.get("sender.id");
    String[] args = new String[]{destinationDirectoryOption, destinationDirectory,
        senderIdOption, senderId};
    RxExtractB1.main(args);

    System.out.println("Finished running new code");
  }

  public void resetStatusesOnSourceTables() throws SQLException {
    Statement stmt = connection.createStatement();
    ResultSet rs;
    String resetStatusesString =
        "UPDATE NCPDP_CLAIM "
      + "SET status = 'NEW'";
    rs = stmt.executeQuery(resetStatusesString);
    
    connection.commit();

  }

  /**
   * 
   * @throws IOException
   */
  public void copyExtractFilesToExternalTablesDir() throws IOException {
    String externalTablesDirString = (String) properties.get("external.tables.dir");
    File externalTablesDir = new File(externalTablesDirString);
    
    // Copy the extracted legacy file to the external tables directory
    String legacyExtractDirString = (String) properties.get("legacy.extract.dir");
    File legacyExtractDir = new File(legacyExtractDirString);
    if (legacyExtractDir.isDirectory()) {
      String[] children = legacyExtractDir.list();
      if (children.length != 1)
        throw new RuntimeException(
            "Too many files encountered in:" + legacyExtractDir);
      File legacyExtractFile = new File(legacyExtractDir, children[0]);
      File legacyExternalTablesFile = new File(externalTablesDir, "b1orig.txt");
      FileUtils.fastCopyFile(legacyExtractFile, legacyExternalTablesFile);
    }

    // Copy the extracted new file to the external tables directory
    String newExtractDirString = (String) properties.get("new.extract.dir");
    File newExtractDir = new File(newExtractDirString);
    if (newExtractDir.isDirectory()) {
      String[] children = newExtractDir.list();
      if (children.length != 1)
        throw new RuntimeException(
            "Too many files encountered in:" + newExtractDir);
      File newExtractFile = new File(newExtractDir, children[0]);
      File newExternalTablesFile = new File(externalTablesDir, "b1new.txt");
      FileUtils.fastCopyFile(newExtractFile, newExternalTablesFile);
    }

  }

  public int getClaimCount(String inStatus) throws SQLException {
    // Make sure data got loaded
    Statement stmt = connection.createStatement();
    String claimCountQueryString = "SELECT COUNT (*) AS NUM_CLAIMS FROM NCPDP_CLAIM";
    if (inStatus != null) {
      claimCountQueryString += " WHERE status = '" + inStatus + "'";
    }
    ResultSet rs;
    rs = stmt.executeQuery(claimCountQueryString);
    
    // Get the result
    int numClaims = -1;
    if (rs.next()) {
      numClaims = rs.getInt("NUM_CLAIMS");
    }

    return numClaims;
  }

}
