package gov.va.med.hac.edi.regression;

import gov.va.med.hac.edi.era.jpa.ERepos835Load;
import gov.va.med.hac.edi.era.jpa.HacStg835Load;
import gov.va.med.hac.edi.reposload.ncpdp.jpa.NCPDPLoad;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

import javax.naming.NamingException;

public class StgToEreposRegression extends LoadRegression {

  public StgToEreposRegression(Connection inConnection, Properties inProperties, Boolean inVerbose) {
    super(inConnection, inProperties, inVerbose);
  }

    
  /**
   * This method applies the "template method" pattern.
   * It drives the overall algorithm for loading data into database tables.
   * Subclasses may change the implementations of the pieces, but not the algorithm.
   * 
   */
  public int dataLoadRegression() throws IOException, SQLException,
       InterruptedException, DbTransactionEnderException, NamingException {
	
	if(isSingleSchema())
	{
		dropTables(legacySchema, legacySuffix);
	}
    // The legacy code deletes the source files, so copy them in again
    setupBeforeLegacyRun();

    // Run the legacy version of the code, which will fill the target tables, not the legacy tables.
    runLegacyCode();
    
	String javaExecPropertyName = "java.exec";
	String legacyClasspathPropertyName = "legacy.classpath2";
	String legacyProcessPropertyName = "legacy.process2";
	String legacyJavaArgsPropertyName = "legacy.javaargs";
	String legacyAppArgsPropertyName = "legacy.appargs";
    runLegacyCode(javaExecPropertyName, legacyClasspathPropertyName, legacyProcessPropertyName, legacyJavaArgsPropertyName, legacyAppArgsPropertyName);
    // The legacy code deletes the source files, so copy them in again
    setupBetweenRuns();

    // Fill the new tables using the new version of the code
    runNewCode();
    
    // Call a PL/SQL procedure to compare the legacy contents against the "new" contents.
    int diffs = compareAllTablesLegacyToNew();

    return diffs;
  }
  
  /**
   * This method executes the refactored code in this JVM. If the refactored
   * process does not run correctly you may want to convert this method to use
   * a ProcessBuilder that sets up an isolated environment.
   * 
   * @throws Exception
   */
  public void runNewCode() throws IOException, NamingException, DbTransactionEnderException {
    String loadFile = (String) properties.get("legacy.appargs");
    
    String[] hacStgArgs = new String[]{"-i " + loadFile};
    HacStg835Load.main(hacStgArgs);
    System.err.println("The data is loaded.");
    String[] args = new String[]{};
    ERepos835Load.main(args);
  }

  

}
