package gov.va.med.hac.edi.regression;
import gov.va.med.hac.edi.reposload.x12.jpa.X12eReposLoad;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;

import java.io.IOException;
import java.sql.*;
import java.util.Properties;

import javax.naming.NamingException;

public class x12eReposLoadRegression extends LoadRegression {

	public x12eReposLoadRegression(Connection inConnection,
			Properties inProperties, boolean inVerbose) {
		super(inConnection, inProperties, inVerbose);
	}
	
	@Override
	public void setupBeforeLegacyRunSingleSchema() throws SQLException,
			IOException {
		dropTables(legacySchema, legacySuffix);
		clearTables(newSchema, newSuffix);
	}
	
  @Override
	public void runLegacyCode()
	throws IOException, InterruptedException {
	  String javaExecPropertyName = "java.exec";
	  String legacyClasspathPropertyName = "legacy.classpath";
		String legacyProcessPropertyName = "legacy.process";
		String legacyJavaArgsPropertyName = "legacy.javaargs";
		String legacyLoadingDir = "legacy.loading.dir";

		runLegacyProcess(javaExecPropertyName, legacyClasspathPropertyName,
				legacyProcessPropertyName, legacyJavaArgsPropertyName,
				legacyLoadingDir, "stgdir.professional.data", "legacy.appargs.professional");

		runLegacyProcess(javaExecPropertyName, legacyClasspathPropertyName,
				legacyProcessPropertyName, legacyJavaArgsPropertyName,
				legacyLoadingDir, "stgdir.institutional.data", "legacy.appargs.institutional");

		runLegacyProcess(javaExecPropertyName, legacyClasspathPropertyName,
				legacyProcessPropertyName, legacyJavaArgsPropertyName,
				legacyLoadingDir, "stgdir.dental.data", "legacy.appargs.dental");
	}

	private void runLegacyProcess(
	    String javaExecPropertyName,
			String legacyClasspathPropertyName,
			String legacyProcessPropertyName,
			String legacyJavaArgsPropertyName,
			String legacyLoadingDir,
			String stagingDirPropertyName,
			String argsPropertyName)
			throws IOException, InterruptedException {

		
		clearDirectory(properties.getProperty(legacyLoadingDir));
		copyFiles(properties.getProperty(stagingDirPropertyName),properties.getProperty(legacyLoadingDir));

		// Call the runLegacyCode method with 5 parameters; this version is not overridden here,
		//   and goes back to RegressionBase, probably
		runLegacyCode(javaExecPropertyName, legacyClasspathPropertyName,
				legacyProcessPropertyName, legacyJavaArgsPropertyName,
				argsPropertyName);
	}

	@Override
	public void setupBetweenRunsSingleSchema() throws SQLException, IOException {
		String legacyLoadingDir = (String) properties.get("legacy.loading.dir");
		clearDirectory(legacyLoadingDir);
		cloneTables(newSchema, newSuffix, legacySchema, legacySuffix);
		clearTables(newSchema, newSuffix);
	}
	
	@Override
	public void runNewCode() throws IOException, NamingException,
			DbTransactionEnderException {

		// Different Data Files for Each Process
		String argsForProfessionalDataPropertyName = "args.professional.data";
		String argsForInstitutionalDataPropertyName = "args.institutional.data";
		String argsForDentalDataPropertyName = "ars.dental.data";
		runNewCode("new.loading.dir", "stgdir.professional.data",
		    "stgdir.institutional.data",
		    "stgdir.dental.data");
	}

	private void runNewCode(String newLoadingDir,
			String dirWithProfessionalDataPropertyName,
			String dirWithInstitutionalDataPropertyName,
			String dirWithDentalDataPropertyName)
	throws IOException {

		runNewProcess(newLoadingDir, dirWithProfessionalDataPropertyName,
				"type.professional");

		runNewProcess(newLoadingDir, dirWithInstitutionalDataPropertyName,
				"type.institutional");

		runNewProcess(newLoadingDir, dirWithDentalDataPropertyName,
				"type.dental");
	}

	private void runNewProcess(String newLoadingDirPropertyName,
			String newStgDirPropertyName, String typePropertyName)
			throws IOException {
    String loadingDirectory = properties.getProperty(newLoadingDirPropertyName);
    String stagingDirectory = properties.getProperty(newStgDirPropertyName);
    String type = properties.getProperty(typePropertyName);
    
		clearDirectory(loadingDirectory);
		copyFiles(stagingDirectory,loadingDirectory);
		
    String sourcePathOption = "-p";
    String typeOption = "-t";

    String[] args = new String[]{sourcePathOption, loadingDirectory, typeOption, type};
		X12eReposLoad.main(args);
	}

  public int getClaimCount() throws SQLException {
    // Make sure data got loaded
    Statement stmt = connection.createStatement();
    String claimCountQueryString = "SELECT COUNT (*) AS NUM_CLAIMS FROM CLAIM";
    ResultSet rs;
    rs = stmt.executeQuery(claimCountQueryString);
    
    // Get the result
    int numClaims = -1;
    if (rs.next()) {
      numClaims = rs.getInt("NUM_CLAIMS");
    }

    return numClaims;
  }

}
