package gov.va.med.hac.edi.ewv2.beans.reference;

import gov.va.med.hac.edi.ewv2.exceptions.ActionErrorException;
import gov.va.med.hac.edi.ewv2.exceptions.NonUniqueRecordsException;
import gov.va.med.hac.edi.ewv2.persistence.EwvAdministrator;
import gov.va.med.hac.edi.ewv2.security.IDCheckFilter;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.contexts.Context;
import org.jboss.seam.log.Log;

@Name("Ewv2AdministratorsInfo")
@Scope(ScopeType.SESSION)
public class AdministratorsInfoBean {
	
	
    @Logger
    Log                    log;
    
    @In(create = true)
    protected EntityManager  entityManager;
    
    @In
    protected Context        sessionContext;
    
    private static final long serialVersionUID = -2046592591578029243L;
    

	  @SuppressWarnings("unchecked")
	public String getAdminDetailsOfUser() throws ActionErrorException, NonUniqueRecordsException
	  {
		  EwvAdministrator ret = null;
		  String queryString ="select administrator from EwvAdministrator AS administrator where administrator.administratorName = :adminName"; 
	      
	      try {
	          Query query = entityManager.createQuery(queryString);
	          query.setParameter("adminName", getNTUsername());
	          
	          ret =  (EwvAdministrator) query.getSingleResult();
	          log.info("Administrator Name: " + ret.getAdministratorName());
	          log.info("Active: " + ret.getActive());
	          
	      } catch (Throwable t) {
	          if (t instanceof javax.persistence.NoResultException)
	          {
	              log.info("cannot get record for EwvAdministrator: ");
	          }
	          else if(t instanceof javax.persistence.NonUniqueResultException)
	          {
	              log.error("more than 1 Admin record exists for user : " + getNTUsername() );
	              
	              // this means that there are crap claim records in DB
	              throw new NonUniqueRecordsException("more than 1 Reference exists for active: ");
	          }
	          else
	          {
	              // log exception and route user to error page
	              handleException();
	          }
	      }
	      
	      if (ret != null) {
	    	  return Integer.toString(ret.getActive());
	      } else {
	    	  return "0";
	      }	      
	  }
    
    
    // Return the username under which the current user is logged on to Windows
    public String getNTUsername()
    {
        // The sessionContext should have a value under gov.va.med.hac.sec.User,
        // put there by the IDCheckFilter
        Object usernameString = sessionContext.get(IDCheckFilter.SEC_USER);
        
        return ((usernameString == null) ? "" : usernameString.toString().trim());
    }  
    
	  private void handleException() {
			// TODO Auto-generated method stub
			
	  } 
    
}
