/*
 *
 */
package gov.va.med.fee;

import java.text.NumberFormat;
import java.math.RoundingMode;
import java.util.List;
import java.util.ArrayList;

/**
 * Represents controls for results.
 * 
 * @author vhahacnguyes
 */
public class PageControls {

    /** The number formatter. */
    private NumberFormat integerFormatter = NumberFormat.getInstance();

    /** The Constant DEFAULT_PAGE_SIZE. */
    private static final int DEFAULT_PAGE_SIZE = 10;

    /** The page size. */
    private int pageSize = DEFAULT_PAGE_SIZE;

    /** The Constant DEFAULT_PAGE. */
    private static final int DEFAULT_PAGE = 1;

    /** The page. */
    private int page = DEFAULT_PAGE;

    /** PageControls constructor. */
    public PageControls() {
        integerFormatter.setMinimumIntegerDigits(0);
        integerFormatter.setMaximumFractionDigits(0);
        integerFormatter.setGroupingUsed(false);
        integerFormatter.setRoundingMode(RoundingMode.UP);
    }

    /**
     * Gets the page size.
     * 
     * @return the page size
     */
    public final int getPageSize() {
        return pageSize;
    }

    /**
     * Sets the page size.
     * 
     * @param pageSize
     *            the new page size
     */
    public final void setPageSize(int pageSize) {
        this.pageSize = pageSize;
        this.page = 1;
    }

    /**
     * Gets the page.
     * 
     * @return the page
     */
    public final int getPage() {
        return page;
    }

    /**
     * Sets the page.
     * 
     * @param page
     *            the new page
     */
    public final void setPage(int page) {
        this.page = page;
    }

    /** The numberOfResults. */
    private int numberOfResults;

    /**
     * sets the numberOfResults.
     * 
     * @param numberOfResults
     *            The number of results
     */
    public final void setNumberOfResults(int numberOfResults) {
        this.numberOfResults = numberOfResults;
    }

    /**
     * Gets the numberOfResults.
     * 
     * @return the numberOfResults.
     */
    public final int getNumberOfResults() {
        return this.numberOfResults;
    }

    /**
     * gets the number of pages to display in the pagination piece.
     * 
     * @return the number of pages
     */
    public final int getNumberOfPages() {
        double d = (double) this.numberOfResults / (double) this.pageSize;
        Integer in = new Integer(integerFormatter.format(d));

        return in.intValue();
    }

    /** The pageList. */
    private List<Integer> pageList = new ArrayList();

    /**
     * Gets the pageList.
     * 
     * @return the pageList.
     */
    public final List<Integer> getPageList() {
        pageList.clear();

        for (int i = 1; i <= getNumberOfPages(); i++) {           
            Integer integer = new Integer(i);
            pageList.add(integer);
        }
        return pageList;
    }

}
