/*
 * $Id: SortableLong.java,v 1.1 2009/09/02 20:15:38 vhahacnguyes Exp $
 */
package gov.va.med.fee;

import java.io.Serializable;
import java.lang.reflect.Method;

/**
 * This class wraps an entity for sorting purpose.  It is used to avoid having to
 * modify the entity directly or when the database sorting is not used.
 *
 * @param <T> The entity to wrap
 */
@SuppressWarnings("serial")
public final class SortableLong<T> implements Comparable<T>,  Serializable {

    /** The data. */
    private final T data;

    /** The method name. */
    private final String methodName;

    /**
     * Instantiates a new instance.
     *
     * @param data the data
     * @param methodName the method name
     */
    public SortableLong(T data, String methodName) {
        this.data = data;
        this.methodName = methodName;
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public int compareTo(T o) {
        try {
            Class c = data.getClass();
            Method method = c.getMethod(methodName);

            T o1 = ((SortableLong<T>) o).getData();
            Long v1 = (Long) method.invoke(o1);
            Long v2 = (Long) method.invoke(data);
            return v2.compareTo(v1);
        } catch (Exception e) {
            System.err.println("Error in " + this.getClass().getName() + e.getMessage());
        }
        return 0;
    }

    /**
     * Gets the data.
     *
     * @return the data
     */
    public T getData() {
        return data;
    }

}
