package gov.va.med.fee.claim;

import static org.jboss.seam.ScopeType.SESSION;
// import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.AppUser;
import gov.va.med.fee.DataPage;
import gov.va.med.fee.Page;
import gov.va.med.fee.PageControls;
import gov.va.med.fee.PagedListDataModel;
import gov.va.med.fee.sec.HomePageUserData;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.faces.context.FacesContext;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.DataModel;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;

/**
 * AgedClaimsList class.
 */
@Stateful
@Name("agedClaims")
@Scope(SESSION)
public class AgedClaimsList implements AgedClaims {

   /** The log. */
   @Logger
   private Log log;

   /** The em. */
   @PersistenceContext(unitName = "FPPS_Owner")
   private EntityManager em;

   /** The column to sort by. */
   // @RequestParameter
   private String sortCol = "";

   @In(scope = ScopeType.SESSION, required = false)
   @Out(scope = ScopeType.SESSION, required = false)
   List<Long> claimsToDisapprove = new ArrayList<Long>();

   @In(scope = ScopeType.SESSION, required = false)
   @Out(scope = ScopeType.SESSION, required = false)
   private List<Long> claimsToReroute;


   /**
    * Gets the sort column.
    *
    * @return the sortCol.
    */
   public final String getSortCol() {
      return this.sortCol;
   }

   /**
    * Sets the sort column.
    *
    * @param sortCol
    *           the sort column.
    */
   public final void setSortCol(String sortCol) {
      this.sortCol = sortCol;
   }

   /** The direction to sort. */
   // @RequestParameter
   private String sortDir = "";

   /**
    * Gets the sort direction.
    *
    * @return the sortDir.
    */
   public final String getSortDir() {
      return this.sortDir;
   }

   /**
    * Sets the sort direction.
    *
    * @param sortDir
    *           the sort direction.
    */
   public final void setSortDir(String sortDir) {
      this.sortDir = sortDir;
   }

   public final void doSort(String col) {
      if (this.sortCol.equalsIgnoreCase(col)) {
         this.sortDir = this.sortDir.equalsIgnoreCase("asc") ? "desc" : "asc";
      } else {
         this.sortCol = col;
         this.sortDir = "asc";
      }
   }

   private String showPrintPopup = "false";

   public final String getShowPrintPopup() {
      return this.showPrintPopup;
   }

   public final void setShowPrintPopup(String showPrintPopup) {
      this.showPrintPopup = showPrintPopup;
   }

   /** The homePageUserData bean. */
   @In(value = "homePageUserData", required = false, scope = ScopeType.SESSION)
   private HomePageUserData homePageUserData;

   /** The app user. */
   @In(value = "appUser", required = true, scope = ScopeType.SESSION)
   private AppUser appUser;

   /** The number of results in the search. */
   private int numberOfResults = 0;

   /**
    * Gets the numberOfResults.
    *
    * @return the numberOfResults.
    */
   public final int getNumberOfResults() {
      return this.numberOfResults;
   }

   /** The collection of results. */
   /* @DataModel */
   private List<ClaimListEntity> agedClaimsResults = new ArrayList<ClaimListEntity>();

   /** The pagination. */
   private Page<?> page;
   /** The page controls. */
   private PageControls pageControls;

   /** the constructor. */
   public AgedClaimsList() {
      page = new Page();
      pageControls = new PageControls();
   }

   /**
    * Builds the sql statement and searches the claims.
    */
   public final List<ClaimListEntity> buildClaimsListByType(int startRow, int pageSize, String stations, String privacyRestrictions, String visns) {


      Query q;
      List<?> l;
      Iterator<?> it;
      String orderBy = "";

      /* counts the number of results. */
      this.numberOfResults = 0;

      String sql = " SELECT * "
                 + " FROM (SELECT ROWNUM AS rownum_, ic.* "
                 + " FROM (SELECT c.va_facility_cd, "
                 + " c.claim_index, "
                 + " c.date_created, "
                 + " c.service_type_cd, "
                 + " c.hero_flag, "
                 + " c.begin_service_date, "
                 + " c.claim_status_cd, "
                 + " c.total_billed_charge, "
                 + " c.date_modified, "
                 + " c.patient_name, "
                 + " c.person_id, "
                 + " c.provider_name, "
                 + " c.total_lines, "
                 + " c.lines_closed, "
                 + " c.user_name, "
                 + " c.visn_id_cd, "
                 + " c.date_in_process "
                 + "             FROM fpps_owner.ws_fac_status c "
                 + "             WHERE c.claim_status_cd != 'COMPLETE' "
                 + "             AND c.date_created < SYSDATE - 30 "
                 + "             AND (c.va_facility_cd IN (" + stations + ") ";
                  if (!visns.equalsIgnoreCase("")){
                	  sql+=" OR ( "
                       	 + " c.visn_id_cd IN("+visns+") "
                       	 + " AND " 
                       	 + " (c.va_facility_cd IS NULL OR c.va_facility_cd='') "
                       	 + " ) ";
                  }
                 sql += " ) ";
                 if(!privacyRestrictions.equalsIgnoreCase("")){
                     sql += "   AND c.person_id NOT IN (" + privacyRestrictions + ") ";
                 }









      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimIndex")) {
         orderBy = " ORDER BY c.claim_index " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimType")) {
         orderBy = " ORDER BY c.service_type_cd " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortProjectHero")) {
         orderBy = " ORDER BY c.hero_flag " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortVetName")) {
         orderBy = " ORDER BY c.patient_name " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortVetSsn")) {
         orderBy = " ORDER BY c.person_id " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimDate")) {
         orderBy = " ORDER BY c.begin_service_date " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortDateInProcess")) {
         orderBy = " ORDER BY c.date_in_process " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimAmount")) {
         orderBy = " ORDER BY c.total_billed_charge " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortProvider")) {
         orderBy = " ORDER BY c.provider_name " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortTotalLines")) {
         orderBy = " ORDER BY c.total_lines " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortLinesClosed")) {
         orderBy = " ORDER BY c.lines_closed " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimStatus")) {
         orderBy = " ORDER BY c.claim_status_cd " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortAssignedTo")) {
         orderBy = " ORDER BY c.user_name " + this.sortDir + ", c.va_facility_cd " + this.sortDir;
      }
      if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortFacility")) {
         orderBy = " ORDER BY c.va_facility_cd " + this.sortDir + ", c.user_name " + this.sortDir;
      }

      if (orderBy.indexOf("ORDER BY ") <= 0) {
         orderBy += " ORDER BY c.begin_service_date ASC ";
         this.sortCol = "sortClaimDate";
         this.sortDir = "asc";
      }
      
      //finally, add a claim_index sort to all queries 
      if (!this.sortCol.equalsIgnoreCase("sortClaimIndex")) {
          orderBy += ", c.claim_index " + this.sortDir;
      }
      
      // add order by clauses to sql
      sql += orderBy;

      sql += ") ic) "
          + " WHERE rownum_ BETWEEN " + (startRow + 1) + " AND " + (startRow + pageSize);

      agedClaimsResults.clear();

      q = em.createNativeQuery(sql);

      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      /* loop through the results. */
      while (it.hasNext()) {

         ClaimListEntity cle = new ClaimListEntity();
         Object[] row = (Object[]) it.next();

         cle.setIsSelected(false);
         cle.setFacility((String) row[1]);
         BigDecimal ci = (BigDecimal) row[2];
         cle.setClaimIndex(ci.longValue());
         cle.setClaimType((String) row[4]);
         Character hf = (Character) row[5];
         cle.setProjectHero(hf.charValue());
         cle.setVetName((String) row[10]);
         if(cle.getVetName().trim().equalsIgnoreCase(",")){
             cle.setVetName("");
         }
         cle.setVetSsn((String) row[11]);
         cle.setClaimDate((Date) row[6]);
         cle.setClaimAmount((BigDecimal) row[8]);
         Date dip = null;
         try {
            dip = (Date) row[17];
         } catch (Exception e) {
            log.debug("error in casting date", e);
         }
         cle.setDateInProcess(dip);
         cle.setProviderName((String) row[12]);
         BigDecimal tl = (BigDecimal) row[13];
         cle.setTotalLines(tl.longValue());
         BigDecimal lc = (BigDecimal) row[14];
         cle.setLinesClosed(lc.longValue());
         cle.setClaimStatus((String) row[7]);

         cle.setAssignedTo((String) row[15]);

         /*
          *
          * private String assignedTo;
          */
         agedClaimsResults.add(cle);
      }

      sql = "SELECT count(*) "
         +  "FROM fpps_owner.ws_fac_status c "
         + " WHERE c.claim_status_cd != 'COMPLETE' "
         + " AND c.date_created < SYSDATE - 30 "
         + " AND (c.va_facility_cd IN (" + stations + ") ";
          if (!visns.equalsIgnoreCase("")){
        	  sql+=" OR ( "
               	 + " c.visn_id_cd IN("+visns+") "
               	 + " AND " 
               	 + " (c.va_facility_cd IS NULL OR c.va_facility_cd='') "
               	 + " ) ";
          }
         sql += " ) ";

         if(!privacyRestrictions.equalsIgnoreCase("")){
             sql += "   AND c.person_id NOT IN (" + privacyRestrictions + ") ";
         }




      q = em.createNativeQuery(sql);

      /* query the database. */
      BigDecimal recordCount = (BigDecimal) q.getSingleResult();

      /*
       * set the numberOfResults property and the pageControls numberOfResults.
       */
      this.numberOfResults = recordCount.intValue();
      this.pageControls.setNumberOfResults(numberOfResults);

      return agedClaimsResults;

   }

   /**
    * Page information associated with the Claim Search.
    *
    * @return Page information.
    */
   public final Page<?> getPage() {
      return page;
   }

   /**
    * PageControls information associated with the Claim Search.
    *
    * @return PageControls information.
    */
   public final PageControls getPageControls() {
      return pageControls;
   }

   /**
    * sets the pageControls page number.
    *
    * @param newPage
    *           the newPage int.
    * */
   public final void setNewPage(int newPage) {
      this.pageControls.setPage(newPage);
   }

   /**
    * change the page size of the results data table.
    *
    * @param event
    *           the ValueChangeEvent.
    * */
   public final void changePageSize(ValueChangeEvent event) {
      int newVal = ((Integer) event.getNewValue()).intValue();
      this.pageControls.setPageSize(newVal);
      //FacesContext.getCurrentInstance().renderResponse();
      FacesContext.getCurrentInstance().getApplication().getNavigationHandler().handleNavigation(FacesContext.getCurrentInstance(),null,"RELOAD");
   }

   @Out
   ArrayList<String> printList = new ArrayList<String>();

   public final String print() {
       String ret = "agedPrintFAIL";
       boolean success = false;
       printList.clear();
       try {
          for (ClaimListEntity cle : agedClaimsResults) {
             if (cle.getIsSelected()) {
                String ci = Long.toString(cle.getClaimIndex());
                printList.add(ci);
                this.showPrintPopup = "true";
                ret = "agedPrintSUCCESS";
                success = true;
             }
          }
       } catch (Exception e) {
          log.error(e);
       }

       if (success) {
          StatusMessages.instance().addFromResourceBundle(Severity.INFO, "gov.va.med.fee.MSG_SUCCESS2", "");
       } else {
          StatusMessages.instance().addFromResourceBundle(Severity.ERROR, "gov.va.med.fee.MSG_ERROR2", "");
       }

       return ret;
    }

   @Override
   public final String displayDisapproval() {
      if (claimsToDisapprove != null) {
         claimsToDisapprove.clear();
      } else {
         claimsToDisapprove = new LinkedList<Long>();
      }
      for (ClaimListEntity claim : agedClaimsResults) {
         if (claim.getIsSelected()) {
            claimsToDisapprove.add(claim.getClaimIndex());
         }
      }
      // Navigate to disapproval page form.
      return "disaproveclaims";
   }


   @Override
   public final String displayReroute() {
       if (claimsToReroute != null) {
           claimsToReroute.clear();
       } else {
           claimsToReroute = new ArrayList<Long>();
       }

    for (ClaimListEntity claim : agedClaimsResults) {
         if (claim.getIsSelected()) {
            claimsToReroute.add(claim.getClaimIndex());
         }
      }
      // naviagte to reroute page form.
      return "rerouteClaimsPage";
   }

   public final DataPage<ClaimListEntity> getDataPage(int startRow, int pageSize, String stations, String privacyRestrictions, String visns) {
      // access database here, or call EJB to do so

      List<ClaimListEntity> dp = buildClaimsListByType(startRow, pageSize, stations, privacyRestrictions, visns);
      // List dp = new ArrayList();
      return new DataPage<ClaimListEntity>(numberOfResults, startRow, dp);
   }

   private transient LocalDataModel dataModel;

   public int dataModelCallCounter = 0;

   public String stationsHolder = "";

   public final DataModel getDataModel() {
      dataModelCallCounter++;
      log.debug(" {0} called getDataModel()", dataModelCallCounter);

      String privacyRestrictions = homePageUserData.getPrivacyRestrictions();
      String visns = homePageUserData.getActiveVisns();
      String stations = homePageUserData.getActiveStations().toString();
      stations = stations.replace('[', '\'');
      stations = stations.replace(']', '\'');
      stations = stations.replace(", ", "\', \'");

      if (dataModel == null || !stationsHolder.equalsIgnoreCase(stations)) {
         dataModel = new LocalDataModel(this.pageControls.getPageSize(), stations, privacyRestrictions, visns);
         stationsHolder = stations;
      } else if (dataModel.getPageSize() != this.pageControls.getPageSize()) {
         dataModel.setPageSize(this.pageControls.getPageSize());
      }

      return dataModel;
   }

   @SuppressWarnings("unchecked")
   private class LocalDataModel extends PagedListDataModel {

      String stations;
      String privacyRestrictions;
      String visns;

      public LocalDataModel() {
         super(1);
      }

      public LocalDataModel(int pageSize) {
         super(pageSize);
      }

      public LocalDataModel(int pageSize, String s, String p, String v) {
         super(pageSize);
         stations = s;
         privacyRestrictions = p;
         visns = v;
      }

      private int getPageSize() {
         return super.pageSize;
      }

      private void setPageSize(int pageSize) {
         super.pageSize = pageSize;
      }

      public DataPage<ClaimListEntity> fetchPage(int startRow, int pageSize) {
         // call enclosing managed bean method to fetch the data

         return getDataPage(startRow, pageSize, stations, privacyRestrictions, visns);
      }
   }

   /**
    * {@inheritDoc}
    */
   @Destroy
   @Remove
   public final void destroy() {
      // log.info("destroyed:" + this);
   }

}
