/*
 * $Id: ClaimSearch.java,v 1.10.4.1 2011/10/04 19:20:08 vhahacpennet Exp $
 */
package gov.va.med.fee.claim;

import javax.ejb.Local;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import gov.va.med.domain.fee.Claim;
import gov.va.med.domain.fee.PersonInfo;
import gov.va.med.domain.fee.ProviderInfo;
import gov.va.med.fee.Page;
import gov.va.med.fee.PageControls;
import javax.faces.event.ValueChangeEvent;

/**
 * The Interface ClaimSearch.
 */
@Local
public interface ClaimSearch {

	ClaimPaymentBean getClaimPaymentBean();
	void setClaimPaymentBean(ClaimPaymentBean claimPaymentBean);
    
	// remove after implementation in ClaimPaymentBean
	String getCheckNumberSearch();
	void setCheckNumberSearch(String checkNumberSearch);
	
	
	
    /**
     * Retrieves the result count of the claim search.
     * 
     * @return results count of the claim search.
     */
    int getNumberOfResults();
    
    

    
    /**
     * Retrieves the result of the claim search.
     * 
     * @return results of the claim search.
     */
    List getClaimSearchResults();
    
    /**
     * Retrieves the Claim from the search.
     * 
     * @return the claim searched for.
     */
    Claim getClaim();
    
    /**
     * Retrieves the Claim2 from the search.
     * 
     * @return the claim2 searched for.
     */
    Claim getClaim2();
    
    /**
     * Retrieves the Person associated with the claim.
     * 
     * @return the person information associated with the
     *         claim.
     */
    PersonInfo getPerson();
    
    /**
     * The Provider associated with the claim.
     * 
     * @return the provider associated with the claim.
     * 
     */
    ProviderInfo getProvider();
        
    /**
     * Retrieves the PageControls associated with the claim search.
     * 
     * @return the page controls associated with the claim search.
     */
    PageControls getPageControls();
    
    /**
     * Retrieves the Page associated with the claim search.
     * 
     * @return the page associated with the claim search.
     */
    Page getPage();
    
    /**
     * Retrieves a boolean if a search has been ran or not.
     * 
     * @return the boolean associated with the first search.
     */
    boolean getFirstSearchRan();
    
    /**
     * Retrieves a boolean if a search should return HERO claims only.
     * 
     * @return the boolean associated with the HERO flag.
     */
    boolean getHeroOnly();
    
    /**
     * The hero flag boolean associated with the claim search.
     * 
     * @param heroOnly
     *          the heroOnly flag.
     */
    void setHeroOnly(boolean heroOnly);
    
    /**
     * The no station flag boolean associated with the claim search
     * 
     * @param noStation
     */
    void setNoStation(boolean noStation);
    
    /**
     * Retrieves a boolean if a search should return claims assigned to no Station
     * 
     * @return the boolean associated with noStation flag
     */
    boolean getNoStation();
    
    /**
     * Retrieves the records associated with the claim search.
     */
    void searchClaims();
    
    /**
     * Sets the PageControls page associated with the claim search.
     * 
     * @param newPage
     *          the new page.
     */
    void setNewPage(int newPage);
    
    /**
     * The ValueChangeListener method associated with changing the page size.
     * 
     * @param event
     *          the ValueChangeEvent.
     */
    void changePageSize(ValueChangeEvent event);
    
    /**
     * Sets the claimId string associated with the claim search.
     * 
     * @param claimId
     *          the claimId.
     */
    void setClaimId(String claimId);
    
    /**
     * Gets the claimId string associated with the claim search.
     * 
     * @return the claimId string associated with the claim search.
     */
    String getClaimId();
    
    /**
     * Gets the sort column associated with the claim search.
     * 
     * @return the sort column associated with the claim search.
     */
    String getSortCol();
    
    /**
     * Sets the sort column associated with the claim search.
     * 
     * @param sortCol
     *             the sort column.
     */
    void setSortCol(String sortCol);
    
    /**
     * Gets the sort direction associated with the claim search.
     * 
     * @return the sort direction associated with the claim search.
     */
    String getSortDir();
    
    /**
     * Sets the sort direction associated with the claim search.
     * @param sortDir
     *            the sort direction.
     */
    void setSortDir(String sortDir);
    
    
    /**
     * prints selected claims.
     * 
     * @return string result.
     */
    String print();
    
    
    
    
    void checkNewFacilities();
    
    String displayReroute();

    String displayDisapproval();
    
    /**
    * Destroy.
    */
    void destroy();   
}
