package gov.va.med.fee.claim;

import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.Claim;
import gov.va.med.domain.fee.PersonInfo;
import gov.va.med.domain.fee.ProviderInfo;
import gov.va.med.domain.fee.ServiceType;
import gov.va.med.domain.fee.VaFacility;
import gov.va.med.fee.Page;
import gov.va.med.fee.PageControls;
import gov.va.med.fee.sec.HomePageUserData;

import org.jboss.seam.ScopeType;
import static org.jboss.seam.ScopeType.SESSION;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.datamodel.DataModel;
import org.jboss.seam.annotations.web.RequestParameter;
import org.jboss.seam.log.Log;

import java.math.BigDecimal;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateful;

import javax.faces.context.FacesContext;
import javax.faces.event.ValueChangeEvent;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;



/**
 * ClaimSearchBean class.
 * @author Bill Blackmon
 * @version R1.25.1
 * HAPE POR VA DO# 118-11-D-1009, TO#118-1009-0001; HARRIS
 */
@SuppressWarnings("unused")
@Stateful
@Name("claimSearch")
@Scope(SESSION)
public class ClaimSearchBean implements ClaimSearch {
    /** The log. */
    @Logger
    private Log log;

    /** The em. */
    @PersistenceContext(unitName = "FPPS_Owner")
    private EntityManager em;
    @RequestParameter
    private String sd;
    @RequestParameter
    private String sc;

    /** the contained ClaimPaymentBean */
    private ClaimPaymentBean claimPaymentBean;

    /** The column to sort by. */
    private String sortCol;

    /** The direction to sort. */
    private String sortDir;

    /** The homePageUserData bean. */
    @In(value = "homePageUserData", required = false, scope = ScopeType.SESSION)
    private HomePageUserData homePageUserData;

    /** The app user. */
    @In(value = "appUser", required = true, scope = ScopeType.SESSION)
    private AppUser appUser;
    @In(scope = ScopeType.SESSION, required = false)
    @Out(scope = ScopeType.SESSION, required = false)
    private List<Long> claimsToReroute;
    @In(scope = ScopeType.SESSION, required = false)
    @Out(scope = ScopeType.SESSION, required = false)
    List<Long> claimsToDisapprove = new ArrayList<Long>();

    /** The number of results in the search. */
    private int numberOfResults = 0;

    /** The SQL query for searching claims. */
    private String sql;

    /** The collection of results. */
    @DataModel
    private List<ClaimSearchEntity> results = new ArrayList<ClaimSearchEntity>();

    /** the Claim instance. */
    private Claim claim;

    /**
     * the Claim2 instance -- used to hold secondary claim data i.e. the
     * "end service date" of the claim search parameters
     */
    private Claim claim2;

    /** the PersonInfo instance. */
    private PersonInfo person;

    /** the ProviderInfo instance. */
    private ProviderInfo provider;

    /** The pagination. */
    private Page<?> page;

    /** The page controls. */
    private PageControls pageControls;
    @Out
    ArrayList<String> printList = new ArrayList<String>();

    /**
     * A boolean to hold the value from the search page. Can't map directly to
     * the hibernate object because the HERO_FLAG is not boolean, but char
     * */
    private boolean heroOnly;

    /**
     * A boolean to hold the value from the search page representing claims that are
     * not assigned to a station
     */
    private boolean noStation = false;

    /**
     * A string to hold the value from the search page. Can't map directly to the
     * hibernate object because CLAIM_INDEX is a long. A long type will show a
     * zero in the textbox on the search page.
     */
    private String claimId;

    /**
     * Used to determine if a search has been ran. If true, then show the search
     * results.
     */
    private boolean firstSearchRan = false;
    private HashMap<String, String> searchParametersToSet = new HashMap<String, String>();
    private int classCounter = 0;
    private String facilitiesHolder = "";


    /** Search string passed from check number screen entry */
    private String checkNumberSearch = "";




    /** the constructor. */
    @SuppressWarnings("unchecked")

    // NOTE: called after authentication and before screen display
    public ClaimSearchBean() {
        claim = new Claim();
        claim2 = new Claim();
        person = new PersonInfo();
        provider = new ProviderInfo();
        page = new Page();
        pageControls = new PageControls();
        heroOnly = false;
        claimId = "";

        ServiceType st = new ServiceType();
        claim.setServiceType(st);

        checkNumberSearch = "";


    }





    /**
	 * @return the claimPaymentBean
	 */
	public final ClaimPaymentBean getClaimPaymentBean() {
		return claimPaymentBean;
	}





	/**
	 * @param claimPaymentBean the claimPaymentBean to set
	 */
	public final void setClaimPaymentBean(ClaimPaymentBean claimPaymentBean) {
		this.claimPaymentBean = claimPaymentBean;
	}





	public final String getSd() {
        return this.sd;
    }

    public final void setSd(String sd) {
        this.sd = sd;
    }

    public final String getSc() {
        return this.sc;
    }

    public final void setSc(String sc) {
        this.sc = sc;
    }

    /**
     * Gets the sort column.
     *
     * @return the sortCol.
     */
    public final String getSortCol() {
        return this.sortCol;
    }

    /**
     * Sets the sort column.
     *
     * @param sortCol
     *           the sort column.
     */
    public final void setSortCol(String sortCol) {
        this.sortCol = sortCol;
    }

    /**
     * Gets the sort direction.
     *
     * @return the sortDir.
     */
    public final String getSortDir() {
        return this.sortDir;
    }

    /**
     * Sets the sort direction.
     *
     * @param sortDir
     *           the sort direction.
     */
    public final void setSortDir(String sortDir) {
        this.sortDir = sortDir;
    }

    /**
     * Gets the numberOfResults.
     *
     * @return the numberOfResults.
     */
    public final int getNumberOfResults() {
        return this.numberOfResults;
    }

    /**
     * Sets the heroOnly.
     *
     * @param heroOnly
     *           the heroOnly flag.
     */
    public final void setHeroOnly(boolean heroOnly) {
        this.heroOnly = heroOnly;
    }

    /**
     * Gets the heroOnly flag.
     *
     * @return the heroOnly.
     */
    public final boolean getHeroOnly() {
        return this.heroOnly;
    }

    /**
     * The no station flag boolean associated with the claim search
     *
     * @param noStation
     */
    public final void setNoStation(boolean noStation) {
        this.noStation = noStation;
    }

    /**
     * Retrieves a boolean if a search should return claims assigned to no Station
     *
     * @return the boolean associated with noStation flag
     */
    public final boolean getNoStation() {
        return this.noStation;
    }

    public final String getCheckNumberSearch() {
		return checkNumberSearch;
	}

	public final void setCheckNumberSearch(String checkNumberSearch) {

		this.checkNumberSearch = checkNumberSearch;
	}

	/**
     * Sets the claimId string.
     *
     * @param claimId
     *           the claimId string.
     */
    public final void setClaimId(String claimId) {
        this.claimId = claimId;
    }

    /**
     * Gets the claimId String.
     *
     * @return the claimId.
     */
    public final String getClaimId() {
        return claimId;
    }

    /**
     * Gets the search results List object.
     *
     * @return the results.
     */
    public final List<ClaimSearchEntity> getClaimSearchResults() {
        return results;
    }

    /**
     * Gets the Claim.
     *
     * @return the Claim.
     */
    public final Claim getClaim() {
        return claim;
    }

    /**
     * Gets the Claim 2.
     *
     * @return the Claim.
     */
    public final Claim getClaim2() {
        return claim2;
    }

    /**
     * Gets the Person associated with the Claim.
     *
     * @return the Person.
     */
    public final PersonInfo getPerson() {
        return person;
    }

    /**
     * Gets the Provider associated with the Claim.
     *
     * @return the Provider.
     */
    public final ProviderInfo getProvider() {
        return provider;
    }

    /**
     * Page information associated with the Claim Search.
     *
     * @return Page information.
     */
    public final Page<?> getPage() {
        return page;
    }

    /**
     * PageControls information associated with the Claim Search.
     *
     * @return PageControls information.
     */
    public final PageControls getPageControls() {
        return pageControls;
    }

    /**
     * Sets the firstSearchRan.
     *
     * @param firstSearchRan
     *           the firstSearchRan flag.
     */
    public final void setFirstSearchRan(boolean firstSearchRan) {
        this.firstSearchRan = firstSearchRan;
    }

    /**
     * Gets the firstSearchRan.
     *
     * @return the firstSearchRan.
     */
    public final boolean getFirstSearchRan() {
        return this.firstSearchRan;
    }

    /**
     * A method to build narrowing clauses in the SQL statement. Determines if a
     * database string field is set by the user. If so, append it appropriately
     * using 'LIKE' or '=' depending on ' whether or not there is an Oracle
     * wildcard in the user defined criteria.
     *
     * @return a SQL clause
     * @param searchParam
     *           the search parameter to evaluate.
     * @param sqlColumn
     *           the database column to apply the search parameters to.
     *
     */
    private String appendSqlStringParameters(String searchParam, String prefix,
        String suffix) {
        String ret = "";
        String param = "p" + classCounter++;
        log.debug("searchParam: {0} ", searchParam);

        if ((searchParam != null) && !searchParam.trim().equals("") &&
                !searchParam.trim().equals("%") &&
                !searchParam.trim().equals("_")) {
            if ((searchParam.trim().indexOf("%") >= 0) ||
                    (searchParam.trim().indexOf("_") >= 0)) {
                ret += (" " + prefix + " LIKE :" + param + " " + suffix);
                searchParametersToSet.put(param, searchParam.toUpperCase());
            } else {
                ret += (" " + prefix + " = :" + param + " " + suffix);
                searchParametersToSet.put(param, searchParam.toUpperCase());
            }
        }

// System.out.println("ClaimSearchBean.appendSqlStringParameters() returning: " + ret);

        log.debug("ret: {0} ", ret);

        return ret;
    }

    /**
     * Builds the sql statement and searches the claims.

     */
    public final void searchClaims() {


        Query q;
        List<?> l;
        Iterator<?> it;

        searchParametersToSet = new HashMap<String, String>();

        if ((this.sc != null) && !this.sc.equalsIgnoreCase("")) {
            this.sortCol = this.sc;
        }

        if ((this.sd != null) && !this.sd.equalsIgnoreCase("")) {
            this.sortDir = this.sd;
        }

        /* counts the number of results. */
        this.numberOfResults = 0;

        /* set the firstSearchRan flag */
        this.firstSearchRan = true;

        /*
         * build the base SQL statement. This is used in both returning records
         * and counting the total number of records.
         */
        String privacyRestrictions = homePageUserData.getPrivacyRestrictions();
        String visns;

        //noStation can only be true if user is ADMIN
        if (noStation) { //if VISN only checked and admin: call inclusive sql
            visns = homePageUserData.getVisnSelector(); //va_fac_cd is null
        } else { //otherwise call exclusive sql (NOT CHECKED) if ADMIN returns empty string
            visns = homePageUserData.getVisnSelectorNotNull(); //visn_cd in 1,2..n
                                                               //}
        }

        String activeUserStations;

        if (noStation) { //include non-active stations
            visns = homePageUserData.getVisnSelector(); //va_fac_cd is null
            activeUserStations = homePageUserData.getAllFacilities();
        } else {
            visns = homePageUserData.getVisnSelectorNotNull();
            activeUserStations = homePageUserData.getActiveStations().toString();
        }

        log.debug(activeUserStations);
        activeUserStations = activeUserStations.replace('[', '\'');
        activeUserStations = activeUserStations.replace(']', '\'');
        activeUserStations = activeUserStations.replace(", ", "\', \'");
        log.debug(activeUserStations);

        facilitiesHolder = activeUserStations;


        sql = "SELECT c.claim_index, " + "        	c.service_type_cd, " +
            "        c.hero_flag, " + "        		c.begin_service_date, " +
            "        pi.last_name, " + "        	pi.first_name, " +
            "        pi.middle_name, " + "       	pi.name_suffix, " +
            "        pi.person_id, " +
            "        fpps_owner.get_provider_search (c.claim_index, 1) AS provider_name, " +
            "        fpps_owner.get_provider_search (c.claim_index, 2) AS npi, " +
            "        fpps_owner.get_provider_search (c.claim_index, 3) AS provider_tin, " +
            "        c.user_name, " + "        		c.claim_status_cd, " +
            "        c.patient_control_number, " + "        c.va_facility_cd " +
            "   FROM fpps_owner.claims c, fpps_owner.person_info pi " +
            "   WHERE c.claim_index = pi.claim_index(+) " +
            "   AND (c.va_facility_cd IN (" + activeUserStations + ") ";
        sql += visns;
        sql += " ) ";

        if (!privacyRestrictions.equalsIgnoreCase("")) {
            sql += ("   AND pi.person_id NOT IN (" + privacyRestrictions +
            ") ");
        }



        /* Append the user determined search strings to the SQL statement */
        sql += appendSqlStringParameters(this.claim.getServiceType()
                                                   .getServiceTypeCd(),
            "AND c.service_type_cd", "");


        sql += appendSqlStringParameters(this.claim.getClaimStatusCd(),
            "AND c.claim_status_cd", "");



        sql += appendSqlStringParameters(this.claimId, "AND c.claim_index", "");

        sql += appendSqlStringParameters(this.claim.getPatientControlNumber(),
            "AND c.claim_index IN (SELECT ci.claim_index FROM claims ci WHERE ci.patient_control_number",
            ")");



        sql += appendSqlStringParameters(this.person.getPersonId(),
            "AND c.claim_index IN (SELECT pii.claim_index FROM fpps_owner.person_info pii WHERE pii.person_id",
            ")");


        sql += appendSqlStringParameters(this.person.getLastName(),
            "AND c.claim_index IN (SELECT piii.claim_index FROM fpps_owner.person_info piii WHERE piii.last_name",
            ")");







        /********************************************************************
         * provider search criteria
         *******************************************************************/
        if (((this.provider.getLastName() != null) &&
                !this.provider.getLastName().trim().equalsIgnoreCase("") &&
                !this.provider.getLastName().trim().equalsIgnoreCase("%") &&
                !this.provider.getLastName().trim().equalsIgnoreCase("_")) ||
                ((this.provider.getProviderNpi() != null) &&
                !this.provider.getProviderNpi().trim().equalsIgnoreCase("") &&
                !this.provider.getProviderNpi().trim().equalsIgnoreCase("%") &&
                !this.provider.getProviderNpi().trim().equalsIgnoreCase("_")) ||
                ((this.provider.getLastName() != null) &&
                !this.provider.getProviderTin().trim().equalsIgnoreCase("") &&
                !this.provider.getProviderTin().trim().equalsIgnoreCase("%") &&
                !this.provider.getProviderTin().trim().equalsIgnoreCase("_"))) {
        	sql += " and exists ( "
            		+  " select ws.claim_index "
            		+  " from fpps_owner.provider_info ws "
            		+  " where ws.claim_index = c.claim_index "
            	;

                sql += appendSqlStringParameters(this.provider.getLastName(), " AND ws.provider_name ", "");
                sql += appendSqlStringParameters(this.provider.getProviderTin(), " AND ws.provider_tin ", "");
                sql += appendSqlStringParameters(this.provider.getProviderNpi(), " AND ws.provider_npi ", "");

            sql += " ) ";

  // System.out.println("sql after provider info: " + sql);




        }

        /*******************************************************************/


        /**
         * Get check number here
         */

// IMPLEMENT WILDCARD SEARCH SOMEHOW!
        // check for numeric entries....implement wildcard search
        if( (this.getCheckNumberSearch() != null) && (!this.getCheckNumberSearch().trim().isEmpty())) {


// HERE
/** original         	
  sql += appendSqlStringParameters(this.getCheckNumberSearch(),
            "and c.claim_index in (select claim_index from fee_claim_payments fcp, fee_claim_payment_lines fcpl " +
            		"where fcp.fee_claim_payment_id = fcpl.fee_claim_payment_id  and fcpl.CHECK_NUMBER ", ")");
**/            		
  

    sql += appendSqlStringParameters(this.getCheckNumberSearch(),
            "and c.claim_index in (select claim_index from claim_payment cpay where payment_tracking_number ", ")");

  
// System.out.println("ClaimSearchBean.searchClaims() - sql after check number search appendage is: " + sql);

        }



        /* Append the user determined heroFlag boolean to the SQL statement */
        if (heroOnly) {
            sql += " AND c.hero_flag = 'Y' ";
        }

        /* Append the lower claim date range to the SQL statement */
        if (this.claim.getBeginServiceDate() != null) {
            sql += (" AND c.begin_service_date >= TO_DATE('" +
            String.format("%1$tm/%1$td/%1$tY", this.claim.getBeginServiceDate()) +
            "', 'MM/dd/yyyy')");
        }

        /* Append the upper claim date range to the SQL statement */
        if (this.claim2.getBeginServiceDate() != null) {
            sql += (" AND c.begin_service_date <= TO_DATE('" +
            String.format("%1$tm/%1$td/%1$tY", this.claim2.getBeginServiceDate()) +
            "', 'MM/dd/yyyy')");
        }

        /* Build the SQL ORDER BY clause. */
        String orderBy = "";

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortClaimIndex")) {
            orderBy = " ORDER BY c.claim_index " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortClaimType")) {
            orderBy = " ORDER BY c.service_type_cd " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortClaimStatus")) {
            orderBy = " ORDER BY c.claim_status_cd " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortClaimHERO")) {
            orderBy = " ORDER BY c.hero_flag " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortPersonName")) {
            orderBy = " ORDER BY pi.last_name " + this.sortDir +
                ", pi.first_name " + this.sortDir + ", pi.middle_name " +
                this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortPersonSSN")) {
            orderBy = " ORDER BY pi.person_id " + this.sortDir;
        }

        if ((this.sortCol != null) && this.sortCol.equalsIgnoreCase("sortPcn")) {
            orderBy = " ORDER BY c.patient_control_number " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortClaimDate")) {
            orderBy = " ORDER BY c.begin_service_date " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortProviderName")) {
            orderBy = " ORDER BY provider_name " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortProviderTin")) {
            orderBy = " ORDER BY provider_tin " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortProviderNpi")) {
            orderBy = " ORDER BY npi " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortAssignedTo")) {
            orderBy = " ORDER BY c.user_name " + this.sortDir +
                ", c.va_facility_cd " + this.sortDir;
        }

        if ((this.sortCol != null) &&
                this.sortCol.equalsIgnoreCase("sortFacility")) {
            orderBy = " ORDER BY c.va_facility_cd " + this.sortDir +
                ", c.user_name " + this.sortDir;
        }

        // add default columns to ORDER BY if they haven't been specified in the
        // clause already
        if (orderBy.indexOf("ORDER BY ") > 0) {
            orderBy += ((this.sortCol.equalsIgnoreCase("sortClaimStatus"))
            ? " " : ", c.claim_status_cd ASC ");
        }

        if (orderBy.indexOf("ORDER BY ") > 0) {
            orderBy += ((this.sortCol.equalsIgnoreCase("sortClaimDate")) ? " "
                                                                         : ", c.begin_service_date ASC ");
        }

        if (orderBy.indexOf("ORDER BY ") <= 0) {
            orderBy += " ORDER BY c.claim_status_cd ASC, c.begin_service_date ASC ";
            this.sortCol = "sortClaimStatus";
            this.sortDir = "asc";
        }

        // limit the number of rows to return and add order by clauses to sql
        sql += (" AND ROWNUM < 101 " + orderBy);

        /* clear out any previous search results. */
        results.clear();

        /*
         * load the query object with the query
         */
        q = em.createNativeQuery(sql);

        for (String key : searchParametersToSet.keySet()) {
            q.setParameter(key, searchParametersToSet.get(key));
        }

        /* query the database and put the records into a temporary list. */
        l = q.getResultList();
        it = l.iterator();

        /* loop through the results. */
        while (it.hasNext()) {
            /* build the ClaimSearchEntity object from the SQL results. */
            Object[] row = (Object[]) it.next();

            Claim c = new Claim();
            ServiceType claimServiceType = new ServiceType();
            c.setServiceType(claimServiceType);

            VaFacility claimVaFacility = new VaFacility();
            c.setVaFacility(claimVaFacility);

            PersonInfo p = new PersonInfo();
            ProviderInfo pr = new ProviderInfo();

            BigDecimal ci = (BigDecimal) row[0];
            c.setClaimIndex(ci.longValue());

            String st = (String) row[1];
            c.getServiceType().setServiceTypeCd(st);

            Character hf = (Character) row[2];
            c.setHeroFlag(hf.charValue());
            c.setBeginServiceDate((Date) row[3]);
            p.setLastName((String) row[4]);
            p.setFirstName((String) row[5]);
            p.setMiddleName((String) row[6]);
            p.setNameSuffix((String) row[7]);
            p.setPersonId((String) row[8]);
            pr.setLastName((String) row[9]);

            if (pr.getLastName() != null) {
                pr.setLastName(pr.getLastName().replace(" ", "&nbsp;"));
            }

            pr.setProviderNpi((String) row[10]);
            pr.setProviderTin((String) row[11]);
            c.setUserName((String) row[12]);
            c.setClaimStatusCd((String) row[13]);
            c.setPatientControlNumber((String) row[14]);

            String fac = (String) row[15];
            c.getVaFacility().setVaFacilityCd(fac);

            ClaimSearchEntity cse = new ClaimSearchEntity(false, c, p, pr);

            /* add the ClaimSearchEntity to the results collection. */
            results.add(cse);

            /* increment the numberOfResults counter. */
            numberOfResults++;
        }

        /*
         * set the numberOfResults property and the pageControls numberOfResults.
         */
        numberOfResults = results.size();
        this.pageControls.setNumberOfResults(numberOfResults);

        log.debug("TIMESTAMP 2: {0} ", System.currentTimeMillis());
    }

    /**
     * sets the pageControls page number.
     *
     * @param newPage
     *           the newPage int.
     * */
    public final void setNewPage(int newPage) {
        this.pageControls.setPage(newPage);
        searchClaims();
    }

    /**
     * change the page size of the results data table.
     *
     * @param event
     *           the ValueChangeEvent.
     * */
    public final void changePageSize(ValueChangeEvent event) {
        int newVal = ((Integer) event.getNewValue()).intValue();

        this.pageControls.setPage(1);
        this.pageControls.setPageSize(newVal);

        FacesContext.getCurrentInstance().renderResponse();
    }

    public final String print() {
        boolean success = false;
        String ret = "searchClaimsPrintFAIL";
        printList.clear();

        try {
            for (ClaimSearchEntity cse : results) {
                if (cse.getIsSelected()) {
                    String ci = Long.toString(cse.getClaim().getClaimIndex());
                    printList.add(ci);
                    ret = "searchClaimsPrintSUCCESS";
                    success = true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return ret;
    }

    @Override
    public final String displayReroute() {
        if (claimsToReroute != null) {
            claimsToReroute.clear();
        } else {
            claimsToReroute = new ArrayList<Long>();
        }

        for (ClaimSearchEntity cse : results) {
            if (cse.getIsSelected()) {
                claimsToReroute.add(cse.getClaim().getClaimIndex());
            }
        }

        // Navigate to reroute page form.
        return "rerouteClaimsPage";
    }

    @Override
    public final String displayDisapproval() {
        if (claimsToDisapprove != null) {
            claimsToDisapprove.clear();
        } else {
            claimsToDisapprove = new ArrayList<Long>();
        }

        for (ClaimSearchEntity cse : results) {
            if (cse.getIsSelected()) {
                claimsToDisapprove.add(cse.getClaim().getClaimIndex());
            }
        }

        // Navigate to disapproval page form.
        return "disaproveclaims";
    }

    public final void checkNewFacilities() {
        String activeUserStations = homePageUserData.getActiveStations()
                                                    .toString();
        log.debug(activeUserStations);
        activeUserStations = activeUserStations.replace('[', '\'');
        activeUserStations = activeUserStations.replace(']', '\'');
        activeUserStations = activeUserStations.replace(", ", "\', \'");
        log.debug(activeUserStations);

        if (!facilitiesHolder.equalsIgnoreCase(activeUserStations)) {
            firstSearchRan = false;
            results.clear();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Destroy
    @Remove
    public final void destroy() {
        // log.info("destroyed:" + this);
    }
}
