/*
 * $Id: DiagnosisCodesEntity.java,v 1.1.4.1 2011/01/18 19:13:49 vhahacpennet Exp $
 */
package gov.va.med.fee.claim;



/**
 * The class DiagnosisCodesEntity class holds data about the 
 * diagnosis codes of a PROF or INST claim.
 */
@SuppressWarnings("serial")
public class DiagnosisCodesEntity {

    private int sequenceNumber;
    private String diagnosisCode;
    private String diagnosisCodeQualifier;
    private String poaIndicator;
    private String diagnosisDescription;
    
  
    

    /**
     * Creates a new instance of DiagnosisCodesEntity.
     */
    public DiagnosisCodesEntity() {
     
    }

    /**
     * Constructs a <code>DiagnosisCodesEntity</code> given parameters
     * <code>sequenceNumber</code>
     * <code>diagnosisCode</code>
     * <code>poaIndicator</code>
     * <code>diagnosisDescription</code> 
     * .
     * 
     * @param sequenceNumber
     *            The int sequence number associated with the diag code
     * @param diagnosisCode
     *            The String diagnosis code associated with the diag code
     * @param diagnosisCode
     *            The String qualifier associated with the diag code           
     * @param poaIndicator
     *            The String POA Indicator associated with the diag code
     * @param diagnosisDescription
     *            The String diagnosis description associated with the diag code  
     */
    
    public DiagnosisCodesEntity(int sequenceNumber, String diagnosisCode,
			String diagnosisCodeQualifier, String poaIndicator,
			String diagnosisDescription) {
		super();
		this.sequenceNumber = sequenceNumber;
		this.diagnosisCode = diagnosisCode;
		this.diagnosisCodeQualifier = diagnosisCodeQualifier;
		this.poaIndicator = poaIndicator;
		this.diagnosisDescription = diagnosisDescription;
	}

	/**
     * Gets the sequence number.
     * 
     * @return sequenceNumber.
     */
    public final int getSequenceNumber() {
        return this.sequenceNumber;
    }

    /**
     * Sets the sequenceNumber.
     * 
     * @param sequenceNumber
     *          the sequenceNumber to set.
     */
    public final void setSequenceNumber(int sequenceNumber) {
        this.sequenceNumber = sequenceNumber;
    }
    
    /**
     * Gets the diagnosis Code.
     * 
     * @return diagnosisCode.
     */
    public final String getDiagnosisCode() {
        return this.diagnosisCode;
    }

    /**
     * Sets the diagnosisCode.
     * 
     * @param diagnosisCode
     *          the diagnosisCode to set.
     */
    public final void setDiagnosisCode(String diagnosisCode) {
        this.diagnosisCode = diagnosisCode;
    }


    /**
     * Gets the diagnosisCodeQualifier.
     * At the very least this is used on the diagnosis code tab
     * to tell if the code is admitting, other, reason for visit etc.
     */    
    public String getDiagnosisCodeQualifier() {
		return diagnosisCodeQualifier;
	}


    /**
     * Sets the diagnosisCodeQualifier.
     * 
     * @param diagnosisCodeQualifier
     *          the diagnosisCodeQualifier to set.
     */
	public void setDiagnosisCodeQualifier(String diagnosisCodeQualifier) {
		this.diagnosisCodeQualifier = diagnosisCodeQualifier;
	}

	/**
     * Gets the POA Indicator Code.
     * 
     * @return poaIndicator.
     */
    public final String getPoaIndicator() {
        return this.poaIndicator;
    }

    /**
     * Sets the POA Indicator.
     * 
     * @param poaIndicator
     *          the poaIndicator to set.
     */
    public final void setPoaIndicator(String poaIndicator) {
        this.poaIndicator = poaIndicator;
    }
  
    /**
     * Gets the diagnosis description.
     * 
     * @return diagnosisDescription.
     */
    public final String getDiagnosisDescription() {
        return this.diagnosisDescription;
    }

    /**
     * Sets the Diagnosis description.
     * 
     * @param diagnosisDescription
     *          the diagnosisDescription to set.
     */
    public final void setDiagnosisDescription(String diagnosisDescription) {
        this.diagnosisDescription = diagnosisDescription;
    }
}
