package gov.va.med.fee.claim;

import gov.va.med.domain.fee.AdjustmentReason;
import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.Claim;
import gov.va.med.domain.fee.ClaimQueue;
import gov.va.med.domain.fee.PersonInfo;
import gov.va.med.domain.fee.ProviderInfo;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;

/**
 *
 */
@Stateless
@Name("disapproveClaims")
public class DisapproveClaimsSessionBean implements DisapproveClaims {
   
   @Logger
   Log log;
   
   /** The em. */
   @PersistenceContext(unitName = "FPPS_Owner")
   private EntityManager em;
   
   @In
   List<Long> claimsToDisapprove;
   
   @Out
   DisapprovalFormTo disapprovalFormTo = new DisapprovalFormTo();
   
   @Out(required = false, scope = ScopeType.EVENT)
   List<DisapprovedListEntity> disapprovals;
   
   @Out(required = false, scope = ScopeType.PAGE)
   List<String> adjustmentReasons;
   
   /** The app user. */
   @In(value = "appUser", required = true, scope = ScopeType.SESSION)
   private AppUser appUser;

   List<Long> reconciled = new LinkedList<Long>();
   
   String reconStr = "";

   /**
    * {@inheritDoc}
    */
   @Factory("disapprovals")
   @Override
   public final void populateDisaprovals() {
      if (disapprovals == null) {
         disapprovals = new ArrayList<DisapprovedListEntity>();
      } else {
         disapprovals.clear();
      }
      this.reconciled.clear();
      String qstr = "select c from fClaim c where c.claimIndex = :index ";
      Query q = em.createQuery(qstr);
      for (Long index : claimsToDisapprove) {
         q.setParameter("index", index);
         try {
            if (!isReconciled(index)) {
               Claim claim = (Claim) q.getSingleResult();
               DisapprovedListEntity de = new DisapprovedListEntity();
               de.setClaimId(index.toString());
               de.setVeteranName(getVeteranName(claim.getPersonInfos()));
               de.setBillingProvider(getBillingProvName(claim.getProviderInfos()));
               de.setServiceProvider(getServiceProvName(claim.getProviderInfos()));
               disapprovals.add(de);
            } else {
               reconciled.add(index);
            }
         } catch (NoResultException e) {
            log.error("error:", e);
            log.error("unable to find claim with index {0} for disaproval", index);
         }
      }
      
      if (this.reconciled.size() > 0) {
         String msg = null;
         for(Long i : this.reconciled) {
            msg = msg == null ? i.toString() : (msg + ", " + i.toString());
         }
         this.reconStr = "The following "
            + (this.reconciled.size() > 1 ? "claims" : "claim") 
            + " cannot be disapproved because "
            + (this.reconciled.size() > 1? "they are " : "it is") 
            + " completed, partially reconciled, or rejected: " + msg;
      }
   }
   
   @SuppressWarnings("unchecked")
   @Override
   @Factory("adjustmentReasons")
   public final void populateAdjustmentReasons() {
      if (adjustmentReasons == null) {
         adjustmentReasons = new LinkedList<String>();
         Query q = em.createQuery("select a from fAdjustmentReason a where a.activeFlag='Y'");
         List<AdjustmentReason> ar = q.getResultList();
         List<String> tmp = new LinkedList<String>();
         for (AdjustmentReason reason : ar) {
            //String item = String.format("%-3s", reason.getAdjustmentReasonCd()) + " | " + reason.getDescription();
            if (reason.getAdjustmentReasonCd().matches("^[A-Z].*")) {
               tmp.add(String.format("%-3s", reason.getAdjustmentReasonCd()) + " - " + reason.getDescription());
            } else {
               adjustmentReasons.add(String.format("%3s", reason.getAdjustmentReasonCd()) + " - " + reason.getDescription());
            } 
         }
         Collections.sort(adjustmentReasons);
         Collections.sort(tmp);
         adjustmentReasons.addAll(tmp);
         adjustmentReasons.add(0, "-- Select a Reason --");
      }
   }
   
   
   public final String getReconciled() {
         this.reconStr = "";
         populateDisaprovals();
         return this.reconStr;
   }
   
   final boolean isReconciled(Long claimIndex) {
      Query q = em.createQuery("select clm from fClaim clm where clm.claimIndex = :index");
      q.setParameter("index", claimIndex);
      Claim claim = (Claim) q.getSingleResult();
      String status = claim.getClaimStatusCd();
      if ("ESTABLISH".equalsIgnoreCase(status)) {
         return false;
      } else if ( "INPROCESS".equalsIgnoreCase(status)) {
         Query q1 = em.createNativeQuery("select count(*) from claim_proc_recon cpr where cpr.claim_index = '" + claimIndex + "' ");
         BigDecimal count = (BigDecimal) q1.getSingleResult();
         if (count.intValue() > 0) {
            return true;
         } else {
            return false;
         }
      } else {
         return true;
      }
   }
   
   final String getServiceProvName(Set<ProviderInfo> providers) {
      for (ProviderInfo p : providers) {
         if ("SERVICE FACILITY".equals(p.getProviderType().getProviderTypeCd())
               || "SERVICE LOCATION".equals(p.getProviderType().getProviderTypeCd())) {
            return p.getLastName();
         }
      }
      return null;    
   }
   
   final String getBillingProvName(Set<ProviderInfo> providers) {
      for (ProviderInfo p : providers) {
         if ("BILLING PROVIDER".equals(p.getProviderType().getProviderTypeCd())) {
            return p.getLastName();
         }
      }
      return null;
   }
   
   final String getVeteranName(Set<PersonInfo> persons) {
      for (PersonInfo pi : persons) {
         if ("PATIENT".equals(pi.getPersonType().getPersonTypeCd())) {
            return pi.getFirstName() + " " + pi.getLastName();
         }
      }
      return null;
   }
    
   public final String dispalyDisaprovalPage(List<String> claimIds) {
     return "disaproveClaims";
   }

   @Override
   @Transactional
   public final String disapprove(String reason, String comment) {
      boolean truncatedComment = false;
      String user = appUser.getUserName().toUpperCase();
      try {
         String claimMsg = "";
         for (DisapprovedListEntity de : disapprovals) {
            Long index = Long.parseLong(de.getClaimId());
            log.debug("disapproving claim {0} ", index);

            // create the claim queue record
            ClaimQueue cq = new ClaimQueue();
            cq.setReason(getAdjReasonCd(reason));
            cq.setClaimQueueTypeCd("REJECTION_APPROVAL");
            cq.setCreatedBy(user);
            java.util.Date today = new java.util.Date();
            java.sql.Date sqldate = new java.sql.Date(today.getTime());
            cq.setDateCreated(sqldate);
            cq.setEnqueueDate(sqldate);
            if (comment != null && comment.length() > 250) {
               comment = comment.substring(0, 250);
               truncatedComment = true;
            }
            cq.setEnqueueComments(comment);

            // update the claim status.
            Claim clm = (Claim) em.createQuery("select c from fClaim c where c.claimIndex = :index")
                                  .setParameter("index", index)
                                  .getSingleResult();
            clm.setClaimStatusCd("REJECTED");
            clm.getClaimQueues().add(cq);
            clm.setModifiedBy(user);
            clm.setDateModified(sqldate);
                
            cq.setClaim(clm);

            // update ws
            em.createQuery("update fWsFacStatus ws set ws.claimStatusCd = 'REJECTED' where ws.claimIndex = :index")
              .setParameter("index", index)
              .executeUpdate();

            claimMsg = claimMsg.equals("") ? Long.toString(index) : claimMsg + ", " + Long.toString(index);

         }
         disapprovals.clear();
         claimsToDisapprove.clear();
         this.disapprovalFormTo.setComment("");
         this.disapprovalFormTo.setSelectedReason("-- Select a Reason --");
         StatusMessages.instance().addFromResourceBundle(Severity.INFO, "gov.va.med.fee.MSG_SUCCESS5", claimMsg);
         if (truncatedComment) {
            StatusMessages.instance().addFromResourceBundle(Severity.WARN, "gov.va.med.fee.claim.TRUNC_COMMENT"); 
         }
      } catch (Exception e) {
         StatusMessages.instance().addFromResourceBundle(Severity.INFO, "gov.va.med.fee.MSG_ERROR5", "");
         log.error("Disapproval processing error", e);
      }
      return null;
   }
   
   final String getAdjReasonCd(String reason) {
      String retVal = "";
      //capture any set of charaters prior to the |, which is the reason code
      String regexpat = "([^-]*)";
      Pattern pattern = Pattern.compile(regexpat);
      Matcher matcher = pattern.matcher(reason);
      boolean found = matcher.find();
      if (found) {
         retVal = matcher.group(0).trim(); //there should only be one found 
      }
      return retVal;
   }
   
   @Override
   public String cancel() {
      return "cancel";
   }

}

