/* $Id: RejectClaimsSessionBean.java,v 1.20 2010/08/30 16:55:07 vhahacmagdua Exp $ */
package gov.va.med.fee.claim;

import gov.va.med.domain.fee.AdjustmentGroup;
import gov.va.med.domain.fee.AdjustmentReason;
import gov.va.med.domain.fee.CalcMethod;
import gov.va.med.domain.fee.Claim;
import gov.va.med.domain.fee.ClaimPayment;
import gov.va.med.domain.fee.ClaimPaymentType;
import gov.va.med.domain.fee.ClaimProcRecAdjust;
import gov.va.med.domain.fee.ClaimProcRecon;
import gov.va.med.domain.fee.ClaimProcRemitRemark;
import gov.va.med.domain.fee.ClaimProcStatus;
import gov.va.med.domain.fee.ClaimProcedure;
import gov.va.med.domain.fee.ClaimQueue;
import gov.va.med.domain.fee.ClaimReconciliation;
import gov.va.med.domain.fee.RemittanceRemark;
import gov.va.med.domain.fee.WsFacStatus;
import gov.va.med.fee.sec.HomePageUserData;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.log.Log;

import java.math.BigDecimal;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.ejb.Remove;
import javax.ejb.Stateful;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;


/**
 * The session bean that handles the Reject claim logic.
 */
@Stateful
@Scope(ScopeType.SESSION)
@Name("claimrejection")
@Restrict("#{s:hasRole('FEE_SUPERVISOR') or s:hasRole('FEE_VISN_PGM_MANAGER') or s:hasRole('REJECTED_CLAIMS_CLERK')}")
public class RejectClaimsSessionBean implements RejectClaims {
    private static final String COMPLETE_STATUS = "COMPLETE";
    private static final String ESTABLISH_STATUS = "ESTABLISH";
    private static final String REVERSED_STATUS = "REVERSED";
    private static final String REJECTED_STATUS = "REJECTED";
    @Logger
    private Log log;
    @PersistenceContext(unitName = "FPPS_Owner")
    private EntityManager em;
    private Set<Long> keys = new HashSet<Long>();
    private List<RejectedClaimsTo> currentPage;
    private RejectedClaimsTo currentItem;
    private List<ClaimQueue> currentHist;
    private long currentHistClmId;
    private int currentRow;
    private int origNumOfResults = -1;

    /** The column to sort by. */
    private String sortCol = "";

    /** The direction to sort. */
    private String sortDir = "";
    @In(value = "homePageUserData", required = false, scope = ScopeType.SESSION)
    private HomePageUserData homePageUserData;
    private String prevFac;
    private RejectedClaimsList disapprovedList;

    public Set<Long> getKeys() {
        return keys;
    }

    public void setKeys(Set<Long> keys) {
        this.keys = keys;
    }

    public void setCurrentItem(RejectedClaimsTo currentItem) {
        log.debug("setting current item to claim id: {0}, with object id: {1}.",
            currentItem.getClaimId(), currentItem);
        this.currentItem = currentItem;
    }

    public RejectedClaimsTo getCurrentItem() {
        if ((disapprovedList != null) && (origNumOfResults > disapprovedList.getNumberOfResults())) {
            throw new javax.persistence.OptimisticLockException();
        }

        return currentItem;
    }

    public int getCurrentRow() {
        if ((disapprovedList != null) && (origNumOfResults > disapprovedList.getNumberOfResults())) {
            throw new javax.persistence.OptimisticLockException();
        }

        return currentRow;
    }

    public void setCurrentRow(int currentRow) {
        log.debug("setting current row to : {0}", currentRow);
        this.currentRow = currentRow;
    }

    public void saveComments(String comment) {
        log.debug("saving to comment {0} to current item: {1} with object id: {2}",
            comment, currentItem.getClaimId(), currentItem);
        currentItem.setReviewersComments(comment);
    }

    /**
     * Gets the sort column.
     *
     * @return the sortCol.
     */
    public final String getSortCol() {
        log.debug("call RejectClaimsSessionBean.getSortCol() returning: {0}",
            this.sortCol);

        return this.sortCol;
    }

    /**
     * Sets the sort column.
     *
     * @param sortCol
     *           the sort column.
     */
    public final void setSortCol(String sortCol) {
        log.debug("call RejectClaimsSessionBean.setSortCol({0})", sortCol);
        this.sortCol = sortCol;
    }

    /**
     * Gets the sort direction.
     *
     * @return the sortDir.
     */
    public final String getSortDir() {
        log.debug("call RejectClaimsSessionBean.getSortDir() returning: {0}",
            this.sortDir);

        return this.sortDir;
    }

    /**
     * Sets the sort direction.
     *
     * @param sortDir
     *           the sort direction.
     */
    public final void setSortDir(String sortDir) {
        log.debug("call RejectClaimsSessionBean.setSortDir({0})", sortDir);
        this.sortDir = sortDir;
    }

    public final void doSort(String col) {
        log.debug("call RejectClaimsSessionBean.doSort({0})", col);

        if (this.sortCol.equalsIgnoreCase(col)) {
            this.sortDir = this.sortDir.equalsIgnoreCase("asc") ? "desc" : "asc";
        } else {
            this.sortCol = col;
            this.sortDir = "asc";
        }

        if (disapprovedList != null) {
            disapprovedList.setSortCol(this.sortCol);
            disapprovedList.setSortDir(this.sortDir);
            disapprovedList.rebuildClaimsListByType();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Destroy
    @Remove
    public final void destroy() {
        // log.info("destroyed:" + this);
    }

    public String getReasonCodeDesc(String cd) {
        Query q = em.createQuery(
                "select rc from fAdjustmentReason rc where rc.adjustmentReasonCd = :code ");
        q.setParameter("code", cd);

        AdjustmentReason rc = (AdjustmentReason) q.getSingleResult();

        return rc.getDescription();
    }

    /**
     * @return the rejectedList
     */
    public RejectedClaimsList getDisapprovedList() {
        if (disapprovedList == null) {
            disapprovedList = new RejectedClaimsList();
            disapprovedList.setEm(em);
            disapprovedList.addObserver(this);
        }

        disapprovedList.setSortCol(sortCol);
        disapprovedList.setSortDir(sortDir);
        disapprovedList.setHomePageUserData(homePageUserData);

        if ((origNumOfResults == -1) ||
                !prevFac.equals(homePageUserData.getActiveStations().toString())) {
            origNumOfResults = disapprovedList.getNumberOfResults();
            prevFac = homePageUserData.getActiveStations().toString();
        }

        log.debug("returning from RejectClaimsSessionBean getDisapprovedList: {0}",
            disapprovedList);

        return disapprovedList;
    }

    /**
     * @param rejectedList the rejectedList to set
     */
    public void setDisapprovedList(RejectedClaimsList disapprovedList) {
        log.debug("setting RejectClaimsSessionBean disapproved list to {0}",
            disapprovedList);
        this.disapprovedList = disapprovedList;
    }

    @Override
    public void reverseSelDisapprovals() {
        log.debug("reverseSelDisapprovals()");

        if ((disapprovedList != null) && (origNumOfResults > disapprovedList.getNumberOfResults())) {
            throw new javax.persistence.OptimisticLockException();
        }

        reverseDisapprovals(currentPage);
    }

    @Override
    public void okSelDisapprovals() {
        log.debug("okSelDisapprovals()");

        if ((disapprovedList != null) && (origNumOfResults > disapprovedList.getNumberOfResults())) {
            throw new javax.persistence.OptimisticLockException();
        }

        okDisapprovals(currentPage);
    }

    @Override
    public void reverseSingleDisapproval(long claimId) {
        log.debug("reverseSingleDisapproval({0})", claimId);

        if ((disapprovedList != null) && (origNumOfResults > disapprovedList.getNumberOfResults())) {
            throw new javax.persistence.OptimisticLockException();
        }

        RejectedClaimsTo r = new RejectedClaimsTo();
        r.setClaimId(claimId);
        r.setSelected(true);

        List<RejectedClaimsTo> l = new LinkedList<RejectedClaimsTo>();
        l.add(r);
        reverseDisapprovals(l);
    }

    @Override
    public void okSingleDisapproval(long claimId) {
        log.debug("okSingleDisapproval({0})", claimId);

        if ((disapprovedList != null) && (origNumOfResults > disapprovedList.getNumberOfResults())) {
            throw new javax.persistence.OptimisticLockException();
        }

        RejectedClaimsTo r = new RejectedClaimsTo();
        r.setClaimId(claimId);
        r.setSelected(true);

        List<RejectedClaimsTo> l = new LinkedList<RejectedClaimsTo>();
        l.add(r);
        okDisapprovals(l);
    }

    @Transactional
    void reverseDisapprovals(List<RejectedClaimsTo> rclaims) {
        boolean truncComments = false;
        String user = homePageUserData.getUserName().toUpperCase();
        java.util.Date today = new java.util.Date();
        java.sql.Date sqldate = new java.sql.Date(today.getTime());

        Query q = em.createQuery(
                "select cl from fClaim cl where cl.claimIndex = :index");
        Query q1 = em.createQuery(
                "select f from fWsFacStatus f where f.claimIndex = :index");
        Query q2 = em.createQuery(
                "select q from  fClaimQueue q where q.claim.claimIndex = :index " +
                " and q.dequeueDate is null " +
                " and q.claimQueueTypeCd = 'REJECTION_APPROVAL' " +
                " order by q.enqueueDate desc");
        List<RejectedClaimsTo> removed = new LinkedList<RejectedClaimsTo>();
        String claimMsg = "";

        for (RejectedClaimsTo r : rclaims) {
            if (r.isSelected()) {
                //set claim back to established, will only return single
                q.setParameter("index", r.getClaimId());

                Claim claim = (Claim) q.getSingleResult();
                claim.setClaimStatusCd(ESTABLISH_STATUS);
                claim.setModifiedBy(user);
                claim.setDateModified(sqldate);

                //set claim in ws_fac_stat table back to established, will only return single
                q1.setParameter("index", r.getClaimId());

                WsFacStatus f = (WsFacStatus) q1.getSingleResult();
                f.setClaimStatusCd(ESTABLISH_STATUS);

                //update the claim queue table, may return multiple
                q2.setParameter("index", r.getClaimId());

                List cqrl = q2.getResultList();

                if (cqrl.size() == 0) {
                    throw new javax.persistence.OptimisticLockException();
                }

                ClaimQueue cq = (ClaimQueue) cqrl.get(0); //get the latest entry in the claim queue
                String comment = r.getReviewersComments();

                if ((comment != null) && (comment.length() > 250)) {
                    comment = comment.substring(0, 250);
                    truncComments = true;
                }

                cq.setDequeueComments(comment);
                cq.setDequeueDate(sqldate);
                cq.setOutcome(REVERSED_STATUS);
                cq.setModifiedBy(user);
                cq.setDateModified(sqldate);

                log.debug("reversed disapproval for claim {0}", r.getClaimId());
                claimMsg = claimMsg.equals("") ? Long.toString(r.getClaimId())
                                               : (claimMsg + ", " +
                    Long.toString(r.getClaimId()));

                for (ClaimProcedure claimProc : claim.getClaimProcedures()) {
                    claimProc.setLineStatus(ESTABLISH_STATUS);
                }

                removed.add(r);
            }
        }

        if (this.currentPage != null) {
            this.currentPage.removeAll(removed);
        }

        StatusMessages.instance()
                      .addFromResourceBundle(Severity.INFO,
            "gov.va.med.fee.claim.RejectClaim.SUCCESS_REV", claimMsg);

        if (truncComments) {
            StatusMessages.instance()
                          .addFromResourceBundle(Severity.WARN,
                "gov.va.med.fee.claim.TRUNC_COMMENT");
        }

        origNumOfResults = -1;
    }

    @SuppressWarnings("unchecked")
    @Transactional
    void okDisapprovals(List<RejectedClaimsTo> rclaims) {
        boolean truncComments = false;
        String user = homePageUserData.getUserName().toUpperCase();
        java.util.Date today = new java.util.Date();
        java.sql.Date sqldate = new java.sql.Date(today.getTime());

        Query qcl = em.createQuery(
                "select cl from fClaim cl where cl.claimIndex = :index");
        Query qwfs = em.createQuery(
                "select f from fWsFacStatus f where f.claimIndex = :index");
        Query qcq = em.createQuery(
                "select q from  fClaimQueue q where q.claim.claimIndex = :index " +
                " and q.dequeueDate is null " +
                " and q.claimQueueTypeCd = 'REJECTION_APPROVAL' " +
                " order by q.enqueueDate desc");
        Query qcp = em.createQuery(
                "select cp from fClaimProcedure cp where cp.claim.claimIndex = :index ");

        Query qcpt = em.createQuery(
                "select cp from fClaimPaymentType cp where cp.claimPaymentTypeCode = 'NON'");
        ClaimPaymentType cpt = (ClaimPaymentType) qcpt.getSingleResult();

        Query qcm = em.createQuery(
                "select cm from fCalcMethod cm where cm.calcMethodCd = '-3' ");
        CalcMethod cm = (CalcMethod) qcm.getSingleResult();

        Query qcps = em.createQuery(
                "select cps from fClaimProcStatus cps where cps.claimProcStatusCd = 'REJECTED' ");
        ClaimProcStatus cps = (ClaimProcStatus) qcps.getSingleResult();

        Query qag = em.createQuery(
                "select ag from fAdjustmentGroup ag where ag.adjustmentGroupCd = 'OA'");
        AdjustmentGroup ag = (AdjustmentGroup) qag.getSingleResult();

        Query qar = em.createQuery(
                "select ar from fAdjustmentReason ar where ar.adjustmentReasonCd = '96'");
        AdjustmentReason ar = (AdjustmentReason) qar.getSingleResult();

        Query qrr1 = em.createQuery(
                "select r from fRemittanceRemark r where r.remittanceRemarkCd = 'N1' ");
        RemittanceRemark rr1 = (RemittanceRemark) qrr1.getSingleResult();

        Query qrr2 = em.createQuery(
                "select r from fRemittanceRemark r where r.remittanceRemarkCd = 'M118' ");
        RemittanceRemark rr2 = (RemittanceRemark) qrr2.getSingleResult();

        String claimMsg = "";
        List<RejectedClaimsTo> removed = new LinkedList<RejectedClaimsTo>();

        for (RejectedClaimsTo r : rclaims) {
            if (r.isSelected()) {
                qcl.setParameter("index", r.getClaimId());

                Claim claim = (Claim) qcl.getSingleResult();
                claim.setClaimStatusCd(COMPLETE_STATUS);
                claim.setModifiedBy(user);
                claim.setDateModified(sqldate);

                //set claim in ws_fac_stat table back to established, will only return single
                qwfs.setParameter("index", r.getClaimId());

                WsFacStatus f = (WsFacStatus) qwfs.getSingleResult();
                f.setClaimStatusCd(COMPLETE_STATUS);
                f.setLinesClosed(new Long(claim.getClaimProcedures().size()));

                //update the claim queue table, may return multiple
                qcq.setParameter("index", r.getClaimId());

                List cqrl = qcq.getResultList();

                if (cqrl.size() == 0) {
                    throw new javax.persistence.OptimisticLockException();
                }

                ClaimQueue cq = (ClaimQueue) cqrl.get(0); //get the latest entry in the claim queue
                String comment = r.getReviewersComments();

                if ((comment != null) && (comment.length() > 249)) {
                    comment = comment.substring(0, 250);
                    truncComments = true;
                }

                cq.setDequeueComments(comment);
                cq.setDequeueDate(sqldate);
                cq.setModifiedBy(user);
                cq.setDateModified(sqldate);
                cq.setOutcome("OK");

                // add claim payment data to claim
                ClaimPayment cp = new ClaimPayment();
                cp.setPaymentBatchDate(sqldate);
                cp.setClaimPaymentType(cpt);
                cp.setCreatedBy(user);
                cp.setDateCreated(sqldate);
                cp.setClaim(claim);
                claim.getClaimPayments().add(cp);

                // add claim reconciliation data to claim
                ClaimReconciliation cr = new ClaimReconciliation();
                cr.setSent835Flag('N');
                cr.setCalcMethod(cm);
                cr.setCreatedBy(user);
                cr.setDateCreated(sqldate);
                cr.setClaim(claim);
                claim.getClaimReconciliations().add(cr);

                //update claim procedure records
                qcp.setParameter("index", r.getClaimId());

                List<ClaimProcedure> lcp = qcp.getResultList();

                for (ClaimProcedure claimProc : lcp) {
                    ClaimProcRecon cpr = new ClaimProcRecon();

                    cpr.setClaim(claim);
                    claim.getClaimProcRecons().add(cpr);

                    cpr.setClaimProcedure(claimProc);
                    claimProc.getClaimProcRecons().add(cpr);

                    cpr.setClaimReconciliation(cr);
                    cr.getClaimProcRecons().add(cpr);

                    cpr.setClaimPayment(cp);
                    cp.getClaimProcRecons().add(cpr);

                    cpr.setSent835Flag('N');

                    cpr.setBundleFlag('N');

                    cpr.setClaimProcStatus(cps);

                    cpr.setCreatedBy(homePageUserData.getUserName());

                    cpr.setDateCreated(sqldate);

                    //create procedures reconciliation adjustments
                    ClaimProcRecAdjust cpra = new ClaimProcRecAdjust();
                    cpra.setAdjustmentAmount(getBilledAmount(
                            claimProc.getClaimProcId()));
                    cpra.setAdjustmentGroup(ag);
                    cpra.setAdjustmentReason(ar);
                    cpra.setCreatedBy(user);
                    cpra.setDateCreated(sqldate);
                    cpra.setClaimProcRecon(cpr);
                    cpr.getClaimProcRecAdjusts().add(cpra);

                    //create remit remarks
                    ClaimProcRemitRemark cprr1 = new ClaimProcRemitRemark();
                    cprr1.setRemittanceRemark(rr1);
                    cprr1.setCreatedBy(user);
                    cprr1.setDateCreated(sqldate);
                    cpr.getClaimProcRemitRemarks().add(cprr1);
                    cprr1.setClaimProcRecon(cpr);

                    ClaimProcRemitRemark cprr2 = new ClaimProcRemitRemark();
                    cprr2.setRemittanceRemark(rr2);
                    cprr2.setCreatedBy(user);
                    cprr2.setDateCreated(sqldate);
                    cpr.getClaimProcRemitRemarks().add(cprr2);
                    cprr2.setClaimProcRecon(cpr);

                    // for a rejected claim all the claim procedures will be
                    // in in the REJECTED status with the claim itself in the
                    // COMPLETED status which is needed for 835 creation.
                    claimProc.setLineStatus(REJECTED_STATUS);
                }

                if (this.currentPage != null) {
                    this.currentPage.removeAll(removed);
                }

                claimMsg = claimMsg.equals("") ? Long.toString(r.getClaimId())
                                               : (claimMsg + ", " +
                    Long.toString(r.getClaimId()));
            }
        }

        StatusMessages.instance()
                      .addFromResourceBundle(Severity.INFO,
            "gov.va.med.fee.claim.RejectClaim.SUCCESS_OK", claimMsg);

        if (truncComments) {
            StatusMessages.instance()
                          .addFromResourceBundle(Severity.WARN,
                "gov.va.med.fee.claim.TRUNC_COMMENT");
        }

        origNumOfResults = -1;
    }

    BigDecimal getClaimAmount(long index) {
        Query q = em.createNativeQuery(
                "select sum(billed_amount) from claim_procedure where claim_index = :index");
        q.setParameter("index", index);

        return (BigDecimal) q.getSingleResult();
    }

    BigDecimal getBilledAmount(long clmProcId) {
        Query q = em.createNativeQuery(
                "select billed_amount from claim_procedure where claim_proc_id = :id");
        q.setParameter("id", clmProcId);

        return (BigDecimal) q.getSingleResult();
    }

    @Override
    public void update(List<RejectedClaimsTo> arg) {
        log.debug("setting current page to list of size {0}", arg.size());
        this.currentPage = arg;
        currentItem = null;
        currentHist = null;
    }

    /**
     * @param currentHist the currentHist to set
     */
    public void setCurrentHist(List<ClaimQueue> currentHist) {
        log.debug("setCurrentHist({0})", currentHist);
        this.currentHist = currentHist;

        if (currentHist.size() != 0) {
            setCurrentHistClmId(currentHist.get(0).getClaim().getClaimIndex());
        } else {
            currentHist = null;
            setCurrentHistClmId(0);
        }
    }

    /**
     * @return the currentHist
     */
    public List<ClaimQueue> getCurrentHist() {
        log.debug("getCurrentHist() : returning {0}", currentHist);

        if ((disapprovedList != null) && (origNumOfResults > disapprovedList.getNumberOfResults())) {
            throw new javax.persistence.OptimisticLockException();
        }

        return currentHist;
    }

    /**
     * @return the currentHistClmId
     */
    public long getCurrentHistClmId() {
        return currentHistClmId;
    }

    /**
     * @param currentHistClmId the currentHistClmId to set
     */
    public void setCurrentHistClmId(long currentHistClmId) {
        this.currentHistClmId = currentHistClmId;
    }
}
