/* $Id: RejectedClaimsTo.java,v 1.9 2009/12/30 21:47:58 vhahacmagdua Exp $ */
package gov.va.med.fee.claim;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import gov.va.med.domain.fee.ClaimQueue;


/**
 * Transfer object for the RejectedClaims page.
 *
 */
public class RejectedClaimsTo {
    private long claimId;
    private String type;
    private String hero;
    private String ssn;
    private Date claimDate;
    private BigDecimal amount;
    private String provider;
    private String disapprover;
    private Date disapprovedDate;
    private String reason;
    private String reasonDesc;
    private String disapproversComments;
    private String reviewersComments;
    private List<ClaimQueue> rejHist;
    private boolean selected;


    /**
     * Gets the claim Id in string form;
     *
     *
     * @return The claim Id in String form.
     */
    public final String getClaimIdStr() {
        return Long.toString(claimId);
    }

    /**
     * Getter for the claimId.
     *
     * @return the claims Index in string form.
     */
    public final long getClaimId() {
        return claimId;
    }

    /**
     * Setter for the claimId.
     *
     * @param claimId the Claim's index.
     */
    public final void setClaimId(long claimId) {
        this.claimId = claimId;
    }

    /**
     * Getter for the type of claim. "PROF" (professional),
     * INST (institutional), or "DEN" (dental).
     *
     * @return the type of claim.
     */
    public final String getType() {
        return type;
    }

    /**
     * Getter for the type of claim. "PROF" (professional),
     * INST (institutional), or "DEN" (dental).
     *
     * @param type the type of claim.
     */
    public final void setType(String type) {
        this.type = type;
    }

    /**
     * Getter indicating whether or not this claim falls under the
     * "HERO" project.
     *
     * @return "Y" if this is a HERO claim, "N" if this is
     *         not a HERO claim.
     */
    public final String getHero() {
        return hero;
    }

    /**
     * Setter to indicate whether or not this claim falls under the
     * "HERO" project.
     *
     * @param hero "Y" if this is a HERO claim "N" if not.
     */
    public final void setHero(String hero) {
        this.hero = hero;
    }

    /**
     * Getter for the Social Security Number of the Veteran (without) dashes.
     *
     * @return the Social Security Number of the Veteran (no Dashes).
     */
    public final String getSsn() {
        return ssn;
    }

    /**
     * Setter for the Social Security Number of the Veteran (without) dashes.
     * @param ssn the Social Security Number of the Veteran (no
     *             Dashes) to set.
     */
    public final void setSsn(String ssn) {
        this.ssn = ssn;
    }

    /**
     * @return the claimDate.
     */
    public final Date getClaimDate() {
        return claimDate;
    }

    /**
     * @param claimDate the claimDate to set.
     */
    public final void setClaimDate(Date claimDate) {
        this.claimDate = claimDate;
    }

    /**
     * @return the amount.
     */
    public final BigDecimal getAmount() {
        return amount;
    }

    /**
     * @param amount the amount to set.
     */
    public final void setAmount(BigDecimal amount) {
        this.amount = amount;
    }

    /**
     * @return the provider.
     */
    public final String getProvider() {
        return provider;
    }

    /**
     * @param provider the provider to set.
     */
    public final void setProvider(String provider) {
        this.provider = provider;
    }

    /**
     * @return the disapprover.
     */
    public final String getDisapprover() {
        return disapprover;
    }

    /**
     * @param disapprover the disapprover to set.
     */
    public final void setDisapprover(String disapprover) {
        this.disapprover = disapprover;
    }

    /**
     * @return the disapprovedDate.
     */
    public final Date getDisapprovedDate() {
        return disapprovedDate;
    }

    /**
     * @param disapprovedDate the disapprovedDate to set.
     */
    public final void setDisapprovedDate(Date disapprovedDate) {
        this.disapprovedDate = disapprovedDate;
    }

    /**
     * @return the reason.
     */
    public final String getReason() {
        return reason;
    }

    /**
     * @param reason the reason to set.
     */
    public final void setReason(String reason) {
        this.reason = reason;
    }
    
    /**
     * @return the reasonDesc
     */
    public String getReasonDesc() {
       return reasonDesc;
    }

    /**
     * @param reasonDesc the reasonDesc to set
     */
    public void setReasonDesc(String reasonDesc) {
       this.reasonDesc = reasonDesc;
    }

    /**
     * @return the disapproversComments.
     */
    public final String getDisapproversComments() {
        return disapproversComments;
    }

    /**
     * @param disapproversComments the disapproversComments to set.
     */
    public final void setDisapproversComments(String disapproversComments) {
        this.disapproversComments = disapproversComments;
    }

    /**
     * @return the reviewersComments.
     */
    public final String getReviewersComments() {
        return reviewersComments;
    }

    /**
     * @param reviewersComments the reviewersComments to set.
     */
    public final void setReviewersComments(String reviewersComments) {
        this.reviewersComments = reviewersComments;
    }

    /**
     * @return true if the claim is selected, false otherwise.
     */
    public boolean isSelected() {
        return selected;
    }

    /**
     *
     * @param selected true if the claim is selected, false
     *                 otherwise.
     */
    public void setSelected(boolean selected) {
        this.selected = selected;
    }


   /**
    * @return the rejHist
    */
   public List<ClaimQueue> getRejHist() {
      return rejHist;
   }

   /**
    * @param rejHist the rejHist to set
    */
   public void setRejHist(List<ClaimQueue> rejHist) {
      this.rejHist = rejHist;
   }

   public String toDebugString() {
      return "RejectedClaimsTo [amount=" + amount + ", claimDate=" + claimDate + ", claimId=" + claimId + ", disapprovedDate="
            + disapprovedDate + ", disapprover=" + disapprover + ", disapproversComments=" + disapproversComments + ", hero="
            + hero + ", provider=" + provider + ", reason=" + reason + ", rejHist=" + rejHist + ", reviewersComments="
            + reviewersComments + ", selected=" + selected + ", ssn=" + ssn + ", type=" + type + "]";
   }

   /* (non-Javadoc)
    * @see java.lang.Object#hashCode()
    */
   @Override
   public int hashCode() {
      final int prime = 31;
      int result = 1;
      result = prime * result + ((amount == null) ? 0 : amount.hashCode());
      result = prime * result + ((claimDate == null) ? 0 : claimDate.hashCode());
      result = prime * result + (int) (claimId ^ (claimId >>> 32));
      result = prime * result + ((disapprovedDate == null) ? 0 : disapprovedDate.hashCode());
      result = prime * result + ((disapprover == null) ? 0 : disapprover.hashCode());
      result = prime * result + ((disapproversComments == null) ? 0 : disapproversComments.hashCode());
      result = prime * result + ((hero == null) ? 0 : hero.hashCode());
      result = prime * result + ((provider == null) ? 0 : provider.hashCode());
      result = prime * result + ((reason == null) ? 0 : reason.hashCode());
      result = prime * result + ((reasonDesc == null) ? 0 : reasonDesc.hashCode());
      result = prime * result + ((rejHist == null) ? 0 : rejHist.hashCode());
      result = prime * result + ((reviewersComments == null) ? 0 : reviewersComments.hashCode());
      result = prime * result + (selected ? 1231 : 1237);
      result = prime * result + ((ssn == null) ? 0 : ssn.hashCode());
      result = prime * result + ((type == null) ? 0 : type.hashCode());
      return result;
   }

   /* (non-Javadoc)
    * @see java.lang.Object#equals(java.lang.Object)
    */
   @Override
   public boolean equals(Object obj) {
      if (this == obj)
         return true;
      if (obj == null)
         return false;
      if (getClass() != obj.getClass())
         return false;
      RejectedClaimsTo other = (RejectedClaimsTo) obj;
      if (amount == null) {
         if (other.amount != null)
            return false;
      } else if (!amount.equals(other.amount))
         return false;
      if (claimDate == null) {
         if (other.claimDate != null)
            return false;
      } else if (!claimDate.equals(other.claimDate))
         return false;
      if (claimId != other.claimId)
         return false;
      if (disapprovedDate == null) {
         if (other.disapprovedDate != null)
            return false;
      } else if (!disapprovedDate.equals(other.disapprovedDate))
         return false;
      if (disapprover == null) {
         if (other.disapprover != null)
            return false;
      } else if (!disapprover.equals(other.disapprover))
         return false;
      if (disapproversComments == null) {
         if (other.disapproversComments != null)
            return false;
      } else if (!disapproversComments.equals(other.disapproversComments))
         return false;
      if (hero == null) {
         if (other.hero != null)
            return false;
      } else if (!hero.equals(other.hero))
         return false;
      if (provider == null) {
         if (other.provider != null)
            return false;
      } else if (!provider.equals(other.provider))
         return false;
      if (reason == null) {
         if (other.reason != null)
            return false;
      } else if (!reason.equals(other.reason))
         return false;
      if (reasonDesc == null) {
         if (other.reasonDesc != null)
            return false;
      } else if (!reasonDesc.equals(other.reasonDesc))
         return false;
      if (rejHist == null) {
         if (other.rejHist != null)
            return false;
      } else if (!rejHist.equals(other.rejHist))
         return false;
      if (reviewersComments == null) {
         if (other.reviewersComments != null)
            return false;
      } else if (!reviewersComments.equals(other.reviewersComments))
         return false;
      if (selected != other.selected)
         return false;
      if (ssn == null) {
         if (other.ssn != null)
            return false;
      } else if (!ssn.equals(other.ssn))
         return false;
      if (type == null) {
         if (other.type != null)
            return false;
      } else if (!type.equals(other.type))
         return false;
      return true;
   }


    
    
}
