package gov.va.med.fee.claim;

import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.ejb.Stateless;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.html.HtmlSelectOneMenu;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.naming.InitialContext;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;


import org.jboss.seam.Component;
import org.jboss.seam.ScopeType;
import static org.jboss.seam.ScopeType.SESSION;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.datamodel.DataModel;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.annotations.web.RequestParameter;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.log.Log;

import gov.va.med.domain.fee.*;
import gov.va.med.fee.Page;
import gov.va.med.fee.PageControls;

/**
 * RerouteClaimBean class -session bean for reroute claims page
 *
 * Displays list of claims selected to be rerouted shows reroute history (if
 * applicable) of selected claim executes rerouting of claim to selected
 * destination
 *
 * @author vhahacprestt
 *
 */
// @Stateful
@Stateless
@Name("rerouteClaims")
//@Scope(SESSION)
// remove implements Serializable
@Restrict("#{(s:hasRole('FEE_VISN_PGM_MANAGER') or s:hasRole('FEE_SUPERVISOR') or s:hasRole('FEE_CLERK') or s:hasRole('ADMIN') or s:hasRole('REJECTED_CLAIMS_CLERK') )}")
public class RerouteClaimBean implements RerouteClaims {

   private final String DEFAULT_SELECT_VALUE = "-select-";
   private String selectedClaimId = ""; // id for claim whose reroute history is
                                        // displayed
   private String reason = "";
   private String destination = ""; // for use with radio button function
   private String comment;
   private String historyTableTitle = ""; // header for history title (claim id
                                          // or no history)

   private static final Pattern DIGITS = Pattern.compile("[0-9]*");

   /** The faces messages. */
   @In
   private FacesMessages facesMessages;

   /** The log. */
   @Logger
   private Log log;

   /** The em. */
   @PersistenceContext(unitName = "FPPS_Owner")
   private EntityManager em;

   /** The column to sort by. */
   @RequestParameter
   private String sortCol;

   /** the Rerouting History List */
   //private List<ClaimReroutingHistoryEntity> reroutingHistory = new ArrayList<ClaimReroutingHistoryEntity>();

   @Out(required = false, scope = ScopeType.EVENT)
   private Map<Long, List<ClaimReroutingHistoryEntity>> allClaimsRerouteHistory =
    new LinkedHashMap<Long, List<ClaimReroutingHistoryEntity>>();

   /** claims to reroute list passed by parent page */

   @In(scope = ScopeType.SESSION, required = false)
   @Out(scope = ScopeType.SESSION, required = false)
   private List<Long> claimsToReroute;

   @Out(required = false, scope = ScopeType.EVENT)
   private List<RerouteClaimListEntity> reroutes;

   private List<Long> nonRoutableClaims = new LinkedList<Long>();

   private List<Long> printedClaims = new LinkedList<Long>();

   /** list of claims that were denied reroute (for temp storage) */
   List<RerouteClaimListEntity> deniedReroutes;

   public Map<Long, List<ClaimReroutingHistoryEntity>> getAllClaimsRerouteHistory() {
    return allClaimsRerouteHistory;
   }

   /**
    * Gets the sort column.
    *
    * @return the sortCol.
    */
   public String getSortCol() {
      return this.sortCol;
   }

   /**
    * Sets the sort column.
    *
    * @param sortCol
    *           the sort column.
    */
   public void setSortCol(String sortCol) {
      this.sortCol = sortCol;
   }

   /** The direction to sort. */
   @RequestParameter
   private String sortDir;

   /**
    * Gets the sort direction.
    *
    * @return the sortDir.
    */
   public String getSortDir() {
      return this.sortDir;
   }

   /**
    * Sets the sort direction.
    *
    * @param sortDir
    *           the sort direction.
    */
   public void setSortDir(String sortDir) {
      this.sortDir = sortDir;
   }

   /** The app user. */
   @In(value = "appUser", required = true, scope = ScopeType.SESSION)
   private AppUser appUser;

   /** The SQL query for searching claims. */
   private String sql;

   /** the Query object. */
   private Query q;

   /** the List object. */
   private List l;

   /** collection of visns as SelectItems */
   @DataModel
   private List<SelectItem> visnList;

   /** collection of facilities */
   @DataModel
   private List<SelectItem> facilitiesList;

   /** collection of usernames */
   @DataModel
   private List<SelectItem> userList;

   /** collection of reroute reasons */
   @DataModel
   private List<SelectItem> rerouteReasons;

   /** An Iterator instance. */
   Iterator it;

   /** The pagination. */
   private Page page;
   /** The page controls. */
   private PageControls pageControls;

   private String nonRoutableStr = "";

   private String printedClaimsStr = "";

   /** the String backing beans for destination */
   private String selectedVisnId = DEFAULT_SELECT_VALUE;
   private String facilityId = DEFAULT_SELECT_VALUE;
   private String userName = DEFAULT_SELECT_VALUE;

   /** the constructor. */
   public RerouteClaimBean() {
      page = new Page();
      pageControls = new PageControls();
   }

   /**
    * Factory method for retrieving claims selected for reroute and placing them
    * on the reroute page using the reroutes session variable
    */
   @Factory(value = "reroutes")
   @Override
   public final void populateReroutes() {
      this.nonRoutableClaims.clear();
      this.printedClaims.clear();
      historyTableTitle = "";
      if (reroutes == null) {
         reroutes = new ArrayList<RerouteClaimListEntity>();
      } else {
         reroutes.clear();
      }
      String qstr = "select c from fClaim c where c.claimIndex = :index ";
      Query q = em.createQuery(qstr);

      for (Long index : claimsToReroute) {
         q.setParameter("index", index);
         try {
            Claim claim = (Claim) q.getSingleResult();
            if (routable(claim)) {
               RerouteClaimListEntity cle = new RerouteClaimListEntity();
               cle.setClaimIndex(index.toString());
               // cle.setClaimIndex(index);
               cle.setVetName(getVeteranName(claim.getPersonInfos()));
               cle.setBillingProviderName(getBillingProvName(claim.getProviderInfos()));
               cle.setProviderName(getServiceProvName(claim.getProviderInfos()));
               if(isPrintedClaim(claim))
                printedClaims.add(index);
               reroutes.add(cle);
            } else {
               this.nonRoutableClaims.add(index);
            }
         } catch (NoResultException e) {
            log.error("error:", e);
            log.error("unable to find claim with index {0} for rerouting", index);
         }
      }

      //reroutingHistory.clear();

      allClaimsRerouteHistory.clear();
      for (Long index : claimsToReroute) {
        List<ClaimReroutingHistoryEntity> hist = loadRerouteHistory(index);
        allClaimsRerouteHistory.put(index, hist);
      }

      if (this.nonRoutableClaims.size() > 0) {
         String msg = null;
         for(Long i : this.nonRoutableClaims) {
            msg = msg == null ? i.toString() : (msg + ", " + i.toString());
         }
         this.nonRoutableStr = "The following "
            + (this.nonRoutableClaims.size() > 1 ? "claims" : "claim")
            + " cannot be rerouted because "
            + (this.nonRoutableClaims.size() > 1? "they are " : "it is")
            + " completed, partially reconciled, rejected, or not FPPS: " + msg +"<br/>";
      }
      if(this.printedClaims.size() > 0){
        String pcMsg = null;
        for(Long i: this.printedClaims)
            pcMsg = pcMsg == null ? i.toString(): (pcMsg + ", "+ i.toString());
        this.printedClaimsStr = "The following "
            + (printedClaims.size() > 1 ? "claims have" : "claim has")
            + " been printed: " + pcMsg + ",  and will be rerouted if you continue.";

      }

   }//end populateReroutes

   public String getNonRouted() {
      this.nonRoutableStr = "";
      populateReroutes();
      return this.nonRoutableStr;
   }

   public String getPrintedClaimsStr(){
    this.printedClaimsStr = "";
    populateReroutes();
    return this.printedClaimsStr;
   }


	public boolean routable(Claim claim) {
		boolean retVal = false;
		// only FPPS claims can be rerouted
		if (claim.getVaImpTypeCd() == null
				|| claim.getVaImpTypeCd().trim().length() == 0) {
			if ("ESTABLISH".equalsIgnoreCase(claim.getClaimStatusCd())) {
				retVal = true;
			} else if ("INPROCESS".equalsIgnoreCase(claim.getClaimStatusCd())
					&& !hasReconciledLine(claim)) {
				retVal = true;
			}
		}

		// Vitria payment received, block reroute if payment has been made.
		if (retVal) {
			String pcQueryStr = "select p.claimIndex from fFeeClaimPayments p where p.claimIndex = :index ";
			Query query = em.createQuery(pcQueryStr);
			query.setParameter("index", claim.getClaimIndex());
			try {
				List result = query.getResultList();
				if (result != null && result.size() > 0) {
					retVal = false;
				}
			} catch (Exception ex) {
				log.error(ex);
			}
		}

		return retVal;
	}

   public boolean hasReconciledLine(Claim claim) {
      boolean retVal = false;
      for (ClaimProcedure cp : claim.getClaimProcedures()) {
         if (cp.getClaimProcRecons().size() > 0) {
            retVal = true;
            break;
         }
      }
      return retVal;
   }

   public boolean isPrintedClaim(Claim claim){
    String pcQueryStr = "select pc from fPrintedClaim pc where pc.claim.claimIndex = :index ";
    Query pcQuery = em.createQuery(pcQueryStr);
       pcQuery.setParameter("index", claim.getClaimIndex());
       try{
        PrintedClaim result = (PrintedClaim)pcQuery.getSingleResult();
        if(result!=null){
            return true;
        }
       }catch(Exception ex){
        log.debug("claim {0} not in print queue", claim.getClaimIndex());
       }
    return false;
   }

   /**
    * load reroute history table given a claimID called from claims table on
    * reroute page
    */
   private List<ClaimReroutingHistoryEntity> loadRerouteHistory(long claimIndex) {
      //reroutingHistory.clear();
    List<ClaimReroutingHistoryEntity> reroutingHistory = null;

      long ci = claimIndex;
      selectedClaimId = "" + claimIndex;
      historyTableTitle = "";
      String sqlStr = " SELECT  fcr.transfer_date,  fcrr.description,  transfer_initator,"
            + " fcr.transfer_to_facility_cd, fcr.transfer_to_visn_cd, fcr.transfer_to_user,"
            + " fcr.transferred_to,  fcr.transfer_from_user, "
            + " fcr.transfer_from_visn_cd ,  fcr.transfer_from_facility_cd, fcr.comments  "
            + "FROM FEE_CLAIM_ROUTING fcr "
            + "LEFT JOIN FEE_CLAIM_ROUTINGS_REASON fcrr  ON fcrr.transfer_reason_cd = fcr.transfer_reason_cd "
            + "WHERE fcr.claim_index='" + ci + "'  "
            + "ORDER BY transfer_date DESC ";

      try {
         Query query = em.createNativeQuery(sqlStr);

         List list = query.getResultList();
         if (!list.isEmpty()) {
            reroutingHistory = new ArrayList<ClaimReroutingHistoryEntity>();

            Iterator iterator = list.iterator();
            boolean results = false;

            while (iterator.hasNext()) {
                results = true;
                Object[] row = (Object[]) iterator.next();

                FeeClaimRouting fcr = new FeeClaimRouting();
                FeeClaimRoutingsReason fcrr = new FeeClaimRoutingsReason();

                fcr.setTransferDate((Date) row[0]);
                fcrr.setDescription((String) row[1]);
                fcr.setTransferInitator((String) row[2]);
                fcr.setTransferToFacilityCd((String) row[3]);
                fcr.setTransferToVisnCd((String) row[4]);
                fcr.setTransferToUser((String) row[5]);
                fcr.setTransferredTo((String) row[6]);
                fcr.setTransferFromUser((String) row[7]);
                fcr.setTransferFromVisnCd((String) row[8]);
                fcr.setTransferFromFacilityCd((String) row[9]);
                fcr.setComments((String) row[10]);

                ClaimReroutingHistoryEntity crrhe = new ClaimReroutingHistoryEntity(fcr, fcrr);
                // crrhe.getRouting().getTransferredTo());

                reroutingHistory.add(crrhe);
            }
         }
      } catch (Exception ex) {
         log.error(ex.getMessage(), ex);
      }

      return reroutingHistory;

   }

   public String getSelectedClaimId() {
      return selectedClaimId;
   }

   public void setSelectedClaimId(String selectedClaimId) {
      this.selectedClaimId = selectedClaimId;
   }

   /**
    * update facilities list based on VISN id
    */
   public void changeFacilities() {
      // previous selections are invalidated and replaced with defaults
      this.facilityId = DEFAULT_SELECT_VALUE;
      this.userName = DEFAULT_SELECT_VALUE;
      this.getFacilitiesList();
      this.getUserList(); // when facilities list changes so does the userlist
   }

   /**
    * update users list based on Facility id
    */
   public void changeUsers() {
      this.getUserList();
   }

   /**
    * retrieve list of VISNS from VISN table
    */
   public List<SelectItem> getVisnList() {
      if (visnList == null) {
         visnList = new ArrayList<SelectItem>();
      } else
         visnList.clear();
      // this.selectedVisnId=DEFAULT_SELECT_VALUE;
      sql = "select * from VISN";
      Query query = em.createNativeQuery(sql);
      List results = query.getResultList();
      Iterator it = results.iterator();
      while (it.hasNext()) {
         try {
            Object[] row = (Object[]) it.next();
            BigDecimal id = (BigDecimal) row[0];
            String visnCode = id.toString();
            visnList.add(new SelectItem("VISN" + visnCode, visnCode));
         } catch (NumberFormatException ne) {
            log.error(ne.getMessage(), ne);
         } catch (NoResultException e) {
            log.error(e.getMessage(), e);
         }
      }
      return visnList;
   }

   /**
    * retrieve list of facilities from VA_FACILITY table
    */
   public List<SelectItem> getFacilitiesList() {
      String visnId;
      if (facilitiesList == null) {
         facilitiesList = new ArrayList<SelectItem>();
      } else
         facilitiesList.clear();

      // make sure if no VISN selected the facilities list is empty
      if (selectedVisnId.equals(DEFAULT_SELECT_VALUE)) { // if no visn selected
                                                         // use default for
                                                         // facility
         facilitiesList.add(new SelectItem(DEFAULT_SELECT_VALUE, DEFAULT_SELECT_VALUE));
         return facilitiesList;
      } else
         visnId = selectedVisnId;

      int pos = visnId.indexOf('N');
      String visnNum = visnId.substring(pos + 1);
      if (visnNum.equals(DEFAULT_SELECT_VALUE))
         return facilitiesList; // cannot search on -select-

      // Invalid visn
      Matcher m = DIGITS.matcher(visnNum);
      if (!m.matches()) {
         return facilitiesList;
      }

      /**
       * DEF021536 - deactivate station 358 in FPPS and central server
       * Added "REROUTING_ALLOWED =1" to prevent users from querying
       * Station 358 and any other inactive stations.
       * 
       * modified by Ian Meinert
       */
      
      sql = "select * from VA_FACILITY where VISN_ID_CD = '" + visnNum + "' and VA_TYPE_CD ='FAC' and REROUTING_ALLOWED =1";
      try {
         Query query = em.createNativeQuery(sql);
         facilitiesList.clear();
         /* query the database and put the records into a temporary list. */
         List results = query.getResultList();
         it = results.iterator();
         /* loop through the results. */
         while (it.hasNext()) {
            Object[] row = (Object[]) it.next();
            // retrieve the facility ID and add to list
            facilitiesList.add(new SelectItem((String) row[0], (String) row[0]));
         }
      } catch (NoResultException nre) {
         facilitiesList.clear();
         facilitiesList.add(new SelectItem(DEFAULT_SELECT_VALUE, DEFAULT_SELECT_VALUE));
         log.error(nre.getMessage(), nre);
         return facilitiesList;
      } catch (Exception e) {
         log.error(e.getMessage(), e);
      }

      return facilitiesList;
   }

   /**
    * retrieve list of users from db
    */
   public List<SelectItem> getUserList() {
      if (userList == null) {
         userList = new ArrayList<SelectItem>();
      } else
         userList.clear();
      // this.userName=DEFAULT_SELECT_VALUE;
      if (facilityId.equals(DEFAULT_SELECT_VALUE)) { // if no facility selected
                                                     // use default for facility
         userList.add(new SelectItem(DEFAULT_SELECT_VALUE, DEFAULT_SELECT_VALUE));
         return userList;
      }
      sql = "select distinct APP_USER_ID from USER_ROLE_USAGES where VA_FACILITY_CD = '" + facilityId + "'";
      Iterator iterator = null;
      List list = null;
      Query query = null;
      // APP_USER_ID
      try {
         query = em.createNativeQuery(sql);
         // get list of APP_USER_IDs to query APP_USER table with
         /* query the database and put the records into a temporary list. */
         list = query.getResultList();
         iterator = list.iterator();
      } catch (NoResultException nre) {
         log.error(nre.getMessage(), nre);
         return null;
      } catch (Exception e) {
         log.error(e.getMessage(), e);
      }
      /* loop through the results. */
      while (iterator.hasNext()) {
         BigDecimal id = (BigDecimal) iterator.next(); // get APP_USER_ID
         String sqlSub = "select USER_NAME from APP_USER where APP_USER_ID = '" + id + "'";
         // get USER_NAME and add to pick list
         try {
            query = em.createNativeQuery(sqlSub);
            list = query.getResultList();
            Object row2 = (Object) list.get(0);
            userList.add(new SelectItem((String) row2, (String) row2));
         } catch (NoResultException nre) {
            log.error(nre.getMessage(), nre);
            return null;
         } catch (Exception e) {
            log.error(e.getMessage(), e);
         }
      }
      return userList;
   }

   public String getSelectedVisnId() {
      return selectedVisnId;
   }

   public void setSelectedVisnId(String selectedVisnId) {
      this.selectedVisnId = selectedVisnId;
   }

   public String getFacilityId() {
      return facilityId;
   }

   public void setFacilityId(String facilityId) {
      this.facilityId = facilityId;
   }

   public String getUserName() {
      return userName;
   }

   public void setUserName(String userName) {
      this.userName = userName;
   }

   public String getReason() {
      return reason;
   }

   public void setReason(String reason) {
      this.reason = reason;
   }

   public List<SelectItem> getRerouteReasons() {
      Iterator iterator = null;
      List list = null;
      Query query = null;
      String sqlStr = "select DESCRIPTION from FEE_CLAIM_ROUTINGS_REASON";
      if (rerouteReasons != null)
         return rerouteReasons;
      rerouteReasons = new ArrayList<SelectItem>();
      try {
         query = em.createNativeQuery(sqlStr);
         list = query.getResultList();
         iterator = list.iterator();
      } catch (NoResultException nre) {
         log.error(nre.getMessage(), nre);
         return null;
      } catch (Exception e) {
         log.error(e.getMessage(), e);
      }
      while (iterator.hasNext()) {
         String rReason = (String) iterator.next(); // get reason
         rerouteReasons.add(new SelectItem(rReason, rReason));
      }

      return rerouteReasons;
   }

   /**
    * Rerouting history information associated with the Claim.
    *
    * @return List of rerouting history information.
    */
   /*
   public List<ClaimReroutingHistoryEntity> getReroutingHistory() {
      return reroutingHistory;
   }
   */

   final String getServiceProvName(Set<ProviderInfo> providers) {
      for (ProviderInfo p : providers) {
         if ("SERVICE FACILITY".equals(p.getProviderType().getProviderTypeCd())
               || "SERVICE LOCATION".equals(p.getProviderType().getProviderTypeCd())) {
            return p.getLastName();
         }
      }
      return null;
   }

   final String getBillingProvName(Set<ProviderInfo> providers) {
      for (ProviderInfo p : providers) {
         if ("BILLING PROVIDER".equals(p.getProviderType().getProviderTypeCd())) {
            return p.getLastName();
         }
      }
      return null;
   }

   final String getVeteranName(Set<PersonInfo> persons) {
      for (PersonInfo pi : persons) {
         if ("PATIENT".equals(pi.getPersonType().getPersonTypeCd())) {
            return pi.getFirstName() + " " + pi.getLastName();
         }
      }
      return null;
   }

   public String getDestination() {
      return destination;
   }

   public void setDestination(String destination) {
      this.destination = destination;
   }

   /**
    * @return the comment
    */
   public String getComment() {
      return comment;
   }

   /**
    * @param comment
    *           the comment to set
    */
   public void setComment(String comment) {
      this.comment = comment;
   }

    private boolean transferToNonFPPS(Claim claim, String username, String visn, String facility, String reason, String comments) {
        Connection connection = null;
        CallableStatement cs = null;
        boolean status = true;
        try {
            InitialContext initialContext = new InitialContext();
            javax.sql.DataSource ds = (javax.sql.DataSource) initialContext
                    .lookup("java:/FPPS_OwnerDS");
            connection = ds.getConnection();

            cs = connection.prepareCall
                    ("{call fpps_owner.programtracking_reroute (?,?,?,?,?,?)}");
            cs.setLong(1, claim.getClaimIndex());
            cs.setString(2, username);
            cs.setString(3, visn);
            cs.setString(4, facility);
            cs.setString(5, reason);
            cs.setString(6, comments);
            cs.execute();
        } catch (Exception e) {
            log.fatal(e);
            facesMessages.addFromResourceBundle(
                    Severity.ERROR,
                    "gov.va.med.fee.Reroute.MSG_ERROR_REROUTE");
            status = false;
        } finally {
        	try {
            	if (cs != null) {
            		cs.close();
            	}
            	if (connection != null) {
            		connection.close();
            	}
        	} catch (Exception e) {
        		log.error(e);
        	}
        }

        return status;
    }

   /**
    * reroutes the claim to a selected user/facility/VISN
    */
   public String reroute() {

      Iterator iterator = null;
      List list = null;
      Query query = null;
      String privacyClaimsMsg = "";// string for privacy restricted claims
      String visnRestrictClaimsMsg = "";// string for visn xfer restricted
                                        // claims
      boolean hasPrivacyRestrict = false;// flag for indicating existence of
                                         // privacy restriction
      boolean hasVisnRestrict = false;// flag for indicating existence of visn
                                      // xfer restriction
      historyTableTitle = ""; // clear header for reroute history table
      // temporary store to avoid concurrent access error to primary claim list
      // this list must be clear b4 processing pending reroutes list
      deniedReroutes = new ArrayList<RerouteClaimListEntity>();
      if (!checkForm())
         return "error on reroute data entry";
      // Must iterate through all claims on reroute screen (they will all be
      // rerouted
      // to the same destination)
      // checkForm will check for required/valid VISN ID
      // set these values to empty strings so they don't show up as '-select-'

      // verify visn
      String longVisnId = this.selectedVisnId.substring(this.selectedVisnId.indexOf('N') + 1);
      Visn toVisn = getVISN(longVisnId);
      if (toVisn == null) {
         facesMessages.addFromResourceBundle(
               Severity.ERROR,
               "gov.va.med.fee.Reroute.INVALID_VISN", selectedVisnId);
         return "";
      }

      // verify facility
      if (this.facilityId.equals(DEFAULT_SELECT_VALUE))
         this.facilityId = "";
      if (!this.facilityId.equals("")) { //TDP -replace equality operator
         String sql = "select count(*) from VA_FACILITY where VISN_ID_CD = '"
               + longVisnId + "'" + " and VA_TYPE_CD ='FAC'"
               + " and VA_FACILITY_CD = '" + facilityId + "'";

         query = em.createNativeQuery(sql);
         BigDecimal count = (BigDecimal) query.getSingleResult();
         if (BigDecimal.ZERO.equals(count)) {
            facesMessages.addFromResourceBundle(
                  Severity.ERROR,
                  "gov.va.med.fee.Reroute.INVALID_FACILITY", facilityId);
            return "";
         }
      }

      // verify user
      if (this.userName.equals(DEFAULT_SELECT_VALUE))
         this.userName = "";
      if (this.userName != "") {
         String sql = "select count(*) from user_role_usages r, app_user u where r.va_facility_cd = '"
               + facilityId + "' and r.app_user_id = u.app_user_id and u.user_name='"
               + userName + "'";
         query = em.createNativeQuery(sql);
         BigDecimal count = (BigDecimal) query.getSingleResult();
         if (BigDecimal.ZERO.equals(count)) {
            facesMessages.addFromResourceBundle(
                  Severity.ERROR,
                  "gov.va.med.fee.Reroute.INVALID_USER", userName);
            return "";
         }
      }

      if (this.comment.length() > 250) {
          facesMessages.addFromResourceBundle(
                Severity.ERROR,
                "gov.va.med.fee.Reroute.REROUTE_COMMENTS_SIZE");
          return "";
       }

      FeeClaimRoutingsReason fcrReason = null;
      // perform a lookup in the routing reasons table..
      try {
    	 //this.reason = Clinic of jursidiction resides elsewhere
         query = em.createQuery("from fFeeClaimRoutingsReason r where r.transferReasonCd='" + this.reason + "'");
         fcrReason = (FeeClaimRoutingsReason) query.getSingleResult();
      } catch (NoResultException nre) {
         log.error("error:", nre);
          log.error("reason: {0}, not found in ClaimRoutingsReason table", this.reason);
      } catch (Exception ex) {
         log.error("error:", ex);
      }

      // Check if reroute is to a facility with FBCS implementation
      boolean nonFPPS = false;
      if (!this.facilityId.equals("")) { //TDP -replace equality operator 
        SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
        String today = dateFormat.format(new Date());
        String sqlToday = new StringBuilder("to_date('").append(today).append("','MM/DD/YYYY')").toString();
        sql = new StringBuilder("select va_imp_type_cd from fpps_owner.va_implementation where va_facility_cd = '")
                .append(facilityId)
                .append("' and va_imp_type_cd is not null")
                .append(" and start_date <= ").append(sqlToday)
                .append(" and (end_date is null or end_date >= ").append(sqlToday).append(")")
                .toString();
        query = em.createNativeQuery(sql);
        List type = query.getResultList();
        if (type != null && type.size() > 0) {
            // there shouldn't be more than one result
            String program = (String) type.get(0);
            if (program != null && program.trim().length() > 0) {
            	nonFPPS = true;
            }
        }
      }

      // do FPPS reroute
      Iterator reroutesIterator = reroutes.iterator();
      StringBuilder reroutedClaimsStr = new StringBuilder();
      while (reroutesIterator.hasNext()) {
         RerouteClaimListEntity claim2reroute = (RerouteClaimListEntity) reroutesIterator.next();
         Claim c = getClaim(claim2reroute.getClaimIndex());
         if (c == null) {
            facesMessages.addFromResourceBundle(
                  Severity.ERROR,
                  "gov.va.med.fee.Reroute.REROUTE_CLAIM_NOT_FOUND", claim2reroute.getClaimIndex());
            deniedReroutes.add(claim2reroute);
            continue;
         }

         if (nonFPPS && !"DENT".equals(c.getServiceType().getServiceTypeCd())) {
             if (transferToNonFPPS(c, appUser.getUserName(), longVisnId, facilityId, fcrReason.getTransferReasonCd(), comment)) {
            	  if (reroutedClaimsStr.length() != 0) {
                     reroutedClaimsStr.append(", ");
                  }
                  reroutedClaimsStr.append(c.getClaimIndex());
                  continue;
             } else {
            	 // get out of the loop to display error
            	 break;
             }
         }

         // MAKE SURE TO CHECK REROUTING_ALLOWED FLAG FOR THIS FACILITY!!
         // perform a lookup in the VISN table..
         // convert String selectedVisnId to a long value
         // get facility object associated with claim
         String targetVisn = "" + c.getVisn().getVisnIdCd();
         // check REROUTING_ALLOWED flag if the user is NOT a
         // FEE_VISN_PGM_MANAGER
         // then this check must be performed
         boolean canReroute = false;
         if (hasRole("FEE_VISN_PGM_MANAGER") || hasRole("ADMIN")) {
            canReroute = true;
         } else {
            if (c.getVaFacility() != null) {
               canReroute = c.getVaFacility().isReroutingAllowed();
            }
         }
         // get VISN id code and compare it to the VISN selected value
         // if they are not the same AND the REROUTING_ALLOWED flag == 0
         // -DISALLOW routing
         if (!longVisnId.equals(targetVisn) && !canReroute) {
            // identify claim and put in msg queue
            hasVisnRestrict = true;
            visnRestrictClaimsMsg += " " + c.getClaimIndex();
            deniedReroutes.add(claim2reroute);
            continue; // skip this claim and move on to next....
         }
         PersonInfo pi = getPersonInfo(c);
         if (pi == null) {
            facesMessages.addFromResourceBundle(
                  Severity.ERROR,
                  "gov.va.med.fee.Reroute.REROUTE_BENE_NOT_FOUND", c.getClaimIndex());
            deniedReroutes.add(claim2reroute);
            continue;
         }
         // VERIFY THAT TARGET USER IS ALLOWED TO VIEW CLAIM (re
         // PRIVACY_TO_PERSON table)
         if (!hasAccess(userName, pi.getPersonId())) {
            // identify claim and put in msg queue
            hasPrivacyRestrict = true;
            privacyClaimsMsg += " " + c.getClaimIndex();
            deniedReroutes.add(claim2reroute);
            continue; // skip this claim and move on to next...
         }
         FeeClaimRouting fcr = new FeeClaimRouting();
         fcr.setClaim(c);
         fcr.setTransferFromUser(c.getUserName());
         VaFacility vf = c.getVaFacility() == null ? new VaFacility() : c.getVaFacility();
         String vfc = vf.getVaFacilityCd() == null ? "" : vf.getVaFacilityCd();
         fcr.setTransferFromFacilityCd(vfc);
         fcr.setTransferFromVisnCd("" + c.getVisn().getVisnIdCd());
         // now reset Claim owner to new (rerouted to)
         if (this.userName.equals("") || this.userName.equals(DEFAULT_SELECT_VALUE))
            c.setUserName("");
         else
            c.setUserName(this.userName);
         c.setVaFacility(getVaFacility(this.facilityId)); // if no selection
                                                          // facility is set to
                                                          // null

         c.setVisn(toVisn);
         c.setClaimStatusCd("ESTABLISH");
         c.setModifiedBy(appUser.getUserName());

         Set<ClaimProcedure> claimProcs = c.getClaimProcedures();
         for (ClaimProcedure cp : claimProcs) {
            cp.setLineStatus("ESTABLISH");
         }

         // all claim (c) data now set to new owner
         fcr.setTransferToFacilityCd(this.facilityId);
         fcr.setTransferToUser(this.userName);
         fcr.setTransferToVisnCd(longVisnId);
         java.util.Date today = new java.util.Date();
         java.sql.Date sqlDate = new java.sql.Date(today.getTime());
         c.setDateModified(sqlDate);
         fcr.setTransferDate(sqlDate);
         fcr.setDateCreated(sqlDate); // this is the same as the xfer date??

         fcr.setComments(this.comment);
         fcr.setTransferInitator(appUser.getUserName());
         fcr.setCreatedBy(appUser.getUserName());

         fcr.setFeeClaimRoutingsReason(fcrReason);

         WsFacStatus wfs = null;
         try {
            query = em.createQuery("from fWsFacStatus s where s.claimIndex= " + c.getClaimIndex());
            wfs = (WsFacStatus) query.getSingleResult();
         } catch (NoResultException nre) {
                log.error("error:", nre);
                log.error("Claim index {0} not found in WS_FAC_STATUS table", + c.getClaimIndex());
         } catch (Exception ex) {
            log.error("error:", ex);
            log.error("Error on WS Facility Table query");
         }
         // wfs.setUserName(appUser.getUserName());
         wfs.setUserName(c.getUserName()); // SCOTT 11/05/2009
         wfs.setVaFacilityCd(this.facilityId);
         wfs.setClaimStatusCd("ESTABLISH");
         wfs.setDateInProcess(null);
         wfs.setVisnIdCd(c.getVisn().getVisnIdCd());
         // Update the provider lookup table with facilityData.
         WsProviderLu wProvider = null;
         try {
            query = em.createQuery("from fWsProviderLu s where s.claimIndex= " + c.getClaimIndex());
            wProvider = (WsProviderLu) query.getSingleResult();
         } catch (NoResultException nre) {
            log.fatal("no WS_PROVIDER_LU entry for on claim index: " + c.getClaimIndex());
         }
         if (wProvider != null) {
            wProvider.setVaFacilityCd(this.facilityId);
         }

         // build String of rerouted claims to display in confirmation msg
         if (reroutedClaimsStr.length() != 0) {
            reroutedClaimsStr.append(", ");
         }
         reroutedClaimsStr.append(c.getClaimIndex());
         try {
            em.persist(fcr);
            em.persist(wfs);
            em.persist(wProvider);
         } catch(EntityExistsException ee){
            log.error("error:", ee);
             log.error("Entity already exists in database");
         } catch (Exception ex) {
            log.error("error:", ex);
            log.error("Exception persisting reroute to database");
         }
         // after storing reroute record delete from reroutes list
         // reroutes.remove(claim2reroute);
      }// end while reroutesIterator
      // check denied reroutes and display appropriate error message(s)
      if (hasVisnRestrict) {
         facesMessages.addFromResourceBundle(
               Severity.ERROR,
               "gov.va.med.fee.Reroute.VISN_REROUTE_RESTRICTION", visnRestrictClaimsMsg);
      }
      if (hasPrivacyRestrict) {
         facesMessages.addFromResourceBundle(
               Severity.ERROR,
               "gov.va.med.fee.Reroute.PRIVACY_REROUTE_RESTRICTION", this.userName, privacyClaimsMsg);
      }

      if (reroutedClaimsStr.length() > 0) {
         facesMessages.addFromResourceBundle(
               Severity.INFO,
               "gov.va.med.fee.Reroute.REROUTE_SUCCESS", reroutedClaimsStr);
      }
      reroutes.clear(); // clear all claims from claims to reroute list
      reroutes = deniedReroutes; // repopulate with claims that did NOT get
                                 // rerouted
      reset();
      return "finished";
   }

   private void reset() {
      // reset destination criteria
      this.selectedVisnId = DEFAULT_SELECT_VALUE;
      this.facilityId = DEFAULT_SELECT_VALUE;
      this.userName = DEFAULT_SELECT_VALUE;
      this.reason = "Select a reason...";
      this.comment = "";
   }

   /**
    * returns a Claim object given a claim index
    *
    * @param ci
    * @return
    */
   public Claim getClaim(String ci) {
      Query q;
      Claim claim = null;
      try {
         q = em.createQuery("from fClaim c where c.claimIndex=" + ci);
         claim = (Claim) q.getSingleResult();
      } catch (NoResultException nre) {
         log.error("error:", nre);
         log.error("unable to find claim with index {0} for reroute", ci);
      } catch (Exception ex) {
         log.error(ex.getMessage(), ex);
      }
      return claim;
   }

   /**
    * given a claim returns a personInfo record based on claim's index
    *
    * @param claim
    * @return PersonInfo object
    */
   public PersonInfo getPersonInfo(Claim claim) {
      PersonInfo pi = null;
      Query q;
      try {
         q = em.createQuery("from fPersonInfo p where p.claim.claimIndex = '" + claim.getClaimIndex() + "'");
         pi = (PersonInfo) q.getSingleResult();
      } catch (NoResultException nre) {
         log.error(nre.getMessage(), nre);
         return null;
      } catch (Exception ex) {
         log.error(ex.getMessage(), ex);
         return null;
      }
      return pi;
   }

   /**
    * given a facility id returns a VA Facility object
    *
    * @param facId
    * @return
    */
   public VaFacility getVaFacility(String facId) {
      VaFacility vf = null;
      Query q;
      // perform a lookup in the VaFacility table..
      if (facId.equals("")) // claim should be routed to a VISN only
         return null;
      else {
         try {
            q = em.createQuery("from fVaFacility f where f.vaFacilityCd='" + this.facilityId + "'");
            vf = (VaFacility) q.getSingleResult();
         } catch (NoResultException nre) {
            log.error(nre.getMessage(), nre);
            return null;
         } catch (Exception ex) {
            log.error(ex.getMessage(), ex);
            return null;
         }
         return vf;
      }
   }

   public Visn getVISN(String vId) {
      Matcher m = DIGITS.matcher(vId);
      if (!m.matches()) {
         return null;
      }

      Query q;
      Visn v = null;
      try {
         q = em.createQuery("from fVisn v where v.visnIdCd=" + vId);
         v = (Visn) q.getSingleResult();
      } catch (NoResultException nre) {
         log.error(nre.getMessage(), nre);
         return null;
      } catch (Exception ex) {
         log.error(ex.getMessage(), ex);
         return null;
      }
      return v;
   }

   /**
    * determines if system user has privacy authorization to view claim as per
    * PRIVACY_TO_PERSON table
    */
   public boolean hasAccess(String userName, String bene) {
      try {
         Query query = em.createQuery("select count(*) from fPrivacyToPerson p where p.userName = '" + userName
               + "' AND p.personId = '" + bene + "'");
         Long p2p = (Long) query.getSingleResult();
         if (p2p != 0) {
            return false;
         }
      } catch (NoResultException nre) {
         log.error(nre.getMessage(), nre);
         return true;
      } catch (Exception ex) {
         log.error(ex.getMessage(), ex);
      }
      return true;
   }

   /**
    * Provide error checking for all data entry on form.
    *
    * @return true, if successful
    */
   private boolean checkForm() {
      boolean status = true;
      if (DEFAULT_SELECT_VALUE.equals(this.selectedVisnId) || this.selectedVisnId.equals("")) {
         facesMessages.addToControlFromResourceBundle(
               "paymentType",
               Severity.ERROR,
               "gov.va.med.fee.SELECTION_REQUIRED");
         status = false;
      }

      if (!status) {
         return false;
      }
      return status;
   }

   // VALIDATOR METHODS
   /**
    * validates VISN entry
    */
   public void validateVISN(FacesContext context, UIComponent component, Object obj) {
      String value = obj.toString();
      HtmlSelectOneMenu hsom = (HtmlSelectOneMenu) component;
      if (value.equals("-select-")) {
         hsom.setValid(false);
         FacesMessage message = new FacesMessage("Please Select a VISN.");
         context.addMessage(component.getClientId(context), message);
         StatusMessages.instance().addFromResourceBundle(Severity.ERROR, "gov.va.med.fee.Reroute.MSG_ERROR_REROUTE", "");
      }
   }

   public boolean hasRole(String role) {
      Set<UserRoleUsages> roles = this.appUser.getUserRoleUsages();
      Iterator iterator = roles.iterator();
      while (iterator.hasNext()) {
         UserRoleUsages userRoleUsages = (UserRoleUsages) iterator.next();
         AppRole appRole = userRoleUsages.getAppRole();
         if (appRole.getRoleName().equals(role))
            return true;
      }
      return false;
   }

   /**
    * {@inheritDoc}
    */
   // @Destroy
   // @Remove
   // public final void destroy() {
   // log.info("destroyed:" + this);
   // }

}
