package gov.va.med.fee.claim;

import static org.jboss.seam.ScopeType.SESSION;
import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.Claim;
import gov.va.med.domain.fee.ClaimProcedure;
import gov.va.med.domain.fee.FeeClaimPaymentLines;
import gov.va.med.domain.fee.FeeClaimPayments;
import gov.va.med.fee.Constants;
import gov.va.med.fee.sec.HomePageUserData;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.log.Log;



/**
 * ClaimSearchBean class.
 */

@Stateful
@Name("unmatchedPaymentsRecon")
@Scope(SESSION)
public class UnmatchedPaymentsReconBean implements UnmatchedPaymentsRecon, Serializable  {

	   /** The app user. */
	   @In(value = "appUser", required = true, scope = ScopeType.SESSION)
	   private AppUser appUser;
	   
	   /** The homePageUserData bean. */
	   @In(value = "homePageUserData", required = false, scope = ScopeType.SESSION)
	   private HomePageUserData homePageUserData;	   

	   /** The em. */
	   @PersistenceContext(unitName = "FPPS_Owner")
	   private EntityManager em;

       /** The log. */
       @Logger
       private Log log;

       private String claimStatus;
       
       private FeeClaimPayments payment;
       
       private String paymentId;
       
       // This is used to avoid a "0" appearing on the Claim Index when the user enters the page
       
       private String searchClaimIndex;
       
       private Claim searchLine;
       private Claim searchLine2;
       
       private List<Claim> searchResults;
       
       private String selectedClaimIndex;
       
       private String serviceType = null;
       

//       @PersistenceContext(unitName = "FPPS_Owner")
       private Claim unmatchedClaim;
	
  	   public void clear(){
            searchClaimIndex = null;
            searchResults = null;
            serviceType = null;
            claimStatus = null;
            searchLine = new Claim();
            searchLine2 = new Claim();
	   }

  	   @Remove
	   public final void destroy(){
	   }
  	   
  	   private FeeClaimPayments fillPayment(FeeClaimPayments oldPayment, FeeClaimPayments newPayment){
  		   String newGuid = UUID.randomUUID().toString().toUpperCase();
  		   
  		   newPayment.setBilledAmount(oldPayment.getBilledAmount());
  		   newPayment.setCancellationCode(oldPayment.getCancellationCode());
  		   newPayment.setCancellationDate(oldPayment.getCancellationDate());
  		   newPayment.setCreatedBy(appUser.getUserName());
  		   newPayment.setCoveredDays(oldPayment.getCoveredDays());
  		   newPayment.setDateCreated(new Date(System.currentTimeMillis()));
  		   newPayment.setDateModified(new Date(System.currentTimeMillis()));
  		   newPayment.setDiagnosisRelatedGroupCode(oldPayment.getDiagnosisRelatedGroupCode());
  		   newPayment.setDiagnosisRelatedGroupWeight(oldPayment.getDiagnosisRelatedGroupWeight());
  		   newPayment.setDisbursedAmount(oldPayment.getDisbursedAmount());
  		   // fee Claim Payment Id should be set by the persist method of em???
  		   newPayment.setFeeClaimPaymentId(newGuid);
  		   newPayment.setClaimIndex(Long.parseLong(selectedClaimIndex));
  		   newPayment.setFeeInvoiceDate(oldPayment.getFeeInvoiceDate());
  		   newPayment.setFeeInvoiceNumber(oldPayment.getFeeInvoiceNumber());
  		   // not including feePaymentErrors as we should be fixing them before this point
  		   newPayment.setInterestAmount(oldPayment.getInterestAmount());
  		   newPayment.setPaidAmount(oldPayment.getPaidAmount());
  		   newPayment.setPeriodEndDate(oldPayment.getPeriodEndDate());
  		   newPayment.setPeriodStartDate(oldPayment.getPeriodStartDate());
//  		   newPayment.setResolutionStatusCode(oldPayment.getResolutionStatusCode());
  		   newPayment.setStationNumber(oldPayment.getStationNumber());
  		   newPayment.setTransactionTypeCode(oldPayment.getTransactionTypeCode());  		   

  		   Set <FeeClaimPaymentLines> newLines = fillPaymentLines(oldPayment.getFeeClaimPaymentLines(), newPayment);
  		   
  		   newPayment.setFeeClaimPaymentLines(newLines);
  		   
  		   
  		   return newPayment;
  	   }
  	   
  	   private Set <FeeClaimPaymentLines> fillPaymentLines(Set <FeeClaimPaymentLines> oldLines, FeeClaimPayments fcp){
  		   // Need to correctly creat Lines so as to have new feeClaimPaymentLinesIDs
  		   
  		   HashSet <FeeClaimPaymentLines> newLines = new HashSet<FeeClaimPaymentLines>();
  		   Iterator i = oldLines.iterator();
  		   while(i.hasNext() ){
  			 FeeClaimPaymentLines fcpl = (FeeClaimPaymentLines) i.next();
    		   FeeClaimPaymentLines newFcpl = new FeeClaimPaymentLines();
    		   newFcpl.setFeeClaimPaymentLineId(UUID.randomUUID().toString().toUpperCase());
     		   newFcpl.setLineItemSequence(fcpl.getLineItemSequence());
     		   newFcpl.setServiceDate(fcpl.getServiceDate());
     		   newFcpl.setServiceCodeType(fcpl.getServiceCodeType());
     		   newFcpl.setServicePaidCode(fcpl.getServicePaidCode());
     		   newFcpl.setServiceModifierCode1(fcpl.getServiceModifierCode1());
     		   newFcpl.setServiceModifierCode2(fcpl.getServiceModifierCode2());
     		   newFcpl.setServiceModifierCode3(fcpl.getServiceModifierCode3());
     		   newFcpl.setServiceModifierCode4(fcpl.getServiceModifierCode4());
     		   newFcpl.setBilledAmount(fcpl.getBilledAmount());
     		   newFcpl.setPaidAmount(fcpl.getPaidAmount());
     		   newFcpl.setUnitsPaid(fcpl.getUnitsPaid());
     		   newFcpl.setRemittanceRemarkCode(fcpl.getRemittanceRemarkCode());
     		   newFcpl.setAdjustmentAmount1(fcpl.getAdjustmentAmount1());
     		   newFcpl.setAdjustmentGroupCode1(fcpl.getAdjustmentGroupCode1());
     		   newFcpl.setAdjustmentReasonCode1(fcpl.getAdjustmentReasonCode1());
     		   newFcpl.setAdjustmentAmount2(fcpl.getAdjustmentAmount2());
     		   newFcpl.setAdjustmentGroupCode2(fcpl.getAdjustmentGroupCode2());
     		   newFcpl.setAdjustmentReasonCode2( fcpl.getAdjustmentReasonCode2());
     		   newFcpl.setTypeOfPayment(fcpl.getTypeOfPayment());
     		   newFcpl.setCheckNumber(fcpl.getCheckNumber());
     		   newFcpl.setCheckDate(fcpl.getCheckDate());
     		   newFcpl.setCreatedBy(appUser.getUserName());
     		   newFcpl.setDateCreated(new Date());
     		   newFcpl.setModifiedBy(appUser.getUserName());
     		   newFcpl.setDateModified(new Date());
     		   newFcpl.setFeeClaimPayments(fcp);
  			   newLines.add(newFcpl);
  		   }
  		   return newLines;
  	   }
  	   
  	   public Claim getClaim(){
  		   // Having an issue wiht lazily loading so will be using query
  		   // Might be as simple as @Persistence but for now - doing this.
  		   Query q = em.createQuery("Select cl from fClaim cl where cl.claimIndex = '" + unmatchedClaim.getClaimIndex() + "'");
  		   unmatchedClaim = (Claim) q.getSingleResult();
  		   return unmatchedClaim;
  	   }

  	 public List<ClaimProcedure> getClaimProcedures(){
  		  LinkedList<ClaimProcedure> claimProcedureList = new LinkedList<ClaimProcedure>(unmatchedClaim.getClaimProcedures());
  		  LinkedList<ClaimProcedure> finalClaimProcedureList = new LinkedList<ClaimProcedure>();
  		  finalClaimProcedureList.clear();
  		  int listSize = claimProcedureList.size();
  		  Object[] claimProcedureArray = claimProcedureList.toArray();
  		  
  		  for(long i =0; i < listSize; i++){
  			  for(int j = 0; j < listSize; j++){
  				  if( (((ClaimProcedure) claimProcedureArray[j]).getClaimProcedureSeq()) == (i+1)){
  					finalClaimProcedureList.add((ClaimProcedure) claimProcedureArray[j]);
					   break;
				   }
  			  }
  		  }
  		  
  		  return finalClaimProcedureList;
  	 }
  	 
  	 public String getClaimStatus(){
  		 return claimStatus;
  	 }
  	 
  	 public List<FeeClaimPaymentLines> getFeeClaimPaymentLines(){
  		   // This convoluted process below is for sorting the Set and for dealing with data type manipulation.
  		 
  		   LinkedList<FeeClaimPaymentLines> paymentLinesList = new LinkedList<FeeClaimPaymentLines>(payment.getFeeClaimPaymentLines());
  		   int listSize = paymentLinesList.size();
  		   Object[] paymentLineArray;  
		   
		   paymentLineArray = paymentLinesList.toArray();
		   // Need to initialize and then clear the list.
		   LinkedList<FeeClaimPaymentLines> finalPaymentLinesList = new LinkedList<FeeClaimPaymentLines>(paymentLinesList);
		   finalPaymentLinesList.clear();
		   for(int i = 0; i < listSize; i++){
			   for(int j = 0; j < listSize; j++){
				   if( ( (FeeClaimPaymentLines) paymentLineArray[j]).getLineItemSequence().equals("ALL") ||
				   		Integer.parseInt( ( (FeeClaimPaymentLines) paymentLineArray[j]).getLineItemSequence()) == (i+1)){
					   finalPaymentLinesList.add((FeeClaimPaymentLines) paymentLineArray[j]);
					   break;
				   }
			   }
		   }
 		 	return finalPaymentLinesList;
  	 }
  	 
  	   public FeeClaimPayments getPayment(){
  		   Query q = em.createQuery("Select fcp from fFeeClaimPayments fcp where fcp.feeClaimPaymentId = '" + paymentId + "'");
  		   payment =  (FeeClaimPayments) q.getSingleResult();
  		   return payment;
  	   }
  	 
  	   public String getSearchClaimIndex(){
  		   return searchClaimIndex;
  	   }
  	   
  	   public Claim getSearchLine(){
  		   return searchLine;
  	   }
  	   public Claim getSearchLine2(){
		   return searchLine2;
	   }
  	   
  	   public List<Claim> getSearchResults(){
  		   	return searchResults;
  	   }
  	   
  	   public String getServiceType(){
  		   	return serviceType;
  	   }
  	   
  	   public String matchClaim(){
  		   String success = Constants.UNMATCHED_SUCCESS;
  		   
  		   try{
  			   
  			   /***
  			    * This needs to be reworked to handle to insert a new element and change the status.
  			    *
  			    * 
  			   */
  			   
  			   
	  		   Query q = em.createQuery("Select fcp from fFeeClaimPayments fcp where fcp.feeClaimPaymentId = '" + paymentId + "'");
	  		   payment =  (FeeClaimPayments) q.getSingleResult();
	  		   FeeClaimPayments newPayment = new FeeClaimPayments();
	  		   newPayment = fillPayment(payment, newPayment);
	  		   em.persist(newPayment);
	  		   payment.setResolutionStatusCode(Constants.RESOLVED);
	  		   newPayment.setResolutionStatusCode(Constants.NEW_STATUS);
            clear();
  		   } catch(Exception e){
  			   log.error("matchClaim Error: {0} ", e, e.getMessage());
  			   success = Constants.FAILURE;
  		   }
  		   return success;
  	   }
  	   
  	   public List<Claim> search(){
    	   String activeUserStations = homePageUserData.getActiveStations().toString();
  		   Date beginServiceDate = searchLine.getBeginServiceDate();
  		   String claimInd = searchClaimIndex;
  		   Date endServiceDate = searchLine2.getBeginServiceDate();
   	       String privacyRestrictions = homePageUserData.getPrivacyRestrictions();
  		   BigDecimal totalBilledCharge = searchLine.getTotalBilledCharge();
    	   String visns = homePageUserData.getActiveVisns();
  		   
	       activeUserStations = activeUserStations.replace('[', '\'');
	       activeUserStations = activeUserStations.replace(']', '\'');
	       activeUserStations = activeUserStations.replace(", ", "\', \'");
    	   
  		   String sqlStr = "Select cl from fClaim cl";
  		   if(!privacyRestrictions.equalsIgnoreCase("")){
  			  sqlStr += ", fPersonInfo pi";   
  		   }
  			sqlStr +="  where rownum < 101"
  			   			+ "   AND (cl.vaFacility IN (" + activeUserStations + ") ";
            if (!visns.equalsIgnoreCase("")){
                sqlStr+=" OR cl.visn IN("+visns+")";
            }
           sqlStr+=" ) ";

           if(!privacyRestrictions.equalsIgnoreCase("")){
               sqlStr += "   AND pi.personId NOT IN (" + privacyRestrictions + ") "
               		+ " AND pi.claim.claimIndex = cl.claimIndex ";
           }
  		   
  		   if(totalBilledCharge != null){
  			   sqlStr += " and cl.totalBilledCharge = " + totalBilledCharge;
  		   }
  		   if(claimInd != null && !claimInd.equals("") && !claimInd.equals("0")){
  			   sqlStr += " and cl.claimIndex like '" + claimInd + "'";
  		   }
  		   
  		   
  		   
  		   
  		   
  		   /* Append the lower claim date range to the SQL statement */
   	   if (beginServiceDate != null) {
   	      sqlStr += " AND cl.beginServiceDate >= TO_DATE('" + String.format("%1$tm/%1$td/%1$tY", beginServiceDate) 
                   + "', 'MM/dd/yyyy')";
   	   }
  		   
   	   /* Append the lower claim date range to the SQL statement */
   	   if (endServiceDate != null) {
   	      sqlStr += " AND cl.beginServiceDate <= TO_DATE('" + String.format("%1$tm/%1$td/%1$tY", endServiceDate) 
                   + "', 'MM/dd/yyyy')";
   	   }
  		   
  		   if(serviceType != null && !serviceType.equals("") && !serviceType.equals("%") ){
  			   sqlStr += " and cl.serviceType.serviceTypeCd = '" + serviceType + "'";
  		   }
  		   if(claimStatus != null && !claimStatus.equals("") && !claimStatus.equals("%") ){
  			   sqlStr += " and cl.claimStatusCd = '" + claimStatus + "'";
  		   }
  		   
  		   log.debug("***********sqlStr: {0} ", sqlStr);	
  		   Query q = em.createQuery(sqlStr);
  		   searchResults = q.getResultList();
  		   return searchResults;
  	   }
  	   public String setClaim(String claimIndex){
  		   selectedClaimIndex = claimIndex;
  		   Query q = em.createQuery("Select cl from fClaim cl where cl.claimIndex = '" + claimIndex + "'");
  		   unmatchedClaim = (Claim) q.getSingleResult();
  		   return Constants.UNMATCHED_CLAIM_SUCCESS;
  	   }
  	   
  	   public void setClaimStatus(String stat){
  		   claimStatus = stat;
  	   }  	   
  	   	
  	   
  	   public String setPayment(String pid){
  		     paymentId = pid;
           clear(); // clear states so can start a new search
  		     return Constants.UNMATCHED_SUCCESS;
  	   }
  	   
  	   public void setSearchClaimIndex(String ind){
  		   searchClaimIndex = ind;
  	   }
  	   
  	   public void setSearchLine(String pid){
  	   }
  	   
  	   public void setSearchLine(Claim fcp){
  		   searchLine = fcp;
  	   }
  	   
  	   public void setSearchLine2(Claim fcp){
		   searchLine2 = fcp;
	   }
  	   
  	   public void setServiceType(String servType){
  		   serviceType = servType;
  	   }  	     	  
 	   
  	   public UnmatchedPaymentsReconBean(){
  		   searchLine = new Claim();
  		   searchLine2 = new Claim();
  	   }
  	   
}
