package gov.va.med.fee.claim;

import static org.jboss.seam.ScopeType.SESSION;
import gov.va.med.domain.fee.AdjustmentReason;
import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.Claim;
import gov.va.med.domain.fee.ClaimProcedure;
import gov.va.med.domain.fee.ClaimQueue;
import gov.va.med.domain.fee.FeeClaimRouting;
import gov.va.med.domain.fee.FeeClaimRoutingsReason;
import gov.va.med.domain.fee.HealthcareInfoCode;
import gov.va.med.domain.fee.PayerInfo;
import gov.va.med.domain.fee.PersonInfo;
import gov.va.med.domain.fee.ProviderInfo;
import gov.va.med.domain.fee.RevenueCode;
import gov.va.med.domain.fee.ServiceType;
import gov.va.med.domain.fee.ToothInfo;
import gov.va.med.fee.report.PrintClaims;
import gov.va.med.fee.sec.HomePageUserData;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;



/**
 * ViewClaimBean class.Implements ViewClaim interface
 * Edited by Bill Blackmon, 4/22/3013; original author unknown.
 * @version R1.25.1
 * HAPE POR VA DO# 118-11-D-1009, TO#118-1009-0001; HARRIS
 */
@Stateful
@Name("viewClaim")
@Scope(SESSION)
public class ViewClaimBean implements ViewClaim {

   /** The claimIndex. */
   private long viewClaimIndex;

   /**
    * Gets the claimIndex parameter.
    *
    * @return the claimIndex.
    */
   public final long getViewClaimIndex() {
      return this.viewClaimIndex;
   }

   /**
    * Sets the claimIndex.
    *
    * @param claimIndex
    *           the claimIndex to set.
    */
   public final void setViewClaimIndex(long viewClaimIndex) {
      this.viewClaimIndex = viewClaimIndex;
   }

   /** The log. */
   @SuppressWarnings("unused")
   @Logger
   private Log log;

   /** The em. */
   @PersistenceContext(unitName = "FPPS_Owner", type=PersistenceContextType.EXTENDED)
   private EntityManager em;

   /** The app user. */
   @In(value = "appUser", required = true, scope = ScopeType.SESSION)
   private AppUser appUser;

   /** The homePageUserData bean. */
   @In(value = "homePageUserData", required = false, scope = ScopeType.SESSION)
   private HomePageUserData homePageUserData;

   /** The SQL query for searching claims. */
   private String sql = "SELECT c.claim_index ";

   /** The PrintClaims object. */
   // @In(value = "printClaims", required = true, scope = ScopeType.SESSION)
   @In(value = "printClaims", create = true)
   private PrintClaims printClaims;

   @Out(scope = ScopeType.SESSION)
   List<Long> claimsToDisapprove = new ArrayList<Long>();

   @In(scope = ScopeType.SESSION, required = false)
   @Out(scope = ScopeType.SESSION, required = false)
   private List<Long> claimsToReroute;


   /** the ClaimPaymentBean instance **/
   private ClaimPaymentBean claimPaymentBean;



   /** the Claim instance. */
   private Claim claim;
   /** the PersonInfo instance. */
   private PersonInfo person;
   /** the ProviderInfo serviceProvider instance. */
   private ProviderInfo serviceProvider;
   /** the ProviderInfo billingProvider instance. */
   private ProviderInfo billingProvider;
   /** the ProviderInfo paytoProvider instance. */
   private ProviderInfo paytoProvider;
   /** the ProviderInfo attendingPhysician instance. */
   private ProviderInfo attendingPhysician;
   /** the FeeClaimRouting instance. */
   private FeeClaimRouting routing;
   /** the Rerouting History List */
   private List<ClaimReroutingHistoryEntity> reroutingHistory = new ArrayList<ClaimReroutingHistoryEntity>();
   /** the Rejection History List */
   private List<ClaimRejectionHistoryEntity> rejectionHistory = new ArrayList<ClaimRejectionHistoryEntity>();
   /** the coveredDays string instance. */
   /** the Diagnosis Codes List */
   private List<DiagnosisCodesEntity> diagnosisCodes = new ArrayList<DiagnosisCodesEntity>();


   public static enum DiagCodeSortOrder { BK, BF, BJ, PR, BN, UNKNOWN }

   /** the coveredDays string instance. */
   private String coveredDays;
   /** the totalClaimAmountPaid instance. */
   private BigDecimal totalClaimAmountPaid;

   /** the line items collection for a claim */
   private List<LineItemEntity> claimLineItems = new ArrayList<LineItemEntity>();
   /** the Provider DRG string instance. */
   private String providerDrg;
   /** the Secondary PayerInfo instance */
   private PayerInfo secondaryPayerInfo;
   /** the Tertiary PayerInfo instance */
   private PayerInfo tertiaryPayerInfo;
   /** flag indicating if a claim can be disapproved */
   private boolean notReconciled;



   /** ViewClaimBean constructor. */
   public ViewClaimBean() {
      claim = new Claim();
      person = new PersonInfo();
      serviceProvider = new ProviderInfo();
      billingProvider = new ProviderInfo();
      paytoProvider = new ProviderInfo();
      attendingPhysician = new ProviderInfo();
      routing = new FeeClaimRouting();
      coveredDays = "";
      providerDrg = "";
      secondaryPayerInfo = new PayerInfo();
      tertiaryPayerInfo = new PayerInfo();

      claimPaymentBean = new ClaimPaymentBean();
   }


	/**
 * @return the claimPaymentBean
 */
public final ClaimPaymentBean getClaimPaymentBean() {
	return claimPaymentBean;
}



   /**
    * Gets the Claim.
    *
    * @return the Claim.
    */
   public final Claim getClaim() {
      return claim;
   }


   /**
    * Gets the Person associated with the Claim.
    *
    * @return the Person.
    */
   public final PersonInfo getPerson() {
      return person;
   }

   /**
    * Gets the Service Provider associated with the Claim.
    *
    * @return the serviceProvider.
    */
   public final ProviderInfo getServiceProvider() {
      return serviceProvider;
   }

   /**
    * Gets the Billing Provider associated with the Claim.
    *
    * @return the billingProvider.
    */
   public final ProviderInfo getBillingProvider() {
      return billingProvider;
   }

   /**
    * Gets the Payto Provider associated with the Claim.
    *
    * @return the paytoProvider.
    */
   public final ProviderInfo getPaytoProvider() {
      return paytoProvider;
   }

   /**
    * Gets the Attending Physician Provider associated with the Claim.
    *
    * @return the attendingPhysician.
    */
   public final ProviderInfo getAttendingPhysician() {
      return attendingPhysician;
   }

   /**
    * Routing information associated with the Claim.
    *
    * @return Routing information.
    */
   public final FeeClaimRouting getRouting() {
      return routing;
   }

   /**
    * Rerouting history information associated with the Claim.
    *
    * @return List of rerouting history information.
    */
   public final List<ClaimReroutingHistoryEntity> getReroutingHistory() {
      return reroutingHistory;
   }

   /**
    * Rejection history information associated with the Claim.
    *
    * @return List of rejection history information.
    */
   public final List<ClaimRejectionHistoryEntity> getRejectionHistory() {
      return rejectionHistory;
   }

   /**
    * Number of covered days associated with the Claim.
    *
    * @return coveredDays information.
    */
   public final String getCoveredDays() {
      return coveredDays;
   }

   /**


/**
    * Amount paid associated with the Claim.
    *
    * @return totalClaimAmountPaid.
    */
   public final BigDecimal getTotalClaimAmountPaid() {
      return totalClaimAmountPaid;
   }

   /**
    * Diagnosis Codes information associated with the Claim.
    *
    * @return List of diagnosis codes information.
    */
   public final List<DiagnosisCodesEntity> getDiagnosisCodes() {
      return diagnosisCodes;
   }

   /**
    * Claim line items information associated with the Claim.
    *
    * @return claimLineItems List of information.
    */
   public final List<LineItemEntity> getClaimLineItems() {
      return claimLineItems;
   }

   /**
    * Provider DRG associated with the Claim.
    *
    * @return providerDrg information.
    */
   public final String getProviderDrg() {
      return providerDrg;
   }

   /**
    * Secondary Payer Info associated with the Claim.
    *
    * @return secondaryPayerInfo information.
    */
   public final PayerInfo getSecondaryPayerInfo() {
      return secondaryPayerInfo;
   }

   /**
    * Tertiary Payer Info associated with the Claim.
    *
    * @return tertiaryPayerInfo information.
    */
   public final PayerInfo getTertiaryPayerInfo() {
      return tertiaryPayerInfo;
   }


	public final String doSearch() {
      return findClaimByClaimIndex(this.viewClaimIndex);
   }





	/**
	 * This method is called when I Claim is found and selected. All related information is retrieved.
	 *
	 */


   @SuppressWarnings("unchecked")
public final String findClaimByClaimIndex(long ci) {


// System.out.println("ViewClaimBean.findClaimByClaimIndex() TOP - claim index is: " + ci);





      log.debug("called ViewClaimBean.findClaimByClaimIndex( {0} )", ci);

      Query q;
      List<?> l;
      Iterator<?> it;

      String stations = homePageUserData.getActiveStations().toString();
      stations = stations.replace('[', '\'');
      stations = stations.replace(']', '\'');
      stations = stations.replace(", ", "\', \'");

      String visns = homePageUserData.getVisnSelector();

      String ret;
      ret = "findClaimByClaimIndexFAIL";
      this.viewClaimIndex = ci;

      this.claim.setClaimIndex(this.viewClaimIndex);

      coveredDays = "";
      this.attendingPhysician = null;
      String privacyRestrictions = homePageUserData.getPrivacyRestrictions();


      sql = "select  c.hero_flag, c.service_type_cd,  c.patient_control_number, c.claim_status_cd, "
              + " c.prior_auth_nbr,  c.admission_date,  c.discharge_date,  c.begin_service_date, "
              + " c.end_service_date,  ((c.end_service_date - c.begin_service_date)+1) AS covered, "	// rows[1-10]
              // row[11]
              + " (SELECT pro.first_name      "
              + "  FROM provider_info pro "
              + "  WHERE pro.provider_type_cd='ATTENDING PHYSICIAN'   "
              + "  AND pro.claim_index='" + this.viewClaimIndex + "' "
              + "  AND pro.claim_proc_id is null) as attendingFN, "
              // row[12]
              + " (SELECT pro.last_name "
              + "  FROM provider_info pro "
              + "  WHERE pro.provider_type_cd='ATTENDING PHYSICIAN' "
              + "  AND pro.claim_index='" + this.viewClaimIndex + "' "
              + "  AND pro.claim_proc_id is null) as attendingLN, "
              // row[13]
              + " (SELECT pro.middle_name "
              + "  FROM provider_info pro "
              + "  WHERE pro.provider_type_cd='ATTENDING PHYSICIAN' "
              + "  AND pro.claim_index='" + this.viewClaimIndex + "' "
              + "  AND pro.claim_proc_id is null) as attendingMN, "
              // row[14]
              + " (SELECT pro.name_suffix "
              + "  FROM provider_info pro "
              + "  WHERE pro.provider_type_cd='ATTENDING PHYSICIAN' "
              + "  AND pro.claim_index='" + this.viewClaimIndex + "' "
              + "  AND pro.claim_proc_id is null) as attendingSF, "
              + " c.ambulance_transport_text, "


// HERE - changed 7/16 per Ken
           // row[15]
              + " (Select sum(cp.paid_amount) as amtpaid from claim_payment cp where cp.claim_index = " + this.viewClaimIndex + "), "



/**
 * Select sum(paid_amount) as  amtpaid from claim_payment where claim_index = xxx
 */

 /**
  * Original - 7/16
  */
			  // row[15]
  /**
			  + " (SELECT SUM(cr.paid_amount) "
			  + "  FROM claim_reconciliation cr "
			  + "  WHERE cr.claim_index='" + this.viewClaimIndex + "') as amtpaid, "
**/
              // row[16]
              // WAS row[23]
              + " (select t.hi01proc_cd "
              + "  FROM healthcare_info_code t "
              + "  WHERE t.hi01cd_list_qual='DR ' "
              + "  AND t.claim_index=" + this.viewClaimIndex + ") as providerDRG, "

              // row[17]
              // WAS row[24]
              + " c.va_imp_type_cd"

              + " FROM claims c, person_info p "
              + " WHERE c.claim_index = p.claim_index(+) "
              + " AND c.claim_index=" + this.viewClaimIndex + " "

             + " AND (c.va_facility_cd IN  (" + stations + ") ";

              sql += visns;
              sql += " ) ";

              if(!privacyRestrictions.equalsIgnoreCase("")){
                  sql += "   AND p.person_id NOT IN (" + privacyRestrictions + ") ";
              }


// System.out.println("ViewClaimBean.findClaimByClaimIndex() - PRO info. - SQL String is: " + sql);


      q = em.createNativeQuery(sql);
      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      if (!it.hasNext()) {
         return "findClaimByClaimIndexFAIL";
      }

      /* loop through the results. */


// Set up Claim information - does this EVER run more than once?
      while (it.hasNext())
      {

		         ret = "findClaimByClaimIndexSUCCESS";
		         Object[] row = (Object[]) it.next();

		         Claim c = new Claim();


				 this.claimPaymentBean = new ClaimPaymentBean(this.viewClaimIndex, em);

		         Character hf = (Character) row[0];
		         c.setHeroFlag(hf.charValue());

		         ServiceType claimServiceType = new ServiceType();
		         c.setServiceType(claimServiceType);
		         String st = (String) row[1];
		         c.getServiceType().setServiceTypeCd(st);

		         c.setPatientControlNumber((String) row[2]);
		         c.setClaimStatusCd((String) row[3]);
		         c.setPriorAuthNbr((String) row[4]);
		         c.setAdmissionDate((Date) row[5]);
		         c.setDischargeDate((Date) row[6]);
		         c.setBeginServiceDate((Date) row[7]);
		         c.setEndServiceDate((Date) row[8]);
		         BigDecimal cd = (BigDecimal) row[9];
		         coveredDays = cd != null ? cd.toString() : "";

		         ProviderInfo a = new ProviderInfo();
		         a.setFirstName((String) row[10]);
		         a.setLastName((String) row[11]);
		         a.setMiddleName((String) row[12]);
		         a.setNameSuffix((String) row[13]);

		         c.setAmbulanceTransportText((String) row[14]);

// HERE - see above for row[15] payment SQL
		         totalClaimAmountPaid = (BigDecimal) row[15];

		         // Provider Info
		         this.attendingPhysician = a;

	 			if(row[17] != null) {
					 c.setVaImpTypeCd((String) row[17]);
				}

				c.setClaimIndex(this.viewClaimIndex);
		        this.claim = c;

      }		// end while loop




      q = em.createNativeQuery("select count(*) from claim_proc_recon cpr where cpr.claim_index = " + this.viewClaimIndex + " ");
      BigDecimal count = (BigDecimal) q.getSingleResult();
      this.notReconciled = count.intValue() > 0 ? false : true;

      this.person = null;

// GET PERSON INFORMATION.....
      sql = " SELECT  p.person_id,  p.first_name,  p.last_name,  p.middle_name,  p.address_line1, "
            + " p.address_line2,  p.city,  p.state,  p.zip_code,  p.birth_date,  p.gender "
            + " FROM person_info p  WHERE claim_index=" + this.viewClaimIndex + "";

      q = em.createNativeQuery(sql);
      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      /* loop through the results. */
      while (it.hasNext()) {

         Object[] row = (Object[]) it.next();

         PersonInfo per = new PersonInfo();

         per.setPersonId((String) row[0]);
         per.setFirstName((String) row[1]);
         per.setLastName((String) row[2]);
         per.setMiddleName((String) row[3]);
         per.setAddressLine1((String) row[4]);
         per.setAddressLine2((String) row[5]);
         per.setCity((String) row[6]);
         per.setState((String) row[7]);
         per.setZipCode((String) row[8]);
         per.setBirthDate((Date) row[9]);
         per.setGender((String) row[10]);

         this.person = per;

      }

      this.serviceProvider = null;

      String providerTypeCd = "SERVICE FACILITY";
      if("DENT".equalsIgnoreCase(this.claim.getServiceType().getServiceTypeCd()))
      {
    	  providerTypeCd = "SERVICE LOCATION";
      }

      sql = " SELECT  p.last_name,  p.address_line1,  p.address_line2,  p.city,  p.state, "
            + " p.zip_code,  p.phone_number,  p.provider_tin,  p.contact_name,  p.fax_number,  p.e_mail, "
            + " p.provider_type_cd, p.provider_npi  "
            + "FROM provider_info p  "
            + "WHERE claim_index=" + this.viewClaimIndex + " "
            + " AND (p.provider_type_cd='"+providerTypeCd+"') ";


      q = em.createNativeQuery(sql);
      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      /* loop through the results. */
      while (it.hasNext()) {

         Object[] row = (Object[]) it.next();

         ProviderInfo sp = new ProviderInfo();

         sp.setLastName((String) row[0]);
         sp.setAddressLine1((String) row[1]);
         sp.setAddressLine2((String) row[2]);
         sp.setCity((String) row[3]);
         sp.setState((String) row[4]);
         sp.setZipCode((String) row[5]);
         sp.setPhoneNumber((String) row[6]);
         sp.setProviderTin((String) row[7]);
         sp.setContactName((String) row[8]);
         sp.setFaxNumber((String) row[9]);
         sp.setEMail((String) row[10]);
         sp.setProviderNpi((String) row[12]);

         this.serviceProvider = sp;

      }

      this.paytoProvider = null;

      // set up payto provider
      sql = " SELECT  p.last_name,  p.address_line1, p.address_line2,  p.city,  p.state, "
              + " p.zip_code,  p.phone_number,  p.provider_tin,  p.contact_name,  p.fax_number,  p.e_mail, "
              + " p.provider_type_cd, p.provider_npi "
              + "FROM provider_info p "
              + "WHERE p.provider_type_cd='PAY TO PROVIDER' AND claim_index=" + this.viewClaimIndex + "";

      q = em.createNativeQuery(sql);
      l = q.getResultList();
      if (!l.isEmpty()) {
         Object[] row = (Object[]) l.get(0);

         ProviderInfo bp = new ProviderInfo();

         bp.setLastName((String) row[0]);
         bp.setAddressLine1((String) row[1]);
         bp.setAddressLine2((String) row[2]);
         bp.setCity((String) row[3]);
         bp.setState((String) row[4]);
         bp.setZipCode((String) row[5]);
         bp.setPhoneNumber((String) row[6]);
         bp.setProviderTin((String) row[7]);
         bp.setContactName((String) row[8]);
         bp.setFaxNumber((String) row[9]);
         bp.setEMail((String) row[10]);
         bp.setProviderNpi((String) row[12]);

         this.paytoProvider = bp;
      }

      // set up billing information, for old records prior to 8/2007 use remit address
      this.billingProvider = null;
      sql = "SELECT p.last_name, p.address_line1, p.address_line2, p.city, p.state, p.zip_code, p.phone_number, p.provider_tin, p.contact_name, p.fax_number, p.e_mail, p.provider_type_cd, p.provider_npi, "
                + " (select "
                + "      max(pr.provider_type_cd) "
                + "  from "
                + "      provider_info pr "
                + "  where "
                + "      ( "
                + "          pr.provider_type_cd='BILLING PROVIDER' "
                + "          OR pr.provider_type_cd='REMITADDR' "
                + "      ) "
                + "      and pr.claim_index=" + this.viewClaimIndex + ") as providerType "
                + " FROM "
                + " provider_info p "
                + " WHERE "
                + "  claim_index='" + this.viewClaimIndex + "' "
                + "  and  "
                + "  ( "
                + "    p.provider_type_cd='BILLING PROVIDER' "
                + "   OR  "
                + "   p.provider_type_cd='REMITADDR' "
                + "  )";
// " + this.viewClaimIndex + "
      q = em.createNativeQuery(sql);
      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      /* loop through the results. */
      while (it.hasNext()) {

         Object[] row = (Object[]) it.next();

         ProviderInfo bp = new ProviderInfo();

         bp.setLastName((String) row[0]);
         bp.setAddressLine1((String) row[1]);
         bp.setAddressLine2((String) row[2]);
         bp.setCity((String) row[3]);
         bp.setState((String) row[4]);
         bp.setZipCode((String) row[5]);
         bp.setPhoneNumber((String) row[6]);
         bp.setProviderTin((String) row[7]);
         bp.setContactName((String) row[8]);
         bp.setFaxNumber((String) row[9]);
         bp.setEMail((String) row[10]);
         bp.setProviderNpi((String) row[12]);

         this.billingProvider = bp;

         String providerType = (String) row[11];
         if (providerType.equalsIgnoreCase("BILLING PROVIDER")) {
            break;
         }
      }

      reroutingHistory.clear();

      sql = " SELECT  fcr.transfer_date,  "
            + " fcrr.description,  "
            + " fcr.transferred_to,  "
            + " fcr.transfer_from_user, "
            + " fcr.transfer_from_visn_cd ,  "
            + " fcr.transfer_from_facility_cd, "
            + " fcr.comments, "
            + " fcr.transfer_to_user, "
            + " fcr.transfer_to_visn_cd ,  "
            + " fcr.transfer_to_facility_cd, "
            + " fcr.transfer_initator "
            + "FROM FEE_CLAIM_ROUTING fcr "
            + "LEFT JOIN FEE_CLAIM_ROUTINGS_REASON fcrr  ON fcrr.transfer_reason_cd = fcr.transfer_reason_cd "
            + "WHERE fcr.claim_index='" + this.viewClaimIndex + "'  "
            + "ORDER BY transfer_date DESC ";

      q = em.createNativeQuery(sql);
      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      /* loop through the results. */
      while (it.hasNext()) {
         Object[] row = (Object[]) it.next();

         FeeClaimRouting fcr = new FeeClaimRouting();
         FeeClaimRoutingsReason fcrr = new FeeClaimRoutingsReason();

         fcr.setTransferDate((Date) row[0]);
         fcrr.setDescription((String) row[1]);
         fcr.setTransferredTo((String) row[2]);
         fcr.setTransferFromUser((String) row[3]);
         fcr.setTransferFromVisnCd((String) row[4]);
         fcr.setTransferFromFacilityCd((String) row[5]);
         fcr.setComments((String) row[6]);
         fcr.setTransferToUser((String) row[7]);
         fcr.setTransferToVisnCd((String) row[8]);
         fcr.setTransferToFacilityCd((String) row[9]);
         fcr.setTransferInitator((String) row[10]);

         ClaimReroutingHistoryEntity crrhe = new ClaimReroutingHistoryEntity(fcr, fcrr);
         if (log.isDebugEnabled()) {
            log.debug("Tranferred To: {0} ", crrhe.getRouting().getTransferredTo());
         }
         /* add the ClaimSearchEntity to the results collection. */
         reroutingHistory.add(crrhe);
      }




      rejectionHistory.clear();

      sql = " select " +
      "     t.enqueue_date,  " +
      "     t.created_by,  " +
      "     a.description,  " +
      "     t.enqueue_comments, " +
      "     t.modified_by,  " +
      "     t.outcome, " +
      "     t.dequeue_comments " +
      "     from claim_queue t, adjustment_reason a " +
      "     where t.reason = a.adjustment_reason_cd " +
      "     and t.claim_queue_type_cd='REJECTION_APPROVAL' " +
      "     and t.claim_index= " + this.viewClaimIndex + " " +
      "     order by t.enqueue_date DESC";


// crashes here - 'table not found'
      q = em.createNativeQuery(sql);
      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      /* loop through the results. */
      while (it.hasNext()) {
         Object[] row = (Object[]) it.next();

         ClaimQueue cq = new ClaimQueue();
         AdjustmentReason ar = new AdjustmentReason();

        cq.setEnqueueDate((Date) row[0]);
        cq.setCreatedBy((String) row[1]);
        ar.setDescription((String) row[2]);
        cq.setEnqueueComments((String) row[3]);
        cq.setModifiedBy((String) row[4]);
        cq.setOutcome((String) row[5]);
        cq.setDequeueComments((String) row[6]);


         ClaimRejectionHistoryEntity crhe = new ClaimRejectionHistoryEntity(cq, ar);
         /* add the ClaimSearchEntity to the results collection. */
         rejectionHistory.add(crhe);

       }


      	claimLineItems.clear();

      	final String COMMA = ", ";
      	
    	StringBuffer sbAmts = new StringBuffer();
		StringBuffer sbCodes = new StringBuffer();
	    StringBuffer prevsbAmts = new StringBuffer();
		StringBuffer prevsbCodes = new StringBuffer();
	
		BigDecimal bdBilledAmt = null;

		BigDecimal bdAdjAmt1 = null;
		BigDecimal bdAdjAmt2 = null;
		
	    SimpleDateFormat formatter = new SimpleDateFormat("MM/dd/yyyy");
	    DecimalFormat df = new DecimalFormat("$#0.00");

	    BigDecimal procSeq = null;
	    BigDecimal prevProcSeq = new BigDecimal(-1);
	    BigDecimal backProcSeq = new BigDecimal(0);
	    BigDecimal bdPrevPaidAmt = new BigDecimal(0);
	    BigDecimal bdPrevBilledAmt = new BigDecimal(0);
	    BigDecimal bdFirstBilledAmt = new BigDecimal(0);
	    BigDecimal bu = new BigDecimal(0);
	    BigDecimal prevBu = new BigDecimal(0);
	

	    
	    // BigDecimal bdTotalPaid = new BigDecimal(0);	//result = test.add(new BigDecimal(30));

	    boolean bFirstTimeThrough=true;
	    boolean bSameProc=false;
	      
	    java.sql.Timestamp tsPrevPaymentDate = new java.sql.Timestamp(0);
	    java.sql.Timestamp tsPaymentDate = new java.sql.Timestamp(0);
	    
	    java.sql.Timestamp tsPrevServiceFromDate = new java.sql.Timestamp(0);
	    java.sql.Timestamp tsServiceFromDate = new java.sql.Timestamp(0);
	    
	    java.sql.Timestamp tsPrevServiceToDate = new java.sql.Timestamp(0);
	    java.sql.Timestamp tsServiceToDate = new java.sql.Timestamp(0);
	
	    String sFormattedDate = "";	
	    
	    // String prevPaidAmt = "";
		  
	    String sLineItemCheckNumber = "";
	    String prevLineItemCheckNumber = "";
	    String sTotalAmtPaid = "";
	    
	    String sPrevFormattedDate= "";
	   
	    String procCode = "";
	    String prevProcCode = "";
		    
	    String revCode = null; 			
	    String prevRevCode = null; 		
	    
	    String modifier1 = "";
	    String prevModifier1 = "";
	    
	    String modifier2 = "";
	    String prevModifier2 = "";
	    
	    String modifier3 = "";
	    String prevModifier3 = "";
	    
	    String modifier4 = "";
	    String prevModifier4 = "";
	    
	    String prodDesc = "";
	    String prevProdDesc = "";
	    
	    String diagPointer = "";
	    String prevDiagPointer = "";
	    
	    String lineStatus = "";
	    String prevLineStatus = "";
	    
	    // oral 
	    String toothSurface = "";
	    String prevToothSurface = "";
	    String tsCd2 = "";
	    String prevtsCd2 = "";
	    String tsCd3 = "";
	    String prevtsCd3 = "";
	    String tsCd4 = "";
	    String prevtsCd4 = "";
	    String tsCd5 = "";
	    String prevtsCd5 = "";
	    String oralCavity = "";
	    String prevOralCavity = "";
	    String toothCode = "";
	    String prevToothCode = "";
	    
	    //NDC
	    String ndcCode  = "";   
	    String prevNdcCode  = "";   
	    BigDecimal ndcUnitCnt  = new BigDecimal(0);	 
	    BigDecimal prevNdcUnitCnt  = new BigDecimal(0);	 
	    String ndcUnitOfMeasure  = "";
	    String prevNdcUnitOfMeasure  = "";
	    
	    LineItemEntity prevLIE = null;
	    LineItemEntity nextLIE = null;

	    int iListSize = 0;
	    int iRecCount=0;

	    // list for testing of written records - REMOVE WHEN DONE TESTING
	    @SuppressWarnings("rawtypes")
	    
	    // used for debugging....
	    ArrayList list = new ArrayList();
	    
	    ArrayList detailsList = new ArrayList();




 // 7/17
      	String sql = "select " +
            	" cp.claim_procedure_seq, cp.proc_code, cp.revenue_cd, cp.service_from_date, cp.service_to_date, cp.billed_amount, cp.proc_code_desc, " +
            	" cp.diag_pointer, cp.billed_units, cp.modifier1, cp.modifier2, cp.modifier3, cp.modifier4, " + 
            	" cpay.TREASURY_PAYMENT_DATE as payment_date, cpr.paid_amount, " +
            	" cpay.payment_tracking_number, " +
            	" cadj.adjustment_reason_cd AS adjustment_reason_code_1, " +		
            	" 0 adjustment_reason_code_2, " +			
            	" cadj.adjustment_amount AS adjustment_amount_1, " +				
                " 0 adjustment_amount_2, " +				


            " (SELECT ti.tooth_code FROM TOOTH_INFO ti WHERE ti.claim_index=  " + this.viewClaimIndex + " AND cp.claim_proc_id = ti.claim_proc_id) as toothCode, " +
            	" (SELECT ti.tooth_surface FROM TOOTH_INFO ti WHERE ti.claim_index=  " + this.viewClaimIndex + " AND cp.claim_proc_id = ti.claim_proc_id) as toothSurface, " +
            		" (SELECT ti.tooth_surface_cd2 FROM TOOTH_INFO ti WHERE ti.claim_index=  " + this.viewClaimIndex + " AND cp.claim_proc_id = ti.claim_proc_id) as toothSurfaceCd2, " +
            			" (SELECT ti.tooth_surface_cd3 FROM TOOTH_INFO ti WHERE ti.claim_index=  " + this.viewClaimIndex + " AND cp.claim_proc_id = ti.claim_proc_id) as toothSurfaceCd3, " +
            				" ( SELECT ti.tooth_surface_cd4 FROM TOOTH_INFO ti WHERE ti.claim_index=  " + this.viewClaimIndex + " AND cp.claim_proc_id = ti.claim_proc_id) as toothSurfaceCd4, " +

            		" (SELECT ti.tooth_surface_cd5 FROM TOOTH_INFO ti WHERE ti.claim_index=  " + this.viewClaimIndex + "  AND cp.claim_proc_id = ti.claim_proc_id) as toothSurfaceCd5, " +
            		"  cp.oral_cavity_cd, cp.line_status, " +
                	" cp.ndc_code, cp.ndc_unit_cnt, cast(cp.ndc_unit_of_measure as VARCHAR2(3)) " + 
                	" FROM claim_procedure cp, claim_payment cpay, claim_proc_recon cpr, claim_proc_rec_adjust cadj " +
            		" where " +
            " cp.claim_index = " + this.viewClaimIndex + "  and cp.claim_proc_id = cpr.claim_proc_id(+) and cpr.claim_payment_id = cpay.claim_payment_id(+) and cpr.claim_proc_rec_id = cadj.claim_proc_rec_id (+) " +
        	" order by " +
            " cp.claim_procedure_seq, cpay.payment_tracking_number";


      	q = em.createNativeQuery(sql);

         try {

   	      l = q.getResultList();
   	      iListSize = l.size();
 //  	      System.out.println("size of records for main query is: " + iListSize);

     	  it = l.iterator();

   	  } catch(Exception ex) {
       	  System.out.println("findClaimByClaimIndex() ERROR running payment query: " + ex.getMessage() + "on claim: " + this.viewClaimIndex);
       	  ex.printStackTrace();
      }


      while (it.hasNext())
      {

    	 iRecCount++;
         ClaimPaymentBean cpb = new ClaimPaymentBean();
         ClaimProcedure cp = new ClaimProcedure(); 	       
         RevenueCode rc = new RevenueCode();	        
         ToothInfo ti = new ToothInfo();
         
         NdcData ndcData = new NdcData(); 
         ClaimProcedure cp1 = new ClaimProcedure();


         try
         {

						Object[] row = null;
 						row = (Object[]) it.next();

  						
       	 				if(row[0] != null)
        	 			{
        	 				

        	 				if(iRecCount==1) {
        	 					bFirstTimeThrough=true;
        	 				} else {
        	 					bFirstTimeThrough=false;
        	 				}

        	 				if(iRecCount==1) {
        	 					bFirstTimeThrough=true;
        	 					bSameProc = true;
        	 				} else {
        	 					bFirstTimeThrough=false;
        	 				}

        	 				procSeq = (BigDecimal) row[0];


							// gather codes and amts. for this claim index and sequence.....        	 				
							if(iRecCount > 0) {
									if(procSeq.longValue() == backProcSeq.longValue() ) {
										bSameProc = true;
									} else {
										// System.out.println("NEW RECORD...procSeq is: " + procSeq.longValue());
										 detailsList = getSequenceDetails(procSeq.longValue(), this.viewClaimIndex);	// both longs
										 String codes = (String) detailsList.get(0).toString();
										 String amts = (String) detailsList.get(1).toString();
										 sTotalAmtPaid = (String) detailsList.get(2).toString();
										 sbCodes.append(codes);
										 sbAmts.append(amts);
													
									}	// END IF
							}	// END IF
        	 			
// END OF NEW SQL INSERTION

							backProcSeq = procSeq;
	

 
							// MEDICAL CODE
							cp.setProcCode((String) row[1]);
							if(row[1] != null) {
								procCode = (String) row[1];
								
								if(!bFirstTimeThrough) {
						   	 		prevProcCode = procCode;
						   	 	} 
							}
							
// REVENUE CODE
							if(row[2] != null) {
								 revCode = (String) row[2];

								if(!bFirstTimeThrough) {
						   	 		prevRevCode =revCode;
						   	 		rc.setRevenueCd(prevRevCode);
						   	 	} else {
						   	 		rc.setRevenueCd(revCode);
						   	 	}
							}
							
							// SERVICE FROM DATE
							 if(row[3] != null) {
								 tsServiceFromDate = (java.sql.Timestamp) row[3];

								 if(!bFirstTimeThrough) {
							   	 	tsPrevServiceFromDate = tsServiceFromDate;
							   	 }
							 }	else {
								tsServiceFromDate = new java.sql.Timestamp(-1);

								if(!bFirstTimeThrough) {
						   	 		tsPrevServiceFromDate = tsServiceFromDate;
							   	 }
						    }	// end Service From Date
							 
							 
							// SERVICE TO DATE
							 if(row[4] != null) {
								 tsServiceToDate = (java.sql.Timestamp) row[4];

								 if(!bFirstTimeThrough) {
							   	 	tsPrevServiceToDate = tsServiceToDate;
							   	 }
							 }	else {
						    	
								tsServiceToDate = new java.sql.Timestamp(-1);

								if(!bFirstTimeThrough) {
						   	 		tsPrevServiceToDate = tsServiceToDate;
							   	 }
						    }	// end Service From Date 
							 
							 

							//BILLED AMOUNT
						    if(row[5] != null) {
						   	 	bdBilledAmt = (BigDecimal) row[5];

						   	 	// if it's not the first record, assign previous value to current
						   	 	if(!bFirstTimeThrough) {
						   	 		bdPrevBilledAmt = bdBilledAmt;
						   	 	} 
						    }	

						    // PRODUCT CODE DESCRIPTION
						    if(row[6] != null) {
								prodDesc = (String) row[6];
								
								if(!bFirstTimeThrough) {
						   	 		prevProdDesc = prodDesc;
						   	 	} 
							}
						    
						    // cp.setDiagPointer((String) row[7]);
						    if(row[7] != null) {
								diagPointer = (String) row[7];
								
								if(!bFirstTimeThrough) {
						   	 		prevDiagPointer = diagPointer;
						   	 	} 
							}
						    
						    
							 if(row[8] != null) {
						   	 	bu = (BigDecimal) row[8];

						   	 	// if it's not the first record, assign previous value to current
						   	 	if(!bFirstTimeThrough) {
						   	 		prevBu = bu;
						   	 	} else {
						   	 		// nothing to point to previous......leave vals empty..
						   	 	}
						    }
						    
						    
						    
						    
						    // MODIFIER 1
							if(row[9] != null) {
								modifier1 = (String) row[9];
								
								if(!bFirstTimeThrough) {
						   	 		prevModifier1 =modifier1;
						   	 	} 
							}
							
							
							 // MODIFIER 2
							if(row[10] != null) {
								modifier2 = (String) row[10];
								
								if(!bFirstTimeThrough) {
						   	 		prevModifier2 =modifier2;
						   	 	} 
							}
							
							
							 // MODIFIER 3
							if(row[11] != null) {
								modifier3 = (String) row[11];
								
								if(!bFirstTimeThrough) {
						   	 		prevModifier3 =modifier3;
						   	 	} 
							}
							
							// MODIFIER 4
							if(row[12] != null) {
								modifier4 = (String) row[12];
								
								if(!bFirstTimeThrough) {
						   	 		prevModifier4 =modifier4;
						   	 	} 
							}
							

							
					    
// PAYMENT DATE
								 if(row[13] != null) {
									 tsPaymentDate = (java.sql.Timestamp) row[13];
									 sFormattedDate = formatter.format(tsPaymentDate);
									 
									 if(!bFirstTimeThrough) {
								   	 	tsPrevPaymentDate = tsPaymentDate;
								   	    sPrevFormattedDate = sFormattedDate;
								   	 }
									 
								 }
// CHECK NUMBER
									 if(row[15] != null) 
									 {
										 sLineItemCheckNumber = (String) row[15];

										 if(!bFirstTimeThrough) {
											prevLineItemCheckNumber = sLineItemCheckNumber; 
										 }
									 }	
 									    
								// LINE STATUS
								 if(row[27] != null) {
									 lineStatus = (String) row[27];
										 
									 if(!bFirstTimeThrough) {
										 prevLineStatus = lineStatus; 
									 }
								 }
								 
								 // TEETH
								 
								 // Tooth Code
								 if(row[20] != null) {
									 toothCode = (String) row[20];
										 
									 if(!bFirstTimeThrough) {
										 prevToothCode = toothCode; 
									 }
								 }
								 
								 
								 // Tooth Surface
								 if(row[21] != null) {
									 toothSurface = (String) row[21];
										 
									 if(!bFirstTimeThrough) {
										 prevToothSurface = toothSurface; 
									 }
								 }
								 
								 // tsCd2
								 if(row[22] != null) {
									 tsCd2 = (String) row[22];
										 
									 if(!bFirstTimeThrough) {
										 prevtsCd2  = tsCd2; 
									 }
								 }
								 
								 // tsCd3
								 if(row[23] != null) {
									 tsCd3 = (String) row[23];
										 
									 if(!bFirstTimeThrough) {
										 prevtsCd3  = tsCd3; 
									 }
								 }
								 
								 // tsCd4
								 if(row[24] != null) {
									 tsCd4 = (String) row[24];
										 
									 if(!bFirstTimeThrough) {
										 prevtsCd4  = tsCd4; 
									 }
								 }
								 
								// tsCd5
								 if(row[25] != null) {
									 tsCd5 = (String) row[25];
										 
									 if(!bFirstTimeThrough) {
										 prevtsCd5  = tsCd5; 
									 }
								 }
								 
								// tsCd5
								 if(row[26] != null) {
									 oralCavity = (String) row[26];
										 
									 if(!bFirstTimeThrough) {
										 prevOralCavity  = oralCavity; 
									 }
								 }
								 
								 if(row[28] != null) {
									 ndcCode = (String) row[28];
									 
										if(!bFirstTimeThrough) {
											prevNdcCode = ndcCode;
								   	 	} 		 
								 }
									 
								 if(row[29] != null) {
									 ndcUnitCnt = (BigDecimal) row[29];
									 
										if(!bFirstTimeThrough) {
											prevNdcUnitCnt = ndcUnitCnt;
								   	 	} 
								 }	
								 
								 if(row[30] != null) {
									 ndcUnitOfMeasure = (String) row[30];
//									 ndcUnitOfMeasure = uom.toString();
									 
										if(!bFirstTimeThrough) {
											prevNdcUnitOfMeasure = ndcUnitOfMeasure;
								   	 	} 
								 }	
							// if(!bDone)		// not last record
							// {

											if(procSeq.longValue()==prevProcSeq.longValue() )
											{


														// GATHER VALUES FOR AMTS AND CODES STRING BUFFER....CODE OMITTED FOR CLARITY
														// REMOVE THIS if/else - NOT NECESSARY.

											} else { 	// new record - write previous record


												// if it's first time through, set values to current value
												// OK
												if(bFirstTimeThrough) {
													
// System.out.println("Setting first time formatted Payment Date to: " + sFormattedDate);	
													cpb.setLineItemTreasuryPaymentDateFormatted(sFormattedDate);
													
													cp.setClaimProcedureSeq(procSeq.longValue());
													cp.setBilledAmount(bdBilledAmt);
													
													cpb.setLineItemPaidAmountFormatted(sTotalAmtPaid);
													
													cpb.setLineItemCheckNumber(sLineItemCheckNumber);
													cp.setProcCode(procCode);
													cp.setServiceFromDate(tsServiceFromDate);		        
													cp.setServiceToDate(tsServiceToDate);
													cp.setBilledUnits(bu.longValue());
													cp.setProcCodeDesc(prodDesc);
													cp.setDiagPointer(diagPointer);
													cp.setModifier1(modifier1);
													cp.setModifier2(modifier2);
													cp.setModifier3(modifier3);
													cp.setModifier4(modifier4);
													cp.setLineStatus(lineStatus);
													cp.setRevenueCode(rc);
													
													ti.setToothCode(toothCode);
													ti.setToothSurface(toothSurface);
													ti.setToothSurfaceCd2(tsCd2);
													ti.setToothSurfaceCd3(tsCd3);
													ti.setToothSurfaceCd4(tsCd4);
													ti.setToothSurfaceCd5(tsCd5);
													cp.setOralCavityCd(oralCavity);
													
													if (ndcCode.length() > 0){
														ndcData.setClaimProcedureSeq(procSeq.longValue());
														ndcData.setNdcCode(ndcCode);
														ndcData.setNdcUnitCnt(ndcUnitCnt);
														ndcData.setNdcUnitOfMeasure(ndcUnitOfMeasure);
														
													}
													
													try {
														if( (sbCodes.length() > 2) && (sbAmts.length() > 2) ) {
															sbCodes=new StringBuffer(sbCodes.deleteCharAt(sbCodes.length()-2));
															sbAmts=new StringBuffer(sbAmts.deleteCharAt(sbAmts.length()-2));
															cpb.setLineItemCodes(sbCodes.toString());
															cpb.setLineItemAdjAmts(sbAmts.toString());
														  }
													} catch(IndexOutOfBoundsException  iob)	{
														  System.out.println("ViewClaimBean.findClaimByClaimIndex() ERROR: " + iob.getMessage());
													 	  iob.printStackTrace();
													 	  log.error(iob.getMessage(), iob);
													} catch(Exception ex) {
														 System.out.println("ViewClaimBean.findClaimByClaimIndex() ERROR: " + ex.getMessage());
													 	 ex.printStackTrace();
													 	 log.error(ex.getMessage(), ex);
													}
													
												} else {	// else set values to previous values
													
													cpb.setLineItemTreasuryPaymentDateFormatted(sPrevFormattedDate);													
													
													cp.setClaimProcedureSeq(procSeq.longValue());
													cp.setBilledAmount(bdPrevBilledAmt);
													
													cpb.setLineItemPaidAmountFormatted(sTotalAmtPaid);
													cpb.setLineItemCheckNumber(prevLineItemCheckNumber);
													cp.setProcCode(prevProcCode);
													cp.setRevenueCode(rc);
													cp.setServiceFromDate(tsPrevServiceFromDate);		        
													cp.setServiceToDate(tsPrevServiceToDate);
													
													
													cp.setModifier1(prevModifier1);
													cp.setModifier2(prevModifier2);
													cp.setModifier3(prevModifier3);
													cp.setModifier4(prevModifier4);
													
													cp.setBilledUnits(prevBu.longValue());
													cp.setProcCodeDesc(prevProdDesc);
													cp.setDiagPointer(prevDiagPointer);
													cp.setLineStatus(prevLineStatus);	
													
													ti.setToothCode(prevToothCode);
													ti.setToothSurface(prevToothSurface);
													ti.setToothSurfaceCd2(prevtsCd2);
													ti.setToothSurfaceCd3(prevtsCd3);
													ti.setToothSurfaceCd4(prevtsCd4);
													ti.setToothSurfaceCd5(prevtsCd5);
													cp.setOralCavityCd(prevOralCavity);
													
													if (ndcCode.length() > 0){
														ndcData.setClaimProcedureSeq(procSeq.longValue());
														ndcData.setNdcCode(prevNdcCode);
														ndcData.setNdcUnitCnt(prevNdcUnitCnt);
														ndcData.setNdcUnitOfMeasure(prevNdcUnitOfMeasure);
														
													}
													
													
													
													try {
														if( (sbCodes.length() > 2) && (sbAmts.length() > 2) ) {
															sbCodes=new StringBuffer(sbCodes.deleteCharAt(sbCodes.length()-2));
															sbAmts=new StringBuffer(sbAmts.deleteCharAt(sbAmts.length()-2));
															cpb.setLineItemCodes(sbCodes.toString());
															cpb.setLineItemAdjAmts(sbAmts.toString());
														  }
													} catch(IndexOutOfBoundsException  iob)	{
													//	  System.out.println("ViewClaimBean.findClaimByClaimIndex() ERROR: " + iob.getMessage());
													// 	  iob.printStackTrace();
													 	  log.error(iob.getMessage(), iob);
													} catch(Exception ex) {
													//	 System.out.println("ViewClaimBean.findClaimByClaimIndex() ERROR: " + ex.getMessage());
													// 	 ex.printStackTrace();
													 	 log.error(ex.getMessage(), ex);
													}
													
													
													
									 			}

											    if (ndcCode.length() > 0){
												    
											    	NdcData ndcData1 = new NdcData();
											    	nextLIE = new LineItemEntity(cp, ti, cpb, ndcData1);
												    claimLineItems.add(nextLIE);
												    
											    	ClaimPaymentBean cpb1 = new ClaimPaymentBean();
											    	cpb1.setLineItemTreasuryPaymentDateFormatted("");
											    	cpb1.setLineItemCodes(null);
													cpb1.setLineItemAdjAmts("");
													cpb1.setLineItemPaidAmountFormatted("");
													cpb1.setLineItemCheckNumber("");
													
											    	ToothInfo ti1 = new ToothInfo();
											    	prevLIE = new LineItemEntity(cp1, ti1, cpb1, ndcData);
												    claimLineItems.add(prevLIE);
											    }
											    else {
											    	
											    	NdcData ndcData1 = new NdcData();
												    prevLIE = new LineItemEntity(cp, ti, cpb, ndcData1);
												    claimLineItems.add(prevLIE);
											    	
											    }
											    

											    // clean up
											    prevLIE = null;
											    nextLIE=null;
											    sbCodes.setLength(0);
												sbAmts.setLength(0);

												prevsbCodes.setLength(0);
												prevsbAmts.setLength(0);
												ndcCode = "";

// SWAP PROCDECURE REFERENCES - DO NOT CHANGE...
												prevProcSeq=procSeq;
// 							}	// end bDone Check

        	 			}	// END row[0] != null CHECK
        	 	}

      }catch(Exception e) {
        	 		        	 System.out.println("ViewClaimBean.findClaimByClaimIndex() - error: " + e.getMessage());
        	 		        	 e.printStackTrace();
        	 		        	 log.error("ViewClaimBean.findClaimByClaimIndex(): error getting line items payment information - " + e.getMessage());
      }	// end record loop try statement

}	//////////////////////////////////////           end while loop and record processing for line level payment information //////////////////////////////////////////


// SECONDARY PAYER INFORMATION
      secondaryPayerInfo = null;

      sql = " select t.plan_name, t.plan_id  "
         +  " from fpps_owner.payer_info t  "
         +  " where t.claim_index='" + this.viewClaimIndex + "' "
         +  " AND PAYER_TYPE_CD='SECONDARY' ";

      q = em.createNativeQuery(sql);
      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      /* loop through the results. */
      while (it.hasNext()) {

         Object[] row = (Object[]) it.next();

         PayerInfo payer = new PayerInfo();
         payer.setPlanName((String) row[0]);
         payer.setPlanId((String) row[1]);

         this.secondaryPayerInfo = payer;
      }


// TERTIARY PAYER INFORMATION
      tertiaryPayerInfo = null;

      sql = " select t.plan_name, t.plan_id  "
         +  " from fpps_owner.payer_info t  "
         +  " where t.claim_index='" + this.viewClaimIndex + "' "
         +  " AND PAYER_TYPE_CD='TERTIARY' ";

      q = em.createNativeQuery(sql);
      /* query the database and put the records into a temporary list. */
      l = q.getResultList();
      it = l.iterator();

      /* loop through the results. */
      while (it.hasNext()) {

         Object[] row = (Object[]) it.next();

         PayerInfo payer = new PayerInfo();
         payer.setPlanName((String) row[0]);
         payer.setPlanId((String) row[1]);

         this.tertiaryPayerInfo = payer;
      }





      diagnosisCodes.clear();

      q = em.createNativeQuery("select c.poa_indicator from claims c where c.claim_index = :index");
      q.setParameter("index", this.viewClaimIndex);
      String poa = (String) q.getSingleResult();

      q = em.createQuery("select hi from fHealthcareInfoCode hi where hi.claim.claimIndex = :index and (hi.hi01cdListQual like 'BF%' or hi.hi01cdListQual like 'BK%' or hi.hi01cdListQual like 'PR%' or hi.hi01cdListQual like 'BN%' or hi.hi01cdListQual like 'BJ%' or hi.hi01cdListQual = 'ABF' or hi.hi01cdListQual = 'ABK' or hi.hi01cdListQual = 'APR' or hi.hi01cdListQual = 'ABN' or hi.hi01cdListQual = 'ABJ') order by hi.seqNbr");
      q.setParameter("index", this.viewClaimIndex);
      List<HealthcareInfoCode> hiCodes =  q.getResultList();

      /* loop through the results. */
      int counter = 1;
      for (HealthcareInfoCode c : hiCodes) {

         String qual1 = c.getHi01cdListQual();
         String code1 = c.getHi01procCd();
         String desc1 = c.getHi01procDesc();
         String poa1 =  c.getHi01cndtnResponseCd();
         String qual2 = c.getHi02cdListQual();
         String code2 = c.getHi02procCd();
         String desc2 = c.getHi02procDesc();
         String poa2 =  c.getHi02cndtnResponseCd();
         String qual3 = c.getHi03cdListQual();
         String code3 = c.getHi03procCd();
         String desc3 = c.getHi03procDesc();
         String poa3 =  c.getHi03cndtnResponseCd();
         String qual4 = c.getHi04cdListQual();
         String code4 = c.getHi04procCd();
         String desc4 = c.getHi04procDesc();
         String poa4 =  c.getHi04cndtnResponseCd();
         String qual5 = c.getHi05cdListQual();
         String code5 = c.getHi05procCd();
         String desc5 = c.getHi05procDesc();
         String poa5 =  c.getHi05cndtnResponseCd();
         String qual6 = c.getHi06cdListQual();
         String code6 = c.getHi06procCd();
         String desc6 = c.getHi06procDesc();
         String poa6 =  c.getHi06cndtnResponseCd();
         String qual7 = c.getHi07cdListQual();
         String code7 = c.getHi07procCd();
         String desc7 = c.getHi07procDesc();
         String poa7 =  c.getHi07cndtnResponseCd();
         String qual8 = c.getHi08cdListQual();
         String code8 = c.getHi08procCd();
         String desc8 = c.getHi08procDesc();
         String poa8 =  c.getHi08cndtnResponseCd();
         String qual9 = c.getHi09cdListQual();
         String code9 = c.getHi09procCd();
         String desc9 = c.getHi09procDesc();
         String poa9 =  c.getHi09cndtnResponseCd();
         String qual10 = c.getHi10cdListQual();
         String code10 = c.getHi10procCd();
         String desc10 = c.getHi10procDesc();
         String poa10 =  c.getHi10cndtnResponseCd();
         String qual11 = c.getHi11cdListQual();
         String code11 = c.getHi11procCd();
         String desc11 = c.getHi11procDesc();
         String poa11 =  c.getHi11cndtnResponseCd();
         String qual12 = c.getHi12cdListQual();
         String code12 = c.getHi12procCd();
         String desc12 = c.getHi12procDesc();
         String poa12 =  c.getHi12cndtnResponseCd();

         if (poa == null || poa.equalsIgnoreCase("null")) {
            poa = "";
         }

         log.debug("=========================POA INDICATOR pre parse: " + poa);
         poa = poa.startsWith("POA") ? poa.substring(3, poa.length()) : poa;
         poa = poa.indexOf("Z") > 0 ? poa.substring(0, poa.indexOf("Z")) : poa;

         // poa = poa.length()>4 ? poa.substring(3, poa.length()-1) : poa;
         log.debug("=========================POA INDICATOR post parse: " + poa);

         DiagnosisCodesEntity dce;

         dce = evaluateDiag(counter, code1, poa, desc1, qual1, poa1);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code2, poa, desc2, qual2, poa2);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code3, poa, desc3, qual3, poa3);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code4, poa, desc4, qual4, poa4);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code5, poa, desc5, qual5, poa5);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code6, poa, desc6, qual6, poa6);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code7, poa, desc7, qual7, poa7);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code8, poa, desc8, qual8, poa8);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code9, poa, desc9, qual9, poa9);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code10, poa, desc10, qual10, poa10);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code11, poa, desc11, qual11, poa11);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }
         dce = evaluateDiag(counter, code12, poa, desc12, qual12, poa12);
         if (dce != null) {
            diagnosisCodes.add(dce);
            counter++;
         }


// does record loop continue to here???
// System.out.println("END OF RECORD LOOP.......");

}		// END OF RECORD LOOP

      //Sorting now rather than earlier with group by because POA indicator depends on
      //the order of the codes in the database so the above code would pick the wrong
      //poa if a different order by clause were used
      sortDiagCodes(diagnosisCodes);
      recreateSequenceNumberForScreen(diagnosisCodes);

      log.debug("ViewClaimBean.findClaimByClaimIndex return: {0} ", ret);

      return ret;

   }

   //This method numbers the sequence number associated with each diagnosis code
   //starting at 1 for the first code in the list ending at length for the last
   private void recreateSequenceNumberForScreen(List<DiagnosisCodesEntity> diagnosisCodesToRenumber) {
	   int diagCodeSeqNumber = 1;
	   for(DiagnosisCodesEntity entityToRenumber : diagnosisCodesToRenumber)
	   {
		   entityToRenumber.setSequenceNumber(diagCodeSeqNumber);
		   diagCodeSeqNumber++;
	   }
   }

    //Sort the diagnosis codes for proper ordering on the screen
	private void sortDiagCodes(List<DiagnosisCodesEntity> diaCodes) {
		Collections.sort(diaCodes, new Comparator<DiagnosisCodesEntity>(){

		  private DiagCodeSortOrder returnDiagCodeEnum(DiagnosisCodesEntity arg0)
			{
			  	if(arg0.getDiagnosisCodeQualifier() == null || arg0.getDiagnosisCodeQualifier().trim() == null){
			  		log.warn("ViewClaimBean.sortDiagCodes passed blank diag code code");
			  		return DiagCodeSortOrder.UNKNOWN;
			  	}

			  	String trimmedCode = arg0.getDiagnosisCodeQualifier().trim();
				if (trimmedCode.equalsIgnoreCase("BK")) return DiagCodeSortOrder.BK;
				if (trimmedCode.equalsIgnoreCase("BF")) return DiagCodeSortOrder.BF;
				if (trimmedCode.equalsIgnoreCase("BJ")) return DiagCodeSortOrder.BJ;
				if (trimmedCode.equalsIgnoreCase("PR")) return DiagCodeSortOrder.PR;
				if (trimmedCode.equalsIgnoreCase("BN")) return DiagCodeSortOrder.BN;
				if (trimmedCode.equalsIgnoreCase("ABK")) return DiagCodeSortOrder.BK;
				if (trimmedCode.equalsIgnoreCase("ABF")) return DiagCodeSortOrder.BF;
				if (trimmedCode.equalsIgnoreCase("ABJ")) return DiagCodeSortOrder.BJ;
				if (trimmedCode.equalsIgnoreCase("APR")) return DiagCodeSortOrder.PR;
				if (trimmedCode.equalsIgnoreCase("ABN")) return DiagCodeSortOrder.BN;

				log.error("ViewClaimBean.sortDiagCodes unknown diag code found: {0}", arg0.getDiagnosisCodeQualifier());
				return DiagCodeSortOrder.UNKNOWN;
			}

	    	@Override
			public int compare(DiagnosisCodesEntity arg0, DiagnosisCodesEntity arg1) {
	    		DiagCodeSortOrder aDiagCode = returnDiagCodeEnum(arg0);
	    		DiagCodeSortOrder bDiagCode = returnDiagCodeEnum(arg1);
	    		return aDiagCode.compareTo(bDiagCode);
			}});
	}

   public final DiagnosisCodesEntity evaluateDiag(int seq, String code, String poa, String desc, String qual, String poa5010) {

      log.debug("evaluateDiag({0}, {1}, {2}, {3})",new Object[]{seq, code, poa, desc});
      DiagnosisCodesEntity dc = null;


      if (code != null && !code.equals("") && qual != null && (qual.startsWith("BK") || qual.startsWith("BJ") || qual.startsWith("PR") || qual.startsWith("BF") || qual.startsWith("BN") ||
    		  												qual.startsWith("ABK") || qual.startsWith("ABJ") || qual.startsWith("APR") || qual.startsWith("ABF") || qual.startsWith("ABN"))) {
         String p = (poa.length() >= seq) ? (poa.charAt(seq - 1) + "") : "";

         //If the poa string is null or empty but the 5010 poa is populated
         //then set the poa indicator to the one sent in 5010
         if( (poa == null || poa.isEmpty() || poa.trim().isEmpty()) &&
        	 (poa5010 != null && !poa5010.isEmpty() && !poa5010.trim().isEmpty()))
         {
        	p = poa5010;
         }

         dc = new DiagnosisCodesEntity(seq, code, qual, p, desc);
         log.debug("evalDiag = success");
      } else {
         log.debug("evalDiag = fail");
      }

      return dc;
   }

   @SuppressWarnings({ "rawtypes", "unchecked" })
public ArrayList getSequenceDetails(long seq, long viewClaimIndex) {
	   
	    final String COMMA = ", ";
	   	ArrayList list = new ArrayList();
	    DecimalFormat df = new DecimalFormat("$#0.00");
	    String prevAdjReasonCode = "";
	    String sTotal = "";
	    StringBuffer sbCodes = new StringBuffer();
	    String prevAdjAmt = "";
	    StringBuffer sbAmts = new StringBuffer();
	    
	    BigDecimal bdAdjAmt1 = new BigDecimal(0);
	    
	    BigDecimal bdPaidAmt = new BigDecimal(0);
	    
	    float bdTotal = 0;
	    float fTotalPaidAmt = 0;
	    float fTemp = 0;
	   
//	    System.out.println("getSequenceDetails() - seq and index are: " + seq + " " + viewClaimIndex);
	   
	   // //  cpay.TREASURY_PAYMENT_DATE as payment_date " +
	    String sql = " select cp.claim_procedure_seq,  adjustment_reason_cd adjustment_reason_code_1,  adjustment_amount  adjustment_amount_1, cpr.paid_amount " + 
					" from " + 
					" claim_procedure cp, claim_proc_recon cpr, claim_proc_rec_adjust cadj " +
					" where " + 
				    " cp.claim_index = " + viewClaimIndex + " and cp.claim_procedure_seq =  " + seq + " and " + 
				    " cp.claim_proc_id = cpr.claim_proc_id and  cpr.claim_proc_rec_id = CADJ.CLAIM_PROC_REC_ID(+) " + 
					" order by  claim_procedure_seq "; 
		try {
			
// System.out.println("getSequenceDetails() - sql is: " + sql);

			    Query q = em.createNativeQuery(sql);
			      /* query the database and put the records into a temporary list. */
			    List  l = q.getResultList();
			    Iterator  it = l.iterator();
		
	  	         while (it.hasNext()) {
		
			        Object[] row = (Object[]) it.next();
			         
			        if(row != null) { 
					      // SEQUENCE
					     	 if(row[0] != null)
					 			{
					     	  	BigDecimal dprocSeq = (BigDecimal) row[0];
					     	  	System.out.println("Detail proc seq is: " + dprocSeq.longValue());
					 		  }	
				     	  	
				     	  	
				     	  
				     	 // ADJUSTMENT CODES - STRING BUFFER		        	 
							 if(row[1] != null) {
								 prevAdjReasonCode = (String) row[1];
								 sbCodes.append(prevAdjReasonCode+COMMA);
// System.out.println("CODES: " + sbCodes);												 
							 }	// END OF ADJUSTMENT CODES
						
						       	 
						     // PAYMEMNT AMOUNTS - STRING BUFFER 				        	 
							 if(row[2] != null) {
								 bdAdjAmt1 = (BigDecimal) row[2];
								 prevAdjAmt  = df.format(bdAdjAmt1.floatValue());
								 bdTotal+=bdAdjAmt1.floatValue();
								 sbAmts.append(prevAdjAmt+COMMA);
								 
//								  System.out.println("AMTS: " + sbAmts.toString());
							 }	// END OF PAYMENT AMOUNTS
							 
							 if(row[3] != null) {
								 bdPaidAmt = (BigDecimal) row[3];
								 fTemp = bdPaidAmt.floatValue();
								 fTotalPaidAmt += fTemp;
								 sTotal = df.format(fTotalPaidAmt);
// System.out.println("detail() returned - PAID AMT IS: " + sTotal + " for claim: " + viewClaimIndex);								 
							 }
							 
							 
						 
			        	}	// end row null check
					}		// end while
			         
	  	         
						list.add(sbCodes);	// sbCodes is StringBuffer
						list.add(sbAmts);
						list.add(sTotal);
						
// System.out.println("details() - TOTAL PAID AMT.: " + sTotal);						
		} catch(Exception e) {
			System.out.println("ERROR in getSequenceDetails() - " + e.getMessage());
			e.printStackTrace();
			log.error("ViewClaimBean.getSequenceDetails(): ERROR: " + e.getMessage());
			
		}
						
		   
	   return list;
	   
   }	// end getSequenceDetails
   
   
   public final void printClaim() {
      try {
         String ci = Long.toString(this.viewClaimIndex);
         ArrayList<String> claimToPrint = new ArrayList<String>();
         claimToPrint.add(ci);
         printClaims.displayClaims(claimToPrint);
      } catch (Exception e) {
         log.error(e.getMessage(), e);
      }
   }

   public final String processWithoutPrinting() {
      Query q;
      boolean success = false;
      String ret = "";



      String lsql = "UPDATE claims c SET c.claim_status_cd='INPROCESS', c.date_modified=sysdate, c.modified_by='"+appUser.getUserName()+"' "
         + " WHERE c.claim_index=" + this.viewClaimIndex;
      String sql2 = "UPDATE fpps_owner.ws_fac_status c SET c.claim_status_cd='INPROCESS', c.date_modified=sysdate  "
         + " WHERE c.claim_index=" + this.viewClaimIndex;
      String sql3 = "UPDATE fpps_owner.ws_fac_status c SET c.date_in_process=sysdate "
          + " WHERE c.claim_index=" + this.viewClaimIndex
          + " AND c.date_in_process IS NULL";
      //update procedure line status
      String sql4 = "UPDATE claim_procedure c SET c.line_status='INPROCESS' WHERE c.claim_index=" + this.viewClaimIndex;


      try {
         q = em.createNativeQuery(lsql);
         q.executeUpdate();
         q = em.createNativeQuery(sql2);
         q.executeUpdate();
         q = em.createNativeQuery(sql3);
         q.executeUpdate();
         q = em.createNativeQuery(sql4);
         q.executeUpdate();
         q = em.createNativeQuery("COMMIT");
         q.executeUpdate();

         success = true;
      } catch (Exception e) {
         log.error(e.getMessage(), e);
      }

      findClaimByClaimIndex(this.viewClaimIndex);

      if (success) {
          StatusMessages.instance().addFromResourceBundle(Severity.INFO, "gov.va.med.fee.MSG_SUCCESS4", "");
       } else {
          StatusMessages.instance().addFromResourceBundle(Severity.ERROR, "gov.va.med.fee.MSG_ERROR4", "");
       }

      return ret;
   }

   @Override
   public final String displayDisapproval(Long index) {
      claimsToDisapprove.clear();
      claimsToDisapprove.add(index);
      return "disapproveclaims";
   }

   @Override
   public final String displayReroute(Long index) {
       if (claimsToReroute != null) {
           claimsToReroute.clear();
       } else {
           claimsToReroute = new ArrayList<Long>();
       }

       claimsToReroute.add(index);
       return "rerouteClaimsPage";
   }

   public final boolean getNotReconciled() {
      return this.notReconciled;
   }

   /**
    * {@inheritDoc}
    */
   @Destroy
   @Remove
   public final void destroy() {
      // log.info("destroyed:" + this);
   }

}
