/*
 * $Id: PrintClaimsBean.java,v 1.2 2010/05/28 16:32:18 vhahacnguyes Exp $
 */
package gov.va.med.fee.report;

import gov.va.med.domain.fee.AppUser;
import gov.va.med.fee.report.jasper.ReportGenerator;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;

import org.jboss.seam.Component;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.log.Log;
import org.jboss.seam.log.Logging;

/**
 * Print claims backing bean.
 */
@Stateless
@SuppressWarnings("serial")
@Name("printClaims")
public class PrintClaimsBean implements PrintClaims, Serializable {

    /** The log. */
    private final Log log = Logging.getLog(this.getClass());

    /**
     * Generates the PDF claim forms.
     * @param selectedList the list of claims to be printed
     * @return the status code
     */
    @SuppressWarnings("unchecked")
    public final String displayClaims(List<String> selectedList) {
        String claims = selectedList.toString();
        claims = claims.replace("[", "");
        claims = claims.replace("]", "");
        Map params = new HashMap();

        params.put("Claim_Indices", claims);

        ReportGenerator gen = new ReportGenerator();
        if (gen.printReport(params, new StringBuilder(FeeReportBean.REPORT_PATH)
            .append("Claims").append(FeeReportBean.TEMPLATE_EXTENSION)
            .toString(), "Claims")) {

            createPrintedClaimEntry(selectedList);

            return FeeReportBean.NEXT;
        }

        return FeeReportBean.ERROR;
    }

    /**
     * Inserts an entry into the printed claim table to indicate that claim have been
     * printed.  Regular JDBC calls are being used as Hibernate requires the full
     * Claim object to be retrieved.
     *
     * @param list list of claim index
     */
    private void createPrintedClaimEntry(List<String> list) {
        EntityManager em = (EntityManager) Component.getInstance("FPPSEntityManager");
        AppUser appUser = (AppUser) Component.getInstance("appUser");

        Connection c = null;
        Object del = em.getDelegate();

        if (del instanceof org.jboss.seam.persistence.HibernateSessionProxy) {
            c = ((org.jboss.seam.persistence.HibernateSessionProxy) del)
                    .connection();
        }

        String sql = new String("insert into"
                + " printed_claim(printed_claim_id, claim_index, created_by, date_created)"
                + " values(fpps_owner.mig.nextVal, ?, ?, sysdate)");

		String index = null;
		try {
			Iterator<String> iter = list.iterator();
			while (iter.hasNext()) {
				index = iter.next();
				PreparedStatement stmt = c.prepareStatement(sql);
				stmt.setLong(1, Long.parseLong(index));
				stmt.setString(2, appUser.getUserName());

				stmt.executeUpdate();
				stmt.close();
			}
		} catch (SQLException e) {
			log.error("Error: " + e.getMessage()
					+ " - Could not insert printed claim index: " + index);
		} finally {
			try {
				c.close();
			} catch (SQLException e) {
				log.error(e);
			}
		}
    }
}
