/*
 * $Id: HomePageUserData.java,v 1.19.4.1 2011/10/04 19:25:43 vhahacpennet Exp $
 */
package gov.va.med.fee.sec;

import org.jboss.seam.log.Log;
import org.jboss.seam.log.Logging;

import java.math.BigDecimal;

import java.text.MessageFormat;

import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.persistence.EntityManager;


/**
 * POJO storing data related to the home page.
 */
@SuppressWarnings("serial")
public class HomePageUserData extends ClaimCount implements java.io.Serializable {
    private static final String NON_VISN_SPECIFIC = "or c.va_facility_cd is null";
    private static final MessageFormat VISN_SPECIFIC = new MessageFormat(
            "or (c.va_facility_cd is null and c.visn_id_cd in ({0}))");

    //use this when you want to exclude station=null
    private static final MessageFormat VISN_SPECIFIC_NON_NULL = new MessageFormat(
            "or (c.visn_id_cd in ({0}))");

    /** The Constant COUNT_STATUS_QUERY. */
    public static final MessageFormat COUNT_STATUS_QUERY = new MessageFormat(
            "select c.va_facility_cd, c.claim_status_cd, count(c.claim_index) " +
            "from fpps_owner.ws_fac_status c where c.claim_status_cd != ''COMPLETE'' " +
            "and (c.va_facility_cd in ({0}) {1}) " +
            "group by c.va_facility_cd, c.claim_status_cd " +
            "order by c.va_facility_cd");

    /** The Constant ASSIGNED_COUNT_QUERY. */
    public static final MessageFormat ASSIGNED_COUNT_QUERY = new MessageFormat(
            "SELECT c.va_facility_cd, count(c.claim_index) " +
            " FROM fpps_owner.ws_fac_status c " + "WHERE c.claim_index IN " +
            "(SELECT fcr2.claim_index " +
            "FROM fpps_owner.fee_claim_routing fcr2 " +
            "WHERE (fcr2.claim_index, fcr2.transfer_date) IN " +
            "(SELECT fcr.claim_index, MAX(fcr.transfer_date) " +
            "FROM fpps_owner.fee_claim_routing fcr " +
            "WHERE fcr.claim_index IN " + "(SELECT DISTINCT fcr1.claim_index " +
            "FROM fpps_owner.fee_claim_routing fcr1 " +
            "WHERE fcr1.transferred_to = ''{1}'' or fcr1.transfer_to_user = ''{1}'') " +
            "GROUP BY fcr.claim_index) " +
            "AND (fcr2.transferred_to = ''{1}'' or fcr2.transfer_to_user = ''{1}'')) " +
            "AND (c.va_facility_cd IN ({0}) {2})" +
            "AND c.claim_status_cd != ''COMPLETE'' " +
            "GROUP By c.va_facility_cd " + "order by c.va_facility_cd");
    private Log log = Logging.getLog(HomePageUserData.class);

    /** The user name. */
    private String userName;

    /** The last login. */
    private Date lastLogin;

    /** The user facilities. */
    private final Map<String, UserFacilityData> userFacilities = new LinkedHashMap<String, UserFacilityData>();
    private String privacyRestrictions = "";
    private String activeVisns = "";

    /**
     * Gets the active stations.
     *
     * @return the active stations
     */
    public final Set<String> getActiveStations() {
        Set<String> activeStations = new LinkedHashSet<String>();
        Collection<UserFacilityData> fs = userFacilities.values();

        for (UserFacilityData d : fs) {
            if (d.isActive()) {
                activeStations.add(d.getFacilityCode());
            }
        }

        return activeStations;
    }

    /**
     * Gets the roles.
     *
     * @return the roles
     */
    public final SortedSet<String> getActiveRoles() {
        SortedSet<String> activeRoles = new TreeSet<String>();
        Collection<UserFacilityData> fs = userFacilities.values();

        for (UserFacilityData d : fs) {
            if (d.isActive()) {
                activeRoles.addAll(d.getRoles());
            }
        }

        return activeRoles;
    }

    /**
     * Gets the active stations string.
     *
     * @return the active stations string
     */
    public final String getActiveStationsStr() {
        return getString(getActiveStations());
    }

    /**
     * Gets the active roles str.
     *
     * @return the active roles str
     */
    public final String getActiveRolesStr() {
        return getString(getActiveRoles());
    }

    /**
     * retrieves ALL facilities (active/inactive)
     */
    public final String getAllFacilities() {
        return getFacilitiesString();
    }

    private String getString(Set<String> set) {
        if (set == null) {
            return null;
        }

        String s = set.toString();
        s = s.replace('[', ' ');
        s = s.replace(']', ' ');
        s = s.replace('_', ' ');

        return s;
    }

    /**
     * Gets the user name.
     *
     * @return the user name
     */
    public final String getUserName() {
        return userName;
    }

    /**
     * Sets the user name.
     *
     * @param userName the new user name
     */
    public final void setUserName(String userName) {
        this.userName = userName;
    }

    /**
     * Gets the user facilities.
     *
     * @return the user facilities
     */
    public final Map<String, UserFacilityData> getUserFacilities() {
        return userFacilities;
    }

    /**
     * Gets the last login.
     *
     * @return the last login
     */
    public final Date getLastLogin() {
        return lastLogin;
    }

    /**
     * Gets the last login str.
     *
     * @return the last login str
     */
    public final String getLastLoginStr() {
        if (lastLogin == null) {
            return "";
        }

        String str = new java.text.SimpleDateFormat("EEE MM/dd/yyyy HH:mm z").format(lastLogin);

        return str;
    }

    /**
     * Sets the last login.
     *
     * @param lastLogin the new last login
     */
    public final void setLastLogin(final Date lastLogin) {
        this.lastLogin = lastLogin;
    }

    /**
     * Gets the facilities string.
     *
     * @return the facilities string
     */
    private String getFacilitiesString() {
        Set<String> f = userFacilities.keySet();
        Object[] items = f.toArray();

        StringBuilder buff = new StringBuilder("");

        if ((items != null) && (items.length > 0)) {
            for (int i = 0; i < items.length; i++) {
                if (i != 0) {
                    buff.append(",");
                }

                buff.append("'").append(items[i]).append("'");
            }
        }
        return buff.toString();
    }

    /**
     * Returns true for admin or visn manager.
     * @return
     */
    public boolean hasVisnLevelAccess() {
        String roles = getActiveRolesStr();

        if (roles != null) {
            if ((roles.indexOf("FEE VISN PGM MANAGER") != -1) ||
                    roles.matches("^ ADMIN.*")) {
                return true;
            }
        }

        return false;
    }

    /**
     * Returns the condition to be added to the query for VISN claims when user is admin or
     * VISN manager.  This assumes the Claims table is aliased as "c".
     * @return query condition
     */
    public String getVisnSelector() {
        String roles = getActiveRolesStr();

        if (roles != null) {
            if (roles.matches("^ ADMIN.*")) { //return everything

                return NON_VISN_SPECIFIC; //fac_cd = null
            } else if (roles.indexOf("FEE VISN PGM MANAGER") != -1) {
                String activeVisns = getActiveVisns();
                activeVisns = activeVisns.replaceAll(" ", "");
                activeVisns = activeVisns.replaceAll("'", ""); // visn code is number

                String formattedSql = VISN_SPECIFIC.format(new Object[] {
                            activeVisns
                        });

                return formattedSql;
            }
        }

        return "";
    }

    /**
     * Use this method to exclude va_facility_cd = null from db query
     * when ADMIN is performing claim search
     */
    public String getVisnSelectorNotNull() {
        String roles = getActiveRolesStr();

        if (roles != null) {
            if (roles.matches("^ ADMIN.*")) {
                return "";
            } else if (roles.indexOf("FEE VISN PGM MANAGER") != -1) {
                String activeVisns = getActiveVisns();
                activeVisns = activeVisns.replaceAll(" ", "");
                activeVisns = activeVisns.replaceAll("'", ""); // visn code is number

                String formattedSql = VISN_SPECIFIC_NON_NULL.format(new Object[] {
                            activeVisns
                        });

                return formattedSql;
            }
        }

        return "";
    }

    /**
     * Gets the claim counts.
     *
     * @param em the em
     */
    @SuppressWarnings("unchecked")
    public final void getClaimCounts(EntityManager em) {
        String facilities = getFacilitiesString();
        String visnQuery = getVisnSelector();

        String formattedSql = COUNT_STATUS_QUERY.format(new Object[] {
                    facilities, visnQuery
                });

        List<Object> list = em.createNativeQuery(formattedSql).getResultList();

        for (Object o : list) {
            Object[] row = (Object[]) o;

            int i = 0;
            String facility = (String) row[i++];
            String status = (String) row[i++];
            int count = ((java.math.BigDecimal) row[i++]).intValue();

            UserFacilityData facData = userFacilities.get(facility);
            facData.setCount(status, count);
        }

        formattedSql = ASSIGNED_COUNT_QUERY.format(new Object[] {
                    facilities, userName, visnQuery
                });
        list = em.createNativeQuery(formattedSql).getResultList();

        for (Object o : list) {
            Object[] row = (Object[]) o;

            int i = 0;
            String facility = (String) row[i++];
            int count = ((java.math.BigDecimal) row[i++]).intValue();

            UserFacilityData facData = userFacilities.get(facility);
            facData.setClaimsAssigned(count);
        }

        summarize();
    }

    /**
     * Gets the privacy restrictions.
     */
    public final String getPrivacyRestrictions() {
        return this.privacyRestrictions;
    }

    /**
     * Sets the privacy restrictions.
     *
     * @param em the em
     */
    @SuppressWarnings("unchecked")
    public final void setPrivacyRestrictions(EntityManager em) {
        this.privacyRestrictions = "";

        String comma = "";
        String formattedSql = " SELECT p2p.person_id " +
            " FROM fpps_owner.privacy_to_person p2p " +
            " WHERE upper(p2p.user_name) = upper('" + this.userName + "')";

        List<String> list = em.createNativeQuery(formattedSql).getResultList();

        for (String o : list) {
            String person_id = o;
            this.privacyRestrictions += (comma + "'" + person_id + "'");
            comma = ", ";
        }
    }

    /**
     * Gets the Active Visns.
     */
    public final String getActiveVisns() {
        return this.activeVisns;
    }

    /**
     * Sets the Active Visns.
     *
     * @param em the em
     */
    @SuppressWarnings("unchecked")
    public final void setActiveVisns(EntityManager em) {
        this.activeVisns = "";

        String comma = "";
        String s = getActiveStationsStr();
        s = s.equalsIgnoreCase("") ? s : ("'" + s + "'");
        s = s.replaceAll(", ", "', '");
        s = s.replaceAll(" ", "");

        String formattedSql = " select distinct(f.visn_id_cd) " +
            " from fpps_owner.va_facility f, user_role_usages uru, app_user au, app_role ar " +
            " where f.va_facility_cd = uru.va_facility_cd " +
            " and au.app_user_id = uru.app_user_id  " +
            " and ar.app_role_id = uru.app_role_id " + " and au.user_name='" +
            this.userName + "' " +
            " and (ar.role_name='FEE_VISN_PGM_MANAGER' or ar.role_name='ADMIN')" +
            " and f.va_facility_cd IN(" + s + ") ";

        log.debug(formattedSql);

        List<BigDecimal> list = em.createNativeQuery(formattedSql)
                                  .getResultList();

        for (BigDecimal o : list) {
            String visn = o.toString();
            this.activeVisns += (comma + "'" + visn + "'");
            comma = ", ";
        }
    }

    /**
     * Summarize.
     */
    public final void summarize() {
        setClaimsAssigned(0);
        setClaimsAwaiting(0);
        setClaimsInProcess(0);
        setClaimsError(0);
        setClaimsRejected(0);

        for (UserFacilityData data : userFacilities.values()) {
            setClaimsAssigned(getClaimsAssigned() + data.getClaimsAssigned());
            setClaimsAwaiting(getClaimsAwaiting() + data.getClaimsAwaiting());
            setClaimsInProcess(getClaimsInProcess() +
                data.getClaimsInProcess());
            setClaimsError(getClaimsError() + data.getClaimsError());
            setClaimsRejected(getClaimsRejected() + data.getClaimsRejected());
        }
    }
}
