/*
 * $Id: IDCheckFilter.java,v 1.5 2009/12/03 17:08:12 vhahacmagdua Exp $
 */
package gov.va.med.fee.sec;

import java.io.IOException;
import java.net.URLEncoder;
import java.text.MessageFormat;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.jboss.seam.log.Log;
import org.jboss.seam.annotations.Logger;


/**
 * The Class IDCheckFilter.
 */
public final class IDCheckFilter implements Filter {
	
	/** The log. */
	   @SuppressWarnings("unused")
	   @Logger
	   private Log log;
	
	

    /** The Constant GUID. */
    public static final String GUID = "gov.va.med.fee.sec.GUID";

    /** The Constant ERROR_PAGE. */
    public static final String ERROR_PAGE = "gov.va.med.fee.sec.ErrorPage";

    /** The post back. */
    private String postBack;

    /** The asp check. */
    private String aspCheck;

    /** The default page. */
    private static String defaultPage;

    /** The error page. */
    private String errorPage;

    /**
     * Instantiates a new iD check filter.
     */
    public IDCheckFilter() {
    }

    /**
    * {@inheritDoc}
    */
    public void init(FilterConfig conf) throws ServletException {
        postBack = conf.getInitParameter("postBack");
        aspCheck = conf.getInitParameter("aspCheckPage");
        defaultPage = conf.getInitParameter("defaultPage");
        errorPage = conf.getInitParameter("errorPage");

        System.out.println(new StringBuffer("Security Filter Parameters [")
	        .append("Post Back:" + postBack)
	        .append(",Asp Page:" + aspCheck)
	        .append(",Default Page:" + defaultPage)
	        .append(",Error Page:" + errorPage)
	        .append("]").toString());
        
       // no output on console........... 
                
    }

    /**
     * Wraps the string in single quote.
     *
     * @param in the in
     *
     * @return the string
     */
    private String inQuote(String in) {
        String out = new StringBuilder("'").append(in).append("'").toString();
        return out;
    }

    /**
     * Post to asp page.
     *
     * @param hreq the hreq
     * @param hres the hres
     * @param capturedUrl the captured url
     *
     * @throws IOException Signals that an I/O exception has occurred.
     * @throws ServletException the servlet exception
     */
    private void postAsp(HttpServletRequest hreq, HttpServletResponse hres,
    		String sessionId, String capturedUrl) throws IOException {
        String host = hreq.getLocalName();
        int port = hreq.getLocalPort();
        host = new StringBuffer(host).append(':').append(port).toString();

        String asp = inQuote(aspCheck);
        postBack = MessageFormat.format(postBack, host);
        
        String encryptedPostBackUrl = "";
        String encryptedDefaultUrl = "";

        try {
        	encryptedPostBackUrl = IDCheckAESEncryption.encrypt(postBack);
            encryptedDefaultUrl = IDCheckAESEncryption.encrypt(MessageFormat.format(capturedUrl, host));
        } catch (Exception e) {
            System.err.println(e);
        }
        
        HttpSession session = hreq.getSession();

        String sessionid = inQuote(sessionId);

        session.setAttribute("aspCheck", asp);
        session.setAttribute("c", inQuote(URLEncoder.encode(encryptedDefaultUrl, "UTF-8")));
        session.setAttribute("u", inQuote(URLEncoder.encode(encryptedPostBackUrl, "UTF-8")));
        session.setAttribute("s", sessionid);

        hres.sendRedirect("/FPPS/home.seam");
    }

    /**
    * {@inheritDoc}
    */
    public void destroy() {
    }

    /**
    * {@inheritDoc}
    */
    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain) throws IOException, ServletException {

        HttpServletRequest hreq = (HttpServletRequest) request;
        HttpServletResponse hres = (HttpServletResponse) response;
        HttpSession session = hreq.getSession();

        String url = hreq.getRequestURL().toString();

        // When there is an in session, user has been authenticated
        String currentId = (String) session.getAttribute(GUID);

        if (isNull(currentId) && !isSkip(url)) {
        	// The id should be unique but it was not.
            String id = session.getId() + session.hashCode();
            postAsp(hreq, hres, id, defaultPage);
            return;
        }

        request.setAttribute(ERROR_PAGE, errorPage);

        chain.doFilter(request, response);
    }

    /**
     * If url matches one of the pattern, it should be skipped.
     *
     * @param url the url
     *
     * @return true, if checks if is skip
     */
    private boolean isSkip(String url) {
        return contains(url, postBack) || contains(url, errorPage)
                || url.endsWith(".css") || url.endsWith(".gif") ? true : false;
    }

    /**
     * Checks if is null.
     *
     * @param s the string
     *
     * @return true, if is null
     */
    private boolean isNull(String s) {
        return s == null || s.length() == 0 ? true : false;
    }

    /**
     * Contains.
     *
     * @param s1 the string 1
     * @param s2 the string 2
     *
     * @return true, if successful
     */
    private boolean contains(String s1, String s2) {
        return s1.indexOf(s2) > -1 ? true : false;
    }
    
    public static String getDefaultPage() {
    	return defaultPage;
    }

}
