package gov.va.med.fee.tool;

import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.ContactInfo;
import gov.va.med.domain.fee.VaFacility;

import java.util.Date;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.framework.EntityHome;

/**
 * The Class ContactInfoHome.
 */
@Name("contactInfoHome")
@Restrict("#{s:hasRole('FEE_SUPERVISOR') || s:hasRole('FEE_VISN_PGM_MANAGER') || s:hasRole('ADMIN')}")
public class ContactInfoHome extends EntityHome<ContactInfo> {

    /** The va facility home. */
    @In(create = true)
    private VaFacilityHome vaFacilityHome;

    /** The app user. */
    @In(value = "appUser", required = true, scope = ScopeType.SESSION)
    private AppUser appUser;

    /**
     * Sets the contact info contact info id.
     *
     * @param id the new contact info contact info id
     */
    public void setContactInfoContactInfoId(Long id) {
        setId(id);
    }

    /**
     * Gets the contact info contact info id.
     *
     * @return the contact info contact info id
     */
    public Long getContactInfoContactInfoId() {
        return (Long) getId();
    }

    /**
     * Wire.
     */
    public void wire() {
        getInstance();
        VaFacility vaFacility = vaFacilityHome.getDefinedInstance();
        if (vaFacility != null) {
            getInstance().setVaFacility(vaFacility);
        }
    }

    /**
     * Checks if is wired.
     *
     * @return true, if is wired
     */
    public boolean isWired() {
        if (getInstance().getVaFacility() == null) {
            return false;
        }
        return true;
    }

    /**
     * Gets the defined instance.
     *
     * @return the defined instance
     */
    public ContactInfo getDefinedInstance() {
        return isIdDefined() ? getInstance() : null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    @Transactional
    public String persist() {
        getInstance().setCreatedBy(appUser.getUserName());
        getInstance().setDateCreated(new Date());
        return super.persist();
    }

    /**
     * Overrides to return the correct persistent context name for this app.
     *
     * @return persistence context name
     */
    @Override
    protected final String getPersistenceContextName() {
       return "FPPSEntityManager";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected ContactInfo createInstance() {
        ContactInfo contactInfo = new ContactInfo();
        return contactInfo;
    }

}
