package gov.va.med.fee.tool;

import gov.va.med.domain.fee.MedicalCode;

import java.util.Arrays;

import org.jboss.seam.annotations.Name;
import org.jboss.seam.framework.EntityQuery;


/**
 * MedicalCodeList class used to display medical codes.
 */
@Name("medicalCodeList")
public class MedicalCodeList extends EntityQuery<MedicalCode> {
    /** maximum number of Medical codes to return. */
    private static final int MAX_RESULTS = 15;

    /** the query to use to select medical codes. */
    private static final String EJBQL = "select medicalCode from fMedicalCode medicalCode";

    /** query restrictions. */
    private static final String[] RESTRICTIONS = {
            "lower(medicalCode.createdBy) like concat(lower(#{medicalCodeList.medicalCode.createdBy}),'%')",
            "lower(medicalCode.description) like concat(lower(#{medicalCodeList.medicalCode.description}),'%')",
            "lower(medicalCode.drgExemptFlag) like concat(lower(#{medicalCodeList.medicalCode.drgExemptFlag}),'%')",
            "lower(medicalCode.medicalCd) like concat(lower(#{medicalCodeList.medicalCode.medicalCd}),'%')",
            "lower(medicalCode.medicalCodeType) like concat(lower(#{medicalCodeList.medicalCode.medicalCodeType}),'%')"
            };

    /** medical code instance. */
    private final MedicalCode medicalCode = new MedicalCode();

    /**
     * Default constructor.
     */
    public MedicalCodeList() {
        setEjbql(EJBQL);
        setRestrictionExpressionStrings(Arrays.asList(RESTRICTIONS));
        setMaxResults(MAX_RESULTS);
    }

    /**
     * Overrides to return the correct persistent context name for this app.
     *
     * @return persistence context name
     */
    @Override
    protected final String getPersistenceContextName() {
       return "FPPSEntityManager";
    }

    /**
     * Factory method used to return the MedicalCode instance.
     *
     * !!!DO NOT MAKE THIS METHOD FINAL!!!
     *
     * @return the Medical code instance.
     */
    public MedicalCode getMedicalCode() {
        return medicalCode;
    }
}
