/** $Id: PrivacyRestrictionsSessionBean.java,v 1.4 2009/12/04 19:35:48 vhahacdoansc Exp $ */
package gov.va.med.fee.tool;

import gov.va.med.domain.fee.PrivacyToPerson;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.ejb.Stateless;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.international.StatusMessage.Severity;

import java.util.LinkedList;

import org.jboss.seam.log.Log;

import gov.va.med.domain.fee.AppUser;

import javax.faces.component.html.HtmlInputTextarea;

/**
 * This class User Administration requests.
 */
@Stateless
@Name("privacyRestrictions")
@Restrict("#{s:hasRole('FEE_SUPERVISOR') or s:hasRole('ADMIN') or s:hasRole('FPPS_HELP_DESK')}")
public class PrivacyRestrictionsSessionBean implements PrivacyRestrictions {

    /** The app user. */
    @In(value = "appUser", required = true, scope = ScopeType.SESSION)
    private AppUser appUser;

    private List<PrivacyToPerson> allRestrictions = new LinkedList<PrivacyToPerson>();

    /** The em. */
    @PersistenceContext(unitName = "FPPS_Owner")
    private EntityManager em;

    /** The log. */
    @Logger
    private Log log;

    private String ntNameToAdd = "";
    private String reasonToAdd = "";
    private String ssnToAdd = "";
    private String typeToAdd = "";

    public PrivacyRestrictionsSessionBean() {

    }

    public final List<PrivacyToPerson> getAllRestrictions() {
        populateAllRestrictions();
        return allRestrictions;
    }

    public final String getNtNameToAdd() {
        return ntNameToAdd;
    }

    public final void setNtNameToAdd(String ntNameToAdd) {
        this.ntNameToAdd = ntNameToAdd;
    }

    public final String getReasonToAdd() {
        return reasonToAdd;
    }

    public final void setReasonToAdd(String reasonToAdd) {
        this.reasonToAdd = reasonToAdd;
    }

    public final String getSsnToAdd() {
        return ssnToAdd;
    }

    public final void setSsnToAdd(String ssnToAdd) {
        this.ssnToAdd = ssnToAdd;
    }

    public final String getTypeToAdd() {
        return typeToAdd;
    }

    public final void setTypeToAdd(String typeToAdd) {
        this.typeToAdd = typeToAdd;
    }

    public final String addPrivacyRestriction() {
        String ret = "addPrivacyRestrictionFAIL";
        boolean success = false;
        String sql;
        Query q;
        List<?> l;
        Iterator it;

        //check if privacy restriction is already in place for this user + ssn
        sql = " SELECT COUNT(*) FROM fpps_owner.privacy_to_person p "
                + " WHERE UPPER(p.user_name)=UPPER(:ntname) AND p.person_id=:ssn";
        q = em.createNativeQuery(sql);
        q.setParameter("ntname", this.ntNameToAdd);
        q.setParameter("ssn", this.ssnToAdd);
        BigDecimal recordCount = (BigDecimal) q.getSingleResult();

        if (recordCount.intValue() > 0) {
            StatusMessages.instance().addFromResourceBundle(Severity.ERROR,
                    "gov.va.med.fee.privacy_restriction.ADD_ERROR_ON_EXISTING",
                    "");
            return ret;
        } 
        
      //check if user already has any of this person's claims assigned to them.
      //if so, they will need to be rerouted to someone else before the restriction can be created.              
        sql = " SELECT c.claim_index FROM claims c, person_info p "
              + " WHERE c.claim_index=p.claim_index "
              + " AND UPPER(c.user_name)=UPPER(:ntname) AND p.person_id=:ssn";              
        q = em.createNativeQuery(sql);
        q.setParameter("ntname", this.ntNameToAdd);
        q.setParameter("ssn", this.ssnToAdd);                      
        l = q.getResultList();
        it = l.iterator();

         //loop through the results. 
        String claims = "";
        String comma = "";
        while (it.hasNext()) {            
            BigDecimal claim = (BigDecimal) it.next();                 
            claims+=comma+claim.toString();
            comma = ", ";            
        }
        log.debug("claims = " + claims);
        
        if (!claims.equalsIgnoreCase("")) {
            StatusMessages.instance().addFromResourceBundle(Severity.ERROR,
                    "gov.va.med.fee.privacy_restriction.ADD_ERROR_ON_CLAIMS_ASSIGNED",
                    claims);
            return ret;
        } 
        
      
        
        
        //passed checks, proceed with creating privacy restriction 
        
        sql = " INSERT INTO fpps_owner.privacy_to_person(privacy_request_id, user_name, person_id, reason, request_type_flag, created_by, date_created) "
                + " VALUES(fpps_owner.mig.nextval, :ntname, :ssn, :reason, :type, :appuser, sysdate) ";
        q = em.createNativeQuery(sql);
        q.setParameter("ntname", this.ntNameToAdd.trim().toUpperCase());
        q.setParameter("ssn", this.ssnToAdd);
        q.setParameter("reason", this.reasonToAdd);
        q.setParameter("type", this.typeToAdd);
        q.setParameter("appuser", appUser.getUserName());

        q.executeUpdate();
        q = em.createNativeQuery("COMMIT");
        q.executeUpdate();
        ret = "addPrivacyRestrictionSUCCESS";
        success = true;

        ssnToAdd = "";
        ntNameToAdd = "";
        typeToAdd = "";
        reasonToAdd = "";
        

        if (success) {
            StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                    "gov.va.med.fee.privacy_restriction.ADD_SUCCESS", "");
        } else {
            StatusMessages.instance().addFromResourceBundle(Severity.ERROR,
                    "gov.va.med.fee.privacy_restriction.ADD_ERROR", "");
        }

        return ret;
    }

    public final String delete(int privacyRequestId) {
        String ret = "deletePrivacyRestrictionFAIL";
        String sql;
        Query q;
        boolean success = false;

        try {
            sql = " DELETE FROM fpps_owner.privacy_to_person p "
                    + " WHERE p.privacy_request_id=:id ";

            q = em.createNativeQuery(sql);
            q.setParameter("id", privacyRequestId);
            q.executeUpdate();
            q = em.createNativeQuery("COMMIT");
            q.executeUpdate();
            ret = "deletePrivacyRestrictionSUCCESS";
            success = true;
        } catch (Exception e) {
            log.error(e);
        }

        if (success) {
            StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                    "gov.va.med.fee.privacy_restriction.DELETE_SUCCESS", "");
        } else {
            StatusMessages.instance().addFromResourceBundle(Severity.ERROR,
                    "gov.va.med.fee.privacy_restriction.DELETE_ERROR", "");
        }

        return ret;
    }

    @SuppressWarnings("unchecked")
    public final void populateAllRestrictions() {
        String sql;
        Query q;
        List<?> l;
        Iterator it;

        this.allRestrictions.clear();

        sql = " SELECT "
                + "     p.person_id, "
                + "     p.user_name, "
                + "     p.reason, "
                + "     p.created_by, "
                + "     p.date_created, "
                + "p.request_type_flag, "
                + "p.privacy_request_id "
                + " FROM privacy_to_person p "
                + " ORDER BY p.date_created DESC, p.person_id ASC, p.user_name ASC ";

        q = em.createNativeQuery(sql);
        l = q.getResultList();

        it = l.iterator();

        /* loop through the results. */
        while (it.hasNext()) {
            PrivacyToPerson p = new PrivacyToPerson();
            Object[] row = (Object[]) it.next();

            p.setPersonId((String) row[0]);
            p.setUserName((String) row[1]);
            p.setReason((String) row[2]);
            p.setCreatedBy((String) row[3]);

            Date dc = new Date();
            try {
                dc = (Date) row[4];
            } catch (Exception e) {
                log.debug("Error in extracting date data ", e);
            }
            p.setDateCreated(dc);

            p.setRequestTypeFlag((String) row[5]);

            BigDecimal pid = (BigDecimal) row[6];
            p.setPrivacyRequestId(pid.longValue());

            allRestrictions.add(p);
        }
    }

    public final void validateTextAreaLength(FacesContext context,
            UIComponent component, Object obj) {
        String value = obj.toString();
        HtmlInputTextarea t = (HtmlInputTextarea) component;
        if (value.length() > 200) {
            t.setValid(false);
            FacesMessage message = new FacesMessage("200 characters max");
            context.addMessage(component.getClientId(context), message);
        }
    }

}
