package gov.va.med.fee.tool;

import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.VaFacility;
import gov.va.med.domain.fee.ZipCode;
import gov.va.med.fee.FeeMessages;

import java.util.Date;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.framework.EntityHome;
import org.jboss.seam.international.StatusMessage.Severity;

/**
 * The Class ZipCodeHome.
 */
@Name("zipCodeHome")
@Restrict("#{s:hasRole('FEE_SUPERVISOR') || s:hasRole('FEE_VISN_PGM_MANAGER') || s:hasRole('ADMIN')}")
public class ZipCodeHome extends EntityHome<ZipCode> {

    /** The app user. */
    @In(value = "appUser", required = true, scope = ScopeType.SESSION)
    private AppUser appUser;

    /** The va facility home. */
    @In(create = true)
    private VaFacilityHome vaFacilityHome;

    /** The faces messages. */
    @In
    private FacesMessages facesMessages;


    /**
     * Sets the zip code.
     *
     * @param id the new zip
     */
    public void setZipCodeZipCode(String id) {
        setId(id);
    }

    /**
     * Gets the zip code.
     *
     * @return the zip code
     */
    public String getZipCodeZipCode() {
        return (String) getId();
    }

    /**
     * Wire.
     */
    public void wire() {
        getInstance();
        VaFacility vaFacility = vaFacilityHome.getDefinedInstance();
        if (vaFacility != null) {
            getInstance().setVaFacility(vaFacility);
        }
    }

    /**
     * Checks if is wired.
     *
     * @return true, if is wired
     */
    public boolean isWired() {
        if (getInstance().getVaFacility() == null) {
            return false;
        }
        return true;
    }

    /**
     * Gets the defined instance.
     *
     * @return the defined instance
     */
    public ZipCode getDefinedInstance() {
        return isIdDefined() ? getInstance() : null;
    }

    /**
     * Check.
     *
     * @return true, if successful
     */
    public boolean check() {
        ZipCode z = getEntityManager().find(ZipCode.class, getInstance().getZipCode());
        if (z != null) {
            FeeMessages.addFromResourceBundle(
                    "zipCode:zipCodeField:zipCode",
                    javax.faces.application.FacesMessage.SEVERITY_ERROR,
                    "gov.va.med.fee.tools.StationMaint.ZIP_EXIST");
            return false;
        }
        return true;
    }

    /**
     * Saves only if zip code has not been entered.
     *
     * @return the string status
     */
    @Override
    @Transactional
    public String persist() {
        if (check()) {
            getInstance().setCreatedBy(appUser.getUserName());
            getInstance().setDateCreated(new Date());
            return super.persist();
        } else {
            facesMessages.addFromResourceBundle(
                Severity.ERROR,
                "gov.va.med.fee.tools.StationMaint.ZIP_EXIST");
        }
        return null;
    }


    /**
     * Overrides to return the correct persistent context name for this app.
     *
     * @return persistence context name
     */
    @Override
    protected final String getPersistenceContextName() {
       return "FPPSEntityManager";
    }

    /**
     * Creates an instance and sets default value.
     * @return the new instance
     */
    @Override
    protected ZipCode createInstance() {
        ZipCode zipCode = new ZipCode();
        zipCode.setActiveFlag('Y');
        return zipCode;
    }
}
